@@ -6,27 +6,38 @@
  * Open Phone Abstraction Library (OPAL)
  * Formally known as the Open H323 project.
  *
- * Copyright (C) 2004-2006 Post Increment
+ * Copyright (C) 2004-2011 Post Increment
  *
- * The contents of this file are subject to the Mozilla Public License
- * Version 1.0 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * Software distributed under the License is distributed on an "AS IS"
- * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
- * the License for the specific language governing rights and limitations
- * under the License.
+ * - Redistributions of source code must retain the above copyright
+ *   notice, this list of conditions and the following disclaimer.
+
+ * - Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in the
+ *   documentation and/or other materials provided with the distribution.
  *
- * The Original Code is Open Phone Abstraction Library.
+ * - Neither the name of the Xiph.org Foundation nor the names of its
+ *   contributors may be used to endorse or promote products derived from
+ *   this software without specific prior written permission.
  *
- * The Initial Developer of the Original Code is Post Increment
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+ * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Contributor(s): ______________________________________.
- *
- * $Revision: 26053 $
- * $Author: rjongbloed $
- * $Date: 2011-06-20 03:40:20 -0500 (Mon, 20 Jun 2011) $
+ * $Revision: 28932 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 06:34:20 -0600 (Fri, 18 Jan 2013) $
  */
 
 #ifndef OPAL_CODEC_OPALPLUGIN_H
@@ -50,7 +61,7 @@ extern "C" {
 #pragma warning(disable:4201)
 #endif
 
-#ifdef _WIN32
+#ifdef _MSC_VER
 #  ifdef PLUGIN_CODEC_DLL_EXPORTS
 #    define PLUGIN_CODEC_DLL_API __declspec(dllexport)
 #  else
@@ -78,8 +89,9 @@ extern "C" {
 #define  PLUGIN_CODEC_VERSION_FAX       4    // added fax
 #define  PLUGIN_CODEC_VERSION_OPTIONS   5    // added options handling
 #define  PLUGIN_CODEC_VERSION_INTERSECT 6    // added media option intersection merge functionality
+#define  PLUGIN_CODEC_VERSION_H245_DEF_GEN_PARAM 7 // added suppression of H.245 generic parameters via default
 
-#define  PLUGIN_CODEC_VERSION PLUGIN_CODEC_VERSION_INTERSECT // Always latest version
+#define  PLUGIN_CODEC_VERSION PLUGIN_CODEC_VERSION_H245_DEF_GEN_PARAM // Always latest version
 
 #define PLUGIN_CODEC_API_VER_FN       PWLibPlugin_GetAPIVersion
 #define PLUGIN_CODEC_API_VER_FN_STR   "PWLibPlugin_GetAPIVersion"
@@ -109,7 +121,7 @@ enum {
 
 struct PluginCodec_information {
   // start of version 1 fields
-  time_t timestamp;                     // codec creation time and date - obtain with command: date -u "+%c = %s"
+  time_t timestamp_deprecated;
 
   const char * sourceAuthor;            // source code author
   const char * sourceVersion;           // source code version
@@ -129,6 +141,7 @@ struct PluginCodec_information {
   unsigned short codecLicenseCode;      // codec license code
   // end of version 1 fields
 
+  const char * timestamp;               // String form of timestamp for plug in, generally as provided by __TIMESTAMP__
 };
 
 enum PluginCodec_Flags {
@@ -165,9 +178,12 @@ enum PluginCodec_Flags {
   PluginCodec_MediaExtensionMask     = 0x0400,
   PluginCodec_MediaTypeExtVideo      = 0x0400,
 
-  PluginCodec_ComfortNoiseMask       = 0x0800,
+  PluginCodec_ComfortNoiseMask       = 0x0800,  // Audio only
   PluginCodec_ComfortNoise           = 0x0800,
 
+  PluginCodec_ErrorConcealmentMask   = 0x0800,  // Video only
+  PluginCodec_ErrorConcealment       = 0x0800,
+
   PluginCodec_EmptyPayloadMask       = 0x1000,
   PluginCodec_EmptyPayload           = 0x1000,
 
@@ -181,9 +197,13 @@ enum PluginCodec_Flags {
   PluginCodec_ChannelsMask           = 0x003f0000
 };
 
+#define PluginCodec_SetChannels(n) (((n-1)<<PluginCodec_ChannelsPos)&PluginCodec_ChannelsMask)
+
+
 enum PluginCodec_CoderFlags {
   PluginCodec_CoderSilenceFrame      = 1,    // request audio codec to create silence frame
-  PluginCodec_CoderForceIFrame       = 2     // request video codec to force I frame
+  PluginCodec_CoderForceIFrame       = 2,    // request video codec to force I frame
+  PluginCodec_CoderPacketLoss        = 4     // indicate to video codec packets were lost
 };
 
 enum PluginCodec_ReturnCoderFlags {
@@ -203,6 +223,7 @@ struct PluginCodec_Definition;
 #define PLUGINCODEC_CONTROL_FREE_CODEC_OPTIONS    "free_codec_options"
 #define PLUGINCODEC_CONTROL_GET_OUTPUT_DATA_SIZE  "get_output_data_size"
 #define PLUGINCODEC_CONTROL_SET_CODEC_OPTIONS     "set_codec_options"
+#define PLUGINCODEC_CONTROL_GET_ACTIVE_OPTIONS    "get_active_options"
 #define PLUGINCODEC_CONTROL_TO_NORMALISED_OPTIONS "to_normalised_options"
 #define PLUGINCODEC_CONTROL_TO_CUSTOMISED_OPTIONS "to_customised_options"
 #define PLUGINCODEC_CONTROL_SET_INSTANCE_ID       "set_instance_id"
@@ -262,6 +283,8 @@ enum PluginCodec_OptionMerge {
 #define PluginCodec_H245_OLC           0x02000000
 #define PluginCodec_H245_ReqMode       0x01000000
 #define PluginCodec_H245_OrdinalMask   0x0000ffff
+#define PluginCodec_H245_PositionMask  0x00ff0000
+#define PluginCodec_H245_PositionShift 16
 
 typedef int (*PluginCodec_MergeFunction)(char ** result, const char * dest, const char * src);
 typedef void (*PluginCodec_FreeFunction)(char * string);
@@ -280,6 +303,7 @@ struct PluginCodec_Option {
   const char *                 m_maximum;
   PluginCodec_MergeFunction    m_mergeFunction; // Used if m_merge==PluginCodec_CustomMerge
   PluginCodec_FreeFunction     m_freeFunction;
+  const char *                 m_H245Default;
 };
 
 
@@ -289,9 +313,10 @@ struct PluginCodec_Option {
 #define PLUGINCODEC_OPTION_CHANNELS                   "Channels"
 #define PLUGINCODEC_OPTION_FRAME_TIME                 "Frame Time"
 #define PLUGINCODEC_OPTION_MAX_FRAME_SIZE             "Max Frame Size"
-#define PLUGINCODEC_OPTION_MAX_TX_PACKET_SIZE         "Max Tx Packet Size"
+#define PLUGINCODEC_OPTION_MAX_TX_PACKET_SIZE         "Max Tx Packet Size"   /* Really max PAYLOAD size */
 #define PLUGINCODEC_OPTION_MAX_BIT_RATE               "Max Bit Rate"
 #define PLUGINCODEC_OPTION_TARGET_BIT_RATE            "Target Bit Rate"
+#define PLUGINCODEC_OPTION_RATE_CONTROL_PERIOD        "Rate Control Period"
 #define PLUGINCODEC_OPTION_RX_FRAMES_PER_PACKET       "Rx Frames Per Packet"
 #define PLUGINCODEC_OPTION_TX_FRAMES_PER_PACKET       "Tx Frames Per Packet"
 #define PLUGINCODEC_OPTION_FRAME_WIDTH                "Frame Width"
@@ -303,6 +328,9 @@ struct PluginCodec_Option {
 #define PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF "Temporal Spatial Trade Off"
 #define PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD        "Tx Key Frame Period"
 
+#define PLUGINCODEC_OPTION_PROTOCOL      "Protocol"
+#define PLUGINCODEC_OPTION_PROTOCOL_H323 "H.323"
+#define PLUGINCODEC_OPTION_PROTOCOL_SIP  "SIP"
 
 
 // Full definition of the codec
@@ -311,7 +339,7 @@ struct PluginCodec_Definition {
   unsigned int version;                     // codec structure version
 
   // start of version 1 fields
-  struct PluginCodec_information * info;   // license information
+  const struct PluginCodec_information * info;   // license information
 
   unsigned int flags;                      // b0-3: 0 = audio,        1 = video
                                            // b4:   0 = raw input,    1 = RTP input
@@ -366,10 +394,240 @@ struct PluginCodec_Definition {
   // to this structure without an API version change!!!!
 };
 
-typedef struct PluginCodec_Definition * (* PluginCodec_GetCodecFunction)(unsigned int *, unsigned int);
+typedef const struct PluginCodec_Definition * (* PluginCodec_GetCodecFunction)(unsigned int *, unsigned int);
 typedef unsigned (* PluginCodec_GetAPIVersionFunction)();
 
+
 ///////////////////////////////////////////////////////////////////
+
+#define PLUGINCODEC_RAW_AUDIO "L16"
+#define PLUGINCODEC_RAW_VIDEO "YUV420P"
+#define PLUGINCODEC_VIDEO_CLOCK 90000
+#define PLUGINCODEC_MAX_FRAME_RATE 30
+
+#define PLUGINCODEC_CIF_WIDTH     352
+#define PLUGINCODEC_CIF_HEIGHT    288
+
+#define PLUGINCODEC_CIF4_WIDTH   (PLUGINCODEC_CIF_WIDTH*2)
+#define PLUGINCODEC_CIF4_HEIGHT  (PLUGINCODEC_CIF_HEIGHT*2)
+
+#define PLUGINCODEC_CIF16_WIDTH  (PLUGINCODEC_CIF_WIDTH*4)
+#define PLUGINCODEC_CIF16_HEIGHT (PLUGINCODEC_CIF_HEIGHT*4)
+
+#define PLUGINCODEC_QCIF_WIDTH   (PLUGINCODEC_CIF_WIDTH/2)
+#define PLUGINCODEC_QCIF_HEIGHT  (PLUGINCODEC_CIF_HEIGHT/2)
+
+#define PLUGINCODEC_QCIF4_WIDTH  (PLUGINCODEC_CIF4_WIDTH/2)
+#define PLUGINCODEC_QCIF4_HEIGHT (PLUGINCODEC_CIF4_HEIGHT/2)
+
+#define PLUGINCODEC_SQCIF_WIDTH   128
+#define PLUGINCODEC_SQCIF_HEIGHT   96
+
+
+#ifndef __TIMESTAMP__
+#define __TIMESTAMP__ "0"
+#endif
+
+#define PLUGINCODEC_LICENSE(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15) \
+  static struct PluginCodec_information MyLicenseInfo = { 0,p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,__TIMESTAMP__ }
+
+
+/// Declare a pair of plug in definition entries for a codec
+#define PLUGINCODEC_CODEC_PAIR(MediaFormat,    /**< Media Format */ \
+                               PayloadName,    /**< IANA RTP payload code */ \
+                               Description,    /**< Description text */ \
+                               SampleRate,     /**< Sample rate */ \
+                               BitsPerSecond,  /**< Maximum bits per second */ \
+                               FrameTime,      /**< Microseconds per frame */ \
+                               p1,p2,p3,p4, \
+                               PayloadType,    /**< IANA RTP payload type code */ \
+                               H323type,       /**< h323CapabilityType enumeration */ \
+                               H323data,       /**< Data to go with h323CapabilityType */ \
+                               CreateEncoder,  /**< Create encoder function */ \
+                               DestroyEncoder, /**< Destroy encoder function */ \
+                               EncodeMedia,    /**< Encode media function */ \
+                               CreateDecoder,  /**< Create decoder function */ \
+                               DestroyDecoder, /**< Destroy decoder function */ \
+                               DecodeMedia,    /**< Decode media function */ \
+                               ControlsTable,  /**< Codec controls tables */ \
+                               Flags,          /**< Flags */ \
+                               RawFormat,      /**< Raw format */ \
+                               UserData        /**< User Data */ \
+                               ) \
+  { \
+    PLUGIN_CODEC_VERSION, &MyLicenseInfo, Flags, Description, RawFormat, MediaFormat, UserData, \
+    SampleRate, BitsPerSecond, FrameTime, {{ p1,p2,p3,p4 }}, PayloadType, PayloadName, \
+    CreateEncoder, DestroyEncoder, EncodeMedia, ControlsTable, H323type, H323data \
+  }, \
+  { \
+    PLUGIN_CODEC_VERSION, &MyLicenseInfo, Flags, Description, MediaFormat, RawFormat, UserData, \
+    SampleRate, BitsPerSecond, FrameTime, {{ p1,p2,p3,p4 }}, PayloadType, PayloadName, \
+    CreateDecoder, DestroyDecoder, DecodeMedia, ControlsTable, H323type, H323data \
+  }
+
+#define PLUGINCODEC_AUDIO_CODEC(MediaFormat,     /**< Media Format */ \
+                                PayloadName,     /**< IANA RTP payload code */ \
+                                Description,     /**< Description text */ \
+                                SampleRate,      /**< Sample rate */ \
+                                BitsPerSecond,   /**< Maximum bits per second */ \
+                                SamplesPerFrame, /**< Samples per audio frame */ \
+                                RecFramesPerPacket, /**< Recommended frames per packet */ \
+                                MaxFramesPerPacket, /**< Maximum frames per packet */ \
+                                RtpFlags,         /**< Extra flags typically if RTP payload type is fixed */ \
+                                PayloadType,      /**< IANA RTP payload type code */ \
+                                H323type,         /**< h323CapabilityType enumeration */ \
+                                H323data,         /**< Data to go with h323CapabilityType */ \
+                                CreateEncoder,    /**< Create encoder function */ \
+                                DestroyEncoder,   /**< Destroy encoder function */ \
+                                EncodeAudio,      /**< Encode media function */ \
+                                CreateDecoder,    /**< Create decoder function */ \
+                                DestroyDecoder,   /**< Destroy decoder function */ \
+                                DecodeAudio,      /**< Decode media function */ \
+                                ControlsTable     /**< Codec controls tables */ \
+                                ) \
+         PLUGINCODEC_CODEC_PAIR(MediaFormat, \
+                                PayloadName, \
+                                Description, \
+                                SampleRate, \
+                                BitsPerSecond, \
+                                (SamplesPerFrame)*1000000/(SampleRate), \
+                                SamplesPerFrame, \
+                                (BitsPerSecond)*(SampleRate)/(SamplesPerFrame)/8, \
+                                RecFramesPerPacket, \
+                                MaxFramesPerPacket, \
+                                PayloadType, \
+                                H323type, \
+                                H323data, \
+                                CreateEncoder, \
+                                DestroyEncoder, \
+                                EncodeAudio, \
+                                CreateDecoder, \
+                                DestroyDecoder, \
+                                DecodeAudio, \
+                                ControlsTable, \
+                                PluginCodec_MediaTypeAudio | /* audio codec */ \
+                                PluginCodec_InputTypeRaw |   /* raw input data */ \
+                                PluginCodec_OutputTypeRaw |  /* raw output data */ \
+                                (RtpFlags), \
+                                PLUGINCODEC_RAW_AUDIO, \
+                                NULL)
+
+#define PLUGINCODEC_ONE_AUDIO_CODEC(MediaFormat,     /**< Media Format */ \
+                                    PayloadName,     /**< IANA RTP payload code */ \
+                                    Description,     /**< Description text */ \
+                                    SampleRate,      /**< Sample rate */ \
+                                    BitsPerSecond,   /**< Maximum bits per second */ \
+                                    SamplesPerFrame, /**< Samples per audio frame */ \
+                                    RecFramesPerPacket, /**< Recommended frames per packet */ \
+                                    MaxFramesPerPacket, /**< Maximum frames per packet */ \
+                                    RtpFlags,         /**< Extra flags typically if RTP payload type is fixed */ \
+                                    PayloadType,      /**< IANA RTP payload type code */ \
+                                    H323type,         /**< h323CapabilityType enumeration */ \
+                                    H323data          /**< Data to go with h323CapabilityType */ \
+                                ) \
+    static struct PluginCodec_Definition CodecDefinitionTable[] = { \
+            PLUGINCODEC_AUDIO_CODEC(MediaFormat, \
+                                    PayloadName, \
+                                    Description, \
+                                    SampleRate, \
+                                    BitsPerSecond, \
+                                    SamplesPerFrame, \
+                                    RecFramesPerPacket, \
+                                    MaxFramesPerPacket, \
+                                    RtpFlags, \
+                                    PayloadType, \
+                                    H323type, \
+                                    H323data, \
+                                    MyCreateEncoder, \
+                                    MyDestroyEncoder, \
+                                    MyEncodeAudio, \
+                                    MyCreateDecoder, \
+                                    MyDestroyDecoder, \
+                                    MyDecodeAudio, \
+                                    MyControlsTable \
+                                    ) \
+    }
+
+#define PLUGINCODEC_VIDEO_CODEC(MediaFormat,     /**< Media Format */ \
+                                PayloadName,     /**< IANA RTP payload code */ \
+                                Description,     /**< Description text */ \
+                                BitsPerSecond,   /**< Maximum bits per second */ \
+                                MaxWidth,        /**< Max resolution (width) */ \
+                                MaxHeight,       /**< Max resolution (height) */ \
+                                RtpFlags,        /**< Extra flags typically if RTP payload type is fixed */ \
+                                PayloadType,     /**< IANA RTP payload type code */ \
+                                H323type,        /**< h323CapabilityType enumeration */ \
+                                H323data,        /**< Data to go with h323CapabilityType */ \
+                                CreateEncoder,   /**< Create encoder function */ \
+                                DestroyEncoder,  /**< Destroy encoder function */ \
+                                EncodeVideo,     /**< Encode media function */ \
+                                CreateDecoder,   /**< Create decoder function */ \
+                                DestroyDecoder,  /**< Destroy decoder function */ \
+                                DecodeVideo,     /**< Decode media function */ \
+                                ControlsTable    /**< Codec controls tables */ \
+                                ) \
+         PLUGINCODEC_CODEC_PAIR(MediaFormat, \
+                                PayloadName, \
+                                Description, \
+                                SampleRate, \
+                                BitsPerSecond, \
+                                PLUGINCODEC_VIDEO_CLOCK, \
+                                BitsPerSecond, \
+                                1000000/PLUGINCODEC_MAX_FRAME_RATE, \
+                                MaxWidth, \
+                                MaxHeight, \
+                                0,PLUGINCODEC_MAX_FRAME_RATE, \
+                                PayloadType, \
+                                H323type, \
+                                H323data, \
+                                CreateEncoder, \
+                                DestroyEncoder, \
+                                EncodeVideo, \
+                                CreateDecoder, \
+                                DestroyDecoder, \
+                                DecodeVideo, \
+                                ControlsTable, \
+                                PluginCodec_MediaTypeVideo | /* video codec */ \
+                                PluginCodec_InputTypeRTP |   /* RTP input data */ \
+                                PluginCodec_OutputTypeRTP |  /* RTP output data */ \
+                                (RtpFlags), \
+                                PLUGINCODEC_RAW_VIDEO, \
+                                NULL)
+
+#define PLUGINCODEC_ONE_VIDEO_CODEC(MediaFormat,     /**< Media Format */ \
+                                    PayloadName,     /**< IANA RTP payload code */ \
+                                    Description,     /**< Description text */ \
+                                    BitsPerSecond,   /**< Maximum bits per second */ \
+                                    MaxWidth,        /**< Max resolution (width) */ \
+                                    MaxHeight,       /**< Max resolution (height) */ \
+                                    RtpFlags,        /**< Extra flags typically if RTP payload type is fixed */ \
+                                    PayloadType,     /**< IANA RTP payload type code */ \
+                                    H323type,        /**< h323CapabilityType enumeration */ \
+                                    H323data         /**< Data to go with h323CapabilityType */ \
+                                ) \
+    static struct PluginCodec_Definition CodecDefinitionTable[] = { \
+            PLUGINCODEC_VIDEO_CODEC(MediaFormat, \
+                                    PayloadName, \
+                                    Description, \
+                                    BitsPerSecond, \
+                                    MaxWidth, \
+                                    MaxHeight, \
+                                    RtpFlags, \
+                                    PayloadType, \
+                                    H323type, \
+                                    H323data, \
+                                    CreateEncoder, \
+                                    DestroyEncoder, \
+                                    EncodeAudio, \
+                                    CreateDecoder, \
+                                    DestroyDecoder, \
+                                    DecodeAudio, \
+                                    ControlsTable \
+                                    ) \
+    }
+
+
+//////////////////////////////////////////////////////////////////
 //
 //  H.323 specific values
 //
@@ -595,16 +853,37 @@ enum {
 #define PluginCodec_RTP_MinHeaderSize  (12)
 #define PluginCodec_RTP_MaxPayloadSize (PluginCodec_RTP_MaxPacketSize - PluginCodec_RTP_MinHeaderSize)
 
-#define PluginCodec_RTP_GetHeaderLength(ptr)      ((((unsigned char*)(ptr))[0] & 0x0f)*4 + PluginCodec_RTP_MinHeaderSize)
-#define PluginCodec_RTP_GetPayloadPtr(ptr)          ((unsigned char*)(ptr) + PluginCodec_RTP_GetHeaderLength(ptr))
-#define PluginCodec_RTP_GetPayloadType(ptr)        (((unsigned char*)(ptr))[1] & 0x7f)
-#define PluginCodec_RTP_SetPayloadType(ptr, type)  (((unsigned char*)(ptr))[1] = (((unsigned char*)(ptr))[1] & 0x80) | (type & 0x7f))
-#define PluginCodec_RTP_GetMarker(ptr)            ((((unsigned char*)(ptr))[1] & 0x80) != 0)
-#define PluginCodec_RTP_SetMarker(ptr, mark)       (((unsigned char*)(ptr))[1] = (((unsigned char*)(ptr))[1] & 0x7f) | (mark != 0 ? 0x80 : 0))
-#define PluginCodec_RTP_GetTimestamp(ptr)         ((((unsigned char*)(ptr))[4] << 24) | (((unsigned char*)(ptr))[5] << 16) | (((unsigned char*)(ptr))[6] << 8) | ((unsigned char*)(ptr))[7])
-#define PluginCodec_RTP_SetTimestamp(ptr, ts)     ((((unsigned char*)(ptr))[4] = ((ts) >> 24)),(((unsigned char*)(ptr))[5] = ((ts) >> 16)),(((unsigned char*)(ptr))[6] = ((ts) >> 8)),(((unsigned char*)(ptr))[7] = (ts)))
-#define PluginCodec_RTP_GetSequenceNumber(ptr)    ((((unsigned char*)(ptr))[2] << 8) | ((unsigned char*)(ptr))[3])
-#define PluginCodec_RTP_SetSequenceNumber(ptr, sn)((((unsigned char*)(ptr))[2] = ((sn) >> 8)),(((unsigned char*)(ptr))[3] = (sn)))
+#define PluginCodec_RTP_GetWORD(ptr, off)       ((((unsigned char*)(ptr))[off] << 8) | ((unsigned char*)(ptr))[off+1])
+
+#define PluginCodec_RTP_GetDWORD(ptr, off)      ((((unsigned char*)(ptr))[off  ] << 24)|\
+                                                 (((unsigned char*)(ptr))[off+1] << 16)|\
+                                                 (((unsigned char*)(ptr))[off+2] << 8 )|\
+                                                  ((unsigned char*)(ptr))[off+3])
+
+#define PluginCodec_RTP_SetWORD(ptr, off, val)  ((((unsigned char*)(ptr))[off  ] = (unsigned char)((val) >> 8 )),\
+                                                 (((unsigned char*)(ptr))[off+1] = (unsigned char) (val)      ))
+
+#define PluginCodec_RTP_SetDWORD(ptr, off, val) ((((unsigned char*)(ptr))[off  ] = (unsigned char)((val) >> 24)),\
+                                                 (((unsigned char*)(ptr))[off+1] = (unsigned char)((val) >> 16)),\
+                                                 (((unsigned char*)(ptr))[off+2] = (unsigned char)((val) >> 8 )),\
+                                                 (((unsigned char*)(ptr))[off+3] = (unsigned char) (val)      ))
+
+#define PluginCodec_RTP_GetCSRCHdrLength(ptr)      ((((unsigned char*)(ptr))[0] & 0x0f)*4 + PluginCodec_RTP_MinHeaderSize)
+#define PluginCodec_RTP_GetExtHdrLength(ptr)       ((((unsigned char*)(ptr))[0] & 0x10) ? (PluginCodec_RTP_GetWORD(ptr, PluginCodec_RTP_GetCSRCHdrLength(ptr)+2)*4+4) : 0)
+
+#define PluginCodec_RTP_GetHeaderLength(ptr)       (PluginCodec_RTP_GetCSRCHdrLength(ptr) + PluginCodec_RTP_GetExtHdrLength(ptr))
+#define PluginCodec_RTP_GetPayloadPtr(ptr)           ((unsigned char*)(ptr) + PluginCodec_RTP_GetHeaderLength(ptr))
+#define PluginCodec_RTP_GetPayloadType(ptr)         (((unsigned char*)(ptr))[1] & 0x7f)
+#define PluginCodec_RTP_SetPayloadType(ptr, type)   (((unsigned char*)(ptr))[1] = (unsigned char)((((unsigned char*)(ptr))[1] & 0x80) | (type & 0x7f)))
+#define PluginCodec_RTP_GetMarker(ptr)             ((((unsigned char*)(ptr))[1] & 0x80) != 0)
+#define PluginCodec_RTP_SetMarker(ptr, mark)        (((unsigned char*)(ptr))[1] = (unsigned char)((((unsigned char*)(ptr))[1] & 0x7f) | (mark != 0 ? 0x80 : 0)))
+#define PluginCodec_RTP_GetTimestamp(ptr)          PluginCodec_RTP_GetDWORD(ptr, 4)
+#define PluginCodec_RTP_SetTimestamp(ptr, ts)      PluginCodec_RTP_SetDWORD(ptr, 4, ts)
+#define PluginCodec_RTP_GetSequenceNumber(ptr)     PluginCodec_RTP_GetWORD(ptr, 2)
+#define PluginCodec_RTP_SetSequenceNumber(ptr, sn) PluginCodec_RTP_SetWORD(ptr, 2, sn)
+#define PluginCodec_RTP_GetSSRC(ptr)               PluginCodec_RTP_GetDWORD(ptr, 8)
+#define PluginCodec_RTP_SetSSRC(ptr, ssrc)         PluginCodec_RTP_SetDWORD(ptr, 8, ssrc)
+
 
 
 /////////////////
@@ -631,12 +910,6 @@ inline unsigned char * OPAL_VIDEO_FRAME_DATA_PTR(const PluginCodec_Video_FrameHe
 extern "C" {
 #endif
 
-#define PLUGIN_CODEC_VIDEO_SET_FRAME_SIZE_FN    "set_frame_size"    // argument is struct PluginCodec_VideoSetFrameInfo
-struct PluginCodec_Video_SetFrameInfo {
-  int width;
-  int height;
-};
-
 
 /////////////////
 //
@@ -650,14 +923,14 @@ struct PluginCodec_Video_SetFrameInfo {
 #  define PLUGIN_CODEC_IMPLEMENT(name) \
 unsigned int Opal_StaticCodec_##name##_GetAPIVersion() \
 { return PWLIB_PLUGIN_API_VERSION; } \
-static struct PluginCodec_Definition * PLUGIN_CODEC_GET_CODEC_FN(unsigned * count, unsigned /*version*/); \
-struct PluginCodec_Definition * Opal_StaticCodec_##name##_GetCodecs(unsigned * p1, unsigned p2) \
+static const struct PluginCodec_Definition * PLUGIN_CODEC_GET_CODEC_FN(unsigned * count, unsigned /*version*/); \
+struct const PluginCodec_Definition * Opal_StaticCodec_##name##_GetCodecs(unsigned * p1, unsigned p2) \
 { return PLUGIN_CODEC_GET_CODEC_FN(p1,p2); } \
 
 #  define PLUGIN_CODEC_IMPLEMENT_ALL(name, table, ver) \
 unsigned int Opal_StaticCodec_##name##_GetAPIVersion() \
 { return PWLIB_PLUGIN_API_VERSION; } \
-PLUGIN_CODEC_DLL_API struct PluginCodec_Definition * Opal_StaticCodec_##name##_GetCodecs(unsigned * count, unsigned version) \
+PLUGIN_CODEC_DLL_API const struct PluginCodec_Definition * Opal_StaticCodec_##name##_GetCodecs(unsigned * count, unsigned version) \
 { *count = sizeof(table)/sizeof(struct PluginCodec_Definition); return version < ver ? NULL : table; }
 
 
@@ -669,7 +942,7 @@ PLUGIN_CODEC_DLL_API unsigned int PLUGIN_CODEC_API_VER_FN() \
 
 #  define PLUGIN_CODEC_IMPLEMENT_ALL(name, table, ver) \
 PLUGIN_CODEC_IMPLEMENT(name) \
-PLUGIN_CODEC_DLL_API struct PluginCodec_Definition * PLUGIN_CODEC_GET_CODEC_FN(unsigned * count, unsigned version) \
+PLUGIN_CODEC_DLL_API const struct PluginCodec_Definition * PLUGIN_CODEC_GET_CODEC_FN(unsigned * count, unsigned version) \
 { *count = sizeof(table)/sizeof(struct PluginCodec_Definition); return version < ver ? NULL : table; }
 
 
