@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26036 $
+ * $Revision: 29929 $
  * $Author: rjongbloed $
- * $Date: 2011-06-16 20:40:56 -0500 (Thu, 16 Jun 2011) $
+ * $Date: 2013-06-12 02:11:18 -0500 (Wed, 12 Jun 2013) $
  */
 
 #ifndef OPAL_CODEC_OPALPLUGINMGR_H
@@ -100,7 +100,7 @@ class OpalPluginCodecHandler : public PObject
     virtual void RegisterVideoTranscoder(const PString & src, const PString & dst, const PluginCodec_Definition * codec, bool v);
 #endif
 
-#if OPAL_T38_CAPABILITY
+#if OPAL_FAX
     virtual OpalMediaFormatInternal * OnCreateFaxFormat(OpalPluginCodecManager & mgr,
                                           const PluginCodec_Definition * codecDefn,
                                                             const char * fmtName,
@@ -119,6 +119,8 @@ class OpalPluginCodecManager : public PPluginModuleManager
     OpalPluginCodecManager(PPluginManager * pluginMgr = NULL);
     ~OpalPluginCodecManager();
 
+    PFACTORY_GET_SINGLETON(PFactory<PPluginModuleManager>, OpalPluginCodecManager);
+
     void RegisterStaticCodec(const H323StaticPluginCodecFactory::Key_T & name,
                              PluginCodec_GetAPIVersionFunction getApiVerFn,
                              PluginCodec_GetCodecFunction getCodecFn);
@@ -220,7 +222,7 @@ class OpalPluginTranscoder
     OpalPluginTranscoder(const PluginCodec_Definition * defn, bool isEnc);
     ~OpalPluginTranscoder();
 
-    bool UpdateOptions(const OpalMediaFormat & fmt);
+    bool UpdateOptions(OpalMediaFormat & fmt);
     bool ExecuteCommand(const OpalMediaCommand & command);
     bool Transcode(const void * from, unsigned * fromLen, void * to, unsigned * toLen, unsigned * flags) const
     {
@@ -233,8 +235,11 @@ class OpalPluginTranscoder
     bool   isEncoder;
     void * context;
 
-    OpalPluginControl setCodecOptions;
+    OpalPluginControl setCodecOptionsControl;
+    OpalPluginControl getActiveOptionsControl;
+    OpalPluginControl freeOptionsControl;
     OpalPluginControl getOutputDataSizeControl;
+    OpalPluginControl getCodecStatistics;
 };
 
 
@@ -316,6 +321,10 @@ class OpalPluginVideoTranscoder : public OpalVideoTranscoder, public OpalPluginT
     OpalPluginVideoTranscoder(const PluginCodec_Definition * codec, bool isEncoder);
     ~OpalPluginVideoTranscoder();
 
+#if OPAL_STATISTICS
+    virtual void GetStatistics(OpalMediaStatistics & statistics) const;
+#endif
+
     PBoolean ConvertFrames(const RTP_DataFrame & src, RTP_DataFrameList & dstList);
     bool UpdateMediaFormats(const OpalMediaFormat & input, const OpalMediaFormat & output);
     PBoolean ExecuteCommand(const OpalMediaCommand & command);
@@ -326,10 +335,10 @@ class OpalPluginVideoTranscoder : public OpalVideoTranscoder, public OpalPluginT
     bool DecodeFrame(const RTP_DataFrame & src, RTP_DataFrameList & dstList);
 
     RTP_DataFrame * m_bufferRTP;
+    DWORD           m_lastSequenceNumber;
     DWORD           m_lastDecodedTimestamp; // For missing marker bit detection
     DWORD           m_lastMarkerTimestamp;  // For continuous marker bit detection
     bool            m_badMarkers;
-    PSimpleTimer    m_videoPictureLossTimer;
 
 #if PTRACING
     unsigned m_consecutiveIntraFrames;
@@ -340,7 +349,7 @@ class OpalPluginVideoTranscoder : public OpalVideoTranscoder, public OpalPluginT
 
 ///////////////////////////////////////////////////////////////////////////////
 
-#if OPAL_T38_CAPABILITY
+#if OPAL_FAX
 
 class OpalPluginFaxFormatInternal : public OpalMediaFormatInternal, public OpalPluginMediaFormatInternal
 {
@@ -357,7 +366,7 @@ class OpalPluginFaxFormatInternal : public OpalMediaFormatInternal, public OpalP
     virtual bool IsValidForProtocol(const PString & protocol) const;
 };
 
-#endif // OPAL_T38_CAPABILITY
+#endif // OPAL_FAX
 
 
 //////////////////////////////////////////////////////
@@ -489,15 +498,6 @@ class H323AudioPluginCapability : public H323AudioCapability,
     unsigned pluginSubType;
 };
 
-#define OPAL_DECLARE_EMPTY_AUDIO_CAPABILITY(fmt, type) \
-class fmt##_CapabilityRegisterer { \
-  public: \
-    fmt##_CapabilityRegisterer() \
-    { H323CapabilityFactory::Register(fmt, new H323AudioPluginCapability(fmt, fmt, type)); } \
-}; \
-
-#define OPAL_DEFINE_EMPTY_AUDIO_CAPABILITY(fmt) \
-static fmt##_CapabilityRegisterer fmt##_CapabilityRegisterer_instance; \
 
 //////////////////////////////////////////////////////////////////////////////
 //
@@ -516,15 +516,6 @@ class H323PluginG7231Capability : public H323AudioPluginCapability
     virtual PBoolean OnReceivedPDU(const H245_AudioCapability & cap,  unsigned & packetSize);
 };
 
-#define OPAL_DECLARE_EMPTY_G7231_CAPABILITY(fmt, annex) \
-class fmt##_CapabilityRegisterer { \
-  public: \
-    fmt##_CapabilityRegisterer() \
-    { H323CapabilityFactory::Register(fmt, new H323PluginG7231Capability(fmt, annex)); } \
-}; \
-
-#define OPAL_DEFINE_EMPTY_G7231_CAPABILITY(fmt) \
-static fmt##_CapabilityRegisterer fmt##_CapabilityRegisterer_instance; \
 
 //////////////////////////////////////////////////////////////////////////////
 //
@@ -571,33 +562,6 @@ class H323CodecPluginGenericAudioCapability : public H323GenericAudioCapability,
 
 #if OPAL_VIDEO
 
-//////////////////////////////////////////////////////////////////////////////
-//
-// Class for handling most audio plugin capabilities
-//
-
-class H323VideoPluginCapability : public H323VideoCapability,
-                             public H323PluginCapabilityInfo
-{
-  PCLASSINFO(H323VideoPluginCapability, H323VideoCapability);
-  public:
-    H323VideoPluginCapability(const PluginCodec_Definition * codecDefn,
-                              const OpalMediaFormat & mediaFormat,
-                              unsigned _pluginSubType);
-
-    virtual PString GetFormatName() const;
-
-    virtual unsigned GetSubType() const;
-
-    static bool SetOptionsFromMPI(OpalMediaFormat & mediaFormat, int frameWidth, int frameHeight, int frameRate);
-
-    virtual void PrintOn(std::ostream & strm) const;
-
-  protected:
-    unsigned pluginSubType;
-    unsigned h323subType;   // only set if using capability without codec
-};
-
 //////////////////////////////////////////////////////////////////////////////
 //
 // Class for handling non standard video capabilities
@@ -646,16 +610,19 @@ class H323CodecPluginGenericVideoCapability : public H323GenericVideoCapability,
 // Class for handling H.261 plugin capabilities
 //
 
-class H323H261PluginCapability : public H323VideoPluginCapability
+class H323H261Capability : public H323VideoCapability
 {
-  PCLASSINFO(H323H261PluginCapability, H323VideoPluginCapability);
+    PCLASSINFO(H323H261Capability, H323VideoCapability);
   public:
-    H323H261PluginCapability(const PluginCodec_Definition * codecDefn, const OpalMediaFormat & mediaFormat);
+    H323H261Capability();
 
     Comparison Compare(const PObject & obj) const;
 
     virtual PObject * Clone() const;
 
+    virtual PString GetFormatName() const;
+    virtual unsigned GetSubType() const;
+
     virtual PBoolean OnSendingPDU(
       H245_VideoCapability & pdu  /// PDU to set information on
     ) const;
@@ -669,21 +636,35 @@ class H323H261PluginCapability : public H323VideoPluginCapability
     );
 };
 
+
+class H323H261PluginCapability : public H323H261Capability,
+                                 public H323PluginCapabilityInfo
+{
+    PCLASSINFO(H323H261PluginCapability, H323VideoCapability);
+  public:
+    H323H261PluginCapability(const PluginCodec_Definition * codecDefn, const OpalMediaFormat & mediaFormat);
+
+    virtual PObject * Clone() const;
+};
+
+
 //////////////////////////////////////////////////////////////////////////////
 //
 // Class for handling H.263 plugin capabilities
 //
 
-class H323H263PluginCapability : public H323VideoPluginCapability
+class H323H263Capability : public H323VideoCapability
 {
-  PCLASSINFO(H323H263PluginCapability, H323VideoPluginCapability);
   public:
-    H323H263PluginCapability(const PluginCodec_Definition * codecDefn, const OpalMediaFormat & mediaFormat);
+    H323H263Capability(const PString & variant);
 
     Comparison Compare(const PObject & obj) const;
 
     virtual PObject * Clone() const;
 
+    virtual PString GetFormatName() const;
+    virtual unsigned GetSubType() const;
+
     virtual PBoolean OnSendingPDU(
       H245_VideoCapability & pdu  /// PDU to set information on
     ) const;
@@ -695,7 +676,25 @@ class H323H263PluginCapability : public H323VideoPluginCapability
     virtual PBoolean OnReceivedPDU(
       const H245_VideoCapability & pdu  /// PDU to get information from
     );
-    virtual PBoolean IsMatch(const PASN_Choice & subTypePDU) const;
+    virtual PBoolean IsMatch(
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
+    ) const;
+
+  protected:
+    PString m_variant;
+};
+
+class H323H263PluginCapability : public H323H263Capability,
+                                 public H323PluginCapabilityInfo
+{
+    PCLASSINFO(H323H263PluginCapability, H323VideoCapability);
+  public:
+    H323H263PluginCapability(const PluginCodec_Definition * codecDefn, const OpalMediaFormat & mediaFormat)
+      : H323H263Capability(mediaFormat)
+      , H323PluginCapabilityInfo(codecDefn, mediaFormat)
+    {
+    }
 };
 
 #endif // OPAL_VIDEO
