@@ -22,9 +22,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25239 $
+ * $Revision: 27199 $
  * $Author: rjongbloed $
- * $Date: 2011-03-03 07:41:53 -0600 (Thu, 03 Mar 2011) $
+ * $Date: 2012-03-15 02:19:15 -0500 (Thu, 15 Mar 2012) $
  */
 
 #ifndef OPAL_CODEC_RFC2833_H
@@ -36,15 +36,34 @@
 
 #include <opal/buildopts.h>
 
-#include <rtp/rtp.h>
+#include <opal/mediafmt.h>
+#include <rtp/rtp_session.h>
 
 
-class OpalMediaFormat;
+class OpalRTPConnection;
 
 
 ///////////////////////////////////////////////////////////////////////////////
 
-class OpalRFC2833Info : public PObject {
+class OpalRFC2833EventsMask : public std::vector<bool>
+{
+  public:
+    enum { NumEvents = 256 };
+    OpalRFC2833EventsMask(bool defaultValue = false);
+    OpalRFC2833EventsMask(const char * defaultValues);
+    OpalRFC2833EventsMask & operator&=(const OpalRFC2833EventsMask & other);
+    friend ostream & operator<<(ostream & strm, const OpalRFC2833EventsMask & mask);
+    friend istream & operator>>(istream & strm,       OpalRFC2833EventsMask & mask);
+};
+
+typedef OpalMediaOptionValue<OpalRFC2833EventsMask> OpalRFC288EventsOption;
+const PCaselessString & OpalRFC288EventsName();
+
+
+///////////////////////////////////////////////////////////////////////////////
+
+class OpalRFC2833Info : public PObject
+{
     PCLASSINFO(OpalRFC2833Info, PObject);
   public:
     // the following values are mandated by RFC 2833
@@ -87,13 +106,14 @@ class OpalRFC2833Info : public PObject {
     unsigned timestamp;
 };
 
-class OpalRTPConnection;
 
-class OpalRFC2833Proto : public PObject {
+///////////////////////////////////////////////////////////////////////////////
+
+class OpalRFC2833Proto : public PObject
+{
     PCLASSINFO(OpalRFC2833Proto, PObject);
   public:
     OpalRFC2833Proto(
-      OpalRTPConnection & conn,
       const PNotifier & receiveNotifier,
       const OpalMediaFormat & mediaFormat
     );
@@ -117,18 +137,12 @@ class OpalRFC2833Proto : public PObject {
       unsigned timestamp
     );
 
-    RTP_DataFrame::PayloadTypes GetPayloadType() const { return m_payloadType; }
+    void UseRTPSession(bool rx, OpalRTPSession * session);
 
-    void SetPayloadType(
-      RTP_DataFrame::PayloadTypes type ///<  new payload type
-    ) { m_payloadType = type; }
-
-    const RTP_Session::FilterNotifier & GetReceiveHandler() const { return m_receiveHandler; }
-
-    PString GetTxCapability() const;
-    PString GetRxCapability() const;
-    void SetTxCapability(const PString & codes, bool merge);
-    void SetRxCapability(const PString & codes);
+    OpalMediaFormat GetTxMediaFormat() const;
+    OpalMediaFormat GetRxMediaFormat() const;
+    void SetTxMediaFormat(const OpalMediaFormat & mediaFormat);
+    void SetRxMediaFormat(const OpalMediaFormat & mediaFormat);
 
     static PINDEX ASCIIToRFC2833(char tone, bool hasNSE);
     static char RFC2833ToASCII(PINDEX rfc2833, bool hasNSE);
@@ -140,12 +154,13 @@ class OpalRFC2833Proto : public PObject {
     PDECLARE_NOTIFIER(PTimer, OpalRFC2833Proto, ReceiveTimeout);
     PDECLARE_NOTIFIER(PTimer, OpalRFC2833Proto, AsyncTimeout);
 
-    OpalRTPConnection         & m_connection;
-    RTP_DataFrame::PayloadTypes m_payloadType;
-    std::vector<bool>           m_txCapabilitySet;
-    std::vector<bool>           m_rxCapabilitySet;
+    OpalMediaFormat             m_baseMediaFormat;
+    RTP_DataFrame::PayloadTypes m_txPayloadType;
+    RTP_DataFrame::PayloadTypes m_rxPayloadType;
+    OpalRFC2833EventsMask       m_txEvents;
+    OpalRFC2833EventsMask       m_rxEvents;
     PNotifier                   m_receiveNotifier;
-    RTP_Session::FilterNotifier m_receiveHandler;
+    OpalRTPSession::FilterNotifier m_receiveHandler;
 
     enum {
       ReceiveIdle,
@@ -153,6 +168,7 @@ class OpalRFC2833Proto : public PObject {
       ReceiveEnding
     } m_receiveState;
 
+    PMutex   m_receiveMutex;
     BYTE     m_receivedTone;
     unsigned m_tonesReceived;
     PTimer   m_receiveTimer;
@@ -166,16 +182,15 @@ class OpalRFC2833Proto : public PObject {
       TransmitEnding3,
     } m_transmitState;
 
-    RTP_Session * m_rtpSession;
-    PTimer        m_asyncTransmitTimer;
-    PTimer        m_asyncDurationTimer;
-    DWORD         m_transmitTimestamp;
-    bool          m_rewriteTransmitTimestamp;
-    PTimeInterval m_asyncStart;
-    BYTE          m_transmitCode;
-    unsigned      m_transmitDuration;
-
-    PMutex m_mutex;
+    PMutex           m_sendMutex;
+    OpalRTPSession * m_rtpSession;
+    PTimer           m_asyncTransmitTimer;
+    PTimer           m_asyncDurationTimer;
+    DWORD            m_transmitTimestamp;
+    bool             m_rewriteTransmitTimestamp;
+    PTimeInterval    m_asyncStart;
+    BYTE             m_transmitCode;
+    unsigned         m_transmitDuration;
 };
 
 
