@@ -22,9 +22,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23735 $
+ * $Revision: 29230 $
  * $Author: rjongbloed $
- * $Date: 2009-10-30 21:20:26 -0500 (Fri, 30 Oct 2009) $
+ * $Date: 2013-03-10 20:20:22 -0500 (Sun, 10 Mar 2013) $
  */
 
 #ifndef OPAL_CODEC_SILENCEDETECT_H
@@ -44,16 +44,16 @@ class OpalSilenceDetector : public PObject
 {
     PCLASSINFO(OpalSilenceDetector, PObject);
   public:
-    enum Mode {
+    P_DECLARE_ENUM(Modes,
       NoSilenceDetection,
       FixedSilenceDetection,
-      AdaptiveSilenceDetection,
-      NumModes
-    };
+      AdaptiveSilenceDetection
+    );
+    typedef Modes Mode; // Backward compatibility
 
     struct Params {
       Params(
-        Mode mode = AdaptiveSilenceDetection, ///<  New silence detection mode
+        Modes mode = AdaptiveSilenceDetection, ///<  New silence detection mode
         unsigned threshold = 0,               ///<  Threshold value if FixedSilenceDetection
         unsigned signalDeadband = 10,         ///<  10 milliseconds of signal needed
         unsigned silenceDeadband = 400,       ///<  400 milliseconds of silence needed
@@ -66,7 +66,7 @@ class OpalSilenceDetector : public PObject
           m_adaptivePeriod(adaptivePeriod)
         { }
 
-      Mode     m_mode;             /// Silence detection mode
+      Modes    m_mode;             /// Silence detection mode
       unsigned m_threshold;        /// Threshold value if FixedSilenceDetection
       unsigned m_signalDeadband;   /// milliseconds of signal needed
       unsigned m_silenceDeadband;  /// milliseconds of silence needed
@@ -107,14 +107,14 @@ class OpalSilenceDetector : public PObject
       const int clockRate     ///< Sampling clock rate for the preprocessor
     );
 
-    /**Get the cyrrent sampling clock rate.
+    /**Get the current sampling clock rate.
       */
     int GetClockRate() const { return clockRate; }
 
     /**Get silence detection status
 
-       The inTalkBurst value is PTrue if packet transmission is enabled and
-       PFalse if it is being suppressed due to silence.
+       The inTalkBurst value is true if packet transmission is enabled and
+       false if it is being suppressed due to silence.
 
        The currentThreshold value is the value from 0 to 32767 which is used
        as the threshold value for 16 bit PCM data.
