@@ -23,9 +23,9 @@
  *
  * Contributor(s): 
  *
- * $Revision: 26320 $
+ * $Revision: 29334 $
  * $Author: rjongbloed $
- * $Date: 2011-08-17 06:58:09 -0500 (Wed, 17 Aug 2011) $
+ * $Date: 2013-03-26 22:08:07 -0500 (Tue, 26 Mar 2013) $
  */
 
 #ifndef OPAL_CODEC_VIDCODEC_H
@@ -121,6 +121,8 @@ class OpalVideoTranscoder : public OpalTranscoder
 
        The default behaviour checks for a OpalVideoUpdatePicture and sets the
        updatePicture member variable if that is the command.
+
+       @returns true if command is handled.
       */
     virtual PBoolean ExecuteCommand(
       const OpalMediaCommand & command    ///<  Command to execute.
@@ -145,11 +147,13 @@ class OpalVideoTranscoder : public OpalTranscoder
     virtual void GetStatistics(OpalMediaStatistics & statistics) const;
 #endif
 
+    virtual bool HasErrorConcealment() const  { return m_errorConcealment; }
     bool WasLastFrameIFrame() const { return lastFrameWasIFrame; }
     void ForceIFrame()              { forceIFrame = true; }
   //@}
 
   protected:
+    bool   m_errorConcealment;
     PINDEX inDataSize;
     PINDEX outDataSize;
     bool   forceIFrame;
@@ -171,7 +175,7 @@ OPAL_DEFINE_MEDIA_COMMAND(OpalVideoFreezePicture, "Freeze Picture");
   */
 class OpalVideoUpdatePicture : public OpalMediaCommand
 {
-  PCLASSINFO(OpalVideoUpdatePicture, OpalMediaCommand);
+    PCLASSINFO_WITH_CLONE(OpalVideoUpdatePicture, OpalMediaCommand);
   public:
     virtual PString GetName() const;
 };
@@ -183,7 +187,7 @@ class OpalVideoUpdatePicture : public OpalMediaCommand
   */
 class OpalVideoPictureLoss : public OpalVideoUpdatePicture
 {
-  PCLASSINFO(OpalVideoPictureLoss, OpalVideoUpdatePicture);
+    PCLASSINFO_WITH_CLONE(OpalVideoPictureLoss, OpalVideoUpdatePicture);
   public:
     OpalVideoPictureLoss(unsigned sequenceNumber = 0, unsigned timestamp = 0);
 
@@ -201,22 +205,28 @@ class OpalVideoPictureLoss : public OpalVideoUpdatePicture
 /**This is an indication to the video encoder to change the way it is to
    encode the video stream.
 
-   The trade off integer is a value that trade spatial quality (blockiness)
-   for temparaly quality (frame rate). A value of zero is maxium spatial
-   quality and a value fo 31 is maximum frame rate.
+   The trade off integer is a value that trade spatial quality (blocky-ness)
+   for temporal quality (frame rate). A value of zero will maximise spatial
+   quality, sacrificing frame rate and a value of 31 will maximize frame rate
+   and sacrifice 
   */
 class OpalTemporalSpatialTradeOff : public OpalMediaCommand
 {
-  PCLASSINFO(OpalTemporalSpatialTradeOff, OpalMediaCommand);
+    PCLASSINFO_WITH_CLONE(OpalTemporalSpatialTradeOff, OpalMediaCommand);
   public:
     OpalTemporalSpatialTradeOff(int tradeoff) : m_tradeOff(tradeoff) { }
 
     virtual PString GetName() const;
 
-    int GetTradeOff() const { return m_tradeOff; }
+    enum {
+      MaximiseSpatial = 0,
+      MaximiseTemporal = 31
+    };
+
+    unsigned GetTradeOff() const { return m_tradeOff; }
 
   protected:
-    int m_tradeOff;
+    unsigned m_tradeOff;
 };
 
 
