@@ -18,9 +18,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23268 $
+ * $Revision: 27410 $
  * $Author: rjongbloed $
- * $Date: 2009-08-25 03:37:19 -0500 (Tue, 25 Aug 2009) $
+ * $Date: 2012-04-02 02:36:39 -0500 (Mon, 02 Apr 2012) $
  */
 
 #ifndef OPAL_H224_H224HANDLER_H
@@ -92,7 +92,7 @@ public:
   /**Return whether this client has extra capabilities.
      Default returns FALSE.
     */
-  virtual PBoolean HasExtraCapabilities() const { return PFalse; }
+  virtual PBoolean HasExtraCapabilities() const { return false; }
 
   /**Called if the CME client received an Extra Capabilities PDU for this client.
      Default does nothing.
@@ -219,24 +219,25 @@ private:
 
 class OpalH224MediaStream : public OpalMediaStream
 {
-  PCLASSINFO(OpalH224MediaStream, OpalMediaStream);
-  
-public:
-  OpalH224MediaStream(OpalConnection & connection,
-                      OpalH224Handler & h224Handler,
-                      const OpalMediaFormat & mediaFormat,
-                      unsigned sessionID,
-                      PBoolean isSource);
-  ~OpalH224MediaStream();
-  
-  virtual void OnStartMediaPatch();
-  virtual PBoolean Close();
-  virtual PBoolean ReadPacket(RTP_DataFrame & packet);
-  virtual PBoolean WritePacket(RTP_DataFrame & packet);
-  virtual PBoolean IsSynchronous() const { return PFalse; }
-  virtual PBoolean RequiresPatchThread() const { return isSource ? PFalse : PTrue; }
-  
-private:
+    PCLASSINFO(OpalH224MediaStream, OpalMediaStream);
+    
+  public:
+    OpalH224MediaStream(OpalConnection & connection,
+                        OpalH224Handler & h224Handler,
+                        const OpalMediaFormat & mediaFormat,
+                        unsigned sessionID,
+                        PBoolean isSource);
+    ~OpalH224MediaStream();
+    
+    virtual void OnStartMediaPatch();
+    virtual PBoolean ReadPacket(RTP_DataFrame & packet);
+    virtual PBoolean WritePacket(RTP_DataFrame & packet);
+    virtual PBoolean IsSynchronous() const { return false; }
+    virtual PBoolean RequiresPatchThread() const { return IsSink(); }
+    
+  private:
+    virtual void InternalClose();
+
     OpalH224Handler & h224Handler;
 };
 
