@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23943 $
+ * $Revision: 28351 $
  * $Author: rjongbloed $
- * $Date: 2010-01-17 19:22:38 -0600 (Sun, 17 Jan 2010) $
+ * $Date: 2012-09-13 19:58:35 -0500 (Thu, 13 Sep 2012) $
  */
 
 #ifndef OPAL_H323_CHANNELS_H
@@ -53,13 +53,13 @@ class H245_OpenLogicalChannel_forwardLogicalChannelParameters;
 class H245_OpenLogicalChannel_reverseLogicalChannelParameters;
 class H245_H2250LogicalChannelParameters;
 class H245_H2250LogicalChannelAckParameters;
+class H245_ArrayOf_GenericInformation;
 class H245_MiscellaneousCommand_type;
 class H245_MiscellaneousIndication_type;
 
 class H323EndPoint;
 class H323Connection;
 class H323Capability;
-class H323_RTP_Session;
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -72,7 +72,7 @@ class H323ChannelNumber : public PObject
   PCLASSINFO(H323ChannelNumber, PObject);
 
   public:
-    H323ChannelNumber() { number = 0; fromRemote = PFalse; }
+    H323ChannelNumber() { number = 0; fromRemote = false; }
     H323ChannelNumber(unsigned number, PBoolean fromRemote);
 
     virtual PObject * Clone() const;
@@ -153,7 +153,7 @@ class H323Channel : public PObject
        During OLC negotations teh master may change the session number being
        used for the logical channel.
 
-       Returns false if the sesion could not be renumbered.
+       Returns false if the session could not be renumbered.
       */
     virtual bool SetSessionID(
       unsigned sessionID   ///< New session ID
@@ -163,7 +163,7 @@ class H323Channel : public PObject
        This is primarily used to determine if media bypass is possible for the
        call between two connections.
 
-       The default behaviour returns PFalse.
+       The default behaviour returns false.
      */
     virtual PBoolean GetMediaTransportAddress(
       OpalTransportAddress & data,        ///<  Data channel address
@@ -172,23 +172,27 @@ class H323Channel : public PObject
 
     /**Set the initial bandwidth for the channel.
        This calculates the initial bandwidth required by the channel and
-       returns PTrue if the connection can support this bandwidth.
+       returns true if the connection can support this bandwidth.
 
        The default behaviour gets the bandwidth requirement from the codec
        object created by the channel.
      */
     virtual PBoolean SetInitialBandwidth() = 0;
 
+    /**PreOpen the channel.
+       This occurs when we send an OLC, full open happens when the OLC comes
+       back.
+
+       Default behaviour calls Open().
+      */
+    virtual bool PreOpen();
+
     /**Open the channel.
        The default behaviour just calls connection.OnStartLogicalChannel() and
        if successful sets the opened member variable.
       */
     virtual PBoolean Open();
 
-    /**This is called when the channel can start transferring data.
-     */
-    virtual PBoolean Start() = 0;
-
     /**This is called to clean up any threads on connection termination.
      */
     virtual void Close();
@@ -199,13 +203,18 @@ class H323Channel : public PObject
 
     /**Get the media stream associated with this logical channel.
 
-       If the argument is set to PTrue, the mediaStream is about to be deleted
+       If the argument is set to true, the mediaStream is about to be deleted
        so all internal references to the mediaStream must be removed.
 
        The default behaviour returns NULL.
       */
     virtual OpalMediaStreamPtr GetMediaStream() const;
 
+    /**Set the media stream associated with this logical channel.
+       The default behaviour does nothing.
+      */
+    virtual void SetMediaStream(OpalMediaStreamPtr mediaStream);
+
 
     /**Fill out the OpenLogicalChannel PDU for the particular channel type.
      */
@@ -227,7 +236,7 @@ class H323Channel : public PObject
        local machine via the H245LogicalChannelDict::Open() function, and
        the request has been acknowledged by the remote endpoint.
 
-       The default behaviour just returns PTrue.
+       The default behaviour just returns true.
      */
     virtual PBoolean OnReceivedPDU(
       const H245_OpenLogicalChannel & pdu,    ///<  Open PDU
@@ -238,7 +247,7 @@ class H323Channel : public PObject
        local machine via the H245LogicalChannelDict::Open() function, and
        the request has been acknowledged by the remote endpoint.
 
-       The default behaviour just returns PTrue.
+       The default behaviour just returns true.
      */
     virtual PBoolean OnReceivedAckPDU(
       const H245_OpenLogicalChannelAck & pdu  ///<  Acknowledgement PDU
@@ -295,43 +304,19 @@ class H323Channel : public PObject
 
     /**Get the bandwidth used by the channel in 100's of bits/sec.
      */
-    unsigned GetBandwidthUsed() const { return bandwidthUsed; }
+    OpalBandwidth GetBandwidthUsed() const { return m_bandwidthUsed; }
 
     /**Get the bandwidth used by the channel in 100's of bits/sec.
      */
-    PBoolean SetBandwidthUsed(
-      unsigned bandwidth  ///<  New bandwidth
+    bool SetBandwidthUsed(
+      OpalBandwidth bandwidth  ///<  New bandwidth
     );
 
     /**Get the capability that created this channel.
       */
     const H323Capability & GetCapability() const { return *capability; }
-
-    /**Get the "pause" flag.
-       A paused channel is one that prevents the annunciation of the channels
-       data. For example for audio this would mute the data, for video it would
-       still frame.
-
-       Note that channel is not stopped, and may continue to actually receive
-       data, it is just that nothing is done with it.
-      */
-    PBoolean IsPaused() const { return paused; }
-
-    /**Set the "pause" flag.
-       A paused channel is one that prevents the annunciation of the channels
-       data. For example for audio this would mute the data, for video it would
-       still frame.
-
-       Note that channel is not stopped, and may continue to actually receive
-       data, it is just that nothing is done with it.
-      */
-    void SetPause(
-      PBoolean pause   ///<  New pause flag
-    ) { paused = pause; }
   //@}
 
-    virtual void OnMediaCommand(OpalMediaCommand &) { }
-
   protected:
     virtual void InternalClose();
 
@@ -341,11 +326,10 @@ class H323Channel : public PObject
     H323ChannelNumber      number;
     H323ChannelNumber      reverseChannel;
     bool                   opened;
-    bool                   paused;
     PAtomicInteger         m_terminating;
 
   private:
-    unsigned bandwidthUsed;
+    OpalBandwidth m_bandwidthUsed;
 };
 
 
@@ -389,46 +373,48 @@ class H323UnidirectionalChannel : public H323Channel
 
     /**Set the initial bandwidth for the channel.
        This calculates the initial bandwidth required by the channel and
-       returns PTrue if the connection can support this bandwidth.
+       returns true if the connection can support this bandwidth.
 
        The default behaviour gets the bandwidth requirement from the codec
        object created by the channel.
      */
     virtual PBoolean SetInitialBandwidth();
 
+    /**PreOpen the channel.
+       This occurs when we send an OLC, full open happens when the OLC comes
+       back.
+
+       Default behaviour create media stream.
+      */
+    virtual bool PreOpen();
+
     /**Open the channel.
       */
     virtual PBoolean Open();
-
-    /**This is called when the channel can start transferring data.
-       The default action is to start one threads, with it either calling
-       Receive() or Transmit() depending on the receiver member variable.
-     */
-    virtual PBoolean Start();
-
-    /**Process a miscellaneous command on the logical channel.
-       The default behaviour passes this on to the codec if not NULL.
-     */
-    virtual void OnMiscellaneousCommand(
-      const H245_MiscellaneousCommand_type & type  ///<  Command to process
-    );
   //@}
 
   /**@name Member variable access */
   //@{
+    OpalMediaFormat GetMediaFormat() const { return m_mediaFormat; }
+
     /**Get the media stream associated with this logical channel.
-       The default behaviour returns NULL.
+       The default behaviour returns m_mediaStream.
       */
     virtual OpalMediaStreamPtr GetMediaStream() const;
+
+    /**Set the media stream associated with this logical channel.
+       The default behaviour sets m_mediaStream and m_mediaFormat.
+      */
+    virtual void SetMediaStream(OpalMediaStreamPtr mediaStream);
   //@}
 
-    void OnMediaCommand(OpalMediaCommand & command);
 
   protected:
     virtual void InternalClose();
 
     bool               receiver;
-    OpalMediaStreamPtr mediaStream;
+    OpalMediaFormat    m_mediaFormat;
+    OpalMediaStreamPtr m_mediaStream;
 };
 
 
@@ -460,12 +446,6 @@ class H323BidirectionalChannel : public H323Channel
        direction for th latter case.
      */
     virtual Directions GetDirection() const;
-
-    /**This is called when the channel can start transferring data.
-       The default action is to start two threads, one calls Receive() and the
-       other calls Transmit().
-     */
-    virtual PBoolean Start();
   //@}
 };
 
@@ -506,6 +486,11 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
       H245_OpenLogicalChannelAck & ack        ///<  Acknowledgement PDU
     ) const;
 
+    virtual void OnSendOpenAck(
+      H245_H2250LogicalChannelAckParameters & param ///<  Acknowledgement PDU
+    ) const;
+
+
     /**This is called after a request to create a channel occurs from the
        local machine via the H245LogicalChannelDict::Open() function, and
        the request has been acknowledged by the remote endpoint.
@@ -538,12 +523,15 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
       H245_H2250LogicalChannelParameters & param  ///<  Open PDU to send.
     ) const;
 
+    /**Alternate RTP port information for Same NAT
+      */
+    virtual PBoolean OnSendingAltPDU(
+      H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
+    ) const = 0;
+
     /**This is called when request to create a channel is received from a
        remote machine and is about to be acknowledged.
      */
-    virtual void OnSendOpenAck(
-      H245_H2250LogicalChannelAckParameters & param ///<  Acknowledgement PDU
-    ) const;
 
     /**This is called after a request to create a channel occurs from the
        local machine via the H245LogicalChannelDict::Open() function, and
@@ -566,246 +554,14 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
       const H245_H2250LogicalChannelAckParameters & param ///<  Acknowledgement PDU
     );
 
-    /**Set the dynamic payload type used by this channel.
+    /**Alternate RTP port information for Same NAT
       */
-    virtual PBoolean SetDynamicRTPPayloadType(
-      int newType  ///<  New RTP payload type number
-    );
+    virtual PBoolean OnReceivedAckAltPDU(
+      const H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
+    ) = 0;
 
-    RTP_DataFrame::PayloadTypes GetDynamicRTPPayloadType() const { return rtpPayloadType; }
+    RTP_DataFrame::PayloadTypes GetDynamicRTPPayloadType() const;
   //@}
-
-  protected:
-    RTP_DataFrame::PayloadTypes rtpPayloadType;
-};
-
-
-///////////////////////////////////////////////////////////////////////////////
-
-/**This class is for encpsulating the IETF Real Time Protocol interface.
- */
-class H323_RTPChannel : public H323_RealTimeChannel
-{
-  PCLASSINFO(H323_RTPChannel, H323_RealTimeChannel);
-
-  public:
-  /**@name Construction */
-  //@{
-    /**Create a new channel.
-     */
-    H323_RTPChannel(
-      H323Connection & connection,        ///<  Connection to endpoint for channel
-      const H323Capability & capability,  ///<  Capability channel is using
-      Directions direction,               /// Direction of channel
-      RTP_Session & rtp                   /// RTP session for channel
-    );
-
-    /// Destroy the channel
-    ~H323_RTPChannel();
-  //@}
-
-  /**@name Overrides from class H323Channel */
-  //@{
-    /**Indicate the session number of the channel.
-       Return session for channel. This returns the session ID of the
-       RTP_Session member variable.
-     */
-    virtual unsigned GetSessionID() const;
-
-    /**Set the session number of the channel.
-       During OLC negotations teh master may change the session number being
-       used for the logical channel.
-
-       Returns false if the sesion could not be renumbered.
-      */
-    virtual bool SetSessionID(
-      unsigned sessionID   ///< New session ID
-    );
-  //@}
-
-  /**@name Overrides from class H323_RealTimeChannel */
-  //@{
-    /**Fill out the OpenLogicalChannel PDU for the particular channel type.
-     */
-    virtual PBoolean OnSendingPDU(
-      H245_H2250LogicalChannelParameters & param  ///<  Open PDU to send.
-    ) const;
-
-    /**This is called when request to create a channel is received from a
-       remote machine and is about to be acknowledged.
-     */
-    virtual void OnSendOpenAck(
-      H245_H2250LogicalChannelAckParameters & param ///<  Acknowledgement PDU
-    ) const;
-
-    /**This is called after a request to create a channel occurs from the
-       local machine via the H245LogicalChannelDict::Open() function, and
-       the request has been acknowledged by the remote endpoint.
-
-       The default behaviour sets the remote ports to send UDP packets to.
-     */
-    virtual PBoolean OnReceivedPDU(
-      const H245_H2250LogicalChannelParameters & param, ///<  Acknowledgement PDU
-      unsigned & errorCode                              ///<  Error on failure
-    );
-
-    /**This is called after a request to create a channel occurs from the
-       local machine via the H245LogicalChannelDict::Open() function, and
-       the request has been acknowledged by the remote endpoint.
-
-       The default behaviour sets the remote ports to send UDP packets to.
-     */
-    virtual PBoolean OnReceivedAckPDU(
-      const H245_H2250LogicalChannelAckParameters & param ///<  Acknowledgement PDU
-    );
-  //@}
-
-  protected:
-    RTP_Session      & rtpSession;
-    H323_RTP_Session & rtpCallbacks;
-};
-
-
-///////////////////////////////////////////////////////////////////////////////
-
-/**This class is for encpsulating the IETF Real Time Protocol interface as used
-by a remote host.
- */
-class H323_ExternalRTPChannel : public H323_RealTimeChannel
-{
-  PCLASSINFO(H323_ExternalRTPChannel, H323_RealTimeChannel);
-
-  public:
-  /**@name Construction */
-  //@{
-    /**Create a new channel.
-     */
-    H323_ExternalRTPChannel(
-      H323Connection & connection,        ///<  Connection to endpoint for channel
-      const H323Capability & capability,  ///<  Capability channel is using
-      Directions direction,               ///<  Direction of channel
-      unsigned sessionID                  ///<  Session ID for channel
-    );
-    /**Create a new channel.
-     */
-    H323_ExternalRTPChannel(
-      H323Connection & connection,        ///<  Connection to endpoint for channel
-      const H323Capability & capability,  ///<  Capability channel is using
-      Directions direction,               ///<  Direction of channel
-      unsigned sessionID,                 ///<  Session ID for channel
-      const H323TransportAddress & data,  ///<  Data address
-      const H323TransportAddress & control///<  Control address
-    );
-    /**Create a new channel.
-     */
-    H323_ExternalRTPChannel(
-      H323Connection & connection,        ///<  Connection to endpoint for channel
-      const H323Capability & capability,  ///<  Capability channel is using
-      Directions direction,               ///<  Direction of channel
-      unsigned sessionID,                 ///<  Session ID for channel
-      const PIPSocket::Address & ip,      ///<  IP address of media server
-      WORD dataPort                       ///<  Data port (control is dataPort+1)
-    );
-  //@}
-
-  /**@name Overrides from class H323Channel */
-  //@{
-    /**Indicate the session number of the channel.
-       Return session for channel. This returns the session ID of the
-       RTP_Session member variable.
-     */
-    virtual unsigned GetSessionID() const;
-
-    /**Get the media transport address for the connection.
-       This is primarily used to determine if media bypass is possible for the
-       call between two connections.
-
-       The default behaviour returns PFalse.
-     */
-    virtual PBoolean GetMediaTransportAddress(
-      OpalTransportAddress & data,        ///<  Data channel address
-      OpalTransportAddress & control      ///<  Control channel address
-    ) const;
-
-    /**Start the channel.
-      */
-    virtual PBoolean Start();
-
-    /**Handle channel data reception.
-
-       This is called by the thread started by the Start() function and is
-       typically a loop writing to the codec and reading from the transport
-       (eg RTP_session).
-      */
-    virtual void Receive();
-
-    /**Handle channel data transmission.
-
-       This is called by the thread started by the Start() function and is
-       typically a loop reading from the codec and writing to the transport
-       (eg an RTP_session).
-      */
-    virtual void Transmit();
-  //@}
-
-  /**@name Overrides from class H323_RealTimeChannel */
-  //@{
-    /**Fill out the OpenLogicalChannel PDU for the particular channel type.
-     */
-    virtual PBoolean OnSendingPDU(
-      H245_H2250LogicalChannelParameters & param  ///<  Open PDU to send.
-    ) const;
-
-    /**This is called when request to create a channel is received from a
-       remote machine and is about to be acknowledged.
-     */
-    virtual void OnSendOpenAck(
-      H245_H2250LogicalChannelAckParameters & param ///<  Acknowledgement PDU
-    ) const;
-
-    /**This is called after a request to create a channel occurs from the
-       local machine via the H245LogicalChannelDict::Open() function, and
-       the request has been acknowledged by the remote endpoint.
-
-       The default behaviour sets the remote ports to send UDP packets to.
-     */
-    virtual PBoolean OnReceivedPDU(
-      const H245_H2250LogicalChannelParameters & param, ///<  Acknowledgement PDU
-      unsigned & errorCode                              ///<  Error on failure
-    );
-
-    /**This is called after a request to create a channel occurs from the
-       local machine via the H245LogicalChannelDict::Open() function, and
-       the request has been acknowledged by the remote endpoint.
-
-       The default behaviour sets the remote ports to send UDP packets to.
-     */
-    virtual PBoolean OnReceivedAckPDU(
-      const H245_H2250LogicalChannelAckParameters & param ///<  Acknowledgement PDU
-    );
-  //@}
-
-    void SetExternalAddress(
-      const H323TransportAddress & data,  ///<  Data address
-      const H323TransportAddress & control///<  Control address
-    );
-
-    const H323TransportAddress & GetRemoteMediaAddress()        const { return remoteMediaAddress; }
-    const H323TransportAddress & GetRemoteMediaControlAddress() const { return remoteMediaControlAddress; }
-
-    PBoolean GetRemoteAddress(
-      PIPSocket::Address & ip,
-      WORD & dataPort
-    ) const;
-
-  protected:
-    void Construct(H323Connection & conn, unsigned id);
-
-    unsigned             sessionID;
-    H323TransportAddress externalMediaAddress;
-    H323TransportAddress externalMediaControlAddress;
-    H323TransportAddress remoteMediaAddress;
-    H323TransportAddress remoteMediaControlAddress;
 };
 
 
@@ -841,8 +597,6 @@ class H323DataChannel : public H323UnidirectionalChannel
   /**@name Overrides from class H323Channel */
   //@{
     /**Indicate the session number of the channel.
-       Return session for channel. This returns the session ID of the
-       RTP_Session member variable.
      */
     virtual unsigned GetSessionID() const;
 
@@ -890,16 +644,16 @@ class H323DataChannel : public H323UnidirectionalChannel
        This is called on receipt of an OpenLogicalChannel request.
 
        The default behaviour creates a compatible listener using the
-       connections control channel as a basis and returns PTrue if successful.
+       connections control channel as a basis and returns true if successful.
       */
     virtual PBoolean CreateListener();
 
     /**Create the H323Transport class to be used.
        This is called on receipt of an OpenLogicalChannelAck response. It
-       should not return PTrue unless the transport member variable is set.
+       should not return true unless the transport member variable is set.
 
        The default behaviour uses the connection signalling channel to create
-       the transport and returns PTrue if successful.
+       the transport and returns true if successful.
       */
     virtual PBoolean CreateTransport();
   //@}
