@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
+ * $Revision: 28549 $
  * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Date: 2012-11-16 05:31:10 -0600 (Fri, 16 Nov 2012) $
  */
 
 #ifndef OPAL_H323_GKCLIENT_H
@@ -42,6 +42,7 @@
 
 #if OPAL_H323
 
+#include <opal/mediafmt.h>
 #include <h323/h225ras.h>
 #include <h323/h235auth.h>
 
@@ -85,7 +86,7 @@ class H323Gatekeeper : public H225_RAS
     virtual PBoolean WriteTo(
       H323TransactionPDU & pdu,
       const H323TransportAddressArray & addresses,
-      PBoolean callback = PTrue
+      PBoolean callback = true
     );
   //@}
 
@@ -144,8 +145,8 @@ class H323Gatekeeper : public H225_RAS
     /**Register with gatekeeper.
      */
     PBoolean RegistrationRequest(
-      PBoolean autoReregister = PTrue,  ///<  Automatic register on unregister
-      PBoolean didGkDiscovery = PFalse  ///<  discovery procedure was done right before
+      PBoolean autoReregister = true,  ///<  Automatic register on unregister
+      PBoolean didGkDiscovery = false  ///<  discovery procedure was done right before
     );
 
     /**Unregister with gatekeeper.
@@ -171,7 +172,7 @@ class H323Gatekeeper : public H225_RAS
     struct AdmissionResponse {
       AdmissionResponse();
 
-      unsigned rejectReason;                      /// Reject reason if returns PFalse
+      unsigned rejectReason;                      /// Reject reason if returns false
 
       PBoolean gatekeeperRouted;                      /// Flag for call is through gk
       PINDEX endpointCount;                       /// Number of endpoints that can be returned
@@ -187,7 +188,7 @@ class H323Gatekeeper : public H225_RAS
     PBoolean AdmissionRequest(
       H323Connection & connection,      ///<  Connection we wish to change.
       AdmissionResponse & response,     ///<  Response parameters to ARQ
-      PBoolean ignorePreGrantedARQ = PFalse  ///<  Flag to force ARQ to be sent
+      PBoolean ignorePreGrantedARQ = false  ///<  Flag to force ARQ to be sent
     );
 
     /**Disengage request to gatekeeper.
@@ -201,7 +202,7 @@ class H323Gatekeeper : public H225_RAS
      */
     PBoolean BandwidthRequest(
       H323Connection & connection,    ///<  Connection we wish to change.
-      unsigned requestedBandwidth     ///<  New bandwidth wanted in 0.1kbps
+      OpalBandwidth requestedBandwidth     ///<  New bandwidth wanted in bps
     );
 
     /**Send an unsolicited info response to the gatekeeper.
@@ -242,7 +243,7 @@ class H323Gatekeeper : public H225_RAS
 
     /**Determine if the endpoint is registered with the gatekeeper.
       */
-    PBoolean IsRegistered() const { return registrationFailReason == RegistrationSuccessful; }
+    PBoolean IsRegistered() const { return m_registrationFailReason == RegistrationSuccessful; }
 
     enum RegistrationFailReasons {
       RegistrationSuccessful,
@@ -254,11 +255,16 @@ class H323Gatekeeper : public H225_RAS
       SecurityDenied,
       TransportError,
       NumRegistrationFailReasons,
-      RegistrationRejectReasonMask = 0x8000
+      GatekeeperRejectReasonMask = 0x4000,
+      RegistrationRejectReasonMask = 0x8000,
+      UnregistrationRejectReasonMask = 0x10000
     };
+    static PString GetRegistrationFailReasonString(RegistrationFailReasons reason);
+    friend ostream & operator<<(ostream & strm, RegistrationFailReasons reason) { return strm << GetRegistrationFailReasonString(reason); }
+
     /**Get the registration fail reason.
      */
-    RegistrationFailReasons GetRegistrationFailReason() const { return registrationFailReason; }
+    RegistrationFailReasons GetRegistrationFailReason() const { return m_registrationFailReason; }
 
     /**Get the gatekeeper name.
        The gets the name of the gatekeeper. It will be of the form id@address
@@ -290,6 +296,9 @@ class H323Gatekeeper : public H225_RAS
     { return gkRouteAddress; }
   //@}
 
+#if OPAL_H460
+    H460_FeatureSet & GetFeatures();
+#endif
 
   protected:
     bool StartGatekeeper(const H323TransportAddress & address);
@@ -327,38 +336,22 @@ class H323Gatekeeper : public H225_RAS
       unsigned unregisteredTag
     );
     
-    virtual H323Transport * CreateTransport(PIPSocket::Address bindng = PIPSocket::GetDefaultIpAny(), WORD port = 0, PBoolean reuseAddr = PFalse);
+    virtual H323Transport * CreateTransport(PIPSocket::Address bindng = PIPSocket::GetDefaultIpAny(), WORD port = 0, PBoolean reuseAddr = false);
     
     // Handling interface changes
     void OnAddInterface(const PIPSocket::InterfaceEntry & entry, PINDEX priority);
     void OnRemoveInterface(const PIPSocket::InterfaceEntry & entry, PINDEX priority);
-    void UpdateConnectionStatus();
-
+    bool SetListenerAddresses(H225_ArrayOf_TransportAddress & pdu);
 
     // Gatekeeper registration state variables
-    PBoolean     discoveryComplete;
+    bool     discoveryComplete;
     PString  endpointIdentifier;
-    RegistrationFailReasons registrationFailReason;
+    RegistrationFailReasons m_registrationFailReason;
+    void SetRegistrationFailReason(unsigned reason, unsigned commandMask);
+    void SetRegistrationFailReason(RegistrationFailReasons reason);
     
-    enum {
-      HighPriority = 80,
-      LowPriority  = 40,
-    };
-    class InterfaceMonitor : public PInterfaceMonitorClient
-    {
-      PCLASSINFO(InterfaceMonitor, PInterfaceMonitorClient);
-      
-      public:
-        InterfaceMonitor(H323Gatekeeper & gk, PINDEX priority);
-      
-      protected:
-        virtual void OnAddInterface(const PIPSocket::InterfaceEntry & entry);
-        virtual void OnRemoveInterface(const PIPSocket::InterfaceEntry & entry);
-        
-        H323Gatekeeper & gk;
-    };
-    InterfaceMonitor highPriorityMonitor;
-    InterfaceMonitor lowPriorityMonitor;
+    PDECLARE_InterfaceNotifier(H323Gatekeeper, OnHighPriorityInterfaceChange);
+    PDECLARE_InterfaceNotifier(H323Gatekeeper, OnLowPriorityInterfaceChange);
     
     class AlternateInfo : public PObject {
       PCLASSINFO(AlternateInfo, PObject);
@@ -385,7 +378,7 @@ class H323Gatekeeper : public H225_RAS
         AlternateInfo & operator=(const AlternateInfo &) { return *this; }
     };
     PSortedList<AlternateInfo> alternates;
-    PBoolean               alternatePermanent;
+    bool               alternatePermanent;
     PSemaphore         requestMutex;
     H235Authenticators authenticators;
 
@@ -397,14 +390,14 @@ class H323Gatekeeper : public H225_RAS
     H323TransportAddress gkRouteAddress;
 
     // Gatekeeper operation variables
-    PBoolean       autoReregister;
-    PBoolean       reregisterNow;
+    bool       autoReregister;
+    bool       reregisterNow;
     PTimer     timeToLive;
-    PBoolean       requiresDiscovery;
+    bool       requiresDiscovery;
     PTimer     infoRequestRate;
-    PBoolean       willRespondToIRR;
+    bool       willRespondToIRR;
     PThread  * monitor;
-    PBoolean       monitorStop;
+    bool       monitorStop;
     PSyncPoint monitorTickle;
 
     PDictionary<POrdinalKey, H323ServiceControlSession> serviceControlSessions;
