@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21293 $
+ * $Revision: 26465 $
  * $Author: rjongbloed $
- * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
+ * $Date: 2011-09-23 22:00:15 -0500 (Fri, 23 Sep 2011) $
  */
 
 #ifndef OPAL_H323_H225RAS_H
@@ -286,9 +286,11 @@ class H225_RAS : public H323Transactor
     virtual void OnSendRequestInProgress(H225_RequestInProgress &);
     virtual PBoolean OnReceiveRequestInProgress(const H323RasPDU &, const H225_RequestInProgress &);
     virtual PBoolean OnReceiveRequestInProgress(const H225_RequestInProgress &);
-	
-	virtual PBoolean OnSendFeatureSet(unsigned, H225_FeatureSet &) const { return PFalse; }
-	virtual void OnReceiveFeatureSet(unsigned, const H225_FeatureSet &) const {}
+    
+    virtual PBoolean OnSendFeatureSet(unsigned, H225_FeatureSet &) const { return false; }
+    virtual void OnReceiveFeatureSet(unsigned, const H225_FeatureSet &) const {}
+
+    virtual void DisableFeatureSet() const {}
 
 
     /**Handle unknown PDU type.
