@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24489 $
+ * $Revision: 29635 $
  * $Author: rjongbloed $
- * $Date: 2010-06-10 23:58:11 -0500 (Thu, 10 Jun 2010) $
+ * $Date: 2013-05-02 12:58:06 -0500 (Thu, 02 May 2013) $
  */
 
 #ifndef OPAL_H323_H323CAPS_H
@@ -71,6 +71,12 @@ class H245_CapabilityIdentifier;
 class H245_GenericCapability;
 class H245_GenericParameter;
 
+
+#if OPAL_H239
+  const OpalMediaFormat & GetH239VideoMediaFormat();
+#endif
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 /**This class describes the interface to a capability of the endpoint, usually
@@ -254,11 +260,11 @@ class H323Capability : public PObject
        descendent was created. This completes reading fields from the PDU
        into the classes members.
 
-       If the function returns PFalse then the received PDU codec description
+       If the function returns false then the received PDU codec description
        is not supported, so will be ignored.
        
          The default behaviour sets the capabilityDirection member variable
-         from the PDU and then returns PTrue. Note that this means it is very
+         from the PDU and then returns true. Note that this means it is very
          important to call the ancestor function when overriding.
      */
     virtual PBoolean OnReceivedPDU(
@@ -280,7 +286,8 @@ class H323Capability : public PObject
     /**Compare the PDU part of the capability.
       */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
 
     /**Validate that the capability is usable given the connection.
@@ -288,7 +295,7 @@ class H323Capability : public PObject
        application to determine if this capability should be used in TCS or
        OLC pdus.
 
-       The default behaviour returns PTrue.
+       The default behaviour returns true.
       */
     virtual PBoolean IsUsable(
       const H323Connection & connection
@@ -299,10 +306,10 @@ class H323Capability : public PObject
   //@{
     enum CapabilityDirection {
       e_Unknown,
-      e_Receive,
-      e_Transmit,
-      e_ReceiveAndTransmit,
-      e_NoDirection,
+      e_Receive,            ///< Receive capability
+      e_Transmit,           ///< Transmit only capability
+      e_ReceiveAndTransmit, ///< Symmetric capability
+      e_NoDirection,        ///< Ca;ability type has no direction
       NumCapabilityDirections
     };
 
@@ -333,13 +340,10 @@ class H323Capability : public PObject
     );
 
     /// Get the payload type for the capaibility
-    RTP_DataFrame::PayloadTypes GetPayloadType() const { return rtpPayloadType; }
+    RTP_DataFrame::PayloadTypes GetPayloadType() const { return GetMediaFormat().GetPayloadType(); }
 
     /// Set the payload type for the capaibility
-    void SetPayloadType(RTP_DataFrame::PayloadTypes pt) { rtpPayloadType = pt; }
-
-    /// Attach a QoS specification to this channel
-    virtual void AttachQoS(RTP_QOS *) { }
+    void SetPayloadType(RTP_DataFrame::PayloadTypes pt) { GetWritableMediaFormat().SetPayloadType(pt); }
   //@}
 
 #if PTRACING
@@ -352,10 +356,11 @@ class H323Capability : public PObject
 
     unsigned            assignedCapabilityNumber;  /// Unique ID assigned to capability
     CapabilityDirection capabilityDirection;
-    RTP_DataFrame::PayloadTypes rtpPayloadType;
+
+    mutable OpalMediaFormat m_mediaFormat;
 
   private:
-    mutable OpalMediaFormat m_mediaFormat;
+    P_REMOVE_VIRTUAL(PBoolean, IsMatch(const PASN_Choice &) const, false);
 
   friend class H323Capabilities;
 };
@@ -428,7 +433,7 @@ class H323NonStandardCapabilityInfo
 
     /**This function validates and uses the non-standard data field.
 
-       The default behaviour returns PTrue if data is equal to fixedData.
+       The default behaviour returns true if data is equal to fixedData.
       */
     virtual PBoolean OnReceivedPDU(
       const PBYTEArray & data  ///<  Data field in PDU received
@@ -499,6 +504,7 @@ class H323GenericCapabilityInfo
     );
 
     PBoolean IsMatch(
+      const OpalMediaFormat & mediaFormat,
       const H245_GenericCapability & param  ///< Non standard field in PDU received
     ) const;
     PObject::Comparison CompareInfo(
@@ -507,7 +513,7 @@ class H323GenericCapabilityInfo
 
 
     PString  m_identifier;
-    unsigned maxBitRate;
+    unsigned m_maxBitRate;
 };
 
 /**This class describes the interface to a codec that has channels based on
@@ -536,10 +542,6 @@ class H323RealTimeCapability : public H323Capability
     H323RealTimeCapability();
     H323RealTimeCapability(const H323RealTimeCapability &rtc);
     virtual ~H323RealTimeCapability();
-    void AttachQoS(RTP_QOS * _rtpqos);
-
-  protected:
-    RTP_QOS * rtpqos;
   //@}
 };
 
@@ -575,7 +577,7 @@ class H323AudioCapability : public H323RealTimeCapability
     /**Get the default RTP session.
        This function gets the default RTP session ID for the capability
        type. For example audio capabilities return the value
-       RTP_Session::DefaultAudioSessionID etc.
+       H323Capability::DefaultAudioSessionID etc.
 
        The default behaviour returns zero, indicating it is not an RTP
        based capability.
@@ -682,7 +684,7 @@ class H323AudioCapability : public H323RealTimeCapability
        descendent was created. This completes reading fields from the PDU
        into the classes members.
 
-       If the function returns PFalse then the received PDU codec description
+       If the function returns false then the received PDU codec description
        is not supported, so will be ignored.
        
        The default behaviour calls the OnReceivedPDU() that takes a
@@ -840,7 +842,8 @@ class H323NonStandardAudioCapability : public H323AudioCapability,
     /**Compare the nonStandardData part of the capability, if applicable.
       */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 };
@@ -864,7 +867,7 @@ class H323GenericAudioCapability : public H323AudioCapability,
       */
     H323GenericAudioCapability(
       const PString & capabilityId,    ///< generic codec identifier
-      PINDEX maxBitRate = 0               ///< maxBitRate parameter for the GenericCapability
+      unsigned fixedBitRate            ///< Fixed bit rate for audio
     );
   //@}
 
@@ -931,9 +934,34 @@ class H323GenericAudioCapability : public H323AudioCapability,
     /**Compare the generic part of the capability, if applicable.
      */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
+
+  protected:
+    bool m_fixedBitRate;
+};
+
+
+template <const char * oid, const OpalAudioFormat & fmtFunc(), unsigned fixedBitRate = 0>
+class H323GenericAudioCapabilityTemplate : public H323GenericAudioCapability
+{
+  public:
+    H323GenericAudioCapabilityTemplate()
+      : H323GenericAudioCapability(oid, fixedBitRate)
+    {
+    }
+
+    virtual PObject * Clone() const
+    {
+      return new H323GenericAudioCapabilityTemplate(*this);
+    }
+
+    virtual PString GetFormatName() const
+    {
+      return fmtFunc();
+    }
 };
 
 
@@ -963,7 +991,7 @@ class H323VideoCapability : public H323RealTimeCapability
     /**Get the default RTP session.
        This function gets the default RTP session ID for the capability
        type. For example audio capabilities return the value
-       RTP_Session::DefaultAudioSessionID etc.
+       H323Capability::DefaultAudioSessionID etc.
 
        The default behaviour returns zero, indicating it is not an RTP
        based capability.
@@ -1034,16 +1062,16 @@ class H323VideoCapability : public H323RealTimeCapability
      */
     virtual PBoolean OnSendingPDU(
       H245_VideoMode & pdu  ///<  PDU to set information on
-    ) const = 0;
+    ) const;
 
     /**This function is called whenever and incoming TerminalCapabilitySet
        PDU is received on the control channel, and a new H323Capability
        descendent was created. This completes reading fields from the PDU
        into the classes members.
 
-       If the function returns PFalse then the received PDU codec description
+       If the function returns false then the received PDU codec description
        is not supported, so will be ignored. The default behaviour simply
-       returns PTrue.
+       returns true.
      */
     virtual PBoolean OnReceivedPDU(
       const H245_Capability & pdu  ///<  PDU to get information from
@@ -1191,7 +1219,8 @@ class H323NonStandardVideoCapability : public H323VideoCapability,
     /**Compare the nonStandardData part of the capability, if applicable.
       */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 };
@@ -1280,12 +1309,34 @@ class H323GenericVideoCapability : public H323VideoCapability,
     /**Compare the generic part of the capability, if applicable.
      */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 };
 
 
+template <const char * oid, const OpalVideoFormat & fmtFunc()>
+class H323GenericVideoCapabilityTemplate : public H323GenericVideoCapability
+{
+  public:
+    H323GenericVideoCapabilityTemplate()
+      : H323GenericVideoCapability(oid)
+    {
+    }
+
+    virtual PObject * Clone() const
+    {
+      return new H323GenericVideoCapabilityTemplate(*this);
+    }
+
+    virtual PString GetFormatName() const
+    {
+      return fmtFunc();
+    }
+};
+
+
 #if OPAL_H239
 
 /**This class descibes an extended ideo capability, as used in H.239.
@@ -1357,7 +1408,8 @@ class H323ExtendedVideoCapability : public H323GenericVideoCapability
     /**Compare the generic part of the capability, if applicable.
      */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 
@@ -1430,9 +1482,9 @@ class H323GenericControlCapability : public H323Capability,
        descendent was created. This completes reading fields from the PDU
        into the classes members.
 
-       If the function returns PFalse then the received PDU codec description
+       If the function returns false then the received PDU codec description
        is not supported, so will be ignored. The default behaviour simply
-       returns PTrue.
+       returns true.
      */
     virtual PBoolean OnReceivedPDU(
       const H245_Capability & pdu  ///<  PDU to get information from
@@ -1441,7 +1493,8 @@ class H323GenericControlCapability : public H323Capability,
     /**Compare the generic part of the capability, if applicable.
      */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 
@@ -1489,10 +1542,6 @@ class H323H239VideoCapability : public H323ExtendedVideoCapability
     /**Create a copy of the object.
       */
     virtual PObject * Clone() const;
-
-    /**Print name of capability.
-      */
-    void PrintOn(ostream & strm) const;
   //@}
 
   /**@name Identification functions */
@@ -1601,7 +1650,7 @@ class H323DataCapability : public H323Capability
     /**Get the default RTP session.
        This function gets the default RTP session ID for the capability
        type. For example audio capabilities return the value
-       RTP_Session::DefaultAudioSessionID etc.
+       H323Capability::DefaultAudioSessionID etc.
 
        The default behaviour returns 3, indicating a data session.
       */
@@ -1678,9 +1727,9 @@ class H323DataCapability : public H323Capability
        descendent was created. This completes reading fields from the PDU
        into the classes members.
 
-       If the function returns PFalse then the received PDU codec description
+       If the function returns false then the received PDU codec description
        is not supported, so will be ignored. The default behaviour simply
-       returns PTrue.
+       returns true.
      */
     virtual PBoolean OnReceivedPDU(
       const H245_Capability & pdu  ///<  PDU to get information from
@@ -1829,7 +1878,8 @@ class H323NonStandardDataCapability : public H323DataCapability,
     /**Compare the nonStandardData part of the capability, if applicable.
       */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 };
@@ -2012,9 +2062,9 @@ class H323_UserInputCapability : public H323Capability
        descendent was created. This completes reading fields from the PDU
        into the classes members.
 
-       If the function returns PFalse then the received PDU codec description
+       If the function returns false then the received PDU codec description
        is not supported, so will be ignored. The default behaviour simply
-       returns PTrue.
+       returns true.
      */
     virtual PBoolean OnReceivedPDU(
       const H245_Capability & pdu  ///<  PDU to get information from
@@ -2037,7 +2087,7 @@ class H323_UserInputCapability : public H323Capability
        application to determine if this capability should be used in TCS or
        OLC pdus.
 
-       The default behaviour will check for early versions and return PFalse
+       The default behaviour will check for early versions and return false
        for RFC2833 mode.
       */
     virtual PBoolean IsUsable(
@@ -2046,9 +2096,10 @@ class H323_UserInputCapability : public H323Capability
   //@}
 
     static void AddAllCapabilities(
-      H323Capabilities & capabilities,        ///<  Table to add capabilities to
-      PINDEX descriptorNum,   ///<  The member of the capabilityDescriptor to add
-      PINDEX simultaneous     ///<  The member of the SimultaneousCapabilitySet to add
+      H323Capabilities & capabilities,  ///<  Table to add capabilities to
+      PINDEX descriptorNum,             ///<  The member of the capabilityDescriptor to add
+      PINDEX simultaneous,              ///<  The member of the SimultaneousCapabilitySet to add
+      bool includeRFC2833 = true
     );
 
   protected:
@@ -2169,7 +2220,9 @@ class H323Capabilities : public PObject
     PINDEX AddMediaFormat(
       PINDEX descriptorNum,    ///<  The member of the capabilityDescriptor to add
       PINDEX simultaneous,     ///<  The member of the SimultaneousCapabilitySet to add
-      const OpalMediaFormat & mediaFormat ///<  Media format to add.
+      const OpalMediaFormat & mediaFormat, ///<  Media format to add.
+      H323Capability::CapabilityDirection direction =
+                H323Capability::e_Receive ///<  Indicate capability direction
     );
 
     /**Add all matching capabilities to descriptor lists.
@@ -2180,7 +2233,7 @@ class H323Capabilities : public PObject
       PINDEX descriptorNum,    ///<  The member of the capabilityDescriptor to add
       PINDEX simultaneous,     ///<  The member of the SimultaneousCapabilitySet to add
       const PString & name,    ///<  New capabilities name, if using "known" one.
-      PBoolean exact = PFalse       ///<  Capability name must be exact match
+      PBoolean exact = false       ///<  Capability name must be exact match
     );
 
     // this function is retained for backwards compatibility
@@ -2189,7 +2242,7 @@ class H323Capabilities : public PObject
       PINDEX descriptorNum,    ///<  The member of the capabilityDescriptor to add
       PINDEX simultaneous,     ///<  The member of the SimultaneousCapabilitySet to add
       const PString & name,    ///<  New capabilities name, if using "known" one.
-      PBoolean exact = PFalse       ///<  Capability name must be exact match
+      PBoolean exact = false       ///<  Capability name must be exact match
     )
     { return AddAllCapabilities(descriptorNum, simultaneous, name, exact); }
 
@@ -2272,7 +2325,7 @@ class H323Capabilities : public PObject
       const PString & formatName, ///<  Wildcard format name to search for
       H323Capability::CapabilityDirection direction = H323Capability::e_Unknown,
             ///<  Optional direction to include into search criteria
-      PBoolean exact = PFalse       ///<  Capability name must be exact match
+      PBoolean exact = false       ///<  Capability name must be exact match
     ) const;
 
     /**Find the first capability in the table of the specified direction.
@@ -2310,7 +2363,8 @@ class H323Capabilities : public PObject
        NULL if no capability meeting the criteria was found
       */
     H323Capability * FindCapability(
-      const H245_DataType & dataType  ///<  H245 data type of codec
+      const H245_DataType & dataType,  ///<  H245 data type of codec
+      const PString & mediaPacketization = PString::Empty()  ///< Media packetization, if present
     ) const;
 
     /**Find the capability given the H.245 data type PDU.
@@ -2319,7 +2373,8 @@ class H323Capabilities : public PObject
        NULL if no capability meeting the criteria was found
       */
     H323Capability * FindCapability(
-      const H245_ModeElement & modeElement  ///<  H245 data type of codec
+      const H245_ModeElement & modeElement,  ///<  H245 data type of codec
+      const PString & mediaPacketization = PString::Empty()  ///< Media packetization, if present
     ) const;
 
     /**Find the capability given the type codecs.
@@ -2385,15 +2440,13 @@ class H323Capabilities : public PObject
       */
     OpalMediaFormatList GetMediaFormats() const;
 
-    const PStringSet & GetMediaPacketizations() const { return mediaPacketizations; }
+    const PStringSet & GetMediaPacketizations() const { return m_mediaPacketizations; }
   //@}
 
-  protected:
-    void SetMediaPacketizations(const PStringSet & packetizations) { mediaPacketizations = packetizations; }
-    
+  protected:    
     H323CapabilitiesList table;
     H323CapabilitiesSet  set;
-    PStringSet           mediaPacketizations;
+    PStringSet           m_mediaPacketizations;
 };
 
 
