@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28445 $
+ * $Revision: 29960 $
  * $Author: rjongbloed $
- * $Date: 2012-10-02 20:11:02 -0500 (Tue, 02 Oct 2012) $
+ * $Date: 2013-06-13 23:26:53 -0500 (Thu, 13 Jun 2013) $
  */
 
 #ifndef OPAL_H323_H323CON_H
@@ -42,7 +42,7 @@
 
 #if OPAL_H323
 
-#include <opal/rtpconn.h>
+#include <rtp/rtpconn.h>
 #include <opal/guid.h>
 #include <opal/buildopts.h>
 #include <h323/h323caps.h>
@@ -86,11 +86,14 @@ class H245_MiscellaneousCommand;
 class H245_MiscellaneousIndication;
 class H245_JitterIndication;
 class H245_ArrayOf_GenericParameter;
+class H245_ArrayOf_GenericInformation;
 
 class H323SignalPDU;
 class H323ControlPDU;
 class H323EndPoint;
 class H323TransportAddress;
+class H323_RTPChannel;
+typedef OpalMediaSession H323RTPSession;
 
 class H235Authenticators;
 
@@ -232,6 +235,10 @@ class H323Connection : public OpalRTPConnection
       */
     virtual void OnReleased();
 
+    /** Get the remote transport address
+      */
+    virtual OpalTransportAddress GetRemoteAddress() const { return GetTransport().GetRemoteAddress(); }
+
     /**Get the destination address of an incoming connection.
        This will, for example, collect a phone number from a POTS line, or
        get the fields from the H.225 SETUP pdu in a H.323 connection.
@@ -292,8 +299,8 @@ class H323Connection : public OpalRTPConnection
       bool isSource                      ///< Stream is a source/sink
     );
 
-#if OPAL_FAX
-    /**Switch to/from FAX mode.
+#if OPAL_T38_CAPABILITY
+    /**Switch to/from T.38 fax mode.
       */
     virtual bool SwitchFaxMediaStreams(
       bool toT38  ///< Enable T.38 or return to audio mode
@@ -308,22 +315,50 @@ class H323Connection : public OpalRTPConnection
       bool isSource                        ///< Stream is a source/sink
     );
     
-    /**Request close of a specific media stream.
-       Note that this is usually asymchronous, the OnClosedMediaStream() function is
-       called when the stream is really closed.
+    /**Call back for closed a media stream.
+
+       The default behaviour calls the OpalEndPoint function of the same name.
       */
-    virtual bool CloseMediaStream(
-      OpalMediaStream & stream  ///< Stream to close
+    virtual void OnClosedMediaStream(
+      const OpalMediaStream & stream     ///<  Media stream being closed
     );
 
-    /**Get information on the media channel for the connection.
-       The default behaviour returns true and fills the info structure if
-       there is a media channel active for the sessionID.
+    /**Get transports for the media session on the connection.
+       This is primarily used by the media bypass feature controlled by the
+       OpalManager::GetMediaTransferMode() function. It allows one side of the
+       call to get the transport address of the media on the other side, so it
+       can pass it on, bypassing the local host.
+
+       @return true if a transport address is available and may be used to pass
+               on to a remote system for direct access.
      */
-    virtual PBoolean GetMediaInformation(
-      unsigned sessionID,     ///<  Session ID for media channel
-      MediaInformation & info ///<  Information on media channel
+    virtual bool GetMediaTransportAddresses(
+      const OpalMediaType & mediaType,       ///< Media type for session to return information
+      OpalTransportAddressArray & transports ///<  Information on media session
     ) const;
+
+#if OPAL_H239
+    /**Send request for ability to send presentation video.
+       This requests the "presentation token" in H.239 terminology. It gives
+       permission to start secondary video channels with "presentation"
+       role. This is done via OpalCall::OpenSourceMediaStreams().
+
+       If called, and returns true, the OnChangedPresentationRole() should be
+       monitored for being this connections GetLocalPartyURL() value,
+       indicating success. Any other value indicates failure.
+
+       Default behaviour uses H.239 if available, otherwise returns false.
+      */
+    virtual bool RequestPresentationRole(
+      bool release   ///< Indicate we are acquiring or releasing the token
+    );
+
+    /**Indicate current presentation token ownership.
+
+       Default behaviour returns m_h239TokenOwned.
+      */
+    virtual bool HasPresentationRole() const;
+#endif // OPAL_H239
   //@}
 
   /**@name Signalling Channel */
@@ -687,47 +722,6 @@ class H323Connection : public OpalRTPConnection
       H323Connection & secondaryCall  ///<  Secondary call for consultation
     );
 
-    /**Place the call on hold, suspending all media channels (H.450.4).  Note it is the responsibility 
-       of the application layer to delete the MOH Channel if music on hold is provided to the remote
-       endpoint.  So far only Local Hold has been implemented. 
-     */
-    bool HoldCall(
-      PBoolean localHold   ///<  true for Local Hold, false for Remote Hold
-    );
-
-    /**Retrieve the call from hold, activating all media channels (H.450.4).
-       This method examines the call hold state and performs the necessary
-       actions required to retrieve a Near-end or Remote-end call on hold.
-       NOTE: Only Local Hold is implemented so far. 
-    */
-    bool RetrieveCall();
-
-    /**Set the alternative media channel.  This channel can be used to provide
-       Media On Hold (MOH) for a near end call hold operation or to provide
-       Recorded Voice Anouncements (RVAs).  If this method is not called before
-       a call hold operation is attempted, no media on hold will be provided
-       for the held endpoint.
-      */
-    void SetHoldMedia(
-      PChannel * audioChannel
-    );
-
-    /**Determine if Meadia On Hold is enabled.
-      */
-    PBoolean IsMediaOnHold() const;
-
-    /**Determine if held.
-      */
-    PBoolean IsLocalHold() const;
-
-    /**Determine if held.
-      */
-    PBoolean IsRemoteHold() const;
-
-    /**Determine if the current call is held or in the process of being held.
-      */
-    PBoolean IsCallOnHold() const;
-
     /**Begin a call intrusion request.
        Calls h45011handler->IntrudeCall where SS pdu is added to Call Setup
        message.
@@ -1052,6 +1046,14 @@ class H323Connection : public OpalRTPConnection
       const H323ControlPDU & pdu
     );
 
+    /**Handle a single received PDU from the control channel.
+       This is an internal function and is unlikely to be used by applications.
+    */
+    virtual PBoolean HandleReceivedControlPDU(
+      PBoolean readStatus,
+      PPER_Stream & strm
+    );
+
     /**This function is called from the HandleControlPDU() function
        for unhandled PDU types.
 
@@ -1208,7 +1210,7 @@ class H323Connection : public OpalRTPConnection
       unsigned logicalChannel,
       unsigned terminalLabel
     );
-#endif
+#endif // OPAL_H239
 
     /**Error discriminator for the OnControlProtocolError() function.
       */
@@ -1284,6 +1286,14 @@ class H323Connection : public OpalRTPConnection
       */
     virtual void OnSetLocalCapabilities();
 
+    /**Callback to modify the outgoing H245_OpenLogicalChannel.
+       The default behaviour does nothing.
+      */
+    virtual void OnSendH245_OpenLogicalChannel(
+      H245_OpenLogicalChannel & /*open*/, 
+      PBoolean /*forward*/
+    ) { }
+
     /**Return if this H245 connection is a master or slave
      */
     PBoolean IsH245Master() const;
@@ -1357,7 +1367,7 @@ class H323Connection : public OpalRTPConnection
     /**Start a logical channel for fast start.
        Internal function, not for normal use.
       */
-    virtual void StartFastStartChannel(
+    virtual void OpenFastStartChannel(
       unsigned sessionID,               ///<  Session ID to find logical channel.
       H323Channel::Directions direction ///<  Direction of channel to start
     );
@@ -1399,7 +1409,8 @@ class H323Connection : public OpalRTPConnection
     virtual PBoolean OnOpenLogicalChannel(
       const H245_OpenLogicalChannel & openPDU,  ///<  Received PDU for the channel open
       H245_OpenLogicalChannelAck & ackPDU,      ///<  PDU to send for acknowledgement
-      unsigned & errorCode                      ///<  Error to return if refused
+      unsigned & errorCode,                     ///<  Error to return if refused
+      H323Channel & channel                     ///<  Channel being opened
     );
 
     /**Callback for when a logical channel conflict has occurred.
@@ -1461,9 +1472,8 @@ class H323Connection : public OpalRTPConnection
       const H323Capability & capability, ///<  Capability creating channel
       H323Channel::Directions dir,       ///<  Direction of channel
       unsigned sessionID,                ///<  Session ID for RTP channel
-      const H245_H2250LogicalChannelParameters * param,
+      const H245_H2250LogicalChannelParameters * param
                                          ///<  Parameters for channel
-      RTP_QOS * rtpqos = NULL            ///<  QoS for RTP
     );
     
     /**Creates a new instance of an RTP channel.
@@ -1472,7 +1482,7 @@ class H323Connection : public OpalRTPConnection
     virtual H323_RTPChannel * CreateRTPChannel(
       const H323Capability & capability,
       H323Channel::Directions direction,
-      RTP_Session & rtp
+      H323RTPSession & rtp
     );
 
     /**This function is called when the remote endpoint want's to create
@@ -1588,20 +1598,13 @@ class H323Connection : public OpalRTPConnection
 
   /**@name Bandwidth Management */
   //@{
-    /**Set the available bandwidth in 100's of bits/sec.
-       Note if the force parameter is true this function will close down
-       active logical channels to meet the new bandwidth requirement.
+    /**Get the rx and/or tx bandwidth currently used.
+       This totals the bandwidth used by open logical channels and returns that
+       bandwidth used in bits/sec
       */
-    virtual PBoolean SetBandwidthAvailable(
-      unsigned newBandwidth,    ///<  New bandwidth limit
-      PBoolean force = false        ///<  Force bandwidth limit
-    );
-
-    /**Get the bandwidth currently used.
-       This totals the open channels and returns the total bandwidth used in
-       100's of bits/sec
-      */
-    virtual unsigned GetBandwidthUsed() const;
+    virtual OpalBandwidth GetBandwidthUsed(
+      OpalBandwidth::Direction dir   ///< Bandwidth direction
+    ) const;
   //@}
 
   /**@name Indications */
@@ -1713,36 +1716,6 @@ class H323Connection : public OpalRTPConnection
 
   /**@name RTP Session Management */
   //@{
-    /**Get an H323 RTP session for the specified ID.
-       If there is no session of the specified ID, NULL is returned.
-      */
-    virtual H323_RTP_Session * GetSessionCallbacks(
-      unsigned sessionID
-    ) const;
-
-    /**Use an RTP session for the specified ID and for the given direction.
-       If there is no session of the specified ID, a new one is created using
-       the information provided in the tranport parameter. If the system
-       does not support the specified transport, NULL is returned.
-      */
-    virtual RTP_Session * UseSession(
-      const OpalTransport & transport,
-                   unsigned sessionID,
-      const OpalMediaType & mediatype,  ///<  media type
-                  RTP_QOS * rtpqos = NULL
-    );
-
-    /**Callback from the RTP session for statistics monitoring.
-       This is called every so many packets on the transmitter and receiver
-       threads of the RTP session indicating that the statistics have been
-       updated.
-
-       The default behaviour calls H323EndPoint::OnRTPStatistics().
-      */
-    virtual void OnRTPStatistics(
-      const RTP_Session & session   ///<  Session with statistics
-    ) const;
-
     /**Get the names of the codecs in use for the RTP session.
        If there is no session of the specified ID, an empty string is returned.
       */
@@ -1750,7 +1723,15 @@ class H323Connection : public OpalRTPConnection
       unsigned sessionID
     ) const;
 
-  //@}
+    /**Determine if the RTP session needs to accommodate a NAT router.
+       This override does a check for H.460.19, and if disabled, prevents
+       OpalRTPConnection::DetermineRTPNAT() from being called.
+      */
+    virtual void DetermineRTPNAT(
+      const PIPSocket::Address & localAddr,   ///< Local physical address of connection
+      const PIPSocket::Address & peerAddr,    ///< Remote physical address of connection
+      const PIPSocket::Address & signalAddr   ///< Remotes signaling address as indicated by protocol of connection
+    );
 
   /**@name Request Mode Changes */
   //@{
@@ -1854,7 +1835,7 @@ class H323Connection : public OpalRTPConnection
 
     /**Get the call direction for this connection.
      */
-    PBoolean HadAnsweredCall() const { return !originating; }
+    PBoolean HadAnsweredCall() const { return !IsOriginating(); }
 
     /**Determined if connection is gatekeeper routed.
      */
@@ -1916,6 +1897,22 @@ class H323Connection : public OpalRTPConnection
       const H225_EndpointType & pdu ///<  PDU from which to extract application info.
     );
     
+    /**Goal supplied for this connection provided in SETUP
+      */
+    enum ConferenceGoal {
+        e_Create,
+        e_Invite,
+        e_Join
+    };
+
+    /**Get the conference goal for this connection.
+     */
+    ConferenceGoal GetConferenceGoal() { return m_conferenceGoal; }
+
+    /**Set the conference goal for this connection.
+     */
+    void SetConferenceGoal(ConferenceGoal goal) { m_conferenceGoal = goal; }
+
     /**Get the remotes capability table for this connection.
      */
     const H323Capabilities & GetLocalCapabilities() const { return localCapabilities; }
@@ -1930,7 +1927,7 @@ class H323Connection : public OpalRTPConnection
 
     /**Get the signalling channel being used.
       */
-    const H323Transport * GetSignallingChannel() const { return signallingChannel; }
+    const H323Transport * GetSignallingChannel() const { return m_signallingChannel; }
 
     /**Get the signalling channel protocol version number.
       */
@@ -1993,7 +1990,8 @@ class H323Connection : public OpalRTPConnection
       unsigned seconds  ///<  max duration of call in seconds
     );
   //@}
-    
+
+
 #if OPAL_H239
     /**Get the local H.239 control capability.
      */
@@ -2018,13 +2016,6 @@ class H323Connection : public OpalRTPConnection
     
     virtual void OnReceiveFeatureSet(unsigned, const H225_FeatureSet &) const;
 
-#if OPAL_H460
-    /** Get the connection FeatureSet
-     */
-    virtual H460_FeatureSet * GetFeatureSet();
-#endif
-
-    
 #if OPAL_H450
     /**
      * get the H4507 handler
@@ -2033,6 +2024,80 @@ class H323Connection : public OpalRTPConnection
     H4507Handler&  getH4507handler(){return *h4507handler;}
 #endif
 
+#if OPAL_H460
+    /** Get the connection FeatureSet
+     */
+    virtual H460_FeatureSet * GetFeatureSet();
+
+    /** Call to set the direction of call establishment
+      */
+    void H46019SetCallReceiver();
+
+    /** Enable H46019 for this call
+      */
+    void H46019Enabled();
+
+    /**Received OLC Generic Information. This is used to supply alternate RTP
+          destination information in the generic information field in the OLC for the
+          purpose of probing for an alternate route to the remote party.
+    */
+    virtual PBoolean OnReceiveOLCGenericInformation(
+      unsigned sessionID,
+      const H245_ArrayOf_GenericInformation & alternate
+    ) const;
+
+    /**Send Generic Information in the OLC. This is used to include generic
+          information in the openlogicalchannel
+    */
+   virtual PBoolean OnSendingOLCGenericInformation(
+     const unsigned & sessionID,              ///< Session Information
+     H245_ArrayOf_GenericInformation & gen,   ///< Generic OLC/OLCack message
+     PBoolean isAck
+   ) const;
+#endif
+    
+#if OPAL_H460_NAT
+    virtual PUDPSocket * GetNatSocket(unsigned session, PBoolean rtp);
+
+    /** Set RTP NAT information callback
+      */
+    virtual void SetRTPNAT(unsigned sessionid, PUDPSocket * _rtp, PUDPSocket * _rtcp);
+
+
+    /**Session Information 
+       This contains session information which is passed to the socket handler
+       when creating RTP socket pairs.
+      */
+    class SessionInformation : public PObject
+    {
+      public:
+        SessionInformation(const OpalGloballyUniqueID & id, const PString & token, unsigned session);
+
+        const PString & GetCallToken();
+
+        unsigned GetSessionID() const;
+
+        const OpalGloballyUniqueID & GetCallIdentifer();
+
+        const PString & GetCUI();
+
+      protected:
+        OpalGloballyUniqueID m_callID;
+        PString              m_callToken;
+        unsigned             m_sessionID;
+        PString              m_CUI;
+    };
+
+    SessionInformation * BuildSessionInformation(unsigned sessionID) const;
+
+    struct NAT_Sockets 
+    {
+        PUDPSocket * rtp;
+        PUDPSocket * rtcp;
+    };
+#endif // OPAL_H460_NAT
+
+
     /** Callback for media commands.
         Calls the SendIntraFrameRequest on the rtp session
 
@@ -2042,7 +2107,25 @@ class H323Connection : public OpalRTPConnection
       OpalMediaStream & stream,         ///< Stream command executed on
       const OpalMediaCommand & command  ///< Media command being executed
     );
-    
+
+    /**Compatibility workarounds.
+      */
+    enum CompatibilityIssues {
+      e_NoMultipleTunnelledH245, ///< Cannot have more than one tunnelled H.245 message
+      e_BadMasterSlaveConflict,  ///< Endpoint does not repropose on OLC master/slave conflict refect
+      e_NoUserInputCapability,   ///< Endpoint ignores TCS if has User Input Capability
+      NumCompatibilityIssues
+    };
+
+    /// Determine if we must compensate for remote endpoint.
+    bool HasCompatibilityIssue(
+      CompatibilityIssues issue
+    ) const;
+
+    PBoolean StartHandleControlChannel();
+    virtual PBoolean OnStartHandleControlChannel();
+    void EndHandleControlChannel();
+ 
   protected:
     /**Internal function to check if call established.
        This checks all the criteria for establishing a call an initiating the
@@ -2056,7 +2139,7 @@ class H323Connection : public OpalRTPConnection
     void MonitorCallStatus();
     PDECLARE_NOTIFIER(PThread, H323Connection, StartOutgoing);
     PDECLARE_NOTIFIER(PThread, H323Connection, NewOutgoingControlChannel);
-    PDECLARE_NOTIFIER(PThread, H323Connection, NewIncomingControlChannel);
+    PDECLARE_AcceptHandlerNotifier(H323Connection, NewIncomingControlChannel);
 
     H323EndPoint & endpoint;
 
@@ -2081,8 +2164,8 @@ class H323Connection : public OpalRTPConnection
     PBYTEArray         gkAccessTokenData;
     bool               addAccessTokenToSetup;
 
-    H323Transport * signallingChannel;
-    H323Transport * controlChannel;
+    OpalTransportPtr m_signallingChannel;
+    OpalTransportPtr m_controlChannel;
     OpalListener  * controlListener;
     bool            h245Tunneling;
     H323SignalPDU * h245TunnelRxPDU;
@@ -2092,7 +2175,7 @@ class H323Connection : public OpalRTPConnection
     H323SignalPDU * connectPDU;
     H323SignalPDU * progressPDU;
 
-    enum ConnectionStates {
+    P_DECLARE_TRACED_ENUM(ConnectionStates,
       NoConnectionActive,
       AwaitingGatekeeperAdmission,
       AwaitingTransportConnect,
@@ -2100,15 +2183,16 @@ class H323Connection : public OpalRTPConnection
       AwaitingLocalAnswer,
       HasExecutedSignalConnect,
       EstablishedConnection,
-      ShuttingDownConnection,
-      NumConnectionStates
-    } connectionState;
+      ShuttingDownConnection
+    );
+    ConnectionStates connectionState;
 
     unsigned   h225version;
     unsigned   h245version;
     bool       h245versionSet;
     bool       doH245inSETUP;
     bool       lastPDUWasH245inSETUP;
+    bool       m_forceSymmetricTCS;
 
     bool       mustSendDRQ;
     bool       mediaWaitForConnect;
@@ -2117,36 +2201,29 @@ class H323Connection : public OpalRTPConnection
     bool       earlyStart;
     PString    t38ModeChangeCapabilities;
     PSyncPoint digitsWaitFlag;
+    bool       endSessionSent;
     bool       endSessionNeeded;
     PSyncPoint endSessionReceived;
     PTimer     enforcedDurationLimit;
-
-    // Used as part of a local call hold operation involving MOH
-    PChannel * holdMediaChannel;
     bool       isConsultationTransfer;
 
+    ConferenceGoal m_conferenceGoal;
+
+#if OPAL_H450
     /** Call Intrusion flag and parameters */
     bool     isCallIntrusion;
     unsigned callIntrusionProtectionLevel;
+#endif
 
-    enum FastStartStates {
+    P_DECLARE_TRACED_ENUM(FastStartStates,
       FastStartDisabled,
       FastStartInitiate,
       FastStartResponse,
-      FastStartAcknowledged,
-      NumFastStartStates
-    };
-    FastStartStates        fastStartState;
-    H323LogicalChannelList fastStartChannels;
-    OpalMediaStreamPtr     fastStartMediaStream;
+      FastStartAcknowledged
+    );
+    FastStartStates        m_fastStartState;
+    H323LogicalChannelList m_fastStartChannels;
     
-#if PTRACING
-    static const char * GetConnectionStatesName(ConnectionStates s);
-    friend ostream & operator<<(ostream & o, ConnectionStates s) { return o << GetConnectionStatesName(s); }
-    static const char * GetFastStartStateName(FastStartStates s);
-    friend ostream & operator<<(ostream & o, FastStartStates s) { return o << GetFastStartStateName(s); }
-#endif
-
 
     // The following pointers are to protocol procedures, they are pointers to
     // hide their complexity from the H323Connection classes users.
@@ -2157,7 +2234,11 @@ class H323Connection : public OpalRTPConnection
     H245NegRoundTripDelay            * roundTripDelayProcedure;
 
 #if OPAL_H239
-    bool m_h239Control;
+    bool     m_h239Control;
+    unsigned m_h239SymmetryBreaking;
+    unsigned m_h239TokenChannel;
+    unsigned m_h239TerminalLabel;
+    bool     m_h239TokenOwned;
 #endif
 
 #if OPAL_H450
@@ -2170,14 +2251,38 @@ class H323Connection : public OpalRTPConnection
 #endif
 
 #if OPAL_H460
+    bool disableH460;
     H460_FeatureSet * features;
+
+    bool m_H46019CallReceiver;
+    bool m_H46019enabled;
+    bool m_h245Connect;
+
+    bool m_H46024Aenabled;
+    bool m_H46024Ainitator;
+    PINDEX m_H46024Astate;
+
+    bool m_H46024Benabled;
+    PINDEX m_H46024Bstate;
 #endif
 
-  private:
-    PChannel * SwapHoldMediaChannels(PChannel * newChannel);
+#if OPAL_VIDEO
+    PSimpleTimer m_h245FastUpdatePictureTimer;
+#endif
 
+#if OPAL_H460_NAT
+    PMutex NATSocketMutex;
+    std::map<unsigned, NAT_Sockets> m_NATSockets;
+#endif // OPAL_H460_NAT
+
+  private:
     P_REMOVE_VIRTUAL_VOID(CleanUpOnCallEnd());
     P_REMOVE_VIRTUAL_VOID(OnCleared());
+    P_REMOVE_VIRTUAL_VOID(OnRTPStatistics(const OpalRTPSession &) const);
+    P_REMOVE_VIRTUAL(PBoolean, OnOpenLogicalChannel(const H245_OpenLogicalChannel&,H245_OpenLogicalChannelAck&,unsigned&), false);
+    P_REMOVE_VIRTUAL(PBoolean, OnOpenLogicalChannel(const H245_OpenLogicalChannel &, H245_OpenLogicalChannelAck &, unsigned &, const unsigned &), false);
+    P_REMOVE_VIRTUAL_VOID(NatDetection(const PIPSocket::Address &, const PIPSocket::Address &));
+    P_REMOVE_VIRTUAL(PBoolean, OnNatDetected(), false);
 };
 
 
