@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23525 $
- * $Author: csoutheren $
- * $Date: 2009-09-24 01:02:24 -0500 (Thu, 24 Sep 2009) $
+ * $Revision: 29731 $
+ * $Author: rjongbloed $
+ * $Date: 2013-05-16 22:48:59 -0500 (Thu, 16 May 2013) $
  */
 
 #ifndef OPAL_H323_H323EP_H
@@ -42,13 +42,14 @@
 
 #if OPAL_H323
 
-#include <opal/rtpep.h>
+#include <rtp/rtpep.h>
 #include <opal/manager.h>
 #include <opal/call.h>
 #include <opal/transports.h>
 #include <h323/h323con.h>
 #include <h323/h323caps.h>
 #include <h323/h235auth.h>
+#include <h323/gkclient.h>
 #include <asn/h225.h>
 
 #if OPAL_H460
@@ -128,12 +129,12 @@ class H323EndPoint : public OpalRTPEndPoint
 
        The proto field is optional when passed to a specific endpoint. If it
        is present, however, it must agree with the endpoints protocol name or
-       PFalse is returned.
+       false is returned.
 
        This function usually returns almost immediately with the connection
        continuing to occur in a new background thread.
 
-       If PFalse is returned then the connection could not be established. For
+       If false is returned then the connection could not be established. For
        example if a PSTN endpoint is used and the assiciated line is engaged
        then it may return immediately. Returning a non-NULL value does not
        mean that the connection will succeed, only that an attempt is being
@@ -145,7 +146,7 @@ class H323EndPoint : public OpalRTPEndPoint
       OpalCall & call,                  ///<  Owner of connection
       const PString & party,            ///<  Remote party to call
       void * userData  = NULL,          ///<  Arbitrary data to pass to connection
-      unsigned int options = NULL,      ///<  options to pass to conneciton
+      unsigned int options = 0,         ///<  options to pass to connection
       OpalConnection::StringOptions * stringOptions = NULL
     );
   //@}
@@ -383,7 +384,7 @@ class H323EndPoint : public OpalRTPEndPoint
     PBoolean IsRegisteredWithGatekeeper() const;
 
     /**Unregister and delete the gatekeeper we are registered with.
-       The return value indicates PFalse if there was an error during the
+       The return value indicates false if there was an error during the
        unregistration. However the gatekeeper is still removed and its
        instance deleted regardless of this error.
      */
@@ -410,29 +411,20 @@ class H323EndPoint : public OpalRTPEndPoint
       */
     virtual H235Authenticators CreateAuthenticators();
 
-    /**Called when the gatekeeper sends a GatekeeperConfirm
+    /**Called when the gatekeeper status changes.
       */
-    virtual void  OnGatekeeperConfirm();
-
-    /**Called when the gatekeeper sends a GatekeeperReject
-      */
-    virtual void  OnGatekeeperReject();
-
-    /**Called when the gatekeeper sends a RegistrationConfirm
-      */
-    virtual void OnRegistrationConfirm();
-
-    /**Called when the gatekeeper sends a RegistrationReject
-      */
-    virtual void  OnRegistrationReject();
+    virtual void OnGatekeeperStatus(
+      H323Gatekeeper::RegistrationFailReasons status
+    );
   //@}
 
   /**@name Connection management */
   //@{
     /**Handle new incoming connetion from listener.
       */
-    virtual PBoolean NewIncomingConnection(
-      OpalTransport * transport  ///<  Transport connection came in on
+    virtual void NewIncomingConnection(
+      OpalListener & listener,            ///<  Listner that created transport
+      const OpalTransportPtr & transport  ///<  Transport connection came in on
     );
 
     /**Create a connection that uses the specified call.
@@ -494,14 +486,6 @@ class H323EndPoint : public OpalRTPEndPoint
       const PString & secondaryCallToken  ///<  Token of secondary call
     );
 
-    /**Place the call on hold, suspending all media channels (H.450.4)
-    * NOTE: Only Local Hold is implemented so far. 
-    */
-    void HoldCall(
-      const PString & token,        ///<  Existing connection to be transferred
-      PBoolean localHold   ///<  true for Local Hold, false for Remote Hold
-    );
-
     /** Initiate Call intrusion
         Designed similar to MakeCall function
       */
@@ -553,9 +537,9 @@ class H323EndPoint : public OpalRTPEndPoint
        the OnReceivedSignalSetup() function before it sends the Call
        Proceeding PDU. It gives an opportunity for an application to alter
        the request before transmission to the other endpoint. If this function
-       returns PFalse then the Call Proceeding PDU is not sent at all.
+       returns false then the Call Proceeding PDU is not sent at all.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
        @param connection the connection associated to the call proceeding
        @param callProceedingPDU the call processding to modify
        @return if false, do no send the connect pdu  
@@ -569,9 +553,9 @@ class H323EndPoint : public OpalRTPEndPoint
        the H323Connection::SetConnected function before it sends the connect  PDU. 
        It gives an opportunity for an application to alter
        the request before transmission to the other endpoint. If this function
-       returns PFalse then the connect PDU is not sent at all.
+       returns false then the connect PDU is not sent at all.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
        @param connection the connection associated to the connect
        @param connectPDU the connect to modify
        @return if false, do no send the connect pdu  
@@ -587,10 +571,10 @@ class H323EndPoint : public OpalRTPEndPoint
        application to alter the reply before transmission to the other
        endpoint.
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
      */
     virtual PBoolean OnIncomingCall(
       H323Connection & connection,    ///<  Connection that was established
@@ -599,10 +583,10 @@ class H323EndPoint : public OpalRTPEndPoint
     );
 
     /**Called when an outgoing call connects
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
       */
     virtual PBoolean OnOutgoingCall(
       H323Connection & conn, 
@@ -611,7 +595,7 @@ class H323EndPoint : public OpalRTPEndPoint
 
     /**Handle a connection transfer.
        This gives the application an opportunity to abort the transfer.
-       The default behaviour just returns PTrue.
+       The default behaviour just returns true.
       */
     virtual PBoolean OnCallTransferInitiate(
       H323Connection & connection,    ///<  Connection to transfer
@@ -620,7 +604,7 @@ class H323EndPoint : public OpalRTPEndPoint
 
     /**Handle a transfer via consultation.
        This gives the transferred-to user an opportunity to abort the transfer.
-       The default behaviour just returns PTrue.
+       The default behaviour just returns true.
       */
     virtual PBoolean OnCallTransferIdentify(
       H323Connection & connection    ///<  Connection to transfer
@@ -658,10 +642,10 @@ class H323EndPoint : public OpalRTPEndPoint
        receives the optional Alerting PDU from the remote endpoint. That is
        when the remote "phone" is "ringing".
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
      */
     virtual PBoolean OnAlerting(
       H323Connection & connection,    ///<  Connection that was established
@@ -671,7 +655,7 @@ class H323EndPoint : public OpalRTPEndPoint
 
     /** A call back function when the alerting is about to be sent, 
         can be used by the application to alter the alerting Pdu
-        @return if PFalse, then the alerting is not sent
+        @return if false, then the alerting is not sent
      */
     virtual PBoolean OnSendAlerting(
       H323Connection & connection,  ///< onnection that was established
@@ -690,9 +674,9 @@ class H323EndPoint : public OpalRTPEndPoint
     /**A call back function when a connection indicates it is to be forwarded.
        An H323 application may handle this call back so it can make
        complicated decisions on if the call forward ius to take place. If it
-       decides to do so it must call MakeCall() and return PTrue.
+       decides to do so it must call MakeCall() and return true.
 
-       The default behaviour simply returns PFalse and that the automatic
+       The default behaviour simply returns false and that the automatic
        call forwarding should take place. See ForwardConnection()
       */
     virtual PBoolean OnConnectionForwarded(
@@ -702,7 +686,7 @@ class H323EndPoint : public OpalRTPEndPoint
     );
 
     /**Forward the call using the same token as the specified connection.
-       Return PTrue if the call is being redirected.
+       Return true if the call is being redirected.
 
        The default behaviour will replace the current call in the endpoints
        call list using the same token as the call being redirected. Not that
@@ -731,17 +715,6 @@ class H323EndPoint : public OpalRTPEndPoint
     virtual PBoolean IsConnectionEstablished(
       const PString & token   ///<  Token for identifying connection
     );
-
-    /**A call back function whenever a connection is broken.
-       This indicates that a connection to a remote endpoint is no longer
-       available.
-
-       The default behaviour does nothing.
-      */
-    virtual void OnConnectionCleared(
-      H323Connection & connection,    ///<  Connection that was established
-      const PString & token           ///<  Token for identifying connection
-    );
   //@}
 
 
@@ -749,7 +722,7 @@ class H323EndPoint : public OpalRTPEndPoint
   //@{
     /**Call back for opening a logical channel.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
       */
     virtual PBoolean OnStartLogicalChannel(
       H323Connection & connection,    ///<  Connection for the channel
@@ -765,18 +738,6 @@ class H323EndPoint : public OpalRTPEndPoint
       const H323Channel & channel     ///<  Channel being started
     );
 
-    /**Callback from the RTP session for statistics monitoring.
-       This is called every so many packets on the transmitter and receiver
-       threads of the RTP session indicating that the statistics have been
-       updated.
-
-       The default behaviour does nothing.
-      */
-    virtual void OnRTPStatistics(
-      const H323Connection & connection,  ///<  Connection for the channel
-      const RTP_Session & session         ///<  Session with statistics
-    ) const;
-
     /**Call back from GK admission confirm to notify the 
      * Endpoint it is behind a NAT (GNUGK Gatekeeper).
      * The default does nothing. 
@@ -808,7 +769,7 @@ class H323EndPoint : public OpalRTPEndPoint
        An application may override this to display call credit information
        on registration, or when a call is started.
 
-       The canDisplayAmountString member variable must also be set to PTrue
+       The canDisplayAmountString member variable must also be set to true
        for this to operate.
 
        The default behaviour does nothing.
@@ -837,19 +798,10 @@ class H323EndPoint : public OpalRTPEndPoint
 
   /**@name Additional call services */
   //@{
-    /** Called when an endpoint receives a SETUP PDU with a
-        conference goal of "invite"
-      
-        The default behaviour is to return PFalse, which will close the connection
-     */
-    virtual PBoolean OnConferenceInvite(
-      const H323SignalPDU & setupPDU
-    );
-
     /** Called when an endpoint receives a SETUP PDU with a
         conference goal of "callIndependentSupplementaryService"
       
-        The default behaviour is to return PFalse, which will close the connection
+        The default behaviour is to return false, which will close the connection
      */
     virtual PBoolean OnCallIndependentSupplementaryService(
       const H323SignalPDU & setupPDU
@@ -858,7 +810,7 @@ class H323EndPoint : public OpalRTPEndPoint
     /** Called when an endpoint receives a SETUP PDU with a
         conference goal of "capability_negotiation"
       
-        The default behaviour is to return PFalse, which will close the connection
+        The default behaviour is to return false, which will close the connection
      */
     virtual PBoolean OnNegotiateConferenceCapabilities(
       const H323SignalPDU & setupPDU
@@ -968,14 +920,25 @@ class H323EndPoint : public OpalRTPEndPoint
       PBoolean mode ///<  New default mode
     ) { disableH245inSetup = mode; } 
 
+    /**Get the default H.245 tunneling mode.
+      */
+    bool IsForcedSymmetricTCS() const
+      { return m_forceSymmetricTCS; }
+
+    /**Set the default H.245 tunneling mode.
+      */
+    void ForceSymmetricTCS(
+      bool mode ///<  New default mode
+    ) { m_forceSymmetricTCS = mode; } 
+
     /** find out if h245 is disabled or enabled 
-      * @return PTrue if h245 is disabled 
+      * @return true if h245 is disabled 
       */
     PBoolean IsH245Disabled() const
     { return m_bH245Disabled; }
 
     /**Disable/Enable H.245, used at least for h450.7 calls
-     * @param  bH245Disabled PTrue if h245 has to be disabled 
+     * @param  bH245Disabled true if h245 has to be disabled 
      */
     void DisableH245(PBoolean bH245Disabled) { m_bH245Disabled = bH245Disabled; } 
     
@@ -1045,6 +1008,10 @@ class H323EndPoint : public OpalRTPEndPoint
       e_MCUWithAVMP = 190
     };
 
+    /**Get the endpoint terminal type.
+     */
+    void SetTerminalType(TerminalTypes type) { terminalType = type; }
+
     /**Get the endpoint terminal type.
      */
     TerminalTypes GetTerminalType() const { return terminalType; }
@@ -1082,14 +1049,6 @@ class H323EndPoint : public OpalRTPEndPoint
       unsigned maxDelay    ///<  New maximum jitter buffer delay in milliseconds
     ) { manager.SetAudioJitterDelay(minDelay, maxDelay); }
 
-    /**Get the initial bandwidth parameter.
-     */
-    unsigned GetInitialBandwidth() const { return initialBandwidth; }
-
-    /**Get the initial bandwidth parameter.
-     */
-    void SetInitialBandwidth(unsigned bandwidth) { initialBandwidth = bandwidth; }
-
 #if OPAL_H239
     /**Get the default H.239 control capability.
      */
@@ -1109,7 +1068,7 @@ class H323EndPoint : public OpalRTPEndPoint
     /**Called when an incoming PDU contains a feature set
      */
     virtual void OnReceiveFeatureSet(unsigned, const H225_FeatureSet &);
-	
+    
     /**Load the Base FeatureSet usually called when you initialise the endpoint prior to 
        registering with a gatekeeper.
       */
@@ -1137,6 +1096,32 @@ class H323EndPoint : public OpalRTPEndPoint
         This creates a new instance of the featureSet
       */
     H460_FeatureSet * GetFeatureSet() { return features.DeriveNewFeatureSet(); };
+
+    virtual void NATMethodCallBack(
+      const PString & /*NatID*/,    ///< Method Identifier
+      PINDEX /*msgID*/,             ///< Message Identifer
+      const PString & /*message*/   ///< Message
+    ) { }
+
+    /** Disable H.460.18 Feature. (By Default it is enabled)
+      */    
+    void H46018Enable(PBoolean enable);
+
+    /** Query whether we are using H.460.18
+      */
+    PBoolean H46018IsEnabled();
+
+    /** Signal that H.460.18 has been received. ie. We are behind a NAT/FW
+      */
+    void H46018Received() {};
+
+    /** Whether H.460.18 is in Operation for this call
+      */
+    PBoolean H46018InOperation();
+
+    /**Get the complete list of Gatekeeper features
+      */
+    H460_FeatureSet *GetGatekeeperFeatures();
 #endif
 
     /**Determine if the address is "local", ie does not need STUN
@@ -1200,14 +1185,6 @@ class H323EndPoint : public OpalRTPEndPoint
      */
     WORD GetRtpIpPortPair() { return manager.GetRtpIpPortPair(); }
 
-    /**Get the IP Type Of Service byte for RTP channels.
-     */
-    BYTE GetRtpIpTypeofService() const { return manager.GetRtpIpTypeofService(); }
-
-    /**Set the IP Type Of Service byte for RTP channels.
-     */
-    void SetRtpIpTypeofService(unsigned tos) { manager.SetRtpIpTypeofService(tos); }
-
     /**Get the default timeout for calling another endpoint.
      */
     const PTimeInterval & GetSignallingChannelCallTimeout() const { return signallingChannelCallTimeout; }
@@ -1258,9 +1235,9 @@ class H323EndPoint : public OpalRTPEndPoint
 
     /**Set the amount of time with no media that should cause call to clear
      */
-    PBoolean SetNoMediaTimeout(
+    void SetNoMediaTimeout(
       const PTimeInterval & newInterval  ///<  New timeout for media
-    ) { return manager.SetNoMediaTimeout(newInterval); }
+    ) { manager.SetNoMediaTimeout(newInterval); }
 
     /**Get the default timeout for GatekeeperRequest and Gatekeeper discovery.
      */
@@ -1340,11 +1317,54 @@ class H323EndPoint : public OpalRTPEndPoint
     unsigned GetNextH450CallIdentityValue() const { return ++nextH450CallIdentity; }
 #endif
 
-    PString GetDefaultTransport() const;
+    /**Get the default transports for the endpoint type.
+       Overrides the default behaviour to return udp and tcp.
+      */
+    virtual PString GetDefaultTransport() const;
+
+    /**Get the default signal port for this endpoint.
+     */
+    virtual WORD GetDefaultSignalPort() const;
+
+    /// Gets the current regular expression for the compatibility issue
+    PString GetCompatibility(
+      H323Connection::CompatibilityIssues issue    ///< Issue being worked around
+    ) const;
+
+    /** Indicate a compatibility issue with remote endpoint product type.
+        Note the regular expression is always "extended".
+
+        @returns true of regular expression is legal.
+      */
+    bool SetCompatibility(
+      H323Connection::CompatibilityIssues issue,    ///< Issue being worked around
+      const PString & regex         ///< Regular expression for endpoint matching, see OpalProductInfo::AsString()
+    );
+
+    /** Indicate a compatibility issue with remote endpoint product type.
+        This will add to the regular expression for the issue a new value to
+        try and match. It effectively adds "|" + regex to the existing regular
+        expression.
+
+        Note the regular expression is always "extended".
+      */
+    bool AddCompatibility(
+      H323Connection::CompatibilityIssues issue,    ///< Issue being worked around
+      const PString & regex         ///< Regular expression for endpoint matching, see OpalProductInfo::AsString()
+    );
+
+    /// Determine if we must compensate for remote endpoint.
+    bool HasCompatibilityIssue(
+      H323Connection::CompatibilityIssues issue,            ///< Issue being worked around
+      const OpalProductInfo & productInfo   ///< Product into for check for issue
+    ) const;
   //@}
 
   protected:
-    bool InternalCreateGatekeeper(H323Transport * transport);
+    bool InternalCreateGatekeeper(
+      H323Transport * transport,
+      const H323TransportAddress & gkAddress
+    );
     H323Connection * InternalMakeCall(
       OpalCall & call,
       const PString & existingToken,           ///<  Existing connection to be transferred
@@ -1363,6 +1383,7 @@ class H323EndPoint : public OpalRTPEndPoint
     PBoolean        disableFastStart;
     PBoolean        disableH245Tunneling;
     PBoolean        disableH245inSetup;
+    bool            m_forceSymmetricTCS;
     PBoolean        m_bH245Disabled; /* enabled or disabled h245 */
     PBoolean        canDisplayAmountString;
     PBoolean        canEnforceDurationLimit;
@@ -1434,8 +1455,26 @@ class H323EndPoint : public OpalRTPEndPoint
 #if OPAL_H460
     bool            disableH460;
     H460_FeatureSet features;
+    bool m_h46018enabled;
 #endif
 
+    typedef map<H323Connection::CompatibilityIssues, PRegularExpression> CompatibilityEndpoints;
+    CompatibilityEndpoints m_compatibility;
+
+  private:
+    P_REMOVE_VIRTUAL_VOID(OnConnectionCleared(H323Connection &, const PString &));
+    P_REMOVE_VIRTUAL_VOID(OnRTPStatistics(const H323Connection &, const OpalRTPSession &) const);
+    P_REMOVE_VIRTUAL(PBoolean, OnConferenceInvite(const H323SignalPDU &), false);
+    P_REMOVE_VIRTUAL_VOID(OnGatekeeperConfirm());
+    P_REMOVE_VIRTUAL_VOID(OnGatekeeperReject());
+    P_REMOVE_VIRTUAL_VOID(OnRegistrationConfirm());
+    P_REMOVE_VIRTUAL_VOID(OnRegistrationReject());
+
+#if OPAL_H460
+  // This is because there h323plus had
+  // a public way of adding a connection
+  friend class H46018Transport;
+#endif
 };
 
 #endif // OPAL_H323
