@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 29738 $
+ * $Author: rjongbloed $
+ * $Date: 2013-05-19 18:28:42 -0500 (Sun, 19 May 2013) $
  */
 
 #ifndef OPAL_H323_H323NEG_H
@@ -62,12 +62,12 @@ class H245Negotiator : public PObject
     H245Negotiator(H323EndPoint & endpoint, H323Connection & connection);
 
   protected:
-    PDECLARE_NOTIFIER(PTimer, H245Negotiator, HandleTimeout);
+    PDECLARE_NOTIFIER(PTimer, H245Negotiator, HandleTimeoutUnlocked);
+    virtual void HandleTimeout();
 
     H323EndPoint   & endpoint;
     H323Connection & connection;
     PTimer           replyTimer;
-    PMutex           mutex;
 };
 
 
@@ -86,7 +86,7 @@ class H245NegMasterSlaveDetermination : public H245Negotiator
     PBoolean HandleAck(const H245_MasterSlaveDeterminationAck & pdu);
     PBoolean HandleReject(const H245_MasterSlaveDeterminationReject & pdu);
     PBoolean HandleRelease(const H245_MasterSlaveDeterminationRelease & pdu);
-    void HandleTimeout(PTimer &, INT);
+    void HandleTimeout();
 
     PBoolean IsMaster() const     { return status == e_DeterminedMaster; }
     PBoolean IsDetermined() const { return state == e_Idle && status != e_Indeterminate; }
@@ -126,13 +126,13 @@ class H245NegTerminalCapabilitySet : public H245Negotiator
   public:
     H245NegTerminalCapabilitySet(H323EndPoint & endpoint, H323Connection & connection);
 
-    PBoolean Start(PBoolean renegotiate, PBoolean empty = PFalse);
-    void Stop(PBoolean dec = PFalse);
+    PBoolean Start(PBoolean renegotiate, PBoolean empty = false);
+    void Stop(PBoolean dec = false);
     PBoolean HandleIncoming(const H245_TerminalCapabilitySet & pdu);
     PBoolean HandleAck(const H245_TerminalCapabilitySetAck & pdu);
     PBoolean HandleReject(const H245_TerminalCapabilitySetReject & pdu);
     PBoolean HandleRelease(const H245_TerminalCapabilitySetRelease & pdu);
-    void HandleTimeout(PTimer &, INT);
+    void HandleTimeout();
 
     bool HasSentCapabilities() const { return state >= e_InProgress; }
     bool IsSendingCapabilities() const { return state == e_InProgress; }
@@ -187,20 +187,14 @@ class H245NegLogicalChannel : public H245Negotiator
     virtual PBoolean HandleRequestCloseAck(const H245_RequestChannelCloseAck & pdu);
     virtual PBoolean HandleRequestCloseReject(const H245_RequestChannelCloseReject & pdu);
     virtual PBoolean HandleRequestCloseRelease(const H245_RequestChannelCloseRelease & pdu);
-    virtual void HandleTimeout(PTimer &, INT);
+    virtual void HandleTimeout();
 
-    H323Channel * GetChannel();
+    H323Channel * GetChannel() const;
 
-    bool IsAwaitingEstablishment() const { return state == e_AwaitingEstablishment; }
+    bool IsAwaitingEstablishment() const { return state == e_AwaitingEstablishment || state == e_Establishing; }
     bool IsEstablished() const { return state == e_Established; }
 
   protected:
-    virtual PBoolean OpenWhileLocked(
-      const H323Capability & capability,
-      unsigned sessionID,
-      unsigned replacementFor = 0
-    );
-    virtual PBoolean CloseWhileLocked();
     virtual void Release();
 
 
@@ -211,6 +205,7 @@ class H245NegLogicalChannel : public H245Negotiator
     enum States {
       e_Released,
       e_AwaitingEstablishment,
+      e_Establishing,
       e_Established,
       e_AwaitingRelease,
       e_AwaitingConfirmation,
@@ -258,10 +253,9 @@ class H245NegLogicalChannels : public H245Negotiator
     virtual PBoolean HandleRequestCloseRelease(const H245_RequestChannelCloseRelease & pdu);
 
     H323ChannelNumber GetNextChannelNumber();
-    PINDEX GetSize() const { return channels.GetSize(); }
-    H323Channel * GetChannelAt(PINDEX i);
+    H245LogicalChannelDict & GetChannels() { return channels; }
+    const H245LogicalChannelDict & GetChannels() const { return channels; }
     H323Channel * FindChannel(unsigned channelNumber, PBoolean fromRemote);
-    H245NegLogicalChannel & GetNegLogicalChannelAt(PINDEX i);
     H245NegLogicalChannel * FindNegLogicalChannel(unsigned channelNumber, PBoolean fromRemote);
     H323Channel * FindChannelBySession(unsigned rtpSessionId, PBoolean fromRemote);
     void RemoveAll();
@@ -287,7 +281,7 @@ class H245NegRequestMode : public H245Negotiator
     virtual PBoolean HandleAck(const H245_RequestModeAck & pdu);
     virtual PBoolean HandleReject(const H245_RequestModeReject & pdu);
     virtual PBoolean HandleRelease(const H245_RequestModeRelease & pdu);
-    virtual void HandleTimeout(PTimer &, INT);
+    virtual void HandleTimeout();
 
   protected:
     PBoolean awaitingResponse;
@@ -308,7 +302,7 @@ class H245NegRoundTripDelay : public H245Negotiator
     PBoolean StartRequest();
     PBoolean HandleRequest(const H245_RoundTripDelayRequest & pdu);
     PBoolean HandleResponse(const H245_RoundTripDelayResponse & pdu);
-    void HandleTimeout(PTimer &, INT);
+    void HandleTimeout();
 
     PTimeInterval GetRoundTripDelay() const { return roundTripTime; }
     PBoolean IsRemoteOffline() const { return retryCount == 0; }
