@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23923 $
+ * $Revision: 28394 $
  * $Author: rjongbloed $
- * $Date: 2010-01-11 19:49:45 -0600 (Mon, 11 Jan 2010) $
+ * $Date: 2012-09-24 00:17:07 -0500 (Mon, 24 Sep 2012) $
  */
 
 #ifndef OPAL_H323_H323PDU_H
@@ -214,11 +214,11 @@ class H323SignalPDU : public H225_H323_UserInformation
        incoming connection.
       */
     PString GetDestinationAlias(
-      PBoolean firstAliasOnly = PFalse   ///<  Only return the first possible alias
+      PBoolean firstAliasOnly = false   ///<  Only return the first possible alias
     ) const;
 
     /**Get the source endpoints identification as a phone number.
-       This returns PFalse if the remote never provided any alias or Q.931
+       This returns false if the remote never provided any alias or Q.931
        field that indicated a valid e.164 telephone number.
       */
     PBoolean GetSourceE164(
@@ -226,7 +226,7 @@ class H323SignalPDU : public H225_H323_UserInformation
     ) const;
 
     /**Get the destiation  phone number.
-       This returns PFalse if the remote never provided any alias or Q.931
+       This returns false if the remote never provided any alias or Q.931
        field that indicated a valid e.164 telephone number.
       */
     PBoolean GetDestinationE164(
@@ -437,6 +437,8 @@ class H323RasPDU : public H225_RasMessage, public H323TransactionPDU
     virtual H323TransactionPDU * ClonePDU() const;
     virtual void DeletePDU();
 
+    PDECLARE_WriteConnectCallback(H323RasPDU, WriteGRQ);
+
     // new functions
     H225_GatekeeperRequest       & BuildGatekeeperRequest(unsigned seqNum);
     H225_GatekeeperConfirm       & BuildGatekeeperConfirm(unsigned seqNum);
@@ -493,11 +495,9 @@ Q931::CauseValues H323TranslateFromCallEndReason(
 
 void H323GetApplicationInfo(OpalProductInfo & info, const H225_VendorIdentifier & vendor);
 
-void H323SetRTPPacketization(
+bool H323SetRTPPacketization(
   H245_ArrayOf_RTPPayloadType & rtpPacketizations,
-  PINDEX & packetizationsCount,
-  const OpalMediaFormat & mediaFormat,
-  RTP_DataFrame::PayloadTypes payloadType
+  const PStringSet & mediaPacketizations
 );
 bool H323SetRTPPacketization(
   H245_RTPPayloadType & rtpPacketization,
@@ -507,16 +507,11 @@ bool H323SetRTPPacketization(
 bool H323SetRTPPacketization(
   H245_RTPPayloadType & rtpPacketization,
   const PString & mediaPacketizationString,
-  const OpalMediaFormat & mediaFormat,
   RTP_DataFrame::PayloadTypes payloadType
 );
 PString H323GetRTPPacketization(
   const H245_RTPPayloadType & rtpPacketization
 );
-bool H323GetRTPPacketization(
-  OpalMediaFormat & mediaFormat,
-  const H245_RTPPayloadType & rtpPacketization
-);
 
 PString H323GetCapabilityIdentifier(
   const H245_CapabilityIdentifier & capId
@@ -542,33 +537,28 @@ unsigned H323GetGenericParameterInteger(
 );
 H245_ParameterValue * H323AddGenericParameter(
   H245_ArrayOf_GenericParameter & params,
-  unsigned ordinal,
-  bool reorder = true
+  unsigned ordinal
 );
 void H323AddGenericParameterBoolean(
   H245_ArrayOf_GenericParameter & params,
   unsigned ordinal,
-  bool value = true,
-  bool reorder = true
+  bool value
 );
 void H323AddGenericParameterInteger(
   H245_ArrayOf_GenericParameter & params,
   unsigned ordinal,
   unsigned value,
-  H245_ParameterValue::Choices subType = H245_ParameterValue::e_unsignedMin,
-  bool reorder = true
+  H245_ParameterValue::Choices subType
 );
 void H323AddGenericParameterString(
   H245_ArrayOf_GenericParameter & params,
   unsigned ordinal,
-  const PString & value,
-  bool reorder = true
+  const PString & value
 );
 void H323AddGenericParameterOctets(
   H245_ArrayOf_GenericParameter & params,
   unsigned ordinal,
-  const PBYTEArray & value,
-  bool reorder = true
+  const PBYTEArray & value
 );
 
 #if PTRACING
