@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 25568 $
+ * $Author: rjongbloed $
+ * $Date: 2011-04-14 05:04:08 -0500 (Thu, 14 Apr 2011) $
  */
 
 #ifndef OPAL_H323_TRANSADDR_H
@@ -87,17 +87,23 @@ class H323TransportAddress : public OpalTransportAddress
       const char * proto = NULL ///<  default to udp
     );
 
-    PBoolean SetPDU(
-      H225_ArrayOf_TransportAddress & pdu,  ///<  List of transport addresses listening on
-      const OpalTransport & associatedTransport ///<  Associated transport for precendence and translation
-    );
     PBoolean SetPDU(H225_TransportAddress & pdu, WORD defPort = 0) const;
     PBoolean SetPDU(H245_TransportAddress & pdu, WORD defPort = 0) const;
 };
 
 
-PDECLARE_ARRAY(H323TransportAddressArray, H323TransportAddress)
+class H323TransportAddressArray : public PArray<H323TransportAddress>
+{
+    typedef PArray<H323TransportAddress> ParentClass;
+    PCLASSINFO(H323TransportAddressArray, ParentClass);
+
+  protected:
+    inline H323TransportAddressArray(int dummy, const H323TransportAddressArray * c)
+      : ParentClass(dummy, c) { }
+
   public:
+    H323TransportAddressArray(PINDEX initialSize = 0)
+      : ParentClass(initialSize) { }
     H323TransportAddressArray(
       const OpalTransportAddress & address
     ) { AppendAddress(address); }
@@ -130,6 +136,11 @@ PDECLARE_ARRAY(H323TransportAddressArray, H323TransportAddress)
       const H323TransportAddress & address
     );
 
+    virtual PObject * Clone() const
+    {
+      return new H323TransportAddressArray(0, this);
+    }
+
   protected:
     void AppendStringCollection(
       const PCollection & coll
