@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 24722 $
- * $Author: dereksmithies $
- * $Date: 2010-09-23 23:32:35 -0500 (Thu, 23 Sep 2010) $
+ * $Revision: 27079 $
+ * $Author: rjongbloed $
+ * $Date: 2012-02-29 23:12:01 -0600 (Wed, 29 Feb 2012) $
  */
 
 #ifndef OPAL_IAX2_CALLPROCESSOR_H
@@ -76,9 +76,6 @@ class IAX2CallProcessor : public IAX2Processor
   /**Get the iax2 encryption info */
   IAX2Encryption & GetEncryptionInfo() { return encryption; }
 
-  /**Call back from the IAX2Connection class */
-  virtual void Release(OpalConnection::CallEndReason releaseReason = OpalConnection::EndedByLocalUser);
-
   /**From the IAX2Connection class. CAlling this sends a hangup frame */
   void ClearCall(OpalConnection::CallEndReason releaseReason = OpalConnection::EndedByLocalUser);
 
