@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 23414 $
+ * $Revision: 27079 $
  * $Author: rjongbloed $
- * $Date: 2009-09-10 03:53:29 -0500 (Thu, 10 Sep 2009) $
+ * $Date: 2012-02-29 23:12:01 -0600 (Wed, 29 Feb 2012) $
  */
 
 #ifndef OPAL_IAX2_IAX2CON_H
@@ -148,8 +148,6 @@ class IAX2Connection : public OpalConnection
       CallEndReason reason = EndedByLocalUser /// Reason for call clearing
       );
 
-  OpalConnection::SendUserInputModes GetRealSendUserInputMode() const;
-
   /**Provided as a link between the iax endpoint and the iax processor */
   void SendDtmf(const PString & dtmf);
 
@@ -212,23 +210,6 @@ class IAX2Connection : public OpalConnection
       */
   void OnEstablished();
 
-  /**Release the current connection.
-     This removes the connection from the current call. The call may
-     continue if there are other connections still active on it. If this was
-     the last connection for the call then the call is disposed of as well.
-     
-     Note that this function will return quickly as the release and
-     disposal of the connections is done by another thread.
-  
-     This sends an IAX2 hangup frame to the remote endpoint.
- 
-     The ConnectionRun that manages packets going into/out of the
-     IAX2Connection will continue to run, and will send the appropriate
-     IAX2 hangup messages. Death of the Connection thread will happen when the 
-     OnReleased thread of the OpalConnection runs */
-  virtual void Release( CallEndReason reason = EndedByLocalUser ///<Reason for call release
-		);
-
   /**Indicate to remote endpoint an alert is in progress.  If this is
      an incoming connection and the AnswerCallResponse is in a
      AnswerCallDeferred or AnswerCallPending state, then this function
@@ -342,19 +323,25 @@ class IAX2Connection : public OpalConnection
       value defined by FullFrameVoice::AudioSc */
   unsigned int ChooseCodec();
   
-    /**Return true if the current connection is on hold.
-       The bool parameter indicates if we are testing if the remote system
-       has us on hold, or we have them on hold.
+    /**Put the current connection on hold, suspending all media streams.
+       The \p fromRemote parameter indicates if we a putting the remote on
+       hold (false) or it is a request for the remote to put us on hold (true).
+
+       The /p placeOnHold parameter indicates of teh command/request is for
+       going on hold or retrieving from hold.
      */
-    virtual bool IsConnectionOnHold(
-      bool fromRemote  ///< Test if remote has us on hold, or we have them
+    virtual bool Hold(
+      bool fromRemote,  ///< Flag for if remote has us on hold, or we have them
+      bool placeOnHold  ///< Flag for setting on or off hold
+    );
+
+    /**Return true if the current connection is on hold.
+       The \p fromRemote parameter indicates if we are testing if the remote
+       system has us on hold, or we have them on hold.
+     */
+    virtual bool IsOnHold(
+      bool fromRemote  ///< Flag for if remote has us on hold, or we have them
     );
-  
-  /**Take the current connection off hold*/
-  virtual bool RetrieveConnection();
-  
-  /**Put the current connection on hold, suspending all media streams.*/
-  virtual bool HoldConnection();
   
   /**Signal that the remote side has put the connection on hold*/
   void RemoteHoldConnection();
@@ -390,7 +377,7 @@ class IAX2Connection : public OpalConnection
      function when an application wishes to redirct an unwanted incoming
      call.
      
-     The return value is PTrue if the call is to be forwarded, PFalse
+     The return value is true if the call is to be forwarded, false
      otherwise. Note that if the call is forwarded the current connection is
      cleared with teh ended call code of EndedByCallForwarded.
   */
@@ -405,7 +392,7 @@ class IAX2Connection : public OpalConnection
   
   /**Test to see if it is a status query type iax frame (eg lagrq) and
      handle it. If the frame is a status query, and it is handled,
-     return PTrue */
+     return true */
   //static PBoolean IsStatusQueryEthernetFrame(IAX2Frame *frame);
     
   /**Return reference to the endpoint class */
@@ -474,10 +461,9 @@ class IAX2Connection : public OpalConnection
 
 #endif // OPAL_IAX2_IAX2CON_H
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 2 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file. 
+ * With the comment below, the tab key does auto indent to 2 spaces.     
+ *
  * Local Variables:
  * mode:c
  * c-basic-offset:2
