@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21909 $
- * $Author: dereksmithies $
- * $Date: 2009-01-09 03:48:34 +0000 (Fri, 09 Jan 2009) $
+ * $Revision: 29658 $
+ * $Author: rjongbloed $
+ * $Date: 2013-05-07 21:16:26 -0500 (Tue, 07 May 2013) $
  */
 
 #ifndef OPAL_IAX2_IAX2EP_H
@@ -80,7 +80,7 @@ public:
 
   /**Activate this thread to process all frames in the lists
    */
-  void ProcessList();
+  void ProcessList() { activate.Signal(); }
 
   /**Cause this thread to die immediately */
   void Terminate();
@@ -105,8 +105,9 @@ class IAX2EndPoint : public OpalEndPoint
 {
   PCLASSINFO(IAX2EndPoint, OpalEndPoint);
  public:
+   enum { DefaultUdpPort = 4569 };
+
   /**@name Construction */
-  
   //@{
   /**Create the endpoint, and define local variables */
   IAX2EndPoint(
@@ -119,14 +120,16 @@ class IAX2EndPoint : public OpalEndPoint
   
   /**@name connection Connection handling */
   //@{
-  /**Handle new incoming connection from listener.
+    /**Get the default transports for the endpoint type.
+       Overrides the default behaviour to return udp and tcp.
+      */
+    virtual PString GetDefaultTransport() const;
+
+    /**Get the default signal port for this endpoint.
+     */
+    virtual WORD GetDefaultSignalPort() const;
+
 
-  The default behaviour does nothing.
-  */
-  virtual PBoolean NewIncomingConnection(
-    OpalTransport * transport  /// Transport connection came in on
-  );
-  
   /**Set up a connection to a remote party.
      This is called from the OpalManager::MakeConnection() function once
      it has determined that this is the endpoint for the protocol.
@@ -143,12 +146,12 @@ class IAX2EndPoint : public OpalEndPoint
      
      The proto field is optional when passed to a specific endpoint. If it
      is present, however, it must agree with the endpoints protocol name or
-     PFalse is returned.
+     false is returned.
      
      This function usually returns almost immediately with the connection
      continuing to occur in a new background thread.
      
-     If PFalse is returned then the connection could not be established. For
+     If false is returned then the connection could not be established. For
      example if a PSTN endpoint is used and the assiciated line is engaged
      then it may return immediately. Returning a non-NULL value does not
      mean that the connection will succeed, only that an attempt is being
@@ -156,7 +159,7 @@ class IAX2EndPoint : public OpalEndPoint
      
      The default behaviour is pure.
   */
-  virtual PBoolean MakeConnection(
+  virtual PSafePtr<OpalConnection> MakeConnection(
 			      OpalCall & call,          ///<  Owner of connection
 			      const PString & party,    ///<  Remote party to call
 			      void * userData = NULL,   ///<  Arbitrary data to pass to connection
@@ -247,8 +250,8 @@ class IAX2EndPoint : public OpalEndPoint
   
     /**Handle new incoming connection from listener.
        
-    A return value of PTrue indicates that the transport object should be
-    deleted by the caller. PFalse indicates that something else (eg the
+    A return value of true indicates that the transport object should be
+    deleted by the caller. false indicates that something else (eg the
     connection) has taken over responsibility for deleting the transport.
     
     Well, that is true of Opal. In iax2, we do all the work of creating a new
@@ -309,9 +312,10 @@ class IAX2EndPoint : public OpalEndPoint
     userIndex      = 1,     /*!< the username, or alias field            */
     transportIndex = 2,     /*!< the transport, or transport field       */
     addressIndex   = 3,     /*!< the address, or 192.168.1.1 field       */
-    extensionIndex = 4,     /*!< the extension, or "extension"" field    */
-    contextIndex   = 5,     /*!< the context,   or "+context" field      */
-    maximumIndex   = 6      /*!< the number of possible fields           */
+    portIndex      = 4,     /*!< the port number                         */
+    extensionIndex = 5,     /*!< the extension, or "extension"" field    */
+    contextIndex   = 6,     /*!< the context,   or "+context" field      */
+    maximumIndex   = 7      /*!< the number of possible fields           */
   };
 
   /**Given a remote party name of the format:
@@ -396,6 +400,11 @@ class IAX2EndPoint : public OpalEndPoint
   
   /**Report if this iax2 endpoint class is correctly initialised */
   PBoolean InitialisedOK() { return (transmitter != NULL) && (receiver != NULL); }
+
+  /**Report if there are frames (from the ethernet) waiting to be
+     processed */
+  PBoolean EthernetFramesToBeProcessed() 
+  { return packetsReadFromEthernet.GetSize() > 0; }
   //@}
   
  protected:
@@ -407,13 +416,13 @@ class IAX2EndPoint : public OpalEndPoint
      list. */
   IAX2IncomingEthernetFrames incomingFrameHandler;
 
-  /**List of iax packets which has been read from the ethernet, and
+  /**List of iax2 packets which has been read from the ethernet, and
      is to be sent to the matching IAX2Connection */
   IAX2FrameList   packetsReadFromEthernet;
   
   /**The socket on which all data is sent/received.*/
   PUDPSocket  *sock;
-  
+
   /**Number of active calls */
   int callnumbs;
   
@@ -444,21 +453,30 @@ class IAX2EndPoint : public OpalEndPoint
     
   /**For the supplied IAX2Frame, pass it to a connection in the
      connectionsActive structure.  If no matching connection is found, return
-     PFalse;
+     false;
      
      If a matching connections is found, give the frame to the
-     connection (for the connection to process) and return PTrue;
+     connection (for the connection to process) and return true;
   */
   PBoolean ProcessInMatchingConnection(IAX2Frame *f);  
-    
-  /**The TokenTranslationDict may need a new entry. Examine
-     the list of active connections, to see if any match this frame.
-     If any do, then we add a new translation entry in tokenTable;
+
+  /**For the supplied IAX2Frame, pass it to a connection in the
+     connectionsActive structure.  We are told the token of the connection
+     that matches this particular frame. 
      
-     If a matching connection is found in connectionsActive, create a
-     new translation entry and return PTrue. The connection, after
-     processing the frame, will then delete the frame. */
-  PBoolean AddNewTranslationEntry(IAX2Frame *f);
+
+     @return true - this is the normal and expected outcome. 
+     @return false - connection not found - say it died immediately
+     before this method was called */
+    PBoolean ProcessFrameInConnection(IAX2Frame *f, const PString & token);  
+    
+  /**Take the supplied frame, and compare it with all the existing
+     connections.  If any of the existing connections has a matching
+     dest call number, then put this frame to that connection.  
+
+     Update the token translation dictionary if the supplied frame has
+     a valid connection token. */
+  PBoolean ProcessInConnectionTestAll(IAX2Frame *f);
   
   /**tokenTable is a hack to allow IAX2 to fit in with one of the
      demands of the opal library. 
@@ -502,10 +520,9 @@ class IAX2EndPoint : public OpalEndPoint
 
 #endif // OPAL_IAX2_IAX2EP_H
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 2 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file.
+ * With the comment below, the tab key does auto indent to 2 spaces.    
+ *
  * Local Variables:
  * mode:c
  * c-basic-offset:2
