@@ -25,9 +25,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21293 $
+ * $Revision: 27997 $
  * $Author: rjongbloed $
- * $Date: 2008-10-12 18:24:41 -0500 (Sun, 12 Oct 2008) $
+ * $Date: 2012-07-10 22:23:17 -0500 (Tue, 10 Jul 2012) $
  */
 
 #ifndef OPAL_IAX2_REGPROCESSOR_H
@@ -84,7 +84,7 @@ class IAX2RegProcessor : public IAX2Processor
   PString userName;
   PString password;
   
-  INT registrationRefreshTime;
+  unsigned registrationRefreshTime;
   
   /** bit mask of the different flags to indicate call status*/
   enum RegistrationState {
@@ -169,7 +169,7 @@ class IAX2RegProcessor : public IAX2Processor
   @return true if the frame is out of order, which deletes the supplied frame
   @return false, and does not destroy the supplied frame*/
   virtual PBoolean IncomingMessageOutOfOrder(IAX2FullFrame *)
-       { return PFalse; };
+       { return false; };
 
   
   /**A class that generates random values used for varying 
