@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23593 $
- * $Author: csoutheren $
- * $Date: 2009-10-06 01:23:00 -0500 (Tue, 06 Oct 2009) $
+ * $Revision: 28756 $
+ * $Author: rjongbloed $
+ * $Date: 2013-01-03 00:37:34 -0600 (Thu, 03 Jan 2013) $
  */
 
 #ifndef OPAL_IM_MSRP_H
@@ -33,31 +33,16 @@
 
 #include <ptlib.h>
 #include <opal/buildopts.h>
-#include <opal/rtpconn.h>
-#include <opal/manager.h>
-#include <opal/mediastrm.h>
-#include <opal/mediatype.h>
-#include <im/im.h>
-#include <ptclib/inetprot.h>
-#include <ptclib/guid.h>
-#include <ptclib/mime.h>
-
-#if OPAL_SIP
-#include <sip/sdp.h>
-#endif
 
 #if OPAL_HAS_MSRP
 
-class OpalMSRPMediaType : public OpalIMMediaType 
-{
-  public:
-    OpalMSRPMediaType();
-    virtual OpalMediaSession * CreateMediaSession(OpalConnection & conn, unsigned sessionID) const;
+#include <opal/mediastrm.h>
+#include <opal/mediasession.h>
+#include <ptclib/url.h>
+#include <ptclib/inetprot.h>
 
-#if OPAL_SIP
-    SDPMediaDescription * CreateSDPMediaDescription(const OpalTransportAddress & localAddress);
-#endif
-};
+
+class OpalManager;
 
 
 ////////////////////////////////////////////////////////////////////////////
@@ -266,10 +251,10 @@ class OpalMSRPManager : public PObject
     PThread * m_listenerThread;
 
     PMutex m_connectionInfoMapAddMutex;
-    typedef std::map<std::string, PSafePtr<Connection> > ConnectionInfoMapType;
+    typedef std::map<PString, PSafePtr<Connection> > ConnectionInfoMapType;
     ConnectionInfoMapType m_connectionInfoMap;
 
-    typedef std::map<std::string, CallBack> CallBackMap;
+    typedef std::map<PString, CallBack> CallBackMap;
     CallBackMap m_callBacks;
     PMutex m_callBacksMutex;
 
@@ -285,30 +270,27 @@ class OpalMSRPMediaSession : public OpalMediaSession
 {
   PCLASSINFO(OpalMSRPMediaSession, OpalMediaSession);
   public:
-    OpalMSRPMediaSession(OpalConnection & connection, unsigned sessionId);
-    OpalMSRPMediaSession(const OpalMSRPMediaSession & _obj);
+    static const PCaselessString & TCP_MSRP();
+
+    OpalMSRPMediaSession(const Init & init);
     ~OpalMSRPMediaSession();
 
-    bool Open(const PURL & remoteParty);
-
-    virtual void Close();
-
     virtual PObject * Clone() const { return new OpalMSRPMediaSession(*this); }
 
-    virtual bool IsActive() const { return true; }
-
-    virtual bool IsRTP() const { return false; }
-
-    virtual bool HasFailed() const { return false; }
-
-    virtual OpalTransportAddress GetLocalMediaAddress() const;
+    virtual const PCaselessString & GetSessionType() const { return TCP_MSRP(); }
+    virtual bool Open(const PString & localInterface, const OpalTransportAddress & remoteAddress, bool isMediaAddress);
+    virtual bool Close();
+    virtual OpalTransportAddress GetLocalAddress(bool isMediaAddress = true) const;
+    virtual OpalTransportAddress GetRemoteAddress(bool isMediaAddress = true) const;
+    virtual bool SetRemoteAddress(const OpalTransportAddress & remoteAddress, bool isMediaAddress = true);
+#if OPAL_SIP
+    virtual SDPMediaDescription * CreateSDPMediaDescription();
+#endif
 
     PURL GetLocalURL() const { return m_localUrl; }
     PURL GetRemoteURL() const { return m_remoteUrl; }
     void SetRemoteURL(const PURL & url) { m_remoteUrl = url; }
 
-    virtual void SetRemoteMediaAddress(const OpalTransportAddress &, const OpalMediaFormatList & );
-
     virtual bool WritePacket(      
       RTP_DataFrame & frame
     );
@@ -319,12 +301,6 @@ class OpalMSRPMediaSession : public OpalMediaSession
       PINDEX & read
     );
 
-#if OPAL_SIP
-    virtual SDPMediaDescription * CreateSDPMediaDescription(
-      const OpalTransportAddress & localAddress
-    );
-#endif
-
     virtual OpalMediaStream * CreateMediaStream(
       const OpalMediaFormat & mediaFormat, 
       unsigned sessionID, 
@@ -345,11 +321,17 @@ class OpalMSRPMediaSession : public OpalMediaSession
     PURL m_remoteUrl;
     PSafePtr<OpalMSRPManager::Connection> m_connectionPtr;
     OpalTransportAddress m_remoteAddress;
+
+  private:
+    OpalMSRPMediaSession(const OpalMSRPMediaSession & other)
+      : OpalMediaSession(Init(other.m_connection, other.m_sessionId, other.m_mediaType, false)), m_manager(other.m_manager) { }
+    void operator=(const OpalMSRPMediaSession &) { }
 };
 
+
 ////////////////////////////////////////////////////////////////////////////
 
-class OpalMSRPMediaStream : public OpalIMMediaStream
+class OpalMSRPMediaStream : public OpalMediaStream
 {
   public:
     OpalMSRPMediaStream(
@@ -363,7 +345,8 @@ class OpalMSRPMediaStream : public OpalIMMediaStream
 
     ~OpalMSRPMediaStream();
 
-    virtual PBoolean RequiresPatchThread() const   { return !isSource; }
+    virtual PBoolean IsSynchronous() const         { return false; }
+    virtual PBoolean RequiresPatchThread() const   { return IsSink(); }
 
     /**Read raw media data from the source media stream.
        The default behaviour reads from the PChannel object.
@@ -380,22 +363,20 @@ class OpalMSRPMediaStream : public OpalIMMediaStream
     );
 
     virtual bool Open();
-    virtual bool Close();
 
     PURL GetRemoteURL() const           { return m_msrpSession.GetRemoteURL(); }
     void SetRemoteURL(const PURL & url) { m_msrpSession.SetRemoteURL(url); }
 
     PDECLARE_NOTIFIER2(OpalMSRPManager, OpalMSRPMediaStream, OnReceiveMSRP, OpalMSRPManager::IncomingMSRP &);
-
-
   //@}
+
   protected:
+    virtual void InternalClose() { }
+
     OpalMSRPMediaSession & m_msrpSession;
-    PString m_remoteParty;
-    RFC4103Context m_rfc4103Context;
+    PString                m_remoteParty;
 };
 
-//                       schemeName,user,   passwd, host,   defUser,defhost, query,  params, frags,  path,   rel,    port
 
 #endif // OPAL_HAS_MSRP
 
