@@ -26,9 +26,9 @@
  * 
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21293 $
+ * $Revision: 27154 $
  * $Author: rjongbloed $
- * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
+ * $Date: 2012-03-07 19:21:33 -0600 (Wed, 07 Mar 2012) $
  */
 
 #ifndef OPAL_LIDS_LIDEP_H
@@ -40,6 +40,8 @@
 
 #include <opal/buildopts.h>
 
+#if OPAL_LID
+
 #include <opal/endpoint.h>
 #include <lids/lid.h>
 #include <codec/silencedetect.h>
@@ -87,12 +89,12 @@ class OpalLineEndPoint : public OpalEndPoint
 
        The proto field is optional when passed to a specific endpoint. If it
        is present, however, it must agree with the endpoints protocol name or
-       PFalse is returned.
+       false is returned.
 
        This function usually returns almost immediately with the connection
        continuing to occur in a new background thread.
 
-       If PFalse is returned then the connection could not be established. For
+       If false is returned then the connection could not be established. For
        example if a PSTN endpoint is used and the assiciated line is engaged
        then it may return immediately. Returning a non-NULL value does not
        mean that the connection will succeed, only that an attempt is being
@@ -100,12 +102,12 @@ class OpalLineEndPoint : public OpalEndPoint
 
        The default behaviour is pure.
      */
-    virtual PBoolean MakeConnection(
+    virtual PSafePtr<OpalConnection> MakeConnection(
       OpalCall & call,          ///< Owner of connection
       const PString & party,    ///< Remote party to call
       void * userData = NULL,   ///< Arbitrary data to pass to connection
       unsigned int options = 0,  ///<  options to pass to conneciton
-      OpalConnection::StringOptions * stringOptions  = NULL
+      OpalConnection::StringOptions * stringOptions  = NULL ///< Options to pass to connection
     );
 
     /**Get the data formats this endpoint is capable of operating.
@@ -138,7 +140,7 @@ class OpalLineEndPoint : public OpalEndPoint
       */
     PSafePtr<OpalLineConnection> GetLIDConnectionWithLock(
       const PString & token,     ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite
+      PSafetyMode mode = PSafeReadWrite   ///< Lock mode
     ) { return PSafePtrCast<OpalConnection, OpalLineConnection>(GetConnectionWithLock(token, mode)); }
 
     /**Add a line to the endpoint.
@@ -146,7 +148,7 @@ class OpalLineEndPoint : public OpalEndPoint
        should not be deleted directly. Use the RemoveLine() function to
        delete the line.
 
-       Returns PTrue if the lines device was open and the line was added.
+       Returns true if the lines device was open and the line was added.
       */
     PBoolean AddLine(
       OpalLine * line
@@ -186,7 +188,7 @@ class OpalLineEndPoint : public OpalEndPoint
 
        Note the device should already be open or no lines are added.
 
-       Returns PTrue if at least one line was added.
+       Returns true if at least one line was added.
       */
     virtual PBoolean AddLinesFromDevice(
       OpalLineInterfaceDevice & device  ///<  Device to add lines
@@ -202,7 +204,7 @@ class OpalLineEndPoint : public OpalEndPoint
     /**Add a line interface devices to the endpoint by name.
        This calls AddDeviceName() for each entry in the array.
 
-       Returns PTrue if at least one line from one device was added.
+       Returns true if at least one line from one device was added.
       */
     PBoolean AddDeviceNames(
       const PStringArray & descriptors  ///<  Device descritptions to add
@@ -214,7 +216,7 @@ class OpalLineEndPoint : public OpalEndPoint
        as would be returned from the OpalLineInterfaceDevice::GetAllDevices()
        function.
 
-       Returns PTrue if at least one line was added or the descriptor was
+       Returns true if at least one line was added or the descriptor was
        already present.
       */
     PBoolean AddDeviceName(
@@ -236,7 +238,7 @@ class OpalLineEndPoint : public OpalEndPoint
 
        Note the device should already be open or no lines are added.
 
-       Returns PTrue if at least one line was added.
+       Returns true if at least one line was added.
       */
     virtual PBoolean AddDevice(
       OpalLineInterfaceDevice * device    ///<  Device to add
@@ -256,7 +258,7 @@ class OpalLineEndPoint : public OpalEndPoint
     /**Get the line by name.
        The lineName parameter may be "*" to matche the first line.
 
-       If the enableAudio flag is PTrue then the EnableAudio() function is
+       If the enableAudio flag is true then the EnableAudio() function is
        called on the line and it is returns only if successful. This
        effectively locks the line for exclusive use of the caller.
       */
@@ -272,6 +274,21 @@ class OpalLineEndPoint : public OpalEndPoint
     void SetDefaultLine(
       const PString & lineName  ///<  Name of line to set to default.
     );
+
+    /**Set the country code set for the device.
+       This may change the line analogue coefficients, ring detect, call
+       disconnect detect and call progress tones to fit the countries
+       telephone network.
+      */
+    bool SetCountryCode(
+      OpalLineInterfaceDevice::T35CountryCodes country   ///<  Country code for device
+    );
+
+    /**Set the country code set for the device.
+      */
+    bool SetCountryCodeName(
+      const PString & countryName   ///<  Country code for device
+    );
   //@}
 
 
@@ -424,6 +441,17 @@ class OpalLineConnection : public OpalConnection
       OpalMediaStream & stream    ///<  New media stream being opened
     );
 
+    /**Call back for closed a media stream.
+
+       Note that this is called while the OpalMediaStream is locked. Care needs to be
+       taken to avoid deadlocks.
+
+       The default behaviour calls the OpalEndPoint function of the same name.
+      */
+    virtual void OnClosedMediaStream(
+      const OpalMediaStream & stream     ///<  Media stream being closed
+    );
+
     /**Set  the volume (gain) for the audio media channel to the specified percentage.
       */
     virtual PBoolean SetAudioVolume(
@@ -489,12 +517,22 @@ class OpalLineConnection : public OpalConnection
       */
     OpalLine & GetLine() { return line; }
 
-    /** delay in msec to wait between the dial tone detection and dialing the dtmf 
-      * @param uiDial the dial delay to set
+    /** Get the prompt tone used on POTS lines.
+        Defaults to OpalLineInterfaceDevice::DialTone.
+      */
+    OpalLineInterfaceDevice::CallProgressTones GetPromptTone() const { return m_promptTone; }
+
+    /** Set the prompt tone used on POTS lines.
+      */
+    void SetPromptTone(OpalLineInterfaceDevice::CallProgressTones tone) { m_promptTone = tone; }
+
+    /** Delay in msec to wait between the dial tone detection and dialing the dtmf 
      */
-    void setDialDelay(unsigned int uiDialDelay) { m_dialParams.m_dialStartDelay = uiDialDelay;}
-    
-    /** delay in msec to wait between the dial tone detection and dialing the dtmf 
+    void setDialDelay(
+      unsigned int uiDialDelay  ///< Dialing delay
+    ) { m_dialParams.m_dialStartDelay = uiDialDelay;}
+
+    /** Delay in msec to wait between the dial tone detection and dialing the dtmf 
      * @return uiDialDelay the dial delay to get
      */
     unsigned int getDialDelay() const { return m_dialParams.m_dialStartDelay; }
@@ -505,7 +543,9 @@ class OpalLineConnection : public OpalConnection
     OpalLine        & line;
     bool              wasOffHook;
     unsigned          minimumRingCount;
+    PString           m_dialedNumber;
     OpalLineInterfaceDevice::DialParams m_dialParams;
+    OpalLineInterfaceDevice::CallProgressTones m_promptTone;
 
     PDECLARE_NOTIFIER(PThread, OpalLineConnection, HandleIncoming);
     PThread         * handlerThread;
@@ -524,7 +564,7 @@ class OpalLineMediaStream : public OpalMediaStream
     /**Construct a new media stream for Line Interface Devices.
       */
     OpalLineMediaStream(
-      OpalLineConnection & conn,
+      OpalLineConnection & conn,            ///< Owner connection
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       PBoolean isSource,                       ///<  Is a source stream
@@ -532,6 +572,9 @@ class OpalLineMediaStream : public OpalMediaStream
     );
   //@}
 
+    ~OpalLineMediaStream();
+
+
   /**@name Overrides of OpalMediaStream class */
   //@{
     /**Open the media stream.
@@ -541,12 +584,6 @@ class OpalLineMediaStream : public OpalMediaStream
       */
     virtual PBoolean Open();
 
-    /**Close the media stream.
-
-       The default does nothing.
-      */
-    virtual PBoolean Close();
-
     /**Read an RTP frame of data from the source media stream.
        The default behaviour simply calls ReadData() on the data portion of the
        RTP_DataFrame and sets the frames timestamp and marker from the internal
@@ -589,25 +626,27 @@ class OpalLineMediaStream : public OpalMediaStream
        The default behaviour does nothing.
       */
     virtual PBoolean SetDataSize(
-      PINDEX dataSize  ///<  New data size
+      PINDEX dataSize,  ///< New data size (in total)
+      PINDEX frameTime  ///< Individual frame time (if applicable)
     );
 
     /**Indicate if the media stream is synchronous.
-       Returns PTrue for LID streams.
+       Returns true for LID streams.
       */
     virtual PBoolean IsSynchronous() const;
 
     /**Indicate if the media stream requires a OpalMediaPatch thread (active patch).
-       This is called on the source stream and is passed the sink stream that the
-       patch will initially be using. The function could conditionally require
-       the patch thread to execute a thread reading and writing data, or prevent
-       it from doing so as it can do so in hardware in some way, e.g. if both
-       streams where on the same OpalLineInterfaceDevice.
+       This is called on the source/sink stream and is passed the sink/source
+       stream that the patch will initially be using. The function could
+       conditionally require the patch thread to execute a thread reading and
+       writing data, or prevent  it from doing so as it can do so in hardware
+       in some way.
 
-       The default behaviour simply returns true.
+       The default behaviour here determines if both streams are on the same
+       OpalLineInterfaceDevice and returns false if so.
       */
     virtual PBoolean RequiresPatchThread(
-      OpalMediaStream * sinkStream  ///< Sink stream for this source
+      OpalMediaStream * stream  ///< Other stream in patch
     ) const;
   //@}
 
@@ -619,6 +658,8 @@ class OpalLineMediaStream : public OpalMediaStream
   //@}
 
   protected:
+    virtual void InternalClose();
+
     OpalLine & line;
     bool       notUsingRTP;
     bool       useDeblocking;
@@ -664,6 +705,8 @@ class OpalLineSilenceDetector : public OpalSilenceDetector
 };
 
 
+#endif // OPAL_LID
+
 #endif // OPAL_LIDS_LIDEP_H
 
 
