@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25893 $
+ * $Revision: 29938 $
  * $Author: rjongbloed $
- * $Date: 2011-05-25 20:31:43 -0500 (Wed, 25 May 2011) $
+ * $Date: 2013-06-12 21:19:41 -0500 (Wed, 12 Jun 2013) $
  */
 
 #ifndef OPAL_OPAL_H
@@ -80,7 +80,7 @@ typedef struct OpalHandleStruct * OpalHandle;
 typedef struct OpalMessage OpalMessage;
 
 /// Current API version
-#define OPAL_C_API_VERSION 24
+#define OPAL_C_API_VERSION 29
 
 
 ///////////////////////////////////////
@@ -304,7 +304,10 @@ typedef void (OPAL_EXPORT *OpalFreeMessageFunction)(OpalMessage * message);
 #define OPAL_PREFIX_LOCAL "local"   ///< Local endpoint supported string for OpalInitialise()
 #define OPAL_PREFIX_POTS  "pots"    ///< Plain Old Telephone System supported string for OpalInitialise()
 #define OPAL_PREFIX_PSTN  "pstn"    ///< Public Switched Network supported string for OpalInitialise()
+#define OPAL_PREFIX_FAX   "fax"     ///< G.711 fax supported string for OpalInitialise()
+#define OPAL_PREFIX_T38   "t38"     ///< G.711 fax supported string for OpalInitialise()
 #define OPAL_PREFIX_IVR   "ivr"     ///< Interactive Voice Response supported string for OpalInitialise()
+#define OPAL_PREFIX_IM    "im"      ///< Interactive Voice Response supported string for OpalInitialise()
 
 #define OPAL_PREFIX_ALL OPAL_PREFIX_H323  " " \
                         OPAL_PREFIX_SIP   " " \
@@ -313,7 +316,10 @@ typedef void (OPAL_EXPORT *OpalFreeMessageFunction)(OpalMessage * message);
                         OPAL_PREFIX_LOCAL " " \
                         OPAL_PREFIX_POTS  " " \
                         OPAL_PREFIX_PSTN  " " \
-                        OPAL_PREFIX_IVR
+                        OPAL_PREFIX_FAX  " " \
+                        OPAL_PREFIX_T38  " " \
+                        OPAL_PREFIX_IVR  " " \
+                        OPAL_PREFIX_IM
 
 
 /**Type code for messages defined by OpalMessage.
@@ -411,6 +417,24 @@ typedef enum OpalMessageType {
   OpalIndCompletedIVR,          /**<Indicates completion of the IVR (VXML) script. This message is returned in
                                     the OpalGetMessage() function. See the OpalStatusIVR structure for
                                     more information. */
+  OpalCmdAuthorisePresence,     /**<Permit or deny authority for the remote presentity to view the presence
+                                    state of a local presentity. See the OpalPresenceStatus structure for
+                                    more information. */
+  OpalCmdSubscribePresence,     /**<Subscribe to the change in presence state for a presentity. See the
+                                    OpalPresenceStatus structure for more information. */
+  OpalCmdSetLocalPresence,      /**<Set, and publish, the local presence state. See the OpalPresenceStatus
+                                    structure for more information. */
+  OpalIndPresenceChange,        /**<Indicates a change the the presence state for a given presentity. This
+                                    message is returned in the OpalGetMessage() function. See the
+                                    OpalPresenceStatus structure for more information. */
+  OpalCmdSendIM,                /**<Send an Instant Message. See the OpalInstantMessage structure for more
+                                    information. */
+  OpalIndReceiveIM,             /**<Indicates receipt of an instant message. This message is returned in the
+                                    OpalGetMessage() function. See the OpalInstantMessage structure for
+                                    more information. */
+  OpalIndSentIM,                /**<Get indication of the disposition of a sent instant message. This message
+                                    is returned in the OpalGetMessage() function. See the OpalInstantMessage
+                                    structure for more information. */
 
 // Always add new messages to ethe end to maintain backward compatibility
   OpalMessageTypeCount
@@ -524,8 +548,11 @@ typedef enum OpalMediaTiming {
   OpalMediaTimingNoChange,      /**< No change to the media data type. */
   OpalMediaTimingSynchronous,   /**< Indicate the read/write function is going to handle
                                      all real time aspects of the media flow. */
-  OpalMediaTimingAsynchronous   /**< Indicate the read/write function does not handle
-                                     the real time aspects of the media flow. */
+  OpalMediaTimingAsynchronous,  /**< Indicate the read/write function does not require
+                                     real time aspects of the media flow. */
+  OpalMediaTimingSimulated      /**< Indicate the read/write function does not handle
+                                     the real time aspects of the media flow and they
+                                     must be simulated by the OPAL library. */
 } OpalMediaTiming;
 
 
@@ -543,6 +570,23 @@ typedef enum OpalMediaTiming {
       command.m_param.m_general.m_mediaMask = "RFC4175*";
       response = OpalSendMessage(hOPAL, &command);
       </code>
+
+   For m_mediaOrder and m_mediaMask, each '\n' seperated sub-string in the
+   array is checked using a simple wildcard matching algorithm.
+
+   The '*' character indicates substrings, for example: "G.711*" would remove
+   "G.711-uLaw-64k" and "G.711-ALaw-64k".
+
+   The '@' character indicates a type of media format, so say "\@video" would
+   remove all video codecs.
+
+   The '!' character indicates a negative test. That is the entres that do NOT
+   match the string are removed. The string after the '!' may contain '*' and
+   '@' characters.
+
+   It should be noted that when the ! operator is used, they are combined
+   differently to the usual application of each entry in turn. Thus, the string
+   "!A\n!B" will result in keeping <i>both</i> A and B formats.
   */
 typedef struct OpalParamGeneral {
   const char * m_audioRecordDevice;   /**< Audio recording device name */
@@ -564,10 +608,12 @@ typedef struct OpalParamGeneral {
                                            which may automatically be transmitted automatically. If NULL
                                            no change is made, but if "" then all media is prevented from
                                            auto-starting.  */
-  const char * m_natRouter;           /**< The host name or IP address of the Network Address Translation
-                                           router which may be between the endpoint and the Internet. */
-  const char * m_stunServer;          /**< The host name or IP address of the STUN server which may be
-                                           used to determine the NAT router characteristics automatically. */
+  const char * m_natMethod;           /**< The Network Address Translation method to use. If an empty string
+                                           and m_natServer is not empty, then "STUN" is assumed. For backward
+                                           compatibility, if this is a host name or IP address of then a
+                                           "Fixed" method for the NAT router is used. */
+  const char * m_natServer;           /**< The host name or IP address of the NAT (e.g. STUN) server which may
+                                           be used to determine the NAT router characteristics automatically. */
   unsigned     m_tcpPortBase;         /**< Base of range of ports to use for TCP communications. This may
                                            be required by some firewalls. */
   unsigned     m_tcpPortMax;          /**< Max of range of ports to use for TCP communications. This may
@@ -638,9 +684,13 @@ typedef struct OpalParamGeneral {
                                            to the application to send a OpalCmdAlerting message to
                                            indicate to the remote system that we are "ringing".
                                            If zero then no change is made. */
-  OpalMediaTiming m_mediaTiming;      /**< Indicate that the media read/write callback function
+  OpalMediaTiming m_mediaTiming;      /**< Indicate how the media read/write callback function
                                            handles the real time aspects of the media flow.
-                                           0=no change, 1=synchronous, 2=asynchronous. */
+                                           0=no change, 1=synchronous, 2=asynchronous,
+                                           3=simulate synchronous. */
+  OpalMediaTiming m_videoSourceTiming;/**< Indicate that the video read callback function
+                                           handles the real time aspects of the media flow.
+                                           This can override the m_mediaTiming. */
 } OpalParamGeneral;
 
 
@@ -767,6 +817,17 @@ typedef struct OpalParamProtocol {
       if (response != NULL && response->m_type == OpalCmdRegistration)
         m_AddressOfRecord = response->m_param.m_registrationInfo.m_identifier
 
+      // Presence registration
+      memset(&command, 0, sizeof(command));
+      command.m_type = OpalCmdRegistration;
+      command.m_param.m_registrationInfo.m_protocol = "pres";
+      command.m_param.m_registrationInfo.m_identifier = "sip:rjongbloed@ekiga.net";
+      command.m_param.m_registrationInfo.m_password = "secret";
+      command.m_param.m_registrationInfo.m_timeToLive = 300;
+      response = OpalSendMessage(hOPAL, &command);
+      if (response != NULL && response->m_type == OpalCmdRegistration)
+        m_AddressOfRecord = response->m_param.m_registrationInfo.m_identifier
+
       // unREGISTER
       memset(&command, 0, sizeof(command));
       command.m_type = OpalCmdRegistration;
@@ -828,6 +889,10 @@ typedef struct OpalParamRegistration {
                                      A value of OPAL_LINE_APPEARANCE_EVENT_PACKAGE will cause the
                                      OpalIndLineAppearance to be sent.
                                      Other values are currently not supported. */
+  const char * m_attributes;    /**< Protocol dependent information in the form:
+                                           key=value\n
+                                           key=value\n
+                                           etc */
 } OpalParamRegistration;
 
 
@@ -955,7 +1020,9 @@ typedef struct OpalParamSetUpCall {
 
   const char * m_protocolCallId;  /**< ID assigned by the underlying protocol for the call. 
                                        Only available in version 18 and above */
-
+  OpalParamProtocol m_overrides;  /**< Overrides for the default parameters for the protocol.
+                                       For example, m_userName and m_displayName can be
+                                       changed on a call by call basis. */
 } OpalParamSetUpCall;
 
 
@@ -990,6 +1057,23 @@ typedef struct OpalStatusIncomingCall {
 } OpalStatusIncomingCall;
 
 
+/**Incoming call response parameters for OpalCmdAlerting and OpalCmdAnswerCall messages.
+
+   When a new call is detected via the OpalIndIncomingCall indication, the application
+   should respond with OpalCmdClearCall, which does not use this structure, or
+   OpalCmdAnswerCall, which does. An optional OpalCmdAlerting may also be sent
+   which also uses this structure to allow for the override of default call parameters
+   such as suer name or display name on a call by call basis.
+  */
+typedef struct OpalParamAnswerCall {
+  const char * m_callToken;       ///< Call token for call to be answered.
+  OpalParamProtocol m_overrides;  /**< Overrides for the default parameters for the protocol.
+                                       For example, m_userName and m_displayName can be
+                                       changed on a call by call basis. */
+  unsigned m_withMedia;           /**< When used with OpalCmdAlerting, if non-zero this
+                                       indicates that early media is to be started. */
+} OpalParamAnswerCall;
+
 /**Type code for media stream status/control.
    This is used by the OpalIndMediaStream indication and OpalCmdMediaStream command
    in the OpalStatusMediaStream structure.
@@ -1030,6 +1114,10 @@ typedef struct OpalStatusMediaStream {
   OpalMediaStates m_state;       /**< For OpalIndMediaStream this indicates the status of the stream.
                                       For OpalCmdMediaStream this indicates the state to move to, see
                                       OpalMediaStates for more information. */
+  int             m_volume;      /**< Set the volume for the media stream as a percentage. Note this
+                                      is dependent on the stream type and may be ignored. Also, a
+                                      percentage of zero does not indicate muting, it indicates no
+                                      change in volume. Use -1, to mute. */
 } OpalStatusMediaStream;
 
 
@@ -1104,6 +1192,103 @@ typedef struct OpalStatusLineAppearance {
 } OpalStatusLineAppearance;
 
 
+/**Type code for presence states.
+   This is used by the OpalPresenceStatus structure.
+  */
+typedef enum OpalPresenceStates {
+  OpalPresenceAuthRequest = -100, ///< Authorisation to view a users state is required
+  OpalUnknownPresentity = -4,     ///< Presentity does not exist
+  OpalPresenceError,              ///< Something bad happened
+  OpalPresenceForbidden,          ///< Access to presence information was specifically forbidden
+  OpalPresenceNone,               ///< No presence status - not the same as Unavailable or Away
+  OpalPresenceUnchanged,          ///< State has not changed from last time
+  OpalPresenceAvailable,          ///< User has a presence and is available to be contacted
+  OpalPresenceUnavailable         ///< User has a presence, but is cannot be contacted
+} OpalPresenceStates;
+
+
+/**Opal Presence information for the various presence messages.
+
+   For OpalIndPresenceChange, m_entity is the local presentity, m_target is
+   the presentity for which the status is changing. This may be a remote
+   presentity, or the same as the m_entity field. The latter would occur after
+   OpalCmdSetLocalPresence, for example, and can indicate if that operation
+   was successful or not.
+
+   For OpalCmdAuthorisePresence, m_entity is the local presentity, m_target is
+   the remote presentity asking for permission to view the local presentities
+   status. The m_state is used to deny acces (OpalPresenceForbidden), deny
+   access politely (OpalPresenceUnavailable) or permit access
+   (OpalPresenceAvailable). While this is usually called in response to an
+   OpalIndPresenceChange with m_state == OpalPresenceAuthRequest, it can also
+   be sent at any time to remove an authorisation. In this case m_state is set
+   to OpalPresenceNone.
+
+   For OpalCmdSetLocalPresence, m_target is unused, m_state should be a
+   positive value. If m_state is OpalPresenceUnchanged then no change is made
+   and the current presence state is returned. The m_note field can be used
+   to provide extra information about the state change.
+
+   For OpalCmdSubscribePresence, m_entity is the local presentity, m_target is
+   the remote presentity we wish to monitor. The m_state is OpalPresenceNone
+   when we wish to stop monitoring, any other value to request. The m_note
+   field may be used to give extra information to the remote system if
+   authorisation is required.
+  */
+typedef struct OpalPresenceStatus {
+  const char *       m_entity;    /**<For OpalIndPresenceChange, this is the presentity
+                                      whose state had changed, usually a remote. For other
+                                      messages, this is the local registered presentity. */
+  const char *       m_target;    /**<The presentity that is being informed about the state
+                                      change. Only used for OpalIndPresenceChange. */
+  const char *       m_service;   /**<Device/system for the presentity that is getting a
+                                      state change. Ignored for commands. */
+  const char *       m_contact;   /**<Contact address, typically a URL, for the service. */
+  const char *       m_capabilities;/**<Capabilities for the service. A '\n' separated list
+                                      of keywords, such as "audio", "Video", "text" etc. */
+  OpalPresenceStates m_state;     /**<The new state of the target entity. */
+  const char *       m_activities;/**<The optional activities, if m_state is OpalPresenceAvailable
+                                      or OpalPresenceUnavailable. Typically something like
+                                      "Busy" or "Away". This can be a '\n' seaparated
+                                      list of simultaneous activities. */
+  const char *       m_note;      /**<Additional "note" that may be attached to the state
+                                      change, e.g. "I want to be frends with you". If
+                                      m_state is OpalPresenceError, then this may contain
+                                      extra information on the error. */
+  const char *       m_infoType;  ///< MIME tyupe for m_infoData, e.g. application/pidf+xml
+  const char *       m_infoData;  ///< Raw information as provided by underlying protocol, e.g. XML.
+} OpalPresenceStatus;
+
+
+/**Opal Instant Message information for the various presence messages.
+   This can be filled out and used in the OpalCmdSendIM command. The result
+   of that transmission is returned by OpalIndSentIM, where m_textBody contains
+   a string indicating the disposition of the message.
+
+   The OpalIndReceiveIM message uses this structure for incoming instant
+   messages from a remote.
+  */
+typedef struct OpalInstantMessage {
+  const char *  m_from;      /**<Address from whom the message is sent. */
+  const char *  m_to;        /**<Address to which the message is sent. */
+  const char *  m_host;      /**<Optional host/proxy. If blank then it is
+                                derived from the m_to address. */
+  const char *  m_conversationId; /**<Conversation identifier. This may be
+                                provided by the caller if athe conversation
+                                exists. If starting a new conversation, leave
+                                empty and OpalCmdSendIM will return it. */
+  const char *  m_textBody;  /**<Simple text body, ignored if m_bodyCount > 0.
+                                 This will always be MIME type "text/plain" */
+  unsigned      m_bodyCount; /**<Count of bodies in m_mimeType and m_bodies */
+  const char ** m_mimeType;  /**<MIME type for each body, e.g. "text/html" */
+  const char ** m_bodies;    /**<Body data for each MIME type */
+  unsigned      m_messageId; /**<Identifer for this message. This can be used
+                                 to match a message sent with OpalCmdSendIM with
+                                 the disposition in OpalIndSentIM. It is not set
+                                 by the user, and is returned by OpalCmdSendIM. */
+} OpalInstantMessage;
+
+
 /**Type of mixing for video when recording.
    This is used by the OpalCmdStartRecording command in the OpalParamRecording structure.
   */
@@ -1212,7 +1397,7 @@ typedef struct OpalStatusCallCleared {
   */
 typedef enum OpalCallEndReason {
   OpalCallEndedByLocalUser,         /// Local endpoint application cleared call
-  OpalCallEndedByNoAccept,          /// Local endpoint did not accept call OnIncomingCall()=PFalse
+  OpalCallEndedByNoAccept,          /// Local endpoint did not accept call OnIncomingCall()=false
   OpalCallEndedByAnswerDenied,      /// Local endpoint declined to answer call
   OpalCallEndedByRemoteUser,        /// Remote endpoint application cleared call
   OpalCallEndedByRefusal,           /// Remote endpoint refused call
@@ -1266,8 +1451,9 @@ struct OpalMessage {
     OpalParamRegistration    m_registrationInfo;   ///< Used by OpalCmdRegistration
     OpalStatusRegistration   m_registrationStatus; ///< Used by OpalIndRegistration
     OpalParamSetUpCall       m_callSetUp;          ///< Used by OpalCmdSetUpCall/OpalIndProceeding/OpalIndAlerting/OpalIndEstablished
-    const char *             m_callToken;          ///< Used by OpalCmdAnswerCall/OpalCmdHoldcall/OpalCmdRetrieveCall/OpalCmdStopRecording/OpalCmdAlerting
+    const char *             m_callToken;          ///< Used by OpalCmdHoldcall/OpalCmdRetrieveCall/OpalCmdStopRecording
     OpalStatusIncomingCall   m_incomingCall;       ///< Used by OpalIndIncomingCall
+    OpalParamAnswerCall      m_answerCall;         ///< Used by OpalCmdAnswerCall/OpalCmdAlerting
     OpalStatusUserInput      m_userInput;          ///< Used by OpalIndUserInput/OpalCmdUserInput
     OpalStatusMessageWaiting m_messageWaiting;     ///< Used by OpalIndMessageWaiting
     OpalStatusLineAppearance m_lineAppearance;     ///< Used by OpalIndLineAppearance
@@ -1278,6 +1464,9 @@ struct OpalMessage {
     OpalParamRecording       m_recording;          ///< Used by OpalCmdStartRecording
     OpalStatusTransferCall   m_transferStatus;     ///< Used by OpalIndTransferCall
     OpalStatusIVR            m_ivrStatus;          ///< Used by OpalIndCompletedIVR
+    OpalPresenceStatus       m_presenceStatus;     ///< used by OpalCmdAuthorisePresence/OpalCmdSubscribePresence/OpalIndPresenceChange/OpalCmdSetLocalPresence
+    OpalInstantMessage       m_instantMessage;     ///< Used by OpalCmdSendIM/OpalIndReceiveIM
+
   } m_param;
 };
 
@@ -1298,7 +1487,7 @@ class OpalMessagePtr
     OpalMessageType GetType() const;
     void SetType(OpalMessageType type);
 
-    const char               * GetCallToken() const;          ///< Used by OpalCmdAnswerCall/OpalCmdHoldCall/OpalCmdRetrieveCall/OpalCmdStopRecording/OpalCmdAlerting
+    const char               * GetCallToken() const;          ///< Used by OpalCmdHoldCall/OpalCmdRetrieveCall/OpalCmdStopRecording
     void                       SetCallToken(const char * token);
 
     const char               * GetCommandError() const;       ///< Used by OpalIndCommandError
@@ -1309,6 +1498,7 @@ class OpalMessagePtr
     OpalStatusRegistration   * GetRegistrationStatus() const; ///< Used by OpalIndRegistration
     OpalParamSetUpCall       * GetCallSetUp() const;          ///< Used by OpalCmdSetUpCall/OpalIndProceeding/OpalIndAlerting/OpalIndEstablished
     OpalStatusIncomingCall   * GetIncomingCall() const;       ///< Used by OpalIndIncomingCall
+    OpalParamAnswerCall      * GetAnswerCall() const;         ///< Used by OpalCmdAnswerCall/OpalCmdAlerting
     OpalStatusUserInput      * GetUserInput() const;          ///< Used by OpalIndUserInput/OpalCmdUserInput
     OpalStatusMessageWaiting * GetMessageWaiting() const;     ///< Used by OpalIndMessageWaiting
     OpalStatusLineAppearance * GetLineAppearance() const;     ///< Used by OpalIndLineAppearance
@@ -1318,6 +1508,8 @@ class OpalMessagePtr
     OpalParamSetUserData     * GetSetUserData() const;        ///< Used by OpalCmdSetUserData
     OpalParamRecording       * GetRecording() const;          ///< Used by OpalCmdStartRecording
     OpalStatusTransferCall   * GetTransferStatus() const;     ///< Used by OpalIndTransferCall
+    OpalPresenceStatus       * GetPresenceStatus() const;     ///< Used by OpalCmdAuthorisePresence/OpalCmdSubscribePresence/OpalIndPresenceChange/OpalCmdSetLocalPresence
+    OpalInstantMessage       * GetInstantMessage() const;     ///< Used by OpalCmdSendIM/OpalIndReceiveIM
 
   protected:
     OpalMessage * m_message;
