@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24547 $
+ * $Revision: 29572 $
  * $Author: rjongbloed $
- * $Date: 2010-06-30 01:36:31 -0500 (Wed, 30 Jun 2010) $
+ * $Date: 2013-04-24 23:28:21 -0500 (Wed, 24 Apr 2013) $
  */
 
 #ifndef OPAL_OPAL_CALL_H
@@ -94,7 +94,7 @@ class OpalCall : public PSafeObject
   //@{
     /**Indicate tha all connections in call are connected and media is going.
       */
-    PBoolean IsEstablished() const { return isEstablished; }
+    PBoolean IsEstablished() const { return m_isEstablished; }
 
     /**Call back to indicate that the call has been established.
        At this point in time every connection in the call is in the
@@ -108,9 +108,9 @@ class OpalCall : public PSafeObject
 
     /**Get the call clearand reason for this connection shutting down.
        Note that this function is only generally useful in the
-       H323EndPoint::OnConnectionCleared() function. This is due to the
+       OpalEndPoint::OnClearedCall() function. This is due to the
        connection not being cleared before that, and the object not even
-       exiting after that.
+       existing after that.
 
        If the call is still active then this will return OpalConnection::NumCallEndReasons.
       */
@@ -192,7 +192,7 @@ class OpalCall : public PSafeObject
        remote endpoint is "ringing". Generally some time after the
        MakeConnection() function was called, this is function is called.
 
-       If PFalse is returned the connection is aborted.
+       If false is returned the connection is aborted.
 
        If an application overrides this function, it should generally call the
        ancestor version for correct operation. An application would typically
@@ -291,6 +291,10 @@ class OpalCall : public PSafeObject
       const OpalConnection & connection  ///<  Source requesting formats
     ) const;
 
+    /**Get number of connections in call.
+      */
+    PINDEX GetConnectionCount() const { return connectionsActive.GetSize(); }
+
     /**Get the specified active connection in call.
       */
     PSafePtr<OpalConnection> GetConnection(
@@ -319,14 +323,20 @@ class OpalCall : public PSafeObject
     }
 
     /**Put call on hold.
+       This function places the remote user of the network connection in the
+       call on hold.
       */
     bool Hold();
 
     /**Retrieve call from hold.
+       This function retrieves the remote user of the network connection in
+       the call from hold.
       */
     bool Retrieve();
 
     /**Indicate if call is in hold.
+       This function determines if the remote user of the network connection
+       in the call is currently on hold.
       */
     bool IsOnHold() const;
 
@@ -371,18 +381,31 @@ class OpalCall : public PSafeObject
   /**@name Media management */
   //@{
     /**Get the media formats of the connections in call.
-       This returns the intersection of all the media formats that all
-       connections in the call, optionally excepting the one provided as a
-       parameter, are capable of.
+       This returns the intersection of all the media formats available in
+       connections in the call, not including the parameter. It is in
+       esence the "local" media formats available to the connection.
 
        This will also add to the list all media formats for which there are
        transcoders registered.
       */
     virtual OpalMediaFormatList GetMediaFormats(
-      const OpalConnection & connection,  ///<  Connection requesting formats
-      PBoolean includeSpecifiedConnection     ///<  Include parameters media
+      const OpalConnection & connection  ///<  Connection requesting formats
     );
 
+    /**Adjust media formats available on a connection.
+       This is called by a connection after it has called
+       OpalCall::GetMediaFormats() to get all media formats that it can use so
+       that an application may remove or reorder the media formats before they
+       are used to open media streams.
+
+       The default behaviour calls the OpalManager function of the same name.
+      */
+    virtual void AdjustMediaFormats(
+      bool local,                         ///<  Media formats a local ones to be presented to remote
+      const OpalConnection & connection,  ///<  Connection that is about to use formats
+      OpalMediaFormatList & mediaFormats  ///<  Media formats to use
+    ) const;
+
     /**Open source media streams for the specified connection.
        A source media stream is opened for the connection, if successful then
        sink media streams are created for every other connection in the call.
@@ -396,10 +419,11 @@ class OpalCall : public PSafeObject
       OpalConnection & connection,              ///<  Connection requesting open
       const OpalMediaType & mediaType,          ///<  Media type of channel to open
       unsigned sessionID = 0,                   ///<  Session to start streams on
-      const OpalMediaFormat & preselectedFormat = OpalMediaFormat()  ///< Format for source stream to choose from
+      const OpalMediaFormat & preselectedFormat = OpalMediaFormat(),  ///< Format for source stream to choose from
 #if OPAL_VIDEO
-      , OpalVideoFormat::ContentRole contentRole = OpalVideoFormat::eNoRole ///< Content role for video
+      OpalVideoFormat::ContentRole contentRole = OpalVideoFormat::eNoRole, ///< Content role for video
 #endif
+      bool transfer = false
     );
 
     /**Select media format pair from the source/destination list.
@@ -415,18 +439,6 @@ class OpalCall : public PSafeObject
       OpalMediaFormat & dstFormat             ///<  Selected destination format to be used
     ) const;
 
-    /**Callback from the RTP session for statistics monitoring.
-       This is called every so many packets on the transmitter and receiver
-       threads of the RTP session indicating that the statistics have been
-       updated.
-
-       The default behaviour does nothing.
-      */
-    virtual void OnRTPStatistics(
-      const OpalConnection & connection,  ///<  Connection for the channel
-      const RTP_Session & session         ///<  Session with statistics
-    );
-
     /**Start the media streams on the connections.
      */
     virtual void StartMediaStreams();
@@ -434,13 +446,6 @@ class OpalCall : public PSafeObject
     /**Close the media streams on the connections.
      */
     virtual void CloseMediaStreams();
-
-    /**See if the media can bypass the local host.
-     */
-    virtual PBoolean IsMediaBypassPossible(
-      const OpalConnection & connection,  ///<  Source connection
-      unsigned sessionID                  ///<  Session ID for media channel
-    ) const;
   //@}
 
   /**@name User indications */
@@ -480,13 +485,13 @@ class OpalCall : public PSafeObject
      */
     const PString & GetToken() const { return myToken; }
 
-    /**Get the A party for the call.
+    /**Get the A party URI for the call.
        Note this will be available even after the A party connection has been
        released from the call.
      */
     const PString & GetPartyA() const { return m_partyA; }
 
-    /**Get the B party for the call.
+    /**Get the B party URI for the call.
        Note this will be available even after the B party connection has been
        released from the call. Also this will only be the first B party if the
        object represents a conference call with more that 2 parties.
@@ -502,15 +507,48 @@ class OpalCall : public PSafeObject
       const PString & b
     ) { m_partyB = b; }
 
+    /**Get the A party display name for the call.
+       Note this will be available even after the A party connection has been
+       released from the call.
+     */
+    const PString & GetNameA() const { return m_nameA; }
+
+    /**Get the B party display name for the call.
+       Note this will be available even after the B party connection has been
+       released from the call. Also this will only be the first B party if the
+       object represents a conference call with more that 2 parties.
+     */
+    const PString & GetNameB() const { return m_nameB; }
+
     /**Get indication that A-Party is the network.
        This will indicate if the call is "incoming" or "outgoing" by looking at
        the type of the A-party connection.
       */
-    bool IsNetworkOriginated() const;
+    bool IsNetworkOriginated() const { return m_networkOriginated; }
+
+    /**Get remote/network party URI.
+      */
+    const PString & GetRemoteParty() const { return m_networkOriginated ? m_partyA : m_partyB; }
+
+    /**Get local party URI.
+      */
+    const PString & GetLocalParty() const { return m_networkOriginated ? m_partyB : m_partyA; }
+
+    /**Get remote/network party display name.
+      */
+    const PString & GetRemoteName() const { return m_networkOriginated ? m_nameA : m_nameB; }
+
+    /**Get local party display name.
+      */
+    const PString & GetLocalName() const { return m_networkOriginated ? m_nameB : m_nameA; }
 
     /**Get the time the call started.
      */
-    const PTime & GetStartTime() const { return startTime; }
+    const PTime & GetStartTime() const { return m_startTime; }
+
+    /**Get the time the call started.
+     */
+    const PTime & GetEstablishedTime() const { return m_establishedTime; }
   //@}
 
 #if OPAL_HAS_MIXER
@@ -565,9 +603,16 @@ class OpalCall : public PSafeObject
 
     void InternalOnClear();
 
-  protected:
     void SetPartyNames();
 
+#if OPAL_T38_CAPABILITY
+    bool IsSwitchingT38()     const { return m_T38SwitchState != e_NotSwitchingT38; }
+    bool IsSwitchingToT38()   const { return m_T38SwitchState == e_SwitchingToT38;  }
+    void ResetSwitchingT38()        { m_T38SwitchState = e_NotSwitchingT38; }
+    void SetSwitchingT38(bool to)   { m_T38SwitchState = to ? e_SwitchingToT38 : e_SwitchingFromT38; }
+#endif
+
+  protected:
     bool EnumerateConnections(
       PSafePtr<OpalConnection> & connection,
       PSafetyMode mode,
@@ -580,12 +625,17 @@ class OpalCall : public PSafeObject
 
     PString m_partyA;
     PString m_partyB;
-    PTime   startTime;
-    bool    isEstablished;
-    bool    isClearing;
+    PString m_nameA;
+    PString m_nameB;
+    bool    m_networkOriginated;
+    PTime   m_startTime;
+    PTime   m_establishedTime;
+    bool    m_isEstablished;
+    bool    m_isClearing;
+    PAtomicBoolean m_isCleared;
 
     OpalConnection::CallEndReason callEndReason;
-    PSyncPoint                  * endCallSyncPoint;
+    std::list<PSyncPoint *> m_endCallSyncPoint;
 
     PSafeList<OpalConnection> connectionsActive;
 
@@ -593,10 +643,24 @@ class OpalCall : public PSafeObject
     OpalRecordManager * m_recordManager;
 #endif
 
+#if OPAL_SCRIPT
+    PDECLARE_ScriptFunctionNotifier(OpalCall, ScriptClear);
+#endif
+
+#if OPAL_T38_CAPABILITY
+    enum {
+      e_NotSwitchingT38,
+      e_SwitchingToT38,
+      e_SwitchingFromT38
+    } m_T38SwitchState;
+#endif
+
   //use to add the connection to the call's connection list
   friend OpalConnection::OpalConnection(OpalCall &, OpalEndPoint &, const PString &, unsigned int, OpalConnection::StringOptions *);
   //use to remove the connection from the call's connection list
   friend OpalConnection::~OpalConnection();
+
+  P_REMOVE_VIRTUAL_VOID(OnRTPStatistics(const OpalConnection &, const OpalRTPSession &));
 };
 
 
