@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26231 $
+ * $Revision: 30003 $
  * $Author: rjongbloed $
- * $Date: 2011-07-19 21:23:34 -0500 (Tue, 19 Jul 2011) $
+ * $Date: 2013-06-19 22:38:44 -0500 (Wed, 19 Jun 2013) $
  */
 
 #ifndef OPAL_OPAL_ENDPOINT_H
@@ -42,9 +42,11 @@
 #include <opal/mediafmt.h>
 #include <opal/transports.h>
 
+
 class OpalCall;
 class OpalMediaStream;
 
+
 /**This class describes an endpoint base class.
    Each protocol (or psuedo-protocol) would create a descendant off this
    class to manage its particular subsystem. Typically this would involve
@@ -107,18 +109,24 @@ class OpalEndPoint : public PObject
 
   /**@name Listeners management */
   //@{
-    /**Add a set of listeners to the endoint.
+    /**Start a set of listeners on the endoint.
        This allows for the automatic creating of incoming call connections. 
        If the list is empty then GetDefaultListeners() is used.
 
+       The \p add parameter indicatest that existing listeners are left running
+       and new ones are added. If false then listeners that are not in the
+       \p interfaces list are stopped. Listeners not already running are
+       started and ones that are already running are untouched.
+
        Note: while the \p interfaces parameter is a string array, each element
        of the array should be compatible with OpalTransportAddress.
 
        See OpalTransportAddress for more details on the syntax of an interface
        definition.
       */
-    PBoolean StartListeners(
-      const PStringArray & interfaces ///<  Address of interface to listen on.
+    bool StartListeners(
+      const PStringArray & interfaces, ///<  Addresses of interfaces to listen on.
+      bool add = true                  ///<  Add listeners to existing list
     );
 
     /**Add a listener to the endoint.
@@ -142,16 +150,32 @@ class OpalEndPoint : public PObject
     );
 
     /**Get the default listeners for the endpoint type.
-       Default behaviour returns empty list if defaultSignalPort is zero, else
-       one entry using tcp and INADDR_ANY, eg tcp$*:1720
+       Default behaviour uses GetDefaultTransport() to produce a list of
+       listener addresses based on IPv4 and IPv6 versions of INADDR_ANY.
       */
     virtual PStringArray GetDefaultListeners() const;
 
     /**Get comma separated list of transport protocols to create if
        no explicit listeners started.
+       Transport protocols are as per OpalTransportAddress, e.g. "udp$". It
+       may also have a ":port" after it if that transport type does not use
+       the value from GetDefaultSignalPort().
       */
     virtual PString GetDefaultTransport() const;
 
+    /**Get the default signal port for this endpoint.
+     */
+    virtual WORD GetDefaultSignalPort() const;
+
+#if OPAL_PTLIB_SSL
+    /** Apply the SSL certificates/key for SSL based calls, e.g. sips or h323s
+      */
+    virtual bool ApplySSLCredentials(
+      PSSLContext & context,    ///< Context to which t set certificates
+      bool create               ///< Create self signed cert/key if required
+    ) const;
+#endif
+
     /**Find a listener given the transport address.
       */
     OpalListener * FindListener(
@@ -185,25 +209,19 @@ class OpalEndPoint : public PObject
       PBoolean excludeLocalHost = true,       ///<  Flag to exclude 127.0.0.1
       const OpalTransport * associatedTransport = NULL
                           ///<  Associated transport for precedence and translation
-    );
+    ) const;
 
     /**Handle new incoming connection.
        This will either create a new connection object or utilise a previously
        created connection on the same transport address and reference number.
       */
-    PDECLARE_NOTIFIER(PThread, OpalEndPoint, ListenerCallback);
-
-    /**Handle new incoming connection from listener.
-
-       A return value of true indicates that the transport object should be
-       deleted by the caller. false indicates that something else (eg the
-       connection) has taken over responsibility for deleting the transport.
-
-       The default behaviour just returns true.
-      */
-    virtual PBoolean NewIncomingConnection(
-      OpalTransport * transport  ///<  Transport connection came in on
+#if DOXYGEN
+    virtual void NewIncomingConnection(
+      OpalListener & listener,            ///<  Listner that created transport
+      const OpalTransportPtr & transport  ///<  Transport connection came in on
     );
+#endif
+    PDECLARE_AcceptHandlerNotifier(OpalEndPoint, NewIncomingConnection);
 
     /**Call back for a new connection has been constructed.
        This is called after CreateConnection has returned a new connection.
@@ -421,19 +439,46 @@ class OpalEndPoint : public PObject
     /**A call back function to monitor the progress of a transfer.
        When a transfer operation is initiated, the Transfer() function will
        generally return immediately and the transfer may take some time. This
-       call back can give indicateion to the application of the progress of
+       call back can give an indication to the application of the progress of
+       the transfer.
        the transfer.
 
-       The exact format of the \p info parameter is dependent on the protocol
-       being used. It will always have a value info["result"] which will be
-       "success" or "failed". Other variables are protocol dependent.
+       For example in SIP, the OpalCall::Transfer() function will have sent a
+       REFER request to the remote party. The remote party sends us NOTIFY
+       requests about the progress of the REFER request.
 
-       For SIP, there is an additional info["state"] containing the NOTIFY
-       subscription state and an info["code"] entry containing the 3 digit
-       code returned in the NOTIFY body.
+       An application can now make a decision during the transfer operation
+       to short circuit the sequence, or let it continue. It can also
+       determine if the transfer did not go through, and it should "take back"
+       the call. Note no action is required to "take back" the call other than
+       indicate to the user that they are back on.
 
        A return value of false will immediately disconnect the current call.
 
+       The exact format of the \p info parameter is dependent on the protocol
+       being used. As a minimum, it will always have a values info["result"]
+       and info["party"].
+
+       The info["party"] indicates the part the \p connection is playing in
+       the transfer. This will be:
+          "A"   party being transferred
+          "B"   party initiating the transfer of "A"
+          "C"   party "A" is being transferred to
+
+       The info["result"] will be at least one of the following:
+          "success"     Transfer completed successfully (party A or B)
+          "incoming"    New call was from a transfer (party C)
+          "started"     Transfer operation has started (party A)
+          "progress"    Transfer is in progress (party B)
+          "blind"       Transfer is blind, no further notification (party B)
+          "error"       Transfer could not begin (party B)
+          "failed"      Transfer started but did not complete (party A or B)
+
+       For SIP, there may be an additional info["state"] containing the NOTIFY
+       subscription state, an info["code"] entry containing the 3 digit
+       code returned in the NOTIFY body and info["Referred-By"] indicating the
+       URI of party B. Other fields may also be present.
+
        The default behaviour calls the OpalManager function of the same name.
        The default action of that function is to return false, thereby
        releasing the connection if the info["result"] == "success".
@@ -449,6 +494,8 @@ class OpalEndPoint : public PObject
        call disposed of. Note that this function returns quickly and the
        disposal happens at some later time by a background thread. This it is
        safe to call this function from anywhere.
+
+       If \p sync is not NULL then it is signalled when the calls are cleared.
       */
     virtual PBoolean ClearCall(
       const PString & token,    ///<  Token for identifying connection
@@ -457,8 +504,8 @@ class OpalEndPoint : public PObject
     );
 
     /**Clear a current connection.
-       This hangs up the connection to a remote endpoint. Note that these functions
-       are synchronous
+       This hangs up the connection to a remote endpoint. Note that this function
+       is always synchronous. If \p sync is NULL then a local PSyncPoint is used.
       */
     virtual PBoolean ClearCallSynchronous(
       const PString & token,    ///<  Token for identifying connection
@@ -478,13 +525,21 @@ class OpalEndPoint : public PObject
     );
 
     /**Find a connection that uses the specified token.
-       This searches the endpoint for the connection that contains the token
-       as provided by functions such as MakeConnection().
+       This searches the endpoint for the connection that is identified by the
+       connection token as provided by functions such as MakeConnection().
+
+       The \p token string may also be the call token that identifies the
+       OpalCall instance. The first connection in that call that has this
+       endpoint as it's endpoint is returned.
+
+       Finally, the \p token string may also be of the form prefix:name,
+       e.g. ivr:fred, and the GetLocalPartyName() is used to locate the
+       connection.
       */
     PSafePtr<OpalConnection> GetConnectionWithLock(
       const PString & token,     ///<  Token to identify connection
       PSafetyMode mode = PSafeReadWrite ///< Locking mode
-    ) { return connectionsActive.FindWithLock(token, mode); }
+    ) const;
 
     /**Find a connection that uses the specified token.
        This searches the endpoint for the connection that contains the token
@@ -496,7 +551,7 @@ class OpalEndPoint : public PObject
     PSafePtr<ConnClass> GetConnectionWithLockAs(
       const PString & token,     ///<  Token to identify connection
       PSafetyMode mode = PSafeReadWrite ///< Locking mode
-    )
+    ) const
     {
       PSafePtr<ConnClass> connection = PSafePtrCast<OpalConnection, ConnClass>(GetConnectionWithLock(token, mode));
       if (connection == NULL) {
@@ -582,6 +637,45 @@ class OpalEndPoint : public PObject
       const OpalMediaStream & stream     ///<  Media stream being closed
     );
 
+    /**Set media security methods in priority order.
+       The \p security parameter is an array of names for security methods,
+       e.g. { "Clear", "AES_CM_128_HMAC_SHA1_80", "AES_CM_128_HMAC_SHA1_32" }.
+       Note "Clear" is not compulsory and if absent will mean that a secure
+       media is required or the call will not proceed.
+
+       An empty list is assumed to be "Clear".
+      */
+    void SetMediaCryptoSuites(
+      const PStringArray & security   ///< New security methods to use
+    );
+
+    /**Get media security methods in priority order.
+       Returns an array of names for security methods,
+       e.g. { "Clear", "AES_CM_128_HMAC_SHA1_80", "AES_CM_128_HMAC_SHA1_32" }.
+      */
+    PStringArray GetMediaCryptoSuites() const
+      { return m_mediaCryptoSuites.IsEmpty() ? GetAllMediaCryptoSuites() : m_mediaCryptoSuites; }
+
+    /**Get all possible media security methods for this endpoint type.
+       Note this will always return "Clear" as the first entry.
+      */
+    virtual PStringArray GetAllMediaCryptoSuites() const;
+
+#if P_NAT
+    /** Get all NAT Methods
+      */
+    PNatStrategy & GetNatMethods() const;
+
+    /**Return the NAT method to use.
+       Returns NULL if address is a local address as per IsLocalAddress().
+       Always returns the NAT method if address is zero.
+       Note, the pointer is NOT to be deleted by the user.
+      */
+    virtual PNatMethod * GetNatMethod(
+      const PIPSocket::Address & remoteAddress = PIPSocket::GetDefaultIpAny()
+    ) const;
+#endif
+
 #if OPAL_VIDEO
     /**Create an PVideoInputDevice for a source media stream.
       */
@@ -638,6 +732,8 @@ class OpalEndPoint : public PObject
     );
   //@}
 
+
+#if OPAL_HAS_IM
   /**@name Instant Messaging */
   //@{
     /**Send text message
@@ -663,10 +759,20 @@ class OpalEndPoint : public PObject
       const OpalIM & message
     );
   //@}
+#endif // OPAL_HAS_IM
+
 
   /**@name Other services */
   //@{
-    /**Callback called when Message Waiting Indication (MWI) is received
+    /**Callback called when Message Waiting Indication (MWI) is received.
+       Multiple callbacks may occur with each MessageWaitingType. A \p type
+       of NumMessageWaitingTypes indicates the server is unable to distinguish
+       the message type.
+
+       The \p extraInfo parameter is generally of the form "a/b" where a and b
+       unsigned integers representing new and old message count. However, it
+       may be a simple "yes" or "no" if the remote cannot provide a message
+       count.
      */
     virtual void OnMWIReceived (
       const PString & party,                ///< Name of party MWI is for
@@ -674,6 +780,49 @@ class OpalEndPoint : public PObject
       const PString & extraInfo             ///< Addition information on the MWI
     );
 
+    /**Get conference state information for all nodes.
+       This obtains the state of one or more conferences managed by this
+       endpoint. If this endpoint does not do conferencing, then false is
+       returned.
+
+       The \p name parameter may be one of the aliases for the conference, or
+       the internal URI for the conference. An empty string indicates all 
+       active conferences are to be returned.
+
+       Note that if the \p name does not match an active conference, true is
+       still returned, but the states list will be empty.
+
+       The default behaviour returns false indicating this is not a
+       conferencing endpoint.
+      */
+    virtual bool GetConferenceStates(
+      OpalConferenceStates & states,           ///< List of conference states
+      const PString & name = PString::Empty() ///< Name for specific node, empty string is all
+    ) const;
+
+    /**Call back when conferencing state information changes.
+       If a conferencing endpoint type detects a change in a conference nodes
+       state, as would be returned by GetConferenceStatus() then this function
+       will be called on all endpoints in the OpalManager.
+
+       The \p uri parameter is as is the internal URI for the conference.
+
+       Default behaviour does nothing.
+      */
+    virtual void OnConferenceStatusChanged(
+      OpalEndPoint & endpoint,  /// < Endpoint sending state change
+      const PString & uri,      ///< Internal URI of conference node that changed
+      OpalConferenceState::ChangeType change ///< Change that occurred
+    );
+
+    /**Build a list of network accessible URIs given a user name.
+       This typically gets URI's like sip:user@interface, h323:user@interface
+       etc, for each listener of each endpoint.
+      */
+    virtual PStringList GetNetworkURIs(
+      const PString & name
+    ) const;
+
     /** Execute garbage collection for endpoint.
         Returns true if all garbage has been collected.
         Default behaviour deletes the objects in the connectionsActive list.
@@ -695,10 +844,6 @@ class OpalEndPoint : public PObject
       */
     PBoolean HasAttribute(Attributes opt) const { return (attributeBits&opt) != 0; }
 
-    /**Get the default signal port for this endpoint.
-     */
-    WORD GetDefaultSignalPort() const { return defaultSignalPort; }
-
     /**Get the product info for all endpoints.
       */
     const OpalProductInfo & GetProductInfo() const { return productInfo; }
@@ -727,13 +872,18 @@ class OpalEndPoint : public PObject
       */
     void SetDefaultDisplayName(const PString & name) { defaultDisplayName = name; }
 
-    /**Get the initial bandwidth parameter.
+    /**Get the initial bandwidth parameter for a connection.
      */
-    unsigned GetInitialBandwidth() const { return initialBandwidth; }
+    OpalBandwidth GetInitialBandwidth(
+      OpalBandwidth::Direction dir   ///< Bandwidth direction
+    ) const;
 
-    /**Get the initial bandwidth parameter.
+    /**Set the initial bandwidth parameter for a connection.
      */
-    void SetInitialBandwidth(unsigned bandwidth) { initialBandwidth = bandwidth; }
+    void SetInitialBandwidth(
+      OpalBandwidth::Direction dir,   ///< Bandwidth direction
+      OpalBandwidth bandwidth         ///< New bandwidth
+    );
 
     /**Get the set of listeners (incoming call transports) for this endpoint.
      */
@@ -760,23 +910,18 @@ class OpalEndPoint : public PObject
     void SetSendUserInputMode(OpalConnection::SendUserInputModes mode) { defaultSendUserInputMode = mode; }
   //@}
 
-#if OPAL_PTLIB_SSL
-    /** Get the name of the file to use as SSL certificate for SSL based calls, e.g. sips or h323s
-      */
-    PString GetSSLCertificate() const;
-#endif
-
   protected:
     OpalManager   & manager;
     PCaselessString prefixName;
     unsigned        attributeBits;
-    WORD            defaultSignalPort;
     PINDEX          m_maxSizeUDP;
     OpalProductInfo productInfo;
     PString         defaultLocalPartyName;
     PString         defaultDisplayName;
+    PStringArray    m_mediaCryptoSuites;
 
-    unsigned initialBandwidth;  // in 100s of bits/sev
+    OpalBandwidth m_initialRxBandwidth;
+    OpalBandwidth m_initialTxBandwidth;
     OpalConnection::StringOptions      m_defaultStringOptions;
     OpalConnection::SendUserInputModes defaultSendUserInputMode;
 
@@ -788,16 +933,16 @@ class OpalEndPoint : public PObject
     } connectionsActive;
     OpalConnection * AddConnection(OpalConnection * connection);
 
-    PMutex inUseFlag;
-
     friend void OpalManager::GarbageCollection();
-    friend void OpalConnection::Release(CallEndReason reason);
+    friend void OpalConnection::Release(CallEndReason,bool);
 
   private:
     P_REMOVE_VIRTUAL(PBoolean, OnIncomingConnection(OpalConnection &, unsigned), false);
     P_REMOVE_VIRTUAL(PBoolean, OnIncomingConnection(OpalConnection &), false);
     P_REMOVE_VIRTUAL_VOID(AdjustMediaFormats(const OpalConnection &, OpalMediaFormatList &) const);
     P_REMOVE_VIRTUAL_VOID(OnMessageReceived(const PURL&,const PString&,const PURL&,const PString&,const PString&,const PString&));
+    P_REMOVE_VIRTUAL(OpalMediaSession *, CreateMediaSession(OpalConnection &, unsigned, const OpalMediaType &), NULL);
+    P_REMOVE_VIRTUAL(PBoolean, NewIncomingConnection(OpalTransport *), false);
 };
 
 
