@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25002 $
+ * $Revision: 29200 $
  * $Author: rjongbloed $
- * $Date: 2011-01-04 22:39:39 -0600 (Tue, 04 Jan 2011) $
+ * $Date: 2013-03-04 22:07:48 -0600 (Mon, 04 Mar 2013) $
  */
 
 #ifndef OPAL_OPAL_MEDIAFMT_H
@@ -36,16 +36,11 @@
 #pragma interface
 #endif
 
-#ifdef _MSC_VER
-#if _MSC_VER < 1300   
-#pragma warning(disable:4663)
-#endif
-#endif
-
 #include <opal/buildopts.h>
 
 #include <opal/mediatype.h>
 #include <rtp/rtp.h>
+#include <ptlib/bitwise_enum.h>
 
 #if OPAL_VIDEO
 #include <ptlib/videoio.h>
@@ -53,14 +48,54 @@
 
 #include <limits>
 
-#ifdef min
-#undef min
-#endif
-#ifdef max
-#undef max
+
+class PluginCodec_OptionMap;
+class OpalMediaFormat;
+class H225_BandWidth;
+
+
+///////////////////////////////////////////////////////////////////////////////
+
+/** Type for bandwidth (bitrate) specification.
+    Currently 32 bit unsigned, but could be set to 64 bit if we ever need
+    more than 4Gb/s
+  */
+class OpalBandwidth {
+  public:
+    enum Direction {
+      Rx = 1,
+      Tx,
+      RxTx,
+      TxRx = RxTx
+    };
+    friend std::ostream & operator<<(std::ostream & strm, OpalBandwidth::Direction dir);
+
+    typedef unsigned int_type;
+
+    __inline OpalBandwidth(int_type bps = 0) : m_bps(bps) { }
+    __inline OpalBandwidth & operator=(int_type bps) { m_bps = bps; return *this; }
+    __inline operator int_type() const { return m_bps; }
+
+    __inline OpalBandwidth & operator+=(const OpalBandwidth & bw) { m_bps += bw.m_bps; return *this; }
+    __inline OpalBandwidth & operator-=(const OpalBandwidth & bw) { m_bps += bw.m_bps; return *this; }
+    __inline OpalBandwidth & operator&=(const OpalBandwidth & bw) { m_bps &= bw.m_bps; return *this; }
+    __inline OpalBandwidth & operator+=(int_type bps) { m_bps += bps; return *this; }
+    __inline OpalBandwidth & operator-=(int_type bps) { m_bps += bps; return *this; }
+
+    friend std::ostream & operator<<(std::ostream & strm, const OpalBandwidth & bw);
+    friend std::istream & operator>>(std::istream & strm, OpalBandwidth & bw);
+
+#if OPAL_H323
+    OpalBandwidth(const H225_BandWidth & bw);
+    OpalBandwidth & operator=(const H225_BandWidth & bw);
+    void SetH225(H225_BandWidth & bw) const;
 #endif
 
-class OpalMediaFormat;
+    static OpalBandwidth Max() { return OpalBandwidth(UINT_MAX); }
+
+  protected:
+    int_type m_bps;
+};
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -92,6 +127,20 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
 
   /**@name Operations */
   //@{
+    /**Add a format(s) to the list.
+       If any format is invalid or already in the list then it is not added.
+      */
+    OpalMediaFormatList & operator+=(
+      const char * wildcard    ///< Wildcard for formats to add
+    ) { PConstString w(wildcard); return operator+=(w); }
+
+    /**Add a format(s) to the list.
+       If any format is invalid or already in the list then it is not added.
+      */
+    OpalMediaFormatList & operator+=(
+      const PString & wildcard    ///< Wildcard for formats to add
+    );
+
     /**Add a format to the list.
        If the format is invalid or already in the list then it is not added.
       */
@@ -120,14 +169,25 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
       const OpalMediaFormatList & formats    ///<  Formats to remove
     );
 
-    /**Get a format iterator in the list matching the payload type.
+    /**Get a format iterator in the list matching the payload type, etc.
+       If the \p clockRate is 0 then it takes no part in the search.
 
+       The \p rtpEncodingName, if not NULL or empty string, is searched for
+       first and only if not found is the payload type used. This is because
+       it is possible (though discouraged) for someone to override a standard
+       payload type, e.g. GSM, with another encoding name.
+
+       If the \p protocol is NULL or empty string, then it takes no part in the
+       search.
+
+       The search begins at the \t start iterator, if this is end() or simply
+       const_iterator() then the search is started at the begining.
 
        Returns end() if not in list.
       */
     const_iterator FindFormat(
       RTP_DataFrame::PayloadTypes rtpPayloadType, ///<  RTP payload type code
-      const unsigned clockRate,                   ///<  clock rate
+      const unsigned clockRate = 0,               ///<  clock rate
       const char * rtpEncodingName = NULL,        ///<  RTP payload type name
       const char * protocol = NULL,               ///<  protocol to be valid for (if NULL, then all)
       const_iterator start = const_iterator()     ///<  location to start search
@@ -169,27 +229,16 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
       const PString & wildcard    ///<  Wildcard string name.
     ) const { return FindFormat(wildcard) != end(); }
 
-    /**Remove all the formats specified.
-       Each string in the array is checked using the wildcard matching algorithm in
-       FindFormat().
-
-       The '*' character indicates substrings, for example: "G.711*" would
-       match "G.711-uLaw-64k" and "G.711-ALaw-64k".
-
-       The '@' character indicates a type of media format, so say "\@video"
-       would find the first video codec.
-
-       The '!' character indicates a negative test. That is the first entry
-       that does NOT match the string is returned. The string after the '!'
-       may contain '*' and '@' characters.
-
-       I should be noted that when the ! operator is used, codecs are removed
-       at that time and later ! operators may not have an affect. For example
-       {"!G.729","!G.728"} will NOT give you two codecs but on G.729.
+    /**Determine if a list contains a specific media type
       */
-    void Remove(
-      const PStringArray & mask
-    );
+    bool HasType(
+      const OpalMediaType & type,
+      bool mustBeTransportable = true
+    ) const;
+
+    /**Get list of all media types in format list.
+      */
+    OpalMediaTypeList GetMediaTypes() const;
 
     /**Reorder the formats in the list.
        The order variable is an array of names and the list is reordered
@@ -210,19 +259,36 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
       const PStringArray & order
     );
 
-    /**Determine if a list contains a specific media type
+    /**Remove all the formats specified.
+       Each string in the array is checked using the wildcard matching algorithm in
+       FindFormat().
+
+       The '*' character indicates substrings, for example: "G.711*" would
+       remove "G.711-uLaw-64k" and "G.711-ALaw-64k".
+
+       The '@' character indicates a type of media format, so say "\@video"
+       would remove all video codecs.
+
+       The '!' character indicates a negative test. That is the entres that do
+       NOT match the string are removed. The string after the '!' may contain
+       '*' and '@' characters.
+
+       I should be noted that when the ! operator is used, they are combined
+       differently to the usual application of each entry in turn. Thus, the
+       array {"!A","!B"} will result in keeping <i>both</i> A and B formats.
+     */
+    void Remove(
+      const PStringArray & mask
+    );
+
+    /**Remove all the non-transportable formats specified.
       */
-    bool HasType(
-      const OpalMediaType & type,
-      bool mustBeTransportable = true
-    ) const;
+    void RemoveNonTransportable();
   //@}
 
   private:
     virtual PINDEX Append(PObject *) { return P_MAX_INDEX; }
     virtual PINDEX Insert(const PObject &, PObject *) { return P_MAX_INDEX; }
-    virtual PINDEX InsertAt(PINDEX, PObject *) { return P_MAX_INDEX; }
-    virtual PBoolean SetAt(PINDEX, PObject *) { return false; }
 };
 
 
@@ -295,29 +361,51 @@ class OpalMediaOption : public PObject
 
     const PString & GetFMTPDefault() const { return m_FMTPDefault; }
     void SetFMTPDefault(const char * value) { m_FMTPDefault = value; }
+
+    void SetFMTP(const char * name, const char * dflt)
+    {
+      m_FMTPName = name;
+      m_FMTPDefault = dflt;
+    }
+#define OPAL_SET_MEDIA_OPTION_FMTP(opt, name, dflt) (opt)->SetFMTP(name, dflt)
+#else
+#define OPAL_SET_MEDIA_OPTION_FMTP(opt, name, dflt)
 #endif // OPAL_SIP
 
 #if OPAL_H323
     struct H245GenericInfo {
-      H245GenericInfo() { memset(this, 0, sizeof(*this)); }
-      unsigned ordinal:16;
+      H245GenericInfo();
+      H245GenericInfo(
+        unsigned mask,
+        const char * dflt = NULL
+      );
+
+      unsigned ordinal;
+
       enum Modes {
         None,
         Collapsing,
         NonCollapsing
-      } mode:3;
+      } mode;
+
       enum IntegerTypes {
         UnsignedInt,
         Unsigned32,
         BooleanArray
-      } integerType:3;
-      bool excludeTCS:1;
-      bool excludeOLC:1;
-      bool excludeReqMode:1;
+      } integerType;
+
+      bool    excludeTCS;
+      bool    excludeOLC;
+      bool    excludeReqMode;
+      int     position;     // Position in sequence for parameter
+      PString defaultValue; // Do not include parameter if this value
     };
 
     const H245GenericInfo & GetH245Generic() const { return m_H245Generic; }
     void SetH245Generic(const H245GenericInfo & genericInfo) { m_H245Generic = genericInfo; }
+#define OPAL_SET_MEDIA_OPTION_H245(opt, ...) (opt)->SetH245Generic(OpalMediaOption::H245GenericInfo(__VA_ARGS__))
+#else
+#define OPAL_SET_MEDIA_OPTION_H245(opt, ...)
 #endif // OPAL_H323
 
   protected:
@@ -344,15 +432,13 @@ class OpalMediaOptionValue : public OpalMediaOption
     OpalMediaOptionValue(
       const char * name,
       bool readOnly,
-      MergeType merge = MinMerge,
-      T value = 0,
-      T minimum = std::numeric_limits<T>::min(),
-      T maximum = std::numeric_limits<T>::max()
-    ) : OpalMediaOption(name, readOnly, merge),
-        m_value(value),
-        m_minimum(minimum),
-        m_maximum(maximum)
-    { }
+      OpalMediaOption::MergeType merge = OpalMediaOption::MinMerge,
+      T value = T()
+    )
+      : OpalMediaOption(name, readOnly, merge)
+      , m_value(value)
+    {
+    }
 
     virtual PObject * Clone() const
     {
@@ -366,27 +452,19 @@ class OpalMediaOptionValue : public OpalMediaOption
 
     virtual void ReadFrom(istream & strm)
     {
-      T temp = 0;
-      strm >> temp;
-      if (temp >= m_minimum && temp <= m_maximum)
-        m_value = temp;
-      else
-        strm.setstate(ios::badbit);
+      strm >> m_value;
     }
 
-    static __inline bool     Intersect(bool     a, bool     b) { return a && b; }
-    static __inline int      Intersect(int      a, int      b) { return a & b; }
-    static __inline unsigned Intersect(unsigned a, unsigned b) { return a & b; }
-    static __inline double   Intersect(double   a, double   b) { return std::min(a, b); }
-
     virtual bool Merge(const OpalMediaOption & option)
     {
       if (m_merge != IntersectionMerge)
         return OpalMediaOption::Merge(option);
+
       const OpalMediaOptionValue * otherOption = PDownCast(const OpalMediaOptionValue, &option);
       if (otherOption == NULL)
         return false;
-      SetValue(Intersect(GetValue(), otherOption->GetValue()));
+
+      m_value &= otherOption->m_value;
       return true;
     }
 
@@ -402,9 +480,8 @@ class OpalMediaOptionValue : public OpalMediaOption
       return EqualTo;
     }
 
-    virtual void Assign(
-      const OpalMediaOption & option
-    ) {
+    virtual void Assign(const OpalMediaOption & option)
+    {
       const OpalMediaOptionValue * otherOption = PDownCast(const OpalMediaOptionValue, &option);
       if (otherOption != NULL)
         m_value = otherOption->m_value;
@@ -417,35 +494,94 @@ class OpalMediaOptionValue : public OpalMediaOption
 
     void SetValue(T value)
     {
-      if (value < m_minimum)
-        m_value = m_minimum;
-      else if (value > m_maximum)
-        m_value = m_maximum;
-      else
-        m_value = value;
-    }
-
-    void SetMinimum(T m)
-    {
-      m_minimum = m;
-    }
-
-    void SetMaximum(T m)
-    {
-      m_maximum = m;
+      m_value = value;
     }
 
   protected:
     T m_value;
+};
+
+
+template <typename T>
+class OpalMediaOptionNumericalValue : public OpalMediaOptionValue<T>
+{
+    typedef OpalMediaOptionValue<T> BaseClass;
+    PCLASSINFO(OpalMediaOptionNumericalValue, BaseClass);
+  public:
+    OpalMediaOptionNumericalValue(
+      const char * name,
+      bool readOnly,
+      OpalMediaOption::MergeType merge = OpalMediaOption::MinMerge,
+      T value = 0,
+      T minimum = std::numeric_limits<T>::min(),
+      T maximum = std::numeric_limits<T>::max()
+    )
+      : BaseClass(name, readOnly, merge, value)
+      , m_minimum(minimum)
+      , m_maximum(maximum)
+    {
+    }
+
+    virtual PObject * Clone() const
+    {
+      return new OpalMediaOptionNumericalValue(*this);
+    }
+
+    virtual void ReadFrom(istream & strm)
+    {
+      T temp = 0;
+      strm >> temp;
+      if (strm.fail())
+        return;
+      if (temp >= m_minimum && temp <= m_maximum)
+        this->m_value = temp;
+      else
+        strm.setstate(ios::badbit);
+    }
+
+    void SetValue(T value)
+    {
+      if (value < m_minimum)
+        this->m_value = m_minimum;
+      else if (value > m_maximum)
+        this->m_value = m_maximum;
+      else
+        this->m_value = value;
+    }
+
+    void SetMinimum(T m)
+    {
+      this->m_minimum = m;
+    }
+
+    void SetMaximum(T m)
+    {
+      this->m_maximum = m;
+    }
+
+  protected:
     T m_minimum;
     T m_maximum;
 };
 
 
-typedef OpalMediaOptionValue<bool>     OpalMediaOptionBoolean;
-typedef OpalMediaOptionValue<int>      OpalMediaOptionInteger;
-typedef OpalMediaOptionValue<unsigned> OpalMediaOptionUnsigned;
-typedef OpalMediaOptionValue<double>   OpalMediaOptionReal;
+typedef OpalMediaOptionNumericalValue<bool>     OpalMediaOptionBoolean;
+typedef OpalMediaOptionNumericalValue<int>      OpalMediaOptionInteger;
+typedef OpalMediaOptionNumericalValue<unsigned> OpalMediaOptionUnsigned;
+
+// Wrapper class so we can implement intersection (&= operator) for floating point
+class OpalMediaOptionRealValue
+{
+    double m_value;
+  public:
+    OpalMediaOptionRealValue(double value = 0) : m_value(value) { }
+    operator double() const { return m_value; }
+    void operator&=(double other) { if (m_value > other) m_value = other; }
+    friend ostream & operator<<(ostream & strm, const OpalMediaOptionRealValue & value) { return strm << value.m_value; }
+    friend istream & operator>>(istream & strm,       OpalMediaOptionRealValue & value) { return strm >> value.m_value; }
+};
+
+typedef OpalMediaOptionNumericalValue<OpalMediaOptionRealValue> OpalMediaOptionReal;
 
 
 class OpalMediaOptionEnum : public OpalMediaOption
@@ -569,16 +705,16 @@ class OpalMediaFormatInternal : public PObject
     PCLASSINFO(OpalMediaFormatInternal, PObject);
   public:
     OpalMediaFormatInternal(
-      const char * fullName,
+      const char  * fullName,
       const OpalMediaType & mediaType,
       RTP_DataFrame::PayloadTypes rtpPayloadType,
-      const char * encodingName,
-      PBoolean     needsJitter,
-      unsigned bandwidth,
-      PINDEX   frameSize,
-      unsigned frameTime,
-      unsigned clockRate,
-      time_t timeStamp
+      const char  * encodingName,
+      bool          needsJitter,
+      OpalBandwidth bandwidth,
+      PINDEX        frameSize,
+      unsigned      frameTime,
+      unsigned      clockRate,
+      time_t        timeStamp
     );
 
     const PCaselessString & GetName() const { return formatName; }
@@ -617,6 +753,11 @@ class OpalMediaFormatInternal : public PObject
     virtual bool IsValidForProtocol(const PString & protocol) const;
 
   protected:
+    bool AdjustByOptionMaps(
+      PTRACE_PARAM(const char * operation,)
+      bool (*adjuster)(PluginCodec_OptionMap & original, PluginCodec_OptionMap & changed)
+    );
+
     PCaselessString              formatName;
     RTP_DataFrame::PayloadTypes  rtpPayloadType;
     PString                      rtpEncodingName;
@@ -649,9 +790,9 @@ class OpalMediaFormat : public PContainer
 {
     PCLASSINFO(OpalMediaFormat, PContainer)
   public:
-    OpalMediaFormat(const OpalMediaFormat & c) : PContainer(c), m_info(c.m_info) { }
+    OpalMediaFormat(const OpalMediaFormat & c);
+    virtual ~OpalMediaFormat();
     OpalMediaFormat & operator=(const OpalMediaFormat & c)     { AssignContents(c); return *this; }
-    virtual ~OpalMediaFormat()                                 { Destruct(); }
     virtual PBoolean MakeUnique();
   protected:
     virtual void DestroyContents();
@@ -683,7 +824,7 @@ class OpalMediaFormat : public PContainer
       RTP_DataFrame::PayloadTypes rtpPayloadType, ///<  RTP payload type code
       const char * encodingName,                  ///<  RTP encoding name
       PBoolean     needsJitter,                   ///<  Indicate format requires a jitter buffer
-      unsigned bandwidth,                         ///<  Bandwidth in bits/second
+      OpalBandwidth bandwidth,                    ///<  Bandwidth in bits/second
       PINDEX   frameSize,                         ///<  Size of frame in bytes (if applicable)
       unsigned frameTime,                         ///<  Time for frame in RTP units (if applicable)
       unsigned clockRate,                         ///<  Clock rate for data (if applicable)
@@ -840,7 +981,7 @@ class OpalMediaFormat : public PContainer
 
     /**Get the RTP encoding name that is to be used for this media format.
       */
-    const char * GetEncodingName() const { PWaitAndSignal m(m_mutex); return m_info == NULL ? "" : m_info->rtpEncodingName.GetPointer(); }
+    const char * GetEncodingName() const { PWaitAndSignal m(m_mutex); return m_info == NULL ? "" : (const char *)m_info->rtpEncodingName; }
 
     /** Get the media type for this format
       */
@@ -849,26 +990,30 @@ class OpalMediaFormat : public PContainer
     /**Determine if the media format requires a jitter buffer. As a rule an
        audio codec needs a jitter buffer and all others do not.
       */
-    bool NeedsJitterBuffer() const { PWaitAndSignal m(m_mutex); return m_info != NULL && m_info->GetOptionBoolean(NeedsJitterOption(), false); }
+    bool NeedsJitterBuffer() const { return GetOptionBoolean(NeedsJitterOption()); }
     static const PString & NeedsJitterOption();
 
-    /**Get the average bandwidth used in bits/second.
+    /**Get the maximum bandwidth used in bits/second.
       */
-    unsigned GetBandwidth() const { PWaitAndSignal m(m_mutex); return m_info == NULL ? 0 : m_info->GetOptionInteger(MaxBitRateOption(), 0); }
+    OpalBandwidth GetMaxBandwidth() const { return GetOptionInteger(MaxBitRateOption()); }
     static const PString & MaxBitRateOption();
+
+    /**Get the used bandwidth used in bits/second.
+      */
+    OpalBandwidth GetUsedBandwidth() const { return GetOptionInteger(TargetBitRateOption(), GetOptionInteger(MaxBitRateOption())); }
     static const PString & TargetBitRateOption();
 
     /**Get the maximum frame size in bytes. If this returns zero then the
        media format has no intrinsic maximum frame size, eg a video format
        would return zero but G.723.1 would return 24.
       */
-    PINDEX GetFrameSize() const { PWaitAndSignal m(m_mutex); return m_info == NULL ? 0 : m_info->GetOptionInteger(MaxFrameSizeOption(), 0); }
+    PINDEX GetFrameSize() const { return GetOptionInteger(MaxFrameSizeOption()); }
     static const PString & MaxFrameSizeOption();
 
     /**Get the frame time in RTP timestamp units. If this returns zero then
        the media format is not real time and has no intrinsic timing eg T.120
       */
-    unsigned GetFrameTime() const { PWaitAndSignal m(m_mutex); return m_info == NULL ? 0 : m_info->GetOptionInteger(FrameTimeOption(), 0); }
+    unsigned GetFrameTime() const { return GetOptionInteger(FrameTimeOption()); }
     static const PString & FrameTimeOption();
 
     /**Get the number of RTP timestamp units per millisecond.
@@ -882,7 +1027,7 @@ class OpalMediaFormat : public PContainer
 
     /**Get the clock rate in Hz for this format.
       */
-    unsigned GetClockRate() const { PWaitAndSignal m(m_mutex); return m_info == NULL ? 0 : m_info->GetOptionInteger(ClockRateOption(), 1000); }
+    unsigned GetClockRate() const { return GetOptionInteger(ClockRateOption(), AudioClockRate); }
     static const PString & ClockRateOption();
 
     /**Get the name of the OpalMediaOption indicating the protocol the format is being used on.
@@ -890,6 +1035,10 @@ class OpalMediaFormat : public PContainer
     static const PString & ProtocolOption();
 
     /**Get the name of the OpalMediaOption indicating the maximum RTP payload size.
+       Note this is a read only parameter and not generally set by the user. It
+       is intended to get the OpalManager::GetMaxRtpPayloadSize() value to the
+       codec (especially plug ins) before encoding starts in case the codec
+       requires some form of initialisation based on this value.
       */
     static const PString & MaxTxPacketSizeOption();
 
@@ -988,23 +1137,31 @@ class OpalMediaFormat : public PContainer
     ) { PWaitAndSignal m(m_mutex); MakeUnique(); return m_info != NULL && m_info->SetOptionReal(name, value); }
 
     /**Get the option value of the specified name as an index into an
-       enumeration list. The default value is returned if the option is not
-       present.
+       enumeration list.
+       If the merge mode is IntersectionMerge, then the enum is assume to be a
+       PBitwiseEnum and the multiple names separated by spaces can be used.
+
+       The default value is returned if the option is not present.
       */
-    PINDEX GetOptionEnum(
+    template <typename Enum>
+    Enum GetOptionEnum(
       const PString & name,   ///<  Option name
-      PINDEX dflt = 0         ///<  Default value if option not present
-    ) const { PWaitAndSignal m(m_mutex); return m_info == NULL ? dflt : m_info->GetOptionEnum(name, dflt); }
+      Enum dflt = (Enum)0     ///<  Default value if option not present
+    ) const { PWaitAndSignal m(m_mutex); return m_info == NULL ? dflt : (Enum)m_info->GetOptionEnum(name, dflt); }
 
     /**Set the option value of the specified name as an index into an enumeration.
+       If the merge mode is IntersectionMerge, then the enum is assume to be a
+       PBitwiseEnum and the multiple names separated by spaces can be used.
+
        Note the option will not be added if it does not exist, the option
        must be explicitly added using AddOption().
 
        Returns false of the option is not present or is not of the same type.
       */
+    template <typename Enum>
     bool SetOptionEnum(
       const PString & name,   ///<  Option name
-      PINDEX value            ///<  New value for option
+      Enum value              ///<  New value for option
     ) { PWaitAndSignal m(m_mutex); MakeUnique(); return m_info != NULL && m_info->SetOptionEnum(name, value); }
 
     /**Get the option value of the specified name as a string. The default
@@ -1064,6 +1221,13 @@ class OpalMediaFormat : public PContainer
       const OpalMediaFormat & mediaFormat  ///<  Media format to copy to master list
     );
 
+    /**Remove the media format from master format list entry.
+       The media format must already be registered. Returns false if not.
+      */
+    static bool RemoveRegisteredMediaFormat(
+      const OpalMediaFormat & mediaFormat  ///<  Media format to copy to master list
+    );
+
     /**
       * Add a new option to this media format
       */
@@ -1077,14 +1241,20 @@ class OpalMediaFormat : public PContainer
       */
     bool HasOption(const PString & name) const { PWaitAndSignal m(m_mutex); return m_info != NULL && m_info->FindOption(name) != NULL; }
 
-    /**
-      * Get a pointer to the specified media format option.
-      * Returns NULL if thee option does not exist.
+    /** Get a pointer to the specified media format option.
+        Returns NULL if thee option does not exist.
       */
     OpalMediaOption * FindOption(
       const PString & name
     ) const { PWaitAndSignal m(m_mutex); return m_info == NULL ? NULL : m_info->FindOption(name); }
 
+    /** Get a pointer to the specified media format option.
+        Returns NULL if thee option does not exist.
+      */
+    template <class T> T * FindOptionAs(
+      const PString & name
+    ) const { return dynamic_cast<T *>(FindOption(name)); }
+
     /** Returns true if the media format is valid for the protocol specified
         This allow plugin codecs to customise which protocols they are valid for
         The default implementation returns true unless the protocol is H.323
@@ -1128,6 +1298,9 @@ class OpalMediaFormat : public PContainer
 #if OPAL_H323
     static const PString & MediaPacketizationOption();
     static const PString & MediaPacketizationsOption();
+    PStringArray GetMediaPacketizations() const;
+    PStringSet GetMediaPacketizationSet() const { return PStringSet(GetMediaPacketizations()); }
+    void SetMediaPacketizations(const PStringSet & packetizations);
 #endif
 
   private:
@@ -1157,7 +1330,7 @@ class OpalAudioFormatInternal : public OpalMediaFormatInternal
       unsigned txFrames,
       unsigned maxFrames,
       unsigned clockRate,
-      time_t timeStamp
+      time_t timeStamp = 0
     );
     virtual PObject * Clone() const;
     virtual bool Merge(const OpalMediaFormatInternal & mediaFormat);
@@ -1201,7 +1374,7 @@ class OpalVideoFormatInternal : public OpalMediaFormatInternal
       unsigned maxFrameHeight,
       unsigned maxFrameRate,
       unsigned maxBitRate,
-      time_t timeStamp
+      time_t timeStamp = 0
     );
     virtual PObject * Clone() const;
     virtual bool Merge(const OpalMediaFormatInternal & mediaFormat);
@@ -1234,9 +1407,8 @@ class OpalVideoFormat : public OpalMediaFormat
     static const PString & MaxRxFrameHeightOption();
     static const PString & TemporalSpatialTradeOffOption();
     static const PString & TxKeyFramePeriodOption();
-    static const PString & RateControlEnableOption();
-    static const PString & RateControllerOption();
-    static const PString & RateControllerBitRateScalerOption();
+    static const PString & RateControlPeriodOption(); // Period over which the rate controller maintains the target bit rate.
+    static const PString & RateControllerOption(); // String for controller algorithm. Empty is none.
 
     /**The "role" of the content in the video stream based on this media
        format. This is based on RFC4796 and H.239 semantics and is an
@@ -1248,69 +1420,41 @@ class OpalVideoFormat : public OpalMediaFormat
           Speaker        Live (0x02)          speaker
           SignLanguage   Live (0x02)          sl
       */
-    enum ContentRole {
+    P_DECLARE_STREAMABLE_ENUM(ContentRole,
       eNoRole,
       ePresentation,
       eMainRole,
       eSpeaker,
-      eSignLanguage,
-      eNumRoles
-    };
+      eSignLanguage
+    );
+
     enum { ContentRoleMask = 15 };
     __inline static unsigned ContentRoleBit(ContentRole contentRole) { return contentRole != eNoRole ? (1<<(contentRole-1)) : 0; }
     static const PString & ContentRoleOption();
     static const PString & ContentRoleMaskOption();
+
+    /// RTP/RTCP Feedback options
+    P_DECLARE_STREAMABLE_BITWISE_ENUM_EX(
+      RTCPFeedback,
+      5,
+      (
+        e_NoRTCPFb,
+        e_PLI,      ///< Picture Loss Indication
+        e_FIR,      ///< Full Intra-frame Request
+        e_TMMBR,    ///< Temporary Maximum Media Stream Bit Rate Request
+        e_TSTR,     ///< Temporal/Spatial Tradeoff Request
+        e_VBCM      ///< Video Back Channel Messages
+      ),
+      "", "pli", "fir", "tmmbr", "tstr", "vcbm"
+    );
+    /// RTP/RTCP Feedback options
+    static const PString & RTCPFeedbackOption();
 };
 #endif
 
-// List of known media formats
 
-#define OPAL_PCM16          "PCM-16"
-#define OPAL_PCM16S         "PCM-16S"
-#define OPAL_PCM16_16KHZ    "PCM-16-16kHz"
-#define OPAL_PCM16S_16KHZ   "PCM-16S-16kHz"
-#define OPAL_PCM16_32KHZ    "PCM-16-32kHz"
-#define OPAL_PCM16S_32KHZ   "PCM-16S-32kHz"
-#define OPAL_PCM16_48KHZ    "PCM-16-48kHz"
-#define OPAL_PCM16S_48KHZ   "PCM-16S-48kHz"
-#define OPAL_L16_MONO_8KHZ  "Linear-16-Mono-8kHz"
-#define OPAL_L16_STEREO_8KHZ "Linear-16-Stereo-8kHz"
-#define OPAL_L16_MONO_16KHZ "Linear-16-Mono-16kHz"
-#define OPAL_L16_STEREO_16KHZ "Linear-16-Stereo-16kHz"
-#define OPAL_L16_MONO_32KHZ "Linear-16-Mono-32kHz"
-#define OPAL_L16_STEREO_32KHZ "Linear-16-Stereo-32kHz"
-#define OPAL_L16_MONO_48KHZ "Linear-16-Mono-48kHz"
-#define OPAL_L16_STEREO_48KHZ "Linear-16-Stereo-48kHz"
-#define OPAL_G711_ULAW_64K  "G.711-uLaw-64k"
-#define OPAL_G711_ALAW_64K  "G.711-ALaw-64k"
-#define OPAL_G722           "G.722"
-#define OPAL_G7221          "G.722.1"
-#define OPAL_G7222          "G.722.2"
-#define OPAL_G726_40K       "G.726-40K"
-#define OPAL_G726_32K       "G.726-32K"
-#define OPAL_G726_24K       "G.726-24K"
-#define OPAL_G726_16K       "G.726-16K"
-#define OPAL_G728           "G.728"
-#define OPAL_G729           "G.729"
-#define OPAL_G729A          "G.729A"
-#define OPAL_G729B          "G.729B"
-#define OPAL_G729AB         "G.729A/B"
-#define OPAL_G7231          "G.723.1"
-#define OPAL_G7231_6k3      OPAL_G7231
-#define OPAL_G7231_5k3      "G.723.1(5.3k)"
-#define OPAL_G7231A_6k3     "G.723.1A(6.3k)"
-#define OPAL_G7231A_5k3     "G.723.1A(5.3k)"
-#define OPAL_GSM0610        "GSM-06.10"
-#define OPAL_GSMAMR         "GSM-AMR"
-#define OPAL_iLBC           "iLBC"
-#define OPAL_H261           "H.261"
-#define OPAL_H263           "H.263"
-#define OPAL_H264_MODE0     "H.264-0"
-#define OPAL_H264_MODE1     "H.264-1"
-#define OPAL_MPEG4          "MPEG4"
-#define OPAL_RFC2833        "UserInput/RFC2833"
-#define OPAL_CISCONSE       "NamedSignalEvent"
-#define OPAL_T38            "T.38"
+#include <codec/known.h>
+
 
 extern const OpalAudioFormat & GetOpalPCM16();
 extern const OpalAudioFormat & GetOpalPCM16S();
@@ -1331,7 +1475,9 @@ extern const OpalAudioFormat & GetOpalL16_STEREO_48KHZ();
 extern const OpalAudioFormat & GetOpalG711_ULAW_64K();
 extern const OpalAudioFormat & GetOpalG711_ALAW_64K();
 extern const OpalAudioFormat & GetOpalG722();
-extern const OpalAudioFormat & GetOpalG7221();
+extern const OpalAudioFormat & GetOpalG7221_24K();
+extern const OpalAudioFormat & GetOpalG7221_32K();
+extern const OpalAudioFormat & GetOpalG7221_48K();
 extern const OpalAudioFormat & GetOpalG7222();
 extern const OpalAudioFormat & GetOpalG726_40K();
 extern const OpalAudioFormat & GetOpalG726_32K();
@@ -1350,6 +1496,15 @@ extern const OpalAudioFormat & GetOpalGSM0610();
 extern const OpalAudioFormat & GetOpalGSMAMR();
 extern const OpalAudioFormat & GetOpaliLBC();
 
+#if OPAL_VIDEO
+extern const OpalVideoFormat & GetOpalH261();
+extern const OpalVideoFormat & GetOpalH263();
+extern const OpalVideoFormat & GetOpalH263plus();
+extern const OpalVideoFormat & GetOpalH264_MODE0();
+extern const OpalVideoFormat & GetOpalH264_MODE1();
+extern const OpalVideoFormat & GetOpalMPEG4();
+#endif // OPAL_VIDEO
+
 extern const OpalMediaFormat & GetOpalRFC2833();
 
 #if OPAL_T38_CAPABILITY
@@ -1377,7 +1532,10 @@ extern const OpalMediaFormat & GetOpalT38();
 #define OpalG711_ULAW_64K  GetOpalG711_ULAW_64K()
 #define OpalG711_ALAW_64K  GetOpalG711_ALAW_64K()
 #define OpalG722           GetOpalG722()
-#define OpalG7221          GetOpalG7221()
+#define OpalG7221          OpalG7221_32K
+#define OpalG7221_24K      GetOpalG7221_24K()
+#define OpalG7221_32K      GetOpalG7221_32K()
+#define OpalG7221_48K      GetOpalG7221_48K()
 #define OpalG7222          GetOpalG7222()
 #define OpalG726_40K       GetOpalG726_40K()
 #define OpalG726_32K       GetOpalG726_32K()
@@ -1399,32 +1557,21 @@ extern const OpalMediaFormat & GetOpalT38();
 #define OpalCiscoNSE       GetOpalCiscoNSE()
 #define OpalT38            GetOpalT38()
 
+#if OPAL_VIDEO
+#define OpalH261       GetOpalH261()
+#define OpalH263       GetOpalH263()
+#define OpalH263plus   GetOpalH263plus()
+#define OpalH264       GetOpalH264_MODE1()
+#define OpalH264_MODE0 GetOpalH264_MODE0()
+#define OpalH264_MODE1 GetOpalH264_MODE1()
+#define OpalMPEG4      GetOpalMPEG4()
+#endif
+
 #define OpalL16Mono8kHz    OpalL16_MONO_8KHZ
 #define OpalL16Mono16kHz   OpalL16_MONO_16KHZ
 #define OpalG711uLaw       OpalG711_ULAW_64K
 #define OpalG711ALaw       OpalG711_ALAW_64K
 
-#define OPAL_T140             "T.140"
-#define OpalT140              GetOpalT140()
-extern const OpalMediaFormat & GetOpalT140();
-
-#if OPAL_HAS_MSRP
-#define OPAL_MSRP           "MSRP"
-#define OpalMSRP            GetOpalMSRP()
-extern const OpalMediaFormat & GetOpalMSRP();
-#endif
-
-#if OPAL_HAS_SIPIM
-#define OPAL_SIPIM             "SIP-IM"
-#define OpalSIPIM              GetOpalSIPIM()
-extern const OpalMediaFormat & GetOpalSIPIM();
-#endif
-
-#ifdef _MSC_VER
-#if _MSC_VER < 1300
-#pragma warning(default:4663)
-#endif
-#endif
 
 #endif  // OPAL_OPAL_MEDIAFMT_H
 
