@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24362 $
+ * $Revision: 29330 $
  * $Author: rjongbloed $
- * $Date: 2010-05-20 03:24:51 -0500 (Thu, 20 May 2010) $
+ * $Date: 2013-03-26 19:55:21 -0500 (Tue, 26 Mar 2013) $
  */
 
 #ifndef OPAL_OPAL_MEDIASTRM_H
@@ -42,11 +42,12 @@
 
 #include <opal/mediafmt.h>
 #include <opal/mediacmd.h>
+#include <rtp/jitter.h>
 #include <ptlib/safecoll.h>
 #include <ptclib/guid.h>
 
 
-class RTP_Session;
+class OpalRTPSession;
 class OpalMediaPatch;
 class OpalLine;
 class OpalConnection;
@@ -54,6 +55,59 @@ class OpalRTPConnection;
 class OpalMediaStatistics;
 
 
+typedef PSafePtr<OpalMediaPatch, PSafePtrMultiThreaded> OpalMediaPatchPtr;
+
+
+/*!< \page pageOpalMediaStream Media streams in the OPAL library
+
+\section secOverviewM Overview
+This page of the documentation is designed to provide an understanding of the
+use of the OpalMediaStream class in the OPAL library. 
+
+A voip call that is handled in OPAL consists of two instances of the
+OpalConnection class (or derivative) plus instances of the
+OpalMediaStream class (or derivatives). Consequently, for handling a
+H.323 call which is directed to the PC speaker/microphone, there will
+H323Connection and a PCSSConnection. OPAL requires that media can be
+transferred between these two clases. To transfer the media, OPAL uses
+the OpalMediaStream.
+
+Key to understanding the OpalMediaStream class is that this class is
+responsible for transferring media from one instance of an
+OpalConnection to another instance of an OpalConnection.
+
+\section secDescriptionM Description
+
+From \ref pageOpalConnections, we see that derivatives of the
+OpalConnection class take many different forms (depending on the
+protocol). Further, the media from those protocols may well have
+different forms. For example, H.323 and PCSS have quite different
+format for the media. The class OpalMediaStream (and derivatives) is
+responsible for making the format of the media (from each OpalConnection
+instance) consistent.
+
+The standard output format of an OpalMediaStream (or derivative) is a
+RTP_DataFrame. Consequently, different OpalConnection instances (or
+derivatives) can be connected together and exchange media (where the
+media is encapsulated inside instances of the RTP_DataFrame class).
+
+When handling an audio only call, each OpalConnection instance (or
+derivative) has two attached OpalMediaStream instances (or
+derivatives). When the call contains video and audio, each
+OpalConnection instance (or derivative) will have four attached
+OpalMediaStream instances (or derivatives).
+
+For handling a H.323 call, which is an audio only call, that goes the speaker/microphone of the box, there will be 
+\li one instance of a H323Connection
+\li one instance of a OpalPCSSConnection
+\li two instances of a OpalAudioMediaStream
+\li two instances of a OpalRTPMediaStream
+
+Note that an OpalMediaStream instance is designed to take media in one
+direction.
+
+*/
+
 /**This class describes a media stream as used in the OPAL system. A media
    stream is the channel through which media data is trasferred between OPAL
    entities. For example, data being sent to an RTP session over a network
@@ -68,7 +122,7 @@ class OpalMediaStream : public PSafeObject
     /**Construct a new media stream.
       */
     OpalMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource                        ///<  Is a source stream
@@ -104,6 +158,16 @@ class OpalMediaStream : public PSafeObject
       */
     virtual OpalMediaFormat GetMediaFormat() const;
 
+    /**Set a new media format for the stream.
+       Unlike UpdateMediaFormat() this will shut down the patch and attempt to
+       create new transcoders to meet the requirement.
+
+       @returns false if the no transcoding can be found.
+      */
+    virtual bool SetMediaFormat(
+      const OpalMediaFormat & mediaFormat   ///<  New media format
+    );
+
     /**Update the media format. This can be used to adjust the
        parameters of a codec at run time. Note you cannot change the basic
        media format, eg change GSM0610 to G.711, only options for that
@@ -113,38 +177,32 @@ class OpalMediaStream : public PSafeObject
        The default behaviour updates the mediaFormat member variable and
        pases the value on to the OpalMediaPatch.
       */
-    virtual bool UpdateMediaFormat(
-      const OpalMediaFormat & mediaFormat,  ///<  New media format
-      bool fromPatch = false                ///<  Is being called from OpalMediaPatch
+    bool UpdateMediaFormat(
+      const OpalMediaFormat & mediaFormat   ///<  New media format
     );
 
     /**Execute the command specified to the transcoder. The commands are
-       highly context sensitive, for example VideoFastUpdate would only apply
-       to a video transcoder.
+       highly context sensitive, for example OpalVideoUpdatePicture would only
+       apply to a video transcoder.
 
        The default behaviour passes the command on to the OpalMediaPatch.
+
+       @returns true if command is handled.
       */
     virtual PBoolean ExecuteCommand(
       const OpalMediaCommand & command    ///<  Command to execute.
     );
 
-    /**Set a notifier to receive commands generated by the transcoder. The
-       commands are highly context sensitive, for example VideoFastUpdate
-       would only apply to a video transcoder.
-
-       The default behaviour passes the command on to the OpalMediaPatch and
-       sets the member variable commandNotifier.
-      */
-    virtual void SetCommandNotifier(
-      const PNotifier & notifier    ///<  Command to execute.
-    );
-
     /**Open the media stream using the media format.
 
        The default behaviour simply sets the isOpen variable to true.
       */
     virtual PBoolean Open();
 
+    /**Returns true if the media stream is open.
+      */
+    virtual bool IsOpen() const;
+
     /**Start the media stream.
 
        The default behaviour calls Resume() on the associated OpalMediaPatch
@@ -154,7 +212,8 @@ class OpalMediaStream : public PSafeObject
 
     /**Close the media stream.
 
-       The default does nothing.
+       The default marks the stream as closed and calls
+       OpalConnection::OnClosedMediaStream().
       */
     virtual PBoolean Close();
 
@@ -262,10 +321,12 @@ class OpalMediaStream : public PSafeObject
     virtual PBoolean RequiresPatchThread() const; // For backward compatibility
 
     /**Enable jitter buffer for the media stream.
+       Returns true if a jitter buffer is enabled/disabled. Returns false if
+       no jitter buffer exists for the media stream.
 
-       The default behaviour does nothing.
+       The default behaviour does nothing and returns false.
       */
-    virtual void EnableJitterBuffer() const;
+    virtual bool EnableJitterBuffer(bool enab = true) const;
   //@}
 
   /**@name Member variable access */
@@ -276,11 +337,11 @@ class OpalMediaStream : public PSafeObject
 
     /**Determine of media stream is a source or a sink.
       */
-    bool IsSource() const { return isSource; }
+    bool IsSource() const { return m_isSource; }
 
     /**Determine of media stream is a source or a sink.
       */
-    bool IsSink() const { return !isSource; }
+    bool IsSink() const { return !m_isSource; }
 
     /**Get the session number of the stream.
      */
@@ -313,18 +374,16 @@ class OpalMediaStream : public PSafeObject
 
     /**Get the paused state for stream.
       */
-    bool IsPaused() const { return paused; }
+    bool IsPaused() const { return m_paused; }
 
     /**Set the paused state for stream.
        This will stop reading/writing data from the stream.
+       Returns true if the pause state was changed
       */
-    virtual void SetPaused(
-      bool pause    ///< Indicate that the stream should be paused
+    virtual bool SetPaused(
+      bool pause,             ///< Indicate that the stream should be paused
+      bool fromPatch = false  ///<  Is being called from OpalMediaPatch
     );
-
-    /**Returns true if the media stream is open.
-      */
-    bool IsOpen() const { return isOpen; }
     
     /**Set the patch thread that is using this stream.
       */
@@ -332,57 +391,71 @@ class OpalMediaStream : public PSafeObject
       OpalMediaPatch * patch  ///<  Media patch thread
     );
 
-    /**Remove the patch thread that is using this stream.
-       This function is useful in case of streams which can be accessed by
-       multiple instances of OpalMediaPatch.
-
-       The default behaviour simply sets patchThread to NULL.
-    */
-    virtual void RemovePatch(OpalMediaPatch * patch);
-
     /**Get the patch thread that is using the stream.
       */
-    OpalMediaPatch * GetPatch() const { return mediaPatch; }
+    OpalMediaPatchPtr GetPatch() const { return m_mediaPatch; }
 
     /**Add a filter to the owning patch safely.
       */
-    void AddFilter(const PNotifier & Filter, const OpalMediaFormat & Stage =  OpalMediaFormat());
+    void AddFilter(
+      const PNotifier & filter,   ///< Filter notifier to be called.
+      const OpalMediaFormat & stage = OpalMediaFormat() ///< Stage in codec pipeline to call filter
+    ) const;
 
     /**Remove a filter from the owning patch safely.
       */
-    bool RemoveFilter(const PNotifier & Filter, const OpalMediaFormat & Stage);
+    bool RemoveFilter(
+      const PNotifier & filter,   ///< Filter notifier to be called.
+      const OpalMediaFormat & stage = OpalMediaFormat() ///< Stage in codec pipeline to call filter
+    ) const;
 
 #if OPAL_STATISTICS
     virtual void GetStatistics(OpalMediaStatistics & statistics, bool fromPatch = false) const;
 #endif
   //@}
 
+    virtual bool InternalUpdateMediaFormat(const OpalMediaFormat & mediaFormat);
+
   protected:
     void IncrementTimestamp(PINDEX size);
+    bool InternalWriteData(const BYTE * data, PINDEX length, PINDEX & written);
+    OpalMediaPatchPtr InternalSetPatchPart1(OpalMediaPatch * newPatch);
+    void InternalSetPatchPart2(const OpalMediaPatchPtr & oldPatch);
+    virtual bool InternalSetJitterBuffer(const OpalJitterBuffer::Init & init) const;
+
+    /**Close any internal components of the stream.
+       This should be used in preference to overriding the Close() function as
+       it is guaranteed to be called exactly once and avoids race conditions in
+       the shut down sequence of a media stream.
+      */
+    virtual void InternalClose() = 0;
 
     OpalConnection & connection;
     unsigned         sessionID;
     PString          identifier;
     OpalMediaFormat  mediaFormat;
-    bool             paused;
-    bool             isSource;
-    bool             isOpen;
+    bool             m_paused;
+    bool             m_isSource;
+    bool             m_isOpen;
     PINDEX           defaultDataSize;
     unsigned         timestamp;
     bool             marker;
-    unsigned         mismatchedPayloadTypes;
 
-    OpalMediaPatch * mediaPatch;
-    PNotifier        commandNotifier;
+    OpalMediaPatchPtr m_mediaPatch;
 
     RTP_DataFrame::PayloadTypes m_payloadType;
     unsigned                    m_frameTime;
     PINDEX                      m_frameSize;
 
+    typedef OpalMediaPatchPtr PatchPtr; // For backward compatibility
+
   private:
     P_REMOVE_VIRTUAL_VOID(OnPatchStart());
     P_REMOVE_VIRTUAL_VOID(OnPatchStop());
     P_REMOVE_VIRTUAL_VOID(OnStopMediaPatch());
+    P_REMOVE_VIRTUAL_VOID(RemovePatch(OpalMediaPatch *));
+
+  friend class OpalMediaPatch;
 };
 
 typedef PSafePtr<OpalMediaStream> OpalMediaStreamPtr;
@@ -404,8 +477,12 @@ class OpalMediaStreamPacing
       bool & marker     ///< RTP Marker
     );
 
+    bool UpdateMediaFormat(
+      const OpalMediaFormat & mediaFormat   ///<  New media format
+    );
+
   protected:
-    bool           m_isAudio;
+    bool           m_timeOnMarkers;
     unsigned       m_frameTime;
     PINDEX         m_frameSize;
     unsigned       m_timeUnits;
@@ -424,14 +501,14 @@ class OpalNullMediaStream : public OpalMediaStream, public OpalMediaStreamPacing
     /**Construct a new media stream for RTP sessions.
       */
     OpalNullMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
       bool isSynchronous = false           ///<  Can accept data and block accordingly
     );
     OpalNullMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
@@ -460,6 +537,15 @@ class OpalNullMediaStream : public OpalMediaStream, public OpalMediaStreamPacing
       PINDEX & written     ///<  Length of data actually written
     );
 	
+    /**Set the paused state for stream.
+       This will stop reading/writing data from the stream.
+       Returns true if the pause state was changed
+      */
+    virtual bool SetPaused(
+      bool pause,             ///< Indicate that the stream should be paused
+      bool fromPatch = false  ///<  Is being called from OpalMediaPatch
+    );
+
     /**Indicate if the media stream requires a OpalMediaPatch thread (active patch).
        The default behaviour returns the value of m_isSynchronous.
       */
@@ -472,6 +558,9 @@ class OpalNullMediaStream : public OpalMediaStream, public OpalMediaStreamPacing
   //@}
 
   protected:
+    virtual void InternalClose() { }
+    virtual bool InternalUpdateMediaFormat(const OpalMediaFormat & newMediaFormat);
+
     bool m_isSynchronous;
     bool m_requiresPatchThread;
 };
@@ -490,12 +579,10 @@ class OpalRTPMediaStream : public OpalMediaStream
        This will add a reference to the rtpSession passed in.
       */
     OpalRTPMediaStream(
-      OpalRTPConnection & conn,
+      OpalRTPConnection & conn,            ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       bool isSource,                       ///<  Is a source stream
-      RTP_Session & rtpSession,            ///<  RTP session to stream to/from
-      unsigned minAudioJitterDelay,        ///<  Minimum jitter buffer size (if applicable)
-      unsigned maxAudioJitterDelay         ///<  Maximum jitter buffer size (if applicable)
+      OpalRTPSession & rtpSession          ///<  RTP session to stream to/from
     );
 
     /**Destroy the media stream for RTP sessions.
@@ -512,28 +599,27 @@ class OpalRTPMediaStream : public OpalMediaStream
       */
     virtual PBoolean Open();
 
-    /**Close the media stream.
-
-       The default does nothing.
+    /**Returns true if the media stream is open.
       */
-    virtual PBoolean Close();
+    virtual bool IsOpen() const;
 
     /**Set the paused state for stream.
        This will stop reading/writing data from the stream.
       */
-    virtual void SetPaused(
-      bool pause    ///< Indicate that the stream should be paused
+    virtual bool SetPaused(
+      bool pause,             ///< Indicate that the stream should be paused
+      bool fromPatch = false  ///<  Is being called from OpalMediaPatch
     );
 
     /**Read an RTP frame of data from the source media stream.
-       The new behaviour simply calls RTP_Session::ReadData().
+       The new behaviour simply calls OpalRTPSession::ReadData().
       */
     virtual PBoolean ReadPacket(
       RTP_DataFrame & packet
     );
 
     /**Write an RTP frame of data to the sink media stream.
-       The new behaviour simply calls RTP_Session::WriteData().
+       The new behaviour simply calls OpalRTPSession::WriteData().
       */
     virtual PBoolean WritePacket(
       RTP_DataFrame & packet
@@ -560,12 +646,6 @@ class OpalRTPMediaStream : public OpalMediaStream
       */
     virtual PBoolean RequiresPatchThread() const;
 
-    /**Enable jitter buffer for the media stream.
-
-       The default behaviour does nothing.
-      */
-    virtual void EnableJitterBuffer() const;
-
     /**Set the patch thread that is using this stream.
       */
     virtual PBoolean SetPatch(
@@ -574,7 +654,7 @@ class OpalRTPMediaStream : public OpalMediaStream
 
     /** Return current RTP session
       */
-    virtual RTP_Session & GetRtpSession() const
+    virtual OpalRTPSession & GetRtpSession() const
     { return rtpSession; }
 
 #if OPAL_STATISTICS
@@ -583,9 +663,14 @@ class OpalRTPMediaStream : public OpalMediaStream
   //@}
 
   protected:
-    RTP_Session & rtpSession;
-    unsigned      minAudioJitterDelay;
-    unsigned      maxAudioJitterDelay;
+    virtual void InternalClose();
+    virtual bool InternalSetJitterBuffer(const OpalJitterBuffer::Init & init) const;
+
+    OpalRTPSession & rtpSession;
+#if OPAL_VIDEO
+    bool             m_forceIntraFrameFlag;
+    PSimpleTimer     m_forceIntraFrameTimer;
+#endif
 };
 
 
@@ -601,7 +686,7 @@ class OpalRawMediaStream : public OpalMediaStream
     /**Construct a new media stream for channel.
       */
     OpalRawMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
@@ -646,18 +731,14 @@ class OpalRawMediaStream : public OpalMediaStream
       bool autoDelete = true  ///< Auto delete channel on exit or replacement
     );
 
-    /**Close the media stream.
-
-       Closes the associated PChannel.
-      */
-    virtual PBoolean Close();
-
     /**Get average signal level in last frame.
       */
     virtual unsigned GetAverageSignalLevel();
   //@}
 
   protected:
+    virtual void InternalClose();
+
     PChannel * m_channel;
     bool       m_autoDelete;
     PMutex     m_channelMutex;
@@ -684,7 +765,7 @@ class OpalFileMediaStream : public OpalRawMediaStream, public OpalMediaStreamPac
     /**Construct a new media stream for files.
       */
     OpalFileMediaStream(
-      OpalConnection &,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
@@ -695,7 +776,7 @@ class OpalFileMediaStream : public OpalRawMediaStream, public OpalMediaStreamPac
     /**Construct a new media stream for files.
       */
     OpalFileMediaStream(
-      OpalConnection & ,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
@@ -709,7 +790,6 @@ class OpalFileMediaStream : public OpalRawMediaStream, public OpalMediaStreamPac
        Returns true for LID streams.
       */
     virtual PBoolean IsSynchronous() const;
-  //@}
 
     virtual PBoolean ReadData(
       BYTE * data,      ///<  Data buffer to read to
@@ -725,6 +805,7 @@ class OpalFileMediaStream : public OpalRawMediaStream, public OpalMediaStreamPac
       PINDEX length,       ///<  Length of data to read.
       PINDEX & written     ///<  Length of data actually written
     );
+  //@}
 
   protected:
     PFile file;
@@ -747,7 +828,7 @@ class OpalAudioMediaStream : public OpalRawMediaStream
     /**Construct a new media stream for audio.
       */
     OpalAudioMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
@@ -760,7 +841,7 @@ class OpalAudioMediaStream : public OpalRawMediaStream
     /**Construct a new media stream for audio.
       */
     OpalAudioMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
@@ -812,12 +893,13 @@ class OpalVideoMediaStream : public OpalMediaStream
     /**Construct a new media stream for channel.
       */
     OpalVideoMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       PVideoInputDevice * inputDevice,     ///<  Device to use for video grabbing
       PVideoOutputDevice * outputDevice,   ///<  Device to use for video display
-      bool autoDelete = true               ///<  Automatically delete PVideoDevices
+      bool autoDeleteInput = true,         ///<  Automatically delete PVideoInputDevice
+      bool autoDeleteOutput = true         ///<  Automatically delete PVideoOutputDevice
     );
 
     /**Delete attached channel if autoDelete enabled.
@@ -834,11 +916,16 @@ class OpalVideoMediaStream : public OpalMediaStream
       */
     virtual PBoolean Open();
 
-    /**Close the media stream.
+    /**Execute the command specified to the transcoder. The commands are
+       highly context sensitive, for example OpalVideoUpdatePicture would only
+       apply to a video transcoder.
 
-       The default does nothing.
+       The default behaviour checks for video fast update and then
+       passes the command on to the OpalMediaPatch.
       */
-    virtual PBoolean Close();
+    virtual PBoolean ExecuteCommand(
+      const OpalMediaCommand & command    ///<  Command to execute.
+    );
 
     /**Read raw media data from the source media stream.
        The default behaviour simply calls ReadPacket() on the data portion of the
@@ -874,25 +961,48 @@ class OpalVideoMediaStream : public OpalMediaStream
       PINDEX frameTime  ///< Individual frame time (if applicable)
     );
 
+    /** Set the input device
+      */
+    virtual void SetVideoInputDevice(
+      PVideoInputDevice * device,
+      bool autoDelete = true
+    );
+
     /** Get the input device (e.g. for statistics)
       */
-    virtual PVideoInputDevice * GetVideoInputDevice() const {
-      return inputDevice;
+    virtual PVideoInputDevice * GetVideoInputDevice() const
+    {
+      return m_inputDevice;
     }
 
+    /** Set the output device
+      */
+    virtual void SetVideoOutputDevice(
+      PVideoOutputDevice * device,
+      bool autoDelete = true
+    );
+
     /** Get the output device (e.g. for statistics)
       */
-    virtual PVideoOutputDevice * GetVideoOutputDevice() const {
-      return outputDevice;
+    virtual PVideoOutputDevice * GetVideoOutputDevice() const
+    {
+      return m_outputDevice;
     }
 
   //@}
 
   protected:
-    PVideoInputDevice  * inputDevice;
-    PVideoOutputDevice * outputDevice;
-    bool                 autoDelete;
-    PTimeInterval        lastGrabTime;
+    virtual void InternalClose();
+    virtual bool InternalUpdateMediaFormat(const OpalMediaFormat & newMediaFormat);
+    bool InternalAdjustDevices();
+
+    PVideoInputDevice  * m_inputDevice;
+    PVideoOutputDevice * m_outputDevice;
+    bool                 m_autoDeleteInput;
+    bool                 m_autoDeleteOutput;
+    PTimeInterval        m_lastGrabTime;
+    bool                 m_needKeyFrame;
+    PMutex               m_devicesMutex;
 };
 
 #endif // OPAL_VIDEO
@@ -910,7 +1020,7 @@ class OpalUDPMediaStream : public OpalMediaStream
     /**Construct a new media stream for channel.
       */
     OpalUDPMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
@@ -941,15 +1051,11 @@ class OpalUDPMediaStream : public OpalMediaStream
        Returns false.
       */
     virtual PBoolean IsSynchronous() const;
-
-    /**Close the media stream.
-       Closes the associated OpalTransportUDP.
-      */
-    virtual PBoolean Close();
-
   //@}
 
   private:
+    virtual void InternalClose();
+
     OpalTransportUDP & udpTransport;
 };
 
