@@ -23,33 +23,35 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24162 $
+ * $Revision: 28946 $
  * $Author: rjongbloed $
- * $Date: 2010-03-29 02:46:26 -0500 (Mon, 29 Mar 2010) $
+ * $Date: 2013-01-18 22:27:47 -0600 (Fri, 18 Jan 2013) $
  */
 
 #ifndef OPAL_OPAL_MEDIATYPE_H
 #define OPAL_OPAL_MEDIATYPE_H
 
-#include <ptbuildopts.h>
-#include <ptlib/pfactory.h>
 #include <opal/buildopts.h>
 
+#include <ptlib/pfactory.h>
+#include <ptlib/bitwise_enum.h>
+
+
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
 
 class OpalMediaTypeDefinition;
-class OpalSecurityMode;
 class OpalConnection;
 
 ////////////////////////////////////////////////////////////////////////////
 //
 //  define the factory used for keeping track of OpalMediaTypeDefintions
 //
-typedef PFactory<OpalMediaTypeDefinition> OpalMediaTypeFactory;
-typedef OpalMediaTypeFactory::KeyList_T OpalMediaTypeList;
+class OpalMediaType;
+typedef PFactory<OpalMediaTypeDefinition, OpalMediaType> OpalMediaTypesFactory;
+typedef OpalMediaTypesFactory::KeyList_T OpalMediaTypeList;
 
 
 /** Define the type used to hold the media type identifiers, i.e. "audio", "video", "h.224", "fax" etc
@@ -73,35 +75,28 @@ class OpalMediaType : public std::string     // do not make this PCaselessString
       : std::string((const char *)str) { }
 
     static const OpalMediaType & Audio();
+#if OPAL_VIDEO
     static const OpalMediaType & Video();
+#endif
+#if OPAL_T38_CAPABILITY
     static const OpalMediaType & Fax();
+#endif
     static const OpalMediaType & UserInput();
 
+    OpalMediaTypeDefinition * operator->() const { return GetDefinition(); }
     OpalMediaTypeDefinition * GetDefinition() const;
     static OpalMediaTypeDefinition * GetDefinition(const OpalMediaType & key);
     static OpalMediaTypeDefinition * GetDefinition(unsigned sessionId);
 
-    static OpalMediaTypeFactory::KeyList_T GetList() { return OpalMediaTypeFactory::GetKeyList(); }
+    /** Get a list of all media types.
+        This also assures that Audio() and Video() are the first two elements.
+      */
+    static OpalMediaTypeList GetList();
 
-#if OPAL_SIP
-    static OpalMediaType GetMediaTypeFromSDP(const std::string & key, const std::string & transport);
-#endif  // OPAL_SIP
-
-    enum AutoStartMode {
-      // Do not change order of enum as useful for bitmasking rx/tx
-      OfferInactive,
-      Receive,
-      Transmit,
-      ReceiveTransmit,
-      DontOffer,
-
-      TransmitReceive = ReceiveTransmit
-    };
-
-    __inline friend AutoStartMode operator++(AutoStartMode & mode)                 { return (mode = (AutoStartMode)(mode+1)); }
-    __inline friend AutoStartMode operator--(AutoStartMode & mode)                 { return (mode = (AutoStartMode)(mode-1)); }
-    __inline friend AutoStartMode operator|=(AutoStartMode & m1, AutoStartMode m2) { return (m1 = (AutoStartMode)((m1 & ~DontOffer) | m2)); }
-    __inline friend AutoStartMode operator-=(AutoStartMode & m1, AutoStartMode m2) { return (m1 = (AutoStartMode)((int)m1 & ~((int)m2|DontOffer))); }
+    P_DECLARE_BITWISE_ENUM_EX(AutoStartMode, 3,
+                              (OfferInactive, Receive, Transmit, DontOffer),
+                              ReceiveTransmit = Receive|Transmit,
+                              TransmitReceive = Receive|Transmit);
 
     AutoStartMode GetAutoStart() const;
 };
@@ -117,16 +112,12 @@ __inline ostream & operator << (ostream & strm, const OpalMediaType & mediaType)
 //
 //  this class defines the functions needed to work with the media type, i.e. 
 //
-class OpalRTPConnection;
-class RTP_UDP;
 
-#if OPAL_SIP
 class SDPMediaDescription;
 class OpalTransportAddress;
-#endif
-
 class OpalMediaSession;
 
+
 /** This class defines the type used to define the attributes of a media type
  */
 class OpalMediaTypeDefinition
@@ -135,14 +126,14 @@ class OpalMediaTypeDefinition
     /// Create a new media type definition
     OpalMediaTypeDefinition(
       const char * mediaType,          ///< name of the media type (audio, video etc)
-      const char * sdpType,            ///< name of the SDP type 
+      const char * mediaSession,       ///< name of media session class (via factory)
       unsigned requiredSessionId = 0,  ///< Session ID to use, asserts if already in use
       OpalMediaType::AutoStartMode autoStart = OpalMediaType::DontOffer   ///< Default value for auto-start transmit & receive
     );
 
     // Needed to avoid gcc warning about classes with virtual functions and 
     //  without a virtual destructor
-    virtual ~OpalMediaTypeDefinition() { }
+    virtual ~OpalMediaTypeDefinition();
 
     /** Get flags for media type can auto-start on call initiation.
       */
@@ -153,64 +144,37 @@ class OpalMediaTypeDefinition
     void SetAutoStart(OpalMediaType::AutoStartMode v) { m_autoStart = v; }
     void SetAutoStart(OpalMediaType::AutoStartMode v, bool on) { if (on) m_autoStart |= v; else m_autoStart -= v; }
 
-    /** Indicate type uses RTP for transport.
-        If false, then it uses a generic OpaMediaSession
-      */
-    virtual bool UsesRTP() const { return true; }
-
-    /** Create a media session suitable for the media type.
-      */
-    virtual OpalMediaSession * CreateMediaSession(
-      OpalConnection & connection,  ///< Connection media session is being created for
-      unsigned         sessionID    ///< ID for the media session
-    ) const;
-
-    /** Get the string used for the RTP_FormatHandler PFactory which is used
-        to create the RTP handler for the this media type
-        possible values include "rtp/avp" and "udptl"
-
-        Only valid if UsesRTP return true
-      */
-    virtual PString GetRTPEncoding() const = 0;
-
-    /** Create an RTP session for this media format.
-        By default, this will create a RTP_UDP session with the correct initial format
-
-        Only valid if UsesRTP return true
-      */
-    virtual RTP_UDP * CreateRTPSession(
-      OpalRTPConnection & conn,
-      unsigned sessionID, 
-      bool remoteIsNAT
-    );
-
     /** Return the default session ID for this media type.
       */
     unsigned GetDefaultSessionId() const { return m_defaultSessionId; }
 
-  protected:
-    std::string m_mediaType;
-    unsigned    m_defaultSessionId;
-    OpalMediaType::AutoStartMode m_autoStart;
+    /** Return the default session type (factory name) for this media type.
+      */
+    const PString & GetMediaSessionType() const { return m_mediaSessionType; }
 
 #if OPAL_SIP
-  public:
-    //
-    //  return the SDP type for this media type
-    //
-    virtual std::string GetSDPType() const 
-    { return m_sdpType; }
+    ///  Determine of this media type is valid for SDP m= section
+    virtual bool MatchesSDP(
+      const PCaselessString & sdpMediaType,
+      const PCaselessString & sdpTransport,
+      const PStringArray & sdpLines,
+      PINDEX index
+    );
 
-    //
-    //  create an SDP media description entry for this media type
-    //
+    ///  create an SDP media description entry for this media type
     virtual SDPMediaDescription * CreateSDPMediaDescription(
       const OpalTransportAddress & localAddress
-    ) = 0;
+    ) const;
+#endif // OPAL_SIP
 
   protected:
-    std::string m_sdpType;
-#endif
+    OpalMediaType m_mediaType;
+    PString       m_mediaSessionType;
+    unsigned      m_defaultSessionId;
+    OpalMediaType::AutoStartMode m_autoStart;
+
+  private:
+    P_REMOVE_VIRTUAL(OpalMediaSession *, CreateMediaSession(OpalConnection &, unsigned), NULL);
 };
 
 
@@ -219,49 +183,29 @@ class OpalMediaTypeDefinition
 //  define a macro for declaring a new OpalMediaTypeDefinition factory
 //
 
-#define OPAL_INSTANTIATE_MEDIATYPE2(title, name, cls) \
-namespace OpalMediaTypeSpace { \
-  static PFactory<OpalMediaTypeDefinition>::Worker<cls> static_##title##_##cls(name, true); \
-}; \
+#define OPAL_INSTANTIATE_MEDIATYPE2(cls, name) \
+  PFACTORY_CREATE(OpalMediaTypesFactory, cls, name, true)
 
-#define OPAL_INSTANTIATE_MEDIATYPE(type, cls) \
-  OPAL_INSTANTIATE_MEDIATYPE2(type, #type, cls) \
+#define OPAL_INSTANTIATE_MEDIATYPE(cls) \
+  OPAL_INSTANTIATE_MEDIATYPE2(cls, cls::Name())
 
 
-#ifdef SOLARIS
-template <char * Type, char * sdp>
-#else
-template <char * Type, const char * sdp>
-#endif
+template <const char * Type, unsigned SessionId = 0>
 class SimpleMediaType : public OpalMediaTypeDefinition
 {
   public:
+    static const char * Name() { return Type; }
+
     SimpleMediaType()
-      : OpalMediaTypeDefinition(Type, sdp, 0)
+      : OpalMediaTypeDefinition(Name(), PString::Empty(), SessionId)
     { }
-
-    virtual ~SimpleMediaType()                     { }
-
-    virtual RTP_UDP * CreateRTPSession(OpalRTPConnection & ,unsigned , bool ) { return NULL; }
-
-    PString GetRTPEncoding() const { return PString::Empty(); } 
-
-#if OPAL_SIP
-  public:
-    virtual SDPMediaDescription * CreateSDPMediaDescription(const OpalTransportAddress & ) { return NULL; }
-#endif
 };
 
+#define OPAL_INSTANTIATE_SIMPLE_MEDIATYPE(cls, name, ...) \
+  namespace OpalMediaTypeSpace { extern const char cls[] = name; }; \
+  typedef SimpleMediaType<OpalMediaTypeSpace::cls, ##__VA_ARGS__> cls; \
+  OPAL_INSTANTIATE_MEDIATYPE(cls) \
 
-#define OPAL_INSTANTIATE_SIMPLE_MEDIATYPE(type, sdp) \
-namespace OpalMediaTypeSpace { \
-  char type##_type_string[] = #type; \
-  char type##_sdp_string[] = #sdp; \
-  typedef SimpleMediaType<type##_type_string, type##_sdp_string> type##_MediaType; \
-}; \
-OPAL_INSTANTIATE_MEDIATYPE(type, type##_MediaType) \
-
-#define OPAL_INSTANTIATE_SIMPLE_MEDIATYPE_NO_SDP(type) OPAL_INSTANTIATE_SIMPLE_MEDIATYPE(type, "") 
 
 ////////////////////////////////////////////////////////////////////////////
 //
@@ -272,23 +216,24 @@ class OpalRTPAVPMediaType : public OpalMediaTypeDefinition {
   public:
     OpalRTPAVPMediaType(
       const char * mediaType, 
-      const char * sdpType, 
       unsigned     requiredSessionId = 0,
       OpalMediaType::AutoStartMode autoStart = OpalMediaType::DontOffer
     );
 
-    virtual PString GetRTPEncoding() const;
-
-    OpalMediaSession * CreateMediaSession(OpalConnection & /*conn*/, unsigned /* sessionID*/) const;
+#if OPAL_SIP
+    virtual bool MatchesSDP(const PCaselessString &, const PCaselessString &, const PStringArray &, PINDEX);
+#endif
 };
 
 
 class OpalAudioMediaType : public OpalRTPAVPMediaType {
   public:
+    static const char * Name();
+
     OpalAudioMediaType();
 
 #if OPAL_SIP
-    SDPMediaDescription * CreateSDPMediaDescription(const OpalTransportAddress & localAddress);
+    SDPMediaDescription * CreateSDPMediaDescription(const OpalTransportAddress &) const;
 #endif
 };
 
@@ -297,10 +242,12 @@ class OpalAudioMediaType : public OpalRTPAVPMediaType {
 
 class OpalVideoMediaType : public OpalRTPAVPMediaType {
   public:
+    static const char * Name();
+
     OpalVideoMediaType();
 
 #if OPAL_SIP
-    SDPMediaDescription * CreateSDPMediaDescription(const OpalTransportAddress & localAddress);
+    SDPMediaDescription * CreateSDPMediaDescription(const OpalTransportAddress &) const;
 #endif
 };
 
@@ -312,20 +259,27 @@ class OpalVideoMediaType : public OpalRTPAVPMediaType {
 class OpalFaxMediaType : public OpalMediaTypeDefinition 
 {
   public:
+    static const char * Name();
+    static const PCaselessString & UDPTL();
+
     OpalFaxMediaType();
 
-    PString GetRTPEncoding(void) const;
-    RTP_UDP * CreateRTPSession(OpalRTPConnection & conn,
-                               unsigned sessionID, bool remoteIsNAT);
-
-    OpalMediaSession * CreateMediaSession(OpalConnection & conn, unsigned /* sessionID*/) const;
-
 #if OPAL_SIP
-    SDPMediaDescription * CreateSDPMediaDescription(const OpalTransportAddress & localAddress);
-#endif
-};
+    static const PCaselessString & GetSDPMediaType();
+    static const PString & GetSDPTransportType();
 
-PFACTORY_LOAD(T38PseudoRTP_Handler);
+    virtual bool MatchesSDP(
+      const PCaselessString & sdpMediaType,
+      const PCaselessString & sdpTransport,
+      const PStringArray & sdpLines,
+      PINDEX index
+    );
+
+    SDPMediaDescription * CreateSDPMediaDescription(
+      const OpalTransportAddress & localAddress
+    ) const;
+#endif // OPAL_SIP
+};
 
 #endif // OPAL_T38_CAPABILITY
 
