@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25502 $
+ * $Revision: 29768 $
  * $Author: rjongbloed $
- * $Date: 2011-04-03 23:31:58 -0500 (Sun, 03 Apr 2011) $
+ * $Date: 2013-05-23 02:27:37 -0500 (Thu, 23 May 2013) $
  */
 
 #ifndef OPAL_IM_PRES_ENT_H
@@ -34,11 +34,16 @@
 #include <ptlib.h>
 #include <opal/buildopts.h>
 
+#if OPAL_HAS_PRESENCE
+
 #include <ptlib/pfactory.h>
 #include <ptlib/safecoll.h>
 #include <ptclib/url.h>
 #include <ptclib/guid.h>
-#include <ptclib/vcard.h>
+
+#ifdef P_VCARD
+  #include <ptclib/vcard.h>
+#endif
 
 #include <im/im.h>
 
@@ -51,64 +56,57 @@ class OpalPresentityCommand;
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
-/**Presencu state information
+/**Presence state information.
+   While protocol independent, this is based on the basic model provided
+   by RFC 3863 and RFC 4480.
+
+   A presence change may be sent for the same presentity, but multiple
+   services. A service may be a physical device, e.g. laptop, desktop or
+   smart phone, or a particular piece of software. This is indicated by the
+   m_service field, which can be a completely arbitrary string. This field
+   is ignored and system generated when supplied in SetLocalPresence().
+
+   A particular service has a set of capabilities such as "audio", "Video",
+   "text", "type=text/plain" etc as returned in m_capabilities.
   */
-class OpalPresenceInfo
+class OpalPresenceInfo : public PObject
 {
+    PCLASSINFO_WITH_CLONE(OpalPresenceInfo, PObject);
   public:
-    /// Presence states.
-    enum State {
-      InternalError = -3,    // something bad happened
-      Forbidden     = -2,    // access to presence information was specifically forbidden
-      NoPresence    = -1,    // remove presence status - not the same as Unavailable or Away
+    /** Presence states. Basic states (from RFC 3863) plus system states not
+        included in the specification.
+      */
+    P_DECLARE_STREAMABLE_ENUM_EX(State,StateCount,
+      UnknownUser,-4,     ///< Presentity does not exist
+      InternalError,      ///< Something bad happened
+      Forbidden,          ///< Access to presence information was specifically forbidden
+      NoPresence,         ///< No presence status - not the same as Unavailable or Away
+      Unchanged,          ///< State has not changed from last time
+      Available,          ///< User has a presence and is available to be contacted
+      Unavailable         ///< User has a presence, but is cannot be contacted
+    );
 
-      // basic states (from RFC 3863)
-      Unchanged,
-      Available,
-      Unavailable,
+    State      m_state;        ///< New state for presentity
+    PStringSet m_activities;   ///< Activity (from RFC 4480)
+    PString    m_note;         ///< Additional information about state change
+    PTime      m_when;         ///< Time/date of state change
 
-      // extended states (from RFC 4480)
-      // if this is changed, also change the tables in sippres.cxx and handlers.cxx - look for RFC 4480
-      ExtendedBase    = 100,
-      UnknownExtended = ExtendedBase,
-      Appointment,
-      Away,
-      Breakfast,
-      Busy,
-      Dinner,
-      Holiday,
-      InTransit,
-      LookingForWork,
-      Lunch,
-      Meal,
-      Meeting,
-      OnThePhone,
-      Other,
-      Performance,
-      PermanentAbsence,
-      Playing,
-      Presentation,
-      Shopping,
-      Sleeping,
-      Spectator,
-      Steering,
-      Travel,
-      TV,
-      Vacation,
-      Working,
-      Worship
-    };
+    PURL       m_entity;       ///< The presentity whose state had changed, usually remote
+    PURL       m_target;       ///< The presentity that is being informed about the state change
+    PString    m_service;      ///< Device/system(s) for the presentity that is getting a state change.
+    PString    m_contact;      ///< Contact address for the service.
+    PStringSet m_capabilities; ///< Capabilities of this service, e.g. "audio".
 
-    State   m_state;    ///< New state for presentity
-    PString m_note;     ///< Additional information about state change
-    PURL    m_entity;   ///< The presentity whose state had changed
-    PURL    m_target;   ///< The presentity that is being informed about the state change
+    PString    m_infoType;     ///< MIME tyupe for m_infoData, e.g. application/pidf+xml
+    PString    m_infoData;     ///< Raw information as provided by underlying protocol, e.g. XML.
 
-    OpalPresenceInfo(State state = Unchanged) : m_state(state) { }
+    OpalPresenceInfo(State state = Unchanged);
 
     static PString AsString(State state);
     static State FromString(const PString & str);
     PString AsString() const;
+
+    Comparison Compare(const PObject & other) const;
 };
 
 ostream & operator<<(ostream & strm, OpalPresenceInfo::State state);
@@ -163,15 +161,15 @@ class OpalPresentity : public PSafeObject
         presentity has successfully been indicated as "present" on the server
         only that the underlying system can do so at some time.
       */
-    virtual bool Open() = 0;
+    virtual bool Open();
 
     /** Indicate if the presentity has been successfully opened.
       */
-    virtual bool IsOpen() const = 0;
+    virtual bool IsOpen() const { return m_open; }
 
     /**Close the presentity.
       */
-    virtual bool Close() = 0;
+    virtual bool Close();
   //@}
 
   /**@name Attributes */
@@ -256,14 +254,24 @@ class OpalPresentity : public PSafeObject
         mean the command succeeded, only that the underlying protocol is
         capabable of the action.
       */
-    virtual bool SetLocalPresence(
+    bool SetLocalPresence(
+      const OpalPresenceInfo & info   ///< Presence information to set
+    );
+
+    // For backward compatibility
+    bool SetLocalPresence(
       OpalPresenceInfo::State state,            ///< New state for our presentity
-      const PString & note = PString::Empty()   ///< Additional note attached to the state change
+      const PString & note = PString::Empty()  ///< Additional note attached to the state change
     );
 
     /** Get our presence state
     */
-    virtual bool GetLocalPresence(
+    bool GetLocalPresence(
+      OpalPresenceInfo & info   ///< Previous presence information
+    );
+
+    // For backward compatibility
+    bool GetLocalPresence(
       OpalPresenceInfo::State & state, 
       PString & note
     );
@@ -314,9 +322,10 @@ class OpalPresentity : public PSafeObject
       const AuthorisationRequest & request  ///< Authorisation request information
     );
 
-    typedef PNotifierTemplate<const AuthorisationRequest &> AuthorisationRequestNotifier;
-#define PDECLARE_AuthorisationRequestNotifier(cls, fn) PDECLARE_NOTIFIER2(OpalPresentity, cls, fn, const OpalPresentity::AuthorisationRequest &)
-    #define PCREATE_AuthorisationRequestNotifier(fn) PCREATE_NOTIFIER2(fn, const OpalPresentity::AuthorisationRequest &)
+    typedef PNotifierTemplate<AuthorisationRequest> AuthorisationRequestNotifier;
+    #define PDECLARE_AuthorisationRequestNotifier(cls, fn) PDECLARE_NOTIFIER2(OpalPresentity, cls, fn, OpalPresentity::AuthorisationRequest)
+    #define PDECLARE_ASYNC_AuthorisationRequestNotifier(cls, fn) PDECLARE_ASYNC_NOTIFIER2(OpalPresentity, cls, fn, OpalPresentity::AuthorisationRequest)
+    #define PCREATE_AuthorisationRequestNotifier(fn) PCREATE_NOTIFIER2(fn, OpalPresentity::AuthorisationRequest)
 
     /// Set the notifier for the OnAuthorisationRequest() function.
     void SetAuthorisationRequestNotifier(
@@ -335,9 +344,10 @@ class OpalPresentity : public PSafeObject
       const OpalPresenceInfo & info ///< Info on other presentity that changed state
     );
 
-    typedef PNotifierTemplate<const OpalPresenceInfo &> PresenceChangeNotifier;
-    #define PDECLARE_PresenceChangeNotifier(cls, fn) PDECLARE_NOTIFIER2(OpalPresentity, cls, fn, const OpalPresenceInfo &)
-    #define PCREATE_PresenceChangeNotifier(fn) PCREATE_NOTIFIER2(fn, const OpalPresenceInfo &)
+    typedef PNotifierTemplate< std::auto_ptr<OpalPresenceInfo> > PresenceChangeNotifier;
+    #define PDECLARE_PresenceChangeNotifier(cls, fn) PDECLARE_NOTIFIER2(OpalPresentity, cls, fn, std::auto_ptr<OpalPresenceInfo>)
+    #define PDECLARE_ASYNC_PresenceChangeNotifier(cls, fn) PDECLARE_ASYNC_NOTIFIER2(OpalPresentity, cls, fn, std::auto_ptr<OpalPresenceInfo>)
+    #define PCREATE_PresenceChangeNotifier(fn) PCREATE_NOTIFIER2(fn, std::auto_ptr<OpalPresenceInfo>)
 
     /// Set the notifier for the OnPresenceChange() function.
     void SetPresenceChangeNotifier(
@@ -364,10 +374,12 @@ class OpalPresentity : public PSafeObject
 
       // RFC4482 contact fields, note most of these are duplicated
       // in the vCard structure
+#ifdef P_VCARD
       PvCard  m_vCard;        /**< vCard for the buddy. This is
                                    either a URI pointing to the image, or the image
                                    itself in the form: "data:text/x-vcard,xxxxx
                                    as per RFC2397 */
+#endif
       PURL    m_icon;         /**< The icon/avatar/photo for the buddy. This is
                                    either a URI pointing to the image, or the image
                                    itself in the form: "data:image/jpeg;base64,xxxxx
@@ -482,13 +494,16 @@ class OpalPresentity : public PSafeObject
   //@}
   
   
+#if OPAL_HAS_IM
   /**@name Instant Messaging */
   //@{
     virtual bool SendMessageTo(
       const OpalIM & message
     );
 
-    /** Callback when presentity receives a message
+    /**Called when Instant Message event is received.
+       See OpalIM for details on events. This includes text or composition
+       indication events.
 
         Default implementation calls m_onReceivedMessageNotifier.
       */
@@ -496,15 +511,19 @@ class OpalPresentity : public PSafeObject
       const OpalIM & message ///< incoming message
     );
 
-    typedef PNotifierTemplate<const OpalIM &> ReceivedMessageNotifier;
-    #define PDECLARE_ReceivedMessageNotifier(cls, fn) PDECLARE_NOTIFIER2(OpalPresentity, cls, fn, const OpalIM &)
-    #define PCREATE_ReceivedMessageNotifier(fn) PCREATE_NOTIFIER2(fn, const OpalIM &)
+    typedef PNotifierTemplate<OpalIM> ReceivedMessageNotifier;
+    #define PDECLARE_ReceivedMessageNotifier(cls, fn) PDECLARE_NOTIFIER2(OpalPresentity, cls, fn, OpalIM)
+    #define PDECLARE_ASYNC_ReceivedMessageNotifier(cls, fn) PDECLARE_ASYNC_NOTIFIER2(OpalPresentity, cls, fn, OpalIM)
+    #define PCREATE_ReceivedMessageNotifier(fn) PCREATE_NOTIFIER2(fn, OpalIM)
 
     /// Set the notifier for the OnPresenceChange() function.
     void SetReceivedMessageNotifier(
       const ReceivedMessageNotifier & notifier   ///< Notifier to be called by OnReceivedMessage()
     );
+
+    void Internal_SendMessageToCommand(const OpalSendMessageToCommand & cmd);
   //@}
+#endif // OPAL_HAS_IM
 
     /** Used to set the AOR after the presentity is created
         This override allows the descendant class to convert the internal URL into a real AOR,
@@ -514,9 +533,7 @@ class OpalPresentity : public PSafeObject
       const PURL & aor
     );
 
-    void Internal_SendMessageToCommand(const OpalSendMessageToCommand & cmd);
-
-    virtual PString GetID() const;
+    OpalManager & GetManager() const { return *m_manager; }
 
   protected:
     OpalPresentityCommand * InternalCreateCommand(const char * cmdName);
@@ -528,14 +545,14 @@ class OpalPresentity : public PSafeObject
 
     AuthorisationRequestNotifier m_onAuthorisationRequestNotifier;
     PresenceChangeNotifier       m_onPresenceChangeNotifier;
+#if OPAL_HAS_IM
     ReceivedMessageNotifier      m_onReceivedMessageNotifier;
+#endif // OPAL_HAS_IM
 
+    PAtomicBoolean m_open;
     PMutex m_notificationMutex;
-    PAtomicInteger::IntegerType m_idNumber;
-
     bool m_temporarilyUnavailable;
-    OpalPresenceInfo::State m_localState;      ///< our presentity state
-    PString m_localStateNote;                  ///< Additional note attached to the 
+    OpalPresenceInfo m_localInfo;
 };
 
 
@@ -697,6 +714,7 @@ class OpalSetLocalPresenceCommand : public OpalPresentityCommand, public OpalPre
 };
 
 
+#if OPAL_HAS_IM
 /** Command for sending an IM 
   */
 class OpalSendMessageToCommand : public OpalPresentityCommand
@@ -706,14 +724,18 @@ class OpalSendMessageToCommand : public OpalPresentityCommand
 
     OpalIM m_message;
 };
+#endif // OPAL_HAS_IM
+
 
 ///////////////////////////////////////////////////////////////////////////////
 
 // Include concrete classes here so the factories are initialised
-#if OPAL_SIP && OPAL_PTLIB_EXPAT
-PFACTORY_LOAD(SIP_Presentity);
+#if OPAL_SIP_PRESENCE
+  PFACTORY_LOAD(SIP_Presentity);
 #endif
 
 
-#endif  // OPAL_IM_PRES_ENT_H
+#endif // OPAL_HAS_PRESENCE
 
+
+#endif  // OPAL_IM_PRES_ENT_H
