@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24673 $
+ * $Revision: 27092 $
  * $Author: rjongbloed $
- * $Date: 2010-08-22 20:02:47 -0500 (Sun, 22 Aug 2010) $
+ * $Date: 2012-03-01 21:21:33 -0600 (Thu, 01 Mar 2012) $
  */
 
 #ifndef OPAL_OPAL_TRANSCODERS_H
@@ -40,6 +40,7 @@
 
 #include <opal/mediafmt.h>
 #include <opal/mediacmd.h>
+#include <opal/mediasession.h>
 
 #include <rtp/rtp.h>
 
@@ -80,11 +81,11 @@ class OpalMediaFormatPair : public PObject
        usually overridden by descendent classes to yield the ranking according
        to the semantics of the object.
        
-       The default function is to use the #CompareObjectMemoryDirect()#
+       The default function is to use the <code>CompareObjectMemoryDirect()</code>
        function to do a byte wise memory comparison of the two objects.
 
        @return
-       #LessThan#, #EqualTo# or #GreaterThan#
+       <code>LessThan</code>, <code>EqualTo</code> or <code>GreaterThan</code>
        according to the relative rank of the objects.
      */
     virtual Comparison Compare(
@@ -169,10 +170,12 @@ class OpalTranscoder : public OpalMediaFormatPair
     );
 
     /**Execute the command specified to the transcoder. The commands are
-       highly context sensitive, for example VideoFastUpdate would only apply
-       to a video transcoder.
+       highly context sensitive, for example OpalVideoUpdatePicture would only
+       apply to a video transcoder.
 
-       The default behaviour simply returns PFalse.
+       The default behaviour simply returns false.
+
+       @returns true if command is handled.
       */
     virtual PBoolean ExecuteCommand(
       const OpalMediaCommand & command    ///<  Command to execute.
@@ -196,7 +199,7 @@ class OpalTranscoder : public OpalMediaFormatPair
        The default behaviour makes sure the output list has only one element
        in it and calls the Convert() function.
 
-       Returns PFalse if the conversion fails.
+       Returns false if the conversion fails.
       */
     virtual PBoolean ConvertFrames(
       const RTP_DataFrame & input,  ///<  Input data
@@ -207,7 +210,7 @@ class OpalTranscoder : public OpalMediaFormatPair
        This function takes the input data as a RTP_DataFrame and converts it
        to its output format, placing it into the RTP_DataFrame provided.
 
-       Returns PFalse if the conversion fails.
+       Returns false if the conversion fails.
       */
     virtual PBoolean Convert(
       const RTP_DataFrame & input,  ///<  Input data
@@ -298,7 +301,7 @@ class OpalTranscoder : public OpalMediaFormatPair
     ) { maxOutputSize = size; }
 
     /**Set a notifier to receive commands generated by the transcoder. The
-       commands are highly context sensitive, for example VideoFastUpdate
+       commands are highly context sensitive, for example OpalVideoUpdatePicture
        would only apply to a video transcoder.
       */
     void SetCommandNotifier(
@@ -306,15 +309,15 @@ class OpalTranscoder : public OpalMediaFormatPair
     ) { commandNotifier = notifier; }
 
     /**Get the notifier to receive commands generated by the transcoder. The
-       commands are highly context sensitive, for example VideoFastUpdate
+       commands are highly context sensitive, for example OpalVideoUpdatePicture
        would only apply to a video transcoder.
       */
     const PNotifier & GetCommandNotifier() const { return commandNotifier; }
 
     /// Notify command notifier of command
     void NotifyCommand(
-      OpalMediaCommand & command
-    ) const { if (commandNotifier != PNotifier()) commandNotifier(command, m_sessionID); }
+      const OpalMediaCommand & command
+    ) const;
 
     /// Get session ID for the transcoder (from OpalMediaStream)
     unsigned GetSessionID() const { return m_sessionID; }
@@ -340,6 +343,8 @@ class OpalTranscoder : public OpalMediaFormatPair
 #if OPAL_STATISTICS
     virtual void GetStatistics(OpalMediaStatistics & statistics) const;
 #endif
+
+    void CopyTimestamp(RTP_DataFrame & dst, const RTP_DataFrame & src, bool inToOut) const;
   //@}
 
   protected:
@@ -351,6 +356,8 @@ class OpalTranscoder : public OpalMediaFormatPair
     bool     outputIsRTP, inputIsRTP;
     bool     acceptEmptyPayload;
     bool     acceptOtherPayloads;
+    unsigned m_inClockRate;
+    unsigned m_outClockRate;
 };
 
 
@@ -482,7 +489,7 @@ class OpalStreamedTranscoder : public OpalTranscoder
        This function takes the input data as a RTP_DataFrame and converts it
        to its output format, placing it into the RTP_DataFrame provided.
 
-       Returns PFalse if the conversion fails.
+       Returns false if the conversion fails.
       */
     virtual PBoolean Convert(
       const RTP_DataFrame & input,  ///<  Input data
@@ -538,7 +545,7 @@ class OpalEmptyFramedAudioTranscoder : public OpalFramedTranscoder
     {  }
 
     PBoolean ConvertFrame(const BYTE *, PINDEX &, BYTE *, PINDEX &)
-    { return PFalse; }
+    { return false; }
 };
 
 #define OPAL_DECLARE_EMPTY_TRANSCODER(fmt) \
