@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 25290 $
+ * $Revision: 29047 $
  * $Author: rjongbloed $
- * $Date: 2011-03-07 13:22:29 -0600 (Mon, 07 Mar 2011) $
+ * $Date: 2013-01-29 20:23:19 -0600 (Tue, 29 Jan 2013) $
  */
 
 #ifndef OPAL_OPAL_TRANSPORT_H
@@ -52,13 +52,116 @@ class OpalListener;
 class OpalTransport;
 class OpalInternalTransport;
 
+typedef PSafePtr<OpalTransport> OpalTransportPtr;
+
 
 ////////////////////////////////////////////////////////////////
 
+/**Describe a network transport address.
+   This describes a full definition of a network transport address, which is
+   usually more than just a host address. Broadly it consists of a low level
+   protocol and a protocol specific address/port.
+
+   At present only Internet Protocol based addresses are supported.
+
+   There are some semantic differences when this is being used as an address
+   to connect to, or as an address to listen on. The latter has several
+   "wildcard" variants so it is not necessary to explicitly know a-priori
+   everything about interfaces to start listening for incoming connections.
+
+   The syntax of the OpalTransportAddress string is:
+
+      transport-address = proto "$" proto-address
+
+      proto         = "ip" | "tcp" | "udp" | "tls" | token
+
+      proto-address = addrport | any-string
+      addrport      = addr [ ":" port ]
+
+      addr          = [ host ] [ "%" interface ]
+      host          = hostname | IPv4address | IPv6address | "*"
+
+      hostname      = *( domainlabel "." ) toplabel [ "." ]
+      domainlabel   = alphanum | alphanum *( alphanum | "-" ) alphanum
+      toplabel      = alpha | alpha *( alphanum | "-" ) alphanum
+
+      IPv4address   = 1*digit "." 1*digit "." 1*digit "." 1*digit
+
+      IPv6address   = "[" hexpart [ "%" scope-id ] "]"
+      hexpart       =  hexseq | hexseq "::" [ hexseq ] / "::" [ hexseq ]
+      hexseq        =  hex4 *( ":" hex4)
+      hex4          =  1*4 hexdigit
+      hexdigit      = digit | A-F | a-f
+      scope-id      = number | token
+
+      interface     = number | any-string
+
+      port          = number | token | "*"
+      number        = *digit
+
+      token       =  alpha * (alphanum)
+      alphanum    = alpha | digit
+      alpha       = A-Z | a-z
+      digit       = 0-9
+
+      any-string  = 1* (any-char)
+      any-char    = %20-%7f
+
+   The semantics of certain elements are as follows:
+      addr         Must have at least one of host or "%" interface-name
+      host = "*"   indicates a wildcard match for all IP addresses. Note that
+                   for UDP there is a distinction between "*" and "0.0.0.0" and
+                   "[::]". A "*" will listen on both IPv4 and IPv6, while the
+                   INADDR_ANY and INADDR6_ANY values will only listen on that
+                   version of IP. For tcp "*" is a synonym for "0.0.0.0".
+      port = "*"   indicates port 0. This is only allowed when the
+                   OpalTransportAddress is being used to "listen" on a socket.
+                   It indicates any port number will do and should be allocated
+                   by the operating system.
+      port = token Indicates a string as defined by getservbyname()
+      scope-id     Is dependent on the underlying operating system. For Win32
+                   this must be a number indicating the interface id, for Linux
+                   this may be string, e.g. "eth0". Generally, the scope-id is
+                   only required to disambiguate "link-local" addresses.
+      interface    The number here is not necessarily the same as the number in
+                   scope-id. It is the index into the table returned by the
+                   PIPSocket::GetInterfaceTable() function. The text version of
+                   the interface is dependent on the operating system. For Linux
+                   this is the expected "eth0" etc, for Win32 this can be a
+                   quite long and complex name string.
+
+   Examples:
+      tcp$*:1720
+          Listen  on all interfaces for TCP port 1720.
+      udp$127.0.0.1:5060
+          Indicate UDP port 5060 at local host
+      udp$%ppp0:5060
+          Listen on interface name ppp0, this may come on and off line at any
+          time, and change the actual IP address used at any time. The system
+          will do it's best to "do the right thing" in this situation.
+      udp$10.0.1.1%Win32 Interface Name:5060
+          Explicitly use address 10.0.1.1 on interface "Win32 interface name"
+      tcp$my.host.domain.com:1720"
+          A DNS lookup of my.host.domain.com is used to determine address.
+      udp$[::]:5060
+          Listen on all IPv6 interfaces. This differs from udp$*:5060 in that
+          only IPv6 interfaces are listened to. Similarly udp$0.0.0.0:5060
+          would only listen on IPv4 interfaces.
+      udp$[fe80::224:21ff:fe9e:3e7b%26]:5060
+          Send packet out on link-local address fe80::224:21ff:fe9e:3e7b and
+          interface id 26.
+  */
 class OpalTransportAddress : public PCaselessString
 {
   PCLASSINFO(OpalTransportAddress, PCaselessString);
   public:
+    static const PCaselessString & IpPrefix();
+    static const PCaselessString & UdpPrefix();
+    static const PCaselessString & TcpPrefix();
+#if OPAL_PTLIB_SSL
+    static const PCaselessString & TlsPrefix();
+#endif
+
   /**@name Construction */
   //@{
     OpalTransportAddress();
@@ -77,6 +180,9 @@ class OpalTransportAddress : public PCaselessString
       WORD port,                      ///<  Default port number
       const char * proto = NULL       ///<  Default is "tcp"
     );
+
+    /// Clone function
+    virtual PObject * Clone() const { return new OpalTransportAddress(*this); }
   //@}
 
   /**@name Operations */
@@ -99,6 +205,9 @@ class OpalTransportAddress : public PCaselessString
 
     /**Return the underlying protocol for the transport address.
       */
+    PCaselessString GetProtoPrefix() const { return Left(Find('$')+1); }
+
+    // For backward compatibility
     PCaselessString GetProto(bool withDollar = false) const { return Left(Find('$')+(withDollar?1:0)); }
 
     /**Extract the ip address from transport address.
@@ -115,7 +224,9 @@ class OpalTransportAddress : public PCaselessString
     /**Translate the transport address to a more human readable form.
        Returns the hostname if using IP.
       */
-    virtual PString GetHostName() const;
+    virtual PString GetHostName(
+      bool includeService = false
+    ) const;
 
     enum BindOptions {
       NoBinding,
@@ -203,12 +314,22 @@ class OpalTransportAddress : public PCaselessString
       const char * proto     ///<  Default is "tcp"
     );
 
-    OpalInternalTransport * transport;
+    OpalInternalTransport * m_transport;
 };
 
 
-PDECLARE_ARRAY(OpalTransportAddressArray, OpalTransportAddress)
+class OpalTransportAddressArray : public PArray<OpalTransportAddress>
+{
+    typedef PArray<OpalTransportAddress> ParentClass;
+    PCLASSINFO(OpalTransportAddressArray, ParentClass);
+
+  protected:
+    inline OpalTransportAddressArray(int dummy, const OpalTransportAddressArray * c)
+      : ParentClass(dummy, c) { }
+
   public:
+    OpalTransportAddressArray(PINDEX initialSize = 0)
+      : ParentClass(initialSize) { }
     OpalTransportAddressArray(
       const OpalTransportAddress & address
     ) { AppendAddress(address); }
@@ -232,6 +353,11 @@ PDECLARE_ARRAY(OpalTransportAddressArray, OpalTransportAddress)
       const OpalTransportAddress & address
     );
 
+    virtual PObject * Clone() const
+    {
+      return new OpalTransportAddressArray(0, this);
+    }
+
   protected:
     void AppendStringCollection(
       const PCollection & coll
@@ -285,6 +411,10 @@ class OpalListener : public PObject
       SingleThreadMode
     };
 
+    typedef PNotifierTemplate<const OpalTransportPtr &> AcceptHandler;
+    #define PDECLARE_AcceptHandlerNotifier(cls, fn) PDECLARE_NOTIFIER2(OpalListener, cls, fn, const OpalTransportPtr &)
+
+
     /** Open the listener.
         A thread is spawned to listen for incoming connections. The notifier
         function acceptHandler is called when a new connection is created. The
@@ -299,14 +429,14 @@ class OpalListener : public PObject
         created. That is only a single connection is ever created by this
         listener.
       */
-    virtual PBoolean Open(
-      const PNotifier & acceptHandler,  ///<  Handler function for new connections
+    virtual bool Open(
+      const AcceptHandler & acceptHandler,  ///<  Handler function for new connections
       ThreadMode mode = SpawnNewThreadMode ///<  How handler function is called thread wise
-    ) = 0;
+    );
 
     /** Indicate if the listener is open.
       */
-    virtual PBoolean IsOpen() = 0;
+    virtual bool IsOpen() const = 0;
 
     /**Stop the listener thread and no longer accept incoming connections.
      */
@@ -354,15 +484,12 @@ class OpalListener : public PObject
        Note this function does not return until the Close() function is called
        or there is some other error.
      */
-    PDECLARE_NOTIFIER(PThread, OpalListener, ListenForConnections);
-    PBoolean StartThread(
-      const PNotifier & acceptHandler,  ///<  Handler function for new connections
-      ThreadMode mode                   ///<  How handler function is called thread wise
-    );
+    void ListenForConnections();
+    void TransportThreadMain(OpalTransportPtr transport);
 
     OpalEndPoint & endpoint;
     PThread      * thread;
-    PNotifier      acceptHandler;
+    AcceptHandler  acceptHandler;
     ThreadMode     threadMode;
 };
 
@@ -407,14 +534,19 @@ class OpalListenerIP : public OpalListener
   //@{
     WORD GetListenerPort() const { return listenerPort; }
 
-    virtual const char * GetProtoPrefix() const = 0;
+    virtual const PCaselessString & GetProtoPrefix() const = 0;
   //@}
 
 
   protected:
+    bool CanCreateTransport(
+      const OpalTransportAddress & localAddress,
+      const OpalTransportAddress & remoteAddress
+    ) const;
+
     PIPSocket::Address localAddress;
     WORD               listenerPort;
-    PBoolean               exclusiveListener;
+    bool               exclusiveListener;
 };
 
 
@@ -465,14 +597,14 @@ class OpalListenerTCP : public OpalListenerIP
         that the caller is responsible for calling Accept() and waiting for
         the new connection.
       */
-    virtual PBoolean Open(
-      const PNotifier & acceptHandler,  ///<  Handler function for new connections
+    virtual bool Open(
+      const AcceptHandler & acceptHandler,  ///<  Handler function for new connections
       ThreadMode mode = SpawnNewThreadMode ///<  How handler function is called thread wise
     );
 
     /** Indicate if the listener is open.
       */
-    virtual PBoolean IsOpen();
+    virtual bool IsOpen() const;
 
     /**Stop the listener thread and no longer accept incoming connections.
      */
@@ -494,7 +626,7 @@ class OpalListenerTCP : public OpalListenerIP
 
 
   protected:
-    virtual const char * GetProtoPrefix() const;
+    virtual const PCaselessString & GetProtoPrefix() const;
 
     PTCPSocket listener;
 };
@@ -547,14 +679,14 @@ class OpalListenerUDP : public OpalListenerIP
         that the caller is responsible for calling Accept() and waiting for
         the new connection.
       */
-    virtual PBoolean Open(
-      const PNotifier & acceptHandler,  ///<  Handler function for new connections
+    virtual bool Open(
+      const AcceptHandler & acceptHandler,  ///<  Handler function for new connections
       ThreadMode mode = SpawnNewThreadMode ///<  How handler function is called thread wise
     );
 
     /** Indicate if the listener is open.
       */
-    virtual PBoolean IsOpen();
+    virtual bool IsOpen() const;
 
     /**Stop the listener thread and no longer accept incoming connections.
      */
@@ -594,7 +726,7 @@ class OpalListenerUDP : public OpalListenerIP
 
 
   protected:
-    virtual const char * GetProtoPrefix() const;
+    virtual const PCaselessString & GetProtoPrefix() const;
 
     PMonitoredSocketsPtr listenerBundle;
     PINDEX               m_bufferSize;
@@ -607,16 +739,17 @@ class OpalListenerUDP : public OpalListenerIP
    A "transport" is an object that allows the transfer and processing of data
    from one entity to another.
  */
-class OpalTransport : public PIndirectChannel
+class OpalTransport : public PSafeObject
 {
-    PCLASSINFO(OpalTransport, PIndirectChannel);
-  public:
+    PCLASSINFO(OpalTransport, PSafeObject);
+  protected:
   /**@name Construction */
   //@{
     /**Create a new transport channel.
      */
-    OpalTransport(OpalEndPoint & endpoint);
+    OpalTransport(OpalEndPoint & endpoint, PChannel * channel);
 
+  public:
     /** Destroy the transport channel.
       */
     ~OpalTransport();
@@ -629,6 +762,25 @@ class OpalTransport : public PIndirectChannel
     void PrintOn(
       ostream & strm
     ) const;
+
+    /**Low level write to the channel. This function will block until the
+       requested number of characters are written or the write timeout is
+       reached. The GetLastWriteCount() function returns the actual number
+       of bytes written.
+
+       This will use the <code>writeChannel</code> pointer to actually do the
+       write. If <code>writeChannel</code> is null the this asserts.
+
+       The GetErrorCode() function should be consulted after Write() returns
+       false to determine what caused the failure.
+
+       @return
+       true if at least len bytes were written to the channel.
+     */
+    virtual PBoolean Write(
+      const void * buf, ///< Pointer to a block of memory to write.
+      PINDEX len        ///< Number of bytes to write.
+    );
   //@}
 
   /**@name Operations */
@@ -637,6 +789,13 @@ class OpalTransport : public PIndirectChannel
       */
     virtual PBoolean IsReliable() const = 0;
 
+    /**Get indication of the remote being authenticated.
+       Note, non TLS always return true.
+      */
+    virtual bool IsAuthenticated(
+      const PString & /*domain*/
+    ) const { return true; }
+
     /** Get the interface this transport is bound to.
         This is generally only relevant for datagram based transports such as
         UDP and TCP is always bound to a local interface once open.
@@ -767,7 +926,8 @@ class OpalTransport : public PIndirectChannel
       const PBYTEArray & pdu     ///<  Packet to write
     ) = 0;
 
-    typedef PBoolean (*WriteConnectCallback)(OpalTransport & transport, void * userData);
+    typedef PNotifierTemplate<bool &> WriteConnectCallback;
+    #define PDECLARE_WriteConnectCallback(cls, fn) PDECLARE_NOTIFIER2(OpalTransport, cls, fn, bool &)
 
     /**Write the first packet to the transport, after a connect.
        This will adjust the transport object and call the callback function,
@@ -781,9 +941,17 @@ class OpalTransport : public PIndirectChannel
 
        The default behaviour simply calls the WriteConnectCallback function.
       */
-    virtual PBoolean WriteConnect(
-      WriteConnectCallback function,  ///<  Function for writing data
-      void * userData                 ///<  user data to pass to write function
+    virtual bool WriteConnect(
+      const WriteConnectCallback & function  ///<  Function for writing data
+    );
+
+    /**Set keep alive time and data.
+       The data is sent on the transport if no traffic has occurred for the
+       specified time.
+      */
+    void SetKeepAlive(
+      const PTimeInterval & timeout,
+      const PBYTEArray & data
     );
 
     /**Attach a thread to the transport.
@@ -801,32 +969,57 @@ class OpalTransport : public PIndirectChannel
 
     /**Get the prefix for this transports protocol type.
       */
-    virtual const char * GetProtoPrefix() const = 0;
+    virtual const PCaselessString & GetProtoPrefix() const = 0;
 
-    PMutex & GetWriteMutex() { return m_writeMutex; }
+    bool IsOpen() const
+      { return m_channel != NULL && m_channel->IsOpen(); }
+
+    bool IsGood() const
+      { return IsOpen() && !m_channel->bad() && !m_channel->eof(); }
+
+    PChannel::Errors GetErrorCode(PChannel::ErrorGroup group = PChannel::NumErrorGroups) const
+      { return m_channel != NULL ? m_channel->GetErrorCode(group) : PChannel::NotOpen; }
+
+    PString GetErrorText(PChannel::ErrorGroup group = PChannel::NumErrorGroups) const
+      { return m_channel != NULL ? m_channel->GetErrorText(group) : PString::Empty(); }
+
+    int GetErrorNumber(PChannel::ErrorGroup group = PChannel::NumErrorGroups) const
+      { return m_channel != NULL ? m_channel->GetErrorNumber(group) : -1; }
+
+    void SetReadTimeout(const PTimeInterval & t)
+      { if (m_channel != NULL) m_channel->SetReadTimeout(t); }
+
+    PChannel * GetChannel() const { return m_channel; }
+    void SetChannel(PChannel * chan) { m_channel = chan; }
 
   protected:
+    PDECLARE_NOTIFIER(PTimer, OpalTransport, KeepAlive);
+
     OpalEndPoint & endpoint;
-    PThread      * thread;      ///<  Thread handling the transport
-    PMutex         m_writeMutex;
+    PChannel     * m_channel;
+    PThread      * m_thread;      ///<  Thread handling the transport
+    PTimer         m_keepAliveTimer;
+    PBYTEArray     m_keepAliveData;
 };
 
 
 class OpalTransportIP : public OpalTransport
 {
-  PCLASSINFO(OpalTransportIP, OpalTransport);
-  public:
+    PCLASSINFO(OpalTransportIP, OpalTransport);
+  protected:
   /**@name Construction */
   //@{
     /**Create a new transport channel.
      */
     OpalTransportIP(
       OpalEndPoint & endpoint,    ///<  Endpoint object
+      PChannel * channel,         ///<  Socket
       PIPSocket::Address binding, ///<  Local interface to use
       WORD port                   ///<  Local port to bind to
     );
   //@}
 
+  public:
   /**@name Operations */
   //@{
     /**Get the transport dependent name of the local endpoint.
@@ -861,7 +1054,7 @@ class OpalTransportIP : public OpalTransport
   protected:
     /**Get the prefix for this transports protocol type.
       */
-    virtual const char * GetProtoPrefix() const = 0;
+    virtual const PCaselessString & GetProtoPrefix() const = 0;
 
     PIPSocket::Address localAddress;  // Address of the local interface
     WORD               localPort;
@@ -886,7 +1079,7 @@ class OpalTransportTCP : public OpalTransportIP
     );
     OpalTransportTCP(
       OpalEndPoint & endpoint,    ///<  Endpoint object
-      PTCPSocket * socket         ///<  Socket to use
+      PChannel * socket           ///<  Socket to use
     );
 
     /// Destroy the TCP channel
@@ -936,19 +1129,9 @@ class OpalTransportTCP : public OpalTransportIP
   protected:
     /**Get the prefix for this transports protocol type.
       */
-    virtual const char * GetProtoPrefix() const;
-
-    /**This callback is executed when the Open() function is called with
-       open channels. It may be used by descendent channels to do any
-       handshaking required by the protocol that channel embodies.
-
-       The default behaviour is to simply return true.
-
-       @return
-       Returns true if the protocol handshaking is successful.
-     */
-    virtual PBoolean OnOpen();
+    virtual const PCaselessString & GetProtoPrefix() const;
 
+    bool OnConnectedSocket(PTCPSocket * socket);
 
     PBoolean reuseAddressFlag;
 };
@@ -1107,9 +1290,8 @@ class OpalTransportUDP : public OpalTransportIP
        WriteConnect() calls WriteConnectCallback for each interface. The
        subsequent ReadPDU() returns the answer from the first interface.
       */
-    virtual PBoolean WriteConnect(
-      WriteConnectCallback function,  ///<  Function for writing data
-      void * userData                 ///<  User data to pass to write function
+    virtual bool WriteConnect(
+      const WriteConnectCallback & function  ///<  Function for writing data
     );
 
     /**Set the size of UDP packet reads.
@@ -1122,7 +1304,7 @@ class OpalTransportUDP : public OpalTransportIP
   protected:
     /**Get the prefix for this transports protocol type.
       */
-    virtual const char * GetProtoPrefix() const;
+    virtual const PCaselessString & GetProtoPrefix() const;
 
     OpalManager & manager;
     PINDEX        m_bufferSize;
@@ -1139,8 +1321,14 @@ class OpalInternalTransport : public PObject
 {
     PCLASSINFO(OpalInternalTransport, PObject);
   public:
+    virtual bool Parse(
+      OpalTransportAddress & address,
+      WORD port
+    ) const = 0;
+
     virtual PString GetHostName(
-      const OpalTransportAddress & address
+      const OpalTransportAddress & address,
+      bool includeService
     ) const;
 
     virtual PBoolean GetIpAndPort(
@@ -1169,8 +1357,13 @@ class OpalInternalIPTransport : public OpalInternalTransport
 {
     PCLASSINFO(OpalInternalIPTransport, OpalInternalTransport);
   public:
+    virtual bool Parse(
+      OpalTransportAddress & address,
+      WORD port
+    ) const;
     virtual PString GetHostName(
-      const OpalTransportAddress & address
+      const OpalTransportAddress & address,
+      bool includeService
     ) const;
     virtual PBoolean GetIpAndPort(
       const OpalTransportAddress & address,
@@ -1224,18 +1417,19 @@ typedef OpalInternalIPTransportTemplate<OpalListenerUDP, OpalTransportUDP, OpalT
 #if OPAL_PTLIB_SSL
 
 class PSSLContext;
+class PSSLChannel;
 
-class OpalListenerTCPS : public OpalListenerTCP
+class OpalListenerTLS : public OpalListenerTCP
 {
-  PCLASSINFO(OpalListenerTCPS, OpalListenerTCP);
+  PCLASSINFO(OpalListenerTLS, OpalListenerTCP);
   public:
-    OpalListenerTCPS(
+    OpalListenerTLS(
       OpalEndPoint & endpoint,                 ///<  Endpoint listener is used for
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to listen on
       WORD port = 0,                           ///<  TCP port to listen for connections
       PBoolean exclusive = true               ///< Exclusive listening mode, no other process can accept on the port
     );
-    OpalListenerTCPS(
+    OpalListenerTLS(
       OpalEndPoint & endpoint,                  ///<  Endpoint listener is used for
       const OpalTransportAddress & binding,     ///<  Local interface to listen on
       OpalTransportAddress::BindOptions option  ///< OPtions for binding
@@ -1243,45 +1437,51 @@ class OpalListenerTCPS : public OpalListenerTCP
 
     /** Destroy the listener thread.
       */
-    ~OpalListenerTCPS();
+    ~OpalListenerTLS();
 
-    OpalTransport * Accept(const PTimeInterval & timeout);
-    const char * GetProtoPrefix() const;
+    virtual PBoolean Open(const AcceptHandler & acceptHandler, ThreadMode mode = SpawnNewThreadMode);
+    virtual OpalTransport * Accept(const PTimeInterval & timeout);
+    virtual const PCaselessString & GetProtoPrefix() const;
+    virtual OpalTransport * CreateTransport(
+      const OpalTransportAddress & localAddress,
+      const OpalTransportAddress & remoteAddress
+    ) const;
 
   protected:
-    void Construct();
-
-    PSSLContext * sslContext;
+    PSSLContext * m_sslContext;
 };
 
-class OpalTransportTCPS : public OpalTransportTCP
+
+class OpalTransportTLS : public OpalTransportTCP
 {
-  PCLASSINFO(OpalTransportTCPS, OpalTransportTCP);
+  PCLASSINFO(OpalTransportTLS, OpalTransportTCP);
     public:
-      OpalTransportTCPS(
+      OpalTransportTLS(
+        OpalEndPoint & endpoint,    ///<  Endpoint object
+        PSSLChannel * ssl
+      );
+
+      OpalTransportTLS(
         OpalEndPoint & endpoint,    ///<  Endpoint object
         PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to use
         WORD port = 0,              ///<  Local port to bind to
         PBoolean reuseAddr = false      ///<  Flag for binding to already bound interface
       );
-      OpalTransportTCPS(
-        OpalEndPoint & endpoint,    ///<  Endpoint object
-        PTCPSocket * socket         ///<  Socket to use
-      );
 
       /// Destroy the TCPS channel
-      ~OpalTransportTCPS();
+      ~OpalTransportTLS();
 
-      PBoolean IsCompatibleTransport(const OpalTransportAddress & address) const;
-      PBoolean Connect();
-      PBoolean OnOpen();
-      const char * GetProtoPrefix() const;
-
-    protected:
-      PSSLContext * sslContext;
+      // Overrides
+      virtual PBoolean IsCompatibleTransport(const OpalTransportAddress & address) const;
+      virtual PBoolean Connect();
+      virtual const PCaselessString & GetProtoPrefix() const;
+      virtual bool IsAuthenticated(const PString & domain) const;
 };
 
-typedef OpalInternalIPTransportTemplate<OpalListenerTCPS, OpalTransportTCPS, OpalTransportAddress::Datagram, OpalTransportUDP> OpalInternalTCPSTransport;
+typedef OpalInternalIPTransportTemplate<OpalListenerTLS, OpalTransportTLS, OpalTransportAddress::Datagram, OpalTransportUDP> OpalInternalTLSTransport;
+
+typedef OpalTransportTLS OpalTransportTCPS; // For backward compatibility
+typedef OpalListenerTLS OpalListenerTCPS;
 
 
 #endif // OPAL_PTLIB_SSL
