@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23901 $
+ * $Revision: 29341 $
  * $Author: rjongbloed $
- * $Date: 2010-01-05 18:23:31 -0600 (Tue, 05 Jan 2010) $
+ * $Date: 2013-03-27 02:06:24 -0500 (Wed, 27 Mar 2013) $
  */
 
 #ifndef OPAL_RTP_JITTER_H
@@ -41,10 +41,12 @@
 #include <opal/buildopts.h>
 
 #include <rtp/rtp.h>
+#include <ptlib/safecoll.h>
 
 
 class RTP_JitterBuffer;
 class RTP_JitterBufferAnalyser;
+class OpalRTPSession;
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -57,16 +59,28 @@ class OpalJitterBuffer : public PSafeObject
   PCLASSINFO(OpalJitterBuffer, PSafeObject);
 
   public:
+    /// Initialisation information
+    struct Init {
+      Init(unsigned minDelay = 0, unsigned maxDelay = 0)
+        : m_minJitterDelay(minDelay)
+        , m_maxJitterDelay(maxDelay != 0 ? maxDelay : minDelay)
+        , m_timeUnits(8)
+        , m_packetSize(2048)
+      { }
+
+      unsigned m_minJitterDelay; ///<  Minimum delay in RTP timestamp units
+      unsigned m_maxJitterDelay; ///<  Maximum delay in RTP timestamp units
+      unsigned m_timeUnits;      ///<  Time units, usually 8 or 16
+      PINDEX m_packetSize;       ///<  Max RTP packet size
+    };
+
   /**@name Construction */
   //@{
     /**Constructor for this jitter buffer. The size of this buffer can be
        altered later with the SetDelay method
       */
     OpalJitterBuffer(
-      unsigned minJitterDelay, ///<  Minimum delay in RTP timestamp units
-      unsigned maxJitterDelay, ///<  Maximum delay in RTP timestamp units
-      unsigned timeUnits = 8,  ///<  Time units, usually 8 or 16
-      PINDEX packetSize = 2048 ///<  Max RTP packet size
+      const Init & init  ///< Initialisation information
     );
     
     /** Destructor, which closes this down and deletes the internal list of frames
@@ -84,23 +98,24 @@ class OpalJitterBuffer : public PSafeObject
 
   /**@name Operations */
   //@{
+    /// Start jitter buffer.
+    virtual void Start() { }
+
     /**Set the maximum delay the jitter buffer will operate to.
       */
     void SetDelay(
-      unsigned minJitterDelay, ///<  Minimum delay in RTP timestamp units
-      unsigned maxJitterDelay, ///<  Maximum delay in RTP timestamp units
-      PINDEX packetSize = 2048 ///<  Max RTP packet size
+      const Init & init  ///< Initialisation information
     );
 
-    /** Reset jitter buffer.
-        Jitter buffer is cleared and "restocked" from input data.
+    /**Reset jitter buffer.
       */
-    void Reset() { m_resetJitterBufferNow = true; }
+    void Reset();
 
     /**Write data frame from the RTP channel.
       */
     virtual PBoolean WriteData(
-      const RTP_DataFrame & frame   ///< Frame to feed into jitter buffer
+      const RTP_DataFrame & frame,   ///< Frame to feed into jitter buffer
+      const PTimeInterval & tick = 0 ///< Real time tick for packet arrival
     );
 
     /**Read a data frame from the jitter buffer.
@@ -108,81 +123,89 @@ class OpalJitterBuffer : public PSafeObject
        with zero payload size is returned.
       */
     virtual PBoolean ReadData(
-      RTP_DataFrame & frame   ///<  Frame to extract from jitter buffer
+      RTP_DataFrame & frame,  ///<  Frame to extract from jitter buffer
+      const PTimeInterval & tick = 0 ///< Real time tick for packet removal
     );
 
     /**Get current delay for jitter buffer.
       */
-    DWORD GetJitterTime() const { return currentJitterTime; }
+    DWORD GetCurrentJitterDelay() const { return m_currentJitterDelay; }
+    
+    /**Get minimum delay for jitter buffer.
+      */
+    DWORD GetMinJitterDelay() const { return m_minJitterDelay; }
+    
+    /**Get maximum delay for jitter buffer.
+      */
+    DWORD GetMaxJitterDelay() const { return m_maxJitterDelay; }
 
     /**Get time units.
       */
-    unsigned GetTimeUnits() const { return timeUnits; }
+    unsigned GetTimeUnits() const { return m_timeUnits; }
     
     /**Get total number received packets too late to go into jitter buffer.
       */
-    DWORD GetPacketsTooLate() const { return packetsTooLate; }
+    DWORD GetPacketsTooLate() const { return m_packetsTooLate; }
 
     /**Get total number received packets that overran the jitter buffer.
       */
-    DWORD GetBufferOverruns() const { return bufferOverruns; }
+    DWORD GetBufferOverruns() const { return m_bufferOverruns; }
 
     /**Get maximum consecutive marker bits before buffer starts to ignore them.
       */
-    DWORD GetMaxConsecutiveMarkerBits() const { return maxConsecutiveMarkerBits; }
+    DWORD GetMaxConsecutiveMarkerBits() const { return m_maxConsecutiveMarkerBits; }
 
     /**Set maximum consecutive marker bits before buffer starts to ignore them.
       */
-    void SetMaxConsecutiveMarkerBits(DWORD max) { maxConsecutiveMarkerBits = max; }
+    void SetMaxConsecutiveMarkerBits(DWORD max) { m_maxConsecutiveMarkerBits = max; }
   //@}
 
   protected:
-    class Entry : public RTP_DataFrame
-    {
-      public:
-        Entry(PINDEX sz) : RTP_DataFrame(0, sz) { }
-        PTimeInterval tick;
-    };
-    OpalJitterBuffer::Entry * GetAvailableEntry();
-    void InternalWriteData(OpalJitterBuffer::Entry * availableEntry);
+    DWORD CalculateRequiredTimestamp(DWORD playOutTimestamp) const;
+    bool AdjustCurrentJitterDelay(int delta);
 
-    DWORD         minJitterTime;
-    DWORD         maxJitterTime;
-    unsigned      timeUnits;
-    PINDEX        bufferSize;
-    DWORD         maxConsecutiveMarkerBits;
+    unsigned m_timeUnits;
+    PINDEX   m_packetSize;
+    DWORD    m_minJitterDelay;      ///< Minimum jitter delay in timestamp units
+    DWORD    m_maxJitterDelay;      ///< Maximum jitter delay in timestamp units
+    int      m_jitterGrowTime;      ///< Amount to increase jitter delay by when get "late" packet
+    DWORD    m_jitterShrinkPeriod;  ///< Period (in timestamp units) over which buffer is
+                                    ///< consistently filled before shrinking
+    int      m_jitterShrinkTime;    ///< Amount to shrink jitter delay by if consistently filled
+    DWORD    m_silenceShrinkPeriod; ///< Reduce jitter delay is silent for this long
+    int      m_silenceShrinkTime;   ///< Amount to shrink jitter delay by if consistently silent
+    DWORD    m_jitterDriftPeriod;
 
-    DWORD         currentJitterTime;
-    DWORD         packetsTooLate;
-    unsigned      bufferOverruns;
-    unsigned      consecutiveBufferOverruns;
-    DWORD         consecutiveMarkerBits;
-    bool          markerWarning;
-    PTimeInterval consecutiveEarlyPacketStartTime;
-    DWORD         lastWriteTimestamp;
-    PTimeInterval lastWriteTick;
-    DWORD         jitterCalc;
-    DWORD         targetJitterTime;
-    unsigned      jitterCalcPacketCount;
-    bool          m_resetJitterBufferNow;
+    int      m_currentJitterDelay;
+    DWORD    m_packetsTooLate;
+    DWORD    m_bufferOverruns;
+    DWORD    m_consecutiveMarkerBits;
+    DWORD    m_maxConsecutiveMarkerBits;
+    DWORD    m_consecutiveLatePackets;
 
-    struct FrameQueue : public PList<Entry>
-    {
-      FrameQueue() { DisallowDeleteObjects(); }
-      ~FrameQueue() { AllowDeleteObjects(); }
-    };
-    FrameQueue freeFrames;
-    FrameQueue jitterBuffer;
-    Entry * GetNewest(bool pop);
-    Entry * GetOldest(bool pop);
+    unsigned m_frameTimeCount;
+    DWORD    m_incomingFrameTime;
+    int      m_lastFrameTime[2];
+    DWORD    m_lastSequenceNum;
+    DWORD    m_lastTimestamp;
+    DWORD    m_lastSyncSource;
+    DWORD    m_bufferFilledTime;
+    DWORD    m_bufferLowTime;
+    DWORD    m_bufferEmptiedTime;
+    int      m_timestampDelta;
 
-    Entry * currentFrame;    // storage of current frame
+    enum {
+      e_SynchronisationStart,
+      e_SynchronisationFill,
+      e_SynchronisationShrink,
+      e_SynchronisationDone
+    } m_synchronisationState;
 
-    PMutex bufferMutex;
-    bool   preBuffering;
-    bool   firstReadData;
+    typedef std::map<DWORD, RTP_DataFrame> FrameMap;
+    FrameMap m_frames;
+    PMutex   m_bufferMutex;
 
-    RTP_JitterBufferAnalyser * analyser;
+    RTP_JitterBufferAnalyser * m_analyser;
 };
 
 
@@ -194,13 +217,13 @@ class OpalJitterBufferThread : public OpalJitterBuffer
     PCLASSINFO(OpalJitterBufferThread, OpalJitterBuffer);
  public:
     OpalJitterBufferThread(
-      unsigned minJitterDelay, ///<  Minimum delay in RTP timestamp units
-      unsigned maxJitterDelay, ///<  Maximum delay in RTP timestamp units
-      unsigned timeUnits = 8,  ///<  Time units, usually 8 or 16
-      PINDEX packetSize = 2048 ///<  Max RTP packet size
+      const Init & init  ///< Initialisation information
     );
     ~OpalJitterBufferThread();
 
+    /// Start jiter buffer
+    virtual void Start();
+
     /**Read a data frame from the jitter buffer.
        This function never blocks. If no data is available, an RTP packet
        with zero payload size is returned.
@@ -208,7 +231,8 @@ class OpalJitterBufferThread : public OpalJitterBuffer
        Override of base class so can terminate caller when shutting down.
       */
     virtual PBoolean ReadData(
-      RTP_DataFrame & frame   ///<  Frame to extract from jitter buffer
+      RTP_DataFrame & frame,  ///<  Frame to extract from jitter buffer
+      const PTimeInterval & tick = 0 ///< Real time tick for packet removal
     );
 
     /**This class instance collects data from the outside world in this
@@ -216,16 +240,12 @@ class OpalJitterBufferThread : public OpalJitterBuffer
 
        @return true on successful read, false on faulty read. */
     virtual PBoolean OnReadPacket(
-      RTP_DataFrame & frame,  ///<  Frame read from the RTP session
-      PBoolean loop           ///<  If true, loop as long as data is available, if false, only process once
+      RTP_DataFrame & frame   ///<  Frame read from the RTP session
     ) = 0;
 
   protected:
     PDECLARE_NOTIFIER(PThread, OpalJitterBufferThread, JitterThreadMain);
 
-    /// Internal function to be called from derived class constructor
-    void StartThread();
-
     /// Internal function to be called from derived class destructor
     void WaitForThreadTermination();
 
@@ -234,37 +254,6 @@ class OpalJitterBufferThread : public OpalJitterBuffer
 };
 
 
-/////////////////////////////////////////////////////////////////////////////
-/**A descendant of the OpalJitterBuffer that reads RTP_DataFrame instances
-   from the RTP_Sessions
-  */
-class RTP_JitterBuffer : public OpalJitterBufferThread
-{
-    PCLASSINFO(RTP_JitterBuffer, OpalJitterBufferThread);
- public:
-    RTP_JitterBuffer(
-      RTP_Session & session,   ///<  Associated RTP session tor ead data from
-      unsigned minJitterDelay, ///<  Minimum delay in RTP timestamp units
-      unsigned maxJitterDelay, ///<  Maximum delay in RTP timestamp units
-      unsigned timeUnits = 8,  ///<  Time units, usually 8 or 16
-      PINDEX packetSize = 2048 ///<  Max RTP packet size
-    );
-    ~RTP_JitterBuffer();
-
-    /**This class instance collects data from the outside world in this
-       method.
-
-       @return true on successful read, false on faulty read. */
-    virtual PBoolean OnReadPacket(
-      RTP_DataFrame & frame,  ///<  Frame read from the RTP session
-      PBoolean loop           ///<  If true, loop as long as data is available, if false, only process once
-    );
-
- protected:
-   /**This class extracts data from the outside world by reading from this session variable */
-   RTP_Session & session;
-};
-
 #endif // OPAL_RTP_JITTER_H
 
 
