@@ -1,7 +1,7 @@
 /*
  * handlers.h
  *
- * Session Initiation Protocol endpoint.
+ * Session Initiation Protocol non-connection protocol handlers.
  *
  * Open Phone Abstraction Library (OPAL)
  *
@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24609 $
+ * $Revision: 29768 $
  * $Author: rjongbloed $
- * $Date: 2010-07-29 02:18:34 -0500 (Thu, 29 Jul 2010) $
+ * $Date: 2013-05-23 02:27:37 -0500 (Thu, 23 May 2013) $
  */
 
 #ifndef OPAL_SIP_HANDLERS_H
@@ -44,21 +44,47 @@
 #if OPAL_SIP
 
 #include <opal/pres_ent.h>
-#include <opal/connection.h>
 #include <sip/sippdu.h>
 
 
+/// Separate base class to allow searching sorted list
+class SIPHandlerBase : public PSafeObject 
+{
+    PCLASSINFO(SIPHandlerBase, PSafeObject);
+
+  protected:
+    SIPHandlerBase(const PString & callID) : m_callID(callID) { }
+
+  public:
+    const PString & GetCallID() const
+      { return m_callID; }
+
+  protected:
+    const PString m_callID;
+
+    // Keep a copy of the keys used for easy removal on destruction
+    typedef std::map<PString, PSafePtr<SIPHandler> > IndexMap;
+    std::pair<IndexMap::iterator, bool> m_byAorAndPackage;
+    std::pair<IndexMap::iterator, bool> m_byAuthIdAndRealm;
+    std::pair<IndexMap::iterator, bool> m_byAorUserAndRealm;
+
+  friend class SIPHandlersList;
+};
+
+
 /* Class to handle SIP REGISTER, SUBSCRIBE, MESSAGE, and renew
  * the 'bindings' before they expire.
  */
-class SIPHandler : public PSafeObject 
+class SIPHandler : public SIPHandlerBase, public SIPTransactionOwner
 {
-  PCLASSINFO(SIPHandler, PSafeObject);
+  PCLASSINFO(SIPHandler, SIPHandlerBase);
 
 protected:
   SIPHandler(
+    SIP_PDU::Methods method,
     SIPEndPoint & ep,
-    const SIPParameters & params
+    const SIPParameters & params,
+    const PString & callID = SIPTransaction::GenerateCallID()
   );
 
 public:
@@ -66,7 +92,9 @@ public:
 
   virtual Comparison Compare(const PObject & other) const;
 
-  virtual bool ShutDown();
+  // Overrides from SIPTransactionOwner
+  virtual PString GetAuthID() const        { return m_username; }
+  virtual PString GetPassword() const      { return m_password; }
 
   enum State {
     Subscribed,       // The registration is active
@@ -81,37 +109,27 @@ public:
 
   void SetState (SIPHandler::State s);
 
-  inline SIPHandler::State GetState () 
+  inline SIPHandler::State GetState() const
   { return m_state; }
 
-  virtual OpalTransport * GetTransport();
-
-  virtual SIPAuthentication * GetAuthentication()
-  { return authentication; }
-
-  virtual const SIPURL & GetAddressOfRecord()
+  virtual const SIPURL & GetAddressOfRecord() const
     { return m_addressOfRecord; }
 
   virtual PBoolean OnReceivedNOTIFY(SIP_PDU & response);
 
   virtual void SetExpire(int e);
 
-  virtual int GetExpire()
-    { return expire; }
+  virtual int GetExpire() const
+    { return m_currentExpireTime; }
 
-  virtual PString GetCallID()
-    { return callID; }
-
-  virtual void SetBody(const PString & b)
-    { body = b;}
+  virtual void SetBody(const PString & /*body*/) { }
 
   virtual bool IsDuplicateCSeq(unsigned ) { return false; }
 
-  virtual SIPTransaction * CreateTransaction(OpalTransport & t) = 0;
+  virtual SIPTransaction * CreateTransaction(OpalTransport &) { return NULL; }
 
-  virtual SIP_PDU::Methods GetMethod() = 0;
-  virtual SIPSubscribe::EventPackage GetEventPackage() const
-  { return SIPEventPackage(); }
+  SIP_PDU::Methods GetMethod() const { return m_method; }
+  virtual SIPSubscribe::EventPackage GetEventPackage() const { return SIPSubscribe::EventPackage(); }
 
   virtual void OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnReceivedIntervalTooBrief(SIPTransaction & transaction, SIP_PDU & response);
@@ -120,65 +138,45 @@ public:
   virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnTransactionFailed(SIPTransaction & transaction);
 
-  virtual void OnFailed(const SIP_PDU & response);
+//  virtual void OnFailed(const SIP_PDU & response);
   virtual void OnFailed(SIP_PDU::StatusCodes);
+  virtual void SendStatus(SIP_PDU::StatusCodes code, State state);
 
   bool ActivateState(SIPHandler::State state);
   virtual bool SendNotify(const PObject * /*body*/) { return false; }
 
-  SIPEndPoint & GetEndPoint() const { return endpoint; }
+  SIP_PDU::StatusCodes GetLastResponseStatus() const { return m_lastResponseStatus; }
 
   const OpalProductInfo & GetProductInfo() const { return m_productInfo; }
 
-  const PString & GetUsername() const     { return m_username; }
-  const PString & GetPassword() const     { return m_password; }
-  const PString & GetRealm() const        { return m_realm; }
-  const SIPURL & GetRemoteAddress() const { return m_remoteAddress; }
-  const SIPURL & GetProxy() const         { return m_proxy; }
+  const PString & GetRealm() const { return m_realm; }
 
-  SIPMIMEInfo m_mime;
+  virtual bool ShutDown();
 
 protected:
   virtual PBoolean SendRequest(SIPHandler::State state);
   void RetryLater(unsigned after);
-  PDECLARE_NOTIFIER(PTimer, SIPHandler, OnExpireTimeout);
-  static PBoolean WriteSIPHandler(OpalTransport & transport, void * info);
-  virtual bool WriteSIPHandler(OpalTransport & transport, bool forked);
+  void OnExpireTimeout();
+  PDECLARE_WriteConnectCallback(SIPHandler, WriteTransaction);
 
-  SIPEndPoint               & endpoint;
-
-  SIPAuthentication         * authentication;
   PString                     m_username;
   PString                     m_password;
   PString                     m_realm;
 
-  PSafeList<SIPTransaction>   transactions;
-  OpalTransport             * m_transport;
-  SIPURL                      m_addressOfRecord;
-  SIPURL                      m_remoteAddress;
-  PString                     callID;
-  int                         expire;
-  int                         originalExpire;
-  int                         offlineExpire;
-  PString                     body;
-  unsigned                    authenticationAttempts;
+  const SIP_PDU::Methods      m_method;
+  const SIPURL                m_addressOfRecord;
+  SIPMIMEInfo                 m_mime;
+
+  unsigned                    m_lastCseq;
+  SIP_PDU::StatusCodes        m_lastResponseStatus;
+  int                         m_currentExpireTime;
+  int                         m_originalExpireTime;
+  int                         m_offlineExpireTime;
   State                       m_state;
-  queue<State>                m_stateQueue;
+  std::queue<State>           m_stateQueue;
   bool                        m_receivedResponse;
-  PTimer                      expireTimer; 
-  PTimeInterval               retryTimeoutMin; 
-  PTimeInterval               retryTimeoutMax; 
-  SIPURL                      m_proxy;
+  SIPPoolTimer<SIPHandler>    m_expireTimer; 
   OpalProductInfo             m_productInfo;
-
-  // Keep a copy of the keys used for easy removal on destruction
-  typedef std::map<PString, PSafePtr<SIPHandler> > IndexMap;
-  std::pair<IndexMap::iterator, bool> m_byCallID;
-  std::pair<IndexMap::iterator, bool> m_byAorAndPackage;
-  std::pair<IndexMap::iterator, bool> m_byAuthIdAndRealm;
-  std::pair<IndexMap::iterator, bool> m_byAorUserAndRealm;
-
-  friend class SIPHandlersList;
 };
 
 #if PTRACING
@@ -196,25 +194,27 @@ public:
     const SIPRegister::Params & params
   );
 
-  ~SIPRegisterHandler();
-
   virtual SIPTransaction * CreateTransaction(OpalTransport &);
   virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
-  virtual SIP_PDU::Methods GetMethod()
-    { return SIP_PDU::Method_REGISTER; }
-
-  virtual void OnFailed(SIP_PDU::StatusCodes r);
 
   void UpdateParameters(const SIPRegister::Params & params);
 
   const SIPRegister::Params & GetParams() const { return m_parameters; }
 
+  const SIPURLList & GetContacts() const { return m_contactAddresses; }
+  const SIPURLList & GetServiceRoute() const { return m_serviceRoute; }
+
 protected:
   virtual PBoolean SendRequest(SIPHandler::State state);
-  void SendStatus(SIP_PDU::StatusCodes code, State state);
+  virtual void SendStatus(SIP_PDU::StatusCodes code, State state);
+  PString CreateRegisterContact(const OpalTransportAddress & address, int q);
 
-  SIPRegister::Params m_parameters;
-  unsigned            m_sequenceNumber;
+  SIPRegister::Params  m_parameters;
+  unsigned             m_sequenceNumber;
+  SIPURLList           m_contactAddresses;
+  SIPURLList           m_serviceRoute;
+  OpalTransportAddress m_externalAddress;
+  PAtomicInteger::IntegerType m_rfc5626_reg_id;
 };
 
 
@@ -228,26 +228,20 @@ public:
   virtual SIPTransaction * CreateTransaction (OpalTransport &);
   virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
   virtual PBoolean OnReceivedNOTIFY(SIP_PDU & response);
-  virtual void OnFailed(const SIP_PDU & response);
-  virtual SIP_PDU::Methods GetMethod ()
-    { return SIP_PDU::Method_SUBSCRIBE; }
+  virtual void OnFailed(SIP_PDU::StatusCodes);
   virtual SIPEventPackage GetEventPackage() const
     { return m_parameters.m_eventPackage; }
 
   void UpdateParameters(const SIPSubscribe::Params & params);
 
-  virtual bool IsDuplicateCSeq(unsigned sequenceNumber) { return m_dialog.IsDuplicateCSeq(sequenceNumber); }
-
   const SIPSubscribe::Params & GetParams() const { return m_parameters; }
 
 protected:
-  virtual PBoolean SendRequest(SIPHandler::State state);
-  virtual bool WriteSIPHandler(OpalTransport & transport, bool forked);
-  void SendStatus(SIP_PDU::StatusCodes code, State state);
+  virtual void WriteTransaction(OpalTransport & transport, bool & succeeded);
+  virtual void SendStatus(SIP_PDU::StatusCodes code, State state);
   bool DispatchNOTIFY(SIP_PDU & request, SIP_PDU & response);
 
   SIPSubscribe::Params     m_parameters;
-  SIPDialogContext         m_dialog;
   bool                     m_unconfirmed;
   SIPEventPackageHandler * m_packageHandler;
 };
@@ -259,19 +253,17 @@ class SIPNotifyHandler : public SIPHandler
 public:
   SIPNotifyHandler(
     SIPEndPoint & ep,
-    const PString & targetAddress,
     const SIPEventPackage & eventPackage,
     const SIPDialogContext & dialog
   );
   ~SIPNotifyHandler();
 
   virtual SIPTransaction * CreateTransaction(OpalTransport &);
-  virtual SIP_PDU::Methods GetMethod ()
-    { return SIP_PDU::Method_NOTIFY; }
   virtual SIPEventPackage GetEventPackage() const
     { return m_eventPackage; }
 
-  virtual bool IsDuplicateCSeq(unsigned sequenceNumber) { return m_dialog.IsDuplicateCSeq(sequenceNumber); }
+  virtual void SetBody(const PString & body) { m_body = body; }
+
   virtual bool SendNotify(const PObject * body);
 
   enum Reasons {
@@ -285,12 +277,12 @@ public:
 
 protected:
   virtual PBoolean SendRequest(SIPHandler::State state);
-  virtual bool WriteSIPHandler(OpalTransport & transport, bool forked);
+  virtual void WriteTransaction(OpalTransport & transport, bool & succeeded);
 
   SIPEventPackage          m_eventPackage;
-  SIPDialogContext         m_dialog;
   Reasons                  m_reason;
   SIPEventPackageHandler * m_packageHandler;
+  PString                  m_body;
 };
 
 
@@ -302,17 +294,16 @@ public:
   SIPPublishHandler(SIPEndPoint & ep, 
                     const SIPSubscribe::Params & params,
                     const PString & body);
-  ~SIPPublishHandler();
+
+  virtual void SetBody(const PString & body) { m_body = body; }
 
   virtual SIPTransaction * CreateTransaction(OpalTransport &);
   virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
-  virtual SIP_PDU::Methods GetMethod()
-    { return SIP_PDU::Method_PUBLISH; }
-  virtual SIPEventPackage GetEventPackage() const
-    { return m_parameters.m_eventPackage; }
+  virtual SIPEventPackage GetEventPackage() const { return m_parameters.m_eventPackage; }
 
-private:
+protected:
   SIPSubscribe::Params m_parameters;
+  PString              m_body;
   PString              m_sipETag;
 };
 
@@ -321,24 +312,17 @@ class SIPMessageHandler : public SIPHandler
 {
   PCLASSINFO(SIPMessageHandler, SIPHandler);
 public:
-  SIPMessageHandler(SIPEndPoint & ep, 
-                    const SIPMessage::Params & params,
-                    const PString & body);
-  ~SIPMessageHandler();
+  SIPMessageHandler(SIPEndPoint & ep, const SIPMessage::Params & params);
 
   virtual SIPTransaction * CreateTransaction (OpalTransport &);
-  virtual SIP_PDU::Methods GetMethod ()
-    { return SIP_PDU::Method_MESSAGE; }
-  virtual void OnFailed (SIP_PDU::StatusCodes);
-  virtual void SetBody(const PString & b);
+  virtual void OnFailed(SIP_PDU::StatusCodes);
+  virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
 
-  PURL m_localAddress;
-  PString m_id;
+  void UpdateParameters(const SIPMessage::Params & params);
 
-private:
+protected:
   SIPMessage::Params m_parameters;
-  virtual void OnExpireTimeout(PTimer &, INT);
-  PString m_body;
+  bool               m_messageSent;
 };
 
 
@@ -348,8 +332,6 @@ class SIPPingHandler : public SIPHandler
 public:
   SIPPingHandler(SIPEndPoint & ep, const PURL & to);
   virtual SIPTransaction * CreateTransaction (OpalTransport &);
-  virtual SIP_PDU::Methods GetMethod ()
-    { return SIP_PDU::Method_MESSAGE; }
 };
 
 
@@ -402,6 +384,11 @@ class SIPHandlersList
     /**
      * Find the SIPHandler object with the specified authRealm
      */
+    PSafePtr<SIPHandler> FindSIPHandlerByAuthRealm(const PString & authRealm, PSafetyMode m);
+
+    /**
+     * Find the SIPHandler object with the specified authRealm & user
+     */
     PSafePtr<SIPHandler> FindSIPHandlerByAuthRealm(const PString & authRealm, const PString & userName, PSafetyMode m);
 
     /**
@@ -411,8 +398,8 @@ class SIPHandlersList
      * or 6001@seconix.com when registering 6001@seconix.com to
      * sip.seconix.com
      */
-    PSafePtr<SIPHandler> FindSIPHandlerByUrl(const PURL & url, SIP_PDU::Methods meth, PSafetyMode m);
-    PSafePtr<SIPHandler> FindSIPHandlerByUrl(const PURL & url, SIP_PDU::Methods meth, const PString & eventPackage, PSafetyMode m);
+    PSafePtr<SIPHandler> FindSIPHandlerByUrl(const PString & url, SIP_PDU::Methods meth, PSafetyMode m);
+    PSafePtr<SIPHandler> FindSIPHandlerByUrl(const PString & url, SIP_PDU::Methods meth, const PString & eventPackage, PSafetyMode m);
 
     /**
      * Find the SIPHandler object with the specified registration host.
@@ -424,51 +411,53 @@ class SIPHandlersList
   protected:
     void RemoveIndexes(SIPHandler * handler);
 
-    PMutex m_extraMutex;
-    PSafeList<SIPHandler> m_handlersList;
+    PSafeSortedList<SIPHandlerBase> m_handlersList;
 
     typedef SIPHandler::IndexMap IndexMap;
     PSafePtr<SIPHandler> FindBy(IndexMap & by, const PString & key, PSafetyMode m);
 
-    IndexMap m_byCallID;
     IndexMap m_byAorAndPackage;
     IndexMap m_byAuthIdAndRealm;
     IndexMap m_byAorUserAndRealm;
 };
 
 
+#if OPAL_SIP_PRESENCE
+
 /** Information for SIP "presence" event package notification messages.
   */
 class SIPPresenceInfo : public OpalPresenceInfo
 {
+  PCLASSINFO_WITH_CLONE(SIPPresenceInfo, OpalPresenceInfo)
+
 public:
-  SIPPresenceInfo(
-    const PString & personId = PString::Empty(),
-    State state = Unchanged
-  );
-
-  // basic presence defined by RFC 3863
-  PString m_tupleId;
-  PString m_contact;
-
-  // presence extensions defined by RFC 4480
-  PStringArray m_activities;  // list of activities, seperated by newline
-
   // presence agent
   PString m_presenceAgent;
+  PString m_personId;
 
   PString AsXML() const;
 
+  static bool ParseNotify(
+    SIPSubscribe::NotifyCallbackInfo & notifyInfo,
+    list<SIPPresenceInfo> & info
+  );
+
   void PrintOn(ostream & strm) const;
   friend ostream & operator<<(ostream & strm, const SIPPresenceInfo & info) { info.PrintOn(strm); return strm; }
 
-  static State FromSIPActivityString(const PString & str);
-
-  static bool AsSIPActivityString(State state, PString & str);
-  bool AsSIPActivityString(PString & str) const;
-
-  PString m_personId;
+  // Constructor
+  SIPPresenceInfo(
+    State state = Unchanged
+  ) : OpalPresenceInfo(state) { }
+  SIPPresenceInfo(
+    SIP_PDU::StatusCodes status,
+    bool subscribing
+  );
+  SIPPresenceInfo(
+    const OpalPresenceInfo & info
+  ) : OpalPresenceInfo(info) { }
 };
+#endif // OPAL_SIP_PRESENCE
 
 
 /** Information for SIP "dialog" event package notification messages.
@@ -477,36 +466,26 @@ struct SIPDialogNotification : public PObject
 {
   PCLASSINFO(SIPDialogNotification, PObject);
 
-  enum States {
+  P_DECLARE_ENUM(States,
     Terminated,
     Trying,
     Proceeding,
     Early,
-    Confirmed,
-
-    FirstState = Terminated,
-    LastState = Confirmed
-  };
-  friend States operator++(States & state) { return (state = (States)(state+1)); }
-  friend States operator--(States & state) { return (state = (States)(state-1)); }
+    Confirmed
+  );
   static PString GetStateName(States state);
   PString GetStateName() const { return GetStateName(m_state); }
 
-  enum Events {
-    NoEvent = -1,
+  P_DECLARE_ENUM_EX(Events, NumEvents,
+    NoEvent, -1,
     Cancelled,
     Rejected,
     Replaced,
     LocalBye,
     RemoteBye,
     Error,
-    Timeout,
-
-    FirstEvent = Cancelled,
-    LastEvent = Timeout
-  };
-  friend Events operator++(Events & evt) { return (evt = (Events)(evt+1)); }
-  friend Events operator--(Events & evt) { return (evt = (Events)(evt-1)); }
+    Timeout
+  );
   static PString GetEventName(Events state);
   PString GetEventName() const { return GetEventName(m_eventType); }
 
@@ -540,6 +519,34 @@ struct SIPDialogNotification : public PObject
 };
 
 
+/** Information for SIP "reg" event package (RFC3680) notification messages.
+  */
+class SIPRegNotification : public PObject
+{
+  PCLASSINFO(SIPRegNotification, PObject)
+public:
+  P_DECLARE_ENUM(States,
+    Initial,
+    Active,
+    Terminated
+  );
+  static PString GetStateName(States state);
+  PString GetStateName() const { return GetStateName(m_state); }
+  static States GetStateFromName(const PCaselessString & str);
+
+  SIPRegNotification(
+    const SIPURL & aor = SIPURL(),
+    States state = Initial
+  );
+
+  SIPURL            m_aor;
+  States            m_state;
+  std::list<SIPURL> m_contacts;
+
+  void PrintOn(ostream & strm) const;
+};
+
+
 #endif // OPAL_SIP
 
 #endif // OPAL_SIP_HANDLERS_H
