@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24440 $
+ * $Revision: 28208 $
  * $Author: rjongbloed $
- * $Date: 2010-06-02 00:57:45 -0500 (Wed, 02 Jun 2010) $
+ * $Date: 2012-08-16 20:54:46 -0500 (Thu, 16 Aug 2012) $
  */
 
 #ifndef OPAL_SIP_SDP_H
@@ -43,18 +43,42 @@
 #include <opal/transports.h>
 #include <opal/mediatype.h>
 #include <opal/mediafmt.h>
-#include <rtp/rtp.h>
+#include <rtp/rtp_session.h>
+
+
+
+/**OpalConnection::StringOption key to a boolean indicating the SDP ptime
+   parameter should be included in audio session streams. Default false.
+  */
+#define OPAL_OPT_OFFER_SDP_PTIME "Offer-SDP-PTime"
+
+/**OpalConnection::StringOption key to an integer indicating the SDP
+   rtpcp-fb parameter handling. A value of zero indicates it is not to be
+   offerred to the remote. A value of 1 indicates it is to be offerred
+   without requiring the RTP/AVPF transport, but is included in RTP/AVP.
+   A value of 2 indicates it is to be only offerred in RTP/AVPF with a
+   second session in RTP/AVP mode to be also offerred. Default is 1.
+  */
+#define OPAL_OPT_OFFER_RTCP_FB  "Offer-RTCP-FB"
+
+/**OpalConnection::StringOption key to a boolean indicating the RTCP
+   feedback commands are to be sent irrespective of the SDP rtpcp-fb
+   parameter presented by the remote. Default false.
+  */
+#define OPAL_OPT_FORCE_RTCP_FB  "Force-RTCP-FB"
+
 
 /////////////////////////////////////////////////////////
 
-class SDPBandwidth : public std::map<PString, unsigned>
+class SDPBandwidth : public std::map<PCaselessString, OpalBandwidth>
 {
+  typedef std::map<PCaselessString, OpalBandwidth> BaseClass;
   public:
-    unsigned & operator[](const PString & type);
-    unsigned operator[](const PString & type) const;
+    OpalBandwidth & operator[](const PCaselessString & type);
+    OpalBandwidth operator[](const PCaselessString & type) const;
     friend ostream & operator<<(ostream & out, const SDPBandwidth & bw);
     bool Parse(const PString & param);
-    void SetMin(const PString & type, unsigned value);
+    void SetMax(const PCaselessString & type, OpalBandwidth value);
 };
 
 /////////////////////////////////////////////////////////
@@ -63,7 +87,7 @@ class SDPMediaDescription;
 
 class SDPMediaFormat : public PObject
 {
-  PCLASSINFO(SDPMediaFormat, PObject);
+    PCLASSINFO(SDPMediaFormat, PObject);
   public:
     SDPMediaFormat(
       SDPMediaDescription & parent,
@@ -77,6 +101,7 @@ class SDPMediaFormat : public PObject
     );
 
     virtual void PrintOn(ostream & str) const;
+    virtual PObject * Clone() const { return new SDPMediaFormat(*this); }
 
     RTP_DataFrame::PayloadTypes GetPayloadType() const { return payloadType; }
 
@@ -86,19 +111,23 @@ class SDPMediaFormat : public PObject
     void SetFMTP(const PString & _fmtp); 
     PString GetFMTP() const;
 
-    unsigned GetClockRate(void)                        { return clockRate ; }
-    void SetClockRate(unsigned  v)                     { clockRate = v; }
+    unsigned GetClockRate(void)    { return clockRate ; }
+    void SetClockRate(unsigned  v) { clockRate = v; }
 
     void SetParameters(const PString & v) { parameters = v; }
 
-    const OpalMediaFormat & GetMediaFormat() const;
-    OpalMediaFormat & GetWritableMediaFormat();
+#if OPAL_VIDEO
+    void SetRTCP_FB(const PString & v) { m_rtcp_fb.FromString(v); }
+    OpalVideoFormat::RTCPFeedback GetRTCP_FB() const { return m_rtcp_fb; }
+#endif
+
+    const OpalMediaFormat & GetMediaFormat() const { return m_mediaFormat; }
+    OpalMediaFormat & GetWritableMediaFormat() { return m_mediaFormat; }
 
     bool PreEncode();
-    bool PostDecode(unsigned bandwidth);
+    bool PostDecode(const OpalMediaFormatList & mediaFormats, unsigned bandwidth);
 
   protected:
-    void InitialiseMediaFormat(OpalMediaFormat & mediaFormat) const;
     void SetMediaFormatOptions(OpalMediaFormat & mediaFormat) const;
 
     OpalMediaFormat m_mediaFormat;
@@ -109,15 +138,18 @@ class SDPMediaFormat : public PObject
     PCaselessString encodingName;
     PString parameters;
     PString m_fmtp;
+#if OPAL_VIDEO
+    OpalVideoFormat::RTCPFeedback m_rtcp_fb; // RFC4585
+#endif
 };
 
-PLIST(SDPMediaFormatList, SDPMediaFormat);
+typedef PList<SDPMediaFormat> SDPMediaFormatList;
+
 
 /////////////////////////////////////////////////////////
 
-class SDPMediaDescription : public PObject
+class SDPCommonAttributes
 {
-  PCLASSINFO(SDPMediaDescription, PObject);
   public:
     // The following enum is arranged so it can be used as a bit mask,
     // e.g. GetDirection()&SendOnly indicates send is available
@@ -129,6 +161,45 @@ class SDPMediaDescription : public PObject
       SendRecv
     };
 
+    SDPCommonAttributes()
+      : m_direction(Undefined)
+    { }
+
+    virtual ~SDPCommonAttributes() { }
+
+    virtual void SetDirection(const Direction & d) { m_direction = d; }
+    virtual Direction GetDirection() const { return m_direction; }
+
+    virtual OpalBandwidth GetBandwidth(const PString & type) const { return m_bandwidth[type]; }
+    virtual void SetBandwidth(const PString & type, OpalBandwidth value) { m_bandwidth[type] = value; }
+
+    virtual const SDPBandwidth & GetBandwidth() const { return m_bandwidth; }
+
+    virtual const RTPExtensionHeaders & GetExtensionHeaders() const { return m_extensionHeaders; }
+    virtual void SetExtensionHeader(const RTPExtensionHeaderInfo & ext) { m_extensionHeaders.insert(ext); }
+
+    virtual void ParseAttribute(const PString & value);
+    virtual void SetAttribute(const PString & attr, const PString & value);
+
+    virtual void OutputAttributes(ostream & strm) const;
+
+    static const PCaselessString & ConferenceTotalBandwidthType();
+    static const PCaselessString & ApplicationSpecificBandwidthType();
+    static const PCaselessString & TransportIndependentBandwidthType(); // RFC3890
+
+  protected:
+    Direction           m_direction;
+    SDPBandwidth        m_bandwidth;
+    RTPExtensionHeaders m_extensionHeaders;
+};
+
+
+/////////////////////////////////////////////////////////
+
+class SDPMediaDescription : public PObject, public SDPCommonAttributes
+{
+  PCLASSINFO(SDPMediaDescription, PObject);
+  public:
     SDPMediaDescription(
       const OpalTransportAddress & address,
       const OpalMediaType & mediaType
@@ -136,11 +207,10 @@ class SDPMediaDescription : public PObject
 
     virtual bool PreEncode();
     virtual void Encode(const OpalTransportAddress & commonAddr, ostream & str) const;
-    virtual bool PrintOn(ostream & strm, const PString & str) const;
 
     virtual bool Decode(const PStringArray & tokens);
     virtual bool Decode(char key, const PString & value);
-    virtual bool PostDecode();
+    virtual bool PostDecode(const OpalMediaFormatList & mediaFormats);
 
     // return the string used within SDP to identify this media type
     virtual PString GetSDPMediaType() const = 0;
@@ -158,46 +228,47 @@ class SDPMediaDescription : public PObject
     virtual void AddMediaFormat(const OpalMediaFormat & mediaFormat);
     virtual void AddMediaFormats(const OpalMediaFormatList & mediaFormats, const OpalMediaType & mediaType);
 
+    virtual void SetCryptoKeys(OpalMediaCryptoKeyList & cryptoKeys);
+    virtual OpalMediaCryptoKeyList GetCryptoKeys() const;
+
     virtual void SetAttribute(const PString & attr, const PString & value);
 
-    virtual void SetDirection(const Direction & d) { direction = d; }
-    virtual Direction GetDirection() const { return transportAddress.IsEmpty() ? Inactive : direction; }
+    virtual Direction GetDirection() const { return m_transportAddress.IsEmpty() ? Inactive : m_direction; }
 
-    virtual const OpalTransportAddress & GetTransportAddress() const { return transportAddress; }
+    virtual const OpalTransportAddress & GetTransportAddress() const { return m_transportAddress; }
     virtual PBoolean SetTransportAddress(const OpalTransportAddress &t);
 
-    virtual WORD GetPort() const { return port; }
+    virtual WORD GetPort() const { return m_port; }
 
-    virtual OpalMediaType GetMediaType() const { return mediaType; }
-
-    virtual unsigned GetBandwidth(const PString & type) const { return bandwidth[type]; }
-    virtual void SetBandwidth(const PString & type, unsigned value) { bandwidth[type] = value; }
-
-    virtual const SDPBandwidth & GetBandwidth() const { return bandwidth; }
+    virtual OpalMediaType GetMediaType() const { return m_mediaType; }
 
     virtual void CreateSDPMediaFormats(const PStringArray & tokens);
     virtual SDPMediaFormat * CreateSDPMediaFormat(const PString & portString) = 0;
 
-    virtual PString GetSDPPortList() const = 0;
+    virtual PString GetSDPPortList() const;
 
     virtual void ProcessMediaOptions(SDPMediaFormat & sdpFormat, const OpalMediaFormat & mediaFormat);
 
-    unsigned GetPTime () const { return ptime; }
-    unsigned GetMaxPTime () const { return maxptime; }
+#if OPAL_VIDEO
+    virtual OpalVideoFormat::ContentRole GetContentRole() const { return OpalVideoFormat::eNoRole; }
+#endif
+
+    void SetOptionStrings(const PStringOptions & options) { m_stringOptions = options; }
+    const PStringOptions & GetOptionStrings() const { return m_stringOptions; }
+
+    virtual void Copy(SDPMediaDescription & mediaDescription);
 
   protected:
     virtual SDPMediaFormat * FindFormat(PString & str) const;
 
-    OpalTransportAddress transportAddress;
-    Direction direction;
-    WORD port;
-    WORD portCount;
-    OpalMediaType mediaType;
+    OpalTransportAddress m_transportAddress;
+    PCaselessString      m_transportType;
+    PStringOptions       m_stringOptions;
+    WORD                 m_port;
+    WORD                 m_portCount;
+    OpalMediaType        m_mediaType;
 
-    SDPMediaFormatList formats;
-    SDPBandwidth       bandwidth;
-    unsigned           ptime;
-    unsigned           maxptime;
+    SDPMediaFormatList   formats;
 };
 
 PARRAY(SDPMediaDescriptionArray, SDPMediaDescription);
@@ -205,19 +276,71 @@ PARRAY(SDPMediaDescriptionArray, SDPMediaDescription);
 
 class SDPDummyMediaDescription : public SDPMediaDescription
 {
-  PCLASSINFO(SDPDummyMediaDescription, SDPMediaDescription);
+    PCLASSINFO(SDPDummyMediaDescription, SDPMediaDescription);
   public:
     SDPDummyMediaDescription(const OpalTransportAddress & address, const PStringArray & tokens);
     virtual PString GetSDPMediaType() const;
     virtual PCaselessString GetSDPTransportType() const;
     virtual SDPMediaFormat * CreateSDPMediaFormat(const PString & portString);
     virtual PString GetSDPPortList() const;
+    virtual void Copy(SDPMediaDescription & mediaDescription);
 
   private:
     PStringArray m_tokens;
 };
 
 
+class SDPDummySession : public OpalMediaSession
+{
+    PCLASSINFO(SDPDummySession, OpalMediaSession)
+  public:
+    SDPDummySession(const Init & init);
+    static const PCaselessString & SessionType();
+    virtual const PCaselessString & GetSessionType() const;
+    virtual SDPMediaDescription * CreateSDPMediaDescription();
+    virtual OpalMediaStream * CreateMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, bool isSource);
+
+  private:
+    PStringArray m_tokens;
+};
+
+
+class SDPCryptoSuite : public PObject
+{
+    PCLASSINFO(SDPCryptoSuite, PObject)
+  public:
+    SDPCryptoSuite(unsigned tag);
+
+    bool SetKeyInfo(const OpalMediaCryptoKeyInfo & keyInfo);
+    OpalMediaCryptoKeyInfo * GetKeyInfo() const;
+
+    bool Decode(const PString & attrib);
+    void PrintOn(ostream & strm) const;
+
+    struct KeyParam {
+      KeyParam(const PString & keySalt)
+        : m_keySalt(keySalt)
+        , m_lifetime(0)
+        , m_mkiIndex(0)
+        , m_mkiLength(0)
+      { }
+
+      PString  m_keySalt;
+      PUInt64  m_lifetime;
+      unsigned m_mkiIndex;
+      unsigned m_mkiLength;
+    };
+
+    unsigned GetTag() const { return m_tag; }
+    const PString & GetName() const { return m_suiteName; }
+
+  protected:
+    unsigned       m_tag;
+    PString        m_suiteName;
+    list<KeyParam> m_keyParams;
+    PStringOptions m_sessionParams;
+};
+
 /////////////////////////////////////////////////////////
 //
 //  SDP media description for media classes using RTP/AVP transport (audio and video)
@@ -225,14 +348,27 @@ class SDPDummyMediaDescription : public SDPMediaDescription
 
 class SDPRTPAVPMediaDescription : public SDPMediaDescription
 {
-  PCLASSINFO(SDPRTPAVPMediaDescription, SDPMediaDescription);
+    PCLASSINFO(SDPRTPAVPMediaDescription, SDPMediaDescription);
   public:
     SDPRTPAVPMediaDescription(const OpalTransportAddress & address, const OpalMediaType & mediaType);
+    virtual bool Decode(const PStringArray & tokens);
     virtual PCaselessString GetSDPTransportType() const;
     virtual SDPMediaFormat * CreateSDPMediaFormat(const PString & portString);
     virtual PString GetSDPPortList() const;
-    virtual bool PrintOn(ostream & str, const PString & connectString) const;
-    void SetAttribute(const PString & attr, const PString & value);
+    virtual bool PreEncode();
+    virtual void OutputAttributes(ostream & str) const;
+    virtual void SetCryptoKeys(OpalMediaCryptoKeyList & cryptoKeys);
+    virtual OpalMediaCryptoKeyList GetCryptoKeys() const;
+    virtual void SetAttribute(const PString & attr, const PString & value);
+
+    void EnableFeedback() { m_enableFeedback = true; }
+
+  protected:
+    bool m_enableFeedback;
+#if OPAL_VIDEO
+    OpalVideoFormat::RTCPFeedback m_rtcp_fb;
+#endif
+    PList<SDPCryptoSuite>         m_cryptoSuites;
 };
 
 /////////////////////////////////////////////////////////
@@ -242,14 +378,22 @@ class SDPRTPAVPMediaDescription : public SDPMediaDescription
 
 class SDPAudioMediaDescription : public SDPRTPAVPMediaDescription
 {
-  PCLASSINFO(SDPAudioMediaDescription, SDPRTPAVPMediaDescription);
+    PCLASSINFO(SDPAudioMediaDescription, SDPRTPAVPMediaDescription);
   public:
     SDPAudioMediaDescription(const OpalTransportAddress & address);
     virtual PString GetSDPMediaType() const;
-    virtual bool PrintOn(ostream & str, const PString & connectString) const;
-    void SetAttribute(const PString & attr, const PString & value);
+    virtual void OutputAttributes(ostream & str) const;
+    virtual void SetAttribute(const PString & attr, const PString & value);
+    virtual bool PostDecode(const OpalMediaFormatList & mediaFormats);
+
+  protected:
+    unsigned m_PTime;
+    unsigned m_maxPTime;
 };
 
+
+#if OPAL_VIDEO
+
 /////////////////////////////////////////////////////////
 //
 //  SDP media description for video media
@@ -257,15 +401,24 @@ class SDPAudioMediaDescription : public SDPRTPAVPMediaDescription
 
 class SDPVideoMediaDescription : public SDPRTPAVPMediaDescription
 {
-  PCLASSINFO(SDPVideoMediaDescription, SDPRTPAVPMediaDescription);
+    PCLASSINFO(SDPVideoMediaDescription, SDPRTPAVPMediaDescription);
   public:
     SDPVideoMediaDescription(const OpalTransportAddress & address);
     virtual PString GetSDPMediaType() const;
     virtual bool PreEncode();
-    virtual bool PrintOn(ostream & str, const PString & connectString) const;
-    void SetAttribute(const PString & attr, const PString & value);
+    virtual void OutputAttributes(ostream & str) const;
+    virtual void SetAttribute(const PString & attr, const PString & value);
+    virtual bool PostDecode(const OpalMediaFormatList & mediaFormats);
+    virtual OpalVideoFormat::ContentRole GetContentRole() const { return m_contentRole; }
+
+  protected:
+    OpalVideoFormat::ContentRole m_contentRole;
+    unsigned                     m_contentMask;
 };
 
+#endif // OPAL_VIDEO
+
+
 /////////////////////////////////////////////////////////
 //
 //  SDP media description for application media
@@ -279,12 +432,11 @@ class SDPApplicationMediaDescription : public SDPMediaDescription
     virtual PCaselessString GetSDPTransportType() const;
     virtual SDPMediaFormat * CreateSDPMediaFormat(const PString & portString);
     virtual PString GetSDPMediaType() const;
-    virtual PString GetSDPPortList() const;
 };
 
 /////////////////////////////////////////////////////////
 
-class SDPSessionDescription : public PObject
+class SDPSessionDescription : public PObject, public SDPCommonAttributes
 {
   PCLASSINFO(SDPSessionDescription, PObject);
   public:
@@ -296,52 +448,43 @@ class SDPSessionDescription : public PObject
 
     void PrintOn(ostream & strm) const;
     PString Encode() const;
-    PBoolean Decode(const PString & str);
+    bool Decode(const PString & str, const OpalMediaFormatList & mediaFormats);
 
-    void SetSessionName(const PString & v) { sessionName = v; }
-    PString GetSessionName() const         { return sessionName; }
+    void SetSessionName(const PString & v);
+    PString GetSessionName() const { return sessionName; }
 
-    void SetUserName(const PString & v)    { ownerUsername = v; }
-    PString GetUserName() const            { return ownerUsername; }
+    void SetUserName(const PString & v);
+    PString GetUserName() const { return ownerUsername; }
 
     const SDPMediaDescriptionArray & GetMediaDescriptions() const { return mediaDescriptions; }
 
     SDPMediaDescription * GetMediaDescriptionByType(const OpalMediaType & rtpMediaType) const;
     SDPMediaDescription * GetMediaDescriptionByIndex(PINDEX i) const;
-    void AddMediaDescription(SDPMediaDescription * md) { mediaDescriptions.Append(md); }
+    void AddMediaDescription(SDPMediaDescription * md) { mediaDescriptions.Append(PAssertNULL(md)); }
     
-    void SetDirection(const SDPMediaDescription::Direction & d) { direction = d; }
-    SDPMediaDescription::Direction GetDirection(unsigned) const;
+    virtual SDPMediaDescription::Direction GetDirection(unsigned) const;
     bool IsHold() const;
 
     const OpalTransportAddress & GetDefaultConnectAddress() const { return defaultConnectAddress; }
     void SetDefaultConnectAddress(
       const OpalTransportAddress & address
     );
-	
+  
     time_t GetOwnerSessionId() const { return ownerSessionId; }
     void SetOwnerSessionId(time_t value) { ownerSessionId = value; }
 
-    PINDEX GetOwnerVersion() const { return ownerVersion; }
-    void SetOwnerVersion(PINDEX value) { ownerVersion = value; }
+    unsigned GetOwnerVersion() const { return ownerVersion; }
+    void SetOwnerVersion(unsigned value) { ownerVersion = value; }
 
     OpalTransportAddress GetOwnerAddress() const { return ownerAddress; }
     void SetOwnerAddress(OpalTransportAddress addr) { ownerAddress = addr; }
 
-    unsigned GetBandwidth(const PString & type) const { return bandwidth[type]; }
-    void SetBandwidth(const PString & type, unsigned value) { bandwidth[type] = value; }
-
     OpalMediaFormatList GetMediaFormats() const;
 
-    static const PString & ConferenceTotalBandwidthType();
-    static const PString & ApplicationSpecificBandwidthType();
-    static const PString & TransportIndependentBandwidthType(); // RFC3890
-
   protected:
     void ParseOwner(const PString & str);
 
     SDPMediaDescriptionArray mediaDescriptions;
-    SDPMediaDescription::Direction direction;
 
     PINDEX protocolVersion;
     PString sessionName;
@@ -351,8 +494,6 @@ class SDPSessionDescription : public PObject
     unsigned ownerVersion;
     OpalTransportAddress ownerAddress;
     OpalTransportAddress defaultConnectAddress;
-
-    SDPBandwidth bandwidth;	
 };
 
 /////////////////////////////////////////////////////////
