@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24913 $
+ * $Revision: 29641 $
  * $Author: rjongbloed $
- * $Date: 2010-11-30 16:22:28 -0600 (Tue, 30 Nov 2010) $
+ * $Date: 2013-05-03 07:14:54 -0500 (Fri, 03 May 2013) $
  */
 
 #ifndef OPAL_SIP_SIPCON_H
@@ -40,28 +40,84 @@
 
 #if OPAL_SIP
 
-#include <opal/buildopts.h>
-#include <opal/rtpconn.h>
+#include <rtp/rtpconn.h>
 #include <sip/sippdu.h>
 #include <sip/handlers.h>
 
-#if OPAL_VIDEO
-#include <opal/pcss.h>                  // for OpalPCSSConnection
-#include <codec/vidcodec.h>             // for OpalVideoUpdatePicture command
-#endif
 
-#if OPAL_HAS_IM
-#include <im/rfc4103.h>
-#endif
+class OpalSIPIMContext;
 
-class OpalCall;
-class SIPEndPoint;
+
+/**OpalConnection::StringOption key to a boolean indicating that call
+   deflection/forwarding should use REFER rather tha  302 response.
+   Default false.
+  */
+#define OPAL_OPT_FORWARD_REFER   "Forward-Refer"
+
+/**OpalConnection::StringOption key to a boolean indicating the the state
+   of the "Refer-Sub" header in the REFER request. Default true.
+  */
+#define OPAL_OPT_REFER_SUB       "Refer-Sub"
+
+/**OpalConnection::StringOption key to an integer indicating the the mode
+   for the reliable provisional response system. See PRACKMode for more
+   information. Default is from SIPEndPoint::GetDefaultPRACKMode() which
+   in turn defaults to e_prackSupported.
+  */
+#define OPAL_OPT_PRACK_MODE      "PRACK-Mode"
+
+/**OpalConnection::StringOption key to a boolean indicating that we should
+   make initial SDP offer. Default true.
+  */
+#define OPAL_OPT_INITIAL_OFFER "Initial-Offer"
+
+/**OpalConnection::StringOption key to a string for a regular expression to
+   match the product information, which if matching the remote system, will
+   indicate the remote does not support asymmetric hold as required by the
+   standard.
+   
+   This fault is when SDP sendonly is sent (us putting them on hold), and
+   they reply inactive, which implies them putting us on hold. When we
+   subsequently send recvonly to release our hold to them, they continue to
+   send inactive, and hold is never released.
+
+   Note the OpalProductInfo vendor, name & version strings are concatenated
+   before comparison with the regular expression.
+
+   Defaults to empty string.
+  */
+#define OPAL_OPT_SYMMETRIC_HOLD_PRODUCT "Symmetric-Hold-Product"
+
+/**Allow INVITE with Replaces to operate on "early" dialog.
+   While RFC 3891 explicitly forbids and incoming INVITE with Replaces to
+   replace a call which was incoming to the UA. It is allowed for a dialog
+   originated by the UA receiving the INVITE with Replace, but not for ones
+   where it is "ringing". Unfortunately, some systems (*cough Cisco) do it
+   anyway. This option allows the replacement and dows not send the 481
+   response as required by the specification.
+
+   Defaults to false.
+  */
+#define OPAL_OPT_ALLOW_EARLY_REPLACE "Allow-Early-Replace"
+
+/**OpalConnection::StringOption key to a string representing the precise SDP
+   to be included in the INVITE offer. No media streams are opened or any
+   checks whstsoever made on the string. It is simply included as the body of
+   the INVITE.
+   
+   Note this options presence also prevents handling of the response SDP
+   
+   Defaults to empty string which generates SDP from available
+   media formats and media streams.
+  */
+#define OPAL_OPT_EXTERNAL_SDP "External-SDP"
 
 #define SIP_HEADER_PREFIX      "SIP-Header:"
 #define SIP_HEADER_REPLACES    SIP_HEADER_PREFIX"Replaces"
+#define SIP_HEADER_REFERRED_BY SIP_HEADER_PREFIX"Referred-By"
 #define SIP_HEADER_CONTACT     SIP_HEADER_PREFIX"Contact"
 
-#define OPAL_OPT_SYMMETRIC_HOLD_PRODUCT "Symmetric-Hold-Product"
+#define OPAL_SIP_REFERRED_CONNECTION "Referred-Connection"
 
 
 /////////////////////////////////////////////////////////////////////////
@@ -69,23 +125,37 @@ class SIPEndPoint;
 /**Session Initiation Protocol connection.
  */
 
-class SIPConnection : public OpalRTPConnection
+class SIPConnection : public OpalRTPConnection, public SIPTransactionOwner
 {
-  PCLASSINFO(SIPConnection, OpalRTPConnection);
+    PCLASSINFO(SIPConnection, OpalRTPConnection);
   public:
 
   /**@name Construction */
   //@{
+    struct Init {
+      Init(OpalCall & call, SIPEndPoint & endpoint)
+        : m_call(call)
+        , m_endpoint(endpoint)
+        , m_userData(NULL)
+        , m_invite(NULL)
+        , m_options(0)
+        , m_stringOptions(NULL)
+        { }
+
+      OpalCall    & m_call;      ///< Owner call for connection
+      SIPEndPoint & m_endpoint;  ///< SIP endpoint that controls the connection
+      PString       m_token;     ///< Token to identify the connection
+      SIPURL        m_address;   ///< Destination address for outgoing call
+      void        * m_userData;  ///< User data
+      SIP_PDU     * m_invite;    ///< Invite packet
+      unsigned m_options;        ///< Connection options
+      OpalConnection::StringOptions * m_stringOptions;  ///<  complex string options
+    };
+
     /**Create a new connection.
      */
     SIPConnection(
-      OpalCall & call,                          ///<  Owner call for connection
-      SIPEndPoint & endpoint,                   ///<  Owner endpoint for connection
-      const PString & token,                    ///<  token to identify the connection
-      const SIPURL & address,                   ///<  Destination address for outgoing call
-      OpalTransport * transport,                ///<  Transport INVITE came in on
-      unsigned int options = 0,                 ///<  Connection options
-      OpalConnection::StringOptions * stringOptions = NULL  ///<  complex string options
+      const Init & init         ///< Initialisation parameters
     );
 
     /**Destroy connection.
@@ -113,6 +183,9 @@ class SIPConnection : public OpalRTPConnection
      */
     virtual PString GetIdentifier() const;
 
+    /// Call back for connection to act on changed string options
+    virtual void OnApplyStringOptions();
+
     /**Start an outgoing connection.
        This function will initiate the connection to the remote entity, for
        example in H.323 it sends a SETUP, in SIP it sends an INVITE etc.
@@ -121,6 +194,10 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual PBoolean SetUpConnection();
 
+    /** Get the remote transport address
+      */
+    virtual OpalTransportAddress GetRemoteAddress() const { return m_remoteAddress; }
+
     /**Get the destination address of an incoming connection.
        This will, for example, collect a phone number from a POTS line, or
        get the fields from the H.225 SETUP pdu in a H.323 connection.
@@ -168,31 +245,57 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual bool SetAlertingType(const PString & info);
 
+    /**Get call information of an incoming call.
+       This is protocol dependent information provided about the call. The
+       details are outside the scope of this help.
+
+       For SIP this corresponds to the string contained in the "Call-Info"
+       header field of the INVITE.
+      */
+    virtual PString GetCallInfo() const;
+
     /**Initiate the transfer of an existing call (connection) to a new remote 
        party.
 
-       If remoteParty is a valid call token, then the remote party is transferred
-       to that party (consultation transfer) and both calls are cleared.
+       A REFER command is sent to the remote endpoint to cause it to move the
+       call it has with this endpoint to a new address. This call will, in the
+       end, be cleared. The OnTransferNotify() function can be used to monitor
+       the progress of the transfer in case it fails.
+
+       If remoteParty is a valid call token, then this is short hand for
+       the REFER to the remote endpoint of this call to do an INVITE with
+       Replaces header to the remote party of the supplied tokens call. This
+       is used for consultation transfer where A calls B, B holds A, B calls C,
+       B transfers A to C. The last step is a REFER to A with call details of
+       C that are extracted from the B to C call leg. This short cut is
+       possible because A nd C may be other endpoints but both B's are in this
+       instance of OPAL.
+       
+       In the end, both calls are cleared. The OnTransferNotify() function can
+       be used to monitor the progress of the transfer in case it fails.
      */
     virtual bool TransferConnection(
       const PString & remoteParty   ///<  Remote party to transfer the existing call to
     );
 
     /**Put the current connection on hold, suspending all media streams.
-     */
-    virtual bool HoldConnection();
+       The \p fromRemote parameter indicates if we a putting the remote on
+       hold (false) or it is a request for the remote to put us on hold (true).
 
-    /**Retrieve the current connection from hold, activating all media 
-     * streams.
+       The /p placeOnHold parameter indicates of teh command/request is for
+       going on hold or retrieving from hold.
      */
-    virtual bool RetrieveConnection();
+    virtual bool Hold(
+      bool fromRemote,  ///< Flag for if remote has us on hold, or we have them
+      bool placeOnHold  ///< Flag for setting on or off hold
+    );
 
     /**Return true if the current connection is on hold.
-       The bool parameter indicates if we are testing if the remote system
-       has us on hold, or we have them on hold.
+       The \p fromRemote parameter indicates if we are testing if the remote
+       system has us on hold, or we have them on hold.
      */
-    virtual bool IsConnectionOnHold(
-      bool fromRemote  ///< Test if remote has us on hold, or we have them
+    virtual bool IsOnHold(
+      bool fromRemote  ///< Flag for if remote has us on hold, or we have them
     );
 
     /**Indicate to remote endpoint an alert is in progress.
@@ -207,7 +310,7 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual PBoolean SetAlerting(
       const PString & calleeName,   ///<  Name of endpoint being alerted.
-      PBoolean withMedia
+      PBoolean withMedia            ///<  Flag to alert with/without media
     );
 
     /**Indicate to remote endpoint we are connected.
@@ -220,6 +323,27 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual OpalMediaFormatList GetMediaFormats() const;
     
+    /**Indicate connection requires symmetric media.
+       Default behaviour returns false.
+      */
+    virtual bool RequireSymmetricMediaStreams() const;
+
+#if OPAL_T38_CAPABILITY
+    /**Switch to/from T.38 fax mode.
+      */
+    virtual bool SwitchFaxMediaStreams(
+      bool toT38  ///< T.38 or return to audio mode
+    );
+#endif
+
+    /**Create a new media stream.
+     */
+    virtual OpalMediaStream * CreateMediaStream(
+      const OpalMediaFormat & mediaFormat, ///<  Media format for stream
+      unsigned sessionID,                  ///<  Session number for stream
+      PBoolean isSource                        ///<  Is a source stream
+    );
+
     /**Open source or sink media stream for session.
       */
     virtual OpalMediaStreamPtr OpenMediaStream(
@@ -228,12 +352,46 @@ class SIPConnection : public OpalRTPConnection
       bool isSource                        ///< Stream is a source/sink
     );
 
-    /**Request close of a specific media stream.
-       Note that this is usually asymchronous, the OnClosedMediaStream() function is
-       called when the stream is really closed.
+    /**Call back for closed a media stream.
+
+       The default behaviour calls the OpalEndPoint function of the same name.
       */
-    virtual bool CloseMediaStream(
-      OpalMediaStream & stream  ///< Stream to close
+    virtual void OnClosedMediaStream(
+      const OpalMediaStream & stream     ///<  Media stream being closed
+    );
+
+    /**Get transports for the media session on the connection.
+       This is primarily used by the media bypass feature controlled by the
+       OpalManager::GetMediaTransferMode() function. It allows one side of the
+       call to get the transport address of the media on the other side, so it
+       can pass it on, bypassing the local host.
+
+       @return true if a transport address is available and may be used to pass
+               on to a remote system for direct access.
+     */
+    virtual bool GetMediaTransportAddresses(
+      const OpalMediaType & mediaType,       ///< Media type for session to return information
+      OpalTransportAddressArray & transports ///<  Information on media session
+    ) const;
+
+    /**Call back when patching a media stream.
+       This function is called when a connection has created a new media
+       patch between two streams. This is usually called twice per media patch,
+       once for the source stream and once for the sink stream.
+
+       Note this is not called within the context of the patch thread and is
+       called before that thread has started.
+      */
+    virtual void OnPatchMediaStream(
+      PBoolean isSource,        ///< Is source/sink call
+      OpalMediaPatch & patch    ///<  New patch
+    );
+
+    /**Pause media streams for connection.
+      */
+    virtual void OnPauseMediaStream(
+      OpalMediaStream & strm,     ///< Media stream paused/un-paused
+      bool paused                 ///< Flag for pausing/un-pausing
     );
 
     /**Clean up the termination of the connection.
@@ -260,7 +418,7 @@ class SIPConnection : public OpalRTPConnection
        function when an application wishes to redirect an unwanted incoming
        call.
 
-       The return value is PTrue if the call is to be forwarded, PFalse 
+       The return value is true if the call is to be forwarded, false 
        otherwise. Note that if the call is forwarded, the current connection
        is cleared with the ended call code set to EndedByCallForwarded.
       */
@@ -275,6 +433,16 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual SendUserInputModes GetRealSendUserInputMode() const;
 
+    /**Send a user input indication to the remote endpoint.
+       This is for sending arbitrary strings as user indications.
+
+       The default behaviour is to call SendUserInputTone() for each character
+       in the string.
+      */
+    virtual PBoolean SendUserInputString(
+      const PString & value                   ///<  String value of indication
+    );
+
     /**Send a user input indication to the remote endpoint.
        This sends DTMF emulation user input. If something more sophisticated
        than the simple tones that can be sent using the SendUserInput()
@@ -292,17 +460,6 @@ class SIPConnection : public OpalRTPConnection
        The default behaviour sends the tone using RFC2833.
       */
     PBoolean SendUserInputTone(char tone, unsigned duration);
-
-    /**Callback from the RTP session for statistics monitoring.
-       This is called every so many packets on the transmitter and receiver
-       threads of the RTP session indicating that the statistics have been
-       updated.
-
-       The default behaviour does nothing.
-      */
-    virtual void OnRTPStatistics(
-      const RTP_Session & session         ///<  Session with statistics
-    ) const;
   //@}
 
   /**@name Protocol handling functions */
@@ -356,6 +513,10 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual void OnReceivedPING(SIP_PDU & pdu);
 
+    /**Handle an incoming PRACK PDU
+      */
+    virtual void OnReceivedPRACK(SIP_PDU & pdu);
+
     /**Handle an incoming BYE PDU
       */
     virtual void OnReceivedBYE(SIP_PDU & pdu);
@@ -368,7 +529,7 @@ class SIPConnection : public OpalRTPConnection
        Note this is called before th ACK is sent and thus should do as little as possible.
        All the hard work (SDP processing etc) should be in the usual OnReceivedResponse().
       */
-    virtual void OnReceivedResponseToINVITE(
+    virtual bool OnReceivedResponseToINVITE(
       SIPTransaction & transaction,
       SIP_PDU & response
     );
@@ -389,14 +550,14 @@ class SIPConnection : public OpalRTPConnection
   
     /**Handle an incoming Ringing response PDU
       */
-    virtual void OnReceivedRinging(SIP_PDU & pdu);
+    virtual void OnReceivedRinging(SIPTransaction & transaction, SIP_PDU & pdu);
   
     /**Handle an incoming Session Progress response PDU
       */
-    virtual void OnReceivedSessionProgress(SIP_PDU & pdu);
+    virtual void OnReceivedSessionProgress(SIPTransaction & transaction, SIP_PDU & pdu);
   
     /**Handle an incoming Proxy Authentication Required response PDU
-       Returns: PTrue if handled, if PFalse is returned connection is released.
+       Returns: true if handled, if false is returned connection is released.
       */
     virtual PBoolean OnReceivedAuthenticationRequired(
       SIPTransaction & transaction,
@@ -419,19 +580,6 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual void OnCreatingINVITE(SIPInvite & pdu);
 
-    /**Send a "200 OK" response for the received INVITE message.
-     */
-    virtual PBoolean SendInviteOK(const SDPSessionDescription & sdp);
-	
-    /**Send a response for the received INVITE message.
-     */
-    virtual PBoolean SendInviteResponse(
-      SIP_PDU::StatusCodes code,
-      const char * contact = NULL,
-      const char * extra = NULL,
-      const SDPSessionDescription * sdp = NULL
-    );
-
     enum TypeOfINVITE {
       IsNewINVITE,
       IsDuplicateINVITE,
@@ -443,92 +591,161 @@ class SIPConnection : public OpalRTPConnection
     TypeOfINVITE CheckINVITE(
       const SIP_PDU & pdu
     ) const;
+
+    /**Send an OPTIONS command within this calls dialog.
+       Note if \p reply is non-NULL, this function will block until the
+       transaction completes. Care must be executed in this case that
+       no deadlocks occur.
+      */
+    bool SendOPTIONS(
+      const SIPOptions::Params & params,  ///< Parameters for OPTIONS command
+      SIP_PDU * reply = NULL              ///< Reply to message
+    );
+
+    /**Send an INFO command within this calls dialog.
+       Note if \p reply is non-NULL, this function will block until the
+       transaction completes. Care must be executed in this case that
+       no deadlocks occur.
+      */
+    bool SendINFO(
+      const SIPInfo::Params & params,  ///< Parameters for OPTIONS command
+      SIP_PDU * reply = NULL              ///< Reply to message
+    );
   //@}
 
     OpalTransportAddress GetDefaultSDPConnectAddress(WORD port = 0) const;
 
-    OpalTransport & GetTransport() const { return *transport; }
-    bool SetTransport(OpalTransport * transport);
-
-    SIPEndPoint & GetEndPoint() const { return endpoint; }
-    SIPDialogContext & GetDialog() { return m_dialog; }
-    const SIPDialogContext & GetDialog() const { return m_dialog; }
+    SIPEndPoint & GetEndPoint() const { return SIPTransactionOwner::m_endpoint; }
     SIPAuthentication * GetAuthenticator() const { return m_authentication; }
 
+    /// Mode for reliable provisional responses.
+    enum PRACKMode {
+      e_prackDisabled,  /**< Do not use PRACK if remote asks for 100rel in Supported
+                             field, refuse call with 420 Bad Extension if 100rel is
+                             in Require header field. */
+                             
+      e_prackSupported, /** Add 100rel to Supported header in outgoing INVITE. For
+                            incoming INVITE enable PRACK is either Supported or
+                            Require headers include 100rel. */
+      e_prackRequired   /** Add 100rel to Require header in outgoing INVITE. For
+                            incoming INVITE enable PRACK is either Supported or
+                            Require headers include 100rel, fail the call with
+                            a 421 Extension Required if missing. */
+    };
+    /**Get active PRACK mode. See PRACKMode enum for details.
+      */
+    PRACKMode GetPRACKMode() const { return m_prackMode; }
+
+    /** Return a bit mask of the allowed local SIP methods.
+      */
+    virtual unsigned GetAllowedMethods() const;
+
+    /** REturn true if remote allows the method.
+     */
+    bool DoesRemoteAllowMethod(SIP_PDU::Methods method) const { return (m_allowedMethods&(1<<method)) != 0; }
+
 #if OPAL_VIDEO
     /**Call when SIP INFO of type application/media_control+xml is received.
 
-       Return PFalse if default reponse of Failure_UnsupportedMediaType is to be returned
+       Return false if default reponse of Failure_UnsupportedMediaType is to be returned
 
       */
     virtual PBoolean OnMediaControlXML(SIP_PDU & pdu);
 #endif
 
-    virtual void OnMediaCommand(OpalMediaCommand & note, INT extra);
+    /** Callback for media commands.
+        Calls the SendIntraFrameRequest on the rtp session
+
+       @returns true if command is handled.
+      */
+    virtual bool OnMediaCommand(
+      OpalMediaStream & stream,         ///< Stream command executed on
+      const OpalMediaCommand & command  ///< Media command being executed
+    );
+
+    // Overrides from SIPTransactionOwner
+    virtual PString GetAuthID() const;
+
 
     virtual void OnStartTransaction(SIPTransaction & transaction);
-
     virtual void OnReceivedMESSAGE(SIP_PDU & pdu);
-
-    P_REMOVE_VIRTUAL_VOID(OnMessageReceived(const SIPURL & /*from*/, const SIP_PDU & /*pdu*/));
-    P_REMOVE_VIRTUAL_VOID(OnMessageReceived(const SIP_PDU & /*pdu*/));
-
-#if OPAL_HAS_IM
-    virtual bool TransmitExternalIM(
-      const OpalMediaFormat & format, 
-      RTP_IMFrame & body
-    );
-#endif
+    virtual void OnReceivedSUBSCRIBE(SIP_PDU & pdu);
 
     PString GetLocalPartyURL() const;
 
-  protected:
-    PDECLARE_NOTIFIER(PTimer, SIPConnection, OnSessionTimeout);
-    PDECLARE_NOTIFIER(PTimer, SIPConnection, OnInviteResponseRetry);
-    PDECLARE_NOTIFIER(PTimer, SIPConnection, OnAckTimeout);
+    virtual void PrintOn(ostream & strm) const { OpalRTPConnection::PrintOn(strm); }
 
-    virtual bool OnSendSDP(
-      bool isAnswerSDP,
-      OpalRTPSessionManager & rtpSessions,
+  protected:
+    virtual bool GarbageCollection();
+    typedef SIPPoolTimer<SIPConnection> PoolTimer;
+    void OnSessionTimeout();
+    void OnInviteResponseRetry();
+    void OnInviteResponseTimeout();
+    void OnInviteCollision();
+
+    virtual bool OnSendOfferSDP(
+      SDPSessionDescription & sdpOut,
+      bool offerCurrentOnly
+    );
+    virtual bool OnSendOfferSDPSession(
+      unsigned sessionID,
+      SDPSessionDescription & sdpOut,
+      bool offerOpenMediaStreamOnly
+    );
+
+    virtual bool OnSendAnswerSDP(
       SDPSessionDescription & sdpOut
     );
-    virtual bool OfferSDPMediaDescription(
-      const OpalMediaType & mediaType,
-      unsigned sessionID,
-      OpalRTPSessionManager & rtpSessions,
-      SDPSessionDescription & sdpOut,
-      bool offerOpenMediaStreamOnly = false
+    virtual bool OnSendAnswerSDP(
+      const SDPSessionDescription & sdpOffer,
+      SDPSessionDescription & sdpAnswer
     );
-    virtual bool AnswerSDPMediaDescription(
+    virtual bool OnSendAnswerSDPSession(
       const SDPSessionDescription & sdpIn,
       unsigned sessionIndex,
       SDPSessionDescription & sdpOut
     );
 
-    virtual void OnReceivedSDP(
-      SIP_PDU & pdu
+    virtual void OnReceivedAnswerSDP(
+      SIP_PDU & response,
+      SIPTransaction * transaction
     );
-    virtual bool OnReceivedSDPMediaDescription(
+    virtual bool OnReceivedAnswerSDPSession(
       SDPSessionDescription & sdp,
-      unsigned sessionId
+      unsigned sessionId,
+      bool & multipleFormats
     );
 
     virtual OpalMediaSession * SetUpMediaSession(
       const unsigned rtpSessionId,
       const OpalMediaType & mediaType,
-      SDPMediaDescription * mediaDescription,
+      const SDPMediaDescription & mediaDescription,
       OpalTransportAddress & localAddress,
       bool & remoteChanged
     );
 
     bool SendReINVITE(PTRACE_PARAM(const char * msg));
-    void StartPendingReINVITE();
+    bool StartPendingReINVITE();
 
     friend class SIPInvite;
-    static PBoolean WriteINVITE(OpalTransport & transport, void * param);
-    bool WriteINVITE();
+    PDECLARE_WriteConnectCallback(SIPConnection, WriteINVITE);
+
+    virtual void SendDelayedACK(bool force);
+    void OnDelayedAckTimeout();
+
+    virtual bool SendInviteOK();
+    virtual PBoolean SendInviteResponse(
+      SIP_PDU::StatusCodes code,
+      const SDPSessionDescription * sdp = NULL
+    );
+    virtual void AdjustInviteResponse(
+      SIP_PDU & response
+    );
 
     void UpdateRemoteAddresses();
+#if OPAL_SRTP
+    virtual bool CanDoSRTP() const { return m_dialog.GetRemoteTransportAddress(m_dnsEntry).GetProtoPrefix() == OpalTransportAddress::TlsPrefix(); }
+#endif
 
     void NotifyDialogState(
       SIPDialogNotification::States state,
@@ -536,12 +753,11 @@ class SIPConnection : public OpalRTPConnection
       unsigned eventCode = 0
     );
 
+    virtual bool InviteConferenceParticipant(const PString & conf, const PString & dest);
 
     // Member variables
-    SIPEndPoint         & endpoint;
-    OpalTransport       * transport;
-    bool                  deleteTransport;
-    PStringList           m_allowedEvents;
+    unsigned              m_allowedMethods;
+    PStringSet            m_allowedEvents;
 
     enum HoldState {
       eHoldOff,
@@ -553,38 +769,41 @@ class SIPConnection : public OpalRTPConnection
     };
     HoldState             m_holdToRemote;
     bool                  m_holdFromRemote;
-    PString               forwardParty;
+    PString               m_forwardParty;
+    OpalTransportAddress  m_remoteAddress;
+    SIPURL                m_contactAddress;
+    SIPURL                m_ciscoRemotePartyID;
 
-    SIP_PDU             * originalInvite;
-    PTime                 originalInviteTime;
+    SIP_PDU             * m_lastReceivedINVITE;
+    SIP_PDU             * m_delayedAckInviteResponse;
+    PoolTimer             m_delayedAckTimer;
+    PTimeInterval         m_delayedAckTimeout;
+    SIP_PDU             * m_lastSentAck;
     time_t                m_sdpSessionId;
     unsigned              m_sdpVersion; // Really a sequence number
     bool                  m_needReINVITE;
     bool                  m_handlingINVITE;
+    bool                  m_resolveMultipleFormatReINVITE;
     bool                  m_symmetricOpenStream;
-    SIPDialogContext      m_dialog;
     OpalGloballyUniqueID  m_dialogNotifyId;
     int                   m_appearanceCode;
     PString               m_alertInfo;
-    SIPAuthentication   * m_authentication;
-    unsigned              m_authenticatedCseq;
-    PTimer                sessionTimer;
+    PoolTimer             m_sessionTimer;
 
     std::map<SIP_PDU::Methods, unsigned> m_lastRxCSeq;
 
-    PTimer                    ackTimer;
-    PTimer                    ackRetry;
-    unsigned                  ackRetryCount;
-    SIP_PDU                   ackPacket;
-    bool                      ackReceived;
-    bool                      m_referInProgress;
-    PSafeList<SIPTransaction> forkedInvitations; // Not for re-INVITE
-    PSafeList<SIPTransaction> pendingInvitations; // For re-INVITE
-    PSafeList<SIPTransaction> m_pendingTransactions;
+    PRACKMode      m_prackMode;
+    bool           m_prackEnabled;
+    unsigned       m_prackSequenceNumber;
+    std::queue<SIP_PDU> m_responsePackets;
+    PoolTimer      m_responseFailTimer;
+    PoolTimer      m_responseRetryTimer;
+    unsigned       m_responseRetryCount;
+    PoolTimer      m_inviteCollisionTimer;
 
-#if OPAL_FAX
-    bool m_switchedToFaxMode;
-#endif
+    bool                      m_referInProgress;
+    PSafeList<SIPTransaction> m_forkedInvitations; // Not for re-INVITE
+    PSafeList<SIPTransaction> m_pendingInvitations; // For re-INVITE
 
     enum {
       ReleaseWithBYE,
@@ -595,75 +814,34 @@ class SIPConnection : public OpalRTPConnection
 
     OpalMediaFormatList m_remoteFormatList;
     OpalMediaFormatList m_answerFormatList;
-    void SetRemoteMediaFormats(SDPSessionDescription * sdp);
+    bool SetRemoteMediaFormats(SIP_PDU * pdu);
+
+    std::map<std::string, SIP_PDU *> m_responses;
+
+#if OPAL_HAS_SIPIM
+    PSafePtr<OpalSIPIMContext> m_imContext;
+#endif
+
+#if OPAL_VIDEO
+    PSimpleTimer m_infoPictureFastUpdateTimer;
+#endif
+
+    enum {
+      UserInputMethodUnknown,
+      ReceivedRFC2833,
+      ReceivedINFO
+    } m_receivedUserInputMethod;
+    void OnUserInputInlineRFC2833(OpalRFC2833Info & info, INT type);
+
 
   private:
     P_REMOVE_VIRTUAL_VOID(OnCreatingINVITE(SIP_PDU&));
     P_REMOVE_VIRTUAL_VOID(OnReceivedTrying(SIP_PDU &));
+    P_REMOVE_VIRTUAL_VOID(OnMessageReceived(const SIPURL & /*from*/, const SIP_PDU & /*pdu*/));
+    P_REMOVE_VIRTUAL_VOID(OnMessageReceived(const SIP_PDU & /*pdu*/));
 
   friend class SIPTransaction;
-};
-
-
-/**This class is for encpsulating the IETF Real Time Protocol interface.
- */
-class SIP_RTP_Session : public RTP_UserData
-{
-  PCLASSINFO(SIP_RTP_Session, RTP_UserData);
-
-  /**@name Construction */
-  //@{
-    /**Create a new channel.
-     */
-    SIP_RTP_Session(
-      const SIPConnection & connection  ///<  Owner of the RTP session
-    );
-  //@}
-
-  /**@name Overrides from RTP_UserData */
-  //@{
-    /**Callback from the RTP session for transmit statistics monitoring.
-       This is called every RTP_Session::senderReportInterval packets on the
-       transmitter indicating that the statistics have been updated.
-
-       The default behaviour calls H323Connection::OnRTPStatistics().
-      */
-    virtual void OnTxStatistics(
-      const RTP_Session & session   ///<  Session with statistics
-    ) const;
-
-    /**Callback from the RTP session for receive statistics monitoring.
-       This is called every RTP_Session::receiverReportInterval packets on the
-       receiver indicating that the statistics have been updated.
-
-       The default behaviour calls H323Connection::OnRTPStatistics().
-      */
-    virtual void OnRxStatistics(
-      const RTP_Session & session   ///<  Session with statistics
-    ) const;
-
-#if OPAL_VIDEO
-    /**Callback from the RTP session after an IntraFrameRequest is receieved.
-       The default behaviour executes an OpalVideoUpdatePicture command on the
-       connection's source video stream if it exists.
-      */
-    virtual void OnRxIntraFrameRequest(
-      const RTP_Session & session   ///<  Session with statistics
-    ) const;
-
-    /**Callback from the RTP session after an IntraFrameRequest is sent.
-       The default behaviour does nothing.
-      */
-    virtual void OnTxIntraFrameRequest(
-      const RTP_Session & session   ///<  Session with statistics
-    ) const;
-#endif
-  //@}
-
-    virtual void SessionFailing(RTP_Session & /*session*/);
-
-  protected:
-    const SIPConnection & connection; /// Owner of the RTP session
+  friend class SIP_RTP_Session;
 };
 
 
