@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26270 $
+ * $Revision: 29947 $
  * $Author: rjongbloed $
- * $Date: 2011-08-03 03:29:01 -0500 (Wed, 03 Aug 2011) $
+ * $Date: 2013-06-13 05:59:17 -0500 (Thu, 13 Jun 2013) $
  */
 
 #ifndef OPAL_SIP_SIPEP_H
@@ -44,40 +44,10 @@
 
 #if OPAL_SIP
 
-#include <ptclib/threadpool.h>
-#include <opal/rtpep.h>
+#include <rtp/rtpep.h>
 #include <sip/sipcon.h>
-#include <sip/sippdu.h>
 #include <sip/handlers.h> 
 
-#if OPAL_HAS_SIPIM
-#include <im/sipim.h>
-#endif
-
-class SIPRegisterHandler;
-
-
-//
-//  provide flag so applications know if presence is available
-//
-#define OPAL_HAS_SIP_PRESENCE   1
-
-/////////////////////////////////////////////////////////////////////////
-
-/** Class to hold authentication information 
-  */
-
-class SIPAuthInfo : public PObject
-{
-  public:
-    SIPAuthInfo()
-    { }
-
-    SIPAuthInfo(const PString & u, const PString & p)
-    { username = u; password = p; }
-    PString username;
-    PString password;
-};
 
 /////////////////////////////////////////////////////////////////////////
 
@@ -85,8 +55,7 @@ class SIPAuthInfo : public PObject
  */
 class SIPEndPoint : public OpalRTPEndPoint
 {
-  PCLASSINFO(SIPEndPoint, OpalRTPEndPoint);
-
+    PCLASSINFO(SIPEndPoint, OpalRTPEndPoint);
   public:
   /**@name Construction */
   //@{
@@ -94,8 +63,7 @@ class SIPEndPoint : public OpalRTPEndPoint
      */
     SIPEndPoint(
       OpalManager & manager,
-      unsigned maxConnectionThreads = 10,
-      unsigned maxHandlerThreads = 5
+      unsigned maxThreads = 15
     );
 
     /**Destroy endpoint.
@@ -116,12 +84,23 @@ class SIPEndPoint : public OpalRTPEndPoint
       */
     virtual PString GetDefaultTransport() const;
 
-    /**Handle new incoming connection from listener.
+    /**Get the default signal port for this endpoint.
+     */
+    virtual WORD GetDefaultSignalPort() const;
 
-       The default behaviour does nothing.
+    /**Build a list of network accessible URIs given a user name.
+       This typically gets URI's like sip:user@interface, h323:user@interface
+       etc, for each listener of each endpoint.
       */
-    virtual PBoolean NewIncomingConnection(
-      OpalTransport * transport  ///<  Transport connection came in on
+    virtual PStringList GetNetworkURIs(
+      const PString & name
+    ) const;
+
+    /**Handle new incoming connection from listener.
+      */
+    virtual void NewIncomingConnection(
+      OpalListener & listener,            ///<  Listner that created transport
+      const OpalTransportPtr & transport  ///<  Transport connection came in on
     );
 
     /**Set up a connection to a remote party.
@@ -183,6 +162,19 @@ class SIPEndPoint : public OpalRTPEndPoint
       OpalConnection & connection   ///<  Connection that was established
     );
 
+    /**Call back when conferencing state information changes.
+       If a conferencing endpoint type detects a change in a conference nodes
+       state, as would be returned by GetConferenceStatus() then this function
+       will be called on all endpoints in the OpalManager.
+
+       The \p uri parameter is as is the internal URI for the conference.
+      */
+    virtual void OnConferenceStatusChanged(
+      OpalEndPoint & endpoint,  /// < Endpoint sending state change
+      const PString & uri,      ///< Internal URI of conference node that changed
+      OpalConferenceState::ChangeType change ///< Change that occurred
+    );
+
     /** Execute garbage collection for endpoint.
         Returns true if all garbage has been collected.
         Default behaviour deletes the objects in the connectionsActive list.
@@ -196,14 +188,7 @@ class SIPEndPoint : public OpalRTPEndPoint
        The default implementation is to create a OpalSIPConnection.
       */
     virtual SIPConnection * CreateConnection(
-      OpalCall & call,                         ///<  Owner of connection
-      const PString & token,                   ///<  token used to identify connection
-      void * userData,                         ///<  User data for connection
-      const SIPURL & destination,              ///<  Destination for outgoing call
-      OpalTransport * transport,               ///<  Transport INVITE has been received on
-      SIP_PDU * invite,                        ///<  Original INVITE pdu
-      unsigned int options = 0,                ///<  connection options
-      OpalConnection::StringOptions * stringOptions = NULL ///<  complex string options
+      const SIPConnection::Init & init     ///< Initialisation parameters
 
     );
     
@@ -245,29 +230,28 @@ class SIPEndPoint : public OpalRTPEndPoint
   /**@name Protocol handling routines */
   //@{
 
-    /**Creates an OpalTransport instance, based on the
-       address is interpreted as the remote address to which the transport should connect
+    /**Find or create an OpalTransport instance.
+       A single transport exists to a specific remote system and all SIP entities
+       (connections or handlers) use transports out of this pool.
       */
-    OpalTransport * CreateTransport(
-      const SIPURL & remoteURL,
-      const PString & localInterface = PString::Empty()
+    OpalTransportPtr GetTransport(
+      const SIPTransactionOwner & transactor,
+      SIP_PDU::StatusCodes & reason
     );
 
     virtual void HandlePDU(
-      OpalTransport & transport
+      const OpalTransportPtr & transport
     );
 
     /**Handle an incoming SIP PDU that has been full decoded
       */
-    virtual PBoolean OnReceivedPDU(
-      OpalTransport & transport,
+    virtual bool OnReceivedPDU(
       SIP_PDU * pdu
     );
 
     /**Handle an incoming SIP PDU not assigned to any connection
       */
     virtual bool OnReceivedConnectionlessPDU(
-      OpalTransport & transport, 
       SIP_PDU * pdu
     );
 
@@ -280,44 +264,39 @@ class SIPEndPoint : public OpalRTPEndPoint
 
     /**Handle an incoming INVITE request.
       */
-    virtual PBoolean OnReceivedINVITE(
-      OpalTransport & transport,
+    virtual bool OnReceivedINVITE(
       SIP_PDU * pdu
     );
 
     /**Handle an incoming NOTIFY PDU.
       */
-    virtual PBoolean OnReceivedNOTIFY(
-      OpalTransport & transport,
-      SIP_PDU & response
+    virtual bool OnReceivedNOTIFY(
+      SIP_PDU & request
     );
 
     /**Handle an incoming REGISTER PDU.
       */
-    virtual PBoolean OnReceivedREGISTER(
-      OpalTransport & transport, 
-      SIP_PDU & pdu
+    virtual bool OnReceivedREGISTER(
+      SIP_PDU & request
     );
 
     /**Handle an incoming SUBSCRIBE PDU.
       */
-    virtual PBoolean OnReceivedSUBSCRIBE(
-      OpalTransport & transport, 
-      SIP_PDU & pdu
+    virtual bool OnReceivedSUBSCRIBE(
+      SIP_PDU & request,
+      SIPDialogContext * dialog
     );
 
     /**Handle an incoming MESSAGE PDU.
       */
     virtual bool OnReceivedMESSAGE(
-      OpalTransport & transport,
-      SIP_PDU & response
+      SIP_PDU & request
     );
 
     /**Handle an incoming OPTIONS PDU.
       */
     virtual bool OnReceivedOPTIONS(
-      OpalTransport & transport,
-      SIP_PDU & response
+      SIP_PDU & request
     );
     
     /**Handle a SIP packet transaction failure
@@ -325,18 +304,6 @@ class SIPEndPoint : public OpalRTPEndPoint
     virtual void OnTransactionFailed(
       SIPTransaction & transaction
     );
-    
-    /**Callback from the RTP session for statistics monitoring.
-       This is called every so many packets on the transmitter and receiver
-       threads of the RTP session indicating that the statistics have been
-       updated.
-
-       The default behaviour does nothing.
-      */
-    virtual void OnRTPStatistics(
-      const SIPConnection & connection,  ///<  Connection for the channel
-      const RTP_Session & session         ///<  Session with statistics
-    ) const;
   //@}
  
 
@@ -393,7 +360,14 @@ class SIPEndPoint : public OpalRTPEndPoint
     bool Register(
       const SIPRegister::Params & params, ///< Registration parameters
       PString & aor,                      ///< Resultant address-of-record for unregister
-      bool asynchronous = true            ///< Wait for completion, may take some time
+      SIP_PDU::StatusCodes * reason = NULL ///< If not null, wait for completion, may take some time
+    );
+
+    // For backward compatibility
+    bool Register(
+      const SIPRegister::Params & params, ///< Registration parameters
+      PString & aor,                      ///< Resultant address-of-record for unregister
+      bool asynchronous                   ///< Wait for completion, may take some time
     );
 
     /// Registration function for backward compatibility.
@@ -466,6 +440,14 @@ class SIPEndPoint : public OpalRTPEndPoint
       void               * m_userData;          ///< User data corresponding to this registration
     };
 
+    /** Get current registration status.
+        Returns false if there is no registration that match the aor/call-id
+      */
+    bool GetRegistrationStatus(
+      const PString & token,          /// Address-of-record, or call-id
+      RegistrationStatus & status     ///< Returned status
+    );
+
     /**Callback called when a registration to a SIP registrar status.
      */
     virtual void OnRegistrationStatus(
@@ -616,6 +598,15 @@ class SIPEndPoint : public OpalRTPEndPoint
     /** Information provided on the subscription status. */
     typedef SIPSubscribe::SubscriptionStatus SubscriptionStatus;
 
+    /** Get current registration status.
+        Returns false if there is no registration that match the aor/call-id
+      */
+    bool GetSubscriptionStatus(
+      const PString & token,          /// Address-of-record, or call-id
+      const PString & eventPackage,   ///< Event package being unsubscribed
+      SubscriptionStatus & status     ///< Returned status
+    );
+
     /**Callback called when a subscription to a SIP UA status changes.
      */
     virtual void OnSubscriptionStatus(
@@ -641,12 +632,29 @@ class SIPEndPoint : public OpalRTPEndPoint
       SIP_PDU::StatusCodes reason    ///< Status of subscription
     );
 
+    /** Indicate notification capability for an event package.
+        Default behaviour handles conference and presence event packages if the
+        \p aor parameter matches something we have.
+
+        All other event packages are checked against the m_allowedEvents set
+        member variable.
+      */
+    enum CanNotifyResult {
+      CannotNotify,       ///< Refuse SUBSCRIBE
+      CanNotifyImmediate, ///< Accept SUBSCRIBE and send NOTIFY
+      CanNotifyDeferrred  ///> Accept SUBSCRIBE, but NOTIFY will be sent at a later time by caller
+    };
+
     /** Indicate notifications for the specified event package are supported.
       */
-    virtual bool CanNotify(
-      const PString & eventPackage ///< Event package we support
+    virtual CanNotifyResult CanNotify(
+      const PString & eventPackage, ///< Event package we support
+      const SIPURL & aor            ///< Address of record for SUBSCRIBE
     );
 
+    // For backward compatobility
+    virtual bool CanNotify(const PString & eventPackage);
+
     /** Send notification to all remotes that are subcribed to the event package.
       */
     bool Notify(
@@ -667,12 +675,13 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
 
 
-    /**Send IM text
+    /**Callback called when reg NOTIFY message is received
      */
-    virtual PBoolean Message(
-      OpalIM & message
+    virtual void OnRegInfoReceived(
+      const SIPRegNotification & info  ///< Information on dialog state change
     );
 
+
     /**Send SIP message
      */
     bool SendMESSAGE(
@@ -686,6 +695,32 @@ class SIPEndPoint : public OpalRTPEndPoint
       SIP_PDU::StatusCodes reason
     );
 
+    struct ConnectionlessMessageInfo {
+      ConnectionlessMessageInfo(SIP_PDU & pdu)
+        : m_pdu(pdu)
+        , m_transport(*pdu.GetTransport())
+        , m_status(ResponseSent)
+      { }
+
+      SIP_PDU & m_pdu;
+      OpalTransport & m_transport;
+      enum {
+        NotHandled,
+        SendOK,
+        MethodNotAllowed,
+        ResponseSent
+      } m_status;
+    };
+
+    typedef PNotifierTemplate<ConnectionlessMessageInfo &> ConnectionlessMessageNotifier;
+    #define PDECLARE_ConnectionlessMessageNotifier(cls, fn) PDECLARE_NOTIFIER2(SIPEndPoint, cls, fn, SIPEndPoint::ConnectionlessMessageInfo &)
+    #define PCREATE_ConnectionlessMessageNotifier(fn) PCREATE_NOTIFIER2(fn, SIPEndPoint::ConnectionlessMessageInfo &)
+
+    void SetConnectionlessMessageNotifier(
+      const ConnectionlessMessageNotifier & notifier
+    )
+    { m_onConnectionlessMessage = notifier; }
+
 
     /**Send SIP OPTIONS
      */
@@ -703,7 +738,7 @@ class SIPEndPoint : public OpalRTPEndPoint
 
 
     /**Publish new state information.
-       A expire time of zero will cease to automatically update the publish.
+       An empty body, or expire time of zero, will cease to automatically update the publish.
      */
     bool Publish(
       const SIPSubscribe::Params & params,
@@ -724,6 +759,7 @@ class SIPEndPoint : public OpalRTPEndPoint
     ) const { return activeSIPHandlers.GetAddresses(includeOffline, SIP_PDU::Method_PUBLISH, eventPackage); }
 
 
+#if OPAL_SIP_PRESENCE
     /**Publish new state information.
      * Only the basic & note fields of the PIDF+xml are supported for now.
      */
@@ -742,6 +778,7 @@ class SIPEndPoint : public OpalRTPEndPoint
       const PString & basic,
       const PString & note
     );
+#endif // OPAL_SIP_PRESENCE
 
 
     /**Send a SIP PING to the remote host
@@ -750,6 +787,10 @@ class SIPEndPoint : public OpalRTPEndPoint
       const PURL & to
     );
 
+    /** Get the allowed events for SUBSCRIBE commands.
+      */
+    const PStringSet & GetAllowedEvents() const { return m_allowedEvents; }
+
     /**Get default mode for PRACK support.
       */
     SIPConnection::PRACKMode GetDefaultPRACKMode() const { return m_defaultPrackMode; }
@@ -758,8 +799,8 @@ class SIPEndPoint : public OpalRTPEndPoint
       */
     void SetDefaultPRACKMode(SIPConnection::PRACKMode mode) { m_defaultPrackMode = mode; }
 
-    void SetMIMEForm(PBoolean v) { mimeForm = v; }
-    PBoolean GetMIMEForm() const { return mimeForm; }
+    void SetMaxPacketSizeUDP(PINDEX sz) { m_maxPacketSizeUDP = sz; }
+    PINDEX GetMaxPacketSizeUDP() const { return m_maxPacketSizeUDP; }
 
     void SetMaxRetries(unsigned r) { maxRetries = r; }
     unsigned GetMaxRetries() const { return maxRetries; }
@@ -806,43 +847,60 @@ class SIPEndPoint : public OpalRTPEndPoint
     ) { notifierTimeToLive = t; }
     const PTimeInterval & GetNotifierTimeToLive() const { return notifierTimeToLive; }
     
-    void SetNATBindingTimeout(
+    P_DEPRECATED void SetNATBindingTimeout(
       const PTimeInterval & t
-    ) { natBindingTimeout = t; natBindingTimer.RunContinuous (natBindingTimeout); }
-    const PTimeInterval & GetNATBindingTimeout() const { return natBindingTimeout; }
+    ) { m_keepAliveTimeout = t; }
+    P_DEPRECATED const PTimeInterval & GetNATBindingTimeout() const { return m_keepAliveTimeout; }
+
+    /**Keep Alive Method
+     */
+    enum KeepAliveType {
+      NoKeepAlive,
+      KeepAliveByCRLF,
+      KeepAliveByOPTION,
+      NumMethods
+    };
+    void GetKeepAlive(
+      PTimeInterval & timeout,
+      KeepAliveType & type
+    ) { timeout = m_keepAliveTimeout; type = m_keepAliveType; }
+    void SetKeepAlive(
+      const PTimeInterval & timeout,
+      KeepAliveType type
+    ) { m_keepAliveTimeout = timeout; m_keepAliveType = type; }
+
 
     void AddTransaction(
       SIPTransaction * transaction
-    ) { transactions.SetAt(transaction->GetTransactionID(), transaction); }
+    ) { m_transactions.Append(transaction); }
 
     PSafePtr<SIPTransaction> GetTransaction(const PString & transactionID, PSafetyMode mode = PSafeReadWrite)
-    { return transactions.FindWithLock(transactionID, mode); }
+    { return PSafePtrCast<SIPTransactionBase, SIPTransaction>(m_transactions.FindWithLock(transactionID, mode)); }
     
     /**Return the next CSEQ for the next transaction.
      */
-    unsigned GetNextCSeq() { return ++lastSentCSeq; }
+    unsigned GetNextCSeq() { return ++m_lastSentCSeq; }
+
+    /**Set registration search mode.
+       If true then only the user indicated as "local" address
+       (e.g. from OPAL_OPT_CALLING_PARTY_NAME etc) by the will be used
+       in searches of the registrations. No default to first user of the
+       same domain will be performed. */
+    void SetRegisteredUserMode(bool v) { m_registeredUserMode = v; }
+    bool GetRegisteredUserMode() const { return m_registeredUserMode; }
 
     /**Return the SIPAuthentication for a specific realm.
      */
-    bool GetAuthentication(const PString & authRealm, PString & user, PString & password); 
+    bool GetAuthentication(
+      const PString & realm,
+      PString & authId,
+      PString & password
+    );
     
-    /**Return the registered proxy URL for the given host.
+    /**Return the default Contact URL.
      */
-    virtual SIPURL GetRegisteredProxy(const SIPURL & remoteURL);
-
-    /**Return the registered party name URL for the given host.
-     *
-     * That URL can be used in the FORM field of the PDU's. 
-     * The host part can be different from the registration domain.
-     */
-    virtual SIPURL GetRegisteredPartyName(const SIPURL & remoteURL, const OpalTransport & transport);
-
-
-    /**Return the default registered party name URL.
-     */
-    virtual SIPURL GetDefaultRegisteredPartyName(const OpalTransport & transport);
+    virtual SIPURL GetDefaultLocalURL(const OpalTransport & transport);
     
-
     /**Adjust the outgoing PDU to registered information.
        Various header fields of the PDU must be adjusted to agree with values
        provided to/from the active registration for the domain the call is being
@@ -852,35 +910,20 @@ class SIPEndPoint : public OpalRTPEndPoint
        the given transport is set to the Contact field.
      */
     void AdjustToRegistration(
-      const OpalTransport & transport,
-      SIP_PDU & pdu
+      SIP_PDU & pdu,
+      SIPConnection * connection = NULL,
+      const OpalTransport * transport = NULL
     );
 
-    /**Return the local URL for the given transport and user name.
-     * That URL can be used as via address, and as contact field in outgoing
-     * requests.
-     *
-     * The URL is translated if required.
-     *
-     * If the transport is not running, the first listener transport
-     * will be used, if any.
-     */
-    virtual SIPURL GetLocalURL(
-       const OpalTransport & transport,             ///< Transport on which we can receive new requests
-       const PString & userName = PString::Empty()  ///< The user name part of the contact field
-    );
-    
-    
+
     /**Return the outbound proxy URL, if any.
      */
-    const SIPURL & GetProxy() const { return proxy; }
+    const SIPURL & GetProxy() const { return m_proxy; }
 
-    
     /**Set the outbound proxy URL.
      */
     void SetProxy(const SIPURL & url);
     
-    
     /**Set the outbound proxy URL.
      */
     void SetProxy(
@@ -908,7 +951,7 @@ class SIPEndPoint : public OpalRTPEndPoint
         
     /**Set the User Agent for the endpoint.
      */
-    void SetUserAgent(const PString & str) { userAgentString = str; }
+    void SetUserAgent(const PString & str) { m_userAgentString = str; }
 
 
     /** Return a bit mask of the allowed SIP methods.
@@ -916,36 +959,35 @@ class SIPEndPoint : public OpalRTPEndPoint
     virtual unsigned GetAllowedMethods() const;
 
 
-    /**NAT Binding Refresh Method
-     */
-    enum NATBindingRefreshMethod{
-      None,
-      Options,
-      EmptyRequest,
-      NumMethods
-    };
-
-
-    /**Set the NAT Binding Refresh Method
-     */
-    void SetNATBindingRefreshMethod(const NATBindingRefreshMethod m) { natMethod = m; }
+    typedef KeepAliveType NATBindingRefreshMethod;
+    P_DEPRECATED void SetNATBindingRefreshMethod(const NATBindingRefreshMethod type) { SetKeepAlive(m_keepAliveTimeout, type); }
 
     virtual SIPRegisterHandler * CreateRegisterHandler(const SIPRegister::Params & params);
 
     virtual void OnStartTransaction(SIPConnection & conn, SIPTransaction & transaction);
 
-    void UpdateHandlerIndexes(SIPHandler * handler) { activeSIPHandlers.Update(handler); }
+
+    PSafePtr<SIPHandler> FindSIPHandlerByCallID(const PString & callID, PSafetyMode m)
+      { return activeSIPHandlers.FindSIPHandlerByCallID(callID, m); }
+
+    void UpdateHandlerIndexes(SIPHandler * handler)
+      { activeSIPHandlers.Update(handler); }
+
+
+    SIPThreadPool & GetThreadPool() { return m_threadPool; }
+
 
   protected:
-    PDECLARE_NOTIFIER(PThread, SIPEndPoint, TransportThreadMain);
-    PDECLARE_NOTIFIER(PTimer, SIPEndPoint, NATBindingRefresh);
+    void AddTransport(const OpalTransportPtr & transport);
+    void TransportThreadMain(OpalTransportPtr transport);
 
-    SIPURL        proxy;
-    PString       userAgentString;
+    SIPURL        m_proxy;
+    PString       m_userAgentString;
+    PStringSet    m_allowedEvents;
 
     SIPConnection::PRACKMode m_defaultPrackMode;
 
-    bool          mimeForm;
+    PINDEX        m_maxPacketSizeUDP;
     unsigned      maxRetries;
     PTimeInterval retryTimeoutMin;   // T1
     PTimeInterval retryTimeoutMax;   // T2
@@ -957,67 +999,43 @@ class SIPEndPoint : public OpalRTPEndPoint
     PTimeInterval registrarTimeToLive;
     PTimeInterval notifierTimeToLive;
     PTimeInterval natBindingTimeout;
+    PTimeInterval m_keepAliveTimeout;
+    KeepAliveType m_keepAliveType;
+    bool          m_registeredUserMode;
+    bool          m_shuttingDown;
 
-    bool              m_shuttingDown;
+    // Transport management
+    PSafeDictionary<OpalTransportAddress, OpalTransport> m_transportsTable;
+
+    // Sub-protocol handlers
     SIPHandlersList   activeSIPHandlers;
+    PSafePtr<SIPHandler> FindHandlerByPDU(const SIP_PDU & pdu, PSafetyMode mode);
+
     PStringToString   m_receivedConnectionTokens;
+    PMutex            m_receivedConnectionMutex;
 
-    PSafeDictionary<PString, SIPTransaction> transactions;
+    PSafeSortedList<SIPTransactionBase> m_transactions;
 
-    PTimer                  natBindingTimer;
-    NATBindingRefreshMethod natMethod;
-    PAtomicInteger          lastSentCSeq;
-    int                     m_defaultAppearanceCode;
+    PAtomicInteger m_lastSentCSeq;
+    int            m_defaultAppearanceCode;
 
-    std::map<PString, PSyncPoint> m_registrationComplete;
+    struct RegistrationCompletion {
+      PSyncPoint           m_sync;
+      SIP_PDU::StatusCodes m_reason;
+      RegistrationCompletion() : m_reason(SIP_PDU::Information_Trying) { }
+    };
+    std::map<PString, RegistrationCompletion> m_registrationComplete;
 
+    ConnectionlessMessageNotifier m_onConnectionlessMessage;
+    typedef std::multimap<PString, SIPURL> ConferenceMap;
+    ConferenceMap m_conferenceAOR;
 
     // Thread pooling
-    class SIP_Work
-    {
-      public:
-        SIP_Work(SIPEndPoint & ep, SIP_PDU * pdu, const PString & token);
-        virtual ~SIP_Work();
-
-        virtual void Work();
-
-        SIPEndPoint & m_endpoint;
-        SIP_PDU     * m_pdu;
-        PString       m_token;
-    };
-
-    class WorkThreadPool : public PQueuedThreadPool<SIP_Work>
-    {
-      public:
-        WorkThreadPool(unsigned maxWorkers)
-          : PQueuedThreadPool<SIP_Work>(maxWorkers)
-        { }
-        virtual WorkerThreadBase * CreateWorkerThread();
-    } m_connectionThreadPool, m_handlerThreadPool;
-
+    SIPThreadPool m_threadPool;
 
     // Network interface checking
-    enum {
-      HighPriority = 80,
-      LowPriority  = 30,
-    };
-    class InterfaceMonitor : public PInterfaceMonitorClient
-    {
-        PCLASSINFO(InterfaceMonitor, PInterfaceMonitorClient);
-      public:
-        InterfaceMonitor(SIPEndPoint & manager, PINDEX priority);
-        
-        virtual void OnAddInterface(const PIPSocket::InterfaceEntry & entry);
-        virtual void OnRemoveInterface(const PIPSocket::InterfaceEntry & entry);
-
-      protected:
-        SIPEndPoint & m_endpoint;
-    };
-    InterfaceMonitor m_highPriorityMonitor;
-    InterfaceMonitor m_lowPriorityMonitor;
-
-    friend void InterfaceMonitor::OnAddInterface(const PIPSocket::InterfaceEntry & entry);
-    friend void InterfaceMonitor::OnRemoveInterface(const PIPSocket::InterfaceEntry & entry);
+    PDECLARE_InterfaceNotifier(SIPEndPoint, OnHighPriorityInterfaceChange);
+    PDECLARE_InterfaceNotifier(SIPEndPoint, OnLowPriorityInterfaceChange);
 
     bool m_disableTrying;
 
@@ -1025,29 +1043,15 @@ class SIPEndPoint : public OpalRTPEndPoint
     P_REMOVE_VIRTUAL_VOID(OnReceivedAuthenticationRequired(SIPTransaction &, SIP_PDU &));
     P_REMOVE_VIRTUAL_VOID(OnReceivedOK(SIPTransaction &, SIP_PDU &));
     P_REMOVE_VIRTUAL_VOID(OnMessageFailed(const SIPURL &, SIP_PDU::StatusCodes));
-
-  public:
-    struct ConnectionlessMessageInfo {
-      ConnectionlessMessageInfo(OpalTransport & transport, SIP_PDU & pdu)
-        : m_pdu(pdu), m_transport(transport), m_status(true)
-      { }
-
-      SIP_PDU & m_pdu;
-      OpalTransport & m_transport;
-      bool m_status;
-    };
-
-    typedef PNotifierTemplate<ConnectionlessMessageInfo &> ConnectionlessMessageNotifier;
-    #define PDECLARE_ConnectionlessMessageNotifier(cls, fn) PDECLARE_NOTIFIER2(SIPEndPoint, cls, fn, SIPEndPoint::ConnectionlessMessageInfo &)
-    #define PCREATE_ConnectionlessMessageNotifier(fn) PCREATE_NOTIFIER2(fn, SIPEndPoint::ConnectionlessMessageInfo &)
-
-    void SetConnectionlessMessageNotifier(
-      const ConnectionlessMessageNotifier & notifier
-    )
-    { m_onConnectionlessMessage = notifier; }
-
-  protected:
-    ConnectionlessMessageNotifier m_onConnectionlessMessage;
+    P_REMOVE_VIRTUAL(SIPConnection *,CreateConnection(OpalCall &, const PString &, void *, const SIPURL &, OpalTransport *, SIP_PDU *, unsigned, OpalConnection::StringOptions *), NULL);
+    P_REMOVE_VIRTUAL(PBoolean, OnReceivedPDU(OpalTransport &, SIP_PDU *), false);
+    P_REMOVE_VIRTUAL(bool, OnReceivedConnectionlessPDU(OpalTransport &,  SIP_PDU *), false);
+    P_REMOVE_VIRTUAL(PBoolean, OnReceivedINVITE(OpalTransport &, SIP_PDU *), false);
+    P_REMOVE_VIRTUAL(PBoolean, OnReceivedNOTIFY(OpalTransport &, SIP_PDU &), false);
+    P_REMOVE_VIRTUAL(PBoolean, OnReceivedREGISTER(OpalTransport &, SIP_PDU &), false);
+    P_REMOVE_VIRTUAL(PBoolean, OnReceivedSUBSCRIBE(OpalTransport &, SIP_PDU &, SIPDialogContext *), false);
+    P_REMOVE_VIRTUAL(bool, OnReceivedMESSAGE(OpalTransport &, SIP_PDU &), false);
+    P_REMOVE_VIRTUAL(bool, OnReceivedOPTIONS(OpalTransport &, SIP_PDU &), false);
 };
 
 
