@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25928 $
+ * $Revision: 29727 $
  * $Author: rjongbloed $
- * $Date: 2011-05-30 22:13:33 -0500 (Mon, 30 May 2011) $
+ * $Date: 2013-05-15 21:11:54 -0500 (Wed, 15 May 2013) $
  */
 
 #ifndef OPAL_SIP_SIPPRES_H
@@ -33,12 +33,14 @@
 
 #include <ptlib.h>
 #include <opal/buildopts.h>
-#include <sip/sipep.h>
 
-#if P_EXPAT && OPAL_SIP
+
+#if OPAL_SIP_PRESENCE
 
 #include <opal/pres_ent.h>
 #include <ptclib/pxml.h>
+#include <sip/sipep.h>
+#include <ptlib/notifier_ext.h>
 
 
 class XCAPClient : public PHTTPClient
@@ -211,7 +213,7 @@ class SIPWatcherInfoCommand : public OpalPresentityCommand {
 
 /** Class of the SIP presence systems.
   */
-class SIP_Presentity : public OpalPresentityWithCommandThread
+class SIP_Presentity : public OpalPresentityWithCommandThread, public PValidatedNotifierTarget
 {
     PCLASSINFO(SIP_Presentity, OpalPresentityWithCommandThread);
 
@@ -233,6 +235,7 @@ class SIP_Presentity : public OpalPresentityWithCommandThread
     static const PCaselessString & PIDFEntityKey();
     static const PCaselessString & SubProtocolKey();
     static const PCaselessString & PresenceAgentKey();
+    static const PCaselessString & TransportKey();
     static const PCaselessString & XcapRootKey();
     static const PCaselessString & XcapAuthIdKey();
     static const PCaselessString & XcapPasswordKey();
@@ -244,7 +247,6 @@ class SIP_Presentity : public OpalPresentityWithCommandThread
     virtual PStringArray GetAttributeTypes() const;
 
     virtual bool Open();
-    virtual bool IsOpen() const;
     virtual bool Close();
     virtual BuddyStatus GetBuddyListEx(BuddyList & buddies);
     virtual BuddyStatus SetBuddyListEx(const BuddyList & buddies);
@@ -264,10 +266,10 @@ class SIP_Presentity : public OpalPresentityWithCommandThread
     unsigned GetExpiryTime() const;
 
   protected:
-    PDECLARE_NOTIFIER2(SIPSubscribeHandler, SIP_Presentity, OnPresenceSubscriptionStatus, const SIPSubscribe::SubscriptionStatus &);
-    PDECLARE_NOTIFIER2(SIPSubscribeHandler, SIP_Presentity, OnPresenceNotify, SIPSubscribe::NotifyCallbackInfo &);
-    PDECLARE_NOTIFIER2(SIPSubscribeHandler, SIP_Presentity, OnWatcherInfoSubscriptionStatus, const SIPSubscribe::SubscriptionStatus &);
-    PDECLARE_NOTIFIER2(SIPSubscribeHandler, SIP_Presentity, OnWatcherInfoNotify, SIPSubscribe::NotifyCallbackInfo &);
+    PDECLARE_VALIDATED_NOTIFIER2(SIPSubscribeHandler, SIP_Presentity, OnPresenceSubscriptionStatus, const SIPSubscribe::SubscriptionStatus &);
+    PDECLARE_VALIDATED_NOTIFIER2(SIPSubscribeHandler, SIP_Presentity, OnPresenceNotify, SIPSubscribe::NotifyCallbackInfo &);
+    PDECLARE_VALIDATED_NOTIFIER2(SIPSubscribeHandler, SIP_Presentity, OnWatcherInfoSubscriptionStatus, const SIPSubscribe::SubscriptionStatus &);
+    PDECLARE_VALIDATED_NOTIFIER2(SIPSubscribeHandler, SIP_Presentity, OnWatcherInfoNotify, SIPSubscribe::NotifyCallbackInfo &);
     void OnReceivedWatcherStatus(PXMLElement * watcher);
     void SetPIDFEntity(PURL & entity);
     bool ChangeAuthNode(XCAPClient & xcap, const OpalAuthorisationRequestCommand & cmd);
@@ -278,12 +280,13 @@ class SIP_Presentity : public OpalPresentityWithCommandThread
       const PString & listName = PString::Empty()
     );
 
-    SIPEndPoint           * m_endpoint;
-    SubProtocol             m_subProtocol;
-    PIPSocketAddressAndPort m_presenceAgent;
-    PString                 m_watcherSubscriptionAOR;
-    int                     m_watcherInfoVersion;
-    PString                 m_publishedTupleId;
+    SIPEndPoint * m_endpoint;
+    SubProtocol   m_subProtocol;
+    SIPURL        m_presenceAgentURL;
+    PURL          m_defaultRootURL;
+    PString       m_watcherSubscriptionAOR;
+    int           m_watcherInfoVersion;
+    PString       m_publishedTupleId;
 
     typedef std::map<PString, PString> StringMap;
     StringMap m_watcherAorById;
@@ -296,6 +299,6 @@ class SIP_Presentity : public OpalPresentityWithCommandThread
 };
 
 
-#endif // P_EXPAT && OPAL_SIP
+#endif // OPAL_SIP_PRESENCE
 
 #endif // OPAL_SIP_SIPPRES_H
