@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25336 $
+ * $Revision: 29550 $
  * $Author: rjongbloed $
- * $Date: 2011-03-16 03:51:39 -0500 (Wed, 16 Mar 2011) $
+ * $Date: 2013-04-22 22:08:10 -0500 (Mon, 22 Apr 2013) $
  */
 
 #ifndef OPAL_T38_T38PROTO_H
@@ -37,26 +37,26 @@
 
 #include <opal/buildopts.h>
 
-
 #if OPAL_FAX
 
-#include <ptlib/pipechan.h>
-
 #include <opal/mediafmt.h>
 #include <opal/mediastrm.h>
-#include <opal/localep.h>
+#include <opal/mediasession.h>
+#include <ep/localep.h>
 
 
 class OpalTransport;
-class T38_IFPPacket;
-class PASN_OctetString;
 class OpalFaxConnection;
 
+#if OPAL_PTLIB_ASN
+class T38_IFPPacket;
+class PASN_OctetString;
+#endif
 
 #define OPAL_OPT_STATION_ID  "Station-Id"      ///< String option for fax station ID string
+#define OPAL_OPT_HEADER_INFO "Header-Info"     ///< String option for transmitted fax page header
 #define OPAL_NO_G111_FAX     "No-G711-Fax"     ///< Suppress G.711 fall back
-#define OPAL_SUPPRESS_CED    "Suppress-CED"    ///< Suppress transmission of CED tone
-#define OPAL_IGNORE_CED      "Ignore-CED"      ///< Ignore receipt of CED tone
+#define OPAL_SWITCH_ON_CED   "Switch-On-CED"   ///< Try switch to T.38 on receipt of CED tone
 #define OPAL_T38_SWITCH_TIME "T38-Switch-Time" ///< Seconds for fail safe switch to T.38 mode
 
 #define OPAL_FAX_TIFF_FILE "TIFF-File"
@@ -206,19 +206,19 @@ class OpalFaxConnection : public OpalLocalConnection
   //@{
     virtual PString GetPrefixName() const;
 
-    virtual void OnApplyStringOptions();
     virtual OpalMediaFormatList GetMediaFormats() const;
     virtual void AdjustMediaFormats(bool local, const OpalConnection * otherConnection, OpalMediaFormatList & mediaFormats) const;
-    virtual void AcceptIncoming();
     virtual void OnEstablished();
     virtual void OnReleased();
     virtual OpalMediaStream * CreateMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, PBoolean isSource);
     virtual void OnStartMediaPatch(OpalMediaPatch & patch);
-    virtual void OnStopMediaPatch(OpalMediaPatch & patch);
+    virtual void OnClosedMediaStream(const OpalMediaStream & stream);
     virtual PBoolean SendUserInputTone(char tone, unsigned duration);
     virtual void OnUserInputTone(char tone, unsigned duration);
-    virtual bool SwitchFaxMediaStreams(bool enableFax);
-    virtual void OnSwitchedFaxMediaStreams(bool enabledFax);
+    virtual bool SwitchFaxMediaStreams(bool toT38);
+    virtual void OnSwitchedFaxMediaStreams(bool toT38, bool success);
+    virtual bool OnSwitchingFaxMediaStreams(bool toT38);
+    virtual void OnApplyStringOptions();
   //@}
 
   /**@name New operations */
@@ -230,13 +230,11 @@ class OpalFaxConnection : public OpalLocalConnection
       bool failed   ///< Fax ended with failure
     );
 
-#if OPAL_STATISTICS
     /**Get fax transmission/receipt statistics.
       */
     virtual void GetStatistics(
       OpalMediaStatistics & statistics  ///< Statistics for call
     ) const;
-#endif
 
     /**Get the file to send/receive
       */
@@ -248,36 +246,111 @@ class OpalFaxConnection : public OpalLocalConnection
   //@}
 
   protected:
-    PDECLARE_NOTIFIER(PTimer,  OpalFaxConnection, OnSendCNGCED);
-    PDECLARE_NOTIFIER(PThread, OpalFaxConnection, OpenFaxStreams);
+    PDECLARE_NOTIFIER(PTimer,  OpalFaxConnection, OnSwitchTimeout);
+    void InternalOpenFaxStreams();
+    void InternalOnFaxCompleted();
 
+    void SetFaxMediaFormatOptions(OpalMediaFormat & mediaFormat) const;
 
     OpalFaxEndPoint & m_endpoint;
     PString           m_filename;
     bool              m_receiving;
-    PString           m_stationId;
     bool              m_disableT38;
-    PTimeInterval     m_releaseTimeout;
-    PTimeInterval     m_switchTimeout;
+    unsigned          m_switchTime;
     OpalMediaFormat   m_tiffFileFormat;
-#if OPAL_STATISTICS
-    void InternalGetStatistics(OpalMediaStatistics & statistics, bool terminate) const;
+
+    PTimer m_switchTimer;
+
     OpalMediaStatistics m_finalStatistics;
-#endif
-
-    enum {
-      e_AwaitingSwitchToT38,
-      e_SwitchingToT38,
-      e_CompletedSwitch
-    } m_state;
-    PTimer   m_faxTimer;
-
-  friend class OpalFaxMediaStream;
+    PAtomicBoolean      m_completed;
 };
 
 
 typedef OpalFaxConnection OpalT38Connection; // For backward compatibility
 
+class T38_UDPTLPacket;
+
+class OpalFaxSession : public OpalMediaSession
+{
+  public:
+    static const PCaselessString & UDPTL();
+
+    OpalFaxSession(const Init & init);
+    ~OpalFaxSession();
+
+    virtual const PCaselessString & GetSessionType() const { return UDPTL(); }
+    virtual bool Open(const PString & localInterface, const OpalTransportAddress & remoteAddress, bool isMediaAddress);
+    virtual bool IsOpen() const;
+    virtual bool Close();
+    virtual OpalTransportAddress GetLocalAddress(bool isMediaAddress = true) const;
+    virtual OpalTransportAddress GetRemoteAddress(bool isMediaAddress = true) const;
+    virtual bool SetRemoteAddress(const OpalTransportAddress & remoteAddress, bool isMediaAddress = true);
+
+    virtual void AttachTransport(Transport & transport);
+    virtual Transport DetachTransport();
+
+    virtual OpalMediaStream * CreateMediaStream(
+      const OpalMediaFormat & mediaFormat, 
+      unsigned sessionID, 
+      bool isSource
+    );
+
+    bool WriteData(RTP_DataFrame & frame);
+    bool ReadData(RTP_DataFrame & frame);
+
+    void ApplyMediaOptions(const OpalMediaFormat & mediaFormat);
+
+  protected:
+    void SetFrameFromIFP(RTP_DataFrame & frame, const PASN_OctetString & ifp, unsigned sequenceNumber);
+    void DecrementSentPacketRedundancy(bool stripRedundancy);
+    bool WriteUDPTL();
+
+    Transport          m_savedTransport;
+    PIPSocket        * m_dataSocket;
+    bool               m_shuttingDown;
+
+    bool               m_rawUDPTL; // Put UDPTL directly in RTP payload
+    PINDEX             m_datagramSize;
+
+    int                m_consecutiveBadPackets;
+    bool               m_awaitingGoodPacket;
+    T38_UDPTLPacket  * m_receivedPacket;
+    unsigned           m_expectedSequenceNumber;
+    int                m_secondaryPacket;
+
+    std::map<int, int> m_redundancy;
+    PTimeInterval      m_redundancyInterval;
+    PTimeInterval      m_keepAliveInterval;
+    bool               m_optimiseOnRetransmit;
+    std::vector<int>   m_sentPacketRedundancy;
+    T38_UDPTLPacket  * m_sentPacket;
+    PMutex             m_writeMutex;
+    PTimer             m_timerWriteDataIdle;
+    PDECLARE_NOTIFIER(PTimer,  OpalFaxSession, OnWriteDataIdle);
+};
+
+class OpalFaxMediaStream : public OpalMediaStream
+{
+    PCLASSINFO(OpalFaxMediaStream, OpalMediaStream);
+
+  public:
+    OpalFaxMediaStream(OpalConnection & conn,
+                       const OpalMediaFormat & mediaFormat,
+                       unsigned sessionID,
+                       bool isSource,
+                       OpalFaxSession & session);
+
+    virtual PBoolean Open();
+    virtual PBoolean ReadPacket(RTP_DataFrame & packet);
+    virtual PBoolean WritePacket(RTP_DataFrame & packet);
+    virtual PBoolean IsSynchronous() const;
+    virtual bool InternalUpdateMediaFormat(const OpalMediaFormat & mediaFormat);
+
+  protected:
+    virtual void InternalClose();
+
+    OpalFaxSession & m_session;
+};
 
 #endif // OPAL_FAX
 
