@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24140 $
+ * $Revision: 29451 $
  * $Author: rjongbloed $
- * $Date: 2010-03-22 01:52:25 -0500 (Mon, 22 Mar 2010) $
+ * $Date: 2013-04-07 21:18:19 -0500 (Sun, 07 Apr 2013) $
  */
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
@@ -30,6 +30,10 @@
 
 #include <codec/opalplugin.h>
 
+#ifdef strcasecmp
+#undef strcasecmp // Gets defined in VoIPCodecs/inttypes.h
+#endif
+
 #include "VoIPCodecs/inttypes.h"
 #include "VoIPCodecs/g722.h"
 
@@ -43,7 +47,7 @@
 #define CLOCK_RATE              8000  // Clock rate is not samples/second in this case!
 #define BITS_PER_SECOND         64000 // raw bits per second
 #define FRAME_TIME              1000  // Microseconds in a millisecond
-#define SAMPLES_PER_FRAME       16    // Samples in a millisecond
+#define SAMPLES_PER_FRAME       32    // Samples in a millisecond
 #define	BYTES_PER_FRAME         8     // Bytes in a millisecond
 #define MAX_FRAMES_PER_PACKET   90    // 90 milliseconds, which means RTP packets smaller than 1500 bytes typical LAN maximum
 #define PREF_FRAMES_PER_PACKET  20    // 20 milliseconds
@@ -156,8 +160,7 @@ static struct PluginCodec_Definition g722CodecDefn[] =
     PLUGIN_CODEC_VERSION_WIDEBAND,        // codec API version
     &licenseInfo,                         // license information
 
-    PluginCodec_MediaTypeAudioStreamed |  // audio codec
-    (4 << PluginCodec_BitsPerSamplePos) | // bits per sample
+    PluginCodec_MediaTypeAudio |          // audio codec
     PluginCodec_InputTypeRaw |            // raw input data
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeDynamic,           // specified RTP type
@@ -192,8 +195,7 @@ static struct PluginCodec_Definition g722CodecDefn[] =
     PLUGIN_CODEC_VERSION_WIDEBAND,        // codec API version
     &licenseInfo,                         // license information
 
-    PluginCodec_MediaTypeAudioStreamed |  // audio codec
-    (4 << PluginCodec_BitsPerSamplePos) | // bits per sample
+    PluginCodec_MediaTypeAudio |          // audio codec
     PluginCodec_InputTypeRaw |            // raw input data
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeDynamic,           // specified RTP type
@@ -230,8 +232,7 @@ static struct PluginCodec_Definition g722CodecDefn[] =
     PLUGIN_CODEC_VERSION_WIDEBAND,        // codec API version
     &licenseInfo,                         // license information
 
-    PluginCodec_MediaTypeAudioStreamed |  // audio codec
-    (4 << PluginCodec_BitsPerSamplePos) | // bits per sample
+    PluginCodec_MediaTypeAudio |          // audio codec
     PluginCodec_InputTypeRaw |            // raw input data
     PluginCodec_OutputTypeRaw |           // raw output data
     PluginCodec_RTPTypeExplicit,          // specified RTP type
@@ -266,10 +267,10 @@ static struct PluginCodec_Definition g722CodecDefn[] =
     PLUGIN_CODEC_VERSION_WIDEBAND,        // codec API version
     &licenseInfo,                         // license information
 
-    PluginCodec_MediaTypeAudioStreamed |  // audio codec
-    (4 << PluginCodec_BitsPerSamplePos) | // bits per sample
+    PluginCodec_MediaTypeAudio |          // audio codec
     PluginCodec_InputTypeRaw |            // raw input data
     PluginCodec_OutputTypeRaw |           // raw output data
+    PluginCodec_EmptyPayload |            // Can accept and empty payload
     PluginCodec_RTPTypeExplicit,          // specified RTP type
 
     g722,                                 // text decription
