@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20731 $
- * $Author: ms30002000 $
- * $Date: 2008-08-24 04:36:22 -0500 (Sun, 24 Aug 2008) $
+ * $Revision: 29803 $
+ * $Author: rjongbloed $
+ * $Date: 2013-05-26 23:58:43 -0500 (Sun, 26 May 2013) $
  */
 
 #define _CRT_NONSTDC_NO_DEPRECATE 1
@@ -31,6 +31,10 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifndef _MSC_VER
+#include <stdint.h> // It's not required on MSVC
+#endif
+
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
 #include "plugin-config.h"
 #endif
@@ -84,7 +88,7 @@ static int Speex_Bytes_Per_Frame(int mode, int sampleRate) {
 
 static void * create_encoder(const struct PluginCodec_Definition * codec)
 {
-  int mode = (int)(long)(codec->userData);
+  intptr_t mode = (intptr_t)codec->userData;
 
   struct PluginSpeexContext * context = new PluginSpeexContext;
 
@@ -105,7 +109,7 @@ static int codec_encoder(const struct PluginCodec_Definition * codec,
                                        unsigned * fromLen,
                                            void * to,         
                                        unsigned * toLen,
-                                   unsigned int * flag)
+                                   unsigned int * /*flag*/)
 {
   PluginSpeexContext * context = (PluginSpeexContext *)_context;
 
@@ -115,7 +119,7 @@ static int codec_encoder(const struct PluginCodec_Definition * codec,
 
   // encode PCM data in sampleBuffer to buffer
   int i = 0;
-  int frameLen = codec->parm.audio.samplesPerFrame*2;
+  unsigned  frameLen = codec->parm.audio.samplesPerFrame*2;
   while ((*fromLen >= ((i+1)*frameLen)) && (((i+1)*codec->parm.audio.bytesPerFrame) <= *toLen) ) {
     speex_encode_int(context->coderState, sampleBuffer + i*codec->parm.audio.samplesPerFrame, &context->speexBits); 
     ++i;
@@ -132,7 +136,7 @@ static int codec_encoder(const struct PluginCodec_Definition * codec,
   return 1; 
 }
 
-static void destroy_encoder(const struct PluginCodec_Definition * codec, void * _context)
+static void destroy_encoder(const struct PluginCodec_Definition * /*codec*/, void * _context)
 {
   PluginSpeexContext * context = (PluginSpeexContext *)_context;
 
@@ -198,7 +202,7 @@ static int codec_decoder(const struct PluginCodec_Definition * codec,
   return 1;
 }
 
-static void destroy_decoder(const struct PluginCodec_Definition * codec, void * _context)
+static void destroy_decoder(const struct PluginCodec_Definition * /*codec*/, void * _context)
 {
   struct PluginSpeexContext * context = (struct PluginSpeexContext *)_context;
 
@@ -209,7 +213,7 @@ static void destroy_decoder(const struct PluginCodec_Definition * codec, void *
 
 static int encoder_get_options(
       const PluginCodec_Definition * , 
-      void * context , 
+      void * /*context*/, 
       const char * , 
       void * parm , 
       unsigned * parmLen)
@@ -224,7 +228,7 @@ static int encoder_get_options(
 
 static int valid_for_sip(
       const PluginCodec_Definition * , 
-      void * context , 
+      void * /*context*/, 
       const char * , 
       void * parm , 
       unsigned * parmLen)
@@ -288,7 +292,7 @@ static int coder_get_sip_options(
 }
 
 static int encoder_set_vbr(
-      const PluginCodec_Definition * codec, 
+      const PluginCodec_Definition *, 
       void * _context, 
       const char * , 
       void * parm, 
@@ -303,7 +307,7 @@ static int encoder_set_vbr(
 }
 
 static int decoder_set_vbr(
-      const PluginCodec_Definition * codec, 
+      const PluginCodec_Definition *, 
       void * _context, 
       const char * , 
       void * parm, 
