@@ -25,25 +25,18 @@
  * $Date$
  */
 
-
-#define LOGGING 1
-#define LOG_LEVEL_DEBUG 5
-#define LOG_LEVEL_CONTEXT_ID 3
+#include <codec/opalplugin.h>
 
 extern "C" {
 #include "spandsp.h"
 };
 
-#include <codec/opalplugin.h>
-
-
 #include <stdlib.h>
 
 #if defined(_WIN32) || defined(_WIN32_WCE)
   #define WIN32_LEAN_AND_MEAN
   #include <windows.h>
   #include <io.h>
-  #define strcasecmp _stricmp
   #define access _access
   #define W_OK 2
   #define R_OK 4
@@ -60,6 +53,13 @@ extern "C" {
 #include <map>
 
 
+#define LOGGING 1
+#define LOG_LEVEL_DEBUG 6
+#define LOG_LEVEL_CONTEXT_ID 3
+
+#define   PCM_TRANSMIT_ON_IDLE      true
+#define   DEFAULT_USE_ECM           true
+
 #define   T38_PAYLOAD_CODE          38
 
 #define   BITS_PER_SECOND           14400
@@ -70,12 +70,6 @@ extern "C" {
 #define   MAX_FRAMES_PER_PACKET     1
 
 
-#define   PCM_TRANSMIT_ON_IDLE      true
-#define   USE_ECM                   false
-#define   SUPPORTED_MODEMS_PCM      (T30_SUPPORT_V27TER | T30_SUPPORT_V29)
-#define   SUPPORTED_MODEMS_NO_PCM   (SUPPORTED_MODEMS_PCM /*| T30_SUPPORT_V17*/)
-
-
 #if LOGGING
 
 static PluginCodec_LogFunction LogFunction;
@@ -91,25 +85,9 @@ static PluginCodec_LogFunction LogFunction;
 static void SpanDSP_Message(int level, const char *text)
 {
   if (*text != '\0' && LogFunction != NULL) {
-    if (level >= SPAN_LOG_DEBUG)
-      level = 5;
-    else
-    if (level >= SPAN_LOG_FLOW)
-      level = 4;
-    //else
-    //if (level >= SPAN_LOG_PROTOCOL_WARNING)
-    //  level = 3;
-    else
-    if (level >= SPAN_LOG_PROTOCOL_ERROR)
-      level = 3;
-    //else
-    //if (level >= SPAN_LOG_WARNING)
-    //  level = 3;
-    //else
-    //if (level >= SPAN_LOG_ERROR)
-    //  level = 2;
-    else
-      level = 2;
+    // Close mapping between spandsp levels and OPAL ones, kust one tweak
+    if (level > SPAN_LOG_FLOW)
+      level = 6;
 
     if (!LogFunction(level, NULL, 0, NULL, NULL))
       return;
@@ -161,8 +139,8 @@ static struct PluginCodec_Option const ReceivingOption =
 {
   PluginCodec_BoolOption,     // PluginCodec_OptionTypes
   "Receiving",                // Generic (human readable) option name
-  1,                          // Read Only flag
-  PluginCodec_AlwaysMerge,    // Merge mode
+  true,                       // Read Only flag
+  PluginCodec_OrMerge,        // Merge mode
   "0",                        // Initial value
   NULL,                       // SIP/SDP FMTP name
   NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
@@ -173,34 +151,79 @@ static struct PluginCodec_Option const TiffFileNameOption =
 {
   PluginCodec_StringOption,   // PluginCodec_OptionTypes
   "TIFF-File-Name",           // Generic (human readable) option name
-  1,                          // Read Only flag
-  PluginCodec_AlwaysMerge,    // Merge mode
+  true,                       // Read Only flag
+  PluginCodec_MaxMerge,       // Merge mode
   "",                         // Initial value
   NULL,                       // SIP/SDP FMTP name
   NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
   0                           // H.245 Generic Capability number and scope bits
 };
 
+static struct PluginCodec_Option const StationIdentifierOption =
+{
+  PluginCodec_StringOption,   // PluginCodec_OptionTypes
+  "Station-Identifier",       // Generic (human readable) option name
+  true,                       // Read Only flag
+  PluginCodec_MaxMerge,       // Merge mode
+  "-",                        // Initial value
+  NULL,                       // SIP/SDP FMTP name
+  NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
+  0                           // H.245 Generic Capability number and scope bits
+};
+
+static struct PluginCodec_Option const HeaderInfoOption =
+{
+  PluginCodec_StringOption,   // PluginCodec_OptionTypes
+  "Header-Info",              // Generic (human readable) option name
+  true,                       // Read Only flag
+  PluginCodec_MaxMerge,       // Merge mode
+  "",                         // Initial value
+  NULL,                       // SIP/SDP FMTP name
+  NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
+  0                           // H.245 Generic Capability number and scope bits
+};
+
+static struct PluginCodec_Option const UseEcmOption =
+{
+  PluginCodec_BoolOption,     // PluginCodec_OptionTypes
+  "Use-ECM",                  // Generic (human readable) option name
+  false,                      // Read Only flag
+  PluginCodec_OrMerge,        // Merge mode
+  "0",                        // Initial value
+  NULL,                       // SIP/SDP FMTP name
+  NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
+  0                           // H.245 Generic Capability number and scope bits
+};
+
+
+/*T.38/D.2.3.5
+  T38FaxVersion is negotiated. The entity answering the offer shall return the
+  same or a lower version number.
+ */
 static struct PluginCodec_Option const T38FaxVersion =
 {
   PluginCodec_IntegerOption,  // PluginCodec_OptionTypes
   "T38FaxVersion",            // Generic (human readable) option name
-  0,                          // Read Only flag
+  false,                      // Read Only flag
   PluginCodec_MinMerge,       // Merge mode
   "0",                        // Initial value
   NULL,                       // SIP/SDP FMTP name
-  NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
+  "0",                        // SIP/SDP FMTP default value (option not included in FMTP if have this value)
   0,                          // H.245 Generic Capability number and scope bits
   "0",                        // Minimum value
   "1"                         // Maximum value
 };
 
+/*T.38/D.2.3.5
+  T38FaxRateManagement is declarative and the answer must contain the same value.
+  Note is also compulsory so has no default value.
+ */
 static struct PluginCodec_Option const T38FaxRateManagement =
 {
   PluginCodec_EnumOption,     // PluginCodec_OptionTypes
   "T38FaxRateManagement",     // Generic (human readable) option name
-  0,                          // Read Only flag
-  PluginCodec_MinMerge,       // Merge mode
+  false,                      // Read Only flag
+  PluginCodec_AlwaysMerge,    // Merge mode
   "transferredTCF",           // Initial value
   NULL,                       // SIP/SDP FMTP name
   NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
@@ -208,12 +231,17 @@ static struct PluginCodec_Option const T38FaxRateManagement =
   "localTCF:transferredTCF"   // enum values
 };
 
+/*T.38/D.2.3.5
+  T38MaxBitRate is declarative and the answer is independent of the offer. The
+  parameter simply indicates the maximum transmission bit rate supported by
+  the endpoint.
+ */
 static struct PluginCodec_Option const T38MaxBitRate =
 {
   PluginCodec_IntegerOption,  // PluginCodec_OptionTypes
   "T38MaxBitRate",            // Generic (human readable) option name
-  0,                          // Read Only flag
-  PluginCodec_MinMerge,       // Merge mode
+  false,                      // Read Only flag
+  PluginCodec_NoMerge,        // Merge mode
   "14400",                    // Initial value
   NULL,                       // SIP/SDP FMTP name
   NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
@@ -222,40 +250,61 @@ static struct PluginCodec_Option const T38MaxBitRate =
   "56000"                     // Maximum value
 };
 
+/*T.38/D.2.3.5
+  T38FaxMaxBuffer is declarative and the answer is independent of the offer.
+  This parameter simply signals the buffer space available on the offering
+  endpoint and the answering endpoint. The answering endpoint may have more or
+  less buffer space than the offering endpoint. Each endpoint should be
+  considerate of the available buffer space on the opposite endpoint.
+ */
 static struct PluginCodec_Option const T38FaxMaxBuffer =
 {
   PluginCodec_IntegerOption,  // PluginCodec_OptionTypes
   "T38FaxMaxBuffer",          // Generic (human readable) option name
-  0,                          // Read Only flag
-  PluginCodec_MinMerge,       // Merge mode
+  false,                      // Read Only flag
+  PluginCodec_NoMerge,        // Merge mode
   "2000",                     // Initial value
   NULL,                       // SIP/SDP FMTP name
-  NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
+  "528",                      // SIP/SDP FMTP default value (option not included in FMTP if have this value)
   0,                          // H.245 Generic Capability number and scope bits
-  "100",                       // Minimum value
+  "100",                      // Minimum value
   "9999"                      // Maximum value
 };
 
+/*T.38/D.2.3.5
+  T38FaxMaxDatagram is declarative and the answer is independent of the offer.
+  This parameter signals the largest acceptable datagram for the offering
+  endpoint and the answering endpoint (i.e., the maximum size of the RTP
+  payload). The answering endpoint may accept a larger or smaller datagram
+  than the offering endpoint. Each endpoint should be considerate of the
+  maximum datagram size of the opposite endpoint.
+ */
 static struct PluginCodec_Option const T38FaxMaxDatagram =
 {
   PluginCodec_IntegerOption,  // PluginCodec_OptionTypes
   "T38FaxMaxDatagram",        // Generic (human readable) option name
-  0,                          // Read Only flag
-  PluginCodec_MinMerge,       // Merge mode
-  "528",                      // Initial value
+  false,                      // Read Only flag
+  PluginCodec_NoMerge,        // Merge mode
+  "1400",                     // Initial value
   NULL,                       // SIP/SDP FMTP name
-  NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
+  "528",                      // SIP/SDP FMTP default value (option not included in FMTP if have this value)
   0,                          // H.245 Generic Capability number and scope bits
   "10",                       // Minimum value
   "1500"                      // Maximum value
 };
 
+/*T.38/D.2.3.5
+  T38FaxUdpEC is negotiated only when using UDPTL as the transport. If the
+  answering endpoint supports the offered error correction mode, then it shall
+  return the same value in the answer, otherwise the T38FaxUdpEC parameter
+  shall not be present in the answer.
+ */
 static struct PluginCodec_Option const T38FaxUdpEC =
 {
   PluginCodec_EnumOption,     // PluginCodec_OptionTypes
   "T38FaxUdpEC",              // Generic (human readable) option name
-  0,                          // Read Only flag
-  PluginCodec_MinMerge,       // Merge mode
+  false,                      // Read Only flag
+  PluginCodec_AlwaysMerge,    // Merge mode
   "t38UDPRedundancy",         // Initial value
   NULL,                       // SIP/SDP FMTP name
   NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
@@ -263,45 +312,63 @@ static struct PluginCodec_Option const T38FaxUdpEC =
   "t38UDPFEC:t38UDPRedundancy"// enum values
 };
 
+/*T.38/D.2.3.5
+  T38FaxFillBitRemoval is negotiated. If the answering entity does not support
+  this capability or if the capability was not in the offer, this parameter
+  shall not be present in the answer.
+ */
 static struct PluginCodec_Option const T38FaxFillBitRemoval =
 {
   PluginCodec_BoolOption,     // PluginCodec_OptionTypes
   "T38FaxFillBitRemoval",     // Generic (human readable) option name
-  0,                          // Read Only flag
-  PluginCodec_MinMerge,       // Merge mode
+  false,                      // Read Only flag
+  PluginCodec_AndMerge,       // Merge mode
   "0",                        // Initial value
   NULL,                       // SIP/SDP FMTP name
-  NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
+  "0",                        // SIP/SDP FMTP default value (option not included in FMTP if have this value)
   0                           // H.245 Generic Capability number and scope bits
 };
 
+/*T.38/D.2.3.5
+  T38FaxTranscodingMMR is negotiated. If the answering entity does not support
+  this capability or if the capability was not in the offer, this parameter
+  shall not be present in the answer.
+ */
 static struct PluginCodec_Option const T38FaxTranscodingMMR =
 {
   PluginCodec_BoolOption,     // PluginCodec_OptionTypes
   "T38FaxTranscodingMMR",     // Generic (human readable) option name
-  0,                          // Read Only flag
-  PluginCodec_MinMerge,       // Merge mode
+  false,                      // Read Only flag
+  PluginCodec_AndMerge,       // Merge mode
   "0",                        // Initial value
   NULL,                       // SIP/SDP FMTP name
-  NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
+  "0",                        // SIP/SDP FMTP default value (option not included in FMTP if have this value)
   0                           // H.245 Generic Capability number and scope bits
 };
 
+/*T.38/D.2.3.5
+  T38FaxTranscodingJBIG is negotiated. If the answering entity does not
+  support this capability or if the capability was not in the offer, this
+  parameter shall not be present in the answer.
+ */
 static struct PluginCodec_Option const T38FaxTranscodingJBIG =
 {
   PluginCodec_BoolOption,     // PluginCodec_OptionTypes
   "T38FaxTranscodingJBIG",    // Generic (human readable) option name
-  0,                          // Read Only flag
-  PluginCodec_MinMerge,       // Merge mode
+  false,                      // Read Only flag
+  PluginCodec_AndMerge,       // Merge mode
   "0",                        // Initial value
   NULL,                       // SIP/SDP FMTP name
-  NULL,                       // SIP/SDP FMTP default value (option not included in FMTP if have this value)
+  "0",                        // SIP/SDP FMTP default value (option not included in FMTP if have this value)
   0                           // H.245 Generic Capability number and scope bits
 };
 
 static struct PluginCodec_Option const * const OptionTableTIFF[] = {
+  &StationIdentifierOption,
   &ReceivingOption,
   &TiffFileNameOption,
+  &HeaderInfoOption,
+  &UseEcmOption,
   NULL
 };
 
@@ -315,6 +382,9 @@ static struct PluginCodec_Option const * const OptionTableT38[] = {
   &T38FaxFillBitRemoval,
   &T38FaxTranscodingMMR,
   &T38FaxTranscodingJBIG,
+  &StationIdentifierOption,
+  &HeaderInfoOption,
+  &UseEcmOption,
   NULL
 };
 
@@ -366,6 +436,15 @@ class WaitAndSignal
 };
 
 
+/////////////////////////////////////////////////////////////////
+
+static bool ParseBool(const char * str)
+{
+  return str != NULL && *str != '\0' &&
+            (toupper(*str) == 'Y' || toupper(*str) == 'T' || atoi(str) != 0);
+}
+
+
 /////////////////////////////////////////////////////////////////
 
 class FaxSpanDSP
@@ -387,8 +466,8 @@ class FaxSpanDSP
     FaxSpanDSP()
       : m_referenceCount(1)
       , m_completed(false)
-      , m_useECM(USE_ECM)
-      , m_supported_modems(SUPPORTED_MODEMS_PCM)
+      , m_useECM(DEFAULT_USE_ECM)
+      , m_supported_modems(T30_SUPPORT_V27TER | T30_SUPPORT_V29 | T30_SUPPORT_V17)
     {
     }
 
@@ -437,10 +516,15 @@ class FaxSpanDSP
     {
       PTRACE(3, m_tag << " SetOption: " << option << "=" << value);
 
+      if (strcasecmp(option, UseEcmOption.m_name) == 0) {
+        m_useECM = ParseBool(value);
+        return true;
+      }
+
       return true;
     }
 
-    bool HasError(bool retval)
+    bool HasError(bool retval, const char * errorMsg = NULL)
     {
       if (m_completed)
         return true;
@@ -450,6 +534,10 @@ class FaxSpanDSP
 
       /// Error exit
       m_completed = true;
+      if (errorMsg != NULL) {
+        PTRACE(1, m_tag << " Error: " << errorMsg);
+      }
+
       return true;
     }
 };
@@ -463,24 +551,40 @@ class FaxT38
 #endif
 {
   private:
-    unsigned        m_protoVersion;
-
-    int             m_sequence;
+    unsigned m_protoVersion;
+    unsigned m_RateManagement;
+    unsigned m_MaxBitRate;
+    unsigned m_MaxBuffer;
+    unsigned m_MaxDatagram;
+    unsigned m_UdpEC;
+    bool     m_FillBitRemoval;
+    bool     m_TranscodingMMR;
+    bool     m_TranscodingJBIG;
 
     t38_core_state_t * m_t38core;
-
+    int                m_sequence;
     std::queue< std::vector<uint8_t> > m_t38Queue;
 
 
   protected:
     FaxT38()
       : m_protoVersion(0)
-      , m_sequence(0)
+      , m_RateManagement(1)
+      , m_MaxBitRate(14400)
+      , m_MaxBuffer(2000)
+      , m_MaxDatagram(528)
+      , m_UdpEC(1)
+      , m_FillBitRemoval(false)
+      , m_TranscodingMMR(false)
+      , m_TranscodingJBIG(false)
       , m_t38core(NULL)
+      , m_sequence(0)
     {
     }
 
 
+    unsigned GetMaxBitRate() const { return m_MaxBitRate; }
+
     bool SetOption(const char * option, const char * value)
     {
       if (strcasecmp(option, T38FaxVersion.m_name) == 0) {
@@ -488,6 +592,51 @@ class FaxT38
         return true;
       }
 
+      if (strcasecmp(option, T38FaxRateManagement.m_name) == 0) {
+        if (strcasecmp(value, "transferredTCF") == 0)
+          m_RateManagement = T38_DATA_RATE_MANAGEMENT_TRANSFERRED_TCF;
+        else if (strcasecmp(value, "localTCF") == 0)
+          m_RateManagement = T38_DATA_RATE_MANAGEMENT_LOCAL_TCF;
+        else
+          return false;
+        return true;
+      }
+
+      if (strcasecmp(option, T38MaxBitRate.m_name) == 0) {
+        m_MaxBitRate = atoi(value);
+        return true;
+      }
+
+      if (strcasecmp(option, T38FaxMaxBuffer.m_name) == 0) {
+        m_MaxBuffer = atoi(value);
+        return true;
+      }
+
+      if (strcasecmp(option, T38FaxMaxDatagram.m_name) == 0) {
+        m_MaxDatagram = atoi(value);
+        return true;
+      }
+
+      if (strcasecmp(option, T38FaxUdpEC.m_name) == 0) {
+        m_UdpEC = atoi(value);
+        return true;
+      }
+
+      if (strcasecmp(option, T38FaxFillBitRemoval.m_name) == 0) {
+        m_FillBitRemoval = ParseBool(value);
+        return true;
+      }
+
+      if (strcasecmp(option, T38FaxTranscodingMMR.m_name) == 0) {
+        m_TranscodingMMR = ParseBool(value);
+        return true;
+      }
+
+      if (strcasecmp(option, T38FaxTranscodingJBIG.m_name) == 0) {
+        m_TranscodingJBIG = ParseBool(value);
+        return true;
+      }
+
       return true;
     }
 
@@ -497,6 +646,16 @@ class FaxT38
       m_t38core = t38core;
       InitLogging(t38_core_get_logging_state(m_t38core), m_tag);
       t38_set_t38_version(m_t38core, m_protoVersion);
+      t38_set_data_rate_management_method(m_t38core, m_RateManagement);
+#ifdef HAS_T38_SET_FASTEST_IMAGE_DATA_RATE
+      t38_set_fastest_image_data_rate(m_t38core, m_MaxBitRate);
+#endif
+      t38_set_max_buffer_size(m_t38core, m_MaxBuffer);
+      t38_set_max_datagram_size(m_t38core, m_MaxDatagram);
+      // t38_set_XXXX(m_t38core, m_UdpEC); Don't know what this corresponds to!!
+      t38_set_fill_bit_removal(m_t38core, m_FillBitRemoval);
+      t38_set_mmr_transcoding(m_t38core, m_TranscodingMMR);
+      t38_set_jbig_transcoding(m_t38core, m_TranscodingJBIG);
 
       return true;
     }
@@ -522,7 +681,13 @@ class FaxT38
 
       uint16_t seq = uint16_t(m_sequence++);
 
+#ifdef _MSC_VER
+#pragma warning(disable:4244)
+#endif
       PluginCodec_RTP_SetSequenceNumber(toPtr, seq);
+#ifdef _MSC_VER
+#pragma warning(default:4244)
+#endif
 
       m_t38Queue.pop();
 
@@ -537,20 +702,16 @@ class FaxT38
     {
       int payloadSize = fromLen - PluginCodec_RTP_GetHeaderLength(fromPtr);
 
-      if (payloadSize < 0)
+      if (payloadSize < 0 || m_t38core == NULL)
         return false;
 
-      if (payloadSize > 0) {
-        if (t38_core_rx_ifp_packet(m_t38core,
-                                   PluginCodec_RTP_GetPayloadPtr(fromPtr),
-                                   payloadSize,
-                                   PluginCodec_RTP_GetSequenceNumber(fromPtr)) == -1)
-        {
-          return false;
-        }
-      }
+      if (payloadSize == 0)
+        return true;
 
-      return true;
+      return t38_core_rx_ifp_packet(m_t38core,
+                                    PluginCodec_RTP_GetPayloadPtr(fromPtr),
+                                    payloadSize,
+                                    PluginCodec_RTP_GetSequenceNumber(fromPtr)) != -1;
     }
 
 
@@ -602,44 +763,55 @@ class FaxPCM
 
 /////////////////////////////////////////////////////////////////
 
-struct MyStats : public t30_stats_t
+class MyStats : private t30_stats_t
 {
-  MyStats(bool completed, bool receiving)
-    : m_completed(completed)
-    , m_receiving(receiving)
-  {
-  }
-
   bool m_completed;
   bool m_receiving;
+  char m_phase;
+  std::string m_stationId;
+
+public:
+  MyStats(t30_state_t * t30state, bool completed, bool receiving, char phase)
+    : m_completed(completed)
+    , m_receiving(receiving)
+    , m_phase(phase)
+  {
+    t30_get_transfer_statistics(t30state, this);
+    const char * stationId = t30_get_rx_ident(t30state);
+    if (stationId != NULL && *stationId != '\0')
+      m_stationId = stationId;
+  }
+
+
+  friend std::ostream & operator<<(std::ostream & strm, const MyStats & stats)
+  {
+    static const char * const CompressionNames[4] = { "N/A", "T.4 1d", "T.4 2d", "T.6" };
+
+    strm << "Status=";
+    if (stats.m_completed)
+      strm << stats.current_status << " (" << t30_completion_code_to_str(stats.current_status) << ')';
+    else
+      strm << "-1 (In progress)";
+    strm << "\n"
+            "Bit Rate=" << stats.bit_rate << "\n"
+            "Encoding=" << stats.encoding << ' ' << CompressionNames[stats.encoding&3] << "\n"
+            "Error Correction=" << stats.error_correcting_mode << "\n"
+            "Tx Pages=" << (stats.m_receiving ? -1 : stats.pages_tx) << "\n"
+            "Rx Pages=" << (stats.m_receiving ? stats.pages_rx : -1) << "\n"
+            "Total Pages=" << stats.pages_in_file << "\n"
+            "Image Bytes=" << stats.image_size << "\n"
+            "Resolution=" << stats.x_resolution << 'x' << stats.y_resolution << "\n"
+            "Page Size=" << stats.width << 'x' << stats.length << "\n"
+            "Bad Rows=" << stats.bad_rows << "\n"
+            "Most Bad Rows=" << stats.longest_bad_row_run << "\n"
+            "Correction Retries=" << stats.error_correcting_mode_retries << "\n"
+            "Station Identifier=" << stats.m_stationId << "\n"
+            "Phase=" << stats.m_phase;
+
+    return strm;
+  }
 };
 
-static std::ostream & operator<<(std::ostream & strm, const MyStats & stats)
-{
-  static const char * const CompressionNames[4] = { "N/A", "T.4 1d", "T.4 2d", "T.6" };
-
-  strm << "Status=";
-  if (stats.m_completed)
-    strm << stats.current_status << " (" << t30_completion_code_to_str(stats.current_status) << ')';
-  else
-    strm << "-1 (In progress)";
-  strm << "\n"
-          "Bit Rate=" << stats.bit_rate << "\n"
-          "Encoding=" << stats.encoding << ' ' << CompressionNames[stats.encoding&3] << "\n"
-          "Error Correction=" << stats.error_correcting_mode << "\n"
-          "Tx Pages=" << (stats.m_receiving ? -1 : stats.pages_tx) << "\n"
-          "Rx Pages=" << (stats.m_receiving ? stats.pages_rx : -1) << "\n"
-          "Total Pages=" << stats.pages_in_file << "\n"
-          "Image Bytes=" << stats.image_size << "\n"
-          "Resolution=" << stats.x_resolution << 'x' << stats.y_resolution << "\n"
-          "Page Size=" << stats.width << 'x' << stats.length << "\n"
-          "Bad Rows=" << stats.bad_rows << "\n"
-          "Most Bad Rows=" << stats.longest_bad_row_run << "\n"
-          "Correction Retries=" << stats.error_correcting_mode_retries;
-
-  return strm;
-}
-
 
 class FaxTIFF : public FaxSpanDSP
 {
@@ -647,11 +819,33 @@ class FaxTIFF : public FaxSpanDSP
     bool            m_receiving;
     std::string     m_tiffFileName;
     std::string     m_stationIdentifer;
+    std::string     m_headerInfo;
+    int             m_supported_image_sizes;
+    int             m_supported_resolutions;
+    int             m_supported_compressions;
+    char            m_phase;
 
   protected:
     FaxTIFF()
       : m_receiving(false)
       , m_stationIdentifer("-")
+      , m_supported_image_sizes(T30_SUPPORT_US_LETTER_LENGTH |
+                                T30_SUPPORT_US_LEGAL_LENGTH |
+                                T30_SUPPORT_UNLIMITED_LENGTH |
+                                T30_SUPPORT_A4_LENGTH |
+                                T30_SUPPORT_B4_LENGTH |
+                                T30_SUPPORT_215MM_WIDTH |
+                                T30_SUPPORT_255MM_WIDTH |
+                                T30_SUPPORT_303MM_WIDTH)
+      , m_supported_resolutions(T30_SUPPORT_STANDARD_RESOLUTION |
+                                T30_SUPPORT_FINE_RESOLUTION |
+                                T30_SUPPORT_SUPERFINE_RESOLUTION |
+                                T30_SUPPORT_R8_RESOLUTION |
+                                T30_SUPPORT_R16_RESOLUTION)
+      , m_supported_compressions(T30_SUPPORT_T4_1D_COMPRESSION |
+                                 T30_SUPPORT_T4_2D_COMPRESSION |
+                                 T30_SUPPORT_T6_COMPRESSION)
+      , m_phase('A')
     {
     }
 
@@ -672,7 +866,17 @@ class FaxTIFF : public FaxSpanDSP
       }
 
       if (strcasecmp(option, ReceivingOption.m_name) == 0) {
-        m_receiving = atoi(value) != 0;
+        m_receiving = ParseBool(value);
+        return true;
+      }
+
+      if (strcasecmp(option, StationIdentifierOption.m_name) == 0) {
+        m_stationIdentifer = *value != '\0' ? value : "-";
+        return true;
+      }
+
+      if (strcasecmp(option, HeaderInfoOption.m_name) == 0) {
+        m_headerInfo = value;
         return true;
       }
 
@@ -718,9 +922,22 @@ class FaxTIFF : public FaxSpanDSP
       t30_set_phase_b_handler(t30state, PhaseB, this);
       t30_set_phase_d_handler(t30state, PhaseD, this);
       t30_set_phase_e_handler(t30state, PhaseE, this);
+
       t30_set_tx_ident(t30state, m_stationIdentifer.c_str());
-      t30_set_ecm_capability(t30state, m_useECM);
+      PTRACE(4, m_tag << " Set Station-Identifier to \"" << m_stationIdentifer << '"');
+
+      if (!m_headerInfo.empty()) {
+        if (t30_set_tx_page_header_info(t30state, m_headerInfo.c_str()) < 0)
+          PTRACE(1, m_tag << " Cannot set Header-Info to  \"" << m_headerInfo << '"');
+        else
+          PTRACE(4, m_tag << " Set Header-Info to \"" << m_headerInfo << '"');
+      }
+
       t30_set_supported_modems(t30state, m_supported_modems);
+      t30_set_supported_image_sizes(t30state, m_supported_image_sizes);
+      t30_set_supported_resolutions(t30state, m_supported_resolutions);
+      t30_set_supported_compressions(t30state, m_supported_compressions);
+      t30_set_ecm_capability(t30state, m_useECM);
 
       return true;
     }
@@ -731,8 +948,7 @@ class FaxTIFF : public FaxSpanDSP
       if (t30state == NULL)
         return false;
 
-      MyStats stats(m_completed, m_receiving);
-      t30_get_transfer_statistics(t30state, &stats);
+      MyStats stats(t30state, m_completed, m_receiving, m_phase);
       std::stringstream strm;
       strm << stats;
 
@@ -745,7 +961,7 @@ class FaxTIFF : public FaxSpanDSP
 
       memcpy(fromPtr, str.c_str(), len);
 
-      PTRACE(LOG_LEVEL_DEBUG, m_tag << " SpanDSP statistics:\n" << (char *)fromPtr);
+      PTRACE(4, m_tag << " SpanDSP statistics:\n" << (char *)fromPtr);
 
       return true;
     }
@@ -778,16 +994,16 @@ class FaxTIFF : public FaxSpanDSP
   private:
     void PhaseB(t30_state_t * t30state, int)
     {
-      MyStats stats(m_completed, m_receiving);
-      t30_get_transfer_statistics(t30state, &stats);
-      PTRACE(3, m_tag << " SpanDSP entered Phase B:\n" << stats);
+      m_phase = 'B';
+      PTRACE(3, m_tag << " SpanDSP entered Phase B:\n"
+             << MyStats(t30state, m_completed, m_receiving, m_phase));
     }
 
     void PhaseD(t30_state_t * t30state, int)
     {
-      MyStats stats(m_completed, m_receiving);
-      t30_get_transfer_statistics(t30state, &stats);
-      PTRACE(3, m_tag << " SpanDSP entered Phase D:\n" << stats);
+      m_phase = 'D';
+      PTRACE(3, m_tag << " SpanDSP entered Phase D:\n"
+             << MyStats(t30state, m_completed, m_receiving, m_phase));
     }
 
     void PhaseE(t30_state_t * t30state, int result)
@@ -795,9 +1011,9 @@ class FaxTIFF : public FaxSpanDSP
       if (result >= 0)
         m_completed = true; // Finished, exit codec loops
 
-      MyStats stats(m_completed, m_receiving);
-      t30_get_transfer_statistics(t30state, &stats);
-      PTRACE(3, m_tag << " SpanDSP entered Phase E:\n" << stats);
+      m_phase = 'E';
+      PTRACE(3, m_tag << " SpanDSP entered Phase E:\n"
+             << MyStats(t30state, m_completed, m_receiving, m_phase));
     }
 };
 
@@ -936,9 +1152,11 @@ class T38_PCM : public FaxSpanDSP, public FaxT38, public FaxPCM
       PTRACE(3, m_tag << " Opening T38_PCM/SpanDSP");
   
       m_t38State = t38_gateway_init(NULL, FaxT38::QueueT38, (FaxT38 *)this);
-      if (HasError(m_t38State != NULL))
+      if (HasError(m_t38State != NULL, "t38_gateway_init failed."))
         return false;
 
+      t38_gateway_set_supported_modems(m_t38State, m_supported_modems);
+
       if (HasError(FaxT38::Open(t38_gateway_get_t38_core_state(m_t38State))))
         return false;
 
@@ -946,7 +1164,6 @@ class T38_PCM : public FaxSpanDSP, public FaxT38, public FaxPCM
 
       t38_gateway_set_transmit_on_idle(m_t38State, TransmitOnIdle());
       t38_gateway_set_ecm_capability(m_t38State, m_useECM);
-      t38_gateway_set_supported_modems(m_t38State, m_supported_modems);
       //t38_gateway_set_nsx_suppression(m_t38State, NULL, 0, NULL, 0);
 
       return true;
@@ -969,8 +1186,6 @@ class TIFF_T38 : public FaxTIFF, public FaxT38
       m_tag = tag;
 #endif
 
-      m_supported_modems = SUPPORTED_MODEMS_NO_PCM;
-
       PTRACE(4, m_tag << " Created TIFF_T38");
     }
 
@@ -1079,8 +1294,12 @@ class TIFF_T38 : public FaxTIFF, public FaxT38
 
       PTRACE(3, m_tag << " Opening TIFF_T38/SpanDSP for " << (IsReceiving() ? "receive" : "transmit"));
 
+      // If our max bit rate is 9600 then explicitly remove V.17 or spandsp does 14400 anyway
+      if (GetMaxBitRate() <= 9600)
+        m_supported_modems &= ~T30_SUPPORT_V17;
+
       m_t38State = t38_terminal_init(NULL, !IsReceiving(), FaxT38::QueueT38, (FaxT38 *)this);
-      if (HasError(m_t38State != NULL))
+      if (HasError(m_t38State != NULL, "t38_terminal_init failed."))
         return false;
 
       if (HasError(FaxTIFF::Open(t38_terminal_get_t30_state(m_t38State))))
@@ -1224,7 +1443,7 @@ class TIFF_PCM : public FaxTIFF, public FaxPCM
       PTRACE(3, m_tag << " Opening TIFF_PCM/SpanDSP for " << (IsReceiving() ? "receive" : "transmit"));
 
       m_faxState = fax_init(NULL, !IsReceiving());
-      if (HasError(m_faxState != NULL))
+      if (HasError(m_faxState != NULL, "t38_terminal_init failed."))
         return false;
 
       if (HasError(FaxTIFF::Open(fax_get_t30_state(m_faxState))))
@@ -1287,10 +1506,12 @@ class FaxCodecContext
       WaitAndSignal mutex(InstanceMapMutex);
 
       InstanceMapType::iterator iter = InstanceMap.find(m_key);
-      if (iter != InstanceMap.end() && iter->second->Dereference()) {
-        delete iter->second;
-        InstanceMap.erase(iter);
+      if (iter != InstanceMap.end()) {
         PTRACE(LOG_LEVEL_CONTEXT_ID, KeyToStr(m_key) << " Context Id removed");
+        if (iter->second->Dereference()) {
+          delete iter->second;
+          InstanceMap.erase(iter);
+        }
       }
     }
 
@@ -1769,5 +1990,4 @@ extern "C" {
 PLUGIN_CODEC_IMPLEMENT_ALL(SpanDSP, faxCodecDefn, MY_API_VERSION)
 
 };
-
- 	  	 
+ 
