@@ -25,9 +25,9 @@
  * Contributor(s): Michele Piccini (michele@piccini.com)
  *                 Derek Smithies (derek@indranet.co.nz)
  *
- * $Revision: 20951 $
- * $Author: ms30002000 $
- * $Date: 2008-09-14 05:10:29 -0500 (Sun, 14 Sep 2008) $
+ * $Revision: 27898 $
+ * $Author: rjongbloed $
+ * $Date: 2012-06-25 21:05:22 -0500 (Mon, 25 Jun 2012) $
  */
 
 /*
@@ -47,8 +47,6 @@
   which will determine the size from the received data stream.
  */
 
-//#define DEBUG_OUTPUT 1
-
 #define _CRT_NONSTDC_NO_DEPRECATE 1
 #define _CRT_SECURE_NO_WARNINGS 1
 
@@ -61,7 +59,7 @@
 #include "plugin-config.h"
 #endif
 
-#include <codec/opalplugin.h>
+#include <codec/opalplugin.hpp>
 
 
 #if defined (_WIN32) || defined (_WIN32_WCE)
@@ -80,7 +78,6 @@
 #include <math.h>
 
 #include "critsect.h"
-#include "rtpframe.h"
 
 #ifdef _MSC_VER
 #pragma warning(disable:4800)
@@ -110,44 +107,6 @@ typedef unsigned int u_int;
   #undef max
 #endif
 
-#ifdef _MSC_VER
-   #include "../common/trace.h"
-#else
-   #include "trace.h"
-#endif
-
-
-#if DEBUG_OUTPUT
-#include <stdio.h>
-
-#ifdef _WIN32
-#include <io.h>
-#define CREAT(f) _open(f, _O_WRONLY | _O_TRUNC | _O_CREAT | _O_BINARY, 0600 )
-#else
-#include <sys/types.h>
-#include <sys/stat.h>
-#define CREAT(f) ::creat(f, 0666)
-#endif
-#include <fcntl.h>
-
-static void debug_write_data(int & fd, const char * title, const char * filename, const void * buffer, size_t writeLen)
-{
-  if (fd == -1) {
-   if ((fd = CREAT(filename)) < 0) 
-    printf("cannot open %s file\n", title);
-  }
-  if (fd >= 0) {
-    size_t len = write(fd, buffer, writeLen);
-    if (len != writeLen) 
-      printf("failed to write %s buffer - %i != %i\n", title, len, writeLen);
-    else
-      printf("wrote %i bytes to %s\n", writeLen, title);
-  }
-}
-
-#endif
-
-#define RTP_MIN_HEADER_SIZE 12
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -227,7 +186,7 @@ class H261EncoderContext
 				  , 1.0);
 	videoQuality = std::max ((int)( floor ( tsto / factor)), 1); 
       } 
-      TRACE(4, "H261\tf(tsto=" << tsto << ", bitrate=" << bitrate << ", width=" << width <<", height=" << height << ")=" << videoQuality);
+      PTRACE(4, "H261", "f(tsto=" << tsto << ", bitrate=" << bitrate << ", width=" << width <<", height=" << height << ")=" << videoQuality);
     }
 
     int EncodeFrames(const u_char * src, unsigned & srcLen, u_char * dst, unsigned & dstLen, unsigned int & flags)
@@ -235,21 +194,17 @@ class H261EncoderContext
       WaitAndSignal m(mutex);
 
       // create RTP frame from source buffer
-      RTPFrame srcRTP(src, srcLen);
+      PluginCodec_RTP srcRTP(src, srcLen);
 
       // create RTP frame from destination buffer
-      RTPFrame dstRTP(dst, dstLen, RTP_RFC2032_PAYLOAD);
+      PluginCodec_RTP dstRTP(dst, dstLen);
       dstLen = 0;
 
       // return more pending data frames, if any
       if (videoEncoder->MoreToIncEncode()) {
         unsigned payloadLength = 0;
         videoEncoder->IncEncodeAndGetPacket((u_char *)dstRTP.GetPayloadPtr(), payloadLength); //get next packet on list
-        dstLen = SetEncodedPacket(dstRTP, !videoEncoder->MoreToIncEncode(), RTP_RFC2032_PAYLOAD, lastTimeStamp, payloadLength, flags);
-#if DEBUG_OUTPUT
-static int encoderOutput = -1;
-debug_write_data(encoderOutput, "encoder output", "encoder.output", dstRTP.GetPacketPtr(), dstLen);
-#endif
+        dstLen = SetEncodedPacket(dstRTP, !videoEncoder->MoreToIncEncode(), payloadLength, flags);
         return 1;
       }
 
@@ -263,27 +218,9 @@ debug_write_data(encoderOutput, "encoder output", "encoder.output", dstRTP.GetPa
       // update the video quality
       videoEncoder->SetQualityLevel(videoQuality);
 
-      // get and validate header
-      if (srcRTP.GetPayloadSize() < sizeof(PluginCodec_Video_FrameHeader)) {
-        TRACE(1,"H261\tVideo grab too small");
-        return 0;
-      } 
-
-      PluginCodec_Video_FrameHeader * header = (PluginCodec_Video_FrameHeader *)srcRTP.GetPayloadPtr();
+      PluginCodec_Video_FrameHeader * header = srcRTP.GetVideoHeader();
       if (header->x != 0 && header->y != 0) {
-        TRACE(1,"H261\tVideo grab of partial frame unsupported");
-        return 0;
-      }
-
-      // make sure the incoming frame is big enough for the specified frame size
-      if (srcRTP.GetPayloadSize() < (int)(sizeof(PluginCodec_Video_FrameHeader) + frameWidth*frameHeight*12/8)) {
-        TRACE(1,"H261\tPayload of grabbed frame too small for full frame");
-        return 0;
-      }
-
-      if ((header->width  != 176 && header->width  != 352) ||
-          (header->height != 144 && header->height != 288)) {
-        TRACE(1,"H261\tInvalid frame size");
+        PTRACE(1,"H261", "Video grab of partial frame unsupported");
         return 0;
       }
 
@@ -294,14 +231,8 @@ debug_write_data(encoderOutput, "encoder output", "encoder.output", dstRTP.GetPa
         videoEncoder->SetSize(frameWidth, frameHeight);
       }
 
-#if DEBUG_OUTPUT
-static int encoderYUV = -1;
-debug_write_data(encoderYUV, "encoder input", "encoder.yuv", OPAL_VIDEO_FRAME_DATA_PTR(header),
-                                                             srcRTP.GetPayloadSize() - sizeof(PluginCodec_Video_FrameHeader));
-#endif
-
       // "grab" the frame
-      memcpy(videoEncoder->GetFramePtr(), OPAL_VIDEO_FRAME_DATA_PTR(header), frameWidth*frameHeight*12/8);
+      memcpy(videoEncoder->GetFramePtr(), srcRTP.GetVideoFrameData(), frameWidth*frameHeight*12/8);
 
       // force I-frame, if requested
       if (forceIFrame || (flags & PluginCodec_CoderForceIFrame) != 0) {
@@ -318,35 +249,28 @@ debug_write_data(encoderYUV, "encoder input", "encoder.yuv", OPAL_VIDEO_FRAME_DA
       else {
         unsigned payloadLength = 0;
         videoEncoder->IncEncodeAndGetPacket((u_char *)dstRTP.GetPayloadPtr(), payloadLength); 
-        dstLen = SetEncodedPacket(dstRTP, !videoEncoder->MoreToIncEncode(), RTP_RFC2032_PAYLOAD, lastTimeStamp, payloadLength, flags);
-
-#if DEBUG_OUTPUT
-debug_write_data(encoderOutput, "encoder output", "encoder.output", dstRTP.GetPacketPtr(), dstLen);
-#endif
-
+        dstLen = SetEncodedPacket(dstRTP, !videoEncoder->MoreToIncEncode(), payloadLength, flags);
       }
 
       return 1;
     }
 
   protected:
-    unsigned SetEncodedPacket(RTPFrame & dstRTP, bool isLast, unsigned char payloadCode, unsigned long lastTimeStamp, unsigned payloadLength, unsigned & flags);
+    unsigned SetEncodedPacket(PluginCodec_RTP & dstRTP, bool isLast, unsigned payloadLength, unsigned & flags);
 };
 
 
 
-unsigned H261EncoderContext::SetEncodedPacket(RTPFrame & dstRTP, bool isLast, unsigned char payloadCode, unsigned long lastTimeStamp, unsigned payloadLength, unsigned & flags)
+unsigned H261EncoderContext::SetEncodedPacket(PluginCodec_RTP & dstRTP, bool isLast, unsigned payloadLength, unsigned & flags)
 {
   dstRTP.SetPayloadSize(payloadLength);
   dstRTP.SetMarker(isLast);
-  dstRTP.SetPayloadType(payloadCode);
-  dstRTP.SetTimestamp(lastTimeStamp);
 
   flags = 0;
   flags |= isLast ? PluginCodec_ReturnCoderLastFrame : 0;  // marker bit on last frame of video
   flags |= PluginCodec_ReturnCoderIFrame;                       // sadly, this encoder *always* returns I-frames :(
 
-  return dstRTP.GetFrameLen();
+  return dstRTP.GetPacketSize();
 }
 
 static void * create_encoder(const struct PluginCodec_Definition * /*codec*/)
@@ -409,7 +333,7 @@ static int codec_encoder(const struct PluginCodec_Definition * ,
 
 static int encoder_get_output_data_size(const PluginCodec_Definition *, void *, const char *, void *, unsigned *)
 {
-  return RTP_MTU + RTP_MIN_HEADER_SIZE;
+  return RTP_MTU + PluginCodec_RTP_MinHeaderSize;
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -457,27 +381,23 @@ class H261DecoderContext
       WaitAndSignal m(mutex);
 
       // create RTP frame from source buffer
-      RTPFrame srcRTP(src, srcLen);
+      PluginCodec_RTP srcRTP(src, srcLen);
 
       // create RTP frame from destination buffer
-      RTPFrame dstRTP(dst, dstLen, 0);
+      PluginCodec_RTP dstRTP(dst, dstLen);
       dstLen = 0;
       flags = 0;
 
       // Check for lost packets to help decoder
       bool lostPreviousPacket = false;
-      if ((expectedSequenceNumber == 0) || (expectedSequenceNumber != srcRTP.GetSequenceNumber())) {
+      unsigned sequenceNumber = srcRTP.GetSequenceNumber();
+      if ((expectedSequenceNumber == 0) || (expectedSequenceNumber != sequenceNumber)) {
         lostPreviousPacket = true;
-        TRACE(3,"H261\tDetected loss of one video packet. "
+        PTRACE(3,"H261", "Detected loss of one video packet. "
     	      << expectedSequenceNumber << " != "
-              << srcRTP.GetSequenceNumber() << " Will recover.");
+              << sequenceNumber << " Will recover.");
       }
-      expectedSequenceNumber = (u_short)(srcRTP.GetSequenceNumber()+1);
-
-#if DEBUG_OUTPUT
-static int decoderInput = -1;
-debug_write_data(decoderInput, "decoder input", "decoder.input", srcRTP.GetPacketPtr(), srcRTP.GetFrameLen());
-#endif
+      expectedSequenceNumber = (u_short)(sequenceNumber+1);
 
       videoDecoder->mark(now);
       if (!videoDecoder->decode(srcRTP.GetPayloadPtr(), srcRTP.GetPayloadSize(), lostPreviousPacket)) {
@@ -518,7 +438,6 @@ debug_write_data(decoderInput, "decoder input", "decoder.input", srcRTP.GetPacke
 
       int frameBytes = (frameWidth * frameHeight * 12) / 8;
       dstRTP.SetPayloadSize(sizeof(PluginCodec_Video_FrameHeader) + frameBytes);
-      dstRTP.SetPayloadType(RTP_DYNAMIC_PAYLOAD);
       dstRTP.SetMarker(true);
 
       PluginCodec_Video_FrameHeader * frameHeader = (PluginCodec_Video_FrameHeader *)dstRTP.GetPayloadPtr();
@@ -529,13 +448,7 @@ debug_write_data(decoderInput, "decoder input", "decoder.input", srcRTP.GetPacke
 
       videoDecoder->resetndblk();
 
-      dstLen = dstRTP.GetFrameLen();
-
-#if DEBUG_OUTPUT
-static int decoderOutput = -1;
-debug_write_data(decoderOutput, "decoder output", "decoder.yuv", OPAL_VIDEO_FRAME_DATA_PTR(frameHeader),
-                                                                 dstRTP.GetPayloadSize() - sizeof(PluginCodec_Video_FrameHeader));
-#endif
+      dstLen = dstRTP.GetPacketSize();
       flags = PluginCodec_ReturnCoderLastFrame | PluginCodec_ReturnCoderIFrame;   // TODO: THIS NEEDS TO BE CHANGED TO DO CORRECT I-FRAME DETECTION
       return 1;
     }
@@ -566,12 +479,8 @@ static struct PluginCodec_information licenseInfo = {
 };
 
 static const char YUV420PDesc[]  = { "YUV420P" };
-
-static const char h261QCIFDesc[]  = { "H.261-QCIF" };
-static const char h261CIFDesc[]   = { "H.261-CIF" };
-static const char h261Desc[]      = { "H.261" };
-
-static const char sdpH261[]   = { "h261" };
+static const char h261Desc[]     = { "H.261" };
+static const char sdpH261[]      = { "h261" };
 
 static const char QCIF_MPI[]  = PLUGINCODEC_QCIF_MPI;
 static const char  CIF_MPI[]  =  PLUGINCODEC_CIF_MPI;
@@ -840,6 +749,7 @@ static int valid_for_protocol(const struct PluginCodec_Definition *, void *, con
 
 }
 
+PLUGINCODEC_CONTROL_LOG_FUNCTION_DEF
 
 static PluginCodec_ControlDefn h323EncoderControls[] = {
   { PLUGINCODEC_CONTROL_VALID_FOR_PROTOCOL,    valid_for_protocol },
@@ -849,6 +759,7 @@ static PluginCodec_ControlDefn h323EncoderControls[] = {
   { PLUGINCODEC_CONTROL_FREE_CODEC_OPTIONS,    free_codec_options },
   { PLUGINCODEC_CONTROL_SET_CODEC_OPTIONS,     encoder_set_options },
   { PLUGINCODEC_CONTROL_GET_OUTPUT_DATA_SIZE,  encoder_get_output_data_size },
+  PLUGINCODEC_CONTROL_LOG_FUNCTION_INC
   { NULL }
 };
 
@@ -989,152 +900,6 @@ static struct PluginCodec_Option const * const xcifOptionTable[] = {
 
 static struct PluginCodec_Definition h261CodecDefn[] =
 {
-  { 
-    // CIF only encoder
-    PLUGIN_CODEC_VERSION_OPTIONS,       // codec API version
-    &licenseInfo,                       // license information
-
-    PluginCodec_MediaTypeVideo |        // audio codec
-    PluginCodec_RTPTypeExplicit,        // specified RTP type
-
-    h261CIFDesc,                        // text decription
-    YUV420PDesc,                        // source format
-    h261CIFDesc,                        // destination format
-
-    cifOptionTable,                     // user data 
-
-    H261_CLOCKRATE,                     // samples per second
-    H261_BITRATE,                       // raw bits per second
-    20000,                              // nanoseconds per frame
-
-    {{
-      CIF_WIDTH,                          // frame width
-      CIF_HEIGHT,                         // frame height
-      10,                                 // recommended frame rate
-      60,                                 // maximum frame rate
-    }},
-    
-    RTP_RFC2032_PAYLOAD,                // IANA RTP payload code
-    sdpH261,                            // RTP payload name
-
-    create_encoder,                     // create codec function
-    destroy_encoder,                    // destroy codec
-    codec_encoder,                      // encode/decode
-    h323EncoderControls,                // codec controls
-
-    PluginCodec_H323VideoCodec_h261,    // h323CapabilityType 
-    NULL                                // h323CapabilityData
-  },
-  { 
-    // CIF only decoder
-    PLUGIN_CODEC_VERSION_OPTIONS,       // codec API version
-    &licenseInfo,                       // license information
-
-    PluginCodec_MediaTypeVideo |        // audio codec
-    PluginCodec_RTPTypeExplicit,        // specified RTP type
-
-    h261CIFDesc,                        // text decription
-    h261CIFDesc,                        // source format
-    YUV420PDesc,                        // destination format
-
-    cifOptionTable,                     // user data 
-
-    H261_CLOCKRATE,                     // samples per second
-    H261_BITRATE,                       // raw bits per second
-    20000,                              // nanoseconds per frame
-
-    {{
-      CIF_WIDTH,                        // frame width
-      CIF_HEIGHT,                       // frame height
-      10,                               // recommended frame rate
-      60,                               // maximum frame rate
-    }},
-    
-    RTP_RFC2032_PAYLOAD,                // IANA RTP payload code
-    sdpH261,                            // RTP payload name
-
-    create_decoder,                     // create codec function
-    destroy_decoder,                    // destroy codec
-    codec_decoder,                      // encode/decode
-    h323DecoderControls,                // codec controls
-
-    PluginCodec_H323VideoCodec_h261,    // h323CapabilityType 
-    NULL                                // h323CapabilityData
-  },
-
-  { 
-    // QCIF only encoder
-    PLUGIN_CODEC_VERSION_OPTIONS,       // codec API version
-    &licenseInfo,                       // license information
-
-    PluginCodec_MediaTypeVideo |        // audio codec
-    PluginCodec_RTPTypeExplicit,        // specified RTP type
-
-    h261QCIFDesc,                       // text decription
-    YUV420PDesc,                        // source format
-    h261QCIFDesc,                       // destination format
-
-    qcifOptionTable,                    // user data 
-
-    H261_CLOCKRATE,                     // samples per second
-    H261_BITRATE,                       // raw bits per second
-    20000,                              // nanoseconds per frame
-
-    {{
-      QCIF_WIDTH,                         // frame width
-      QCIF_HEIGHT,                        // frame height
-      10,                                 // recommended frame rate
-      60,                                 // maximum frame rate
-    }},
-    
-    RTP_RFC2032_PAYLOAD,                // IANA RTP payload code
-    sdpH261,                            // RTP payload name
-
-    create_encoder,                     // create codec function
-    destroy_encoder,                    // destroy codec
-    codec_encoder,                      // encode/decode
-    h323EncoderControls,                // codec controls
-
-    PluginCodec_H323VideoCodec_h261,    // h323CapabilityType 
-    NULL                                // h323CapabilityData
-  },
-  { 
-    // QCIF only decoder
-    PLUGIN_CODEC_VERSION_OPTIONS,       // codec API version
-    &licenseInfo,                       // license information
-
-    PluginCodec_MediaTypeVideo |        // audio codec
-    PluginCodec_RTPTypeExplicit,        // specified RTP type
-
-    h261QCIFDesc,                       // text decription
-    h261QCIFDesc,                       // source format
-    YUV420PDesc,                        // destination format
-
-    qcifOptionTable,                    // user data 
-
-    H261_CLOCKRATE,                     // samples per second
-    H261_BITRATE,                       // raw bits per second
-    20000,                              // nanoseconds per frame
-
-    {{
-      QCIF_WIDTH,                       // frame width
-      QCIF_HEIGHT,                      // frame height
-      10,                               // recommended frame rate
-      60,                               // maximum frame rate
-    }},
-    
-    RTP_RFC2032_PAYLOAD,                // IANA RTP payload code
-    sdpH261,                            // RTP payload name
-
-    create_decoder,                     // create codec function
-    destroy_decoder,                    // destroy codec
-    codec_decoder,                      // encode/decode
-    h323DecoderControls,                // codec controls
-
-    PluginCodec_H323VideoCodec_h261,    // h323CapabilityType 
-    NULL                                // h323CapabilityData
-  },
-
   { 
     // Both QCIF and CIF (dynamic) encoder
     PLUGIN_CODEC_VERSION_OPTIONS,       // codec API version
@@ -1215,27 +980,6 @@ extern "C" {
 
   PLUGIN_CODEC_DLL_API struct PluginCodec_Definition * PLUGIN_CODEC_GET_CODEC_FN(unsigned * count, unsigned /*version*/)
   {
-#ifndef _WIN32_WCE
-    char * debug_level = getenv ("PTLIB_TRACE_CODECS");
-    if (debug_level!=NULL) {
-      Trace::SetLevel(atoi(debug_level));
-    }
-    else {
-      Trace::SetLevel(0);
-    }
-
-    debug_level = getenv ("PTLIB_TRACE_CODECS_USER_PLANE");
-    if (debug_level!=NULL) {
-      Trace::SetLevelUserPlane(atoi(debug_level));
-    }
-    else {
-      Trace::SetLevelUserPlane(0);
-    }
-#else
-    Trace::SetLevel(0);
-    Trace::SetLevelUserPlane(0);
-#endif
-
     *count = sizeof(h261CodecDefn) / sizeof(struct PluginCodec_Definition);
     return h261CodecDefn;
   }
