@@ -19,14 +19,18 @@
  *
  */
 
-#include <iostream>
-#include <string.h>
-#include <malloc.h>
-using namespace std;
-
 #include "rfc2190.h"
 
-#define MAX_PACKET_LEN 1024
+#include <codec/opalplugin.hpp>
+
+#include <iostream>
+#include <string.h>
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif
+
+
+#include <codec/opalplugin.hpp>
 
 const unsigned char PSC[3]      = { 0x00, 0x00, 0x80 };
 const unsigned char PSC_Mask[3] = { 0xff, 0xff, 0xfc };
@@ -108,76 +112,110 @@ static int FindGBSC(const unsigned char * base, int len, int & sbit)
 
 #endif
 
+///////////////////////////////////////////////////////////////////////////////////////3Y
+
+RFC2190EncodedFrame::RFC2190EncodedFrame()
+  : m_isIFrame(false)
+{
+}
+
+
+bool RFC2190EncodedFrame::IsIntraFrame() const
+{
+  return m_isIFrame;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////////////3Y
 
 RFC2190Packetizer::RFC2190Packetizer()
+  : TR(0)
+  , m_frameSize(0)
+  , annexD(0)
+  , annexE(0)
+  , annexF(0)
+  , annexG(0)
+  , pQuant(0)
+  , cpm(0)
+  , macroblocksPerGOB(0)
+  , fragPtr(NULL)
+  , m_currentMB(0)
+  , m_currentBytes(0)
 {
-  m_buffer = NULL;
 }
 
-RFC2190Packetizer::~RFC2190Packetizer()
-{
-  free(m_buffer);
-}
 
-int RFC2190Packetizer::Open(unsigned long _timestamp, unsigned long /*maxLen*/)
+bool RFC2190Packetizer::SetResolution(unsigned width, unsigned height)
 {
-  // do a sanity check on the fragment data - must be equal to maxLen
-  {
-    unsigned long len = 0;
-    FragmentListType::iterator r;
-    for (r = fragments.begin(); r != fragments.end(); ++r) 
-      len += r->length;
+  m_currentMB = 0;
+  m_currentBytes = 0;
 
-//    if (len != maxLen) 
-//      cout << "rfc2190: mismatch between encoder length and fragment length - " << len << "/" << maxLen << endl;
+  fragments.resize(0);
+
+  size_t newOutputSize = width*height;
+
+  if (m_buffer != NULL && m_maxSize < newOutputSize) {
+    free(m_buffer);
+    m_buffer = NULL;
   }
-  
-  // save timestamp
-  timestamp = _timestamp;
 
-  const unsigned char * data = m_buffer;
-  size_t dataLen = m_bufferSize;
+  if (m_buffer == NULL) {
+    m_maxSize = newOutputSize;
+#if HAVE_POSIX_MEMALIGN
+    if (posix_memalign((void **)&m_buffer, 64, m_maxSize) != 0) 
+#else
+    if ((m_buffer = (unsigned char *)malloc(m_maxSize)) == NULL) 
+#endif
+    {
+      return false;
+    }
+  }
 
+  return true;
+}
+
+
+bool RFC2190Packetizer::Reset(size_t len)
+{
   // make sure data is at least long enough to contain PSC, TR & minimum PTYPE, PQUANT and CPM
-  if (dataLen < 7)
-    return -1;
+  if (len < 7)
+    return false;
 
   // ensure data starts with PSC
   //     0         1         2
   // 0000 0000 0000 0000 1000 00..
-  if (FindPSC(data, dataLen) != 0)
-    return -2;
+  if (FindPSC(m_buffer, len) != 0)
+    return false;
 
   // get TR
   //     2         3    
   // .... ..XX XXXX XX..
-  TR = ((data[2] << 6) & 0xfc) | (data[3] >> 2);
+  TR = ((m_buffer[2] << 6) & 0xfc) | (m_buffer[3] >> 2);
       
   // make sure mandatory part of PTYPE is present
   //     3    
   // .... ..10
-  if ((data[3] & 0x03) != 2)
-    return -3;
+  if ((m_buffer[3] & 0x03) != 2)
+    return false;
 
   // we don't do split screen, document indicator, full picture freeze
   //     4    
   // XXX. ....
-  if ((data[4] & 0xe0) != 0)
-    return -4;
+  if ((m_buffer[4] & 0xe0) != 0)
+    return false;
 
   // get image size
   //     4
   // ...X XX..
-  frameSize = (data[4] >> 2) & 0x7;
-  macroblocksPerGOB = MacroblocksPerGOBTable[frameSize];
+  m_frameSize = (m_buffer[4] >> 2) & 0x7;
+  macroblocksPerGOB = MacroblocksPerGOBTable[m_frameSize];
   if (macroblocksPerGOB == -1)
-    return -6;
+    return false;
 
   // get I-frame flag
   //     4
   // .... ..X.
-  iFrame = (data[4] & 2) == 0;
+  m_isIFrame = (m_buffer[4] & 2) == 0;
 
   // get annex bits:
   //   Annex D - unrestricted motion vector mode
@@ -187,30 +225,30 @@ int RFC2190Packetizer::Open(unsigned long _timestamp, unsigned long /*maxLen*/)
   //
   //     4         5
   // .... ...X XXX. ....
-  annexD = data[4] & 0x01;
-  annexE = data[5] & 0x80;
-  annexF = data[5] & 0x40;
-  annexG = data[5] & 0x20;
+  annexD = m_buffer[4] & 0x01;
+  annexE = m_buffer[5] & 0x80;
+  annexF = m_buffer[5] & 0x40;
+  annexG = m_buffer[5] & 0x20;
 
   // annex G not supported 
   if (annexG)
-    return -5;
+    return false;
 
   // get PQUANT
   //     5
   // ...X XXXX
-  pQuant = data[5] & 0x1f;
+  pQuant = m_buffer[5] & 0x1f;
 
   // get CPM
   //     6
   // X... ....
-  cpm = (data[6] & 0x80) != 0;
+  cpm = (m_buffer[6] & 0x80) != 0;
 
   // ensure PEI is always 0
   //     6
   // .X.. ....
-  if ((data[6] & 0x40) != 0) 
-    return -6;
+  if ((m_buffer[6] & 0x40) != 0) 
+    return false;
 
 #if 0
   cout << "TR=" << TR 
@@ -228,10 +266,10 @@ int RFC2190Packetizer::Open(unsigned long _timestamp, unsigned long /*maxLen*/)
   // split fragments longer than the maximum
   FragmentListType::iterator r;
   for (r = fragments.begin(); r != fragments.end(); ++r) {
-    while (r->length > MAX_PACKET_LEN) {
+    while (r->length > m_maxPayloadSize) {
       int oldLen = r->length;
-      int newLen = MAX_PACKET_LEN;
-      if ((oldLen - newLen) < MAX_PACKET_LEN)
+      size_t newLen = m_maxPayloadSize;
+      if ((oldLen - newLen) < m_maxPayloadSize)
         newLen = oldLen / 2;
       fragment oldFrag = *r;
       r = fragments.erase(r);
@@ -252,137 +290,149 @@ int RFC2190Packetizer::Open(unsigned long _timestamp, unsigned long /*maxLen*/)
   currFrag = fragments.begin();
   fragPtr = m_buffer;
 
-  return 0;
+  return FFMPEGCodec::EncodedFrame::Reset(len);
 }
 
-int RFC2190Packetizer::GetPacket(RTPFrame & outputFrame, unsigned int & flags)
+
+bool RFC2190Packetizer::AddPacket(const PluginCodec_RTP &, unsigned &)
 {
-  while ((fragments.size() != 0) && (currFrag != fragments.end())) {
+  return false;
+}
+
+
+bool RFC2190Packetizer::GetPacket(PluginCodec_RTP & outputFrame, unsigned int & flags)
+{
+  outputFrame.SetPayloadSize(0);
+  if (fragments.empty() || currFrag == fragments.end())
+    return false;
       
-    // set the timestamp
-    outputFrame.SetTimestamp(timestamp);
-    fragment frag = *currFrag++;
+  fragment frag = *currFrag++;
 
+  // if this fragment starts with a GBSC, then output as Mode A else output as Mode B
+  bool modeA = ((frag.length >= 3) &&
+                (fragPtr[0] == 0x00) &&
+                (fragPtr[1] == 0x00) &&
+                ((fragPtr[2] & 0x80) == 0x80));
 
-    // if this fragment starts with a GBSC, then output as Mode A else output as Mode B
-    bool modeA = ((frag.length >= 3) &&
-                  (fragPtr[0] == 0x00) &&
-                  (fragPtr[1] == 0x00) &&
-                  ((fragPtr[2] & 0x80) == 0x80));
+  // offset of the data
+  size_t headerSize = modeA ? 4 : 8;
 
-    size_t payloadRemaining = outputFrame.GetFrameLen() - outputFrame.GetHeaderSize();
-
-    // offset of the data
-    size_t offs = modeA ? 4 : 8;
-
-    // make sure RTP storage is sufficient
-    if ((frag.length + offs) > payloadRemaining) {
-      //std::cout << "no room for Mode " << (modeA ? 'A' : 'B') << " frame - " << (frag.length+offs) << " > " << payloadRemaining << std::endl;
-      continue;
-    }
-
-    // set size of final frame
-    outputFrame.SetPayloadSize(offs + frag.length);
-
-    // get ptr to payload that is about to be created
-    unsigned char * ptr = outputFrame.GetPayloadPtr();
-
-    if (modeA) {
-      int sBit = 0;
-      int eBit = 0;
-      ptr[0] = (unsigned char)(((sBit & 7) << 3) | (eBit & 7));
-      ptr[1] = (unsigned char)((frameSize << 5) | (iFrame ? 0 : 0x10) | (annexD ? 0x08 : 0) | (annexE ? 0x04 : 0) | (annexF ? 0x02 : 0));
-      ptr[2] = ptr[3] = 0;
-    }
-    else
-    {
-      // create the Mode B header
-      int sBit = 0;
-      int eBit = 0;
-      int gobn = frag.mbNum / macroblocksPerGOB;
-      int mba  = frag.mbNum % macroblocksPerGOB;
-      ptr[0] = (unsigned char)(0x80 | ((sBit & 7) << 3) | (eBit & 7));
-      ptr[1] = (unsigned char)(frameSize << 5);
-      ptr[2] = (unsigned char)(((gobn << 3) & 0xf8) | ((mba >> 6) & 0x7));
-      ptr[3] = (unsigned char)((mba << 2) & 0xfc);
-      ptr[4] = (iFrame ? 0 : 0x80) | (annexD ? 0x40 : 0) | (annexE ? 0x20 : 0) | (annexF ? 0x010: 0);
-      ptr[5] = ptr[6] = ptr[7] = 0;
-    }
-
-    // copy the data
-    memcpy(ptr + offs, fragPtr, frag.length);
-
-    fragPtr += frag.length;
-
-    // set marker bit
-    flags = 0;
-    if (currFrag == fragments.end()) {
-      flags |= 1;
-      outputFrame.SetMarker(1);
-    }
-    if (iFrame)
-      flags |= 2;
-
-    return 1;
+  // make sure RTP storage is sufficient
+  size_t payloadRequired = headerSize + frag.length;
+  if (!outputFrame.SetPayloadSize(payloadRequired)) {
+    size_t payloadRemaining = outputFrame.GetMaxSize() - outputFrame.GetHeaderSize();
+    PTRACE(2, "RFC2190", "Possible truncation of packet: " << payloadRequired << " > " << payloadRemaining);
+    frag.length = payloadRemaining - headerSize;
   }
 
-  return 0;
+  // get ptr to payload that is about to be created
+  unsigned char * ptr = outputFrame.GetPayloadPtr();
+
+  int sBit = 0;
+  int eBit = 0;
+  if (modeA) {
+    ptr[0] = (unsigned char)(((sBit & 7) << 3) | (eBit & 7));
+    ptr[1] = (unsigned char)((m_frameSize << 5) | (m_isIFrame ? 0 : 0x10) | (annexD ? 0x08 : 0) | (annexE ? 0x04 : 0) | (annexF ? 0x02 : 0));
+    ptr[2] = ptr[3] = 0;
+  }
+  else
+  {
+    // create the Mode B header
+    int gobn = frag.mbNum / macroblocksPerGOB;
+    int mba  = frag.mbNum % macroblocksPerGOB;
+    ptr[0] = (unsigned char)(0x80 | ((sBit & 7) << 3) | (eBit & 7));
+    ptr[1] = (unsigned char)(m_frameSize << 5);
+    ptr[2] = (unsigned char)(((gobn << 3) & 0xf8) | ((mba >> 6) & 0x7));
+    ptr[3] = (unsigned char)((mba << 2) & 0xfc);
+    ptr[4] = (m_isIFrame ? 0 : 0x80) | (annexD ? 0x40 : 0) | (annexE ? 0x20 : 0) | (annexF ? 0x010: 0);
+    ptr[5] = ptr[6] = ptr[7] = 0;
+  }
+
+  // copy the data
+  memcpy(ptr + headerSize, fragPtr, frag.length);
+
+  fragPtr += frag.length;
+
+  // set marker bit
+  if (currFrag == fragments.end()) {
+    flags |= PluginCodec_ReturnCoderLastFrame;
+    outputFrame.SetMarker(1);
+  }
+
+  return true;
 }
 
+
+void RFC2190Packetizer::RTPCallBack(void * data, int size, int mbCount)
+{
+  // sometimes, FFmpeg encodes the same frame multiple times
+  // we need to detect this in order to avoid duplicating the encoded data
+  if ((data == m_buffer) && (fragments.size() != 0)) {
+    m_currentMB = 0;
+    m_currentBytes = 0;
+    fragments.resize(0);
+  }
+
+  // add the fragment to the list
+  fragment frag;
+  frag.length = size;
+  frag.mbNum  = m_currentMB;
+  fragments.push_back(frag);
+
+  m_currentMB += mbCount;
+  m_currentBytes += size;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////////////3Y
 
 RFC2190Depacketizer::RFC2190Depacketizer()
 {
-  NewFrame();
+  Reset();
 }
 
-void RFC2190Depacketizer::NewFrame()
+void RFC2190Depacketizer::Reset()
 {
-  frame.resize(0);
-  first               = true;
-  skipUntilEndOfFrame = false;
-  lastEbit            = 8;
+  RFC2190EncodedFrame::Reset();
+
+  m_first               = true;
+  m_skipUntilEndOfFrame = false;
+  m_lastEbit            = 8;
+  m_isIFrame            = false;
 }
 
-int RFC2190Depacketizer::LostSync(bool & requestIframe, const char * /*reason*/)
+
+bool RFC2190Depacketizer::GetPacket(PluginCodec_RTP &, unsigned &)
 {
-  skipUntilEndOfFrame = true;
-  requestIframe = true;
-  return 0;
+  return false;
 }
 
-int RFC2190Depacketizer::SetPacket(const RTPFrame & inputFrame, bool & requestIFrame, bool & isIFrame)
-{
-  requestIFrame = false;
-  isIFrame      = false;
 
+bool RFC2190Depacketizer::LostSync(unsigned & flags)
+{
+  m_skipUntilEndOfFrame = true;
+  flags |= PluginCodec_ReturnCoderRequestIFrame;
+  PTRACE(2, GetName(), "Error in received packet, resynchronising.");
+  return true;
+}
+
+
+bool RFC2190Depacketizer::AddPacket(const PluginCodec_RTP & packet, unsigned & flags)
+{
   // ignore packets if required
-  if (skipUntilEndOfFrame) {
-    if (inputFrame.GetMarker()) 
-      NewFrame();
-    return 0;
+  if (m_skipUntilEndOfFrame) {
+    if (packet.GetMarker()) 
+      Reset();
+    return true;
   }
 
-  // check if packet is in sequence. If not, skip til end of frame 
-  if (first) {
-    NewFrame();    // make sure this is called before "first = false"
-    first = false;
-    lastSequence = inputFrame.GetSequenceNumber();
-  }
-  else {
-    ++lastSequence;
-    if (inputFrame.GetSequenceNumber() != lastSequence) {
-      return LostSync(requestIFrame, "missed frame");
-    }
-  }
-
-  unsigned payloadLen = inputFrame.GetPayloadSize();
+  unsigned payloadLen = packet.GetPayloadSize();
 
   // payload must be at least as long as mode A header + 1 byte
-  if (payloadLen < 5) 
-    return LostSync(requestIFrame, "payload too small");
+  if (payloadLen < 5)
+    return LostSync(flags);
 
-  unsigned char * payload = inputFrame.GetPayloadPtr();
+  unsigned char * payload = packet.GetPayloadPtr();
   unsigned int sbit = (payload[0] >> 3) & 0x07;
   unsigned hdrLen;
 
@@ -390,27 +440,16 @@ int RFC2190Depacketizer::SetPacket(const RTPFrame & inputFrame, bool & requestIF
 
   // handle mode A frames
   if ((payload[0] & 0x80) == 0) {
-    isIFrame = (payload[1] & 0x10) == 0;
+    m_isIFrame = (payload[1] & 0x10) == 0;
     hdrLen = 4;
     mode = 'A';
-
-#if 0
-    // sanity check data
-    if (payloadLen < (hdrLen+3) ||
-        (payload[hdrLen+0] != 0x00) ||
-        (payload[hdrLen+1] != 0x00) ||
-        ((payload[hdrLen+2] & 0x80) != 0x80)
-       ) {
-      return LostSync(requestIFrame, "Mode A packet not starting with GBSC");
-    }
-#endif
   }
 
   // handle mode B frames
   else if ((payload[0] & 0x40) == 0) {
     if (payloadLen < 9)
-      return LostSync(requestIFrame, "mode B payload too small");
-    isIFrame = (payload[4] & 0x80) == 0;
+      return LostSync(flags);
+    m_isIFrame = (payload[4] & 0x80) == 0;
     hdrLen = 8;
     mode = 'B';
   }
@@ -418,44 +457,37 @@ int RFC2190Depacketizer::SetPacket(const RTPFrame & inputFrame, bool & requestIF
   // handle mode C frames
   else {
     if (payloadLen < 13)
-      return LostSync(requestIFrame, "mode C payload too small");
-    isIFrame = (payload[4] & 0x80) == 0;
+      return LostSync(flags);
+    m_isIFrame = (payload[4] & 0x80) == 0;
     hdrLen = 12;
     mode = 'C';
   }
 
   // if ebit and sbit do not add up, then we have lost sync
-  if (((sbit + lastEbit) & 0x7) != 0) {
-    return LostSync(requestIFrame, "mismatched ebit and sbit");
-  }
+  if (((sbit + m_lastEbit) & 0x7) != 0)
+    return LostSync(flags);
 
   unsigned char * src = payload + hdrLen;
   size_t cpyLen = payloadLen - hdrLen;
 
   // handle first partial byte
-  if ((sbit != 0) && (frame.size() > 0)) {
+  if ((sbit != 0) && m_length > 0) {
     
     static unsigned char smasks[7] = { 0x7f, 0x3f, 0x1f, 0x0f, 0x07, 0x03, 0x01 };
     unsigned smask = smasks[sbit-1];
-    frame[frame.size()-1] |= (*src & smask);
+    m_buffer[m_length-1] |= (*src & smask);
     --cpyLen;
     ++src;
   }
 
-  // copy whole bytes
-  if (cpyLen > 0) {
-    size_t frameSize = frame.size();
-    frame.resize(frameSize + cpyLen);
-    memcpy(&frame[0] + frameSize, src, cpyLen);
-  }
-
   // keep ebit for next time
-  lastEbit = payload[0] & 0x07;
+  m_lastEbit = payload[0] & 0x07;
+
+  // copy whole bytes
+  if (cpyLen > 0)
+    return Append(src,cpyLen);
 
   // return 0 if no frame yet, return 1 if frame is available
-  if (!inputFrame.GetMarker()) 
-    return 0;
-
-  return 1;
+  return true;
 }
 
