@@ -35,10 +35,13 @@
 
  */
 
-#include "theora_plugin.h"
+#ifndef PLUGIN_CODEC_DLL_EXPORTS
+#include "plugin-config.h"
+#endif
 
-#include "trace.h"
-#include "rtpframe.h"
+#include <codec/opalplugin.hpp>
+
+#include "theora_plugin.h"
 
 #include <stdlib.h>
 #if defined(_WIN32) || defined(_WIN32_WCE)
@@ -49,6 +52,7 @@
   #define STRCMPI  strcasecmp
 #endif
 #include <string.h>
+#include <stdio.h>
 
 theoraEncoderContext::theoraEncoderContext()
 {
@@ -153,10 +157,10 @@ int theoraEncoderContext::EncodeFrames(const u_char * src, unsigned & srcLen, u_
   ogg_packet framePacket;
 
   // create RTP frame from source buffer
-  RTPFrame srcRTP(src, srcLen);
+  PluginCodec_RTP srcRTP(src, srcLen);
 
   // create RTP frame from destination buffer
-  RTPFrame dstRTP(dst, dstLen);
+  PluginCodec_RTP dstRTP(dst, dstLen);
 
   dstLen = 0;
 
@@ -170,20 +174,13 @@ int theoraEncoderContext::EncodeFrames(const u_char * src, unsigned & srcLen, u_
   if  (_txTheoraFrame->HasRTPFrames())
   {
     _txTheoraFrame->GetRTPFrame(dstRTP, flags);
-    dstLen = dstRTP.GetFrameLen();
+    dstLen = dstRTP.GetPacketSize();
     return 1;
   }
 
-  if (srcRTP.GetPayloadSize() < sizeof(frameHeader))
-  {
-   TRACE(1, "THEORA\tEncoder\tVideo grab too small, Close down video transmission thread");
-   return 0;
-  }
-
-  frameHeader * header = (frameHeader *)srcRTP.GetPayloadPtr();
-  if (header->x != 0 || header->y != 0)
-  {
-    TRACE(1, "THEORA\tEncoder\tVideo grab of partial frame unsupported, Close down video transmission thread");
+  PluginCodec_Video_FrameHeader * header = srcRTP.GetVideoHeader();
+  if (header->x != 0 || header->y != 0) {
+    PTRACE(1, "THEORA", "Encoder\tVideo grab of partial frame unsupported, Close down video transmission thread");
     return 0;
   }
 
@@ -207,27 +204,26 @@ int theoraEncoderContext::EncodeFrames(const u_char * src, unsigned & srcLen, u_
   yuv.uv_height = (int) (_theoraInfo.height / 2);
   yuv.y_stride  = header->width;
   yuv.uv_stride = (int) (header->width /2);
-  yuv.y         = (unsigned char *)(((unsigned char *)header) + sizeof(frameHeader));
-  yuv.u         = (unsigned char *)((((unsigned char *)header) + sizeof(frameHeader)) 
-                           + (int)(yuv.y_stride*header->height)); 
-  yuv.v         = (unsigned char *)(yuv.u + (int)(yuv.uv_stride *header->height/2)); 
+  yuv.y         = dstRTP.GetVideoFrameData();
+  yuv.u         = yuv.y + yuv.y_stride*header->height;
+  yuv.v         = yuv.u + yuv.uv_stride*header->height/2;
 
   ret = theora_encode_YUVin( &_theoraState, &yuv );
   if (ret != 0) {
     if (ret == -1) {
-      TRACE(1, "THEORA\tEncoder\tEncoding failed: The size of the given frame differs from those previously input (should not happen)")
+      PTRACE(1, "THEORA", "Encoder\tEncoding failed: The size of the given frame differs from those previously input (should not happen)");
     } else {
-      TRACE(1, "THEORA\tEncoder\tEncoding failed: " << theoraErrorMessage(ret));
+      PTRACE(1, "THEORA", "Encoder\tEncoding failed: " << theoraErrorMessage(ret));
     }
     return 0;
   }
 
   ret = theora_encode_packetout( &_theoraState, 0 /* not last Packet */, &framePacket );
   switch (ret) {
-    case  0: TRACE(1, "THEORA\tEncoder\tEncoding failed (packet): No internal storage exists OR no packet is ready"); return 0; break;
-    case -1: TRACE(1, "THEORA\tEncoder\tEncoding failed (packet): The encoding process has completed but something is not ready yet"); return 0; break;
-    case  1: TRACE_UP(4, "THEORA\tEncoder\tSuccessfully encoded OGG packet of " << framePacket.bytes << " bytes"); break;
-    default: TRACE(1, "THEORA\tEncoder\tEncoding failed (packet): " << theoraErrorMessage(ret)); return 0; break;
+    case  0: PTRACE(1, "THEORA", "Encoder\tEncoding failed (packet): No internal storage exists OR no packet is ready"); return 0; break;
+    case -1: PTRACE(1, "THEORA", "Encoder\tEncoding failed (packet): The encoding process has completed but something is not ready yet"); return 0; break;
+    case  1: PTRACE(4, "THEORA", "Encoder\tSuccessfully encoded OGG packet of " << framePacket.bytes << " bytes"); break;
+    default: PTRACE(1, "THEORA", "Encoder\tEncoding failed (packet): " << theoraErrorMessage(ret)); return 0; break;
   }
 
   _txTheoraFrame->SetFromFrame(&framePacket);
@@ -238,7 +234,7 @@ int theoraEncoderContext::EncodeFrames(const u_char * src, unsigned & srcLen, u_
   if (_txTheoraFrame->HasRTPFrames())
   {
     _txTheoraFrame->GetRTPFrame(dstRTP, flags);
-    dstLen = dstRTP.GetFrameLen();
+    dstLen = dstRTP.GetPacketSize();
     return 1;
   }
 
@@ -279,10 +275,10 @@ int theoraDecoderContext::DecodeFrames(const u_char * src, unsigned & srcLen, u_
   WaitAndSignal m(_mutex);
 
   // create RTP frame from source buffer
-  RTPFrame srcRTP(src, srcLen);
+  PluginCodec_RTP srcRTP(src, srcLen);
 
   // create RTP frame from destination buffer
-  RTPFrame dstRTP(dst, dstLen, 0);
+  PluginCodec_RTP dstRTP(dst, dstLen);
   dstLen = 0;
 
   if (!_rxTheoraFrame->SetFromRTPFrame(srcRTP, flags)) {
@@ -309,12 +305,12 @@ int theoraDecoderContext::DecodeFrames(const u_char * src, unsigned & srcLen, u_
     _rxTheoraFrame->GetOggPacket (&oggPacket);
     if (theora_packet_isheader(&oggPacket)) {
 
-      TRACE_UP(4, "THEORA\tDecoder\tGot OGG header packet with size " << oggPacket.bytes);
+      PTRACE(4, "THEORA", "Decoder\tGot OGG header packet with size " << oggPacket.bytes);
 
       // In case we receive new header packets when the stream is already established
       // we have to wait to get both table and header packet until reopening the decoder
       if (_gotHeader && _gotTable) {
-        TRACE(4, "THEORA\tDecoder\tGot OGG header packet after stream was established");
+        PTRACE(4, "THEORA", "Decoder\tGot OGG header packet after stream was established");
         theora_clear( &_theoraState );
         theora_info_clear( &_theoraInfo );
         theora_info_init( &_theoraInfo );
@@ -327,7 +323,7 @@ int theoraDecoderContext::DecodeFrames(const u_char * src, unsigned & srcLen, u_
       theoraComment.vendor = (char*) "theora";
       ret = theora_decode_header( &_theoraInfo, &theoraComment, &oggPacket );
       if (ret != 0) {
-        TRACE(1, "THEORA\tDecoder\tDecoding failed (header packet): " << theoraErrorMessage(ret));
+        PTRACE(1, "THEORA", "Decoder\tDecoding failed (header packet): " << theoraErrorMessage(ret));
         flags = (_gotAGoodFrame ? requestIFrame : 0);
         _gotAGoodFrame = false;
         return 1;
@@ -348,10 +344,10 @@ int theoraDecoderContext::DecodeFrames(const u_char * src, unsigned & srcLen, u_
 
         if (theora_packet_iskeyframe(&oggPacket)) {
 
-          TRACE_UP(4, "THEORA\tDecoder\tGot OGG keyframe data packet with size " << oggPacket.bytes);
+          PTRACE(4, "THEORA", "Decoder\tGot OGG keyframe data packet with size " << oggPacket.bytes);
           ret = theora_decode_packetin( &_theoraState, &oggPacket );
           if (ret != 0) {
-            TRACE(1, "THEORA\tDecoder\tDecoding failed (packet): " << theoraErrorMessage(ret));
+            PTRACE(1, "THEORA", "Decoder\tDecoding failed (packet): " << theoraErrorMessage(ret));
             flags = (_gotAGoodFrame ? requestIFrame : 0);
             _gotAGoodFrame = false;
             return 1;
@@ -364,10 +360,10 @@ int theoraDecoderContext::DecodeFrames(const u_char * src, unsigned & srcLen, u_
 
           if  (_gotIFrame) {
 
-            TRACE_UP(4, "THEORA\tDecoder\tGot OGG non-keyframe data packet with size " << oggPacket.bytes);
+            PTRACE(4, "THEORA", "Decoder\tGot OGG non-keyframe data packet with size " << oggPacket.bytes);
             ret = theora_decode_packetin( &_theoraState, &oggPacket );
             if (ret != 0) {
-              TRACE(1, "THEORA\tDecoder\tDecoding failed (packet): " << theoraErrorMessage(ret));
+              PTRACE(1, "THEORA", "Decoder\tDecoding failed (packet): " << theoraErrorMessage(ret));
               flags = (_gotAGoodFrame ? requestIFrame : 0);
               _gotAGoodFrame = false;
               return 1;
@@ -377,7 +373,7 @@ int theoraDecoderContext::DecodeFrames(const u_char * src, unsigned & srcLen, u_
           } 
           else {
 
-            TRACE(1, "THEORA\tDecoder\tGot OGG non-keyframe data Packet but still waiting for keyframe");
+            PTRACE(1, "THEORA", "Decoder\tGot OGG non-keyframe data Packet but still waiting for keyframe");
             flags = (_gotAGoodFrame ? requestIFrame : 0);
             _gotAGoodFrame = false;
             return 1;
@@ -386,72 +382,68 @@ int theoraDecoderContext::DecodeFrames(const u_char * src, unsigned & srcLen, u_
       } 
       else {
 
-        TRACE(1, "THEORA\tDecoder\tGot OGG data packet but still waiting for header and/or table Packets");
+        PTRACE(1, "THEORA", "Decoder\tGot OGG data packet but still waiting for header and/or table Packets");
         return 0;
       }
     }
   }
 
-  TRACE_UP(4, "THEORA\tDecoder\tNo more OGG packets to decode");
+  PTRACE(4, "THEORA", "Decoder\tNo more OGG packets to decode");
 
-  if (gotFrame) {
-
-    int size = _theoraInfo.width * _theoraInfo.height;
-    int frameBytes = (int) (size * 3 / 2);
-    PluginCodec_Video_FrameHeader * header = (PluginCodec_Video_FrameHeader *)dstRTP.GetPayloadPtr();
-
-    TRACE_UP(4, "THEORA\tDecoder\tDecoded Frame with resolution: " << _theoraInfo.width << "x" << _theoraInfo.height);
-
-    header->x = header->y = 0;
-    header->width = _theoraInfo.width;
-    header->height = _theoraInfo.height;
-
-    unsigned int i = 0;
-    int width2 = (header->width >> 1);
-
-    uint8_t* dstY = (uint8_t*) OPAL_VIDEO_FRAME_DATA_PTR(header);
-    uint8_t* dstU = (uint8_t*) dstY + size;
-    uint8_t* dstV = (uint8_t*) dstU + (size >> 2);
-
-    uint8_t* srcY = yuv.y;
-    uint8_t* srcU = yuv.u;
-    uint8_t* srcV = yuv.v;
-
-    for (i = 0 ; i < header->height ; i+=2) {
-
-      memcpy (dstY, srcY, header->width); 
-      srcY +=yuv.y_stride; 
-      dstY +=header->width;
-
-      memcpy (dstY, srcY, header->width); 
-      srcY +=yuv.y_stride; 
-      dstY +=header->width;
-
-      memcpy(dstU, srcU, width2); 
-      srcU+=yuv.uv_stride;
-      dstU+=width2; 
-
-      memcpy (dstV, srcV, width2); 
-      srcV += yuv.uv_stride;
-      dstV +=width2; 
-    }
-
-    dstRTP.SetPayloadSize(sizeof(PluginCodec_Video_FrameHeader) + frameBytes);
-    dstRTP.SetTimestamp(srcRTP.GetTimestamp());
-    dstRTP.SetMarker(1);
-    dstLen = dstRTP.GetFrameLen();
-    flags = PluginCodec_ReturnCoderLastFrame;
-    _frameCounter++;
-    _gotAGoodFrame = true;
-    return 1;
-  } 
-  else { /*gotFrame */
-
-    TRACE(1, "THEORA\tDecoder\tDid not get a decoded frame");
+  if (!gotFrame) {
+    PTRACE(1, "THEORA", "Decoder\tDid not get a decoded frame");
     flags = (_gotAGoodFrame ? requestIFrame : 0);
     _gotAGoodFrame = false;
     return 1;
   }
+
+  PTRACE(4, "THEORA", "Decoder\tDecoded Frame with resolution: " << _theoraInfo.width << "x" << _theoraInfo.height);
+
+  int size = _theoraInfo.width * _theoraInfo.height;
+  int frameBytes = (int) (size * 3 / 2);
+
+  PluginCodec_Video_FrameHeader * header = dstRTP.GetVideoHeader();
+  header->x = header->y = 0;
+  header->width = _theoraInfo.width;
+  header->height = _theoraInfo.height;
+
+  unsigned int i = 0;
+  int width2 = (header->width >> 1);
+
+  uint8_t* dstY = (uint8_t*) dstRTP.GetVideoFrameData();
+  uint8_t* dstU = (uint8_t*) dstY + size;
+  uint8_t* dstV = (uint8_t*) dstU + (size >> 2);
+
+  uint8_t* srcY = yuv.y;
+  uint8_t* srcU = yuv.u;
+  uint8_t* srcV = yuv.v;
+
+  for (i = 0 ; i < header->height ; i+=2) {
+
+    memcpy (dstY, srcY, header->width); 
+    srcY +=yuv.y_stride; 
+    dstY +=header->width;
+
+    memcpy (dstY, srcY, header->width); 
+    srcY +=yuv.y_stride; 
+    dstY +=header->width;
+
+    memcpy(dstU, srcU, width2); 
+    srcU+=yuv.uv_stride;
+    dstU+=width2; 
+
+    memcpy (dstV, srcV, width2); 
+    srcV += yuv.uv_stride;
+    dstV +=width2; 
+  }
+
+  dstRTP.SetPayloadSize(sizeof(PluginCodec_Video_FrameHeader) + frameBytes);
+  dstRTP.SetMarker(true);
+  dstLen = dstRTP.GetPacketSize();
+  flags = PluginCodec_ReturnCoderLastFrame;
+  _frameCounter++;
+  _gotAGoodFrame = true;
+  return 1;
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -595,8 +587,8 @@ static int to_normalised_options(const struct PluginCodec_Definition *, void *,
     capHeight = 480;
   }
 
-  frameWidth = min (capWidth, frameWidth);
-  frameHeight = min (capHeight, frameHeight);
+  frameWidth = std::min(capWidth, frameWidth);
+  frameHeight = std::min(capHeight, frameHeight);
     
   frameWidth -= frameWidth % 16;
   frameHeight -= frameHeight % 16;
@@ -636,8 +628,8 @@ static int to_customised_options(const struct PluginCodec_Definition *, void *,
 
   }
 
-  capWidth = min (capWidth, maxWidth);
-  capHeight = min (capHeight, maxHeight);
+  capWidth = std::min(capWidth, maxWidth);
+  capHeight = std::min(capHeight, maxHeight);
 
   capWidth -= capWidth % 16;
   capHeight -= capHeight % 16;
@@ -684,7 +676,7 @@ static int encoder_set_options(
 	 context->SetMaxRTPFrameSize (atoi(options[i+1]));
       if (STRCMPI(options[i], PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD) == 0)
 	 context->SetMaxKeyFramePeriod (atoi(options[i+1]));
-      TRACE (4, "THEORA\tEncoder\tOption " << options[i] << " = " << atoi(options[i+1]));
+      PTRACE(4, "THEORA", "Encoder\tOption " << options[i] << " = " << atoi(options[i+1]));
     }
     context->ApplyOptions();
   }
@@ -738,22 +730,6 @@ PLUGIN_CODEC_IMPLEMENT(THEORA)
 
 PLUGIN_CODEC_DLL_API struct PluginCodec_Definition * PLUGIN_CODEC_GET_CODEC_FN(unsigned * count, unsigned version)
 {
-  char * debug_level = getenv ("PTLIB_TRACE_CODECS");
-  if (debug_level!=NULL) {
-    Trace::SetLevel(atoi(debug_level));
-  } 
-  else {
-    Trace::SetLevel(0);
-  }
-
-  debug_level = getenv ("PTLIB_TRACE_CODECS_USER_PLANE");
-  if (debug_level!=NULL) {
-    Trace::SetLevelUserPlane(atoi(debug_level));
-  } 
-  else {
-    Trace::SetLevelUserPlane(0);
-  }
-
   if (version < PLUGIN_CODEC_VERSION_VIDEO) {
     *count = 0;
     return NULL;
