@@ -37,6 +37,7 @@
 
 
 #define LOCAL_MEDIA 0
+//#define STUN_SERVER "stun.voxgratia.org"
 
 
 #if defined(_WIN32)
@@ -51,23 +52,31 @@
 
   #define OPEN_LIBRARY(name)             LoadLibrary(name)
   #define GET_LIBRARY_FUNCTION(dll, fn)  GetProcAddress(dll, fn)
+  #define GET_ERRNO GetLastError()
 
   HINSTANCE hDLL;
 
   #define snprintf _snprintf
+  #define strncasecmp strnicmp
+
+  #define TRACE_FILE "debugstream"
 
 #else // _WIN32
 
   #include <memory.h>
   #include <dlfcn.h>
+  #include <errno.h>
 
   #define OPAL_DLL "libopal.so"
 
   #define OPEN_LIBRARY(name)             dlopen(name, RTLD_NOW|RTLD_GLOBAL)
   #define GET_LIBRARY_FUNCTION(dll, fn)  dlsym(dll, (const char *)(fn));
+  #define GET_ERRNO errno
 
   void * hDLL;
 
+  #define TRACE_FILE "trace.txt"
+
 #endif // _WIN32
 
 OpalInitialiseFunction  InitialiseFunction;
@@ -155,9 +164,11 @@ int InitialiseOPAL()
   OpalMessage * response;
   unsigned      version;
 
+  static const char OPALOptions[] = OPAL_PREFIX_ALL " TraceLevel=4 TraceFile=" TRACE_FILE " TraceAppend";
+
 
   if ((hDLL = OPEN_LIBRARY(OPAL_DLL)) == NULL) {
-    fprintf(stderr, "Could not file %s\n", OPAL_DLL);
+    fprintf(stderr, "Could not file %s, error=%u\n", OPAL_DLL, GET_ERRNO);
     return 0;
   }
 
@@ -180,31 +191,21 @@ int InitialiseOPAL()
   ///////////////////////////////////////////////
   // Initialisation
 
-#if LOCAL_MEDIA
-  #define LOCAL_PREFIX OPAL_PREFIX_LOCAL
-#else
-  #define LOCAL_PREFIX OPAL_PREFIX_PCSS
-#endif
-
   version = OPAL_C_API_VERSION;
-  if ((hOPAL = InitialiseFunction(&version,
-                                  OPAL_PREFIX_H323  " "
-                                  OPAL_PREFIX_SIP   " "
-                                  OPAL_PREFIX_IAX2  " "
-                                  LOCAL_PREFIX      " "
-                                  OPAL_PREFIX_IVR
-                                  " TraceLevel=4")) == NULL) {
+  if ((hOPAL = InitialiseFunction(&version, OPALOptions)) == NULL) {
     fputs("Could not initialise OPAL\n", stderr);
     return 0;
   }
 
-
   // General options
   memset(&command, 0, sizeof(command));
   command.m_type = OpalCmdSetGeneralParameters;
   //command.m_param.m_general.m_audioRecordDevice = "Camera Microphone (2- Logitech";
   command.m_param.m_general.m_autoRxMedia = command.m_param.m_general.m_autoTxMedia = "audio";
-  command.m_param.m_general.m_stunServer = "stun.voxgratia.org";
+#ifdef STUN_SERVER
+  command.m_param.m_general.m_natMethod = "STUN";
+  command.m_param.m_general.m_natServer = STUN_SERVER;
+#endif
   command.m_param.m_general.m_mediaMask = "RFC4175*";
 
 #if LOCAL_MEDIA
@@ -253,7 +254,7 @@ static void HandleMessages(unsigned timeout)
   OpalMessage * message;
     
 
-  while ((message = GetMessageFunction(hOPAL, timeout)) != NULL) {
+  while ((message = GetMessageFunction(hOPAL, timeout*1000)) != NULL) {
     switch (message->m_type) {
       case OpalIndRegistration :
         switch (message->m_param.m_registrationStatus.m_status) {
@@ -315,7 +316,9 @@ static void HandleMessages(unsigned timeout)
         if (CurrentCallToken == NULL) {
           memset(&command, 0, sizeof(command));
           command.m_type = OpalCmdAnswerCall;
-          command.m_param.m_callToken = message->m_param.m_incomingCall.m_callToken;
+          command.m_param.m_answerCall.m_callToken = message->m_param.m_incomingCall.m_callToken;
+          command.m_param.m_answerCall.m_overrides.m_userName = "test123";
+          command.m_param.m_answerCall.m_overrides.m_displayName = "Test Called Party";
           if ((response = MySendCommand(&command, "Could not answer call")) != NULL)
             FreeMessageFunction(response);
         }
@@ -371,6 +374,47 @@ static void HandleMessages(unsigned timeout)
           printf("Call cleared: %s\n", message->m_param.m_callCleared.m_reason);
         break;
 
+      case OpalIndPresenceChange :
+        printf("Presence change:\n"
+               "state        = %d\n"
+               "entity       = %s\n"
+               "target       = %s\n"
+               "service      = \"%s\"\n"
+               "contact      = \"%s\"",
+               message->m_param.m_presenceStatus.m_state,
+               message->m_param.m_presenceStatus.m_entity,
+               message->m_param.m_presenceStatus.m_target,
+               message->m_param.m_presenceStatus.m_service,
+               message->m_param.m_presenceStatus.m_contact);
+        if (message->m_param.m_presenceStatus.m_capabilities != NULL && *message->m_param.m_presenceStatus.m_capabilities != '\0')
+          printf(strchr(message->m_param.m_presenceStatus.m_capabilities, '\n') != NULL
+                      ? "%s:\n%s" : "%s = \"%s\"", "\ncapabilities", message->m_param.m_presenceStatus.m_capabilities);
+        if (message->m_param.m_presenceStatus.m_activities != NULL && *message->m_param.m_presenceStatus.m_activities != '\0')
+          printf(strchr(message->m_param.m_presenceStatus.m_activities, '\n') != NULL
+                      ? "%s:\n%s" : "%s   = \"%s\"", "\nactivities", message->m_param.m_presenceStatus.m_activities);
+        if (message->m_param.m_presenceStatus.m_note != NULL && *message->m_param.m_presenceStatus.m_note != '\0')
+          printf(strchr(message->m_param.m_presenceStatus.m_note, '\n') != NULL
+                      ? "%s:\n%s" : "%s         = \"%s\"", "\nnote", message->m_param.m_presenceStatus.m_note);
+        puts("\n\n");
+        break;
+
+      case OpalIndReceiveIM :
+        printf("Received Instant Message: from=%s to=%s id=%s\n%s\n",
+               message->m_param.m_instantMessage.m_from,
+               message->m_param.m_instantMessage.m_to,
+               message->m_param.m_instantMessage.m_conversationId,
+               message->m_param.m_instantMessage.m_textBody);
+        break;
+
+      case OpalIndSentIM :
+        printf("Sent Instant Message: from=%s to=%s id=%s msg=%u disposition=%s\n",
+               message->m_param.m_instantMessage.m_from,
+               message->m_param.m_instantMessage.m_to,
+               message->m_param.m_instantMessage.m_conversationId,
+               message->m_param.m_instantMessage.m_messageId,
+               message->m_param.m_instantMessage.m_textBody);
+        break;
+
       default :
         break;
     }
@@ -393,6 +437,7 @@ int DoCall(const char * from, const char * to)
   command.m_type = OpalCmdSetUpCall;
   command.m_param.m_callSetUp.m_partyA = from;
   command.m_param.m_callSetUp.m_partyB = to;
+  command.m_param.m_callSetUp.m_overrides.m_displayName = "Test Calling Party";
   if ((response = MySendCommand(&command, "Could not make call")) == NULL)
     return 0;
 
@@ -588,8 +633,161 @@ int DoPlay(const char * to, const char * file)
 }
 
 
+int DoPresence(const char * local, int argc, const char * const * argv)
+{
+  // Example cmd line: presence fred@flintstone.com wilma@flintstone.com
+  OpalMessage command;
+  OpalMessage * response;
+  char * attributes = NULL;
+  size_t attrLen = 0;
+  int arg;
+
+  printf("Registering presentity %s\n", local);
+
+  memset(&command, 0, sizeof(command));
+  command.m_type = OpalCmdRegistration;
+  command.m_param.m_registrationInfo.m_protocol = "pres";
+  command.m_param.m_registrationInfo.m_identifier = local;
+  command.m_param.m_registrationInfo.m_timeToLive = 300;
+
+  for (arg = 0; arg < argc; ++arg) {
+    if (strncasecmp(argv[arg], "pwd=", 4) == 0)
+      command.m_param.m_registrationInfo.m_password = argv[arg] + 4;
+    else if (strncasecmp(argv[arg], "host=", 5) == 0)
+      command.m_param.m_registrationInfo.m_hostName = argv[arg] + 5;
+    else {
+      size_t equal = strspn(argv[arg], "ABCDEFGHIJKLMNOPQRSTOVWXYZabcdefghijklmnopqrstovwxyz0123456789-");
+      if (argv[arg][equal] != '=')
+        break;
+      {
+        size_t argLen = strlen(argv[arg]);
+        attributes = (char *)realloc(attributes, attrLen+argLen+2);
+        strcpy(attributes+attrLen, argv[arg]);
+        strcpy(attributes+attrLen+argLen, "\n");
+        attrLen += argLen + 1;
+      }
+    }
+  }
+  command.m_param.m_registrationInfo.m_attributes = attributes;
+
+  response = MySendCommand(&command, "Could not register presentity");
+
+  free(attributes);
+
+  if (response == NULL)
+    return 0;
+
+  FreeMessageFunction(response);
+
+  for (; arg < argc; ++arg) {
+    printf("Subscribing to presentity %s\n", argv[arg]);
+    memset(&command, 0, sizeof(command));
+    command.m_type = OpalCmdSubscribePresence;
+    command.m_param.m_presenceStatus.m_entity = local;
+    command.m_param.m_presenceStatus.m_target = argv[arg];
+    command.m_param.m_presenceStatus.m_state = OpalPresenceAuthRequest;
+
+    if ((response = MySendCommand(&command, "Could not subscribe to presentity")) == NULL)
+      return 0;
+
+    FreeMessageFunction(response);
+  }
+
+  return 1;
+}
+
+
+int UndoPresence(const char * local, int argc, const char * const * argv)
+{
+  OpalMessage command;
+  OpalMessage * response;
+  int arg;
+
+  for (arg = 0; arg < argc; ++arg) {
+    size_t equal = strspn(argv[arg], "ABCDEFGHIJKLMNOPQRSTOVWXYZabcdefghijklmnopqrstovwxyz0123456789-");
+    if (argv[arg][equal] != '=')
+      break;
+  }
+
+  for (; arg < argc; ++arg) {
+    printf("Subscribing to presentity %s\n", argv[arg]);
+    memset(&command, 0, sizeof(command));
+    command.m_type = OpalCmdSubscribePresence;
+    command.m_param.m_presenceStatus.m_entity = local;
+    command.m_param.m_presenceStatus.m_target = argv[arg];
+    command.m_param.m_presenceStatus.m_state = OpalPresenceNone;
+
+    if ((response = MySendCommand(&command, "Could not unsubscribe to presentity")) == NULL)
+      return 0;
+
+    FreeMessageFunction(response);
+  }
+
+  return 1;
+}
+
+
+int DoPresenceChange(const char * local, OpalPresenceStates state)
+{
+  OpalMessage command;
+  OpalMessage * response;
+
+  printf("Changing status for presentity %s\n", local);
+
+  memset(&command, 0, sizeof(command));
+  command.m_type = OpalCmdSetLocalPresence;
+  command.m_param.m_presenceStatus.m_entity = local;
+  command.m_param.m_presenceStatus.m_state = state;
+  command.m_param.m_presenceStatus.m_capabilities = "audio\nvideo\ntext\ntype=text/plain";
+  command.m_param.m_presenceStatus.m_activities = state == OpalPresenceUnavailable ? "busy\ntravel" : "available";
+  command.m_param.m_presenceStatus.m_note = state == OpalPresenceUnavailable ? "Leave me alone" : "Talk to me";
+
+  if ((response = MySendCommand(&command, "Could not change status of presentity")) == NULL)
+    return 0;
+
+  FreeMessageFunction(response);
+  return 1;
+}
+
+
+int DoSendIM(const char * from, const char * to, int argc, const char * const * argv)
+{
+  OpalMessage command;
+  OpalMessage * response;
+
+  printf("Sending message from %s to %s\n", from, to);
+
+  memset(&command, 0, sizeof(command));
+  command.m_type = OpalCmdSendIM;
+  command.m_param.m_instantMessage.m_from = from;
+  command.m_param.m_instantMessage.m_to = to;
+
+  if (argc > 0) {
+    if (strncasecmp(argv[0], "host=", 5) != 0)
+      command.m_param.m_instantMessage.m_textBody = argv[0];
+    else {
+      command.m_param.m_instantMessage.m_host = &argv[0][5];
+      if (argc == 1)
+        return 0;
+      command.m_param.m_instantMessage.m_textBody = argv[1];
+    }
+  }
+
+  if ((response = MySendCommand(&command, "Could not change status of presentity")) == NULL)
+    return 0;
+
+  printf("Sending message number %u, with conversation id is %s\n",
+         response->m_param.m_instantMessage.m_messageId,
+         response->m_param.m_instantMessage.m_conversationId);
+  FreeMessageFunction(response);
+  return 1;
+}
+
+
 typedef enum
 {
+  OpFirst,
+  OpShutdown = OpFirst,
   OpListen,
   OpCall,
   OpMute,
@@ -600,21 +798,40 @@ typedef enum
   OpSubscribe,
   OpRecord,
   OpPlay,
+  OpPresence,
+  OpInstantMessage,
   NumOperations
 } Operations;
 
 static const char * const OperationNames[NumOperations] =
-  { "listen", "call", "mute", "hold", "transfer", "consult", "register", "subscribe", "record", "play" };
+  { "shutdown", "listen", "call", "mute", "hold", "transfer", "consult", "register", "subscribe", "record", "play", "presence", "im" };
 
 static int const RequiredArgsForOperation[NumOperations] =
-  { 2, 3, 3, 3, 4, 4, 3, 3, 3, 3 };
+  { 2, 2, 3, 3, 3, 4, 4, 3, 3, 3, 3, 4, 5 };
+
+static const char * const OperationHelp[NumOperations] = {
+  "",
+  "",
+  "<destination-URL> [ <local-URL> ]",
+  "<destination-URL>",
+  "<destination-URL>",
+  "<destination-URL> <transfer-URL>",
+  "<destination-URL> <transfer-URL>",
+  "<address-of-record> [ <pwd> ]",
+  "<package> <address-of-record> [ <pwd> ]",
+  "<destination-URL> <filename>",
+  "<destination-URL> <filename>",
+  "<local-URL> [ <attr>=<value> ... ] <remote-URL> ...\n"
+  "    attrib one of pwd/host/transport/sub-protocol etc",
+  "<from> <to> [ host=<host> ] <msg>"
+};
 
 
 static Operations GetOperation(const char * name)
 {
   Operations op;
 
-  for (op = OpListen; op < NumOperations; op++) {
+  for (op = OpFirst; op < NumOperations; op++) {
     if (strcmp(name, OperationNames[op]) == 0)
       break;
   }
@@ -623,20 +840,15 @@ static Operations GetOperation(const char * name)
 }
 
 
-int main(int argc, char * argv[])
+int main(int argc, const char * const * argv)
 {
   Operations operation;
-
   
   if (argc < 2 || (operation = GetOperation(argv[1])) == NumOperations || argc < RequiredArgsForOperation[operation]) {
     Operations op;
-    fputs("usage: c_api { ", stderr);
-    for (op = OpListen; op < NumOperations; op++) {
-      if (op > OpListen)
-        fputs(" | ", stderr);
-      fputs(OperationNames[op], stderr);
-    }
-    fputs(" } [ A-party [ B-party | file ] ]\n", stderr);
+    fputs("usage:\n", stderr);
+    for (op = OpListen; op < NumOperations; op++)
+      fprintf(stderr, "  c_api %s %s\n", OperationNames[op], OperationHelp[op]);
     return 1;
   }
 
@@ -646,9 +858,33 @@ int main(int argc, char * argv[])
     return 1;
 
   switch (operation) {
+    case OpShutdown :
+      HandleMessages(5);
+
+      puts("Shutting down.\n");
+      // Test shut down and re-initialisation
+      ShutDownFunction(hOPAL);
+
+      puts("Reinitialising.\n");
+      if (!InitialiseOPAL()) {
+        fputs("Could not re-initialise OPAL\n", stderr);
+        return 1;
+      }
+
+      if (argc > 3) {
+        if (!DoCall(argv[2], argv[3]))
+          break;
+      }
+      else if (argc > 2) {
+        if (!DoCall(NULL, argv[2]))
+          break;
+      }
+      HandleMessages(15);
+      break;
+
     case OpListen :
       puts("Listening.\n");
-      HandleMessages(60000);
+      HandleMessages(120);
       break;
 
     case OpCall :
@@ -659,76 +895,97 @@ int main(int argc, char * argv[])
         if (!DoCall(NULL, argv[2]))
           break;
       }
-      HandleMessages(15000);
+      HandleMessages(15);
       break;
 
     case OpMute :
       if (!DoCall(NULL, argv[2]))
         break;
-      HandleMessages(15000);
+      HandleMessages(15);
       if (!DoMute(1))
         break;
-      HandleMessages(15000);
+      HandleMessages(15);
       if (!DoMute(0))
         break;
-      HandleMessages(15000);
+      HandleMessages(15);
       break;
 
     case OpHold :
       if (!DoCall(NULL, argv[2]))
         break;
-      HandleMessages(15000);
+      HandleMessages(15);
       if (!DoHold())
         break;
-      HandleMessages(15000);
+      HandleMessages(15);
       break;
 
     case OpTransfer :
       if (!DoCall(NULL, argv[2]))
         break;
-      HandleMessages(15000);
+      HandleMessages(15);
       if (!DoTransfer(argv[3]))
         break;
-      HandleMessages(15000);
+      HandleMessages(15);
       break;
 
     case OpConsult :
       if (!DoCall(NULL, argv[2]))
         break;
-      HandleMessages(15000);
+      HandleMessages(15);
       if (!DoHold())
         break;
-      HandleMessages(15000);
+      HandleMessages(15);
       if (!DoCall(NULL, argv[3]))
         break;
-      HandleMessages(15000);
+      HandleMessages(15);
       if (!DoTransfer(HeldCallToken))
         break;
-      HandleMessages(15000);
+      HandleMessages(15);
       break;
 
     case OpRegister :
       if (!DoRegister(argv[2], argv[3]))
         break;
-      HandleMessages(15000);
+      HandleMessages(15);
       break;
 
     case OpSubscribe :
       if (!DoSubscribe(argv[2], argv[3], argv[4]))
         break;
-      HandleMessages(INT_MAX); // More or less forever
+      HandleMessages(60);
       break;
 
     case OpRecord :
       if (!DoRecord(argv[2], argv[3]))
         break;
-      HandleMessages(INT_MAX); // More or less forever
+      HandleMessages(120);
       break;
 
     case OpPlay :
       if (!DoPlay(argv[2], argv[3]))
         break;
-      HandleMessages(INT_MAX); // More or less forever
+      HandleMessages(120);
+      break;
+
+    case OpPresence :
+      if (!DoPresence(argv[2], argc-3, argv+3))
+        break;
+      HandleMessages(5);
+      if (!DoPresenceChange(argv[2], OpalPresenceUnavailable))
+        break;
+      HandleMessages(5);
+      if (!DoPresenceChange(argv[2], OpalPresenceAvailable))
+        break;
+      HandleMessages(5);
+      if (!UndoPresence(argv[2], argc-3, argv+3))
+        break;
+      HandleMessages(60);
+      break;
+
+    case OpInstantMessage :
+      if (!DoSendIM(argv[2], argv[3], argc-4, argv+4))
+        break;
+      HandleMessages(30);
       break;
 
     default :
