@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 19279 $
+ * $Revision: 28628 $
  * $Author: rjongbloed $
- * $Date: 2008-01-17 04:08:34 +0000 (Thu, 17 Jan 2008) $
+ * $Date: 2012-12-03 00:32:46 -0600 (Mon, 03 Dec 2012) $
  */
 
 #include <ptlib.h>
@@ -32,8 +32,8 @@
 #include <ptclib/pwavfile.h>
 #include <ptclib/delaychan.h>
 
-#include <opal/manager.h>
-#include <opal/ivr.h>
+#include <opal/console_mgr.h>
+#include <ep/pcss.h>
 #include <h323/h323ep.h>
 #include <h323/gkclient.h>
 #include <sip/sipep.h>
