@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26320 $
+ * $Revision: 28945 $
  * $Author: rjongbloed $
- * $Date: 2011-08-17 06:58:09 -0500 (Wed, 17 Aug 2011) $
+ * $Date: 2013-01-18 21:35:47 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include "precompile.h"
@@ -36,6 +36,9 @@
 
 #include <ptclib/random.h>
 
+#include <math.h>
+
+
 #define OUTPUT_BPS(strm, rate) \
   if (rate < 10000ULL) strm << rate << ' '; \
   else if (rate < 10000000ULL) strm << rate/1000.0 << " k"; \
@@ -62,66 +65,64 @@ void CodecTest::Main()
 {
   PArgList & args = GetArguments();
 
-  args.Parse("b-bit-rate:"
-             "c-crop."
-             "C-rate-control:"
-             "d-drop:"
-             "D-display-device:"
-             "-display-driver:"
-             "F-audio-frames:"
-             "G-grab-device:"
-             "-grab-driver:"
-             "-grab-format:"
-             "-grab-channel:"
-             "h-help."
-             "i-info."
-             "m-suppress-marker."
-             "M-force-marker."
-             "O-option:"
-             "p-payload-size:"
-             "P-play-device:"
-             "-play-driver:"
-             "-play-buffers:"
-             "r-frame-rate:"
-             "R-record-device:"
-             "-record-driver:"
-             "s-frame-size:"
-             "-single-step."
-             "S-simultaneous:"
-             "T-statistics."
-             "-count:"
-             "-noprompt."
-             "-snr."
-             "-list."
-#if PTRACING
-             "o-output:"             "-no-output."
-             "t-trace."              "-no-trace."
-#endif
-             , FALSE);
+  if (!args.Parse("[Available options are:]"
+                  "-record-driver: audio recorder driver.\n"
+                  "R-record-device: audio recorder device.\n"
+                  "-play-driver: audio player driver.\n"
+                  "P-play-device: audio player device.\n"
+                  "F-audio-frames: audio frames per packet, default 1.\n"
+                  "-play-buffers: audio player buffers, default 8.\n"
+                  "-grab-driver: video grabber driver.\n"
+                  "G-grab-device: video grabber device.\n"
+                  "-grab-format: video grabber format (\"pal\"/\"ntsc\")\n"
+                  "-grab-channel: video grabber channel.\n"
+                  "-display-driver: video display driver to use.\n"
+                  "D-display-device: video display device to use.\n"
+                  "s-frame-size: video frame size (\"qcif\", \"cif\", WxH)\n"
+                  "r-frame-rate: video frame rate (frames/second)\n"
+                  "b-bit-rate: video bit rate (bits/second)\n"
+                  "O-option: set media format option to value, in form  opt=val\n"
+                  "-ext-hdr: Set RTP extension header (RFC5285 one byte value)\n"
+                  "-single-step. video single frame at a time mode\n"
+                  "c-crop. crop rather than scale if resizing\n"
+                  "m-suppress-marker. suppress marker bits to decoder\n"
+                  "M-force-marker. force marker bits to decoder\n"
+                  "p-payload-size: Set size of maximum RTP payload for encoded data\n"
+                  "S-simultanoues: Number of simultaneous encode/decode threads\n"
+                  "T-statistics. output statistics files\n"
+                  "C-rate-control. enable rate control\n"
+                  "d-drop: randomly drop N% of encoded packets\n"
+                  "-count: set number of frames to transcode\n"
+                  "-noprompt. do not prompt for commands, i.e. exit when input closes\n"
+                  "-snr. calculate signal-to-noise ratio between input and output\n"
+                  "i-info. display per-frame info (use multiple times for more info)\n"
+                  "-list. list all available plugin codecs\n"
+                  PTRACE_ARGLIST
+                  "h-help. print this help message.\n"
+                  , false) || args.HasOption('h')) {
+    cerr << "usage: " << GetFile().GetTitle() << " [ options ] fmtname [ fmtname ]\n"
+              "  where fmtname is the Media Format Name for the codec(s) to test, up to two\n"
+              "  formats (one audio and one video) may be specified.\n";
+    args.Usage(cerr) << "\n"
+               "e.g. " << GetFile().GetTitle() << " --grab-device fake --grab-channel 2 GSM-AMR H.264\n\n";
+    return;
+  }
 
-#if PTRACING
-  PTrace::Initialise(args.GetOptionCount('t'),
-                     args.HasOption('o') ? (const char *)args.GetOptionString('o') : NULL,
-         PTrace::Blocks | PTrace::Timestamp | PTrace::Thread | PTrace::FileAndLine);
-#endif
+  PTRACE_INITIALISE(args);
 
   if (args.HasOption("list")) {
-    OpalPluginCodecManager * codecMgr = PFactory<PPluginModuleManager>::CreateInstanceAs<OpalPluginCodecManager>("OpalPluginCodecManager");
-    if (codecMgr == NULL) {
-      cerr << "error: Cannot get codec manager" << endl;
-      return;
-    }
-    PPluginModuleManager::PluginListType pluginList = codecMgr->GetPluginList();
+    OpalPluginCodecManager & codecMgr = OpalPluginCodecManager::GetInstance();
+    PPluginModuleManager::PluginListType pluginList = codecMgr.GetPluginList();
 
     cout << "Plugin codecs:" << endl;
-    for (int i = 0; i < pluginList.GetSize(); i++) {
-      PDynaLink & plugin =  pluginList.GetDataAt(i);
+    for (PPluginModuleManager::PluginListType::iterator it = pluginList.begin(); it != pluginList.end(); ++it) {
+      PDynaLink & plugin = it->second;
 
       PStringList codecNames;
       PluginCodec_GetCodecFunction getCodecs;
       if (plugin.GetFunction(PLUGIN_CODEC_GET_CODEC_FN_STR, (PDynaLink::Function &)getCodecs)) {
         unsigned count = 0;
-        PluginCodec_Definition * codecs = (*getCodecs)(&count, PLUGIN_CODEC_VERSION);
+        const PluginCodec_Definition * codecs = (*getCodecs)(&count, PLUGIN_CODEC_VERSION);
         while (count > 0) {
           PString name(codecs->descr);
           if (codecNames.GetStringsIndex(name) == P_MAX_INDEX)
@@ -142,52 +143,6 @@ void CodecTest::Main()
     return;
   }
 
-  if (args.HasOption('h') || args.GetCount() == 0) {
-    PError << "usage: " << GetFile().GetTitle() << " [ options ] fmtname [ fmtname ]\n"
-              "  where fmtname is the Media Format Name for the codec(s) to test, up to two\n"
-              "  formats (one audio and one video) may be specified.\n"
-              "\n"
-              "Available options are:\n"
-              "  --help                  : print this help message.\n"
-              "  --record-driver drv     : audio recorder driver.\n"
-              "  -R --record-device dev  : audio recorder device.\n"
-              "  --play-driver drv       : audio player driver.\n"
-              "  -P --play-device dev    : audio player device.\n"
-              "  -F --audio-frames n     : audio frames per packet, default 1.\n"
-              "  --play-buffers n        : audio player buffers, default 8.\n"
-              "  --grab-driver drv       : video grabber driver.\n"
-              "  -G --grab-device dev    : video grabber device.\n"
-              "  --grab-format fmt       : video grabber format (\"pal\"/\"ntsc\")\n"
-              "  --grab-channel num      : video grabber channel.\n"
-              "  --display-driver drv    : video display driver to use.\n"
-              "  -D --display-device dev : video display device to use.\n"
-              "  -s --frame-size size    : video frame size (\"qcif\", \"cif\", WxH)\n"
-              "  -r --frame-rate size    : video frame rate (frames/second)\n"
-              "  -b --bit-rate size      : video bit rate (bits/second)\n"
-              "  -O --option opt=val     : set media format option to value\n"
-              "  --single-step           : video single frame at a time mode\n"
-              "  -c --crop               : crop rather than scale if resizing\n"
-              "  -m --suppress-marker    : suppress marker bits to decoder\n"
-              "  -M --force-marker       : force marker bits to decoder\n"
-              "  -p --payload-size sz    : Set size of maximum RTP payload for encoded data\n"
-              "  -S --simultanoues n     : Number of simultaneous encode/decode threads\n"
-              "  -T --statistics         : output statistics files\n"
-              "  -C --rate-control       : enable rate control\n"
-              "  -d --drop N             : randomly drop N% of encoded packets\n"
-              "  --count n               : set number of frames to transcode\n"
-              "  --noprompt              : do not prompt for commands, i.e. exit when input closes\n"
-              "  --snr                   : calculate signal-to-noise ratio between input and output\n"
-              "  -i --info               : display per-frame info (use multiple times for more info)\n"
-              "  --list                  : list all available plugin codecs\n"
-#if PTRACING
-              "  -o or --output file     : file name for output of log messages\n"       
-              "  -t or --trace           : degree of verbosity in error log (more times for more detail)\n"     
-#endif
-              "\n"
-              "e.g. " << GetFile().GetTitle() << " --grab-device fake --grab-channel 2 GSM-AMR H.264\n\n";
-    return;
-  }
-
   g_infoCount = args.GetOptionCount('i');
 
   unsigned threadCount = args.GetOptionString('S').AsInteger();
@@ -335,6 +290,7 @@ int TranscoderThread::InitialiseCodec(PArgList & args, const OpalMediaFormat & r
     m_framesToTranscode = s.AsInteger();
 
   m_calcSNR = args.HasOption("snr");
+  m_extensionHeader = (BYTE)args.GetOptionString("ext-hdr", "255").AsUnsigned();
 
   for (PINDEX i = 0; i < args.GetCount(); i++) {
     OpalMediaFormat mediaFormat = args[i];
@@ -351,8 +307,17 @@ int TranscoderThread::InitialiseCodec(PArgList & args, const OpalMediaFormat & r
       else {
         OpalMediaFormat adjustedRawFormat = rawFormat;
         if (rawFormat == OpalPCM16) {
-          if (mediaFormat.GetClockRate() != rawFormat.GetClockRate() || mediaFormat.GetPayloadType() == RTP_DataFrame::G722)
+          if (mediaFormat.GetPayloadType() == RTP_DataFrame::G722)
             adjustedRawFormat = OpalPCM16_16KHZ;
+          else {
+            PString str = OPAL_PCM16;
+            if (mediaFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption(), 1) == 2)
+              str += 'S';
+            if (mediaFormat.GetClockRate() != 8000)
+              str += '-' + PString(PString::Unsigned, mediaFormat.GetTimeUnits()) + "KHZ";
+            adjustedRawFormat = str;
+          }
+
           if (args.HasOption('F')) {
             unsigned fpp = args.GetOptionString('F').AsUnsigned();
             if (fpp > 0)
@@ -629,7 +594,27 @@ bool VideoThread::Initialise(PArgList & args)
     cout << "driver \"" << driverName << "\" and ";
   cout << "device \"" << m_display->GetDeviceName() << "\" opened." << endl;
 
-  // Configure sizes/speeds
+  // Configure formats/sizes/speeds
+  cout << "Grabber colour format set to " << m_grabber->GetColourFormat() << " (";
+  if (m_grabber->GetColourFormat() == "YUV420P")
+    cout << "native";
+  else
+    cout << "converted to YUV420P";
+  cout << ')' << endl;
+
+  if (!m_display->SetColourFormatConverter("YUV420P")) {
+    cerr << "Video display device could not be set to colour format YUV420P" << endl;
+    return false;
+  }
+
+  cout << "Display colour format set to " << m_display->GetColourFormat() << " (";
+  if (m_display->GetColourFormat() == "YUV420P")
+    cout << "native";
+  else
+    cout << "converted from YUV420P";
+  cout << ')' << endl;
+
+
   unsigned width, height;
   if (args.HasOption("frame-size")) {
     PString sizeString = args.GetOptionString("frame-size");
@@ -665,26 +650,6 @@ bool VideoThread::Initialise(PArgList & args)
     return false;
   }
 
-  cout << "Grabber colour format set to " << m_grabber->GetColourFormat() << " (";
-  if (m_grabber->GetColourFormat() == "YUV420P")
-    cout << "native";
-  else
-    cout << "converted to YUV420P";
-  cout << ')' << endl;
-
-  if (!m_display->SetColourFormatConverter("YUV420P")) {
-    cerr << "Video display device could not be set to colour format YUV420P" << endl;
-    return false;
-  }
-
-  cout << "Display colour format set to " << m_display->GetColourFormat() << " (";
-  if (m_display->GetColourFormat() == "YUV420P")
-    cout << "native";
-  else
-    cout << "converted from YUV420P";
-  cout << ')' << endl;
-
-
   if (args.HasOption("bit-rate")) {
     PString str = args.GetOptionString("bit-rate");
     double bitrate = str.AsReal();
@@ -725,8 +690,11 @@ bool VideoThread::Initialise(PArgList & args)
 
   m_frameTime = mediaFormat.GetFrameTime();
   if (m_encoder != NULL) {
-    if (args.HasOption('p'))
-      m_encoder->SetMaxOutputSize(args.GetOptionString('p').AsUnsigned());
+    if (args.HasOption('p')) {
+      unsigned bytes = args.GetOptionString('p').AsUnsigned();
+      m_encoder->SetMaxOutputSize(bytes);
+      mediaFormat.SetOptionInteger(OpalMediaFormat::MaxTxPacketSizeOption(), bytes);
+    }
     m_encoder->UpdateMediaFormats(OpalMediaFormat(), mediaFormat);
   }
 
@@ -768,7 +736,7 @@ void VideoThread::Main()
   TranscoderThread::Main();
 }
 
-void TranscoderThread::OnTranscoderCommand(OpalMediaCommand & cmd, INT)
+void TranscoderThread::OnTranscoderCommand(OpalMediaCommand & cmd, P_INT_PTR)
 {
   if (PIsDescendant(&cmd, OpalVideoUpdatePicture)) {
     coutMutex.Wait();
@@ -922,7 +890,7 @@ void TranscoderThread::Main()
   // main loop
   //
 
-  RTP_DataFrame * srcFrame_ = NULL;
+  RTP_DataFrame * srcFramePtr = NULL;
 
   while ((m_running && m_framesToTranscode < 0) || (m_framesToTranscode-- > 0)) {
 
@@ -930,10 +898,9 @@ void TranscoderThread::Main()
     //
     //  acquire and format source frame
     //
-    if (srcFrame_ != NULL)
-      delete srcFrame_;
-    srcFrame_ = new RTP_DataFrame(0);
-    RTP_DataFrame & srcFrame = *srcFrame_;
+    delete srcFramePtr;
+    srcFramePtr = new RTP_DataFrame(0);
+    RTP_DataFrame & srcFrame = *srcFramePtr;
 
     {
       bool state = Read(srcFrame);
@@ -973,18 +940,21 @@ void TranscoderThread::Main()
         if (isVideo) {
           if (m_forceIFrame) {
             coutMutex.Wait();
-            cout << "Decoder forced I-frame at input frame " << totalInputFrameCount-1 << endl;
+            cout << "Decoder forced I-Frame at input frame " << totalInputFrameCount-1 << endl;
             coutMutex.Signal();
           }
           if (rateControlForceIFrame) {
             coutMutex.Wait();
-            cout << "Rate controller forced I-frame at input frame " << totalInputFrameCount-1 << endl;
+            cout << "Rate controller forced I-Frame at input frame " << totalInputFrameCount-1 << endl;
             coutMutex.Signal();
           }
           if (m_forceIFrame || rateControlForceIFrame)
             ((OpalVideoTranscoder *)m_encoder)->ForceIFrame();
         }
 
+        if (m_extensionHeader != 255)
+          srcFrame.SetHeaderExtension(1, 1, &m_extensionHeader, RTP_DataFrame::RFC5285_OneByte);
+
         bool state = m_encoder->ConvertFrames(srcFrame, encFrames);
         if (oldEncState != state) {
           oldEncState = state;
@@ -1030,7 +1000,7 @@ void TranscoderThread::Main()
           if (g_infoCount > 2) {
             cout << "\n   data=";
             cout << hex << setfill('0') << ::setw(2);
-            for (PINDEX i = 0; i < PMIN(10, rtp.GetPayloadSize()); ++i)
+            for (PINDEX i = 0; i < std::min((PINDEX)10, rtp.GetPayloadSize()); ++i)
               cout << (int)rtp.GetPayloadPtr()[i] << ' ';
             cout << dec << setfill(' ') << ::setw(0);
           }
@@ -1042,8 +1012,8 @@ void TranscoderThread::Main()
       totalEncodedPacketCount += encFrames.GetSize();
 
       if (isVideo && m_calcSNR) {
-        ((VideoThread *)this)->SaveSNRFrame(srcFrame_);
-        srcFrame_ = NULL;
+        ((VideoThread *)this)->SaveSNRFrame(srcFramePtr);
+        srcFramePtr = NULL;
       }
 
       //////////////////////////////////////////////
@@ -1051,14 +1021,14 @@ void TranscoderThread::Main()
       //  drop encoded frames if required
       //
       if (m_dropPercent > 0) {
-        PINDEX i = 0;
-        while (i < encFrames.GetSize()) {
+        RTP_DataFrameList::iterator it = encFrames.begin();
+        while (it != encFrames.end()) {
           int n = PRandom::Number() % 100; 
           if (n >= m_dropPercent) 
-            i++;
+            ++it;
           else {
             ++totalDroppedPacketCount;
-            encFrames.RemoveAt(i);
+            encFrames.erase(it++);
           }
         }
       }
@@ -1193,6 +1163,8 @@ void TranscoderThread::Main()
     }
   }
 
+  delete srcFramePtr;
+
   //
   //  end of main loop
   //
