@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22675 $
+ * $Revision: 28483 $
  * $Author: rjongbloed $
- * $Date: 2009-05-19 23:23:29 -0500 (Tue, 19 May 2009) $
+ * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include "main.h"
@@ -172,7 +172,7 @@ PBoolean MyPCSSEndPoint::OnShowIncoming(const OpalPCSSConnection & connection)
 
   //((OpalConnection &)connection).AddIMListener(PCREATE_NOTIFIER(OnReceiveIM));
 
-  return PTrue;
+  return true;
 }
 
 
@@ -184,7 +184,7 @@ PBoolean MyPCSSEndPoint::OnShowOutgoing(const OpalPCSSConnection & connection)
 
   cout << "Outgoing call connected" << endl;
   mgr.m_connected.Signal();
-  return PTrue;
+  return true;
 }
 
 void MyPCSSEndPoint::OnReceiveIM(OpalConnection::IMInfo & im, INT)
