@@ -21,14 +21,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21163 $
+ * $Revision: 28481 $
  * $Author: rjongbloed $
- * $Date: 2008-09-25 01:36:06 -0500 (Thu, 25 Sep 2008) $
+ * $Date: 2012-10-05 03:13:17 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
-#include <opal/manager.h>
-#include <opal/ivr.h>
+#include <opal/console_mgr.h>
+#include <ep/ivr.h>
 #include <sip/sipep.h>
 #include <h323/h323ep.h>
 
