@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24885 $
+ * $Revision: 28813 $
  * $Author: rjongbloed $
- * $Date: 2010-11-21 17:31:12 -0600 (Sun, 21 Nov 2010) $
+ * $Date: 2013-01-08 18:20:18 -0600 (Tue, 08 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -104,7 +104,7 @@ PString DisplayableString(const char * str)
 }
 
 
-PString DisplayLicenseInfo(PluginCodec_information * info)
+PString DisplayLicenseInfo(const PluginCodec_information * info)
 {
   PStringStream str;
   if (info == NULL) 
@@ -134,9 +134,9 @@ PString DisplayLicenseInfo(PluginCodec_information * info)
 }
 
 
-void DisplayCodecDefn(PluginCodec_Definition & defn)
+void DisplayCodecDefn(const PluginCodec_Definition & defn)
 {
-  PBoolean isVideo = PFalse;
+  PBoolean isVideo = false;
 
   cout << "  Version:             " << defn.version << endl
        << DisplayLicenseInfo(defn.info)
@@ -147,7 +147,7 @@ void DisplayCodecDefn(PluginCodec_Definition & defn)
       break;
     case PluginCodec_MediaTypeVideo:
       cout << "Video";
-      isVideo = PTrue;
+      isVideo = true;
       break;
     case PluginCodec_MediaTypeAudioStreamed:
       cout << "Streamed audio (" << ((defn.flags & PluginCodec_BitsPerSampleMask) >> PluginCodec_BitsPerSamplePos) << " bits per sample)";
@@ -264,7 +264,7 @@ void DisplayCodecDefn(PluginCodec_Definition & defn)
         if ((data->data == NULL) || (data->dataLength == 0))
           cout << ", empty data";
         else {
-          PBoolean printable = PTrue;
+          PBoolean printable = true;
           for (unsigned i = 0; i < data->dataLength; ++i) 
             printable = printable && (0x20 <= data->data[i] && data->data[i] < 0x7e);
           if (printable)
@@ -366,25 +366,25 @@ void DisplayMediaFormats(const OpalMediaFormatList & mediaList)
     str += ", sip=";
     str += fmt->IsValidForProtocol("sip") ? "yes" : "no";
     cout << setw(max_len+2) << *fmt << "  " << fmt->GetMediaType()
-         << ", bandwidth=" << fmt->GetBandwidth() << ", RTP=" << fmt->GetPayloadType() << str << endl;
+         << ", bandwidth=" << fmt->GetMaxBandwidth() << ", RTP=" << fmt->GetPayloadType() << str << endl;
   }
 
   cout << "\n\n";
 }
 
 
-void DisplayPlugInInfo(const PString & name, const PPluginModuleManager::PluginListType & pluginList)
+void DisplayPlugInInfo(const PString & name, PPluginModuleManager::PluginListType & pluginList)
 {
-  for (int i = 0; i < pluginList.GetSize(); i++) {
-    if (pluginList.GetKeyAt(i) == name) {
-      PDynaLink & dll = pluginList.GetDataAt(i);
+  for (PPluginModuleManager::PluginListType::iterator it = pluginList.begin(); it != pluginList.end(); ++it) {
+    if (it->first == name) {
+      PDynaLink & dll = it->second;
       PluginCodec_GetCodecFunction getCodecs;
       if (!dll.GetFunction(PLUGIN_CODEC_GET_CODEC_FN_STR, (PDynaLink::Function &)getCodecs)) {
         cout << "error: " << name << " is missing the function " << PLUGIN_CODEC_GET_CODEC_FN_STR << endl;
         return;
       }
       unsigned int count;
-      PluginCodec_Definition * codecs = (*getCodecs)(&count, PLUGIN_CODEC_VERSION);
+      const PluginCodec_Definition * codecs = (*getCodecs)(&count, PLUGIN_CODEC_VERSION);
       if (codecs == NULL || count == 0) {
         cout << "error: " << name << " does not define any codecs for this version of the plugin API" << endl;
         return;
@@ -463,8 +463,8 @@ void AudioTest(const PString & fmtName)
     return;
   }
 
-  PINDEX decBlkSize = encoder->GetOptimalDataFrameSize(PTrue);
-  if (decBlkSize != decoder->GetOptimalDataFrameSize(PFalse)) {
+  PINDEX decBlkSize = encoder->GetOptimalDataFrameSize(true);
+  if (decBlkSize != decoder->GetOptimalDataFrameSize(false)) {
     cout << "error: Encoder and decoder have different frame sizes, mode not supported." << endl;
     return;
   }
@@ -472,7 +472,7 @@ void AudioTest(const PString & fmtName)
   PTones tones("110:0.4-0.1/220:0.4-0.1/440:0.4-0.1/880:0.4-0.1/1760:0.4-0.1");
 
   RTP_DataFrame source(decBlkSize), output(decBlkSize);
-  RTP_DataFrame encoded(PMAX(encoder->GetOptimalDataFrameSize(PFalse), decoder->GetOptimalDataFrameSize(PTrue)));
+  RTP_DataFrame encoded(PMAX(encoder->GetOptimalDataFrameSize(false), decoder->GetOptimalDataFrameSize(true)));
   encoded.SetPayloadType(mediaFormat.GetPayloadType());
 
   int count = 0;
@@ -565,7 +565,7 @@ void VideoTest(const PString & fmtName)
 void OpalCodecInfo::Main()
 {
   cout << GetName()
-       << " Version " << GetVersion(PTrue)
+       << " Version " << GetVersion(true)
        << " by " << GetManufacturer()
        << " on " << GetOSClass() << ' ' << GetOSName()
        << " (" << GetOSVersion() << '-' << GetOSHardware() << ")\n\n";
@@ -587,7 +587,7 @@ void OpalCodecInfo::Main()
              "o-output:"
 #endif
              "V-video-test:"
-             , PFalse);
+             , false);
 
 #if PTRACING
   PTrace::Initialise(args.GetOptionCount('t'),
@@ -611,13 +611,13 @@ void OpalCodecInfo::Main()
     needHelp = false;
   }
 
-  OpalPluginCodecManager & codecMgr = *PFactory<PPluginModuleManager>::CreateInstanceAs<OpalPluginCodecManager>("OpalPluginCodecManager");
+  OpalPluginCodecManager & codecMgr = OpalPluginCodecManager::GetInstance();
   PPluginModuleManager::PluginListType pluginList = codecMgr.GetPluginList();
 
   if (args.HasOption('p')) {
     cout << "Plugin codecs:" << endl;
-    for (int i = 0; i < pluginList.GetSize(); i++)
-      cout << "   " << pluginList.GetKeyAt(i) << endl;
+    for (PPluginModuleManager::PluginListType::iterator it = pluginList.begin(); it != pluginList.end(); ++it)
+      cout << "   " << it->first << endl;
     cout << "\n\n";
     needHelp = false;
   }
