@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25632 $
+ * $Revision: 29850 $
  * $Author: rjongbloed $
- * $Date: 2011-04-28 02:27:14 -0500 (Thu, 28 Apr 2011) $
+ * $Date: 2013-05-31 02:34:06 -0500 (Fri, 31 May 2013) $
  */
 
 #ifndef _OpenPhone_MAIN_H
@@ -38,6 +38,7 @@
 
 
 #include <wx/wx.h>
+#include <wx/taskbar.h>
 #include <wx/dataobj.h>
 
 #include <opal/manager.h>
@@ -46,13 +47,15 @@
 #error Cannot compile without PTLib sound channel support!
 #endif
 
-#include <opal/pcss.h>
+#include <ep/pcss.h>
 
 #include <h323/h323.h>
 #include <h323/gkclient.h>
 #include <sip/sip.h>
 #include <t38/t38proto.h>
+#include <im/im_ep.h>
 #include <ptlib/wxstring.h>
+#include <ptlib/notifier_ext.h>
 
 
 #include <list>
@@ -71,6 +74,7 @@ class wxSplitterEvent;
 class wxSpinCtrl;
 class wxListCtrl;
 class wxListEvent;
+class wxCheckListBox;
 class wxNotebook;
 class wxGrid;
 class wxGridEvent;
@@ -85,22 +89,13 @@ class MyPCSSEndPoint : public OpalPCSSEndPoint
   public:
     MyPCSSEndPoint(MyManager & manager);
 
-    virtual OpalMediaFormatList GetMediaFormats() const;
     virtual PSoundChannel * CreateSoundChannel(const OpalPCSSConnection & connection, const OpalMediaFormat & mediaFormat, PBoolean isSource);
 
-    MyManager & m_manager;
-
-/*
-    bool TransmitExternalIM(
-      OpalConnection & conn, 
-      const OpalMediaFormat & format, 
-      RTP_IMFrame & frame
-    );
-*/
-
   private:
     virtual PBoolean OnShowIncoming(const OpalPCSSConnection & connection);
     virtual PBoolean OnShowOutgoing(const OpalPCSSConnection & connection);
+
+    MyManager & m_manager;
 };
 
 class IMDialog;
@@ -117,7 +112,7 @@ class MyH323EndPoint : public H323EndPoint
     MyH323EndPoint(MyManager & manager);
 
   private:
-    virtual void OnRegistrationConfirm();
+    virtual void OnGatekeeperStatus(H323Gatekeeper::RegistrationFailReasons);
 
     MyManager & m_manager;
 };
@@ -192,13 +187,17 @@ class AudioDevicesDialog : public wxDialog
 class VideoControlDialog : public wxDialog
 {
   public:
-    VideoControlDialog(MyManager * manager);
+    VideoControlDialog(MyManager * manager, bool remote);
 
   private:
     bool TransferDataFromWindow();
+    OpalMediaStreamPtr GetStream() const;
 
     MyManager & m_manager;
+    bool        m_remote;
     wxSlider  * m_TargetBitRate;
+    wxSlider  * m_FrameRate;
+    wxSlider  * m_TSTO;
 
     DECLARE_EVENT_TABLE()
 };
@@ -228,8 +227,8 @@ class CallDialog : public wxDialog
     int       m_FaxMode;
 
   private:
-    void OnOK(wxCommandEvent & event);
-    void OnAddressChange(wxCommandEvent & event);
+    void OnOK(wxCommandEvent & /*event*/);
+    void OnAddressChange(wxCommandEvent & /*event*/);
 
     wxComboBox * m_AddressCtrl;
     wxButton   * m_ok;
@@ -237,6 +236,9 @@ class CallDialog : public wxDialog
     DECLARE_EVENT_TABLE()
 };
 
+
+#if OPAL_HAS_IM
+
 class CallIMDialog : public wxDialog
 {
   public:
@@ -246,85 +248,85 @@ class CallIMDialog : public wxDialog
     PwxString m_Address;
 
   private:
-    void OnOK(int code);
-    void OnChanged(wxCommandEvent & event);
+    void OnOK(wxCommandEvent & /*event*/);
+    void OnChanged(wxCommandEvent & /*event*/);
 
+    MyManager  & m_manager;
     wxButton   * m_ok;
     wxComboBox * m_AddressCtrl;
 
     DECLARE_EVENT_TABLE()
 };
 
-class IMDialog : public wxDialog
+
+class IMDialog : public wxDialog, PAsyncNotifierTarget
 {
   public:
-    IMDialog(MyManager * manager, 
-         const PString & conversationId,
- const OpalMediaFormat & format,
-         const PString & localName, 
-            const PURL & remoteAddress, 
-         const PString & remoteName);
+    IMDialog(MyManager * manager, OpalIMContext & context);
     ~IMDialog();
 
-    void AddTextToScreen(const wxString & text, bool fromUs);
-    void SetCompositionIndication(bool idle);
-
-    PString m_conversationId;
+    void AddTextToScreen(const PwxString & text, bool fromUs);
 
   private:
-    void OnSend(wxCommandEvent & event);
-    void OnTextEnter(wxCommandEvent & event);
-    void OnText(wxCommandEvent & event);
-    void OnCloseWindow(wxCloseEvent &event);
+    // PAsyncNotifierTarget override
+    virtual void AsyncNotifierSignal();
+    void OnEvtAsyncNotification(wxCommandEvent & /*event*/);
+
+    PDECLARE_ASYNC_MessageDispositionNotifier(IMDialog, OnMessageDisposition);
+    PDECLARE_ASYNC_MessageReceivedNotifier(IMDialog, OnMessageReceived);
+    PDECLARE_ASYNC_CompositionIndicationNotifier(IMDialog, OnCompositionIndication);
+
+    void OnSend(wxCommandEvent & /*event*/);
+    void OnTextEnter(wxCommandEvent & /*event*/);
+    void OnText(wxCommandEvent & /*event*/);
+    void OnCloseWindow(wxCloseEvent & /*event*/);
 
     void SendCurrentText();
-    void SendCompositionIndicationChanged();
+    void SendCompositionIndication();
+    void ShowDisposition(OpalIMContext::MessageDisposition status);
 
-    OpalMediaFormat m_mediaFormat;
-    MyManager * m_manager;
-    PURL m_remoteAddress;
+    MyManager     & m_manager;
+    OpalIMContext & m_context;
 
-    wxString m_them;
-    PString m_us;
-    PString m_remoteAddrStr;
-
-    wxTextCtrl * m_enteredText;
-    wxTextCtrl * m_textArea;
-    wxTextAttr   m_defaultStyle;
-    wxTextAttr   m_ourStyle;
-    wxTextAttr   m_theirStyle;
+    wxTextCtrl   * m_enteredText;
+    wxTextCtrl   * m_textArea;
+    wxTextAttr     m_defaultStyle;
+    wxTextAttr     m_ourStyle;
+    wxTextAttr     m_theirStyle;
     wxStaticText * m_compositionIndication;
 
     DECLARE_EVENT_TABLE()
 };
 
+#endif // OPAL_HAS_IM
+
 
 class CallPanelBase : public wxPanel
 {
   protected:
     CallPanelBase(
       MyManager & manager,
-      const PwxString & token,
+      const PSafePtr<OpalCall> & call,
       wxWindow * parent,
       const wxChar * resource
     );
 
   public:
-    const PwxString & GetToken() const { return m_token; }
+    PwxString GetToken() const { return m_call->GetToken(); }
 
   protected:
     MyManager & m_manager;
-    PwxString   m_token;
+    PSafePtr<OpalCall> m_call;
 };
 
 
 class CallingPanel : public CallPanelBase
 {
   public:
-    CallingPanel(MyManager & manager, const PwxString & token, wxWindow * parent);
+    CallingPanel(MyManager & manager, const PSafePtr<OpalCall> & call, wxWindow * parent);
 
   private:
-    void OnHangUp(wxCommandEvent & event);
+    void OnHangUp(wxCommandEvent & /*event*/);
 
     DECLARE_EVENT_TABLE()
 };
@@ -333,14 +335,16 @@ class CallingPanel : public CallPanelBase
 class AnswerPanel : public CallPanelBase
 {
   public:
-    AnswerPanel(MyManager & manager, const PwxString & token, wxWindow * parent);
+    AnswerPanel(MyManager & manager, const PSafePtr<OpalCall> & call, wxWindow * parent);
 
     void SetPartyNames(const PwxString & calling, const PwxString & called);
 
   private:
-    void OnAnswer(wxCommandEvent & event);
-    void OnReject(wxCommandEvent & event);
-    void OnChangeAnswerMode(wxCommandEvent & event);
+    void OnAnswer(wxCommandEvent & /*event*/);
+    void OnReject(wxCommandEvent & /*event*/);
+#if OPAL_FAX
+    void OnChangeAnswerMode(wxCommandEvent & /*event*/);
+#endif
 
     DECLARE_EVENT_TABLE()
 };
@@ -415,50 +419,51 @@ class StatisticsPage
 class InCallPanel : public CallPanelBase
 {
   public:
-    InCallPanel(MyManager & manager, const PwxString & token, wxWindow * parent);
+    InCallPanel(MyManager & manager, const PSafePtr<OpalCall> & call, wxWindow * parent);
     virtual bool Show(bool show = true);
 
     void OnStreamsChanged();
     void OnHoldChanged(bool onHold);
 
   private:
-    void OnHangUp(wxCommandEvent & event);
-    void OnHoldRetrieve(wxCommandEvent & event);
-    void OnConference(wxCommandEvent & event);
-    void OnSpeakerMute(wxCommandEvent & event);
-    void OnMicrophoneMute(wxCommandEvent & event);
-    void OnUserInput1(wxCommandEvent & event);
-    void OnUserInput2(wxCommandEvent & event);
-    void OnUserInput3(wxCommandEvent & event);
-    void OnUserInput4(wxCommandEvent & event);
-    void OnUserInput5(wxCommandEvent & event);
-    void OnUserInput6(wxCommandEvent & event);
-    void OnUserInput7(wxCommandEvent & event);
-    void OnUserInput8(wxCommandEvent & event);
-    void OnUserInput9(wxCommandEvent & event);
-    void OnUserInput0(wxCommandEvent & event);
-    void OnUserInputStar(wxCommandEvent & event);
-    void OnUserInputHash(wxCommandEvent & event);
-    void OnUserInputFlash(wxCommandEvent & event);
-    void OnUpdateVU(wxTimerEvent& event);
+    void OnHangUp(wxCommandEvent & /*event*/);
+    void OnHoldRetrieve(wxCommandEvent & /*event*/);
+    void OnConference(wxCommandEvent & /*event*/);
+    void OnSpeakerMute(wxCommandEvent & /*event*/);
+    void OnMicrophoneMute(wxCommandEvent & /*event*/);
+    void OnUserInput1(wxCommandEvent & /*event*/);
+    void OnUserInput2(wxCommandEvent & /*event*/);
+    void OnUserInput3(wxCommandEvent & /*event*/);
+    void OnUserInput4(wxCommandEvent & /*event*/);
+    void OnUserInput5(wxCommandEvent & /*event*/);
+    void OnUserInput6(wxCommandEvent & /*event*/);
+    void OnUserInput7(wxCommandEvent & /*event*/);
+    void OnUserInput8(wxCommandEvent & /*event*/);
+    void OnUserInput9(wxCommandEvent & /*event*/);
+    void OnUserInput0(wxCommandEvent & /*event*/);
+    void OnUserInputStar(wxCommandEvent & /*event*/);
+    void OnUserInputHash(wxCommandEvent & /*event*/);
+    void OnUserInputFlash(wxCommandEvent & /*event*/);
+    void OnUpdateVU(wxTimerEvent & /*event*/);
 
-    void SpeakerVolume(wxScrollEvent & event);
-    void MicrophoneVolume(wxScrollEvent & event);
+    void SpeakerVolume(wxScrollEvent & /*event*/);
+    void MicrophoneVolume(wxScrollEvent & /*event*/);
     void SetVolume(bool microphone, int value, bool muted);
     void UpdateStatistics();
 
-    wxButton  * m_Hold;
-    wxButton  * m_Conference;
-    wxButton  * m_SpeakerHandset;
-    wxCheckBox* m_SpeakerMute;
-    wxCheckBox* m_MicrophoneMute;
-    wxSlider  * m_SpeakerVolume;
-    wxSlider  * m_MicrophoneVolume;
-    wxGauge   * m_vuSpeaker;
-    wxGauge   * m_vuMicrophone;
-    wxTimer     m_vuTimer;
-    unsigned    m_updateStatistics;
-    bool        m_FirstTime;
+    wxButton     * m_Hold;
+    wxButton     * m_Conference;
+    wxButton     * m_SpeakerHandset;
+    wxCheckBox   * m_SpeakerMute;
+    wxCheckBox   * m_MicrophoneMute;
+    wxSlider     * m_SpeakerVolume;
+    wxSlider     * m_MicrophoneVolume;
+    wxGauge      * m_vuSpeaker;
+    wxGauge      * m_vuMicrophone;
+    wxTimer        m_vuTimer;
+    wxStaticText * m_callTime;
+    unsigned       m_updateStatistics;
+    bool           m_FirstTime;
 
     StatisticsPage m_pages[NumPages];
 
@@ -484,7 +489,7 @@ class SpeedDialDialog : public wxDialog, public SpeedDialInfo
     SpeedDialDialog(MyManager *parent, const SpeedDialInfo & info);
 
   private:
-    void OnChange(wxCommandEvent & event);
+    void OnChange(wxCommandEvent & /*event*/);
 
     MyManager & m_manager;
 
@@ -508,7 +513,11 @@ class RegistrationInfo
       Register,
       SubscribeMWI,
       SubcribePresence,
-      SubscribeMLA
+      SubscribeMLA,
+      PublishPresence,
+      PresenceWatcher,
+      OthersRegistration,
+      NumRegType
     };
 
     RegistrationInfo();
@@ -525,6 +534,7 @@ class RegistrationInfo
     PwxString m_User;
     PwxString m_Domain;
     PwxString m_Contact;
+    PwxString m_InstanceId;
     PwxString m_AuthID;
     PwxString m_Password;
     int       m_TimeToLive;
@@ -547,7 +557,7 @@ class RegistrationDialog : public wxDialog
   private:
     DECLARE_EVENT_TABLE()
 
-    void Changed(wxCommandEvent & event);
+    void Changed(wxCommandEvent & /*event*/);
 
     wxButton       * m_ok;
     wxTextCtrl     * m_user;
@@ -576,21 +586,29 @@ class OptionsDialog : public wxDialog
     PwxString m_VendorName;
     PwxString m_ProductName;
     PwxString m_ProductVersion;
+    bool      m_HideMinimised;
 
-    void BrowseSoundFile(wxCommandEvent & event);
-    void PlaySoundFile(wxCommandEvent & event);
+    void BrowseSoundFile(wxCommandEvent & /*event*/);
+    void PlaySoundFile(wxCommandEvent & /*event*/);
 
     ////////////////////////////////////////
     // Networking fields
-    wxString        m_Bandwidth;
+    wxString        m_RxBandwidth;
+    wxString        m_TxBandwidth;
+    int             m_DiffServAudio;
+    int             m_DiffServVideo;
+    int             m_MaxRtpPayloadSize;
+#if OPAL_PTLIB_SSL
+    PwxString       m_CertificateAuthority;
+    PwxString       m_LocalCertificate;
+    PwxString       m_PrivateKey;
+#endif
     int             m_TCPPortBase;
     int             m_TCPPortMax;
     int             m_UDPPortBase;
     int             m_UDPPortMax;
     int             m_RTPPortBase;
     int             m_RTPPortMax;
-    int             m_RTPTOS;
-    int             m_MaxRtpPayloadSize;
     wxRadioButton * m_NoNATUsedRadio;
     wxRadioButton * m_NATRouterRadio;
     wxRadioButton * m_STUNServerRadio;
@@ -599,23 +617,32 @@ class OptionsDialog : public wxDialog
     PwxString       m_STUNServer;
     wxTextCtrl    * m_STUNServerWnd;
     wxListBox     * m_LocalInterfaces;
-    wxRadioBox    * m_InterfaceProtocol;
+    wxChoice      * m_InterfaceProtocol;
     wxComboBox    * m_InterfaceAddress;
     wxTextCtrl    * m_InterfacePort;
     wxButton      * m_AddInterface;
     wxButton      * m_RemoveInterface;
-    void BandwidthClass(wxCommandEvent & event);
-    void NATHandling(wxCommandEvent & event);
-    void SelectedLocalInterface(wxCommandEvent & event);
-    void ChangedInterfaceInfo(wxCommandEvent & event);
-    void AddInterface(wxCommandEvent & event);
-    void RemoveInterface(wxCommandEvent & event);
+    void BandwidthClass(wxCommandEvent & /*event*/);
+
+#if OPAL_PTLIB_SSL
+    void FindCertificateAuthority(wxCommandEvent & /*event*/);
+    void FindLocalCertificate(wxCommandEvent & /*event*/);
+    void FindPrivateKey(wxCommandEvent & /*event*/);
+#endif
+    void NATHandling(wxCommandEvent & /*event*/);
+    void SelectedLocalInterface(wxCommandEvent & /*event*/);
+    void ChangedInterfaceInfo(wxCommandEvent & /*event*/);
+    void AddInterface(wxCommandEvent & /*event*/);
+    void RemoveInterface(wxCommandEvent & /*event*/);
 
     ////////////////////////////////////////
     // Sound fields
     PwxString m_SoundPlayer;
     PwxString m_SoundRecorder;
     int       m_SoundBufferTime;
+#if OPAL_AEC
+    bool      m_EchoCancellation;
+#endif
     PwxString m_LineInterfaceDevice;
     int       m_AEC;
     PwxString m_Country;
@@ -627,14 +654,19 @@ class OptionsDialog : public wxDialog
     int       m_SilenceDeadband;
     bool      m_DisableDetectInBandDTMF;
 
+    wxComboBox * m_soundPlayerCombo;
+    wxComboBox * m_soundRecorderCombo;
     wxComboBox * m_selectedLID;
     wxChoice   * m_selectedAEC;
     wxComboBox * m_selectedCountry;
-    void SelectedLID(wxCommandEvent & event);
+
+    void ChangedSoundPlayer(wxCommandEvent & /*event*/);
+    void ChangedSoundRecorder(wxCommandEvent & /*event*/);
+    void SelectedLID(wxCommandEvent & /*event*/);
 
     ////////////////////////////////////////
     // Video fields
-    PwxString m_VideoGrabber;
+    PwxString m_VideoGrabDevice;
     int       m_VideoGrabFormat;
     int       m_VideoGrabSource;
     int       m_VideoGrabFrameRate;
@@ -649,19 +681,26 @@ class OptionsDialog : public wxDialog
     int       m_VideoGrabBitRate;
     int       m_VideoMaxBitRate;
 
-    wxComboBox * m_videoGrabDevice;
+    wxComboBox * m_videoGrabDeviceCombo;
     wxChoice   * m_videoSourceChoice;
 
+    wxButton           * m_TestVideoCapture;
+    PThread            * m_TestVideoThread;
+    PVideoInputDevice  * m_TestVideoGrabber;
+    PVideoOutputDevice * m_TestVideoDisplay;
+
     void AdjustVideoControls(const PwxString & device);
-    void ChangeVideoGrabber(wxCommandEvent & event);
-    void TestVideoCapture(wxCommandEvent & event);
+    void ChangeVideoGrabDevice(wxCommandEvent & /*event*/);
+    void TestVideoCapture(wxCommandEvent & /*event*/);
+    void TestVideoThreadMain();
+    void StopTestVideo();
 
     ////////////////////////////////////////
     // Fax fields
     PwxString m_FaxStationIdentifier;
     PwxString m_FaxReceiveDirectory;
     int       m_FaxAutoAnswerMode;
-    void BrowseFaxDirectory(wxCommandEvent & event);
+    void BrowseFaxDirectory(wxCommandEvent & /*event*/);
 
     ////////////////////////////////////////
     // IVR fields
@@ -679,13 +718,13 @@ class OptionsDialog : public wxDialog
     wxGrid     * m_PresentityAttributes;
     wxButton   * m_AddPresentity;
     wxButton   * m_RemovePresentity;
-    void AddPresentity(wxCommandEvent & event);
-    void RemovePresentity(wxCommandEvent & event);
-    void SelectedPresentity(wxListEvent & event);
-    void DeselectedPresentity(wxListEvent & event);
-    void EditedPresentity(wxListEvent & event);
+    void AddPresentity(wxCommandEvent & /*event*/);
+    void RemovePresentity(wxCommandEvent & /*event*/);
+    void SelectedPresentity(wxListEvent & /*event*/);
+    void DeselectedPresentity(wxListEvent & /*event*/);
+    void EditedPresentity(wxListEvent & /*event*/);
     bool FillPresentityAttributes(OpalPresentity * presentity);
-    void ChangedPresentityAttribute(wxGridEvent & event);
+    void ChangedPresentityAttribute(wxGridEvent & /*event*/);
 
     ////////////////////////////////////////
     // Codec fields
@@ -700,22 +739,15 @@ class OptionsDialog : public wxDialog
     wxStaticText * m_CodecOptionValueLabel;
     wxStaticText * m_CodecOptionValueError;
 
-    void AddCodec(wxCommandEvent & event);
-    void RemoveCodec(wxCommandEvent & event);
-    void MoveUpCodec(wxCommandEvent & event);
-    void MoveDownCodec(wxCommandEvent & event);
-    void SelectedCodecToAdd(wxCommandEvent & event);
-    void SelectedCodec(wxCommandEvent & event);
-    void SelectedCodecOption(wxListEvent & event);
-    void DeselectedCodecOption(wxListEvent & event);
-    void ChangedCodecOptionValue(wxCommandEvent & event);
-
-    ////////////////////////////////////////
-    // Security fields
-    bool         m_SecureH323;
-    bool         m_SecureSIP;
-    PwxString    m_RTPSecurityModeH323;
-    PwxString    m_RTPSecurityModeSIP;
+    void AddCodec(wxCommandEvent & /*event*/);
+    void RemoveCodec(wxCommandEvent & /*event*/);
+    void MoveUpCodec(wxCommandEvent & /*event*/);
+    void MoveDownCodec(wxCommandEvent & /*event*/);
+    void SelectedCodecToAdd(wxCommandEvent & /*event*/);
+    void SelectedCodec(wxCommandEvent & /*event*/);
+    void SelectedCodecOption(wxListEvent & /*event*/);
+    void DeselectedCodecOption(wxListEvent & /*event*/);
+    void ChangedCodecOptionValue(wxCommandEvent & /*event*/);
 
     ////////////////////////////////////////
     // H.323 fields
@@ -725,22 +757,36 @@ class OptionsDialog : public wxDialog
     int          m_GatekeeperTTL;
     PwxString    m_GatekeeperLogin;
     PwxString    m_GatekeeperPassword;
+
     int          m_DTMFSendMode;
     int          m_CallIntrusionProtectionLevel;
     bool         m_DisableFastStart;
     bool         m_DisableH245Tunneling;
     bool         m_DisableH245inSETUP;
+    bool         m_ForceSymmetricTCS;
     int          m_ExtendedVideoRoles;
     bool         m_EnableH239Control;
+
     wxListBox  * m_Aliases;
     wxTextCtrl * m_NewAlias;
     wxButton   * m_AddAlias;
     wxButton   * m_RemoveAlias;
 
-    void SelectedAlias(wxCommandEvent & event);
-    void ChangedNewAlias(wxCommandEvent & event);
-    void AddAlias(wxCommandEvent & event);
-    void RemoveAlias(wxCommandEvent & event);
+    void SelectedAlias(wxCommandEvent & /*event*/);
+    void ChangedNewAlias(wxCommandEvent & /*event*/);
+    void AddAlias(wxCommandEvent & /*event*/);
+    void RemoveAlias(wxCommandEvent & /*event*/);
+
+#if OPAL_PTLIB_SSL
+    int              m_H323SignalingSecurity;
+    wxCheckListBox * m_H323MediaCryptoSuites;
+    wxButton       * m_H323MediaCryptoSuiteUp;
+    wxButton       * m_H323MediaCryptoSuiteDown;
+    void H323SignalingSecurityChanged(wxCommandEvent & /*event*/);
+    void H323MediaCryptoSuiteChanged(wxCommandEvent & /*event*/);
+    void H323MediaCryptoSuiteUp(wxCommandEvent & /*event*/);
+    void H323MediaCryptoSuiteDown(wxCommandEvent & /*event*/);
+#endif
 
     ////////////////////////////////////////
     // SIP fields
@@ -748,8 +794,10 @@ class OptionsDialog : public wxDialog
     PwxString m_SIPProxy;
     PwxString m_SIPProxyUsername;
     PwxString m_SIPProxyPassword;
+
     int       m_LineAppearanceCode;
     int       m_SIPUserInputMode;
+    int       m_SIPPRACKMode;
 
     wxListCtrl * m_Registrations;
     int          m_SelectedRegistration;
@@ -759,15 +807,26 @@ class OptionsDialog : public wxDialog
     wxButton   * m_MoveDownRegistration;
 
     void RegistrationToList(bool create, RegistrationInfo * registration, int position);
-    void AddRegistration(wxCommandEvent & event);
-    void ChangeRegistration(wxCommandEvent & event);
-    void RemoveRegistration(wxCommandEvent & event);
-    void MoveUpRegistration(wxCommandEvent & event);
-    void MoveDownRegistration(wxCommandEvent & event);
+    void AddRegistration(wxCommandEvent & /*event*/);
+    void ChangeRegistration(wxCommandEvent & /*event*/);
+    void RemoveRegistration(wxCommandEvent & /*event*/);
+    void MoveUpRegistration(wxCommandEvent & /*event*/);
+    void MoveDownRegistration(wxCommandEvent & /*event*/);
     void MoveRegistration(int delta);
-    void SelectedRegistration(wxListEvent & event);
-    void DeselectedRegistration(wxListEvent & event);
-    void ActivateRegistration(wxListEvent & event);
+    void SelectedRegistration(wxListEvent & /*event*/);
+    void DeselectedRegistration(wxListEvent & /*event*/);
+    void ActivateRegistration(wxListEvent & /*event*/);
+
+#if OPAL_PTLIB_SSL
+    int              m_SIPSignalingSecurity;
+    wxCheckListBox * m_SIPMediaCryptoSuites;
+    wxButton       * m_SIPMediaCryptoSuiteUp;
+    wxButton       * m_SIPMediaCryptoSuiteDown;
+    void SIPSignalingSecurityChanged(wxCommandEvent & /*event*/);
+    void SIPMediaCryptoSuiteChanged(wxCommandEvent & /*event*/);
+    void SIPMediaCryptoSuiteUp(wxCommandEvent & /*event*/);
+    void SIPMediaCryptoSuiteDown(wxCommandEvent & /*event*/);
+#endif
 
     ////////////////////////////////////////
     // Routing fields
@@ -785,14 +844,14 @@ class OptionsDialog : public wxDialog
     wxButton   * m_MoveUpRoute;
     wxButton   * m_MoveDownRoute;
 
-    void AddRoute(wxCommandEvent & event);
-    void RemoveRoute(wxCommandEvent & event);
-    void MoveUpRoute(wxCommandEvent & event);
-    void MoveDownRoute(wxCommandEvent & event);
-    void SelectedRoute(wxListEvent & event);
-    void DeselectedRoute(wxListEvent & event);
-    void ChangedRouteInfo(wxCommandEvent & event);
-    void RestoreDefaultRoutes(wxCommandEvent & event);
+    void AddRoute(wxCommandEvent & /*event*/);
+    void RemoveRoute(wxCommandEvent & /*event*/);
+    void MoveUpRoute(wxCommandEvent & /*event*/);
+    void MoveDownRoute(wxCommandEvent & /*event*/);
+    void SelectedRoute(wxListEvent & /*event*/);
+    void DeselectedRoute(wxListEvent & /*event*/);
+    void ChangedRouteInfo(wxCommandEvent & /*event*/);
+    void RestoreDefaultRoutes(wxCommandEvent & /*event*/);
     void AddRouteTableEntry(OpalManager::RouteEntry entry);
 
 #if PTRACING
@@ -807,9 +866,11 @@ class OptionsDialog : public wxDialog
     bool      m_TraceTimestamp;
     bool      m_TraceThreadName;
     bool      m_TraceThreadAddress;
+    bool      m_TraceObjectInstance;
+    bool      m_TraceContextId;
     PwxString m_TraceFileName;
 
-    void BrowseTraceFile(wxCommandEvent & event);
+    void BrowseTraceFile(wxCommandEvent & /*event*/);
 #endif
 
     DECLARE_EVENT_TABLE()
@@ -827,18 +888,39 @@ public:
   OpalMediaFormat mediaFormat;
   const wxChar  * validProtocols;
   int             preferenceOrder;
-  bool            dirty;
 };
 
 typedef std::list<MyMedia> MyMediaList;
 
-class MyManager : public wxFrame, public OpalManager
+
+class MyTaskBarIcon : public wxTaskBarIcon
+{
+  public:
+    MyTaskBarIcon(MyManager & manager);
+
+    virtual wxMenu * CreatePopupMenu();
+    virtual void OnDoubleClick(wxTaskBarIconEvent & /*event*/);
+
+  protected:
+    virtual bool ProcessEvent(wxEvent & /*event*/);
+
+    MyManager & m_manager;
+
+    DECLARE_EVENT_TABLE()
+};
+
+
+class MyManager : public wxFrame, public OpalManager, public PAsyncNotifierTarget
 {
   public:
     MyManager();
     ~MyManager();
 
-    bool Initialise();
+    bool Initialise(bool startMinimised);
+
+    void SetTrayTipText(const PwxString & text);
+    void SetBalloonText(const PwxString & text);
+    wxMenu * CreateTrayMenu();
 
     bool HasSpeedDialName(const wxString & name) const;
     bool HasSpeedDialNumber(const wxString & number, const wxString & ignore) const;
@@ -854,6 +936,7 @@ class MyManager : public wxFrame, public OpalManager
 
     bool HasHandset() const;
 
+#if OPAL_FAX
     enum FaxAnswerModes {
       AnswerVoice,
       AnswerFax,
@@ -862,6 +945,7 @@ class MyManager : public wxFrame, public OpalManager
     FaxAnswerModes m_currentAnswerMode;
     FaxAnswerModes m_defaultAnswerMode;
     void SwitchToFax();
+#endif // OPAL_FAX
 
     MyPCSSEndPoint & GetPCSSEP() { return *pcssEP; }
 
@@ -873,6 +957,10 @@ class MyManager : public wxFrame, public OpalManager
     );
 
   private:
+    // PAsyncNotifierTarget override
+    virtual void AsyncNotifierSignal();
+    void OnEvtAsyncNotification(wxCommandEvent & /*event*/);
+
     // OpalManager overrides
     virtual PBoolean OnIncomingConnection(
       OpalConnection & connection,   ///<  Connection that is calling
@@ -894,6 +982,11 @@ class MyManager : public wxFrame, public OpalManager
       OpalConnection & connection,  ///< Connection being transferred.
       const PStringToString & info  ///< Information on the transfer
     );
+    virtual void AdjustMediaFormats(
+      bool local,                         ///<  Media formats a local ones to be presented to remote
+      const OpalConnection & connection,  ///<  Connection that is about to use formats
+      OpalMediaFormatList & mediaFormats  ///<  Media formats to use
+    ) const;
     virtual PBoolean OnOpenMediaStream(
       OpalConnection & connection,  /// Connection that owns the media stream
       OpalMediaStream & stream    /// New media stream being opened
@@ -901,15 +994,22 @@ class MyManager : public wxFrame, public OpalManager
     virtual void OnClosedMediaStream(
       const OpalMediaStream & stream     ///<  Stream being closed
     );
+    virtual bool OnChangedPresentationRole(
+      OpalConnection & connection,   ///< COnnection that has had the change
+      const PString & newChairURI,   ///< URI for new confernce chair
+      bool request                   ///< Indicates change is requested
+    );
     virtual void OnUserInputString(
       OpalConnection & connection,  /// Connection input has come from
       const PString & value         /// String value of indication
     );
+#if OPAL_FAX
     virtual void OnUserInputTone(
       OpalConnection & connection,  ///<  Connection input has come from
       char tone,                    ///<  Tone received
       int duration                  ///<  Duration of tone
     );
+#endif // OPAL_FAX
     virtual PString ReadUserInput(
       OpalConnection & connection,        ///<  Connection to read input from
       const char * terminators = "#\r\n", ///<  Characters that can terminte input
@@ -930,78 +1030,76 @@ class MyManager : public wxFrame, public OpalManager
       PBoolean & autoDelete                     ///<  Flag for auto delete device
     );
 
+    void OnClose(wxCloseEvent & /*event*/);
+    void OnIconize(wxIconizeEvent & /*event*/);
+    void OnLogMessage(wxCommandEvent & /*event*/);
+    void OnAdjustMenus(wxMenuEvent & /*event*/);
+    void OnStateChange(wxCommandEvent & /*event*/);
+    void OnStreamsChanged(wxCommandEvent & /*event*/);
 
-    void OnClose(wxCloseEvent& event);
-    void OnLogMessage(wxCommandEvent & event);
-    void OnAdjustMenus(wxMenuEvent& event);
-    void OnStateChange(wxCommandEvent & event);
-    void OnStreamsChanged(wxCommandEvent &);
-    void OnRxMessage(wxCommandEvent &);
-    void OnRxCompositionIndicationChanged(wxCommandEvent &);
-    void OnPresence(wxCommandEvent &);
+    void OnMenuQuit(wxCommandEvent & /*event*/);
+    void OnMenuAbout(wxCommandEvent & /*event*/);
+    void OnMenuCall(wxCommandEvent & /*event*/);
+    void OnMenuCallLastDialed(wxCommandEvent & /*event*/);
+    void OnMenuCallLastReceived(wxCommandEvent & /*event*/);
+    void OnCallSpeedDialAudio(wxCommandEvent & /*event*/);
+    void OnCallSpeedDialHandset(wxCommandEvent & /*event*/);
+    void OnSendFax(wxCommandEvent & /*event*/);
+    void OnSendFaxSpeedDial(wxCommandEvent & /*event*/);
+    void OnMenuAnswer(wxCommandEvent & /*event*/);
+    void OnMenuHangUp(wxCommandEvent & /*event*/);
+    void OnNewSpeedDial(wxCommandEvent & /*event*/);
+    void OnViewLarge(wxCommandEvent & /*event*/);
+    void OnViewSmall(wxCommandEvent & /*event*/);
+    void OnViewList(wxCommandEvent & /*event*/);
+    void OnViewDetails(wxCommandEvent & /*event*/);
+    void OnEditSpeedDial(wxCommandEvent & /*event*/);
+    void OnCutSpeedDial(wxCommandEvent & /*event*/);
+    void OnCopySpeedDial(wxCommandEvent & /*event*/);
+    void OnPasteSpeedDial(wxCommandEvent & /*event*/);
+    void OnDeleteSpeedDial(wxCommandEvent & /*event*/);
+    void OnOptions(wxCommandEvent & /*event*/);
+    void OnRequestHold(wxCommandEvent & /*event*/);
+    void OnRetrieve(wxCommandEvent & /*event*/);
+    void OnConference(wxCommandEvent & /*event*/);
+    void OnTransfer(wxCommandEvent & /*event*/);
+    void OnStartRecording(wxCommandEvent & /*event*/);
+    void OnStopRecording(wxCommandEvent & /*event*/);
+    void OnSendAudioFile(wxCommandEvent & /*event*/);
+    void OnAudioDevicePair(wxCommandEvent & /*event*/);
+    void OnAudioDeviceChange(wxCommandEvent & /*event*/);
+    void OnNewCodec(wxCommandEvent & /*event*/);
+    void OnVideoDeviceChange(wxCommandEvent & /*event*/);
+    void OnStartVideo(wxCommandEvent & /*event*/);
+    void OnStopVideo(wxCommandEvent & /*event*/);
+    void OnSendVFU(wxCommandEvent & /*event*/);
+    void OnSendIntra(wxCommandEvent & /*event*/);
+    void OnTxVideoControl(wxCommandEvent & /*event*/);
+    void OnRxVideoControl(wxCommandEvent & /*event*/);
+    void OnMenuPresentationRole(wxCommandEvent & /*event*/);
+    void OnDefVidWinPos(wxCommandEvent & /*event*/);
+    void OnSashPositioned(wxSplitterEvent & /*event*/);
+    void OnSpeedDialActivated(wxListEvent & /*event*/);
+    void OnSpeedDialColumnResize(wxListEvent & /*event*/);
+    void OnSpeedDialRightClick(wxListEvent & /*event*/);
+    void OnSpeedDialEndEdit(wxListEvent & /*event*/);
 
-    void OnMenuQuit(wxCommandEvent& event);
-    void OnMenuAbout(wxCommandEvent& event);
-    void OnMenuCall(wxCommandEvent& event);
-    void OnMenuCallLastDialed(wxCommandEvent& event);
-    void OnMenuCallLastReceived(wxCommandEvent& event);
-    void OnCallSpeedDialAudio(wxCommandEvent& event);
-    void OnCallSpeedDialHandset(wxCommandEvent& event);
-    void OnSendFax(wxCommandEvent& event);
-    void OnSendFaxSpeedDial(wxCommandEvent& event);
-    void OnSendIMSpeedDial(wxCommandEvent& event);
-    void OnStartIM(wxCommandEvent& event);
-    void OnMenuAnswer(wxCommandEvent& event);
-    void OnMenuHangUp(wxCommandEvent& event);
-    void OnNewSpeedDial(wxCommandEvent& event);
-    void OnViewLarge(wxCommandEvent& event);
-    void OnViewSmall(wxCommandEvent& event);
-    void OnViewList(wxCommandEvent& event);
-    void OnViewDetails(wxCommandEvent& event);
-    void OnEditSpeedDial(wxCommandEvent& event);
-    void OnCutSpeedDial(wxCommandEvent& event);
-    void OnCopySpeedDial(wxCommandEvent& event);
-    void OnPasteSpeedDial(wxCommandEvent& event);
-    void OnDeleteSpeedDial(wxCommandEvent& event);
-    void OnOptions(wxCommandEvent& event);
-    void OnRequestHold(wxCommandEvent& event);
-    void OnRetrieve(wxCommandEvent& event);
-    void OnConference(wxCommandEvent& event);
-    void OnTransfer(wxCommandEvent& event);
-    void OnStartRecording(wxCommandEvent& event);
-    void OnStopRecording(wxCommandEvent& event);
-    void OnSendAudioFile(wxCommandEvent& event);
-    void OnAudioDevicePair(wxCommandEvent& event);
-    void OnAudioDevicePreset(wxCommandEvent& event);
-    void OnNewCodec(wxCommandEvent& event);
-    void OnStartVideo(wxCommandEvent& event);
-    void OnStopVideo(wxCommandEvent& event);
-    void OnVFU(wxCommandEvent& event);
-    void OnVideoControl(wxCommandEvent& event);
-    void OnDefVidWinPos(wxCommandEvent& event);
-    void OnSashPositioned(wxSplitterEvent& event);
-    void OnSpeedDialActivated(wxListEvent& event);
-    void OnSpeedDialColumnResize(wxListEvent& event);
-    void OnRightClick(wxListEvent& event);
+    void OnMyPresence(wxCommandEvent & /*event*/);
+#if OPAL_HAS_IM
+    void OnStartIM(wxCommandEvent & /*event*/);
+    void OnInCallIM(wxCommandEvent & /*event*/);
+    void OnSendIMSpeedDial(wxCommandEvent & /*event*/);
+#endif // OPAL_HAS_IM
 
-    void OnMyPresence(wxCommandEvent& event);
-    void OnStartInstantMessage(wxCommandEvent& event);
-
-    void OnEvtRinging(wxCommandEvent & theEvent);
-    void OnEvtEstablished(wxCommandEvent & theEvent);
-    void OnEvtOnHold(wxCommandEvent & theEvent);
-    void OnEvtCleared(wxCommandEvent & theEvent);
+    void OnEvtRinging(wxCommandEvent & /*event*/ theEvent);
+    void OnEvtEstablished(wxCommandEvent & /*event*/ theEvent);
+    void OnEvtOnHold(wxCommandEvent & /*event*/ theEvent);
+    void OnEvtCleared(wxCommandEvent & /*event*/ theEvent);
+    void OnSetTrayTipText(wxCommandEvent & /*event*/);
 
     bool CanDoFax() const;
     bool CanDoIM() const;
 
-    IMDialog * GetOrCreateIMDialog(const PString & conversationId, const PString & local, const PString & remote);
-    void RemoveIMDialog(IMDialog * dialog);
-
-    PDECLARE_NewConversationNotifier(MyManager, OnNewConversation);
-    PDECLARE_IncomingIMNotifier(MyManager, OnIncomingIM);
-    PDECLARE_CompositionIndicationChangedNotifier(MyManager, OnCompositionIndicationChanged);
-
     enum SpeedDialViews {
       e_ViewLarge,
       e_ViewSmall,
@@ -1018,7 +1116,8 @@ class MyManager : public wxFrame, public OpalManager
     );
     bool UpdateSpeedDial(
       int index,
-      const SpeedDialInfo & info
+      const SpeedDialInfo & info,
+      bool saveConfig
     );
 
     enum {
@@ -1038,6 +1137,8 @@ class MyManager : public wxFrame, public OpalManager
       SpeedDialsID
     };
 
+    bool               m_hideMinimised; // Task bar icon gets it back
+    MyTaskBarIcon    * m_taskBarIcon;
     wxSplitterWindow * m_splitter;
     wxNotebook       * m_tabs;
     wxTextCtrl       * m_logWindow;
@@ -1048,7 +1149,7 @@ class MyManager : public wxFrame, public OpalManager
     wxDataFormat       m_ClipboardFormat;
 
     set<SpeedDialInfo> m_speedDialInfo;
-    SpeedDialInfo * GetSelectedSpeedDial(int previous = -1) const;
+    SpeedDialInfo * GetSelectedSpeedDial() const;
 
     MyPCSSEndPoint   * pcssEP;
     OpalLineEndPoint * potsEP;
@@ -1061,6 +1162,9 @@ class MyManager : public wxFrame, public OpalManager
 
     vector<PwxString> m_LocalInterfaces;
     void StartAllListeners();
+#if OPAL_PTLIB_SSL
+    void SetSignalingSecurity(OpalEndPoint * ep, int security, const char * unsecure, const char * secure);
+#endif
 
 #if OPAL_H323
     MyH323EndPoint * h323EP;
@@ -1078,14 +1182,6 @@ class MyManager : public wxFrame, public OpalManager
 
     void StartRegistrations();
     void ReplaceRegistrations(const RegistrationList & newRegistrations);
-
-    // cannot use PDictionary as IMDialog is not descended from PObject
-    typedef std::map<std::string, IMDialog *> IMDialogMap;
-    IMDialogMap m_imDialogMap;
-    PMutex m_imDialogMapMutex;
-
-    bool MonitorPresence(const PString & presentity, const PString & uri, bool start);
-    PDECLARE_PresenceChangeNotifier(MyManager, OnPresenceChange);
 #endif
 
 #if OPAL_CAPI
@@ -1104,6 +1200,10 @@ class MyManager : public wxFrame, public OpalManager
     OpalFaxEndPoint  * m_faxEP;
 #endif
 
+#if OPAL_HAS_IM
+    OpalIMEndPoint * m_imEP;
+#endif
+
     bool      m_autoAnswer;
     PwxString m_LastDialed;
     PwxString m_LastReceived;
@@ -1119,6 +1219,8 @@ class MyManager : public wxFrame, public OpalManager
     int  m_remoteVideoFrameX;
     int  m_remoteVideoFrameY;
 
+    PVideoInputDevice * m_primaryVideoGrabber;
+
     PVideoDevice::OpenArgs m_SecondaryVideoGrabber;
     bool                   m_SecondaryVideoGrabPreview;
     bool                   m_SecondaryVideoOpening;
@@ -1152,7 +1254,7 @@ class MyManager : public wxFrame, public OpalManager
     PTimer        m_RingSoundTimer;
     PDECLARE_NOTIFIER(PTimer, MyManager, OnRingSoundAgain);
     void StopRingSound();
-    void UpdateAudioDevices();
+    void UpdateAudioVideoDevices();
 
     PwxString          m_incomingToken;
     PSafePtr<OpalCall> m_activeCall;
@@ -1185,6 +1287,19 @@ class MyManager : public wxFrame, public OpalManager
     PwxString                  m_lastRecordFile;
     PwxString                  m_lastPlayFile;
 
+    // Presence
+    bool MonitorPresence(const PString & presentity, const PString & uri, bool start);
+    PDECLARE_ASYNC_PresenceChangeNotifier(MyManager, OnPresenceChange);
+
+#if OPAL_HAS_IM
+    // Instant messaging
+    PDECLARE_ASYNC_ConversationNotifier(MyManager, OnConversation);
+
+    // cannot use PDictionary as IMDialog is not descended from PObject
+    typedef std::map<PString, IMDialog *> IMDialogMap;
+    IMDialogMap m_imDialogMap;
+#endif
+
     DECLARE_EVENT_TABLE()
 
   friend class OptionsDialog;
