@@ -24,15 +24,15 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24050 $
+ * $Revision: 27043 $
  * $Author: rjongbloed $
- * $Date: 2010-02-10 17:55:17 -0600 (Wed, 10 Feb 2010) $
+ * $Date: 2012-02-22 01:49:51 -0600 (Wed, 22 Feb 2012) $
  */
 
 #ifndef _OpenPhone_VERSION_H
 #define _OpenPhone_VERSION_H
 
-#include <opal/buildopts.h>
+#include "../../version.h"
 
 #define	PRODUCT_NAME_TEXT	"Open Phone"
 #define	EXE_NAME_TEXT	        "openphone"
@@ -40,12 +40,6 @@
 #define	COPYRIGHT_HOLDER	"Vox Lucida Pty. Ltd. (Australia)"
 
 
-#define MAJOR_VERSION OPAL_MAJOR
-#define MINOR_VERSION OPAL_MINOR
-#define BUILD_TYPE    BetaCode
-#define BUILD_NUMBER  OPAL_BUILD
-
-
 #endif  // _OpenPhone_VERSION_H
 
 
