@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25997 $
+ * $Revision: 29746 $
  * $Author: rjongbloed $
- * $Date: 2011-06-13 19:39:49 -0500 (Mon, 13 Jun 2011) $
+ * $Date: 2013-05-19 22:53:06 -0500 (Sun, 19 May 2013) $
  */
 
 #include "precompile.h"
@@ -64,77 +64,43 @@ void PlayRTP::Main()
 {
   PArgList & args = GetArguments();
 
-  args.Parse("h-help."
-             "m-mapping:"
-             "S-src-ip:"
-             "D-dst-ip:"
-             "s-src-port:"
-             "d-dst-port:"
-             "A-audio-driver:"
-             "a-audio-device:"
-             "V-video-driver:"
-             "v-video-device:"
-             "p-singlestep."
-             "P-payload-file:"
-             "i-info."
-             "f-find."
-             "Y-video-file:"
-             "E:"
-             "T:"
-             "X."
-             "O:"
-             "-session:"
-             "-nodelay."
-#if PTRACING
-             "o-output:"             "-no-output."
-             "t-trace."              "-no-trace."
-#endif
-             , FALSE);
-
-#if PTRACING
-  PTrace::Initialise(args.GetOptionCount('t'),
-                     args.HasOption('o') ? (const char *)args.GetOptionString('o') : NULL,
-         PTrace::Blocks | PTrace::Timestamp | PTrace::Thread | PTrace::FileAndLine);
-#endif
-
-  if (args.HasOption('h') || args.GetCount() == 0) {
-    PError << "usage: " << GetFile().GetTitle() << " [ options ] filename [ filename ... ]\n"
-              "\n"
-              "Available options are:\n"
-              "  --help                   : print this help message.\n"
-              "  -m or --mapping N=fmt    : Set mapping of payload type to format, eg 101=H.264\n"
-              "  -S or --src-ip addr      : Source IP address, default is any\n"
-              "  -D or --dst-ip addr      : Destination IP address, default is any\n"
-              "  -s or --src-port N       : Source UDP port, default is any\n"
-              "  -d or --dst-port N       : Destination UDP port, default is any\n"
-              "  -A or --audio-driver drv : Audio player driver.\n"
-              "  -a or --audio-device dev : Audio player device.\n"
-              "  -V or --video-driver drv : Video display driver to use.\n"
-              "  -v or --video-device dev : Video display device to use.\n"
-              "  -p or --singlestep       : Single step through input data.\n"
-              "  -i or --info             : Display per-frame information.\n"
-              "  -f or --find             : find and display list of RTP sessions.\n"
-              "  -P or --payload-file fn  : write RTP payload to file\n"
-              "  -Y file                  : write decoded video to file\n"
-              "  -E file                  : write event log to file\n"
-              "  -T title                 : put text in extra video information\n"
-              "  -X                       : enable extra video information\n"
-              "  -O --option fmt:opt=val  : Set codec option (may be used multiple times)\n"
-              "                           :  fmt is name of codec, eg \"H.261\"\n"
-              "                           :  opt is name of option, eg \"Target Bit Rate\"\n"
-              "                           :  val is value of option, eg \"48000\"\n"
-              "  --session num            : automatically select session num\n"
-              "  --nodelay                : do not delay as per timestamps\n"
-#if PTRACING
-              "  -o or --output file     : file name for output of log messages\n"       
-              "  -t or --trace           : degree of verbosity in error log (more times for more detail)\n"     
-#endif
-              "\n"
-              "e.g. " << GetFile().GetTitle() << " conversation.pcap\n\n";
+  if (!args.Parse("[Options:]"
+             "m-mapping: Set mapping of payload type to format, eg 101=H.264\n"
+             "S-src-ip: Source IP address, default is any\n"
+             "D-dst-ip: Destination IP address, default is any\n"
+             "s-src-port: Source UDP port, default is any\n"
+             "d-dst-port: Destination UDP port, default is any\n"
+             "A-audio-driver: Audio player driver.\n"
+             "a-audio-device: Audio player device.\n"
+             "V-video-driver: Video display driver to use.\n"
+             "v-video-device: Video display device to use.\n"
+             "p-singlestep. Single step through input data.\n"
+             "P-payload-file: write RTP payload to file\n"
+             "i-info. Display per-frame information.\n"
+             "f-find. find and display list of RTP sessions.\n"
+             "Y-video-file: write decoded video to file\n"
+             "E: write event log to file\n"
+             "T: put text in extra video information\n"
+             "X. enable extra video information\n"
+             "O: Set codec option (may be used multiple times)\r"
+                 "   fmt is name of codec, eg \"H.261\"\r"
+                 "   opt is name of option, eg \"Target Bit Rate\"\r"
+                 "   val is value of option, eg \"48000\"\n"
+             "-session: automatically select session num\n"
+             "-rotate: Rotate on RTP header extension N\n"
+             "-nodelay. do not delay as per timestamps\n"
+             PTRACE_ARGLIST
+             "h-help. print this help message.\n"
+             , false) || args.HasOption('h')) {
+    args.Usage(cerr, "[ options ] filename [ filename ... ]") << "\n"
+               "e.g. " << GetFile().GetTitle() << " conversation.pcap\n\n";
     return;
   }
 
+  PTRACE_INITIALISE(args);
+
   m_extendedInfo = args.HasOption('X') || args.HasOption('T');
+  m_rotateExtensionId = args.GetOptionString("rotate").AsUnsigned();
   m_noDelay      = args.HasOption("nodelay");
 
   PStringArray options = args.GetOptionString('O').Lines();
@@ -166,13 +132,15 @@ void PlayRTP::Main()
   }
 
   OpalPCAPFile pcap;
-  if (!pcap.Open(args[0]))
+  if (!pcap.Open(args[0])) {
+    cerr << "Could not open file \"" << args[0] << "\"\n";
     return;
+  }
 
   if (args.HasOption('f')) {
     OpalPCAPFile::DiscoveredRTPMap discoveredRTPMap;
     if (!pcap.DiscoverRTP(discoveredRTPMap)) {
-      cerr << "error: no RTP sessions found" << endl;
+      cerr << "No RTP sessions found" << endl;
       return;
     }
     cout << "Found " << discoveredRTPMap.size() << " sessions:\n" << discoveredRTPMap << endl;
@@ -204,15 +172,15 @@ void PlayRTP::Main()
   }
 
   if (args.HasOption('S') || args.HasOption('D') || args.HasOption('s') || args.HasOption('d')) {
-    pcap.SetFilterSrcIP(args.GetOptionString('S', PIPSocket::GetDefaultIpAny().AsString()));
-    pcap.SetFilterDstIP(args.GetOptionString('D', PIPSocket::GetDefaultIpAny().AsString()));
+    pcap.SetFilterSrcIP(PIPSocket::Address(args.GetOptionString('S', PIPSocket::GetDefaultIpAny().AsString())));
+    pcap.SetFilterDstIP(PIPSocket::Address(args.GetOptionString('D', PIPSocket::GetDefaultIpAny().AsString())));
     pcap.SetFilterSrcPort(PIPSocket::GetPortByService("udp", args.GetOptionString('s')));
     pcap.SetFilterDstPort(PIPSocket::GetPortByService("udp", args.GetOptionString('d', "5000")));
   }
   else {
     OpalPCAPFile::DiscoveredRTPMap discoveredRTPMap;
     if (!pcap.DiscoverRTP(discoveredRTPMap)) {
-      cerr << "error: no RTP sessions found - please use -S/-D/-s/-d option to specify session manually" << endl;
+      cerr << "No RTP sessions found - please use -S/-D/-s/-d option to specify session manually" << endl;
       return;
     }
 
@@ -223,14 +191,14 @@ void PlayRTP::Main()
       }
     }
     else {
-      cout << "Select one of the following sessions:\n" << discoveredRTPMap << endl;
+      cout << "Select one of the following sessions (index [ format ]):\n" << discoveredRTPMap << endl;
       for (;;) {
         cout << "Select (1-" << discoveredRTPMap.size()*2 << ") ? " << flush;
-        size_t selected;
-        cin >> selected;
-        if (pcap.SetFilters(discoveredRTPMap, selected))
+        PString line;
+        cin >> line;
+        if (pcap.SetFilters(discoveredRTPMap, line.AsUnsigned(), line.Mid(line.Find(' ')).Trim()))
           break;
-        cout << "Session " << selected << " is not valid" << endl;
+        cout << "Session/format " << line << " is not valid" << endl;
       }
     }
   }
@@ -390,8 +358,12 @@ void PlayRTP::Play(OpalPCAPFile & pcap)
   RTP_DataFrame::PayloadTypes lastUnsupportedPayloadType = RTP_DataFrame::IllegalPayloadType;
   DWORD lastTimeStamp = 0;
 
+  PINDEX totalBytes = 0;
+  unsigned intraFrames = 0;
+  int qualitySum = 0;
   unsigned fragmentationCount = 0;
   unsigned nextSequenceNumber = 0;
+  unsigned missingPackets = 0;
   m_packetCount = 0;
 
   RTP_DataFrame extendedData;
@@ -409,8 +381,11 @@ void PlayRTP::Play(OpalPCAPFile & pcap)
       continue;
 
     unsigned thisSequenceNumber = rtp.GetSequenceNumber();
-    if (nextSequenceNumber != 0 && thisSequenceNumber != nextSequenceNumber)
+    if (nextSequenceNumber != 0 && thisSequenceNumber != nextSequenceNumber) {
       cout << "Received SN=" << thisSequenceNumber << ", expected SN=" << nextSequenceNumber << endl;
+      if (thisSequenceNumber > nextSequenceNumber)
+        missingPackets += (thisSequenceNumber - nextSequenceNumber);
+    }
     nextSequenceNumber = thisSequenceNumber+1;
 
     if (rtpStreamPayloadType != rtp.GetPayloadType()) {
@@ -517,6 +492,8 @@ void PlayRTP::Play(OpalPCAPFile & pcap)
     if (m_singleStep) 
       cout << "Input packet of length " << rtp.GetPayloadSize() << (rtp.GetMarker() ? " with MARKER" : "") << " -> ";
 
+    totalBytes += rtp.GetPayloadSize();
+
     m_vfu = false;
     RTP_DataFrameList output;
     if (!m_transcoder->ConvertFrames(rtp, output)) {
@@ -531,9 +508,7 @@ void PlayRTP::Play(OpalPCAPFile & pcap)
     else {
       if (m_singleStep) {
         cout << output.GetSize() << " packets" << endl;
-        char ch;
-        cin >> ch;
-        if (ch == 'c')
+        if (cin.get() == 'c')
           m_singleStep = false;
       }
 
@@ -547,19 +522,47 @@ void PlayRTP::Play(OpalPCAPFile & pcap)
         else {
           ++m_videoFrames;
 
+          OpalMediaStatistics stats;
+          m_transcoder->GetStatistics(stats);
+          if (stats.m_quality >= 0)
+            qualitySum += stats.m_quality;
+
           OpalVideoTranscoder * video = (OpalVideoTranscoder *)m_transcoder;
-          const OpalVideoTranscoder::FrameHeader * frame = (const OpalVideoTranscoder::FrameHeader *)data.GetPayloadPtr();
           if (video->WasLastFrameIFrame()) {
-            m_eventLog << "Frame " << m_videoFrames << ": I-frame received";
+            ++intraFrames;
+            m_eventLog << "Frame " << m_videoFrames << ": I-Frame received";
             if (m_videoError)
               m_eventLog << " - decode error cleared";
             m_eventLog << endl;
             m_videoError = false;
           }
 
+          OpalVideoTranscoder::FrameHeader * frame = (OpalVideoTranscoder::FrameHeader *)data.GetPayloadPtr();
           m_yuvFile.SetFrameSize(frame->width, frame->height + (m_extendedInfo ? m_extraHeight : 0));
 
+          unsigned extensionId;
+          PINDEX extensionLen;
+          BYTE * extensionData = data.GetHeaderExtension(extensionId, extensionLen, 0);
+          if (extensionData != NULL && extensionId == m_rotateExtensionId) {
+            switch (*extensionData >> 4) {
+              case 0 : // Portrait left
+                PColourConverter::RotateYUV420P(90, frame->width, frame->height, OPAL_VIDEO_FRAME_DATA_PTR(frame));
+                std::swap(frame->width, frame->height);
+                break;
+              case 1 : // Landscape up
+                break;
+              case 2 : // Portrait right
+                PColourConverter::RotateYUV420P(-90, frame->width, frame->height, OPAL_VIDEO_FRAME_DATA_PTR(frame));
+                std::swap(frame->width, frame->height);
+                break;
+              case 3 : // Lnadscape down
+                PColourConverter::RotateYUV420P(180, frame->width, frame->height, OPAL_VIDEO_FRAME_DATA_PTR(frame));
+                break;
+            }
+          }
+
           if (!m_extendedInfo) {
+
             m_display->SetFrameSize(frame->width, frame->height);
             m_display->SetFrameData(frame->x, frame->y,
                                     frame->width, frame->height,
@@ -605,8 +608,6 @@ void PlayRTP::Play(OpalPCAPFile & pcap)
 
             m_yuvFile.WriteFrame(OPAL_VIDEO_FRAME_DATA_PTR(extendedFrame));
           }
-          //if (m_vfu)
-          //  m_singleStep = true;
 
           if (m_info > 1)
             cout << ',' << frame->width << ',' << frame->height;
@@ -636,29 +637,47 @@ void PlayRTP::Play(OpalPCAPFile & pcap)
 
   m_eventLog << "Packet " << m_packetCount << ": completed" << endl;
 
-  delete m_transcoder;
-  m_transcoder = NULL;
-
-
   // Output final stats.
-  cout << (m_yuvFile.IsOpen() ? "Written " : "Played ") << m_packetCount << " packets";
+  cout << '\n';
 
-  if (m_videoFrames > 0) {
-    cout << ", " << m_videoFrames << " frames at "
-         << m_yuvFile.GetFrameWidth() << "x" << m_yuvFile.GetFrameHeight();
-    if (!m_yuvFile.IsOpen()) {
-      PTimeInterval playTime = PTimer::Tick() - playStartTick;
-      cout << ", " << playTime << " seconds, "
-           << fixed << setprecision(1)
-           << (m_videoFrames*1000.0/playTime.GetMilliSeconds()) << "fps";
-    }
+  if (m_yuvFile.IsOpen())
+    cout <<   "Written file   : " << m_yuvFile.GetFilePath() << '\n';
+
+  cout <<     "Payload Type   : " << rtpStreamPayloadType;
+  if (m_transcoder != NULL)
+    cout <<   " (" << m_transcoder->GetInputFormat() << ')';
+  cout <<     "\n"
+              "Source         : " << pcap.GetFilterSrcIP() << ':' << pcap.GetFilterSrcPort() << "\n"
+              "Destination    : " << pcap.GetFilterDstIP() << ':' << pcap.GetFilterDstPort() << "\n"
+              "Total packets  : " << m_packetCount << "\n"
+              "Total bytes    : " << totalBytes << '\n';
+
+  PTimeInterval playTime = PTimer::Tick() - playStartTick;
+  if (!m_yuvFile.IsOpen() && playTime > 0) {
+    cout <<   "Duration       : " << playTime << " seconds\n"
+         <<   "Bit rate       : "  << fixed << setprecision(1)
+           << (totalBytes*8/playTime.GetMilliSeconds()) << "kbps\n";
   }
 
-  if (fragmentationCount > 0)
-    cout << ", " << fragmentationCount << " fragments";
-
-  cout << endl;
+  if (m_videoFrames > 0) {
+    cout <<   "Resolution     : " << m_yuvFile.GetFrameWidth() << "x" << m_yuvFile.GetFrameHeight() << "\n"
+              "Video frames   : " << m_videoFrames << "\n"
+              "Intra Frames   : " << intraFrames << '\n';
+    if (m_videoFrames > 0 && qualitySum > 0)
+      cout << "Avg quaility   : " << (qualitySum/m_videoFrames) << '\n';
+    if (!m_yuvFile.IsOpen() && playTime > 0)
+      cout << "Frame rate     : " << fixed << setprecision(1)
+           << (m_videoFrames*1000.0/playTime.GetMilliSeconds()) << "fps\n";
+  }
+  else {
+    if (playTime > 0)
+      cout << "Avg frame time : " << fixed << setprecision(1)
+           << (playTime.GetMilliSeconds()/m_packetCount) << "ms\n";
+  }
 
+  cout <<     "Missing packets: " << missingPackets << "\n"
+              "IP fragments   : " << fragmentationCount
+       << endl;
 
   if (!m_encodedFileName.IsEmpty()) {
     PStringStream args; 
@@ -672,19 +691,24 @@ void PlayRTP::Play(OpalPCAPFile & pcap)
     cmd.WaitForTermination();
     cout << "done" << endl;
   }
+
+  // Clean up
+  delete m_transcoder;
+  m_transcoder = NULL;
 }
 
 
-void PlayRTP::OnTranscoderCommand(OpalMediaCommand & command, INT /*extra*/)
+void PlayRTP::OnTranscoderCommand(OpalMediaCommand & command, P_INT_PTR /*extra*/)
 {
   if (PIsDescendant(&command, OpalVideoUpdatePicture)) {
-    m_eventLog << "Packet " << m_packetCount << ", frame " << m_videoFrames << ": decoding error (VFU sent)";
+    PStringStream msg;
+    msg << "Packet " << m_packetCount << ", frame " << m_videoFrames << ": decoding error";
       
     OpalVideoPictureLoss * loss = dynamic_cast<OpalVideoPictureLoss *>(&command);
     if (loss != NULL)
-      m_eventLog << " for sn=" << loss->GetSequenceNumber() << ", ts=" << loss->GetSequenceNumber();
-    m_eventLog << endl;
-    cout << "Decoder error in received stream: frame " << m_videoFrames << endl;
+      msg << " for sn=" << loss->GetSequenceNumber() << ", ts=" << loss->GetTimestamp();
+    m_eventLog << msg << endl;
+    cout << msg << endl;
     m_videoError = m_vfu = true;
   }
 }
