@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24409 $
+ * $Revision: 26798 $
  * $Author: rjongbloed $
- * $Date: 2010-05-27 18:41:33 -0500 (Thu, 27 May 2010) $
+ * $Date: 2011-12-14 02:30:18 -0600 (Wed, 14 Dec 2011) $
  */
 
 #ifndef _PlayRTP_MAIN_H
@@ -32,6 +32,9 @@
 #include <ptclib/pvidfile.h>
 
 
+class OpalPCAPFile;
+
+
 class PlayRTP : public PProcess
 {
   PCLASSINFO(PlayRTP, PProcess)
@@ -41,21 +44,14 @@ class PlayRTP : public PProcess
     ~PlayRTP();
 
     virtual void Main();
-    void Play(const PFilePath & filename);
-    void Find(const PFilePath & filename);
+    void Play(OpalPCAPFile & pcap);
 
     PDECLARE_NOTIFIER(OpalMediaCommand, PlayRTP, OnTranscoderCommand);
 
-    std::map<RTP_DataFrame::PayloadTypes, OpalMediaFormat> m_payloadType2mediaFormat;
-
-    PIPSocket::Address m_srcIP;
-    PIPSocket::Address m_dstIP;
-
-    WORD m_srcPort;
-    WORD m_dstPort;
     bool m_singleStep;
     int  m_info;
     bool m_extendedInfo;
+    unsigned m_rotateExtensionId;
     bool m_noDelay;
     bool m_writeEventLog;
 
