@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22141 $
+ * $Revision: 29659 $
  * $Author: rjongbloed $
- * $Date: 2009-03-03 02:26:18 +0000 (Tue, 03 Mar 2009) $
+ * $Date: 2013-05-07 22:02:07 -0500 (Tue, 07 May 2013) $
  */
 
 #include <ptlib.h>
@@ -34,6 +34,10 @@
 #include <iax2/iax2.h>
 #endif
 
+#if OPAL_CAPI
+#include <lids/capi_ep.h>
+#endif
+
 #if OPAL_SIP
 #include <sip/sip.h>
 #endif
@@ -48,10 +52,11 @@
 #endif
 
 #include <opal/transcoders.h>
-#include <lids/lidep.h>
-#include <ptclib/pstun.h>
-#include <ptlib/config.h>
 #include <codec/opalpluginmgr.h>
+#include <lids/lidep.h>
+
+#include <ptlib/config.h>
+#include <ptclib/pstun.h>
 
 #include "main.h"
 #include "../../version.h"
@@ -59,7 +64,6 @@
 
 #define new PNEW
 
-
 PCREATE_PROCESS(SimpleOpalProcess);
 
 ///////////////////////////////////////////////////////////////
@@ -74,7 +78,7 @@ SimpleOpalProcess::SimpleOpalProcess()
 void SimpleOpalProcess::Main()
 {
   cout << GetName()
-       << " Version " << GetVersion(PTrue)
+       << " Version " << GetVersion(true)
        << " by " << GetManufacturer()
        << " on " << GetOSClass() << ' ' << GetOSName()
        << " (" << GetOSVersion() << '-' << GetOSHardware() << ")\n\n";
@@ -131,6 +135,8 @@ void SimpleOpalProcess::Main()
              "-sip-listen:"
              "-sip-proxy:"
              "-sip-domain:"
+             "-sip-contact-nomultiple."
+             "-sip-contact-noprivate."  
              "-sip-user-agent:"
              "-sip-ui:"
              "-stun:"
@@ -146,8 +152,8 @@ void SimpleOpalProcess::Main()
              "u-user:"
              "-udp-base:"
              "-udp-max:"
-             "-use-long-mime."
 #if OPAL_VIDEO
+             "C-ratecontrol:"
              "-rx-video." "-no-rx-video."
              "-tx-video." "-no-tx-video."
              "-grabber:"
@@ -155,17 +161,24 @@ void SimpleOpalProcess::Main()
              "-grabchannel:"
              "-display:"
              "-displaydriver:"
+             "-preview:"
+             "-previewdriver:"
              "-video-size:"
              "-video-rate:"
+             "-video-bitrate:"
 #endif
 #if OPAL_IVR
              "V-no-ivr."
              "x-vxml:"
 #endif
 #if OPAL_IAX2
-	     "X-no-iax2."
+             "X-no-iax2."
+	     "-iaxport:"
 #endif
-          , PFalse);
+#if OPAL_CAPI
+             "-no-capi."
+#endif
+          , false);
 
 
   if (args.HasOption('h') || (!args.HasOption('l') && args.GetCount() == 0)) {
@@ -201,11 +214,15 @@ void SimpleOpalProcess::Main()
             "     --grabber dev        : Set the video grabber device.\n"
             "     --grabdriver dev     : Set the video grabber driver (if device name is ambiguous).\n"
             "     --grabchannel num    : Set the video grabber device channel.\n"
-            "     --display dev        : Set the video display device.\n"
-            "     --displaydriver dev  : Set the video display driver (if device name is ambiguous).\n"
+            "     --display dev        : Set the remote video display device.\n"
+            "     --displaydriver dev  : Set the remote video display driver (if device name is ambiguous).\n"
+            "     --preview dev        : Set the local video preview device.\n"
+            "     --previewdriver dev  : Set the local video preview driver (if device name is ambiguous).\n"
             "     --video-size size    : Set the size of the video for all video formats, use\n"
             "                          : \"qcif\", \"cif\", WxH etc\n"
             "     --video-rate rate    : Set the frame rate of video for all video formats\n"
+            "     --video-bitrate rate : Set the bit rate for all video formats\n"
+            "     -C string            : Enable and select video rate control algorithm\n"
             "\n"
 #endif
 
@@ -219,8 +236,9 @@ void SimpleOpalProcess::Main()
             "                          : '*' is all interfaces, (default udp$:*:5060)\n"
             "     --sip-user-agent name: SIP UserAgent name to use.\n"
             "     --sip-ui type        : Set type of user indications to use for SIP. Can be one of 'rfc2833', 'info-tone', 'info-string'.\n"
-            "     --use-long-mime      : Use long MIME headers on outgoing SIP messages\n"
             "     --sip-domain str     : set authentication domain/realm\n"
+            "     --sip-contact-nomultiple : do not send multiple contacts in Contact field\n"
+            "     --sip-contact-noprivate  : do not send private contacts in Contact field\n"
             "\n"
 #endif
 
@@ -280,15 +298,18 @@ void SimpleOpalProcess::Main()
             "     --rtp-base n         : Set RTP port base (default 5000)\n"
             "     --rtp-max n          : Set RTP port max (default base+199)\n"
             "     --rtp-tos n          : Set RTP packet IP TOS bits to n\n"
-	          "     --stun server        : Set STUN server\n"
+#ifdef P_STUN
+	    "     --stun server        : Set STUN server\n"
+#endif
             "\n"
             "Debug options:\n"
-#if PTRACING
-            "  -t --trace              : Enable trace, use multiple times for more detail.\n"
-            "  -o --output             : File for trace output, default is stderr.\n"
-#endif
+            PTRACE_ARGLIST
 #if OPAL_IAX2
             "  -X --no-iax2            : Remove support for iax2\n"
+            "     --iaxport n          : Set port to use (def. 4569)\n"
+#endif
+#if OPAL_CAPI
+            "     --no-capi            : Remove support for CAPI ISDN\n"
 #endif
             "  -h --help               : This help message.\n"
             "\n"
@@ -348,11 +369,7 @@ void SimpleOpalProcess::Main()
     return;
   }
 
-#if PTRACING
-  PTrace::Initialise(args.GetOptionCount('t'),
-                     args.HasOption('o') ? (const char *)args.GetOptionString('o') : NULL,
-                     PTrace::Timestamp|PTrace::Thread|PTrace::FileAndLine);
-#endif
+  PTRACE_INITIALISE(args);
 
   // Create the Opal Manager and initialise it
   opal = new MyManager;
@@ -384,6 +401,9 @@ MyManager::MyManager()
 #if OPAL_IAX2
   iax2EP = NULL;
 #endif
+#if OPAL_CAPI
+  capiEP = NULL;
+#endif
 #if OPAL_IVR
   ivrEP  = NULL;
 #endif
@@ -391,7 +411,7 @@ MyManager::MyManager()
   faxEP = NULL;
 #endif
 
-  pauseBeforeDialing = PFalse;
+  pauseBeforeDialing = false;
 }
 
 
@@ -418,18 +438,22 @@ PBoolean MyManager::Initialise(PArgList & args)
   if (args.HasOption("no-tx-video"))
     SetAutoStartTransmitVideo(false);
 
+  // video input options
   if (args.HasOption("grabber")) {
     PVideoDevice::OpenArgs video = GetVideoInputDevice();
     video.deviceName = args.GetOptionString("grabber");
     video.driverName = args.GetOptionString("grabdriver");
     video.channelNumber = args.GetOptionString("grabchannel").AsInteger();
+    if (args.HasOption("video-rate")) 
+      video.rate = args.GetOptionString("video-rate").AsUnsigned();
     if (!SetVideoInputDevice(video)) {
       cerr << "Unknown grabber device " << video.deviceName << "\n"
               "Available devices are:" << setfill(',') << PVideoInputDevice::GetDriversDeviceNames("") << endl;
-      return PFalse;
+      return false;
     }
   }
 
+  // remote video display options
   if (args.HasOption("display")) {
     PVideoDevice::OpenArgs video = GetVideoOutputDevice();
     video.deviceName = args.GetOptionString("display");
@@ -437,7 +461,19 @@ PBoolean MyManager::Initialise(PArgList & args)
     if (!SetVideoOutputDevice(video)) {
       cerr << "Unknown display device " << video.deviceName << "\n"
               "Available devices are:" << setfill(',') << PVideoOutputDevice::GetDriversDeviceNames("") << endl;
-      return PFalse;
+      return false;
+    }
+  }
+
+  // local video preview options
+  if (args.HasOption("preview")) {
+    PVideoDevice::OpenArgs video = GetVideoPreviewDevice();
+    video.deviceName = args.GetOptionString("preview");
+    video.driverName = args.GetOptionString("previewdriver");
+	if (!SetVideoPreviewDevice(video)) {
+      cerr << "Unknown preview device " << video.deviceName << "\n"
+              "Available devices are:" << setfill(',') << PVideoOutputDevice::GetDriversDeviceNames("") << endl;
+      return false;
     }
   }
 #endif
@@ -458,7 +494,7 @@ PBoolean MyManager::Initialise(PArgList & args)
       SetAudioJitterDelay(minJitter, maxJitter);
     else {
       cerr << "Jitter should be between 20 and 1000 milliseconds.\n";
-      return PFalse;
+      return false;
     }
   }
 
@@ -472,10 +508,12 @@ PBoolean MyManager::Initialise(PArgList & args)
 
   cout << "Jitter buffer: "  << GetMinAudioJitterDelay() << '-' << GetMaxAudioJitterDelay() << " ms\n";
 
+#if P_NAT
   if (args.HasOption("translate")) {
-    SetTranslationAddress(args.GetOptionString("translate"));
-    cout << "External address set to " << GetTranslationAddress() << '\n';
+    SetNATServer("Fixed", args.GetOptionString("translate"));
+    cout << "External address set to " << GetNATServer() << '\n';
   }
+#endif
 
   if (args.HasOption("portbase")) {
     unsigned portbase = args.GetOptionString("portbase").AsUnsigned();
@@ -501,25 +539,30 @@ PBoolean MyManager::Initialise(PArgList & args)
     unsigned tos = args.GetOptionString("rtp-tos").AsUnsigned();
     if (tos > 255) {
       cerr << "IP Type Of Service bits must be 0 to 255.\n";
-      return PFalse;
+      return false;
     }
-    SetRtpIpTypeofService(tos);
+    SetMediaTypeOfService(tos);
   }
 
   cout << "TCP ports: " << GetTCPPortBase() << '-' << GetTCPPortMax() << "\n"
           "UDP ports: " << GetUDPPortBase() << '-' << GetUDPPortMax() << "\n"
           "RTP ports: " << GetRtpIpPortBase() << '-' << GetRtpIpPortMax() << "\n"
-          "RTP IP TOS: 0x" << hex << (unsigned)GetRtpIpTypeofService() << dec << "\n"
-          "STUN server: " << flush;
+          "RTP IP TOS: 0x" << hex << (unsigned)GetMediaTypeOfService() << dec << "\n"
+#ifdef P_STUN
+          "STUN server: "
+#endif
+          << flush;
 
+#ifdef P_STUN
   if (args.HasOption("stun"))
-    SetSTUNServer(args.GetOptionString("stun"));
+    SetNATServer("STUN", args.GetOptionString("stun"));
 
-  if (stun != NULL)
-    cout << stun->GetServer() << " replies " << stun->GetNatTypeName();
+  if (m_natMethod != NULL)
+    cout << m_natMethod->GetServer() << " replies " << m_natMethod->GetNatTypeName();
   else
     cout << "None";
   cout << '\n';
+#endif
 
   OpalMediaFormatList allMediaFormats;
 
@@ -562,25 +605,24 @@ PBoolean MyManager::Initialise(PArgList & args)
   if (!args.HasOption('S')) {
     pcssEP = new MyPCSSEndPoint(*this);
 
-    pcssEP->SetSoundChannelBufferDepth(10);
-
     pcssEP->autoAnswer = args.HasOption('a');
     cout << "Auto answer is " << (pcssEP->autoAnswer ? "on" : "off") << "\n";
           
     if (!pcssEP->SetSoundDevice(args, "sound", PSoundChannel::Recorder))
-      return PFalse;
+      return false;
     if (!pcssEP->SetSoundDevice(args, "sound", PSoundChannel::Player))
-      return PFalse;
+      return false;
     if (!pcssEP->SetSoundDevice(args, "sound-in", PSoundChannel::Recorder))
-      return PFalse;
+      return false;
     if (!pcssEP->SetSoundDevice(args, "sound-out", PSoundChannel::Player))
-      return PFalse;
+      return false;
 
     allMediaFormats += pcssEP->GetMediaFormats();
 
     cout << "Sound output device: \"" << pcssEP->GetSoundChannelPlayDevice() << "\"\n"
             "Sound  input device: \"" << pcssEP->GetSoundChannelRecordDevice() << "\"\n"
 #if OPAL_VIDEO
+			"Video preview device: \"" << GetVideoPreviewDevice().deviceName << "\"\n"
             "Video output device: \"" << GetVideoOutputDevice().deviceName << "\"\n"
             "Video  input device: \"" << GetVideoInputDevice().deviceName << '"'
 #endif
@@ -594,7 +636,7 @@ PBoolean MyManager::Initialise(PArgList & args)
   if (!args.HasOption("no-h323")) {
     h323EP = new H323EndPoint(*this);
     if (!InitialiseH323EP(args, false, h323EP))
-      return PFalse;
+      return false;
   }
 
 #endif
@@ -605,11 +647,16 @@ PBoolean MyManager::Initialise(PArgList & args)
 
   if (!args.HasOption("no-iax2")) {
     iax2EP = new IAX2EndPoint(*this);
-    
+
+    PStringArray listeners;
+    if (args.HasOption("iaxport"))
+      listeners += "udp$*:" + args.GetOptionString("iaxport");
+    iax2EP->StartListeners(listeners);
+
     if (!iax2EP->InitialisedOK()) {
       cerr << "IAX2 Endpoint is not initialised correctly" << endl;
-      cerr << "Is there another application using port 4569? " << endl;
-      return PFalse;
+      cerr << "Is there another application using the iax port? " << endl;
+      return false;
     }
 
     if (args.HasOption('p'))
@@ -622,6 +669,18 @@ PBoolean MyManager::Initialise(PArgList & args)
   }
 #endif
 
+#if OPAL_CAPI
+  ///////////////////////////////////////
+  // Create CAPI handler
+
+  if (!args.HasOption("no-capi")) {
+    capiEP = new OpalCapiEndPoint(*this);
+    
+    if (!capiEP->OpenControllers())
+      cerr << "CAPI Endpoint failed to initialise any controllers." << endl;
+  }
+#endif
+
 #if OPAL_SIP
 
   ///////////////////////////////////////
@@ -635,7 +694,7 @@ PBoolean MyManager::Initialise(PArgList & args)
 
     PString str = args.GetOptionString("sip-ui");
     if (str *= "rfc2833")
-      sipEP->SetSendUserInputMode(OpalConnection::SendUserInputAsSeparateRFC2833);
+      sipEP->SetSendUserInputMode(OpalConnection::SendUserInputAsRFC2833);
     else if (str *= "info-tone")
       sipEP->SetSendUserInputMode(OpalConnection::SendUserInputAsTone);
     else if (str *= "info-string")
@@ -644,35 +703,37 @@ PBoolean MyManager::Initialise(PArgList & args)
     if (args.HasOption("sip-proxy"))
       sipEP->SetProxy(args.GetOptionString("sip-proxy"));
 
-    // set MIME format
-    sipEP->SetMIMEForm(args.HasOption("use-long-mime"));
-
     // Get local username, multiple uses of -u indicates additional aliases
     if (args.HasOption('u')) {
       PStringArray aliases = args.GetOptionString('u').Lines();
       sipEP->SetDefaultLocalPartyName(aliases[0]);
     }
 
-    sipEP->SetRetryTimeouts(10000, 30000);
-
     // Start the listener thread for incoming calls.
     PStringArray listeners = args.GetOptionString("sip-listen").Lines();
     if (!sipEP->StartListeners(listeners)) {
       cerr <<  "Could not open any SIP listener from "
             << setfill(',') << listeners << endl;
-      return PFalse;
+      return false;
     }
     cout <<  "SIP started on " << setfill(',') << sipEP->GetListeners() << setfill(' ') << endl;
 
     if (args.HasOption('r')) {
-      PString registrar = args.GetOptionString('r');
-      cout << "Using SIP registrar " << registrar << " ... " << flush;
-      if (sipEP->Register(registrar, args.GetOptionString('u'), args.GetOptionString('u'), args.GetOptionString('p'), args.GetOptionString("sip-domain")))
-        cout << "done.";
+      SIPRegister::Params params;
+      params.m_registrarAddress = args.GetOptionString('r');
+      params.m_addressOfRecord = args.GetOptionString('u');
+      params.m_password = args.GetOptionString('p');
+      params.m_realm = args.GetOptionString("sip-domain");
+      if (args.HasOption("sip-contact-nomultiple"))
+        params.m_compatibility = SIPRegister::e_CannotRegisterMultipleContacts;
+      else if (args.HasOption("sip-contact-noprivate"))
+        params.m_compatibility = SIPRegister::e_CannotRegisterPrivateContacts  ;
+      PString aor;
+      if (sipEP->Register(params, aor))
+        cout << "Using SIP registrar " << params.m_registrarAddress << " for " << aor << endl;
       else
-        cout << "failed!";
-      cout << endl;
-      pauseBeforeDialing = PTrue;
+        cout << "Could not use SIP registrar " << params.m_registrarAddress << endl;
+      pauseBeforeDialing = true;
     }
   }
 
@@ -699,6 +760,7 @@ PBoolean MyManager::Initialise(PArgList & args)
 #endif
 
 #if OPAL_FAX
+#if OPAL_PTLIB_ASN
   ///////////////////////////////////////
   // Create T38 protocol handler
   {
@@ -706,6 +768,7 @@ PBoolean MyManager::Initialise(PArgList & args)
     faxEP = new OpalFaxEndPoint(*this);
     allMediaFormats += faxEP->GetMediaFormats();
   }
+#endif
 #endif
 
   ///////////////////////////////////////
@@ -714,7 +777,7 @@ PBoolean MyManager::Initialise(PArgList & args)
   if (args.HasOption('d')) {
     if (!SetRouteTable(args.GetOptionString('d').Lines())) {
       cerr <<  "No legal entries in dial peer!" << endl;
-      return PFalse;
+      return false;
     }
   }
 
@@ -785,11 +848,18 @@ PBoolean MyManager::Initialise(PArgList & args)
                                                                                                                                             
 #if OPAL_IAX2
   if (pcssEP != NULL) {
-    AddRouteEntry("iax2:.*  = pc:");
+    AddRouteEntry("iax2:.* = pc:");
     AddRouteEntry("pc:.*   = iax2:<da>");
   }
 #endif
 
+#if OPAL_CAPI
+  if (capiEP != NULL) {
+    AddRouteEntry("isdn:.* = pc:");
+    AddRouteEntry("pc:.*   = isdn:<da>");
+  }
+#endif
+
 #if OPAL_FAX
   if (faxEP != NULL) {
     AddRouteEntry("sip:.*  = t38:<da>");
@@ -817,10 +887,7 @@ PBoolean MyManager::Initialise(PArgList & args)
     srcEP = defaultSrcEP;
 
   allMediaFormats = OpalTranscoder::GetPossibleFormats(allMediaFormats); // Add transcoders
-  for (PINDEX i = 0; i < allMediaFormats.GetSize(); i++) {
-    if (!allMediaFormats[i].IsTransportable())
-      allMediaFormats.RemoveAt(i--); // Don't show media formats that are not used over the wire
-  }
+  allMediaFormats.RemoveNonTransportable();
   allMediaFormats.Remove(GetMediaFormatMask());
   allMediaFormats.Reorder(GetMediaFormatOrder());
 
@@ -830,6 +897,7 @@ PBoolean MyManager::Initialise(PArgList & args)
           "Available codecs: " << allMediaFormats << setfill(' ') << endl;
 
 #if OPAL_VIDEO
+  PString rcOption = args.GetOptionString('C');
   OpalMediaFormat::GetAllRegisteredMediaFormats(allMediaFormats);
   for (PINDEX i = 0; i < allMediaFormats.GetSize(); i++) {
     OpalMediaFormat mediaFormat = allMediaFormats[i];
@@ -849,8 +917,13 @@ PBoolean MyManager::Initialise(PArgList & args)
         unsigned rate = args.GetOptionString("video-rate").AsUnsigned();
         unsigned frameTime = 90000 / rate;
         mediaFormat.SetOptionInteger(OpalMediaFormat::FrameTimeOption(), frameTime);
-        mediaFormat.SetOptionBoolean("Rate Control Enable", true);
       }
+      if (args.HasOption("video-bitrate")) {
+        unsigned rate = args.GetOptionString("video-bitrate").AsUnsigned();
+        mediaFormat.SetOptionInteger(OpalMediaFormat::TargetBitRateOption(), rate);
+      }
+      if (!rcOption.IsEmpty())
+        mediaFormat.SetOptionString(OpalVideoFormat::RateControllerOption(), rcOption);
       OpalMediaFormat::SetRegisteredMediaFormat(mediaFormat);
     }
   }
@@ -892,7 +965,7 @@ PBoolean MyManager::Initialise(PArgList & args)
     allMediaFormats[i].PrintOptions(traceStream);
   traceStream << PTrace::End;
 #endif
-  return PTrue;
+  return true;
 }
 
 #if OPAL_H323
@@ -913,12 +986,12 @@ PBoolean MyManager::InitialiseH323EP(PArgList & args, PBoolean secure, H323EndPo
   }
 
   if (args.HasOption('b')) {
-    unsigned initialBandwidth = args.GetOptionString('b').AsUnsigned()*100;
+    OpalBandwidth initialBandwidth = args.GetOptionString('b').AsUnsigned();
     if (initialBandwidth == 0) {
       cerr << "Illegal bandwidth specified." << endl;
-      return PFalse;
+      return false;
     }
-    h323EP->SetInitialBandwidth(initialBandwidth);
+    h323EP->SetInitialBandwidth(OpalBandwidth::RxTx, initialBandwidth);
   }
 
   h323EP->SetGkAccessTokenOID(args.GetOptionString("gk-token"));
@@ -936,7 +1009,7 @@ PBoolean MyManager::InitialiseH323EP(PArgList & args, PBoolean secure, H323EndPo
   if (!h323EP->StartListeners(listeners)) {
     cerr <<  "Could not open any " << prefix << " listener from "
          << setfill(',') << listeners << endl;
-    return PFalse;
+    return false;
   }
   cout << prefix << " listeners: " << setfill(',') << h323EP->GetListeners() << setfill(' ') << endl;
 
@@ -983,10 +1056,10 @@ PBoolean MyManager::InitialiseH323EP(PArgList & args, PBoolean secure, H323EndPo
       }
       cerr << '.' << endl;
       if (args.HasOption("require-gatekeeper")) 
-        return PFalse;
+        return false;
     }
   }
-  return PTrue;
+  return true;
 }
 
 #endif  //OPAL_H323
@@ -1098,6 +1171,7 @@ void MyManager::Main(PArgList & args)
         cout << help ;
         break;
 
+#if OPAL_HAS_MIXER
       case 'z':
         if (currentCallToken.IsEmpty())
          cout << "Cannot stop or start record whilst no call in progress.\n";
@@ -1110,6 +1184,7 @@ void MyManager::Main(PArgList & args)
           cout << "Recording stopped.\n";
         }
         break;
+#endif
         
       case 'y' :
         if ( pcssEP != NULL &&
@@ -1125,6 +1200,13 @@ void MyManager::Main(PArgList & args)
           cout << "Could not reject connection " << pcssEP->incomingConnectionToken << endl;
         break;
 
+      case 'b' :
+        if ( pcssEP != NULL &&
+            !pcssEP->incomingConnectionToken &&
+            !pcssEP->RejectIncomingConnection(pcssEP->incomingConnectionToken, OpalConnection::EndedByLocalBusy))
+          cout << "Could not reject connection " << pcssEP->incomingConnectionToken << endl;
+        break;
+
 #if OPAL_PTLIB_CONFIG_FILE
       case 'l' :
         ListSpeedDials();
@@ -1242,12 +1324,8 @@ void MyManager::TransferCall(const PString & dest)
     return;
   }
 
-  for (PSafePtr<OpalConnection> connection = call->GetConnection(0); connection != NULL; ++connection) {
-    if (!PIsDescendant(&(*connection), OpalPCSSConnection) && !PIsDescendant(&(*connection), OpalLineConnection)) {
-      connection->TransferConnection(dest.IsEmpty() ? heldCallToken : dest);
-      break;
-    }
-  }
+  if (!call->Transfer(dest))
+    cout << "Call transfer failed!\n";
 }
 
 
@@ -1435,7 +1513,7 @@ PBoolean MyManager::OnOpenMediaStream(OpalConnection & connection,
                                   OpalMediaStream & stream)
 {
   if (!OpalManager::OnOpenMediaStream(connection, stream))
-    return PFalse;
+    return false;
 
   cout << "Started ";
 
@@ -1450,7 +1528,7 @@ PBoolean MyManager::OnOpenMediaStream(OpalConnection & connection,
 
   cout << endl;
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1480,10 +1558,10 @@ PBoolean MyPCSSEndPoint::OnShowIncoming(const OpalPCSSConnection & connection)
     PTime now;
     cout << "\nCall on " << now.AsString("w h:mma")
          << " from " << connection.GetRemotePartyName()
-         << ", answer (Y/N)? " << flush;
+         << ", answer (Yes/No/Busy)? " << flush;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1492,7 +1570,7 @@ PBoolean MyPCSSEndPoint::OnShowOutgoing(const OpalPCSSConnection & connection)
   PTime now;
   cout << connection.GetRemotePartyName() << " is ringing on "
        << now.AsString("w h:mma") << " ..." << endl;
-  return PTrue;
+  return true;
 }
 
 
@@ -1501,17 +1579,17 @@ PBoolean MyPCSSEndPoint::SetSoundDevice(PArgList & args,
                                     PSoundChannel::Directions dir)
 {
   if (!args.HasOption(optionName))
-    return PTrue;
+    return true;
 
   PString dev = args.GetOptionString(optionName);
 
   if (dir == PSoundChannel::Player) {
     if (SetSoundChannelPlayDevice(dev))
-      return PTrue;
+      return true;
   }
   else {
     if (SetSoundChannelRecordDevice(dev))
-      return PTrue;
+      return true;
   }
 
   cerr << "Device for " << optionName << " (\"" << dev << "\") must be one of:\n";
@@ -1520,7 +1598,7 @@ PBoolean MyPCSSEndPoint::SetSoundDevice(PArgList & args,
   for (PINDEX i = 0; i < names.GetSize(); i++)
     cerr << "  \"" << names[i] << "\"\n";
 
-  return PFalse;
+  return false;
 }
 
 // End of File ///////////////////////////////////////////////////////////////
