@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21857 $
+ * $Revision: 28481 $
  * $Author: rjongbloed $
- * $Date: 2008-12-21 22:03:10 -0600 (Sun, 21 Dec 2008) $
+ * $Date: 2012-10-05 03:13:17 -0500 (Fri, 05 Oct 2012) $
  */
 
 #ifndef _SimpleOpal_MAIN_H
@@ -31,8 +31,8 @@
 
 #include <ptclib/ipacl.h>
 #include <opal/manager.h>
-#include <opal/pcss.h>
-#include <opal/ivr.h>
+#include <ep/pcss.h>
+#include <ep/ivr.h>
 
 
 #ifndef OPAL_PTLIB_AUDIO
@@ -45,6 +45,8 @@ class SIPEndPoint;
 class H323EndPoint;
 class H323SEndPoint;
 class IAX2EndPoint;
+class OpalCapiEndPoint;
+
 
 class MyPCSSEndPoint : public OpalPCSSEndPoint
 {
@@ -108,6 +110,9 @@ class MyManager : public OpalManager
 #if OPAL_IAX2
     IAX2EndPoint     * iax2EP;
 #endif
+#if OPAL_CAPI
+    OpalCapiEndPoint * capiEP;
+#endif
 #if OPAL_IVR
     OpalIVREndPoint  * ivrEP;
 #endif
