@@ -21,66 +21,47 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23648 $
+ * $Revision: 26395 $
  * $Author: rjongbloed $
- * $Date: 2009-10-13 03:32:32 -0500 (Tue, 13 Oct 2009) $
+ * $Date: 2011-09-02 05:26:45 -0500 (Fri, 02 Sep 2011) $
  */
 
 #ifndef _SipIM_MAIN_H
 #define _SipIM_MAIN_H
 
-#include <opal/manager.h>
-#include <sip/sipep.h>
+#include <opal/console_mgr.h>
+
 
 #if !OPAL_HAS_IM
 #error Cannot compile IM sample program without IM!
 #endif
 
-class MyManager : public OpalManager
+
+class MyManager : public OpalManagerCLI
 {
-    PCLASSINFO(MyManager, OpalManager)
-
+    PCLASSINFO(MyManager, OpalManagerCLI);
   public:
-    virtual void OnClearedCall(OpalCall & call); // Callback override
-    void OnApplyStringOptions(OpalConnection & conn, OpalConnection::StringOptions & options);
+    MyManager() : m_imEP(NULL) { }
 
-    PSyncPoint m_connected;
-    PSyncPoint m_completed;
-    PString m_callToken;
-    OpalMediaFormat m_imFormat;
-};
+    virtual PString GetArgumentSpec() const;
+    virtual void Usage(ostream & strm, const PArgList & args);
+    virtual bool Initialise(PArgList & args, bool verbose);
 
-class MyPCSSEndPoint : public OpalPCSSEndPoint
-{
-  PCLASSINFO(MyPCSSEndPoint, OpalPCSSEndPoint);
+  protected:
+    OpalIMEndPoint * m_imEP;
 
-  public:
-    MyPCSSEndPoint(MyManager & manager)
-      : OpalPCSSEndPoint(manager)
-    { }
+    bool GetToFromURL(const PArgList & args, PURL & to, PURL & from);
 
-    virtual PBoolean OnShowIncoming(const OpalPCSSConnection & connection);
-    virtual PBoolean OnShowOutgoing(const OpalPCSSConnection & connection);
-};
+    virtual void OnConversation(const OpalIMContext::ConversationInfo & info);
+    virtual void OnMessageReceived(const OpalIM & message);
+    virtual void OnMessageDisposition(const OpalIMContext::DispositionInfo & info);
+    virtual void OnCompositionIndication(const OpalIMContext::CompositionInfo & info);
 
-class SipIM : public PProcess
-{
-    PCLASSINFO(SipIM, PProcess)
+    PDECLARE_NOTIFIER(PCLI::Arguments, MyManager, CmdSend);
+    PDECLARE_NOTIFIER(PCLI::Arguments, MyManager, CmdComp);
 
-  public:
-    SipIM();
-    ~SipIM();
-
-    enum Mode {
-      Use_MSRP,
-      Use_SIPIM,
-      Use_T140
-    };
-
-    virtual void Main();
-
-  private:
-    MyManager * m_manager;
+    PDECLARE_NOTIFIER(PCLI::Arguments, MyManager, CmdSet);
+    PStringToString m_variables;
 };
 
 
