@@ -24,9 +24,9 @@
  *
  * Contributor(s): Post Increment
  *
- * $Revision: 23063 $
- * $Author: dsandras $
- * $Date: 2009-07-04 11:08:33 +0000 (Sat, 04 Jul 2009) $
+ * $Revision: 27415 $
+ * $Author: rjongbloed $
+ * $Date: 2012-04-02 22:20:52 -0500 (Mon, 02 Apr 2012) $
  */
 
 #include <ptlib.h>
@@ -44,9 +44,6 @@ using namespace std;
 
 #include <opal/mediafmt.h>
 
-namespace PWLibStupidLinkerHacks {
-  int rateControlKickerVal = 0;
-};
 
 //
 // 20 bytes for nominal TCP header
@@ -55,12 +52,13 @@ namespace PWLibStupidLinkerHacks {
 #define UDP_OVERHEAD  (20 + 8)
 
 //
-// size of history used for calcluating average packet size
+// size of history used for calculating average packet size
 //
 #define PACKET_HISTORY_SIZE   5
 
 #define new PNEW
 
+
 /////////////////////////////////////////////////////////////////////////////
 //
 //
@@ -119,7 +117,7 @@ unsigned OpalBitRateCalculator::GetTrialBitRate(PINDEX size)
   unsigned trialBitRate = 0;
 
   if (m_history.size() > 0) 
-    trialBitRate = (unsigned)((size + m_historySize) * 8 * 1000 / (m_quanta + (now - m_history.front().m_timeStamp)));
+    trialBitRate = (unsigned)((size + (PInt64)m_historySize) * 8 * 1000 / (m_quanta + (now - m_history.front().m_timeStamp)));
 
   return trialBitRate;
 }
@@ -130,7 +128,7 @@ unsigned OpalBitRateCalculator::GetBitRate()
   Flush(now);
 
   if (m_history.size() > 0) 
-    m_bitRate = (unsigned)((m_historySize * 8 * 1000) / (m_quanta + (now - m_history.front().m_timeStamp)));
+    m_bitRate = (unsigned)(((PInt64)m_historySize * 8 * 1000) / (m_quanta + (now - m_history.front().m_timeStamp)));
 
   return m_bitRate;
 }
@@ -328,6 +326,9 @@ class OpalStandardVideoRateController : public OpalVideoRateController
 };
 
 
+PFACTORY_CREATE(PFactory<OpalVideoRateController>, OpalStandardVideoRateController, "Standard", false);
+
+
 OpalStandardVideoRateController::OpalStandardVideoRateController()
 {
 }
@@ -416,8 +417,7 @@ bool OpalStandardVideoRateController::CheckFrameRate(bool PTRACE_PARAM(reporting
   return false;
 }
 
-bool OpalStandardVideoRateController::CheckBitRate(bool PTRACE_PARAM(reporting),
-                                                   unsigned PTRACE_PARAM(currentBitRate))
+bool OpalStandardVideoRateController::CheckBitRate(bool PTRACE_PARAM(reporting), unsigned PTRACE_PARAM(currentBitRate))
 {
   if (m_bitRateCalc.GetHistoryCount() == 0)
     return false;
@@ -451,6 +451,5 @@ bool OpalStandardVideoRateController::Pop(RTP_DataFrameList & outputPackets, boo
 }
 
 
-static PFactory<OpalVideoRateController>::Worker<OpalStandardVideoRateController> opalStandardVideoRateController("Standard");
 
 #endif // OPAL_VIDEO
