@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26320 $
- * $Author: rjongbloed $
- * $Date: 2011-08-17 06:58:09 -0500 (Wed, 17 Aug 2011) $
+ * $Revision: 28932 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 06:34:20 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -94,6 +94,7 @@ const OpalVideoFormat & GetOpalYUV420P()
 OpalVideoTranscoder::OpalVideoTranscoder(const OpalMediaFormat & inputMediaFormat,
                                          const OpalMediaFormat & outputMediaFormat)
   : OpalTranscoder(inputMediaFormat, outputMediaFormat)
+  , m_errorConcealment(false)
   , inDataSize(10*1024)
   , outDataSize(10*1024)
   , forceIFrame(false)
@@ -120,14 +121,17 @@ bool OpalVideoTranscoder::UpdateMediaFormats(const OpalMediaFormat & input, cons
   PWaitAndSignal mutex(updateMutex);
 
   if (!OpalTranscoder::UpdateMediaFormats(input, output))
-    return PFalse;
+    return false;
 
   SetFrameBytes(inputMediaFormat,  OpalVideoFormat::MaxRxFrameWidthOption(), OpalVideoFormat::MaxRxFrameHeightOption(), inDataSize);
   SetFrameBytes(outputMediaFormat, OpalVideoFormat::FrameWidthOption(),      OpalVideoFormat::FrameHeightOption(),      outDataSize);
 
-  outputMediaFormat.SetOptionInteger(OpalMediaFormat::MaxTxPacketSizeOption(), maxOutputSize);
+  if (outputMediaFormat.GetOptionInteger(OpalMediaFormat::MaxTxPacketSizeOption()) > (int)maxOutputSize) {
+    PTRACE(4, "Media\tReducing \"" << OpalMediaFormat::MaxTxPacketSizeOption() << "\" to " << maxOutputSize);
+    outputMediaFormat.SetOptionInteger(OpalMediaFormat::MaxTxPacketSizeOption(), maxOutputSize);
+  }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -145,10 +149,10 @@ PINDEX OpalVideoTranscoder::GetOptimalDataFrameSize(PBoolean input) const
 
 PBoolean OpalVideoTranscoder::ExecuteCommand(const OpalMediaCommand & command)
 {
-  if (PIsDescendant(&command, OpalVideoUpdatePicture)) {
-    PTRACE_IF(3, !forceIFrame, "Media\tI-frame forced in video stream");
+  if (outputMediaFormat != OpalYUV420P && PIsDescendant(&command, OpalVideoUpdatePicture)) {
+    PTRACE_IF(3, !forceIFrame, "Media\tI-Frame forced in video stream");
     forceIFrame = true; // Reset when I-Frame is sent
-    return PTrue;
+    return true;
   }
 
   return OpalTranscoder::ExecuteCommand(command);
@@ -158,7 +162,7 @@ PBoolean OpalVideoTranscoder::ExecuteCommand(const OpalMediaCommand & command)
 PBoolean OpalVideoTranscoder::Convert(const RTP_DataFrame & /*input*/,
                                   RTP_DataFrame & /*output*/)
 {
-  return PFalse;
+  return false;
 }
 
 
