@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21862 $
- * $Author: csoutheren $
- * $Date: 2008-12-22 21:24:53 -0600 (Mon, 22 Dec 2008) $
+ * $Revision: 28483 $
+ * $Author: rjongbloed $
+ * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -32,10 +32,13 @@
 #pragma implementation "h323h224.h"
 #endif
 
-#if defined(OPAL_HAS_H224) && defined(OPAL_H323)
-
 #include <h224/h323h224.h>
 
+
+#if OPAL_HAS_H224
+
+#if OPAL_H323
+
 #include <h323/h323ep.h>
 #include <h323/h323con.h>
 #include <h323/channels.h>
@@ -101,7 +104,7 @@ PBoolean H323_H224_AnnexQCapability::OnSendingPDU(H245_DataApplicationCapability
   PASN_ObjectId & objectId = capabilityIdentifier;
   objectId.SetValue("0.0.8.224.1.0");
 	
-  return PTrue;
+  return true;
 }
 
 PBoolean H323_H224_AnnexQCapability::OnSendingPDU(H245_DataMode & pdu) const
@@ -116,12 +119,12 @@ PBoolean H323_H224_AnnexQCapability::OnSendingPDU(H245_DataMode & pdu) const
   PASN_ObjectId & objectId = capabilityIdentifier;
   objectId.SetValue("0.0.8.224.1.0");
 	
-  return PTrue;
+  return true;
 }
 
 PBoolean H323_H224_AnnexQCapability::OnReceivedPDU(const H245_DataApplicationCapability & /*pdu*/)
 {
-  return PTrue;
+  return true;
 }
 
 //////////////////////////////////////////////////////////////////////
@@ -180,7 +183,7 @@ PBoolean H323_H224_HDLCTunnelingCapability::OnSendingPDU(H245_DataApplicationCap
   H245_DataProtocolCapability & dataProtocolCapability = pdu.m_application;
   dataProtocolCapability.SetTag(H245_DataProtocolCapability::e_hdlcFrameTunnelling);
 	
-  return PTrue;
+  return true;
 }
 
 PBoolean H323_H224_HDLCTunnelingCapability::OnSendingPDU(H245_DataMode & pdu) const
@@ -191,13 +194,15 @@ PBoolean H323_H224_HDLCTunnelingCapability::OnSendingPDU(H245_DataMode & pdu) co
   H245_DataProtocolCapability & dataProtocolCapability = pdu.m_application;
   dataProtocolCapability.SetTag(H245_DataProtocolCapability::e_hdlcFrameTunnelling);
 	
-  return PTrue;
+  return true;
 }
 
 PBoolean H323_H224_HDLCTunnelingCapability::OnReceivedPDU(const H245_DataApplicationCapability & /*pdu*/)
 {
-  return PTrue;
+  return true;
 }
 
-#endif // defined(OPAL_HAS_H224) && defined(OPAL_H323)
 
+#endif // OPAL_H323
+
+#endif // OPAL_HAS_H224
