@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25210 $
+ * $Revision: 29960 $
  * $Author: rjongbloed $
- * $Date: 2011-02-28 08:24:23 -0600 (Mon, 28 Feb 2011) $
+ * $Date: 2013-06-13 23:26:53 -0500 (Thu, 13 Jun 2013) $
  */
 
 #include <ptlib.h>
@@ -58,7 +58,7 @@
 #define new PNEW
 
 
-#define	MAX_PAYLOAD_TYPE_MISMATCHES 8
+#define MAX_PAYLOAD_TYPE_MISMATCHES 8
 #define RTP_TRACE_DISPLAY_RATE 16000 // 2 seconds
 
 
@@ -140,20 +140,19 @@ H323ChannelNumber & H323ChannelNumber::operator++(int)
 /////////////////////////////////////////////////////////////////////////////
 
 H323Channel::H323Channel(H323Connection & conn, const H323Capability & cap)
-  : endpoint(conn.GetEndPoint()),
-    connection(conn)
+  : endpoint(conn.GetEndPoint())
+  , connection(conn)
+  , capability((H323Capability *)cap.Clone())
+  , opened(false)
+  , m_bandwidthUsed(0)
 {
-  capability = (H323Capability *)cap.Clone();
-  bandwidthUsed = 0;
-  opened = PFalse;
-  paused = PTrue;
+  PTRACE_CONTEXT_ID_FROM(conn);
+  PTRACE_CONTEXT_ID_TO(capability);
 }
 
 
 H323Channel::~H323Channel()
 {
-  connection.SetBandwidthUsed(bandwidthUsed, 0);
-
   delete capability;
 }
 
@@ -177,24 +176,28 @@ bool H323Channel::SetSessionID(unsigned /*sessionID*/)
 
 
 PBoolean H323Channel::GetMediaTransportAddress(OpalTransportAddress & /*data*/,
-                                           OpalTransportAddress & /*control*/) const
+                                               OpalTransportAddress & /*control*/) const
 {
-  return PFalse;
+  return false;
 }
 
 
 void H323Channel::Close()
 {
-  if (opened && m_terminating++ == 0)
+  if (m_terminating++ == 0)
     InternalClose();
 }
 
 
 void H323Channel::InternalClose()
 {
-  // Signal to the connection that this channel is on the way out
-  connection.OnClosedLogicalChannel(*this);
+  if (opened) {
+    // Signal to the connection that this channel is on the way out
+    connection.OnClosedLogicalChannel(*this);
 
+    connection.SetBandwidthUsed(GetDirection() == IsReceiver ? OpalBandwidth::Rx : OpalBandwidth::Tx, m_bandwidthUsed, 0);
+  }
+  
   PTRACE(4, "LogChan\tCleaned up " << number);
 }
 
@@ -205,16 +208,21 @@ OpalMediaStreamPtr H323Channel::GetMediaStream() const
 }
 
 
+void H323Channel::SetMediaStream(OpalMediaStreamPtr)
+{
+}
+
+
 PBoolean H323Channel::OnReceivedPDU(const H245_OpenLogicalChannel & /*pdu*/,
                                 unsigned & /*errorCode*/)
 {
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H323Channel::OnReceivedAckPDU(const H245_OpenLogicalChannelAck & /*pdu*/)
 {
-  return PTrue;
+  return true;
 }
 
 
@@ -224,9 +232,13 @@ void H323Channel::OnSendOpenAck(const H245_OpenLogicalChannel & /*pdu*/,
 }
 
 
-void H323Channel::OnFlowControl(long PTRACE_PARAM(bitRateRestriction))
+void H323Channel::OnFlowControl(long bitRateRestriction)
 {
   PTRACE(3, "LogChan\tOnFlowControl: " << bitRateRestriction);
+
+  OpalMediaStreamPtr stream = GetMediaStream();
+  if (stream != NULL)
+    stream->ExecuteCommand(OpalMediaFlowControl(bitRateRestriction*100));
 }
 
 
@@ -264,19 +276,6 @@ void H323Channel::OnMiscellaneousCommand(const H245_MiscellaneousCommand_type &
 }
 
 
-void H323Channel::OnMediaCommand(OpalMediaCommand & command)
-{
-#if OPAL_VIDEO
-  if (PIsDescendant(&command, OpalVideoUpdatePicture)) {
-    H323ControlPDU pdu;
-    pdu.BuildMiscellaneousCommand(GetNumber(), H245_MiscellaneousCommand_type::e_videoFastUpdatePicture);
-    connection.WriteControlPDU(pdu);
-    return;
-  }
-#endif
-}
-
-
 void H323Channel::OnMiscellaneousIndication(const H245_MiscellaneousIndication_type & PTRACE_PARAM(type))
 {
   PTRACE(3, "LogChan\tOnMiscellaneousIndication: chan=" << number
@@ -295,37 +294,41 @@ void H323Channel::OnJitterIndication(DWORD PTRACE_PARAM(jitter),
 }
 
 
-PBoolean H323Channel::SetBandwidthUsed(unsigned bandwidth)
+PBoolean H323Channel::SetBandwidthUsed(OpalBandwidth bandwidth)
 {
-  PTRACE(3, "LogChan\tBandwidth requested/used = "
-         << bandwidth/10 << '.' << bandwidth%10 << '/'
-         << bandwidthUsed/10 << '.' << bandwidthUsed%10
-         << " kb/s");
-  if (!connection.SetBandwidthUsed(bandwidthUsed, bandwidth)) {
-    bandwidthUsed = 0;
-    return PFalse;
+  if (!connection.SetBandwidthUsed(GetDirection() == IsReceiver ? OpalBandwidth::Rx : OpalBandwidth::Tx, m_bandwidthUsed, bandwidth)) {
+    m_bandwidthUsed = 0;
+    PTRACE(2, "LogChan\tFailed bandwidth request=" << bandwidth << ", used=" << m_bandwidthUsed);
+    return false;
   }
 
-  bandwidthUsed = bandwidth;
-  return PTrue;
+  PTRACE(3, "LogChan\tBandwidth requested=" << bandwidth << ", used=" << m_bandwidthUsed);
+  m_bandwidthUsed = bandwidth;
+  return true;
+}
+
+
+bool H323Channel::PreOpen()
+{
+  return Open();
 }
 
 
 PBoolean H323Channel::Open()
 {
   if (opened)
-    return PTrue;
+    return true;
 
   // Give the connection (or endpoint) a chance to do something with
   // the opening of the codec.
   if (!connection.OnStartLogicalChannel(*this)) {
     PTRACE(1, "LogChan\t" << (GetDirection() == IsReceiver ? "Receive" : "Transmit")
            << " open failed (OnStartLogicalChannel fail)");
-    return PFalse;
+    return false;
   }
 
-  opened = PTrue;
-  return PTrue;
+  opened = true;
+  return true;
 }
 
 
@@ -334,8 +337,9 @@ PBoolean H323Channel::Open()
 H323UnidirectionalChannel::H323UnidirectionalChannel(H323Connection & conn,
                                                      const H323Capability & cap,
                                                      Directions direction)
-  : H323Channel(conn, cap),
-    receiver(direction == IsReceiver)
+  : H323Channel(conn, cap)
+  , receiver(direction == IsReceiver)
+  , m_mediaFormat(capability->GetMediaFormat())
 {
 }
 
@@ -352,49 +356,52 @@ H323Channel::Directions H323UnidirectionalChannel::GetDirection() const
 
 PBoolean H323UnidirectionalChannel::SetInitialBandwidth()
 {
-  return SetBandwidthUsed(capability->GetMediaFormat().GetBandwidth()/100);
+  OpalBandwidth bandwidth;
+  if (GetDirection() == IsTransmitter)
+    bandwidth = capability->GetMediaFormat().GetOptionInteger(OpalMediaFormat::TargetBitRateOption());
+  if (bandwidth == 0)
+    bandwidth = capability->GetMediaFormat().GetOptionInteger(OpalMediaFormat::MaxBitRateOption());
+  return SetBandwidthUsed(bandwidth);
+}
+
+
+bool H323UnidirectionalChannel::PreOpen()
+{
+  if (m_mediaStream != NULL)
+    return true;
+
+  m_mediaStream = connection.CreateMediaStream(m_mediaFormat, GetSessionID(), receiver);
+
+  OpalCall & call = connection.GetCall();
+  OpalMediaType mediaType = m_mediaFormat.GetMediaType();
+
+  if (GetDirection() == IsReceiver) {
+    if (!call.OpenSourceMediaStreams(connection, mediaType, GetSessionID(), m_mediaFormat)) {
+      PTRACE(1, "LogChan\tReceive OpenSourceMediaStreams failed");
+      return false;
+    }
+  }
+  else {
+    PSafePtr<OpalConnection> otherConnection = call.GetOtherPartyConnection(connection);
+    if (otherConnection == NULL) {
+      PTRACE(1, "LogChan\tTransmit failed, no other connection");
+      return false;
+    }
+    if (!call.OpenSourceMediaStreams(*otherConnection, mediaType, GetSessionID(), m_mediaFormat)) {
+      PTRACE(1, "LogChan\tTransmit OpenSourceMediaStreams failed");
+      return false;
+    }
+  }
+
+  m_mediaFormat = m_mediaStream->GetMediaFormat();
+  capability->UpdateMediaFormat(m_mediaFormat);
+  return true;
 }
 
 
 PBoolean H323UnidirectionalChannel::Open()
 {
-  if (opened)
-    return true;
-
-  if (PAssertNULL(mediaStream) == NULL)
-    return false;
-
-  OpalCall & call = connection.GetCall();
-
-  bool ok;
-  if (GetDirection() == IsReceiver)
-    ok = call.OpenSourceMediaStreams(connection, capability->GetMediaFormat().GetMediaType(), GetSessionID(), mediaStream->GetMediaFormat());
-  else {
-    PSafePtr<OpalConnection> otherConnection = call.GetOtherPartyConnection(connection);
-    ok = otherConnection != NULL && call.OpenSourceMediaStreams(*otherConnection, capability->GetMediaFormat().GetMediaType(), GetSessionID());
-  }
-
-  if (ok) {
-    capability->UpdateMediaFormat(mediaStream->GetMediaFormat());
-    return H323Channel::Open();
-  }
-
-  PTRACE(1, "LogChan\t" << (GetDirection() == IsReceiver ? "Receive" : "Transmit")
-         << " open failed (OpalMediaStream::Open fail)");
-  return false;
-}
-
-
-PBoolean H323UnidirectionalChannel::Start()
-{
-  if (!Open())
-    return PFalse;
-
-  if (!mediaStream->Start())
-    return PFalse;
-
-  paused = PFalse;
-  return PTrue;
+  return opened || (PreOpen() && H323Channel::Open());
 }
 
 
@@ -403,10 +410,10 @@ void H323UnidirectionalChannel::InternalClose()
   PTRACE(4, "H323RTP\tCleaning up media stream on " << number);
 
   // If we have source media stream close it
-  if (mediaStream != NULL) {
-    connection.CloseMediaStream(*mediaStream);
-    connection.RemoveMediaStream(*mediaStream);
-    mediaStream.SetNULL();
+  if (m_mediaStream != NULL) {
+    if (!m_mediaStream->Close())
+      connection.RemoveMediaStream(*m_mediaStream);
+    m_mediaStream.SetNULL();
   }
 
   H323Channel::InternalClose();
@@ -415,7 +422,14 @@ void H323UnidirectionalChannel::InternalClose()
 
 OpalMediaStreamPtr H323UnidirectionalChannel::GetMediaStream() const
 {
-  return mediaStream;
+  return m_mediaStream;
+}
+
+
+void H323UnidirectionalChannel::SetMediaStream(OpalMediaStreamPtr mediaStream)
+{
+  m_mediaStream = mediaStream;
+  m_mediaFormat = m_mediaStream->GetMediaFormat();
 }
 
 
@@ -434,12 +448,6 @@ H323Channel::Directions H323BidirectionalChannel::GetDirection() const
 }
 
 
-PBoolean H323BidirectionalChannel::Start()
-{
-  return PTrue;
-}
-
-
 /////////////////////////////////////////////////////////////////////////////
 
 H323_RealTimeChannel::H323_RealTimeChannel(H323Connection & connection,
@@ -447,7 +455,6 @@ H323_RealTimeChannel::H323_RealTimeChannel(H323Connection & connection,
                                            Directions direction)
   : H323UnidirectionalChannel(connection, capability, direction)
 {
-  rtpPayloadType = capability.GetMediaFormat().GetPayloadType();
 }
 
 
@@ -465,6 +472,8 @@ PBoolean H323_RealTimeChannel::OnSendingPDU(H245_OpenLogicalChannel & open) cons
                 H245_OpenLogicalChannel_reverseLogicalChannelParameters_multiplexParameters
                     ::e_h2250LogicalChannelParameters);
 
+    // NOTE Two callbacks?  This may be supposed to fit somewhere else
+    connection.OnSendH245_OpenLogicalChannel(open, false);
     return OnSendingPDU(open.m_reverseLogicalChannelParameters.m_multiplexParameters);
   }
   else {
@@ -473,6 +482,10 @@ PBoolean H323_RealTimeChannel::OnSendingPDU(H245_OpenLogicalChannel & open) cons
                 H245_OpenLogicalChannel_forwardLogicalChannelParameters_multiplexParameters
                     ::e_h2250LogicalChannelParameters);
 
+    if (OnSendingAltPDU(open.m_genericInformation))
+        open.IncludeOptionalField(H245_OpenLogicalChannel::e_genericInformation);
+
+    connection.OnSendH245_OpenLogicalChannel(open, true);
     return OnSendingPDU(open.m_forwardLogicalChannelParameters.m_multiplexParameters);
   }
 }
@@ -489,6 +502,11 @@ PBoolean H323_RealTimeChannel::OnSendingPDU(H245_H2250LogicalChannelParameters &
 }
 
 
+void H323_RealTimeChannel::OnSendOpenAck(H245_H2250LogicalChannelAckParameters & /*param*/) const
+{
+}
+
+
 void H323_RealTimeChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
                                          H245_OpenLogicalChannelAck & ack) const
 {
@@ -499,21 +517,21 @@ void H323_RealTimeChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
 
   // select H225 choice
   ack.m_forwardMultiplexAckParameters.SetTag(
-	  H245_OpenLogicalChannelAck_forwardMultiplexAckParameters::e_h2250LogicalChannelAckParameters);
+      H245_OpenLogicalChannelAck_forwardMultiplexAckParameters::e_h2250LogicalChannelAckParameters);
 
-  OnSendOpenAck(ack.m_forwardMultiplexAckParameters);
-}
+  // get H225 parms
+  H245_H2250LogicalChannelAckParameters & param = ack.m_forwardMultiplexAckParameters;
 
+  // set session ID
+  param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_sessionID);
+  const H245_H2250LogicalChannelParameters & openparam =
+                          open.m_forwardLogicalChannelParameters.m_multiplexParameters;
+  unsigned sessionID = openparam.m_sessionID;
+  param.m_sessionID = sessionID;
 
-void H323_RealTimeChannel::OnSendOpenAck(H245_H2250LogicalChannelAckParameters & param) const
-{
-  // We are master, we use OUR session ID
-  if (connection.IsH245Master()) {
-    param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_sessionID);
-    param.m_sessionID = GetSessionID();
-  }
+  OnSendOpenAck(param);
 
-  PTRACE(3, "H323RTP\tSending open logical channel ACK: sessionID=" << GetSessionID());
+  PTRACE(3, "H323RTP\tSending open logical channel ACK: sessionID=" << sessionID);
 }
 
 
@@ -521,7 +539,7 @@ PBoolean H323_RealTimeChannel::OnReceivedPDU(const H245_OpenLogicalChannel & ope
                                          unsigned & errorCode)
 {
   if (receiver)
-    number = H323ChannelNumber(open.m_forwardLogicalChannelNumber, PTrue);
+    number = H323ChannelNumber(open.m_forwardLogicalChannelNumber, true);
 
   PTRACE(3, "H323RTP\tOnReceivedPDU for channel: " << number);
 
@@ -532,7 +550,7 @@ PBoolean H323_RealTimeChannel::OnReceivedPDU(const H245_OpenLogicalChannel & ope
   if (!capability->OnReceivedPDU(dataType, receiver)) {
     PTRACE(1, "H323RTP\tData type not supported");
     errorCode = H245_OpenLogicalChannelReject_cause::e_dataTypeNotSupported;
-    return PFalse;
+    return false;
   }
 
   if (reverse) {
@@ -548,7 +566,7 @@ PBoolean H323_RealTimeChannel::OnReceivedPDU(const H245_OpenLogicalChannel & ope
 
   PTRACE(1, "H323RTP\tOnly H.225.0 multiplex supported");
   errorCode = H245_OpenLogicalChannelReject_cause::e_unsuitableReverseParameters;
-  return PFalse;
+  return false;
 }
 
 
@@ -582,15 +600,18 @@ PBoolean H323_RealTimeChannel::OnReceivedAckPDU(const H245_OpenLogicalChannelAck
 
   if (!ack.HasOptionalField(H245_OpenLogicalChannelAck::e_forwardMultiplexAckParameters)) {
     PTRACE(1, "H323RTP\tNo forwardMultiplexAckParameters");
-    return PFalse;
+    return false;
   }
 
   if (ack.m_forwardMultiplexAckParameters.GetTag() !=
             H245_OpenLogicalChannelAck_forwardMultiplexAckParameters::e_h2250LogicalChannelAckParameters) {
     PTRACE(1, "H323RTP\tOnly H.225.0 multiplex supported");
-    return PFalse;
+    return false;
   }
 
+  if (ack.HasOptionalField(H245_OpenLogicalChannel::e_genericInformation))
+    OnReceivedAckAltPDU(ack.m_genericInformation);
+
   return OnReceivedAckPDU(ack.m_forwardMultiplexAckParameters);
 }
 
@@ -612,359 +633,21 @@ PBoolean H323_RealTimeChannel::OnReceivedAckPDU(const H245_H2250LogicalChannelAc
     return false;
   }
 
+  if (param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_dynamicRTPPayloadType)) {
+    m_mediaFormat.SetPayloadType((RTP_DataFrame::PayloadTypes)param.m_dynamicRTPPayloadType.GetValue());
+    if (m_mediaStream != NULL)
+      m_mediaStream->UpdateMediaFormat(m_mediaFormat);
+  }
+
   return SetSessionID(sessionID);
 }
 
 
-PBoolean H323_RealTimeChannel::SetDynamicRTPPayloadType(int newType)
+RTP_DataFrame::PayloadTypes H323_RealTimeChannel::GetDynamicRTPPayloadType() const
 {
-  PTRACE(4, "H323RTP\tAttempting to set dynamic RTP payload type: " << newType);
-
-  // This is "no change"
-  if (newType == -1)
-    return PTrue;
-
-  // Check for illegal type
-  if (newType < RTP_DataFrame::DynamicBase || newType >= RTP_DataFrame::IllegalPayloadType)
-    return PFalse;
-
-  // Check for overwriting "known" type
-  if (rtpPayloadType < RTP_DataFrame::DynamicBase)
-    return PFalse;
-
-  rtpPayloadType = (RTP_DataFrame::PayloadTypes)newType;
-
-  OpalMediaStream * mediaStream = GetMediaStream();
-  if (mediaStream != NULL) {
-    OpalMediaFormat adjustedMediaFormat = mediaStream->GetMediaFormat();
-    adjustedMediaFormat.SetPayloadType(rtpPayloadType);
-    mediaStream->UpdateMediaFormat(adjustedMediaFormat);
-  }
-
-  PTRACE(3, "H323RTP\tSet dynamic payload type to " << rtpPayloadType);
-  return PTrue;
-}
-
-
-/////////////////////////////////////////////////////////////////////////////
-
-H323_RTPChannel::H323_RTPChannel(H323Connection & conn,
-                                 const H323Capability & cap,
-                                 Directions direction,
-                                 RTP_Session & r)
-  : H323_RealTimeChannel(conn, cap, direction),
-    rtpSession(r),
-    rtpCallbacks(*(H323_RTP_Session *)r.GetUserData())
-{
-  // If we are the receiver of RTP data then we create a source media stream
-  mediaStream = conn.CreateMediaStream(capability->GetMediaFormat(), GetSessionID(), receiver);
-  PTRACE(3, "H323RTP\t" << (receiver ? "Receiver" : "Transmitter")
-         << " created using session " << GetSessionID());
-}
-
-
-H323_RTPChannel::~H323_RTPChannel()
-{
-}
-
-
-unsigned H323_RTPChannel::GetSessionID() const
-{
-  return rtpSession.GetSessionID();
-}
-
-
-bool H323_RTPChannel::SetSessionID(unsigned sessionID)
-{
-  unsigned oldSessionID = GetSessionID();
-  if (sessionID == oldSessionID)
-    return true;
-
-  return connection.ChangeSessionID(oldSessionID, sessionID);
-}
-
-
-PBoolean H323_RTPChannel::OnSendingPDU(H245_H2250LogicalChannelParameters & param) const
-{
-  return rtpCallbacks.OnSendingPDU(*this, param) && H323_RealTimeChannel::OnSendingPDU(param);
-}
-
-
-void H323_RTPChannel::OnSendOpenAck(H245_H2250LogicalChannelAckParameters & param) const
-{
-  rtpCallbacks.OnSendingAckPDU(*this, param);
-  H323_RealTimeChannel::OnSendOpenAck(param);
-}
-
-
-PBoolean H323_RTPChannel::OnReceivedPDU(const H245_H2250LogicalChannelParameters & param,
-                                    unsigned & errorCode)
-{
-  return rtpCallbacks.OnReceivedPDU(*this, param, errorCode) &&
-         H323_RealTimeChannel::OnReceivedPDU(param, errorCode);
-}
-
-
-PBoolean H323_RTPChannel::OnReceivedAckPDU(const H245_H2250LogicalChannelAckParameters & param)
-{
-  return rtpCallbacks.OnReceivedAckPDU(*this, param) && H323_RealTimeChannel::OnReceivedAckPDU(param);
-}
-
-
-/////////////////////////////////////////////////////////////////////////////
-
-H323_ExternalRTPChannel::H323_ExternalRTPChannel(H323Connection & connection,
-                                                 const H323Capability & capability,
-                                                 Directions direction,
-                                                 unsigned id)
-  : H323_RealTimeChannel(connection, capability, direction)
-{
-  Construct(connection, id);
-}
-
-
-H323_ExternalRTPChannel::H323_ExternalRTPChannel(H323Connection & connection,
-                                                 const H323Capability & capability,
-                                                 Directions direction,
-                                                 unsigned id,
-                                                 const H323TransportAddress & data,
-                                                 const H323TransportAddress & control)
-  : H323_RealTimeChannel(connection, capability, direction),
-    externalMediaAddress(data),
-    externalMediaControlAddress(control)
-{
-  Construct(connection, id);
-}
-
-
-H323_ExternalRTPChannel::H323_ExternalRTPChannel(H323Connection & connection,
-                                                 const H323Capability & capability,
-                                                 Directions direction,
-                                                 unsigned id,
-                                                 const PIPSocket::Address & ip,
-                                                 WORD dataPort)
-  : H323_RealTimeChannel(connection, capability, direction),
-    externalMediaAddress(ip, dataPort),
-    externalMediaControlAddress(ip, (WORD)(dataPort+1))
-{
-  Construct(connection, id);
-}
-
-void H323_ExternalRTPChannel::Construct(H323Connection & conn, unsigned id)
-{
-  mediaStream = new OpalNullMediaStream(conn, capability->GetMediaFormat(), id, receiver);
-  sessionID = id;
-
-  PTRACE(3, "H323RTP\tExternal " << (receiver ? "receiver" : "transmitter")
-         << " created using session " << GetSessionID());
-}
-
-
-unsigned H323_ExternalRTPChannel::GetSessionID() const
-{
-  return sessionID;
-}
-
-
-PBoolean H323_ExternalRTPChannel::GetMediaTransportAddress(OpalTransportAddress & data,
-                                                       OpalTransportAddress & control) const
-{
-  data = remoteMediaAddress;
-  control = remoteMediaControlAddress;
-
-  if (data.IsEmpty() && control.IsEmpty())
-    return PFalse;
-
-  PIPSocket::Address ip;
-  WORD port;
-  if (data.IsEmpty()) {
-    if (control.GetIpAndPort(ip, port))
-      data = OpalTransportAddress(ip, (WORD)(port-1));
-  }
-  else if (control.IsEmpty()) {
-    if (data.GetIpAndPort(ip, port))
-      control = OpalTransportAddress(ip, (WORD)(port+1));
-  }
-
-  return PTrue;
-}
-
-
-PBoolean H323_ExternalRTPChannel::Start()
-{
-  PSafePtr<OpalRTPConnection> otherParty = connection.GetOtherPartyConnectionAs<OpalRTPConnection>();
-  if (otherParty == NULL)
-    return PFalse;
-
-  OpalRTPConnection::MediaInformation info;
-  if (!otherParty->GetMediaInformation(sessionID, info))
-    return PFalse;
-
-  externalMediaAddress = info.data;
-  externalMediaControlAddress = info.control;
-  return Open();
-}
-
-
-void H323_ExternalRTPChannel::Receive()
-{
-  // Do nothing
-}
-
-
-void H323_ExternalRTPChannel::Transmit()
-{
-  // Do nothing
-}
-
-
-PBoolean H323_ExternalRTPChannel::OnSendingPDU(H245_H2250LogicalChannelParameters & param) const
-{
-  param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaGuaranteedDelivery);
-  param.m_mediaGuaranteedDelivery = PFalse;
-
-  param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_silenceSuppression);
-  param.m_silenceSuppression = PFalse;
-
-  // unicast must have mediaControlChannel
-  param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaControlChannel);
-  externalMediaControlAddress.SetPDU(param.m_mediaControlChannel);
-
-  if (receiver) {
-    // set mediaChannel
-    param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel);
-    externalMediaAddress.SetPDU(param.m_mediaChannel);
-  }
-
-  // Set dynamic payload type, if is one
-  RTP_DataFrame::PayloadTypes rtpPayloadType = GetDynamicRTPPayloadType();
-  if (rtpPayloadType >= RTP_DataFrame::DynamicBase && rtpPayloadType < RTP_DataFrame::IllegalPayloadType) {
-    param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_dynamicRTPPayloadType);
-    param.m_dynamicRTPPayloadType = (int)rtpPayloadType;
-  }
-
-  // Set the media packetization field if have an option to describe it.
-  param.m_mediaPacketization.SetTag(H245_H2250LogicalChannelParameters_mediaPacketization::e_rtpPayloadType);
-  if (H323SetRTPPacketization(param.m_mediaPacketization, GetCapability().GetMediaFormat(), GetDynamicRTPPayloadType()))
-    param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaPacketization);
-
-  return H323_RealTimeChannel::OnSendingPDU(param);
-}
-
-
-void H323_ExternalRTPChannel::OnSendOpenAck(H245_H2250LogicalChannelAckParameters & param) const
-{
-  // set mediaControlChannel
-  param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaControlChannel);
-  externalMediaControlAddress.SetPDU(param.m_mediaControlChannel);
-
-  // set mediaChannel
-  param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel);
-  externalMediaAddress.SetPDU(param.m_mediaChannel);
-
-  H323_RealTimeChannel::OnSendOpenAck(param);
-}
-
-
-PBoolean H323_ExternalRTPChannel::OnReceivedPDU(const H245_H2250LogicalChannelParameters & param,
-                                          unsigned & errorCode)
-{
-  if (!H323_RealTimeChannel::OnReceivedPDU(param, errorCode))
-    return false;
-
-  if (!param.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaControlChannel)) {
-    PTRACE(1, "LogChan\tNo mediaControlChannel specified");
-    errorCode = H245_OpenLogicalChannelReject_cause::e_unspecified;
-    return PFalse;
-  }
-
-  remoteMediaControlAddress = param.m_mediaControlChannel;
-  if (remoteMediaControlAddress.IsEmpty())
-    return PFalse;
-
-  if (param.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaChannel)) {
-    remoteMediaAddress = param.m_mediaChannel;
-    if (remoteMediaAddress.IsEmpty())
-      return PFalse;
-  }
-  else {
-    PIPSocket::Address addr;
-    WORD port;
-    if (!remoteMediaControlAddress.GetIpAndPort(addr, port))
-      return PFalse;
-    remoteMediaAddress = OpalTransportAddress(addr, port-1);
-  }
-
-  unsigned id = GetSessionID();
-  if (!remoteMediaAddress.IsEmpty() && (connection.GetMediaTransportAddresses().GetAt(id) == NULL))
-    connection.GetMediaTransportAddresses().SetAt(id, new OpalTransportAddress(remoteMediaAddress));
-
-  return true;
-}
-
-
-PBoolean H323_ExternalRTPChannel::OnReceivedAckPDU(const H245_H2250LogicalChannelAckParameters & param)
-{
-  if (!H323_RealTimeChannel::OnReceivedAckPDU(param))
-    return false;
-
-  if (!param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaControlChannel)) {
-    PTRACE(1, "LogChan\tNo mediaControlChannel specified");
-    return PFalse;
-  }
-
-  remoteMediaControlAddress = param.m_mediaControlChannel;
-  if (remoteMediaControlAddress.IsEmpty())
-    return PFalse;
-
-  if (!param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel)) {
-    PTRACE(1, "LogChan\tNo mediaChannel specified");
-    return PFalse;
-  }
-
-  remoteMediaAddress = param.m_mediaChannel;
-  if (remoteMediaAddress.IsEmpty())
-    return PFalse;
-
-  unsigned id = param.m_sessionID;
-  if (!remoteMediaAddress.IsEmpty() && (connection.GetMediaTransportAddresses().GetAt(id) == NULL))
-    connection.GetMediaTransportAddresses().SetAt(id, new OpalTransportAddress(remoteMediaAddress));
-
-  return PTrue;
-}
-
-
-void H323_ExternalRTPChannel::SetExternalAddress(const H323TransportAddress & data,
-                                                 const H323TransportAddress & control)
-{
-  externalMediaAddress = data;
-  externalMediaControlAddress = control;
-
-  if (data.IsEmpty() || control.IsEmpty()) {
-    PIPSocket::Address ip;
-    WORD port;
-    if (data.GetIpAndPort(ip, port))
-      externalMediaControlAddress = H323TransportAddress(ip, (WORD)(port+1));
-    else if (control.GetIpAndPort(ip, port))
-      externalMediaAddress = H323TransportAddress(ip, (WORD)(port-1));
-  }
-}
-
-
-PBoolean H323_ExternalRTPChannel::GetRemoteAddress(PIPSocket::Address & ip,
-                                               WORD & dataPort) const
-{
-  if (!remoteMediaAddress)
-    return remoteMediaAddress.GetIpAndPort(ip, dataPort);
-
-  if (!remoteMediaControlAddress) {
-    if (remoteMediaControlAddress.GetIpAndPort(ip, dataPort)) {
-      dataPort--;
-      return PTrue;
-    }
-  }
-
-  return PFalse;
+  OpalMediaFormat mediaFormat = m_mediaStream != NULL ? m_mediaStream->GetMediaFormat()
+                                                      : capability->GetMediaFormat();
+  return mediaFormat.GetPayloadType();
 }
 
 
@@ -978,10 +661,10 @@ H323DataChannel::H323DataChannel(H323Connection & conn,
 {
   sessionID = id;
   listener = NULL;
-  autoDeleteListener = PTrue;
+  autoDeleteListener = true;
   transport = NULL;
-  autoDeleteTransport = PTrue;
-  separateReverseChannel = PFalse;
+  autoDeleteTransport = true;
+  separateReverseChannel = false;
 }
 
 
@@ -1024,9 +707,11 @@ PBoolean H323DataChannel::OnSendingPDU(H245_OpenLogicalChannel & open) const
   open.m_forwardLogicalChannelParameters.m_multiplexParameters.SetTag(
               H245_OpenLogicalChannel_forwardLogicalChannelParameters_multiplexParameters
                   ::e_h2250LogicalChannelParameters);
+  H245_H2250LogicalChannelParameters & fparam = open.m_forwardLogicalChannelParameters.m_multiplexParameters;
+  fparam.m_sessionID = GetSessionID();
 
   if (separateReverseChannel)
-    return PTrue;
+    return true;
 
   open.IncludeOptionalField(H245_OpenLogicalChannel::e_reverseLogicalChannelParameters);
   open.m_reverseLogicalChannelParameters.IncludeOptionalField(
@@ -1034,12 +719,14 @@ PBoolean H323DataChannel::OnSendingPDU(H245_OpenLogicalChannel & open) const
   open.m_reverseLogicalChannelParameters.m_multiplexParameters.SetTag(
               H245_OpenLogicalChannel_reverseLogicalChannelParameters_multiplexParameters
                   ::e_h2250LogicalChannelParameters);
+  H245_H2250LogicalChannelParameters & rparam = open.m_reverseLogicalChannelParameters.m_multiplexParameters;
+  rparam.m_sessionID = GetSessionID();
 
   return capability->OnSendingPDU(open.m_reverseLogicalChannelParameters.m_dataType);
 }
 
 
-void H323DataChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
+void H323DataChannel::OnSendOpenAck(const H245_OpenLogicalChannel & /*open*/,
                                     H245_OpenLogicalChannelAck & ack) const
 {
   if (listener == NULL && transport == NULL) {
@@ -1080,30 +767,30 @@ void H323DataChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
 PBoolean H323DataChannel::OnReceivedPDU(const H245_OpenLogicalChannel & open,
                                     unsigned & errorCode)
 {
-  number = H323ChannelNumber(open.m_forwardLogicalChannelNumber, PTrue);
+  number = H323ChannelNumber(open.m_forwardLogicalChannelNumber, true);
 
   PTRACE(3, "LogChan\tOnReceivedPDU for data channel: " << number);
 
   if (!CreateListener()) {
     PTRACE(1, "LogChan\tCould not create listener");
     errorCode = H245_OpenLogicalChannelReject_cause::e_unspecified;
-    return PFalse;
+    return false;
   }
 
   if (separateReverseChannel &&
       open.HasOptionalField(H245_OpenLogicalChannel::e_reverseLogicalChannelParameters)) {
     errorCode = H245_OpenLogicalChannelReject_cause::e_unsuitableReverseParameters;
     PTRACE(1, "LogChan\tOnReceivedPDU has unexpected reverse parameters");
-    return PFalse;
+    return false;
   }
 
   if (!capability->OnReceivedPDU(open.m_forwardLogicalChannelParameters.m_dataType, receiver)) {
     PTRACE(1, "H323RTP\tData type not supported");
     errorCode = H245_OpenLogicalChannelReject_cause::e_dataTypeNotSupported;
-    return PFalse;
+    return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1117,47 +804,47 @@ PBoolean H323DataChannel::OnReceivedAckPDU(const H245_OpenLogicalChannelAck & ac
       PTRACE(3, "LogChan\tseparateReverseChannels");
     if (!ack.HasOptionalField(H245_OpenLogicalChannelAck::e_forwardMultiplexAckParameters)) {
       PTRACE(1, "LogChan\tNo forwardMultiplexAckParameters");
-      return PFalse;
+      return false;
     }
 
     if (ack.m_forwardMultiplexAckParameters.GetTag() !=
               H245_OpenLogicalChannelAck_forwardMultiplexAckParameters::e_h2250LogicalChannelAckParameters) {
       PTRACE(1, "LogChan\tOnly H.225.0 multiplex supported");
-      return PFalse;
+      return false;
     }
 
     const H245_H2250LogicalChannelAckParameters & param = ack.m_forwardMultiplexAckParameters;
 
     if (!param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel)) {
       PTRACE(1, "LogChan\tNo media channel address provided");
-      return PFalse;
+      return false;
     }
 
     address = &param.m_mediaChannel;
 
     if (ack.HasOptionalField(H245_OpenLogicalChannelAck::e_reverseLogicalChannelParameters)) {
       PTRACE(3, "LogChan\treverseLogicalChannelParameters set");
-      reverseChannel = H323ChannelNumber(ack.m_reverseLogicalChannelParameters.m_reverseLogicalChannelNumber, PTrue);
+      reverseChannel = H323ChannelNumber(ack.m_reverseLogicalChannelParameters.m_reverseLogicalChannelNumber, true);
     }
   }
   else {
     if (!ack.HasOptionalField(H245_OpenLogicalChannelAck::e_reverseLogicalChannelParameters)) {
       PTRACE(1, "LogChan\tNo reverseLogicalChannelParameters");
-      return PFalse;
+      return false;
     }
 
     if (ack.m_reverseLogicalChannelParameters.m_multiplexParameters.GetTag() !=
               H245_OpenLogicalChannelAck_reverseLogicalChannelParameters_multiplexParameters
                               ::e_h2250LogicalChannelParameters) {
       PTRACE(1, "LogChan\tOnly H.225.0 multiplex supported");
-      return PFalse;
+      return false;
     }
 
     const H245_H2250LogicalChannelParameters & param = ack.m_reverseLogicalChannelParameters.m_multiplexParameters;
 
     if (!param.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaChannel)) {
       PTRACE(1, "LogChan\tNo media channel address provided");
-      return PFalse;
+      return false;
     }
 
     address = &param.m_mediaChannel;
@@ -1165,15 +852,15 @@ PBoolean H323DataChannel::OnReceivedAckPDU(const H245_OpenLogicalChannelAck & ac
 
   if (!CreateTransport()) {
     PTRACE(1, "LogChan\tCould not create transport");
-    return PFalse;
+    return false;
   }
 
   if (!transport->ConnectTo(H323TransportAddress(*address))) {
     PTRACE(1, "LogChan\tCould not connect to remote transport address: " << *address);
-    return PFalse;
+    return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1183,7 +870,7 @@ PBoolean H323DataChannel::CreateListener()
     listener = connection.GetControlChannel().GetLocalAddress().CreateListener(
                           connection.GetEndPoint(), OpalTransportAddress::HostOnly);
     if (listener == NULL)
-      return PFalse;
+      return false;
 
     PTRACE(3, "LogChan\tCreated listener for data channel: " << *listener);
   }
@@ -1198,7 +885,7 @@ PBoolean H323DataChannel::CreateTransport()
     transport = connection.GetControlChannel().GetLocalAddress().CreateTransport(
                           connection.GetEndPoint(), OpalTransportAddress::HostOnly);
     if (transport == NULL)
-      return PFalse;
+      return false;
 
     PTRACE(3, "LogChan\tCreated transport for data channel: " << *transport);
   }
