@@ -23,9 +23,9 @@
  *
  * Contributor(s): __________________________________
  *
- * $Revision: 24036 $
+ * $Revision: 28483 $
  * $Author: rjongbloed $
- * $Date: 2010-02-06 21:42:13 -0600 (Sat, 06 Feb 2010) $
+ * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -40,6 +40,7 @@
 #include <ptclib/random.h>
 #include <ptclib/cypher.h>
 
+#include <opal/endpoint.h>
 #include <h323/h235auth.h>
 #include <h323/h323pdu.h>
 
@@ -51,11 +52,12 @@
 
 H235Authenticator::H235Authenticator()
 {
-  enabled = PTrue;
+  enabled = true;
   sentRandomSequenceNumber = PRandom::Number()&INT_MAX;
   lastRandomSequenceNumber = 0;
   lastTimestamp = 0;
   timestampGracePeriod = 2*60*60+10; // 2 hours 10 seconds to allow for DST adjustments
+  usage = GKAdmission;
 }
 
 
@@ -82,7 +84,7 @@ PBoolean H235Authenticator::PrepareTokens(PASN_Array & clearTokens,
   PWaitAndSignal m(mutex);
 
   if (!IsActive())
-    return PFalse;
+    return false;
 
   H235_ClearToken * clearToken = CreateClearToken();
   if (clearToken != NULL) {
@@ -101,11 +103,14 @@ PBoolean H235Authenticator::PrepareTokens(PASN_Array & clearTokens,
       clearTokens.Append(clearToken);
   }
 
-  H225_CryptoH323Token * cryptoToken = CreateCryptoToken();
-  if (cryptoToken != NULL)
+  H225_CryptoH323Token * cryptoToken;
+  if ((cryptoToken = CreateCryptoToken(false)) != NULL)
     cryptoTokens.Append(cryptoToken);
 
-  return PTrue;
+  if ((cryptoToken = CreateCryptoToken(true)) != NULL)
+    cryptoTokens.Append(cryptoToken);
+
+  return true;
 }
 
 
@@ -115,7 +120,7 @@ H235_ClearToken * H235Authenticator::CreateClearToken()
 }
 
 
-H225_CryptoH323Token * H235Authenticator::CreateCryptoToken()
+H225_CryptoH323Token * H235Authenticator::CreateCryptoToken(bool)
 {
   return NULL;
 }
@@ -123,7 +128,7 @@ H225_CryptoH323Token * H235Authenticator::CreateCryptoToken()
 
 PBoolean H235Authenticator::Finalise(PBYTEArray & /*rawPDU*/)
 {
-  return PTrue;
+  return true;
 }
 
 
@@ -171,13 +176,13 @@ H235Authenticator::ValidationResult H235Authenticator::ValidateCryptoToken(
 
 PBoolean H235Authenticator::UseGkAndEpIdentifiers() const
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean H235Authenticator::IsSecuredPDU(unsigned, PBoolean) const
 {
-  return PTrue;
+  return true;
 }
 
 
@@ -197,7 +202,7 @@ PBoolean H235Authenticator::AddCapability(unsigned mechanism,
   if (!IsActive()) {
     PTRACE(2, "RAS\tAuthenticator " << *this
             << " not active during GRQ SetCapability negotiation");
-    return PFalse;
+    return false;
   }
 
   PINDEX i;
@@ -221,7 +226,7 @@ PBoolean H235Authenticator::AddCapability(unsigned mechanism,
     algorithmOIDs[size] = oid;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -240,7 +245,7 @@ void H235Authenticators::PreparePDU(H323TransactionPDU & pdu,
   cryptoTokens.RemoveAll();
 
   for (iterator iterAuth = begin(); iterAuth != end(); ++iterAuth) {
-    if (iterAuth->IsSecuredPDU(pdu.GetChoice().GetTag(), PFalse) &&
+    if (iterAuth->IsSecuredPDU(pdu.GetChoice().GetTag(), false) &&
         iterAuth->PrepareTokens(clearTokens, cryptoTokens)) {
       PTRACE(4, "H235RAS\tPrepared PDU with authenticator " << *iterAuth);
     }
@@ -263,10 +268,10 @@ H235Authenticator::ValidationResult
                                        unsigned cryptoOptionalField,
                                        const PBYTEArray & rawPDU)
 {
-  PBoolean noneActive = PTrue;
+  PBoolean noneActive = true;
   for (iterator iterAuth = begin(); iterAuth != end(); ++iterAuth) {
-    if (iterAuth->IsActive() && iterAuth->IsSecuredPDU(pdu.GetChoice().GetTag(), PTrue)) {
-      noneActive = PFalse;
+    if (iterAuth->IsActive() && iterAuth->IsSecuredPDU(pdu.GetChoice().GetTag(), true)) {
+      noneActive = false;
       break;
     }
   }
@@ -284,7 +289,7 @@ H235Authenticator::ValidationResult
   }
 
   for (iterator iterAuth = begin(); iterAuth != end(); ++iterAuth) {
-    if (iterAuth->IsSecuredPDU(pdu.GetChoice().GetTag(), PTrue)) {
+    if (iterAuth->IsSecuredPDU(pdu.GetChoice().GetTag(), true)) {
       H235Authenticator::ValidationResult result = iterAuth->ValidateTokens(clearTokens, cryptoTokens, rawPDU);
       switch (result) {
         case H235Authenticator::e_OK :
@@ -319,6 +324,7 @@ static const char OID_MD5[] = "1.2.840.113549.2.5";
 
 H235AuthSimpleMD5::H235AuthSimpleMD5()
 {
+  usage = AnyApplication;
 }
 
 
@@ -334,7 +340,7 @@ const char * H235AuthSimpleMD5::GetName() const
 }
 
 
-H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken()
+H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken(bool digits)
 {
   if (!IsActive())
     return NULL;
@@ -344,14 +350,33 @@ H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken()
     return NULL;
   }
 
+  if (digits && !OpalIsE164(localId, true))
+    return NULL;
+
   // Cisco compatible hash calculation
   H235_ClearToken clearToken;
 
   // fill the PwdCertToken to calculate the hash
   clearToken.m_tokenOID = "0.0";
 
+  // Create the H.225 crypto token
+  H225_CryptoH323Token * cryptoToken = new H225_CryptoH323Token;
+  cryptoToken->SetTag(H225_CryptoH323Token::e_cryptoEPPwdHash);
+  H225_CryptoH323Token_cryptoEPPwdHash & cryptoEPPwdHash = *cryptoToken;
+
+  // Set the alias
+  if (digits) {
+    cryptoEPPwdHash.m_alias.SetTag(H225_AliasAddress::e_dialedDigits);
+    (PASN_IA5String &)cryptoEPPwdHash.m_alias = localId;
+  }
+  else {
+    cryptoEPPwdHash.m_alias.SetTag(H225_AliasAddress::e_h323_ID);
+    (PASN_BMPString &)cryptoEPPwdHash.m_alias = localId;
+  }
+
+  // Use SetValueRaw to make sure trailing NULL is included
   clearToken.IncludeOptionalField(H235_ClearToken::e_generalID);
-  clearToken.m_generalID.SetValueRaw(localId.AsUCS2()); // Use SetValueRaw to make sure trailing NULL is included
+  clearToken.m_generalID.SetValueRaw(localId.AsUCS2());
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_password);
   clearToken.m_password.SetValueRaw(password.AsUCS2());
@@ -370,14 +395,7 @@ H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken()
   PMessageDigest5::Code digest;
   stomach.Complete(digest);
 
-  // Create the H.225 crypto token
-  H225_CryptoH323Token * cryptoToken = new H225_CryptoH323Token;
-  cryptoToken->SetTag(H225_CryptoH323Token::e_cryptoEPPwdHash);
-  H225_CryptoH323Token_cryptoEPPwdHash & cryptoEPPwdHash = *cryptoToken;
-
   // Set the token data that actually goes over the wire
-  H323SetAliasAddress(localId, cryptoEPPwdHash.m_alias);
-
   cryptoEPPwdHash.m_timeStamp = clearToken.m_timeStamp;
   cryptoEPPwdHash.m_token.m_algorithmOID = OID_MD5;
   cryptoEPPwdHash.m_token.m_hash.SetData(sizeof(digest)*8, (const BYTE *)&digest);
@@ -466,7 +484,7 @@ PBoolean H235AuthSimpleMD5::IsSecuredPDU(unsigned rasPDU, PBoolean received) con
       return received ? !remoteId.IsEmpty() : !localId.IsEmpty();
 
     default :
-      return PFalse;
+      return false;
   }
 }
 
@@ -479,6 +497,7 @@ static const char OID_CAT[] = "1.2.840.113548.10.1.2.1";
 
 H235AuthCAT::H235AuthCAT()
 {
+  usage = GKAdmission;
 }
 
 
@@ -615,7 +634,7 @@ PBoolean H235AuthCAT::IsCapability(const H235_AuthenticationMechanism & mechanis
 {
   if (mechanism.GetTag() != H235_AuthenticationMechanism::e_authenticationBES ||
          algorithmOID.AsString() != OID_CAT)
-    return PFalse;
+    return false;
 
   const H235_AuthenticationBES & bes = mechanism;
   return bes.GetTag() == H235_AuthenticationBES::e_radius;
@@ -627,11 +646,11 @@ PBoolean H235AuthCAT::SetCapability(H225_ArrayOf_AuthenticationMechanism & mecha
 {
   if (!AddCapability(H235_AuthenticationMechanism::e_authenticationBES, OID_CAT,
                      mechanisms, algorithmOIDs))
-    return PFalse;
+    return false;
 
   H235_AuthenticationBES & bes = mechanisms[mechanisms.GetSize()-1];
   bes.SetTag(H235_AuthenticationBES::e_radius);
-  return PTrue;
+  return true;
 }
 
 
@@ -643,7 +662,7 @@ PBoolean H235AuthCAT::IsSecuredPDU(unsigned rasPDU, PBoolean received) const
       return received ? !remoteId.IsEmpty() : !localId.IsEmpty();
 
     default :
-      return PFalse;
+      return false;
   }
 }
 
