@@ -26,9 +26,9 @@
  *
  * Contributor(s): Many thanks to Simon Horne.
  *
- * $Revision: 24442 $
+ * $Revision: 29743 $
  * $Author: rjongbloed $
- * $Date: 2010-06-02 03:17:31 -0500 (Wed, 02 Jun 2010) $
+ * $Date: 2013-05-19 22:43:02 -0500 (Sun, 19 May 2013) $
  */
 
 #include <ptlib.h>
@@ -36,6 +36,10 @@
 #include <opal/buildopts.h>
 #if OPAL_H323
 
+  #ifdef _MSC_VER
+    #pragma message("H.323 support enabled")
+  #endif
+
 #ifdef __GNUC__
 #pragma implementation "h323ep.h"
 #endif
@@ -46,6 +50,7 @@
 #include <opal/call.h>
 #include <h323/h323pdu.h>
 #include <h323/gkclient.h>
+#include <h323/h323rtp.h>
 #include <ptclib/url.h>
 #include <ptclib/enum.h>
 #include <ptclib/pils.h>
@@ -57,11 +62,12 @@
 /////////////////////////////////////////////////////////////////////////////
 
 H323EndPoint::H323EndPoint(OpalManager & manager)
-  : OpalRTPEndPoint(manager, "h323", CanTerminateCall)
+  : OpalRTPEndPoint(manager, "h323", CanTerminateCall|SupportsE164)
   , autoCallForward(true)
   , disableFastStart(false)
   , disableH245Tunneling(false)
   , disableH245inSetup(false)
+  , m_forceSymmetricTCS(false)
   , m_bH245Disabled(false)
   , canDisplayAmountString(false)
   , canEnforceDurationLimit(true)
@@ -107,14 +113,23 @@ H323EndPoint::H323EndPoint(OpalManager & manager)
   , disableH460(false)
 #endif
 {
-  defaultSignalPort = 1720; // Set port in OpalEndPoint class
-
   localAliasNames.AppendString(defaultLocalPartyName);
 
   secondaryConnectionsActive.DisallowDeleteObjects();
 
   manager.AttachEndPoint(this, "h323s");
 
+#if OPAL_H460
+  // We must set time to live to 30 to ensure pinhole open
+  registrationTimeToLive = PTimeInterval(0, 30);  
+  m_h46018enabled = true;
+#endif
+
+  SetCompatibility(H323Connection::e_NoMultipleTunnelledH245, "Cisco IOS");
+  SetCompatibility(H323Connection::e_BadMasterSlaveConflict,  "NetMeeting");
+  AddCompatibility(H323Connection::e_BadMasterSlaveConflict,  "HDX 9004");
+  SetCompatibility(H323Connection::e_NoUserInputCapability,   "AltiServ-ITG");
+
   PTRACE(4, "H323\tCreated endpoint.");
 }
 
@@ -139,11 +154,18 @@ PString H323EndPoint::GetDefaultTransport() const
 {
   return "tcp$"
 #if OPAL_PTLIB_SSL
-         ",tcps$:1300"
+         ",tls$:1300"
 #endif
     ;
 }
 
+
+WORD H323EndPoint::GetDefaultSignalPort() const
+{
+  return DefaultTcpSignalPort;
+}
+
+
 void H323EndPoint::SetEndpointTypeInfo(H225_EndpointType & info) const
 {
   info.IncludeOptionalField(H225_EndpointType::e_vendor);
@@ -230,9 +252,9 @@ bool H323EndPoint::SetGatewaySupportedProtocol(H225_ArrayOf_SupportedProtocols &
 }
 
 
-bool H323EndPoint::OnSetGatewayPrefixes(PStringList & prefixes) const
+bool H323EndPoint::OnSetGatewayPrefixes(PStringList & /*prefixes*/) const
 {
-	return false;
+  return false;
 }
 
 
@@ -312,7 +334,7 @@ PBoolean H323EndPoint::UseGatekeeper(const PString & address,
                                  const PString & localAddress)
 {
   if (gatekeeper != NULL) {
-    PBoolean same = PTrue;
+    PBoolean same = true;
 
     if (!address && address != "*")
       same = gatekeeper->GetTransport().GetRemoteAddress().IsEquivalent(address);
@@ -325,7 +347,7 @@ PBoolean H323EndPoint::UseGatekeeper(const PString & address,
 
     if (same) {
       PTRACE(3, "H323\tUsing existing gatekeeper " << *gatekeeper);
-      return PTrue;
+      return true;
     }
   }
 
@@ -352,11 +374,25 @@ PBoolean H323EndPoint::UseGatekeeper(const PString & address,
   }
 }
 
+static H323TransportAddress GetGatekeeperAddress(const PString & address)
+{
+  PIPSocketAddressAndPort addrPort(address, H225_RAS::DefaultRasUdpPort);
+
+#if OPAL_PTLIB_DNS_RESOLVER
+  if (!addrPort.IsValid()) {
+    PIPSocketAddressAndPortVector addresses;
+    if (PDNS::LookupSRV(address, "_h323rs._udp", addrPort.GetPort(), addresses) && !addresses.empty())
+      addrPort = addresses[0];
+  }
+#endif //OPAL_PTLIB_DNS_RESOLVER
+
+  return H323TransportAddress(addrPort.AsString(), H225_RAS::DefaultRasUdpPort, OpalTransportAddress::UdpPrefix());
+}
 
 PBoolean H323EndPoint::SetGatekeeper(const PString & address, H323Transport * transport)
 {
-  H323TransportAddress h323addr(address, H225_RAS::DefaultRasUdpPort, "udp");
-  return InternalCreateGatekeeper(transport) && gatekeeper->DiscoverByAddress(h323addr);
+  H323TransportAddress h323addr = GetGatekeeperAddress(address);
+  return InternalCreateGatekeeper(transport, h323addr) && gatekeeper->DiscoverByAddress(h323addr);
 }
 
 
@@ -364,29 +400,42 @@ PBoolean H323EndPoint::SetGatekeeperZone(const PString & address,
                                      const PString & identifier,
                                      H323Transport * transport)
 {
-  H323TransportAddress h323addr(address, H225_RAS::DefaultRasUdpPort, "udp");
-  return InternalCreateGatekeeper(transport) && gatekeeper->DiscoverByNameAndAddress(identifier, h323addr);
+  H323TransportAddress h323addr = GetGatekeeperAddress(address);
+  return InternalCreateGatekeeper(transport, h323addr) && gatekeeper->DiscoverByNameAndAddress(identifier, h323addr);
 }
 
 
 PBoolean H323EndPoint::LocateGatekeeper(const PString & identifier, H323Transport * transport)
 {
-  return InternalCreateGatekeeper(transport) && gatekeeper->DiscoverByName(identifier);
+  return InternalCreateGatekeeper(transport, H323TransportAddress()) && gatekeeper->DiscoverByName(identifier);
 }
 
 
 PBoolean H323EndPoint::DiscoverGatekeeper(H323Transport * transport)
 {
-  return InternalCreateGatekeeper(transport) && gatekeeper->DiscoverAny();
+  return InternalCreateGatekeeper(transport, H323TransportAddress()) && gatekeeper->DiscoverAny();
 }
 
 
-bool H323EndPoint::InternalCreateGatekeeper(H323Transport * transport)
+bool H323EndPoint::InternalCreateGatekeeper(H323Transport * transport, const H323TransportAddress & gkAddress)
 {
   RemoveGatekeeper(H225_UnregRequestReason::e_reregistrationRequired);
 
-  if (transport == NULL)
-    transport = new H323TransportUDP(*this);
+  if (transport == NULL) {
+    OpalTransportAddressArray interfaces = GetInterfaceAddresses();
+    for (PINDEX i = 0; i < interfaces.GetSize(); ++i) {
+      if (interfaces[i].IsCompatible(gkAddress)) {
+        PIPSocket::Address ip;
+        if (interfaces[i].GetIpAddress(ip))
+          transport = new H323TransportUDP(*this, ip);
+        break;
+      }
+    }
+    if (transport == NULL) {
+      PTRACE(2, "H323\tCannot find a compatible listener for \"" << gkAddress << '"');
+      return false;
+    }
+  }
 
   gatekeeper = CreateGatekeeper(transport);
   if (gatekeeper == NULL)
@@ -406,7 +455,7 @@ H323Gatekeeper * H323EndPoint::CreateGatekeeper(H323Transport * transport)
 PBoolean H323EndPoint::IsRegisteredWithGatekeeper() const
 {
   if (gatekeeper == NULL)
-    return PFalse;
+    return false;
 
   return gatekeeper->IsRegistered();
 }
@@ -414,7 +463,7 @@ PBoolean H323EndPoint::IsRegisteredWithGatekeeper() const
 
 PBoolean H323EndPoint::RemoveGatekeeper(int reason)
 {
-  PBoolean ok = PTrue;
+  PBoolean ok = true;
 
   if (gatekeeper == NULL)
     return ok;
@@ -444,21 +493,12 @@ void H323EndPoint::SetGatekeeperPassword(const PString & password, const PString
   }
 }
 
-void H323EndPoint::OnGatekeeperConfirm()
+
+void H323EndPoint::OnGatekeeperStatus(H323Gatekeeper::RegistrationFailReasons)
 {
 }
 
-void H323EndPoint::OnGatekeeperReject()
-{
-}
 
-void H323EndPoint::OnRegistrationConfirm()
-{
-}
-
-void H323EndPoint::OnRegistrationReject()
-{
-}
 
 H235Authenticators H323EndPoint::CreateAuthenticators()
 {
@@ -466,10 +506,12 @@ H235Authenticators H323EndPoint::CreateAuthenticators()
 
   PFactory<H235Authenticator>::KeyList_T keyList = PFactory<H235Authenticator>::GetKeyList();
   PFactory<H235Authenticator>::KeyList_T::const_iterator r;
-  for (r = keyList.begin(); r != keyList.end(); ++r)
-    authenticators.Append(PFactory<H235Authenticator>::CreateInstance(*r));
-
-  PTRACE(3, "H323\tAuthenticator list is size " << (int)authenticators.GetSize());
+  for (r = keyList.begin(); r != keyList.end(); ++r) {
+    H235Authenticator * Auth = PFactory<H235Authenticator>::CreateInstance(*r);
+    if ((Auth->GetApplication() == H235Authenticator::GKAdmission) ||
+                  (Auth->GetApplication() == H235Authenticator::AnyApplication)) 
+                               authenticators.Append(Auth);
+  }
 
   return authenticators;
 }
@@ -496,14 +538,17 @@ PSafePtr<OpalConnection> H323EndPoint::MakeConnection(OpalCall & call,
 }
 
 
-PBoolean H323EndPoint::NewIncomingConnection(OpalTransport * transport)
+void H323EndPoint::NewIncomingConnection(OpalListener &, const OpalTransportPtr & transport)
 {
+  if (transport == NULL)
+    return;
+
   PTRACE(3, "H225\tAwaiting first PDU");
   transport->SetReadTimeout(15000); // Await 15 seconds after connect for first byte
   H323SignalPDU pdu;
   if (!pdu.Read(*transport)) {
     PTRACE(1, "H225\tFailed to get initial Q.931 PDU, connection not started.");
-    return PTrue;
+    return;
   }
 
   unsigned callReference = pdu.GetQ931().GetCallReference();
@@ -518,51 +563,45 @@ PBoolean H323EndPoint::NewIncomingConnection(OpalTransport * transport)
   if (connection == NULL) {
     // Get new instance of a call, abort if none created
     OpalCall * call = manager.InternalCreateCall();
-    if (call != NULL)
+    if (call != NULL) {
       connection = CreateConnection(*call, token, NULL, *transport, PString::Empty(), PString::Empty(), &pdu);
-
-    if (!AddConnection(connection)) {
-      PTRACE(1, "H225\tEndpoint could not create connection, "
-                "sending release complete PDU: callRef=" << callReference);
-
-      H323SignalPDU releaseComplete;
-      Q931 &q931PDU = releaseComplete.GetQ931();
-      q931PDU.BuildReleaseComplete(callReference, PTrue);
-      releaseComplete.m_h323_uu_pdu.m_h323_message_body.SetTag(H225_H323_UU_PDU_h323_message_body::e_releaseComplete);
-
-      H225_ReleaseComplete_UUIE &release = releaseComplete.m_h323_uu_pdu.m_h323_message_body;
-      release.m_protocolIdentifier.SetValue(psprintf("0.0.8.2250.0.%u", H225_PROTOCOL_VERSION));
-
-      H225_Setup_UUIE &setup = pdu.m_h323_uu_pdu.m_h323_message_body;
-      if (setup.HasOptionalField(H225_Setup_UUIE::e_callIdentifier)) {
-        release.IncludeOptionalField(H225_Setup_UUIE::e_callIdentifier);
-        release.m_callIdentifier = setup.m_callIdentifier;
-      }
-
-      // Set the cause value
-      q931PDU.SetCause(Q931::TemporaryFailure);
-
-      // Send the PDU
-      releaseComplete.Write(*transport);
-
-      return PTrue;
+      PTRACE(3, "H323\tCreated new connection: " << token);
     }
   }
 
-  PTRACE(3, "H323\tCreated new connection: " << token);
-  connection->AttachSignalChannel(token, transport, PTrue);
+  // Make sure transport is attached before AddConnection()
+  if (connection != NULL)
+    connection->AttachSignalChannel(token, transport, true);
 
-  if (connection->HandleSignalPDU(pdu)) {
+  if (AddConnection(connection) != NULL && connection->HandleSignalPDU(pdu)) {
     // All subsequent PDU's should wait forever
     transport->SetReadTimeout(PMaxTimeInterval);
     connection->HandleSignallingChannel();
-  }
-  else {
-    connection->ClearCall(H323Connection::EndedByTransportFail);
-    PTRACE(1, "H225\tSignal channel stopped on first PDU.");
+    return;
   }
 
-  return PFalse;
+  PTRACE(1, "H225\tEndpoint could not create connection, "
+            "sending release complete PDU: callRef=" << callReference);
+
+  H323SignalPDU releaseComplete;
+  Q931 &q931PDU = releaseComplete.GetQ931();
+  q931PDU.BuildReleaseComplete(callReference, true);
+  releaseComplete.m_h323_uu_pdu.m_h323_message_body.SetTag(H225_H323_UU_PDU_h323_message_body::e_releaseComplete);
+
+  H225_ReleaseComplete_UUIE &release = releaseComplete.m_h323_uu_pdu.m_h323_message_body;
+  release.m_protocolIdentifier.SetValue(psprintf("0.0.8.2250.0.%u", H225_PROTOCOL_VERSION));
+
+  H225_Setup_UUIE &setup = pdu.m_h323_uu_pdu.m_h323_message_body;
+  if (setup.HasOptionalField(H225_Setup_UUIE::e_callIdentifier)) {
+    release.IncludeOptionalField(H225_Setup_UUIE::e_callIdentifier);
+    release.m_callIdentifier = setup.m_callIdentifier;
+  }
+
+  // Set the cause value
+  q931PDU.SetCause(Q931::TemporaryFailure);
+
+  // Send the PDU
+  releaseComplete.Write(*transport);
 }
 
 
@@ -589,7 +628,7 @@ PBoolean H323EndPoint::SetupTransfer(const PString & oldToken,
   PSafePtr<OpalConnection> otherConnection =
     GetConnectionWithLock(oldToken, PSafeReference);
   if (otherConnection == NULL) {
-    return PFalse;
+    return false;
   }
 
   OpalCall & call = otherConnection->GetCall();
@@ -634,8 +673,7 @@ H323Connection * H323EndPoint::InternalMakeCall(OpalCall & call,
   // that the gatekeeper is using.
   H323Transport * transport;
   if (gatekeeper != NULL)
-    transport = gatekeeper->GetTransport().GetLocalAddress().CreateTransport(
-                                          *this, OpalTransportAddress::Streamed);
+    transport = gatekeeper->GetTransport().GetLocalAddress().CreateTransport(*this, OpalTransportAddress::Streamed);
   else if (!stringOptions->Contains(OPAL_OPT_INTERFACE))
     transport = address.CreateTransport(*this, OpalTransportAddress::NoBinding);
   else {
@@ -648,14 +686,7 @@ H323Connection * H323EndPoint::InternalMakeCall(OpalCall & call,
     return NULL;
   }
 
-  inUseFlag.Wait();
-
-  PString newToken;
-  if (existingToken.IsEmpty()) {
-    do {
-      newToken = psprintf("localhost/%u", Q931::GenerateCallReference());
-    } while (connectionsActive.Contains(newToken));
-  }
+  PString newToken(PString::Printf, "localhost/%u", Q931::GenerateCallReference());
 
   H323Connection * connection = CreateConnection(call, newToken, userData, *transport, alias, address, NULL, options, stringOptions);
   if (!AddConnection(connection)) {
@@ -663,9 +694,7 @@ H323Connection * H323EndPoint::InternalMakeCall(OpalCall & call,
     return NULL;
   }
 
-  inUseFlag.Signal();
-
-  connection->AttachSignalChannel(newToken, transport, PFalse);
+  connection->AttachSignalChannel(newToken, transport, false);
 
 #if OPAL_H450
   if (!callIdentity) {
@@ -709,14 +738,6 @@ void H323EndPoint::ConsultationTransfer(const PString & primaryCallToken,
 }
 
 
-void H323EndPoint::HoldCall(const PString & token, PBoolean localHold)
-{
-  PSafePtr<H323Connection> connection = FindConnectionWithLock(token);
-  if (connection != NULL)
-    connection->HoldCall(localHold);
-}
-
-
 PBoolean H323EndPoint::IntrudeCall(const PString & remoteParty,
                                unsigned capabilityLevel,
                                void * userData)
@@ -735,19 +756,19 @@ PBoolean H323EndPoint::OnSendConnect(H323Connection & /*connection*/,
                                  H323SignalPDU & /*connectPDU*/
                                 )
 {
-  return PTrue;
+  return true;
 }
 
 PBoolean H323EndPoint::OnSendSignalSetup(H323Connection & /*connection*/,
                                      H323SignalPDU & /*setupPDU*/)
 {
-  return PTrue;
+  return true;
 }
 
 PBoolean H323EndPoint::OnSendCallProceeding(H323Connection & /*connection*/,
                                         H323SignalPDU & /*callProceedingPDU*/)
 {
-  return PTrue;
+  return true;
 }
 
 void H323EndPoint::OnReceivedInitiateReturnError()
@@ -764,16 +785,13 @@ PBoolean H323EndPoint::ParsePartyName(const PString & remoteParty,
   if (stringOptions != NULL)
     stringOptions->ExtractFromURL(url);
 
-#if OPAL_PTLIB_DNS
+#if OPAL_PTLIB_DNS_RESOLVER
 
   // if there is no gatekeeper, try altarnate address lookup methods
   if (gatekeeper == NULL) {
-    PString hostname = url.GetHostName();
-
-    // No host, so lets try ENUM on the username part
-    if (hostname.IsEmpty()) {
+    if (url.GetHostName().IsEmpty()) {
+      // No host, so lets try ENUM on the username part, and only has digits and +
       PString username = url.GetUserName();
-      // make sure the number has only digits and +
       if (OpalIsE164(username)) {
         PString newName;
         if (PDNS::ENUMLookup(username, "E2U+h323", newName)) {
@@ -783,35 +801,38 @@ PBoolean H323EndPoint::ParsePartyName(const PString & remoteParty,
       }
     }
 
+    /* URL default for H.323 is if no @ then is username, but that does
+       not make sense when no GK, so we switch to being a host only */
+    if (url.GetHostName().IsEmpty()) {
+      url.SetHostName(url.GetUserName());
+      url.SetUserName(PString::Empty());
+    }
+
     // If it is a valid IP address then can't be a domain so do not try SRV record lookup
-    if (!hostname.IsEmpty()) {
-      PIPSocket::Address ip = hostname;
-      if (!ip.IsValid()) {
-        PIPSocketAddressAndPortVector addresses;
-        if (PDNS::LookupSRV(hostname, "_h323._tcp", url.GetPort(), addresses) && !addresses.empty()) {
-          // Only use first entry
-          url.SetHostName(addresses[0].GetAddress().AsString());
-          url.SetPort(addresses[0].GetPort());
-        }
+    if (!PIPSocket::Address(url.GetHostName()).IsValid() && !url.GetPortSupplied()) {
+      PIPSocketAddressAndPortVector addresses;
+      if (PDNS::LookupSRV(url.GetHostName(), "_h323cs._tcp", url.GetPort(), addresses) && !addresses.empty()) {
+        // Only use first entry
+        url.SetHostName(addresses[0].GetAddress().AsString());
+        url.SetPort(addresses[0].GetPort());
       }
     }
   }
 
-#endif
-
-  alias = url.GetUserName();
+#endif // OPAL_PTLIB_DNS_RESOLVER
 
   address = url.GetHostName();
   if (!address && url.GetPort() != 0)
     address.sprintf(":%u", url.GetPort());
 
+  alias = url.GetUserName();
   if (alias.IsEmpty() && address.IsEmpty()) {
     PTRACE(1, "H323\tAttempt to use invalid URL \"" << remoteParty << '"');
-    return PFalse;
+    return false;
   }
 
-  PBoolean gatekeeperSpecified = PFalse;
-  PBoolean gatewaySpecified = PFalse;
+  PBoolean gatekeeperSpecified = false;
+  PBoolean gatewaySpecified = false;
 
   PCaselessString type = url.GetParamVars()("type");
 
@@ -823,26 +844,26 @@ PBoolean H323EndPoint::ParsePartyName(const PString & remoteParty,
       if (server.IsEmpty())
         server = GetDefaultILSServer();
       if (server.IsEmpty())
-        return PFalse;
+        return false;
 
       PILSSession ils;
       if (!ils.Open(server, url.GetPort())) {
         PTRACE(1, "H323\tCould not open ILS server at \"" << server
                << "\" - " << ils.GetErrorText());
-        return PFalse;
+        return false;
       }
 
       PILSSession::RTPerson person;
       if (!ils.SearchPerson(alias, person)) {
         PTRACE(1, "H323\tCould not find "
                << server << '/' << alias << ": " << ils.GetErrorText());
-        return PFalse;
+        return false;
       }
 
       if (!person.sipAddress.IsValid()) {
         PTRACE(1, "H323\tILS user " << server << '/' << alias
                << " does not have a valid IP address");
-        return PFalse;
+        return false;
       }
 
       // Get the IP address
@@ -857,23 +878,23 @@ PBoolean H323EndPoint::ParsePartyName(const PString & remoteParty,
       }
 
       alias = PString::Empty(); // No alias for ILS lookup, only host
-      return PTrue;
+      return true;
 #else
-      return PFalse;
+      return false;
 #endif
     }
 
     if (url.GetParamVars().Contains("gateway"))
-      gatewaySpecified = PTrue;
+      gatewaySpecified = true;
   }
   else if (url.GetScheme() == "h323") {
     if (type == "gw")
-      gatewaySpecified = PTrue;
+      gatewaySpecified = true;
     else if (type == "gk")
-      gatekeeperSpecified = PTrue;
+      gatekeeperSpecified = true;
     else if (!type) {
       PTRACE(1, "H323\tUnsupported host type \"" << type << "\" in h323 URL");
-      return PFalse;
+      return false;
     }
   }
 
@@ -881,12 +902,12 @@ PBoolean H323EndPoint::ParsePartyName(const PString & remoteParty,
   if (gatekeeperSpecified) {
     if (alias.IsEmpty()) {
       PTRACE(1, "H323\tAttempt to use explict gatekeeper without alias!");
-      return PFalse;
+      return false;
     }
 
     if (address.IsEmpty()) {
       PTRACE(1, "H323\tAttempt to use explict gatekeeper without address!");
-      return PFalse;
+      return false;
     }
 
     H323TransportAddress gkAddr = address;
@@ -924,12 +945,12 @@ PBoolean H323EndPoint::ParsePartyName(const PString & remoteParty,
       address = H323TransportAddress(alias, GetDefaultSignalPort(), transports[0]);
       alias = PString::Empty();
     }
-    return PTrue;
+    return true;
   }
 
   // We have a gatekeeper and user provided a host, all done
   if (!address)
-    return PTrue;
+    return true;
 
   // We have a gk and user did not explicitly supply a host, so lets
   // do a check to see if it is an IP address 
@@ -947,24 +968,23 @@ PBoolean H323EndPoint::ParsePartyName(const PString & remoteParty,
     }
   }
 
-  return PTrue;
+  return true;
 }
 
 
 PSafePtr<H323Connection> H323EndPoint::FindConnectionWithLock(const PString & token, PSafetyMode mode)
 {
-  PSafePtr<H323Connection> connnection = PSafePtrCast<OpalConnection, H323Connection>(GetConnectionWithLock(token, mode));
-  if (connnection != NULL)
-    return connnection;
+  PSafePtr<H323Connection> connection = PSafePtrCast<OpalConnection, H323Connection>(GetConnectionWithLock(token, mode));
+  if (connection != NULL)
+    return connection;
 
-  for (PSafePtr<OpalConnection> conn(connectionsActive, mode); conn != NULL; ++conn) {
-    connnection = PSafePtrCast<OpalConnection, H323Connection>(conn);
-    if(connnection != NULL) {      //cast success
-      if (connnection->GetCallIdentifier().AsString() == token)
-        return connnection;
-      if (connnection->GetConferenceIdentifier().AsString() == token)
-        return connnection;
-    }
+  for (PSafePtr<OpalConnection> conn(connectionsActive, PSafeReference); conn != NULL; ++conn) {
+    connection = PSafePtrCast<OpalConnection, H323Connection>(conn);
+    if (  connection != NULL &&
+          connection.SetSafetyMode(PSafeReadOnly) &&
+         (connection->GetCallIdentifier().AsString() == token ||
+          connection->GetConferenceIdentifier().AsString() == token))
+      return connection.SetSafetyMode(mode) ? connection : NULL;
   }
 
   return NULL;
@@ -982,13 +1002,13 @@ PBoolean H323EndPoint::OnIncomingCall(H323Connection & connection,
 PBoolean H323EndPoint::OnCallTransferInitiate(H323Connection & /*connection*/,
                                           const PString & /*remoteParty*/)
 {
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H323EndPoint::OnCallTransferIdentify(H323Connection & /*connection*/)
 {
-  return PTrue;
+  return true;
 }
 
 
@@ -1021,7 +1041,7 @@ PBoolean H323EndPoint::OnAlerting(H323Connection & connection,
 {
   PTRACE(3, "H225\tReceived alerting PDU.");
   ((OpalConnection&)connection).OnAlerting();
-  return PTrue;
+  return true;
 }
 
 PBoolean H323EndPoint::OnSendAlerting(H323Connection & PTRACE_PARAM(connection),
@@ -1031,20 +1051,20 @@ PBoolean H323EndPoint::OnSendAlerting(H323Connection & PTRACE_PARAM(connection),
                                   )
 {
   PTRACE(3, "H225\tOnSendAlerting conn = " << connection);
-  return PTrue;
+  return true;
 }
 
 PBoolean H323EndPoint::OnSentAlerting(H323Connection & PTRACE_PARAM(connection))
 {
   PTRACE(3, "H225\tOnSentAlerting conn = " << connection);
-  return PTrue;
+  return true;
 }
 
 PBoolean H323EndPoint::OnConnectionForwarded(H323Connection & /*connection*/,
                                          const PString & /*forwardParty*/,
                                          const H323SignalPDU & /*pdu*/)
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -1058,11 +1078,11 @@ PBoolean H323EndPoint::ForwardConnection(H323Connection & connection,
                        UINT_MAX,
                        forwardParty,
                        NULL) == NULL)
-    return PFalse;
+    return false;
 
   connection.SetCallEndReason(H323Connection::EndedByCallForwarded);
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1083,13 +1103,7 @@ PBoolean H323EndPoint::OnOutgoingCall(H323Connection & /*connection*/,
                                   const H323SignalPDU & /*connectPDU*/)
 {
   PTRACE(3, "H225\tReceived connect PDU.");
-  return PTrue;
-}
-
-
-void H323EndPoint::OnConnectionCleared(H323Connection & /*connection*/,
-                                       const PString & /*token*/)
-{
+  return true;
 }
 
 
@@ -1124,7 +1138,7 @@ PBoolean H323EndPoint::OnStartLogicalChannel(H323Connection & /*connection*/,
 #if PTRACING
   OnStartStopChannel("Start", channel);
 #endif
-  return PTrue;
+  return true;
 }
 
 
@@ -1137,13 +1151,6 @@ void H323EndPoint::OnClosedLogicalChannel(H323Connection & /*connection*/,
 }
 
 
-void H323EndPoint::OnRTPStatistics(const H323Connection & connection,
-                                   const RTP_Session & session) const
-{
-  manager.OnRTPStatistics(connection, session);
-}
-
-
 void H323EndPoint::OnGatekeeperNATDetect(PIPSocket::Address /* publicAddr*/,
 					 PString & /*gkIdentifier*/,
 					 H323TransportAddress & /*gkRouteAddress*/)
@@ -1171,19 +1178,14 @@ void H323EndPoint::OnServiceControlSession(unsigned type,
   session.OnChange(type, sessionId, *this, connection);
 }
 
-PBoolean H323EndPoint::OnConferenceInvite(const H323SignalPDU & /*setupPDU */)
-{
-  return PFalse;
-}
-
 PBoolean H323EndPoint::OnCallIndependentSupplementaryService(const H323SignalPDU & /* setupPDU */)
 {
-  return PFalse;
+  return false;
 }
 
 PBoolean H323EndPoint::OnNegotiateConferenceCapabilities(const H323SignalPDU & /* setupPDU */)
 {
-  return PFalse;
+  return false;
 }
 
 H323ServiceControlSession * H323EndPoint::CreateServiceControlSession(const H225_ServiceControlDescriptor & contents)
@@ -1222,26 +1224,25 @@ PBoolean H323EndPoint::AddAliasName(const PString & name)
 {
   PAssert(!name, "Must have non-empty string in AliasAddress!");
 
-  if (localAliasNames.GetValuesIndex(name) != P_MAX_INDEX)
-    return PFalse;
+  if (localAliasNames.find(name) != localAliasNames.end())
+    return false;
 
   localAliasNames.AppendString(name);
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H323EndPoint::RemoveAliasName(const PString & name)
 {
-  PINDEX pos = localAliasNames.GetValuesIndex(name);
-  if (pos == P_MAX_INDEX)
-    return PFalse;
+  PStringList::iterator pos = localAliasNames.find(name);
+  if (pos == localAliasNames.end())
+    return false;
 
-  PAssert(localAliasNames.GetSize() > 1, "Must have at least one AliasAddress!");
-  if (localAliasNames.GetSize() < 2)
-    return PFalse;
+  if (!PAssert(localAliasNames.GetSize() > 1, "Must have at least one AliasAddress!"))
+    return false;
 
-  localAliasNames.RemoveAt(pos);
-  return PTrue;
+  localAliasNames.erase(pos);
+  return true;
 }
 
 
@@ -1249,7 +1250,7 @@ bool H323EndPoint::AddAliasNamePattern(const PString & pattern)
 {
   PAssert(!pattern, "Must have non-empty string in AddressPattern !");
 
-  if (localAliasPatterns.GetValuesIndex(pattern) != P_MAX_INDEX)
+  if (localAliasPatterns.find(pattern) != localAliasPatterns.end())
     return false;
 
   localAliasPatterns.AppendString(pattern);
@@ -1262,10 +1263,10 @@ PBoolean H323EndPoint::IsTerminal() const
   switch (terminalType) {
     case e_TerminalOnly :
     case e_TerminalAndMC :
-      return PTrue;
+      return true;
 
     default :
-      return PFalse;
+      return false;
   }
 }
 
@@ -1278,10 +1279,10 @@ PBoolean H323EndPoint::IsGateway() const
     case e_GatewayAndMCWithDataMP :
     case e_GatewayAndMCWithAudioMP :
     case e_GatewayAndMCWithAVMP :
-      return PTrue;
+      return true;
 
     default :
-      return PFalse;
+      return false;
   }
 }
 
@@ -1293,10 +1294,10 @@ PBoolean H323EndPoint::IsGatekeeper() const
     case e_GatekeeperWithDataMP :
     case e_GatekeeperWithAudioMP :
     case e_GatekeeperWithAVMP :
-      return PTrue;
+      return true;
 
     default :
-      return PFalse;
+      return false;
   }
 }
 
@@ -1308,10 +1309,10 @@ PBoolean H323EndPoint::IsMCU() const
     case e_MCUWithDataMP :
     case e_MCUWithAudioMP :
     case e_MCUWithAVMP :
-      return PTrue;
+      return true;
 
     default :
-      return PFalse;
+      return false;
   }
 }
 
@@ -1332,6 +1333,36 @@ PBoolean H323EndPoint::OnSendFeatureSet(unsigned id, H225_FeatureSet & featureSe
 #endif
 }
 
+#if OPAL_H460
+H460_FeatureSet * H323EndPoint::GetGatekeeperFeatures()
+{
+	if (gatekeeper != NULL) {
+		return &gatekeeper->GetFeatures();
+	}
+
+	return NULL;
+}
+
+void H323EndPoint::H46018Enable(PBoolean enable) 
+{ 
+   m_h46018enabled = enable;
+   if (enable) {
+       // Must set reregistrations at between 15 and 45 sec
+       // otherwise the Pinhole in NAT will close
+       registrationTimeToLive = PTimeInterval(0, 30);  
+   } else {
+       // Set timer to whatever gk allocates...
+       registrationTimeToLive = PTimeInterval();       
+   }
+}
+
+
+PBoolean H323EndPoint::H46018IsEnabled() 
+{ 
+	return m_h46018enabled; 
+}
+#endif
+
 
 void H323EndPoint::OnReceiveFeatureSet(unsigned id, const H225_FeatureSet & featureSet)
 {
@@ -1350,12 +1381,73 @@ void H323EndPoint::LoadBaseFeatureSet()
 }
 
 
-bool H323EndPoint::OnFeatureInstance(int instType, const PString & identifer)
+bool H323EndPoint::OnFeatureInstance(int /*instType*/, const PString & /*identifer*/)
 {
   return true;
 }
 
 
+PString H323EndPoint::GetCompatibility(H323Connection::CompatibilityIssues issue) const
+{
+  CompatibilityEndpoints::const_iterator it = m_compatibility.find(issue);
+  return it != m_compatibility.end() ? it->second.GetPattern() : PString::Empty();
+}
+
+
+bool H323EndPoint::SetCompatibility(H323Connection::CompatibilityIssues issue, const PString & regex)
+{
+  if (regex.IsEmpty()) {
+    m_compatibility.erase(issue);
+    return true;
+  }
+
+  PRegularExpression re;
+  if (!re.Compile(regex, PRegularExpression::Extended|PRegularExpression::IgnoreCase))
+    return false;
+
+  m_compatibility[issue] = re;
+  return true;
+}
+
+
+bool H323EndPoint::AddCompatibility(H323Connection::CompatibilityIssues issue, const PString & regex)
+{
+  if (!PAssert(!regex, PInvalidParameter))
+    return false;
+
+  PString pattern;
+
+  CompatibilityEndpoints::iterator it = m_compatibility.find(issue);
+  if (it != m_compatibility.end())
+    pattern = it->second.GetPattern() + '|';
+
+  pattern += regex;
+
+  return SetCompatibility(issue, pattern);
+}
+
+
+bool H323EndPoint::HasCompatibilityIssue(H323Connection::CompatibilityIssues issue, const OpalProductInfo & productInfo) const
+{
+  bool found = false;
+  CompatibilityEndpoints::const_iterator it = m_compatibility.find(issue);
+  if (it != m_compatibility.end())
+    found = productInfo.AsString().FindRegEx(it->second) != P_MAX_INDEX;
+
+  PTRACE(4, "H.323\tChecking compatibility issue " << issue << ", "
+            "product=\"" << productInfo << "\", "
+            "re=\"" << (it != m_compatibility.end() ? it->second.GetPattern() : PString::Empty()) << "\", "
+            "found=" << found);
+  return found;
+}
+
+
+#else
+
+  #ifdef _MSC_VER
+    #pragma message("H.323 support DISABLED")
+  #endif
+
 #endif // OPAL_H323
 
 /////////////////////////////////////////////////////////////////////////////
