@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23567 $
+ * $Revision: 29729 $
  * $Author: rjongbloed $
- * $Date: 2009-10-01 01:20:36 -0500 (Thu, 01 Oct 2009) $
+ * $Date: 2013-05-16 21:20:35 -0500 (Thu, 16 May 2013) $
  */
 
 #include <ptlib.h>
@@ -55,11 +55,22 @@ H245Negotiator::H245Negotiator(H323EndPoint & end, H323Connection & conn)
   : endpoint(end),
     connection(conn)
 {
-  replyTimer.SetNotifier(PCREATE_NOTIFIER(HandleTimeout));
+  PTRACE_CONTEXT_ID_FROM(conn);
+  PTRACE_CONTEXT_ID_TO(replyTimer);
+  replyTimer.SetNotifier(PCREATE_NOTIFIER(HandleTimeoutUnlocked));
 }
 
 
-void H245Negotiator::HandleTimeout(PTimer &, INT)
+void H245Negotiator::HandleTimeoutUnlocked(PTimer &, P_INT_PTR)
+{
+  if (connection.LockReadWrite()) {
+    HandleTimeout();
+    connection.UnlockReadWrite();
+  }
+}
+
+
+void H245Negotiator::HandleTimeout()
 {
 }
 
@@ -78,15 +89,13 @@ H245NegMasterSlaveDetermination::H245NegMasterSlaveDetermination(H323EndPoint &
 
 PBoolean H245NegMasterSlaveDetermination::Start(PBoolean renegotiate)
 {
-  PWaitAndSignal wait(mutex);
-
   if (state != e_Idle) {
     PTRACE(3, "H245\tMasterSlaveDetermination already in progress");
-    return PTrue;
+    return true;
   }
 
   if (!renegotiate && IsDetermined())
-    return PTrue;
+    return true;
 
   retryCount = 1;
   return Restart();
@@ -110,8 +119,6 @@ PBoolean H245NegMasterSlaveDetermination::Restart()
 
 void H245NegMasterSlaveDetermination::Stop()
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tStopping MasterSlaveDetermination: state=" << state);
 
   if (state == e_Idle)
@@ -124,8 +131,6 @@ void H245NegMasterSlaveDetermination::Stop()
 
 PBoolean H245NegMasterSlaveDetermination::HandleIncoming(const H245_MasterSlaveDetermination & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived MasterSlaveDetermination: state=" << state);
 
   if (state == e_Incoming) {
@@ -182,12 +187,10 @@ PBoolean H245NegMasterSlaveDetermination::HandleIncoming(const H245_MasterSlaveD
 
 PBoolean H245NegMasterSlaveDetermination::HandleAck(const H245_MasterSlaveDeterminationAck & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived MasterSlaveDeterminationAck: state=" << state);
 
   if (state == e_Idle)
-    return PTrue;
+    return true;
 
   replyTimer = endpoint.GetMasterSlaveDeterminationTimeout();
 
@@ -204,7 +207,7 @@ PBoolean H245NegMasterSlaveDetermination::HandleAck(const H245_MasterSlaveDeterm
                   << (newStatus == e_DeterminedSlave ? "master" : "slave"));
     reply.BuildMasterSlaveDeterminationAck(newStatus == e_DeterminedMaster);
     if (!connection.WriteControlPDU(reply))
-      return PFalse;
+      return false;
   }
 
   replyTimer.Stop(false);
@@ -214,19 +217,17 @@ PBoolean H245NegMasterSlaveDetermination::HandleAck(const H245_MasterSlaveDeterm
     return connection.OnControlProtocolError(H323Connection::e_MasterSlaveDetermination,
                                              "Master/Slave mismatch");
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H245NegMasterSlaveDetermination::HandleReject(const H245_MasterSlaveDeterminationReject & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived MasterSlaveDeterminationReject: state=" << state);
 
   switch (state) {
     case e_Idle :
-      return PTrue;
+      return true;
 
     case e_Outgoing :
       if (pdu.m_cause.GetTag() == H245_MasterSlaveDeterminationReject_cause::e_identicalNumbers) {
@@ -249,12 +250,10 @@ PBoolean H245NegMasterSlaveDetermination::HandleReject(const H245_MasterSlaveDet
 
 PBoolean H245NegMasterSlaveDetermination::HandleRelease(const H245_MasterSlaveDeterminationRelease & /*pdu*/)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived MasterSlaveDeterminationRelease: state=" << state);
 
   if (state == e_Idle)
-    return PTrue;
+    return true;
 
   replyTimer.Stop(false);
   state = e_Idle;
@@ -264,10 +263,8 @@ PBoolean H245NegMasterSlaveDetermination::HandleRelease(const H245_MasterSlaveDe
 }
 
 
-void H245NegMasterSlaveDetermination::HandleTimeout(PTimer &, INT)
+void H245NegMasterSlaveDetermination::HandleTimeout()
 {
-  PWaitAndSignal wait(mutex);
-
   if (state == e_Idle)
     return;
 
@@ -315,22 +312,20 @@ H245NegTerminalCapabilitySet::H245NegTerminalCapabilitySet(H323EndPoint & end,
   inSequenceNumber = UINT_MAX;
   outSequenceNumber = 0;
   state = e_Idle;
-  receivedCapabilites = PFalse;
+  receivedCapabilites = false;
 }
 
 
 PBoolean H245NegTerminalCapabilitySet::Start(PBoolean renegotiate, PBoolean empty)
 {
-  PWaitAndSignal wait(mutex);
-
   if (state == e_InProgress) {
     PTRACE(2, "H245\tTerminalCapabilitySet already in progress: outSeq=" << outSequenceNumber);
-    return PTrue;
+    return true;
   }
 
   if (!renegotiate && state == e_Confirmed) {
     PTRACE(2, "H245\tTerminalCapabilitySet already sent.");
-    return PTrue;
+    return true;
   }
 
   // Begin the capability exchange procedure
@@ -348,8 +343,6 @@ PBoolean H245NegTerminalCapabilitySet::Start(PBoolean renegotiate, PBoolean empt
 
 void H245NegTerminalCapabilitySet::Stop(PBoolean dec)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tStopping TerminalCapabilitySet: state=" << state);
 
   if (state == e_Idle)
@@ -357,7 +350,7 @@ void H245NegTerminalCapabilitySet::Stop(PBoolean dec)
 
   replyTimer.Stop(false);
   state = e_Idle;
-  receivedCapabilites = PFalse;
+  receivedCapabilites = false;
 
   if (dec) {
     if (outSequenceNumber == 0)
@@ -370,23 +363,18 @@ void H245NegTerminalCapabilitySet::Stop(PBoolean dec)
 
 PBoolean H245NegTerminalCapabilitySet::HandleIncoming(const H245_TerminalCapabilitySet & pdu)
 {
-  mutex.Wait();
-
   PTRACE(3, "H245\tReceived TerminalCapabilitySet:"
                      " state=" << state <<
                      " pduSeq=" << pdu.m_sequenceNumber <<
                      " inSeq=" << inSequenceNumber);
 
   if (pdu.m_sequenceNumber == inSequenceNumber) {
-    mutex.Signal();
     PTRACE(2, "H245\tIgnoring TerminalCapabilitySet, already received sequence number");
-    return PTrue;  // Already had this one
+    return true;  // Already had this one
   }
 
   inSequenceNumber = pdu.m_sequenceNumber;
 
-  mutex.Signal();
-
   H323Capabilities remoteCapabilities(connection, pdu);
 
   const H245_MultiplexCapability * muxCap = NULL;
@@ -397,7 +385,7 @@ PBoolean H245NegTerminalCapabilitySet::HandleIncoming(const H245_TerminalCapabil
   if (connection.OnReceivedCapabilitySet(remoteCapabilities, muxCap,
                     reject.BuildTerminalCapabilitySetReject(inSequenceNumber,
                             H245_TerminalCapabilitySetReject_cause::e_unspecified))) {
-    receivedCapabilites = PTrue;
+    receivedCapabilites = true;
     H323ControlPDU ack;
     ack.BuildTerminalCapabilitySetAck(inSequenceNumber);
     return connection.WriteControlPDU(ack);
@@ -405,46 +393,42 @@ PBoolean H245NegTerminalCapabilitySet::HandleIncoming(const H245_TerminalCapabil
 
   connection.WriteControlPDU(reject);
   connection.ClearCall(H323Connection::EndedByCapabilityExchange);
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H245NegTerminalCapabilitySet::HandleAck(const H245_TerminalCapabilitySetAck & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived TerminalCapabilitySetAck:"
                      " state=" << state <<
                      " pduSeq=" << pdu.m_sequenceNumber <<
                      " outSeq=" << (unsigned)outSequenceNumber);
 
   if (state != e_InProgress)
-    return PTrue;
+    return true;
 
   if (pdu.m_sequenceNumber != outSequenceNumber)
-    return PTrue;
+    return true;
 
   replyTimer.Stop(false);
   state = e_Confirmed;
   PTRACE(3, "H245\tTerminalCapabilitySet Sent.");
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H245NegTerminalCapabilitySet::HandleReject(const H245_TerminalCapabilitySetReject & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived TerminalCapabilitySetReject:"
                      " state=" << state <<
                      " pduSeq=" << pdu.m_sequenceNumber <<
                      " outSeq=" << (unsigned)outSequenceNumber);
 
   if (state != e_InProgress)
-    return PTrue;
+    return true;
 
   if (pdu.m_sequenceNumber != outSequenceNumber)
-    return PTrue;
+    return true;
 
   state = e_Idle;
   replyTimer.Stop(false);
@@ -455,20 +439,16 @@ PBoolean H245NegTerminalCapabilitySet::HandleReject(const H245_TerminalCapabilit
 
 PBoolean H245NegTerminalCapabilitySet::HandleRelease(const H245_TerminalCapabilitySetRelease & /*pdu*/)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived TerminalCapabilityRelease: state=" << state);
 
-  receivedCapabilites = PFalse;
+  receivedCapabilites = false;
   return connection.OnControlProtocolError(H323Connection::e_CapabilityExchange,
                                            "Aborted");
 }
 
 
-void H245NegTerminalCapabilitySet::HandleTimeout(PTimer &, INT)
+void H245NegTerminalCapabilitySet::HandleTimeout()
 {
-  PWaitAndSignal wait(mutex);
-
   if (state == e_Idle)
     return;
 
@@ -522,28 +502,17 @@ H245NegLogicalChannel::~H245NegLogicalChannel()
   replyTimer.Stop();
   PThread::Yield(); // Do this to avoid possible race condition with timer
 
-  mutex.Wait();
   delete channel;
-  mutex.Signal();
 }
 
 
 PBoolean H245NegLogicalChannel::Open(const H323Capability & capability,
                                  unsigned sessionID,
                                  unsigned replacementFor)
-{
-  PWaitAndSignal wait(mutex);
-  return OpenWhileLocked(capability, sessionID, replacementFor);
-}
-
-
-PBoolean H245NegLogicalChannel::OpenWhileLocked(const H323Capability & capability,
-                                            unsigned sessionID,
-                                            unsigned replacementFor)
 {
   if (state != e_Released && state != e_AwaitingRelease) {
     PTRACE(2, "H245\tOpen of channel currently in negotiations: " << channelNumber);
-    return PFalse;
+    return false;
   }
 
   PTRACE(3, "H245\tOpening channel: " << channelNumber);
@@ -562,14 +531,14 @@ PBoolean H245NegLogicalChannel::OpenWhileLocked(const H323Capability & capabilit
   if (!capability.OnSendingPDU(open.m_forwardLogicalChannelParameters.m_dataType)) {
     PTRACE(1, "H245\tOpening channel: " << channelNumber
            << ", capability.OnSendingPDU() failed");
-    return PFalse;
+    return false;
   }
 
   channel = capability.CreateChannel(connection, H323Channel::IsTransmitter, sessionID, NULL);
   if (channel == NULL) {
     PTRACE(1, "H245\tOpening channel: " << channelNumber
            << ", capability.CreateChannel() failed");
-    return PFalse;
+    return false;
   }
 
   channel->SetNumber(channelNumber);
@@ -577,7 +546,7 @@ PBoolean H245NegLogicalChannel::OpenWhileLocked(const H323Capability & capabilit
   if (!channel->OnSendingPDU(open)) {
     PTRACE(1, "H245\tOpening channel: " << channelNumber
            << ", channel->OnSendingPDU() failed");
-    return PFalse;
+    return false;
   }
 
   if (replacementFor > 0) {
@@ -591,12 +560,12 @@ PBoolean H245NegLogicalChannel::OpenWhileLocked(const H323Capability & capabilit
     }
   }
 
-  if (!channel->Open())
-    return PFalse;
+  if (!channel->PreOpen())
+    return false;
 
   if (!channel->SetInitialBandwidth()) {
     PTRACE(2, "H245\tOpening channel: " << channelNumber << ", Insufficient bandwidth");
-    return PFalse;
+    return false;
   }
 
   replyTimer = endpoint.GetLogicalChannelTimeout();
@@ -607,18 +576,18 @@ PBoolean H245NegLogicalChannel::OpenWhileLocked(const H323Capability & capabilit
 
 PBoolean H245NegLogicalChannel::Close()
 {
-  PWaitAndSignal wait(mutex);
-  return CloseWhileLocked();
-}
+  switch (state) {
+    case e_Establishing :
+    case e_AwaitingRelease :
+    case e_Released :
+      return true;
 
+    default :
+      break;
+  }
 
-PBoolean H245NegLogicalChannel::CloseWhileLocked()
-{
   PTRACE(3, "H245\tClosing channel: " << channelNumber << ", state=" << state);
 
-  if (state != e_AwaitingEstablishment && state != e_Established)
-    return PTrue;
-
   replyTimer = endpoint.GetLogicalChannelTimeout();
 
   H323ControlPDU reply;
@@ -648,47 +617,45 @@ PBoolean H245NegLogicalChannel::HandleOpen(const H245_OpenLogicalChannel & pdu)
     channel = NULL;
   }
 
-  state = e_AwaitingEstablishment;
+  state = e_Establishing;
 
   H323ControlPDU reply;
   H245_OpenLogicalChannelAck & ack = reply.BuildOpenLogicalChannelAck(channelNumber);
 
-  PBoolean ok = PFalse;
+  PBoolean ok = false;
 
   unsigned cause = H245_OpenLogicalChannelReject_cause::e_unspecified;
-  if (connection.OnOpenLogicalChannel(pdu, ack, cause))
-    channel = connection.CreateLogicalChannel(pdu, PFalse, cause);
+  channel = connection.CreateLogicalChannel(pdu, FALSE, cause);
 
   if (channel != NULL) {
     channel->SetNumber(channelNumber);
     channel->OnSendOpenAck(pdu, ack);
-    if (channel->GetDirection() == H323Channel::IsBidirectional) {
-      state = e_AwaitingConfirmation;
-      replyTimer = endpoint.GetLogicalChannelTimeout(); // T103
-      ok = PTrue;
+
+    if (connection.OnOpenLogicalChannel(pdu, ack, cause, *channel)) {
+      if (channel->GetDirection() == H323Channel::IsBidirectional) {
+        state = e_AwaitingConfirmation;
+        replyTimer = endpoint.GetLogicalChannelTimeout(); // T103
+        ok = true;
+      }
+      else {
+        ok = channel->Open();
+        if (ok)
+          state = e_Established;
+      }
     }
     else {
-      ok = channel->Start();
-      if (!ok) {
-        // The correct protocol thing to do is reject the channel if we are
-        // the master. However NetMeeting will not then reopen a channel, so
-        // we act like we are a slave and close our end instead.
-        if (connection.IsH245Master() &&
-            connection.GetRemoteApplication().Find("NetMeeting") == P_MAX_INDEX)
-          cause = H245_OpenLogicalChannelReject_cause::e_masterSlaveConflict;
-        else {
-          connection.OnConflictingLogicalChannel(*channel);
-          ok = channel->Start();
-        }
-      }
-
-      if (ok)
-        state = e_Established;
+      channel->Close();
+      delete channel;
+      channel = NULL;
+      ok = false;
     }
   }
 
-  if (ok)
-    mutex.Signal();
+  if (ok) {
+    OpalMediaStreamPtr mediaStream = channel->GetMediaStream();
+    if (mediaStream != NULL)
+      mediaStream->Start();
+  }
   else {
     reply.BuildOpenLogicalChannelReject(channelNumber, cause);
     Release();
@@ -700,8 +667,6 @@ PBoolean H245NegLogicalChannel::HandleOpen(const H245_OpenLogicalChannel & pdu)
 
 PBoolean H245NegLogicalChannel::HandleOpenAck(const H245_OpenLogicalChannelAck & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived open channel ack: " << channelNumber << ", state=" << state);
 
   switch (state) {
@@ -714,7 +679,7 @@ PBoolean H245NegLogicalChannel::HandleOpenAck(const H245_OpenLogicalChannelAck &
 
       if (!channel->OnReceivedAckPDU(pdu)) {
         if (connection.GetRemoteProductInfo().name != "Cisco IOS")
-          return CloseWhileLocked();
+          return Close();
         PTRACE(4, "H245\tWorkaround for Cisco bug, cannot close channel on illegal ack or it hangs up on you.");
         return true;
       }
@@ -723,26 +688,29 @@ PBoolean H245NegLogicalChannel::HandleOpenAck(const H245_OpenLogicalChannelAck &
         H323ControlPDU reply;
         reply.BuildOpenLogicalChannelConfirm(channelNumber);
         if (!connection.WriteControlPDU(reply))
-          return PFalse;
+          return false;
       }
 
       // Channel was already opened when OLC sent, if have error here it is
       // somthing other than an asymmetric codec conflict, so close it down.
-      if (!channel->Start())
-        return CloseWhileLocked();
+      if (channel->Open()) {
+        OpalMediaStreamPtr mediaStream = channel->GetMediaStream();
+        if (mediaStream != NULL)
+          mediaStream->Start();
+      }
+      else
+        return Close();
 
     default :
       break;
   }
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H245NegLogicalChannel::HandleOpenConfirm(const H245_OpenLogicalChannelConfirm & /*pdu*/)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived open channel confirm: " << channelNumber << ", state=" << state);
 
   switch (state) {
@@ -757,26 +725,23 @@ PBoolean H245NegLogicalChannel::HandleOpenConfirm(const H245_OpenLogicalChannelC
       state = e_Established;
       // Channel was already opened when OLC sent, if have error here it is
       // somthing other than an asymmetric codec conflict, so close it down.
-      if (!channel->Start())
-        return CloseWhileLocked();
+      if (!channel->Open())
+        return Close();
 
     default :
       break;
   }
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H245NegLogicalChannel::HandleReject(const H245_OpenLogicalChannelReject & pdu)
 {
-  mutex.Wait();
-
   PTRACE(3, "H245\tReceived open channel reject: " << channelNumber << ", state=" << state);
 
   switch (state) {
     case e_Released :
-      mutex.Signal();
       return connection.OnControlProtocolError(H323Connection::e_LogicalChannel,
                                                "Reject unknown channel");
     case e_Established :
@@ -794,18 +759,15 @@ PBoolean H245NegLogicalChannel::HandleReject(const H245_OpenLogicalChannelReject
       break;
 
     default :
-      mutex.Signal();
       break;
   }
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H245NegLogicalChannel::HandleClose(const H245_CloseLogicalChannel & /*pdu*/)
 {
-  mutex.Wait();
-
   PTRACE(3, "H245\tReceived close channel: " << channelNumber << ", state=" << state);
 
   //if (pdu.m_source.GetTag() == H245_CloseLogicalChannel_source::e_user)
@@ -821,8 +783,6 @@ PBoolean H245NegLogicalChannel::HandleClose(const H245_CloseLogicalChannel & /*p
 
 PBoolean H245NegLogicalChannel::HandleCloseAck(const H245_CloseLogicalChannelAck & /*pdu*/)
 {
-  mutex.Wait();
-
   PTRACE(3, "H245\tReceived close channel ack: " << channelNumber << ", state=" << state);
 
   switch (state) {
@@ -835,28 +795,25 @@ PBoolean H245NegLogicalChannel::HandleCloseAck(const H245_CloseLogicalChannelAck
       break;
 
     default :
-      mutex.Signal();
       break;
   }
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H245NegLogicalChannel::HandleRequestClose(const H245_RequestChannelClose & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived request close channel: " << channelNumber << ", state=" << state);
 
   if (state != e_Established)
-    return PTrue;    // Already closed
+    return true;    // Already closed
 
   H323ControlPDU reply;
   if (connection.OnClosingLogicalChannel(*channel)) {
     reply.BuildRequestChannelCloseAck(channelNumber);
     if (!connection.WriteControlPDU(reply))
-      return PFalse;
+      return false;
 
     // Do normal Close procedure
     replyTimer = endpoint.GetLogicalChannelTimeout();
@@ -879,50 +836,40 @@ PBoolean H245NegLogicalChannel::HandleRequestClose(const H245_RequestChannelClos
 
 PBoolean H245NegLogicalChannel::HandleRequestCloseAck(const H245_RequestChannelCloseAck & /*pdu*/)
 {
-  mutex.Wait();
-
   PTRACE(3, "H245\tReceived request close ack channel: " << channelNumber << ", state=" << state);
 
   if (state == e_AwaitingResponse)
     Release();  // Other end says close OK, so do so.
-  else
-    mutex.Signal();
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H245NegLogicalChannel::HandleRequestCloseReject(const H245_RequestChannelCloseReject & /*pdu*/)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived request close reject channel: " << channelNumber << ", state=" << state);
 
   // Other end refused close, so go back to still having channel open
   if (state == e_AwaitingResponse)
     state = e_Established;
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H245NegLogicalChannel::HandleRequestCloseRelease(const H245_RequestChannelCloseRelease & /*pdu*/)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tReceived request close release channel: " << channelNumber << ", state=" << state);
 
   // Other end refused close, so go back to still having channel open
   state = e_Established;
 
-  return PTrue;
+  return true;
 }
 
 
-void H245NegLogicalChannel::HandleTimeout(PTimer &, INT)
+void H245NegLogicalChannel::HandleTimeout()
 {
-  mutex.Wait();
-
   PTRACE(3, "H245\tTimeout on open channel: " << channelNumber << ", state=" << state);
 
   H323ControlPDU reply;
@@ -938,7 +885,6 @@ void H245NegLogicalChannel::HandleTimeout(PTimer &, INT)
       break;
 
     case e_Released :
-      mutex.Signal();
       return;
 
     default :
@@ -955,10 +901,11 @@ void H245NegLogicalChannel::Release()
   state = e_Released;
   H323Channel * chan = channel;
   channel = NULL;
-  mutex.Signal();
 
   replyTimer.Stop(false);
 
+  PTRACE(4, "H245\tOLC Release: chan=" << chan);
+  
   if (chan != NULL) {
     chan->Close();
     delete chan;
@@ -966,7 +913,7 @@ void H245NegLogicalChannel::Release()
 }
 
 
-H323Channel * H245NegLogicalChannel::GetChannel()
+H323Channel * H245NegLogicalChannel::GetChannel() const
 {
   return channel;
 }
@@ -993,16 +940,14 @@ const char * H245NegLogicalChannel::GetStateName(States s)
 H245NegLogicalChannels::H245NegLogicalChannels(H323EndPoint & end,
                                                H323Connection & conn)
   : H245Negotiator(end, conn),
-    lastChannelNumber(100, PFalse)
+    lastChannelNumber(100, false)
 {
 }
 
 
 void H245NegLogicalChannels::Add(H323Channel & channel)
 {
-  mutex.Wait();
   channels.SetAt(channel.GetNumber(), new H245NegLogicalChannel(endpoint, connection, channel));
-  mutex.Signal();
 }
 
 
@@ -1010,15 +955,11 @@ PBoolean H245NegLogicalChannels::Open(const H323Capability & capability,
                                   unsigned sessionID,
                                   unsigned replacementFor)
 {
-  mutex.Wait();
-
   lastChannelNumber++;
 
   H245NegLogicalChannel * negChan = new H245NegLogicalChannel(endpoint, connection, lastChannelNumber);
   channels.SetAt(lastChannelNumber, negChan);
 
-  mutex.Signal();
-
   return negChan->Open(capability, sessionID, replacementFor);
 }
 
@@ -1036,11 +977,9 @@ PBoolean H245NegLogicalChannels::Close(unsigned channelNumber, PBoolean fromRemo
 
 PBoolean H245NegLogicalChannels::HandleOpen(const H245_OpenLogicalChannel & pdu)
 {
-  H323ChannelNumber chanNum(pdu.m_forwardLogicalChannelNumber, PTrue);
+  H323ChannelNumber chanNum(pdu.m_forwardLogicalChannelNumber, true);
   H245NegLogicalChannel * chan;
 
-  mutex.Wait();
-
   if (channels.Contains(chanNum))
     chan = &channels[chanNum];
   else {
@@ -1048,17 +987,13 @@ PBoolean H245NegLogicalChannels::HandleOpen(const H245_OpenLogicalChannel & pdu)
     channels.SetAt(chanNum, chan);
   }
 
-  chan->mutex.Wait();
-
-  mutex.Signal();
-
   return chan->HandleOpen(pdu);
 }
 
 
 PBoolean H245NegLogicalChannels::HandleOpenAck(const H245_OpenLogicalChannelAck & pdu)
 {
-  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, PFalse);
+  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, false);
   if (chan != NULL)
     return chan->HandleOpenAck(pdu);
 
@@ -1069,7 +1004,7 @@ PBoolean H245NegLogicalChannels::HandleOpenAck(const H245_OpenLogicalChannelAck
 
 PBoolean H245NegLogicalChannels::HandleOpenConfirm(const H245_OpenLogicalChannelConfirm & pdu)
 {
-  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, PTrue);
+  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, true);
   if (chan != NULL)
     return chan->HandleOpenConfirm(pdu);
 
@@ -1080,7 +1015,7 @@ PBoolean H245NegLogicalChannels::HandleOpenConfirm(const H245_OpenLogicalChannel
 
 PBoolean H245NegLogicalChannels::HandleReject(const H245_OpenLogicalChannelReject & pdu)
 {
-  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, PFalse);
+  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, false);
   if (chan != NULL)
     return chan->HandleReject(pdu);
 
@@ -1091,7 +1026,7 @@ PBoolean H245NegLogicalChannels::HandleReject(const H245_OpenLogicalChannelRejec
 
 PBoolean H245NegLogicalChannels::HandleClose(const H245_CloseLogicalChannel & pdu)
 {
-  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, PTrue);
+  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, true);
   if (chan != NULL)
     return chan->HandleClose(pdu);
 
@@ -1102,7 +1037,7 @@ PBoolean H245NegLogicalChannels::HandleClose(const H245_CloseLogicalChannel & pd
 
 PBoolean H245NegLogicalChannels::HandleCloseAck(const H245_CloseLogicalChannelAck & pdu)
 {
-  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, PFalse);
+  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, false);
   if (chan != NULL)
     return chan->HandleCloseAck(pdu);
 
@@ -1113,7 +1048,7 @@ PBoolean H245NegLogicalChannels::HandleCloseAck(const H245_CloseLogicalChannelAc
 
 PBoolean H245NegLogicalChannels::HandleRequestClose(const H245_RequestChannelClose & pdu)
 {
-  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, PFalse);
+  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, false);
   if (chan != NULL)
     return chan->HandleRequestClose(pdu);
 
@@ -1124,7 +1059,7 @@ PBoolean H245NegLogicalChannels::HandleRequestClose(const H245_RequestChannelClo
 
 PBoolean H245NegLogicalChannels::HandleRequestCloseAck(const H245_RequestChannelCloseAck & pdu)
 {
-  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, PTrue);
+  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, true);
   if (chan != NULL)
     return chan->HandleRequestCloseAck(pdu);
 
@@ -1135,7 +1070,7 @@ PBoolean H245NegLogicalChannels::HandleRequestCloseAck(const H245_RequestChannel
 
 PBoolean H245NegLogicalChannels::HandleRequestCloseReject(const H245_RequestChannelCloseReject & pdu)
 {
-  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, PTrue);
+  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, true);
   if (chan != NULL)
     return chan->HandleRequestCloseReject(pdu);
 
@@ -1146,7 +1081,7 @@ PBoolean H245NegLogicalChannels::HandleRequestCloseReject(const H245_RequestChan
 
 PBoolean H245NegLogicalChannels::HandleRequestCloseRelease(const H245_RequestChannelCloseRelease & pdu)
 {
-  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, PFalse);
+  H245NegLogicalChannel * chan = FindNegLogicalChannel(pdu.m_forwardLogicalChannelNumber, false);
   if (chan != NULL)
     return chan->HandleRequestCloseRelease(pdu);
 
@@ -1157,26 +1092,14 @@ PBoolean H245NegLogicalChannels::HandleRequestCloseRelease(const H245_RequestCha
 
 H323ChannelNumber H245NegLogicalChannels::GetNextChannelNumber()
 {
-  PWaitAndSignal wait(mutex);
   lastChannelNumber++;
   return lastChannelNumber;
 }
 
 
-H323Channel * H245NegLogicalChannels::GetChannelAt(PINDEX i)
-{
-  mutex.Wait();
-  H323Channel * chan =  channels.GetDataAt(i).GetChannel();
-  mutex.Signal();
-  return chan;
-}
-
-
 H323Channel * H245NegLogicalChannels::FindChannel(unsigned channelNumber,
                                                   PBoolean fromRemote)
 {
-  PWaitAndSignal wait(mutex);
-
   H323ChannelNumber chanNum(channelNumber, fromRemote);
 
   if (channels.Contains(chanNum))
@@ -1186,35 +1109,20 @@ H323Channel * H245NegLogicalChannels::FindChannel(unsigned channelNumber,
 }
 
 
-H245NegLogicalChannel & H245NegLogicalChannels::GetNegLogicalChannelAt(PINDEX i)
-{
-  PWaitAndSignal wait(mutex);
-  return channels.GetDataAt(i);
-}
-
-
 H245NegLogicalChannel * H245NegLogicalChannels::FindNegLogicalChannel(unsigned channelNumber,
                                                                       PBoolean fromRemote)
 {
   H323ChannelNumber chanNum(channelNumber, fromRemote);
-
-  mutex.Wait();
-  H245NegLogicalChannel * channel = channels.GetAt(chanNum);
-  mutex.Signal();
-
-  return channel;
+  return channels.GetAt(chanNum);
 }
 
 
 H323Channel * H245NegLogicalChannels::FindChannelBySession(unsigned rtpSessionId,
                                                            PBoolean fromRemote)
 {
-  PWaitAndSignal wait(mutex);
-
-  PINDEX i;
   H323Channel::Directions desiredDirection = fromRemote ? H323Channel::IsReceiver : H323Channel::IsTransmitter;
-  for (i = 0; i < GetSize(); i++) {
-    H245NegLogicalChannel & logChan = channels.GetDataAt(i);
+  for (H245LogicalChannelDict::iterator it = channels.begin(); it != channels.end(); ++it) {
+    H245NegLogicalChannel & logChan = it->second;
     if (logChan.IsAwaitingEstablishment() || logChan.IsEstablished()) {
       H323Channel * channel = logChan.GetChannel();
       if (channel != NULL && channel->GetSessionID() == rtpSessionId &&
@@ -1229,15 +1137,11 @@ H323Channel * H245NegLogicalChannels::FindChannelBySession(unsigned rtpSessionId
 
 void H245NegLogicalChannels::RemoveAll()
 {
-  PWaitAndSignal wait(mutex);
-
-  for (PINDEX i = 0; i < channels.GetSize(); i++) {
-    H245NegLogicalChannel & neg = channels.GetDataAt(i);
-    neg.mutex.Wait();
+  for (H245LogicalChannelDict::iterator it = channels.begin(); it != channels.end(); ++it) {
+    H245NegLogicalChannel & neg = it->second;
     H323Channel * channel = neg.GetChannel();
     if (channel != NULL)
       channel->Close();
-    neg.mutex.Signal();
   }
 
   channels.RemoveAll();
@@ -1249,7 +1153,7 @@ void H245NegLogicalChannels::RemoveAll()
 H245NegRequestMode::H245NegRequestMode(H323EndPoint & end, H323Connection & conn)
   : H245Negotiator(end, conn)
 {
-  awaitingResponse = PFalse;
+  awaitingResponse = false;
   inSequenceNumber = UINT_MAX;
   outSequenceNumber = 0;
 }
@@ -1311,7 +1215,7 @@ PBoolean H245NegRequestMode::StartRequest(const H245_ArrayOf_ModeDescription & n
   // Initiate a mode request
   outSequenceNumber = (outSequenceNumber+1)%256;
   replyTimer = endpoint.GetRequestModeTimeout();
-  awaitingResponse = PTrue;
+  awaitingResponse = true;
 
   H323ControlPDU pdu;
   H245_RequestMode & requestMode = pdu.BuildRequestMode(outSequenceNumber);
@@ -1344,10 +1248,10 @@ PBoolean H245NegRequestMode::HandleRequest(const H245_RequestMode & pdu)
     ack.m_response.SetTag(H245_RequestModeAck_response::e_willTransmitLessPreferredMode);
 
   if (!connection.WriteControlPDU(reply_ack))
-    return PFalse;
+    return false;
 
   connection.OnModeChanged(pdu.m_requestedModes[selectedMode]);
-  return PTrue;
+  return true;
 }
 
 
@@ -1357,12 +1261,12 @@ PBoolean H245NegRequestMode::HandleAck(const H245_RequestModeAck & pdu)
          << (awaitingResponse ? " awaitingResponse" : " idle"));
 
   if (awaitingResponse && pdu.m_sequenceNumber == outSequenceNumber) {
-    awaitingResponse = PFalse;
+    awaitingResponse = false;
     replyTimer.Stop(false);
     connection.OnAcceptModeChange(pdu);
   }
 
-  return PTrue;
+  return true;
 }
 
 PBoolean H245NegRequestMode::HandleReject(const H245_RequestModeReject & pdu)
@@ -1371,29 +1275,29 @@ PBoolean H245NegRequestMode::HandleReject(const H245_RequestModeReject & pdu)
          << (awaitingResponse ? " awaitingResponse" : " idle"));
 
   if (awaitingResponse && pdu.m_sequenceNumber == outSequenceNumber) {
-    awaitingResponse = PFalse;
+    awaitingResponse = false;
     replyTimer.Stop(false);
     connection.OnRefusedModeChange(&pdu);
   }
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H245NegRequestMode::HandleRelease(const H245_RequestModeRelease & /*pdu*/)
 {
   PTRACE(3, "H245\tReceived release on request mode: inSeq=" << inSequenceNumber);
-  return PTrue;
+  return true;
 }
 
 
-void H245NegRequestMode::HandleTimeout(PTimer &, INT)
+void H245NegRequestMode::HandleTimeout()
 {
   PTRACE(3, "H245\tTimeout on request mode: outSeq=" << outSequenceNumber
          << (awaitingResponse ? " awaitingResponse" : " idle"));
 
   if (awaitingResponse) {
-    awaitingResponse = PFalse;
+    awaitingResponse = false;
     H323ControlPDU pdu;
     pdu.Build(H245_IndicationMessage::e_requestModeRelease);
     connection.WriteControlPDU(pdu);
@@ -1408,7 +1312,7 @@ void H245NegRequestMode::HandleTimeout(PTimer &, INT)
 H245NegRoundTripDelay::H245NegRoundTripDelay(H323EndPoint & end, H323Connection & conn)
   : H245Negotiator(end, conn)
 {
-  awaitingResponse = PFalse;
+  awaitingResponse = false;
   sequenceNumber = 0;
 
   // Temporary (ie quick) fix for strange Cisco behaviour. If keep trying to
@@ -1419,11 +1323,9 @@ H245NegRoundTripDelay::H245NegRoundTripDelay(H323EndPoint & end, H323Connection
 
 PBoolean H245NegRoundTripDelay::StartRequest()
 {
-  PWaitAndSignal wait(mutex);
-
   replyTimer = endpoint.GetRoundTripDelayTimeout();
   sequenceNumber = (sequenceNumber + 1)%256;
-  awaitingResponse = PTrue;
+  awaitingResponse = true;
 
   PTRACE(3, "H245\tStarted round trip delay: seq=" << sequenceNumber
          << (awaitingResponse ? " awaitingResponse" : " idle"));
@@ -1431,17 +1333,15 @@ PBoolean H245NegRoundTripDelay::StartRequest()
   H323ControlPDU pdu;
   pdu.BuildRoundTripDelayRequest(sequenceNumber);
   if (!connection.WriteControlPDU(pdu))
-    return PFalse;
+    return false;
 
   tripStartTime = PTimer::Tick();
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H245NegRoundTripDelay::HandleRequest(const H245_RoundTripDelayRequest & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tStarted round trip delay: seq=" << sequenceNumber
          << (awaitingResponse ? " awaitingResponse" : " idle"));
 
@@ -1453,8 +1353,6 @@ PBoolean H245NegRoundTripDelay::HandleRequest(const H245_RoundTripDelayRequest &
 
 PBoolean H245NegRoundTripDelay::HandleResponse(const H245_RoundTripDelayResponse & pdu)
 {
-  PWaitAndSignal wait(mutex);
-
   PTimeInterval tripEndTime = PTimer::Tick();
 
   PTRACE(3, "H245\tHandling round trip delay: seq=" << sequenceNumber
@@ -1462,25 +1360,23 @@ PBoolean H245NegRoundTripDelay::HandleResponse(const H245_RoundTripDelayResponse
 
   if (awaitingResponse && pdu.m_sequenceNumber == sequenceNumber) {
     replyTimer.Stop(false);
-    awaitingResponse = PFalse;
+    awaitingResponse = false;
     roundTripTime = tripEndTime - tripStartTime;
     retryCount = 3;
   }
 
-  return PTrue;
+  return true;
 }
 
 
-void H245NegRoundTripDelay::HandleTimeout(PTimer &, INT)
+void H245NegRoundTripDelay::HandleTimeout()
 {
-  PWaitAndSignal wait(mutex);
-
   PTRACE(3, "H245\tTimeout on round trip delay: seq=" << sequenceNumber
          << (awaitingResponse ? " awaitingResponse" : " idle"));
 
   if (awaitingResponse && retryCount > 0)
     retryCount--;
-  awaitingResponse = PFalse;
+  awaitingResponse = false;
 
   connection.OnControlProtocolError(H323Connection::e_RoundTripDelay, "Timeout");
 }
