@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24442 $
+ * $Revision: 29736 $
  * $Author: rjongbloed $
- * $Date: 2010-06-02 03:17:31 -0500 (Wed, 02 Jun 2010) $
+ * $Date: 2013-05-17 03:23:30 -0500 (Fri, 17 May 2013) $
  */
 
 #include <ptlib.h>
@@ -77,7 +77,7 @@ void H323TraceDumpPDU(const char * proto,
   if (!PTrace::CanTrace(3))
     return;
 
-  ostream & trace = PTrace::Begin(3, __FILE__, __LINE__);
+  ostream & trace = PTrace::Begin(PTrace::CanTrace(4) ? 4 : 3, __FILE__, __LINE__);
   trace << proto << '\t' << (writing ? "Send" : "Receiv") << "ing PDU:";
 
   if (PTrace::CanTrace(4)) {
@@ -342,63 +342,33 @@ void H323GetApplicationInfo(OpalProductInfo & info, const H225_VendorIdentifier
 
 ///////////////////////////////////////////////////////////////////////////////
 
-void H323SetRTPPacketization(H245_ArrayOf_RTPPayloadType & rtpPacketizations,
-                             PINDEX & rtpPacketizationCount,
-                             const OpalMediaFormat & mediaFormat,
-                             RTP_DataFrame::PayloadTypes payloadType)
+bool H323SetRTPPacketization(H245_ArrayOf_RTPPayloadType & rtpPacketizations, const PStringSet & mediaPacketizations)
 {
-  PString mediaPacketization = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
-                               mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
-  if (mediaPacketization.IsEmpty())
-    return;
-  
-  // Special case handling by Product Id
-  PString h323ProductId = mediaFormat.GetOptionString("h323ProductId");
-  if (h323ProductId == "NetMeeting")
-    return;
+  PINDEX rtpPacketizationCount = 0;
 
-  PStringArray packetizationStrings = mediaPacketization.Tokenise(",");
-  for (PINDEX i = 0; i < packetizationStrings.GetSize(); i++) {
-    PString & packetizationString = packetizationStrings[i];
-    
+  for (PStringSet::const_iterator it = mediaPacketizations.begin(); it != mediaPacketizations.end(); ++it) {
     rtpPacketizations.SetSize(rtpPacketizationCount+1);
-    if (H323SetRTPPacketization(rtpPacketizations[rtpPacketizationCount], packetizationString, mediaFormat, payloadType)) {
-      
-      // Check if already in list
-      PINDEX test;
-      for (test = 0; test < rtpPacketizationCount; test++) {
-        if (rtpPacketizations[test] == rtpPacketizations[rtpPacketizationCount])
-          break;
-      }
-      if (test == rtpPacketizationCount)
-        rtpPacketizationCount++;
-    }
+    if (H323SetRTPPacketization(rtpPacketizations[rtpPacketizationCount], *it, RTP_DataFrame::IllegalPayloadType))
+      rtpPacketizationCount++;
   }
+
+  return rtpPacketizationCount > 0;
 }
 
 bool H323SetRTPPacketization(H245_RTPPayloadType & rtpPacketization,
                              const OpalMediaFormat & mediaFormat,
                              RTP_DataFrame::PayloadTypes payloadType)
 {
-  PString mediaPacketization = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
-                               mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
-  if (mediaPacketization.IsEmpty())
-    return PFalse;
+  if (payloadType == RTP_DataFrame::IllegalPayloadType)
+    payloadType = mediaFormat.GetPayloadType();
 
-  // Special case handling by Product Id
-  PString h323ProductId = mediaFormat.GetOptionString("h323ProductId");
-  if (h323ProductId == "NetMeeting")
-    return PFalse;
-  
-  PStringArray packetizationStrings = mediaPacketization.Tokenise(",");
-  
-  // Only use the first packetization (= highest priority)
-  return H323SetRTPPacketization(rtpPacketization, packetizationStrings[0], mediaFormat, payloadType);
+  PStringArray mediaPacketizations = mediaFormat.GetMediaPacketizations();
+  return !mediaPacketizations.IsEmpty() &&
+         H323SetRTPPacketization(rtpPacketization, mediaPacketizations[0], payloadType);
 }
 
 bool H323SetRTPPacketization(H245_RTPPayloadType & rtpPacketization,
                              const PString & packetizationString,
-                             const OpalMediaFormat & mediaFormat,
                              RTP_DataFrame::PayloadTypes payloadType)
 {
   if (packetizationString.NumCompare("RFC") == PObject::EqualTo) {
@@ -420,13 +390,12 @@ bool H323SetRTPPacketization(H245_RTPPayloadType & rtpPacketization,
     nonstd.m_data = packetizationString;
   }
 
-  if (payloadType == RTP_DataFrame::IllegalPayloadType)
-    payloadType = mediaFormat.GetPayloadType();
+  if (payloadType != RTP_DataFrame::IllegalPayloadType) {
+    rtpPacketization.IncludeOptionalField(H245_RTPPayloadType::e_payloadType);
+    rtpPacketization.m_payloadType = payloadType;
+  }
 
-  rtpPacketization.IncludeOptionalField(H245_RTPPayloadType::e_payloadType);
-  rtpPacketization.m_payloadType = payloadType;
-
-  return PTrue;
+  return true;
 }
 
 
@@ -441,8 +410,12 @@ PString H323GetRTPPacketization(const H245_RTPPayloadType & rtpPacketization)
 
     case H245_RTPPayloadType_payloadDescriptor::e_oid :
       mediaPacketization = ((const PASN_ObjectId &)rtpPacketization.m_payloadDescriptor).AsString();
-      if (mediaPacketization.IsEmpty()) {
-        PTRACE(1, "RTP_UDP\tInvalid OID in packetization type.");
+      PTRACE_IF(1, mediaPacketization.IsEmpty(), "RTP_UDP\tInvalid OID in packetization type.");
+
+      // Some stupid endpoints (e.g. Polycom) use this, one zero short!
+      if (mediaPacketization == "0.0.8.241.0.0.0") {
+        mediaPacketization    = "0.0.8.241.0.0.0.0";
+        PTRACE(4, "RTP_UDP\tDetected bogus H.264 OID in packetization type, compensating");
       }
       break;
     case H245_RTPPayloadType_payloadDescriptor::e_nonStandardIdentifier :
@@ -460,18 +433,6 @@ PString H323GetRTPPacketization(const H245_RTPPayloadType & rtpPacketization)
 }
 
 
-bool H323GetRTPPacketization(OpalMediaFormat & mediaFormat, const H245_RTPPayloadType & rtpPacketization)
-{
-  PString mediaPacketization = H323GetRTPPacketization(rtpPacketization);
-  if (mediaPacketization.IsEmpty())
-    return false;
-  
-  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), mediaPacketization);
-  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(), mediaPacketization.Left(mediaPacketization.Find(',')));
-  return true;
-}
-
-
 PString H323GetCapabilityIdentifier(const H245_CapabilityIdentifier & capId)
 {
   switch (capId.GetTag()) {
@@ -593,60 +554,47 @@ unsigned H323GetGenericParameterInteger(const H245_ArrayOf_GenericParameter & pa
 }
 
 
-H245_ParameterValue * H323AddGenericParameter(H245_ArrayOf_GenericParameter & params, unsigned ordinal, bool reorder)
+H245_ParameterValue * H323AddGenericParameter(H245_ArrayOf_GenericParameter & params, unsigned ordinal)
 {
   PINDEX size = params.GetSize();
   params.SetSize(size+1);
-
-  PINDEX pos = size;
-  if (reorder) {
-    for (pos = size; pos > 0; pos--) {
-      const H245_GenericParameter & param = params[pos-1];
-      if (param.m_parameterIdentifier.GetTag() == H245_ParameterIdentifier::e_standard &&
-                            ((const PASN_Integer &)param.m_parameterIdentifier) < ordinal)
-        break;
-      params[pos] = param;
-    }
-  }
-
-  H245_GenericParameter & param = params[pos];
+  H245_GenericParameter & param = params[size];
   param.m_parameterIdentifier.SetTag(H245_ParameterIdentifier::e_standard);
   (PASN_Integer &)param.m_parameterIdentifier = ordinal;
   return &param.m_parameterValue;
 }
 
 
-void H323AddGenericParameterBoolean(H245_ArrayOf_GenericParameter & params, unsigned ordinal, bool value, bool reorder)
+void H323AddGenericParameterBoolean(H245_ArrayOf_GenericParameter & params, unsigned ordinal, bool value)
 {
   // Do not include a logical at all if it is false
   if (value)
-    H323AddGenericParameter(params, ordinal, reorder)->SetTag(H245_ParameterValue::e_logical);
+    H323AddGenericParameter(params, ordinal)->SetTag(H245_ParameterValue::e_logical);
 }
 
 
 void H323AddGenericParameterInteger(H245_ArrayOf_GenericParameter & params,
                                     unsigned ordinal,
                                     unsigned value,
-                                    H245_ParameterValue::Choices subType,
-                                    bool reorder)
+                                    H245_ParameterValue::Choices subType)
 {
-  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal, reorder);
+  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal);
   param->SetTag(subType);
   (PASN_Integer &)*param = value;
 }
 
 
-void H323AddGenericParameterString(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PString & value, bool reorder)
+void H323AddGenericParameterString(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PString & value)
 {
-  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal, reorder);
+  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal);
   param->SetTag(H245_ParameterValue::e_octetString);
   (PASN_OctetString &)*param = value;
 }
 
 
-void H323AddGenericParameterOctets(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PBYTEArray & value, bool reorder)
+void H323AddGenericParameterOctets(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PBYTEArray & value)
 {
-  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal, reorder);
+  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal);
   param->SetTag(H245_ParameterValue::e_octetString);
   (PASN_OctetString &)*param = value;
 }
@@ -690,7 +638,9 @@ static void SendFeatureSet(const H323Connection * connection, unsigned code, H22
   if (!connection->OnSendFeatureSet(code,fs))
     return;
 
-  if (code == H460_MessageType::e_callProceeding) {
+  if (code == H460_MessageType::e_callProceeding ||
+      code == H460_MessageType::e_alerting ||
+      code == H460_MessageType::e_connect) {
     pdu.IncludeOptionalField(PDUType::e_featureSet);
     pdu.m_featureSet = fs;
     return;
@@ -735,7 +685,7 @@ H225_Setup_UUIE & H323SignalPDU::BuildSetup(const H323Connection & connection,
   H323EndPoint & endpoint = connection.GetEndPoint();
 
   q931pdu.BuildSetup(connection.GetCallReference());
-  SetQ931Fields(connection, PTrue);
+  SetQ931Fields(connection, true);
 
   m_h323_uu_pdu.m_h323_message_body.SetTag(H225_H323_UU_PDU_h323_message_body::e_setup);
   H225_Setup_UUIE & setup = m_h323_uu_pdu.m_h323_message_body;
@@ -746,29 +696,23 @@ H225_Setup_UUIE & H323SignalPDU::BuildSetup(const H323Connection & connection,
   }
 
   setup.IncludeOptionalField(H225_Setup_UUIE::e_sourceAddress);
-  {
-    PString callingParty;
-    callingParty = connection.GetStringOptions()(OPAL_OPT_CALLING_PARTY_NAME);
-    if (callingParty.IsEmpty())
-      H323SetAliasAddresses(endpoint.GetAliasNames(), setup.m_sourceAddress);
-    else
-      H323SetAliasAddresses(PStringArray(callingParty), setup.m_sourceAddress);
-  }
+  H323SetAliasAddresses(connection.GetLocalAliasNames(), setup.m_sourceAddress);
 
   setup.m_conferenceID = connection.GetConferenceIdentifier();
   setup.m_conferenceGoal.SetTag(H225_Setup_UUIE_conferenceGoal::e_create);
   setup.m_callType.SetTag(H225_CallType::e_pointToPoint);
 
   setup.m_callIdentifier.m_guid = connection.GetCallIdentifier();
-  setup.m_mediaWaitForConnect = PFalse;
-  setup.m_canOverlapSend = PFalse;
+  setup.m_mediaWaitForConnect = false;
+  setup.m_canOverlapSend = false;
 
   if (!destAddr) {
     setup.IncludeOptionalField(H225_Setup_UUIE::e_destCallSignalAddress);
     destAddr.SetPDU(setup.m_destCallSignalAddress);
   }
 
-  PString destAlias = connection.GetRemotePartyName();
+  const OpalConnection::StringOptions & stringOptions = connection.GetStringOptions();
+  PString destAlias = stringOptions(OPAL_OPT_CALLED_PARTY_NAME, connection.GetRemotePartyName());
   if (!destAlias && destAlias != destAddr) {
     setup.IncludeOptionalField(H225_Setup_UUIE::e_destinationAddress);
     setup.m_destinationAddress.SetSize(1);
@@ -779,6 +723,10 @@ H225_Setup_UUIE & H323SignalPDU::BuildSetup(const H323Connection & connection,
       q931pdu.SetCalledPartyNumber(destAlias);
   }
 
+  PString redir(stringOptions(OPAL_OPT_REDIRECTING_PARTY, connection.GetRedirectingParty()));
+  if (!redir.IsEmpty())
+    q931pdu.SetRedirectingNumber(redir);
+
   endpoint.SetEndpointTypeInfo(setup.m_sourceInfo);
   
 #if OPAL_H460
@@ -1166,7 +1114,7 @@ void H323SignalPDU::BuildQ931()
 
 void H323SignalPDU::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision() + 2;
+  std::streamsize indent = strm.precision() + 2;
   strm << "{\n"
        << setw(indent+10) << "q931pdu = " << setprecision(indent) << q931pdu << '\n'
        << setw(indent+10) << "h225pdu = " << setprecision(indent);
@@ -1183,14 +1131,14 @@ PBoolean H323SignalPDU::Read(H323Transport & transport)
     PTRACE_IF(1, transport.GetErrorCode(PChannel::LastReadError) != PChannel::Timeout,
               "H225\tRead error (" << transport.GetErrorNumber(PChannel::LastReadError)
               << "): " << transport.GetErrorText(PChannel::LastReadError));
-    return PFalse;
+    return false;
   }
 
   if (!q931pdu.Decode(rawData)) {
     PTRACE(1, "H225\tParse error of Q931 PDU:\n" << hex << setfill('0')
                                                  << setprecision(2) << rawData
                                                  << dec << setfill(' '));
-    return PFalse;
+    return false;
   }
 
   if (!q931pdu.HasIE(Q931::UserUserIE)) {
@@ -1200,7 +1148,7 @@ PBoolean H323SignalPDU::Read(H323Transport & transport)
                              << setprecision(2) << rawData
                              << dec << setfill(' ') <<
               "\nQ.931 PDU:\n  " << setprecision(2) << q931pdu);
-    return PTrue;
+    return true;
   }
 
   PPER_Stream strm = q931pdu.GetIE(Q931::UserUserIE);
@@ -1212,11 +1160,11 @@ PBoolean H323SignalPDU::Read(H323Transport & transport)
               "\nQ.931 PDU:\n  " << setprecision(2) << q931pdu <<
               "\nPartial PDU:\n  " << setprecision(2) << *this);
     m_h323_uu_pdu.m_h323_message_body.SetTag(H225_H323_UU_PDU_h323_message_body::e_empty);
-    return PTrue;
+    return true;
   }
 
-  H323TraceDumpPDU("H225", PFalse, rawData, *this, m_h323_uu_pdu.m_h323_message_body, 0);
-  return PTrue;
+  H323TraceDumpPDU("H225", false, rawData, *this, m_h323_uu_pdu.m_h323_message_body, 0);
+  return true;
 }
 
 
@@ -1227,17 +1175,17 @@ PBoolean H323SignalPDU::Write(H323Transport & transport)
 
   PBYTEArray rawData;
   if (!q931pdu.Encode(rawData))
-    return PFalse;
+    return false;
 
-  H323TraceDumpPDU("H225", PTrue, rawData, *this, m_h323_uu_pdu.m_h323_message_body, 0);
+  H323TraceDumpPDU("H225", true, rawData, *this, m_h323_uu_pdu.m_h323_message_body, 0);
 
   if (transport.WritePDU(rawData))
-    return PTrue;
+    return true;
 
   PTRACE(1, "H225\tWrite PDU failed ("
          << transport.GetErrorNumber(PChannel::LastWriteError)
          << "): " << transport.GetErrorText(PChannel::LastWriteError));
-  return PFalse;
+  return false;
 }
 
 
@@ -1265,7 +1213,7 @@ PString H323SignalPDU::GetSourceAliases(const H323Transport * transport) const
 
     if (setup.m_sourceAddress.GetSize() > 0) {
       PBoolean needParen = !aliases.IsEmpty();
-      PBoolean needComma = PFalse;
+      PBoolean needComma = false;
       for (PINDEX i = 0; i < setup.m_sourceAddress.GetSize(); i++) {
         PString alias = H323GetAliasAddressString(setup.m_sourceAddress[i]);
         if (alias != displayName && alias != remoteHostName) {
@@ -1274,7 +1222,7 @@ PString H323SignalPDU::GetSourceAliases(const H323Transport * transport) const
           else if (needParen)
             aliases << " (";
           aliases << alias;
-          needComma = PTrue;
+          needComma = true;
         }
       }
       if (needParen && needComma)
@@ -1330,20 +1278,20 @@ PString H323SignalPDU::GetDestinationAlias(PBoolean firstAliasOnly) const
 PBoolean H323SignalPDU::GetSourceE164(PString & number) const
 {
   if (GetQ931().GetCallingPartyNumber(number))
-    return PTrue;
+    return true;
 
   if (m_h323_uu_pdu.m_h323_message_body.GetTag() != H225_H323_UU_PDU_h323_message_body::e_setup)
-    return PFalse;
+    return false;
 
   const H225_Setup_UUIE & setup = m_h323_uu_pdu.m_h323_message_body;
   if (!setup.HasOptionalField(H225_Setup_UUIE::e_sourceAddress))
-    return PFalse;
+    return false;
 
   PINDEX i;
   for (i = 0; i < setup.m_sourceAddress.GetSize(); i++) {
     if (setup.m_sourceAddress[i].GetTag() == H225_AliasAddress::e_dialedDigits) {
       number = (PASN_IA5String &)setup.m_sourceAddress[i];
-      return PTrue;
+      return true;
     }
   }
 
@@ -1351,31 +1299,31 @@ PBoolean H323SignalPDU::GetSourceE164(PString & number) const
     PString str = H323GetAliasAddressString(setup.m_sourceAddress[i]);
     if (OpalIsE164(str)) {
       number = str;
-      return PTrue;
+      return true;
     }
   }
 
-  return PFalse;
+  return false;
 }
 
 
 PBoolean H323SignalPDU::GetDestinationE164(PString & number) const
 {
   if (GetQ931().GetCalledPartyNumber(number))
-    return PTrue;
+    return true;
 
   if (m_h323_uu_pdu.m_h323_message_body.GetTag() != H225_H323_UU_PDU_h323_message_body::e_setup)
-    return PFalse;
+    return false;
 
   const H225_Setup_UUIE & setup = m_h323_uu_pdu.m_h323_message_body;
   if (!setup.HasOptionalField(H225_Setup_UUIE::e_destinationAddress))
-    return PFalse;
+    return false;
 
   PINDEX i;
   for (i = 0; i < setup.m_destinationAddress.GetSize(); i++) {
     if (setup.m_destinationAddress[i].GetTag() == H225_AliasAddress::e_dialedDigits) {
       number = (PASN_IA5String &)setup.m_destinationAddress[i];
-      return PTrue;
+      return true;
     }
   }
 
@@ -1383,11 +1331,11 @@ PBoolean H323SignalPDU::GetDestinationE164(PString & number) const
     PString str = H323GetAliasAddressString(setup.m_destinationAddress[i]);
     if (OpalIsE164(str)) {
       number = str;
-      return PTrue;
+      return true;
     }
   }
 
-  return PFalse;
+  return false;
 }
 
 
@@ -1405,38 +1353,31 @@ void H323SignalPDU::SetQ931Fields(const H323Connection & connection, bool insert
 {
   const PStringList & aliases = connection.GetLocalAliasNames();
 
-  PString localName;
-  PString displayName;
-  PString number;
+  PString localName = connection.GetLocalPartyName();
+  PString displayName = connection.GetDisplayName();
 
-  {
-    const OpalConnection::StringOptions & stringOptions = connection.GetStringOptions();
-    localName   = stringOptions(OPAL_OPT_CALLING_PARTY_NAME, connection.GetLocalPartyName());
-    displayName = stringOptions(OPAL_OPT_CALLING_DISPLAY_NAME, connection.GetDisplayName());
-  }
-
-  if (OpalIsE164(localName)) {
-    number = localName;
-    if (displayName.IsEmpty()) {
+  PString number = connection.GetStringOptions()(OPAL_OPT_CALLING_PARTY_NUMBER);
+  if (number.IsEmpty()) {
+    if (OpalIsE164(localName))
+      number = localName;
+    else {
       for (PStringList::const_iterator alias = aliases.begin(); alias != aliases.end(); ++alias) {
-        if (!OpalIsE164(*alias)) {
-          displayName = *alias;
+        if (OpalIsE164(*alias)) {
+          number = *alias;
           break;
         }
       }
     }
   }
-  else {
-    if (displayName.IsEmpty())
-      displayName = localName;
+
+  if (displayName.IsEmpty()) {
     for (PStringList::const_iterator alias = aliases.begin(); alias != aliases.end(); ++alias) {
-      if (OpalIsE164(*alias)) {
-        number = *alias;
+      if (!OpalIsE164(*alias)) {
+        displayName = *alias;
         break;
       }
     }
   }
-
   q931pdu.SetDisplayName(displayName);
 
   if (insertPartyNumbers) {
@@ -1561,7 +1502,7 @@ H245_TerminalCapabilitySet &
   h225_0.m_receiveMultipointCapability.m_mediaDistributionCapability.SetSize(1);
   h225_0.m_transmitMultipointCapability.m_mediaDistributionCapability.SetSize(1);
   h225_0.m_receiveAndTransmitMultipointCapability.m_mediaDistributionCapability.SetSize(1);
-  h225_0.m_t120DynamicPortCapability = PTrue;
+  h225_0.m_t120DynamicPortCapability = true;
 
   // Set the table of capabilities
   connection.GetLocalCapabilities().BuildPDU(connection, cap);
@@ -1820,7 +1761,12 @@ H245_GenericMessage & H323ControlPDU::BuildGenericRequest(const PString & identi
 {
   H245_GenericMessage & msg = Build(H245_RequestMessage::e_genericRequest);
   H323SetCapabilityIdentifier(identifier, msg.m_messageIdentifier);
+  
+  msg.IncludeOptionalField(H245_GenericMessage::e_subMessageIdentifier);
   msg.m_subMessageIdentifier = subMsgId;
+  
+  // Assume always have a content
+  msg.IncludeOptionalField(H245_GenericMessage::e_messageContent);
   return msg;
 }
 
@@ -1847,6 +1793,8 @@ H245_GenericMessage & H323ControlPDU::BuildGenericCommand(const PString & identi
   msg.IncludeOptionalField(H245_GenericMessage::e_subMessageIdentifier);
   msg.m_subMessageIdentifier = subMsgId;
 
+  // Assume always have a content
+  msg.IncludeOptionalField(H245_GenericMessage::e_messageContent);
   return msg;
 }
 
