@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20344 $
- * $Author: rjongbloed $
- * $Date: 2008-05-30 03:10:56 +0000 (Fri, 30 May 2008) $
+ * $Revision: 28937 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 07:21:17 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -333,7 +333,7 @@ void H450ServiceAPDU::AttachSupplementaryServiceAPDU(H323SignalPDU & pdu)
 PBoolean H450ServiceAPDU::WriteFacilityPDU(H323Connection & connection)
 {
   H323SignalPDU facilityPDU;
-  facilityPDU.BuildFacility(connection, PTrue);
+  facilityPDU.BuildFacility(connection, true);
 
   AttachSupplementaryServiceAPDU(facilityPDU);
 
@@ -376,6 +376,8 @@ void H450ServiceAPDU::ParseEndpointAddress(H4501_EndpointAddress& endpointAddres
 H450xDispatcher::H450xDispatcher(H323Connection & conn)
   : connection(conn)
 {
+  PTRACE_CONTEXT_ID_FROM(conn);
+
   opcodeHandler.DisallowDeleteObjects();
 
   nextInvokeId = 0;
@@ -424,7 +426,7 @@ void H450xDispatcher::AttachToReleaseComplete(H323SignalPDU & pdu)
 
 PBoolean H450xDispatcher::HandlePDU(const H323SignalPDU & pdu)
 {
-PBoolean result = PTrue;
+PBoolean result = true;
   for (PINDEX i = 0; i < pdu.m_h323_uu_pdu.m_h4501SupplementaryService.GetSize(); i++) {
     H4501_SupplementaryService supplementaryService;
 
@@ -478,7 +480,7 @@ PBoolean result = PTrue;
 
 PBoolean H450xDispatcher::OnReceivedInvoke(X880_Invoke & invoke, H4501_InterpretationApdu & interpretation)
 {
-  PBoolean result = PTrue;
+  PBoolean result = true;
   // Get the invokeId
   int invokeId = invoke.m_invokeId.GetValue();
 
@@ -502,7 +504,7 @@ PBoolean H450xDispatcher::OnReceivedInvoke(X880_Invoke & invoke, H4501_Interpret
       if (interpretation.GetTag() != H4501_InterpretationApdu::e_discardAnyUnrecognizedInvokePdu)
         SendInvokeReject(invokeId, 1 /*X880_InvokeProblem::e_unrecognisedOperation*/);
       if (interpretation.GetTag() == H4501_InterpretationApdu::e_clearCallIfAnyInvokePduNotRecognized)
-        result = PFalse;
+        result = false;
     }
     else
       result = opcodeHandler[opcode].OnReceivedInvoke(opcode, invokeId, linkedId, argument);
@@ -512,7 +514,7 @@ PBoolean H450xDispatcher::OnReceivedInvoke(X880_Invoke & invoke, H4501_Interpret
       SendInvokeReject(invokeId, 1 /*X880_InvokeProblem::e_unrecognisedOperation*/);
     PTRACE(2, "H4501\tInvoke of unsupported global opcode:\n  " << invoke);
     if (interpretation.GetTag() == H4501_InterpretationApdu::e_clearCallIfAnyInvokePduNotRecognized)
-      result = PFalse;
+      result = false;
   }
   return result;
 }
@@ -528,13 +530,13 @@ PBoolean H450xDispatcher::OnReceivedReturnResult(X880_ReturnResult & returnResul
       break;
     }
   }
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H450xDispatcher::OnReceivedReturnError(X880_ReturnError & returnError)
 {
-  PBoolean result=PTrue;
+  PBoolean result=true;
   unsigned invokeId = returnError.m_invokeId.GetValue();
   int errorCode = 0;
 
@@ -596,7 +598,7 @@ PBoolean H450xDispatcher::OnReceivedReject(X880_Reject & reject)
       break;
     }
   }
-  return PTrue;
+  return true;
 }
 
 
@@ -669,6 +671,8 @@ H450xHandler::H450xHandler(H323Connection & conn, H450xDispatcher & disp)
     connection(conn),
     dispatcher(disp)
 {
+  PTRACE_CONTEXT_ID_FROM(conn);
+
   currentInvokeId = 0;
 }
 
@@ -695,21 +699,21 @@ void H450xHandler::AttachToReleaseComplete(H323SignalPDU &)
 
 PBoolean H450xHandler::OnReceivedReturnResult(X880_ReturnResult & /*returnResult*/)
 {
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H450xHandler::OnReceivedReturnError(int /*errorCode*/,
                                         X880_ReturnError & /*returnError*/)
 {
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H450xHandler::OnReceivedReject(int /*problemType*/,
                                    int /*problemNumber*/)
 {
-  return PTrue;
+  return true;
 }
 
 
@@ -755,19 +759,19 @@ PBoolean H450xHandler::DecodeArguments(PASN_OctetString * argString,
   if (argString == NULL) {
     if (absentErrorCode >= 0)
       SendReturnError(absentErrorCode);
-    return PFalse;
+    return false;
   }
 
   PPER_Stream argStream(*argString);
   if (argObject.Decode(argStream)) {
     PTRACE(4, "H4501\tSupplementary service argument:\n  "
            << setprecision(2) << argObject);
-    return PTrue;
+    return true;
   }
 
   PTRACE(1, "H4501\tInvalid supplementary service argument:\n  "
          << setprecision(2) << argObject);
-  return PFalse;
+  return false;
 }
 
 
@@ -787,9 +791,9 @@ H4502Handler::H4502Handler(H323Connection & conn, H450xDispatcher & disp)
 
   transferringCallToken = "";
   ctState = e_ctIdle;
-  ctResponseSent = PFalse;
+  ctResponseSent = false;
   CallToken = PString();
-  consultationTransfer = PFalse;
+  consultationTransfer = false;
 
   ctTimer.SetNotifier(PCREATE_NOTIFIER(OnCallTransferTimeOut));
 }
@@ -822,7 +826,7 @@ void H4502Handler::AttachToAlerting(H323SignalPDU & pdu)
   H450ServiceAPDU serviceAPDU;
   serviceAPDU.BuildReturnResult(currentInvokeId);
   serviceAPDU.AttachSupplementaryServiceAPDU(pdu);
-  ctResponseSent = PTrue;
+  ctResponseSent = true;
   currentInvokeId = 0;
 }
 
@@ -836,7 +840,7 @@ void H4502Handler::AttachToConnect(H323SignalPDU & pdu)
   H450ServiceAPDU serviceAPDU;
   serviceAPDU.BuildReturnResult(currentInvokeId);
   serviceAPDU.AttachSupplementaryServiceAPDU(pdu);
-  ctResponseSent = PTrue;
+  ctResponseSent = true;
   currentInvokeId = 0;
 }
 
@@ -855,12 +859,12 @@ void H4502Handler::AttachToReleaseComplete(H323SignalPDU & pdu)
 
   if (ctResponseSent) {
     serviceAPDU.BuildReturnResult(currentInvokeId);
-    ctResponseSent = PFalse;
+    ctResponseSent = false;
     currentInvokeId = 0;
   }
   else {
     serviceAPDU.BuildReturnError(currentInvokeId, H4501_GeneralErrorList::e_notAvailable);
-    ctResponseSent = PTrue;
+    ctResponseSent = true;
     currentInvokeId = 0;
   }
 
@@ -910,10 +914,10 @@ PBoolean H4502Handler::OnReceivedInvoke(int opcode,
 
     default:
       currentInvokeId = 0;
-      return PFalse;
+      return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1006,7 +1010,7 @@ void H4502Handler::OnReceivedCallTransferInitiate(int /*linkedId*/,
                        H4502_CallTransferErrors::e_invalidReroutingNumber))
     return;
 
-  ctResponseSent = PTrue;
+  ctResponseSent = true;
 
   PString remoteParty;
   H450ServiceAPDU::ParseEndpointAddress(ctInitiateArg.m_reroutingNumber, remoteParty);
@@ -1124,7 +1128,7 @@ PBoolean H4502Handler::OnReceivedReturnResult(X880_ReturnResult & returnResult)
         break;
     }
   }
-  return PTrue;
+  return true;
 }
 
 
@@ -1212,7 +1216,7 @@ PBoolean H4502Handler::OnReceivedReturnError(int errorCode, X880_ReturnError &re
         break;
     }
   }
-  return PTrue;
+  return true;
 }
 
 
@@ -1419,7 +1423,7 @@ void H4502Handler::StopctTimer()
 }
 
 
-void H4502Handler::OnCallTransferTimeOut(PTimer &, INT)
+void H4502Handler::OnCallTransferTimeOut(PTimer &, P_INT_PTR)
 {
   switch (ctState) {
     // CT-T3 Timeout
@@ -1493,10 +1497,10 @@ PBoolean H4504Handler::OnReceivedInvoke(int opcode,
 
     default:
       currentInvokeId = 0;
-      return PFalse;
+      return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1596,10 +1600,10 @@ PBoolean H4506Handler::OnReceivedInvoke(int opcode,
 
     default:
       currentInvokeId = 0;
-      return PFalse;
+      return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1646,8 +1650,8 @@ H4507Handler::H4507Handler(H323Connection & conn, H450xDispatcher & disp)
 
 PBoolean H4507Handler::OnReceivedInvoke(int opcode,
                                     int invokeId,
-                                    int linkedId,
-                                    PASN_OctetString *argument)
+                                    int /*linkedId*/,
+                                    PASN_OctetString * /*argument*/)
 {
   currentInvokeId = invokeId;
   PTRACE(3, "H450.7\tOnReceivedInvoke: invokeId = " << invokeId); 
@@ -1667,10 +1671,10 @@ PBoolean H4507Handler::OnReceivedInvoke(int opcode,
     default:
       PTRACE(2, "H450.7\tOnReceivedInvoke, not an interrogate");
       currentInvokeId = 0;
-      return PFalse;
+      return false;
   }
   
-  return PTrue;
+  return true;
 }
 
 
@@ -1751,7 +1755,7 @@ PBoolean H45011Handler::OnReceivedInvoke(int opcode,
                                     int linkedId,
                                     PASN_OctetString * argument)
 {
-  PBoolean result = PTrue;
+  PBoolean result = true;
   currentInvokeId = invokeId;
 
   switch (opcode) {
@@ -1797,7 +1801,7 @@ PBoolean H45011Handler::OnReceivedInvoke(int opcode,
     
     default:
       currentInvokeId = 0;
-      return PFalse;
+      return false;
   }
 
   return result;
@@ -2071,13 +2075,13 @@ void H45011Handler::OnReceivedCallIntrusionIsolate(int /*linkedId*/,
 PBoolean H45011Handler::OnReceivedCallIntrusionForcedRelease(int /*linkedId*/,
                                                          PASN_OctetString *argument)
 {
-  PBoolean result = PTrue;
+  PBoolean result = true;
   PTRACE(4, "H450.11\tReceived ForcedRelease Invoke");
 
   H45011_CIFrcRelArg ciArg;
 
   if(!DecodeArguments(argument, ciArg, -1))
-    return PFalse;
+    return false;
 
   PStringList tokens = endpoint.GetAllConnections();
 
@@ -2091,10 +2095,10 @@ PBoolean H45011Handler::OnReceivedCallIntrusionForcedRelease(int /*linkedId*/,
               activeCallToken = conn->GetCallToken();
               intrudingCallToken = connection.GetCallToken();
               conn->GetRemoteCallIntrusionProtectionLevel(connection.GetCallToken(), (unsigned)ciArg.m_ciCapabilityLevel);
-              result = PTrue;
+              result = true;
               break;
             }
-            result = PFalse;
+            result = false;
           }
         }
       }
@@ -2226,7 +2230,7 @@ PBoolean H45011Handler::OnReceivedReturnResult(X880_ReturnResult & returnResult)
         break;
     }
   }
-  return PTrue;
+  return true;
 }
 
 
@@ -2295,7 +2299,7 @@ void H45011Handler::OnReceivedCIGetCIPLResult(X880_ReturnResult & returnResult)
 
 PBoolean H45011Handler::OnReceivedReturnError(int errorCode, X880_ReturnError &returnError)
 {
-  PBoolean result = PTrue;
+  PBoolean result = true;
   PTRACE(4, "H450.11\tReceived Return Error CODE=" <<errorCode << ", InvokeId=" <<returnError.m_invokeId.GetValue());
   if (currentInvokeId == returnError.m_invokeId.GetValue()) {
     switch (ciState) {
@@ -2315,7 +2319,7 @@ PBoolean H45011Handler::OnReceivedReturnError(int errorCode, X880_ReturnError &r
 
 PBoolean H45011Handler::OnReceivedInvokeReturnError(int errorCode, const bool timerExpiry)
 {
-  PBoolean result = PFalse;
+  PBoolean result = false;
   PTRACE(4, "H450.11\tOnReceivedInvokeReturnError CODE =" << errorCode);
   if (!timerExpiry) {
     // stop timer CI-T1
@@ -2332,14 +2336,14 @@ PBoolean H45011Handler::OnReceivedInvokeReturnError(int errorCode, const bool ti
   switch(errorCode){
     case H45011_CallIntrusionErrors::e_notBusy :
       PTRACE(4, "H450.11\tH45011_CallIntrusionErrors::e_notBusy");
-      result = PTrue;
+      result = true;
       break;
     case H45011_CallIntrusionErrors::e_temporarilyUnavailable :
       PTRACE(4, "H450.11\tH45011_CallIntrusionErrors::e_temporarilyUnavailable");
       break;
     case H45011_CallIntrusionErrors::e_notAuthorized :
       PTRACE(4, "H450.11\tH45011_CallIntrusionErrors::e_notAuthorized");
-      result = PTrue;
+      result = true;
       break;
     default:
       PTRACE(4, "H450.11\tH45011_CallIntrusionErrors::DEFAULT");
@@ -2381,7 +2385,7 @@ PBoolean H45011Handler::OnReceivedGetCIPLReturnError(int PTRACE_PARAM(errorCode)
   ciSendState = e_ci_sAttachToReleseComplete;
   ciReturnState = e_ci_rCallForceReleased;
       
-  return PFalse;
+  return false;
 }
 
 
@@ -2416,12 +2420,12 @@ PBoolean H45011Handler::GetRemoteCallIntrusionProtectionLevel(const PString & to
   serviceAPDU.BuildCallIntrusionGetCIPL(currentInvokeId);
 
   if (!serviceAPDU.WriteFacilityPDU(connection))
-    return PFalse;
+    return false;
 
   PTRACE(4, "H450.11\tStarting timer CI-T5");
   StartciTimer(connection.GetEndPoint().GetCallIntrusionT5());
   ciState = e_ci_GetCIPL;
-  return PTrue;
+  return true;
 }
 
 
@@ -2458,7 +2462,7 @@ void H45011Handler::StopciTimer()
 }
 
 
-void H45011Handler::OnCallIntrudeTimeOut(PTimer &, INT)
+void H45011Handler::OnCallIntrudeTimeOut(PTimer &, P_INT_PTR)
 {
   switch (ciState) {
     // CI-T1 Timeout
@@ -2517,7 +2521,7 @@ PBoolean H45011Handler::OnReceivedReject(int PTRACE_PARAM(problemType), int PTRA
       break;
   }
   ciState = e_ci_Idle;
-  return PTrue;
+  return true;
 };
 
 #endif // OPAL_H450
