@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20262 $
+ * $Revision: 28483 $
  * $Author: rjongbloed $
- * $Date: 2008-05-13 14:09:13 +0200 (mar 13 mai 2008) $
+ * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -134,12 +134,13 @@ Q931::Q931()
 {
   protocolDiscriminator = 8;  // Q931 always has 00001000
   messageType = NationalEscapeMsg;
-  fromDestination = PFalse;
+  fromDestination = false;
   callReference = 0;
 }
 
 
 Q931::Q931(const Q931 & other)
+  : PObject(other)
 {
   operator=(other);
 }
@@ -205,7 +206,7 @@ void Q931::BuildSetupAcknowledge(int callRef)
 {
   messageType = SetupAckMsg;
   callReference = callRef;
-  fromDestination = PTrue;
+  fromDestination = true;
   informationElements.RemoveAll();
 }
 
@@ -214,7 +215,7 @@ void Q931::BuildCallProceeding(int callRef)
 {
   messageType = CallProceedingMsg;
   callReference = callRef;
-  fromDestination = PTrue;
+  fromDestination = true;
   informationElements.RemoveAll();
 }
 
@@ -223,7 +224,7 @@ void Q931::BuildAlerting(int callRef)
 {
   messageType = AlertingMsg;
   callReference = callRef;
-  fromDestination = PTrue;
+  fromDestination = true;
   informationElements.RemoveAll();
 }
 
@@ -235,7 +236,7 @@ void Q931::BuildSetup(int callRef)
     callReference = GenerateCallReference();
   else
     callReference = callRef;
-  fromDestination = PFalse;
+  fromDestination = false;
   informationElements.RemoveAll();
   SetBearerCapabilities(TransferSpeech, 1);
 }
@@ -245,7 +246,7 @@ void Q931::BuildConnect(int callRef)
 {
   messageType = ConnectMsg;
   callReference = callRef;
-  fromDestination = PTrue;
+  fromDestination = true;
   informationElements.RemoveAll();
   SetBearerCapabilities(TransferSpeech, 1);
 }
@@ -287,12 +288,12 @@ PBoolean Q931::Decode(const PBYTEArray & data)
   informationElements.RemoveAll();
 
   if (data.GetSize() < 5) // Packet too short
-    return PFalse;
+    return false;
 
   protocolDiscriminator = data[0];
 
   if (data[1] != 2) // Call reference must be 2 bytes long
-    return PFalse;
+    return false;
 
   callReference = ((data[2]&0x7f) << 8) | data[3];
   fromDestination = (data[2]&0x80) != 0;
@@ -321,14 +322,14 @@ PBoolean Q931::Decode(const PBYTEArray & data)
 
 	// before decrementing the length, make sure it is not zero
 	if (len == 0)
-          return PFalse;
+          return false;
 
         // adjust for protocol discriminator
         len--;
       }
 
       if (offset + len > data.GetSize())
-        return PFalse;
+        return false;
 
       SetIE(discriminator, PBYTEArray((const BYTE *)data+offset, len), TRUE);
 
@@ -336,7 +337,7 @@ PBoolean Q931::Decode(const PBYTEArray & data)
     }
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -357,7 +358,7 @@ PBoolean Q931::Encode(PBYTEArray & data) const
   }
 
   if (!data.SetMinSize(totalBytes))
-    return PFalse;
+    return false;
 
   // Put in Q931 header
   PAssert(protocolDiscriminator < 256, PInvalidParameter);
@@ -409,7 +410,7 @@ PBoolean Q931::Encode(PBYTEArray & data) const
 
 void Q931::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision() + 2;
+  std::streamsize indent = strm.precision() + 2;
   ios::fmtflags flags = strm.flags();
 
   strm << "{\n"
@@ -521,21 +522,13 @@ PString Q931::GetMessageTypeName() const
 
 unsigned Q931::GenerateCallReference()
 {
-  static unsigned LastCallReference;
-  static PMutex mutex;
-  PWaitAndSignal wait(mutex);
+  static PAtomicInteger lastCallReference;
 
-  if (LastCallReference == 0)
-    LastCallReference = PRandom::Number();
-  else
-    LastCallReference++;
+  unsigned ref = ++lastCallReference & 0x7fff;
+  if (ref == 0)
+    ref = ++lastCallReference & 0x7fff;
 
-  LastCallReference &= 0x7fff;
-
-  if (LastCallReference == 0)
-    LastCallReference = 1;
-
-  return LastCallReference;
+  return ref;
 }
 
 
@@ -571,6 +564,46 @@ void Q931::RemoveIE(InformationElementCodes ie)
 }
 
 
+void Q931::SetBearerCapabilities(const PString & str)
+{
+  InformationTransferCapability capability = TransferSpeech;
+  unsigned transferRate = 1; // 64kbps
+  unsigned codingStandard = 0; // ITU-T standardized coding
+  unsigned userInfoLayer1 = 5; // Recommendations H.221 and H.242
+
+  if (str.FindSpan("0123456789ABCDEFabcdef") == P_MAX_INDEX) {
+    PINDEX byteCount = str.GetLength()/2;
+    if (byteCount >= 2) {
+      PBYTEArray data(byteCount);
+      for (PINDEX i = 0; i < byteCount; ++i)
+        data[i] = (BYTE)str.Mid(i*2, 2).AsUnsigned(16);
+      SetIE(BearerCapabilityIE, data);
+      return;
+    }
+  }
+
+  PStringArray caps = str.Tokenise(',');
+  switch (caps.GetSize()) {
+    default :
+      userInfoLayer1 = caps[3].AsUnsigned();
+    case 3 :
+      codingStandard = caps[2].AsUnsigned();
+    case 2 :
+      transferRate = caps[1].AsUnsigned();
+    case 1 :
+      if (caps[0] *= "speech")
+        capability = TransferSpeech;
+      else if (caps[0] *= "digital")
+        capability = TransferUnrestrictedDigital;
+      else
+        capability = (Q931::InformationTransferCapability)caps[0].AsUnsigned();
+    case 0 :
+      break;
+  }
+  SetBearerCapabilities(capability, transferRate, codingStandard, userInfoLayer1);
+}
+
+
 void Q931::SetBearerCapabilities(InformationTransferCapability capability,
                                  unsigned transferRate,
                                  unsigned codingStandard,
@@ -631,13 +664,13 @@ PBoolean Q931::GetBearerCapabilities(InformationTransferCapability & capability,
                                  unsigned * userInfoLayer1)
 {
   if (!HasIE(BearerCapabilityIE))
-    return PFalse;
+    return false;
 
   PBYTEArray data = GetIE(BearerCapabilityIE);
   if (data.GetSize() < 2)
-    return PFalse;
+    return false;
 
-  capability = (InformationTransferCapability)data[0];
+  capability = (InformationTransferCapability)(data[0]&0x1f);
   if (codingStandard != NULL)
     *codingStandard = (data[0] >> 5)&3;
 
@@ -660,18 +693,18 @@ PBoolean Q931::GetBearerCapabilities(InformationTransferCapability & capability,
       break;
     case 0x18 :
       if (data.GetSize() < 3)
-        return PFalse;
+        return false;
       transferRate = data[2]&0x7f;
       nextByte = 3;
       break;
     default :
-      return PFalse;
+      return false;
   }
 
   if (userInfoLayer1 != NULL)
     *userInfoLayer1 = data.GetSize() >= nextByte && ((data[nextByte]>>5)&3) == 1 ? (data[nextByte]&0x1f) : 0;
 
-  return PTrue;
+  return true;
 }
 
 
@@ -774,7 +807,7 @@ PString Q931::GetKeypad() const
   if (digits.IsEmpty())
     return PString();
 
-  return PString((const char *)(const BYTE *)digits, digits.GetSize());
+  return PString(digits);
 }
 
 
@@ -794,11 +827,11 @@ PBoolean Q931::GetProgressIndicator(unsigned & description,
                                 unsigned * location) const
 {
   if (!HasIE(ProgressIndicatorIE))
-    return PFalse;
+    return false;
 
   PBYTEArray data = GetIE(ProgressIndicatorIE);
   if (data.GetSize() < 2)
-    return PFalse;
+    return false;
 
   if (codingStandard != NULL)
     *codingStandard = (data[0]>>5)&0x03;
@@ -806,12 +839,15 @@ PBoolean Q931::GetProgressIndicator(unsigned & description,
     *location = data[0]&0x0f;
   description = data[1]&0x7f;
 
-  return PTrue;
+  return true;
 }
 
 
 void Q931::SetDisplayName(const PString & name)
 {
+  if (name.IsEmpty())
+    return;
+
   PBYTEArray bytes((const BYTE *)(const char *)name, name.GetLength()+1);
   SetIE(DisplayIE, bytes);
 }
@@ -826,7 +862,7 @@ PString Q931::GetDisplayName() const
   if (display.IsEmpty())
     return PString();
 
-  return PString((const char *)(const BYTE *)display, display.GetSize());
+  return PString(display);
 }
 
 
@@ -889,7 +925,7 @@ static PBoolean GetNumberIE(const PBYTEArray & bytes,
   number = PString();
 
   if (bytes.IsEmpty())
-    return PFalse;
+    return false;
 
   if (plan != NULL)
     *plan = bytes[0]&15;
@@ -909,7 +945,7 @@ static PBoolean GetNumberIE(const PBYTEArray & bytes,
   }
   else {
     if (bytes.GetSize() < 2)
-      return PFalse;
+      return false;
 
     if (presentation != NULL)
       *presentation = (bytes[1]>>5)&3;
@@ -925,7 +961,7 @@ static PBoolean GetNumberIE(const PBYTEArray & bytes,
     }
     else {
       if (bytes.GetSize() < 3)
-        return PFalse;
+        return false;
 
       if (reason != NULL)
         *reason = bytes[2]&15;
@@ -935,12 +971,12 @@ static PBoolean GetNumberIE(const PBYTEArray & bytes,
   }
 
   if (bytes.GetSize() < offset)
-    return PFalse;
+    return false;
 
   PINDEX len = bytes.GetSize()-offset;
 
   if (len > 0)
-    memcpy(number.GetPointer(len+1), ((const BYTE *)bytes)+offset, len);
+    memcpy(number.GetPointerAndSetLength(len), ((const BYTE *)bytes)+offset, len);
 
   return !number;
 }
@@ -1093,11 +1129,11 @@ PBoolean Q931::GetChannelIdentification(unsigned * interfaceType,
                                     int      * channelNumber) const
 {
   if (!HasIE(ChannelIdentificationIE))
-    return PFalse;
+    return false;
 
   PBYTEArray bytes = GetIE(ChannelIdentificationIE);
   if (bytes.GetSize() < 1)
-    return PFalse;
+    return false;
 
   *interfaceType        = (bytes[0]>>5) & 0x01;
   *preferredOrExclusive = (bytes[0]>>3) & 0x01;
@@ -1126,17 +1162,17 @@ PBoolean Q931::GetChannelIdentification(unsigned * interfaceType,
       }
       else { // B Channel
         if (bytes.GetSize() < 3)
-          return PFalse;
+          return false;
 
         if (bytes[1] != 0x83)
-          return PFalse;
+          return false;
 
         *channelNumber = bytes[2] & 0x7f;
       }
     }
   }
 
-  return PTrue;
+  return true;
 }
 
 
