@@ -24,11 +24,13 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 23632 $
+ * $Revision: 28937 $
  * $Author: csoutheren $
- * $Date: 2009-10-09 01:02:18 -0500 (Fri, 09 Oct 2009) $
+ * $Date: 2013-01-18 07:21:17 -0600 (Fri, 18 Jan 2013) $
  */
 
+
+
 #include <ptlib.h>
 #include <opal/buildopts.h>
 
@@ -66,6 +68,8 @@
  *
  ********************************************************************************/
 
+
+
 IAX2CallProcessor::IAX2CallProcessor(IAX2EndPoint &ep)
   : IAX2Processor(ep)
 {
@@ -75,23 +79,23 @@ IAX2CallProcessor::IAX2CallProcessor(IAX2EndPoint &ep)
   
   selectedCodec = 0;
   
-  audioCanFlow = PFalse;
-  audioFramesNotStarted = PTrue;
-
+  audioCanFlow = false;
+  audioFramesNotStarted = true;
+  suppressHangupFrame = false;
   con = NULL;
 
-  firstMediaFrame = PTrue;
-  answerCallNow = PFalse;
+  firstMediaFrame = true;
+  answerCallNow = false;
   audioFrameDuration = 0;
   audioCompressedBytes = 0;
   
-  holdCall = PFalse;
-  holdReleaseCall = PFalse;
+  holdCall = false;
+  holdReleaseCall = false;
   
-  doTransfer = PFalse;
+  doTransfer = false;
   
   statusCheckTimer.SetNotifier(PCREATE_NOTIFIER(OnStatusCheck));
-  statusCheckOtherEnd = PFalse;
+  statusCheckOtherEnd = false;
   
   soundBufferState = BufferToSmall;
   callStartTick = PTimer::Tick();
@@ -120,7 +124,7 @@ void IAX2CallProcessor::AssignConnection(IAX2Connection * _con)
 
 void IAX2CallProcessor::PrintOn(ostream & strm) const
 {
-  strm << "In call with " << con->GetRemotePartyAddress() << "  " << remotePhoneNumber << " " << callToken  << endl
+  strm << "In call with " << con->GetRemotePartyURL() << "  " << remotePhoneNumber << " " << callToken  << endl
        << "  Call has been up for " << setprecision(0) << setw(8)
        << (PTimer::Tick() - callStartTick) << " milliseconds" << endl
        << "  Control frames sent " << controlFramesSent    << endl
@@ -131,13 +135,6 @@ void IAX2CallProcessor::PrintOn(ostream & strm) const
        << "  Video frames rcvd   " << videoFramesRcvd      << endl;
 }
 
-void IAX2CallProcessor::Release(OpalConnection::CallEndReason reason)
-{
-  PTRACE(3, "Processor\tRelease(" << reason << ")");
-  PStringStream str;
-  str << reason;
-  Hangup(str);
-}
 
 void IAX2CallProcessor::ClearCall(OpalConnection::CallEndReason reason)
 {
@@ -205,6 +202,11 @@ void IAX2CallProcessor::CheckForHangupMessages()
   if (hangList.IsEmpty()) 
     return;
 
+  if (suppressHangupFrame == true) {
+    Terminate();
+    return;
+  }
+
   if (!IsCallTerminating()) {
     IAX2FullFrameProtocol * f = 
       new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdHangup, IAX2FullFrame::callIrrelevant);
@@ -224,7 +226,7 @@ void IAX2CallProcessor::CheckForHangupMessages()
 void IAX2CallProcessor::ConnectToRemoteNode(PString & newRemoteNode)
 {      // Parse a string like guest@node.name.com/23234
   
-  PTRACE(2, "Connect to remote node " << newRemoteNode);
+  PTRACE(2, "CallProc\tConnect to remote node " << newRemoteNode);
   PStringArray res = IAX2EndPoint::DissectRemoteParty(newRemoteNode);
 
   if (res[IAX2EndPoint::addressIndex].IsEmpty()) {
@@ -245,11 +247,41 @@ void IAX2CallProcessor::ConnectToRemoteNode(PString & newRemoteNode)
     return;
   }
   PTRACE(4, "Resolve " << res[IAX2EndPoint::addressIndex]  << " as ip address " << ip);
-    
-  remote.SetRemotePort(con->GetEndPoint().ListenPortNumber());
+
+  if (!res[IAX2EndPoint::portIndex].IsEmpty())
+	  remote.SetRemotePort(res[IAX2EndPoint::portIndex].AsUnsigned());
+  else
+	  remote.SetRemotePort(con->GetEndPoint().ListenPortNumber());
   remote.SetRemoteAddress(ip);
-    
-  IAX2FullFrameProtocol * f = new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdNew);
+
+  if (res[IAX2EndPoint::userIndex].IsEmpty()) {
+    callingName = userName;
+  } else {
+    callingName = res[IAX2EndPoint::userIndex];
+    userName = callingName;
+  }
+
+  callingExtension = res[IAX2EndPoint::extensionIndex];
+  callingDnid = res[IAX2EndPoint::extensionIndex];
+  callingContext = res[IAX2EndPoint::contextIndex];
+
+  IAX2FullFrameProtocol * f = BuildNewFrameForSending();
+#if P_SSL
+  f->AppendIe(new IAX2IeCallToken());
+#endif
+  TransmitFrameToRemoteEndpoint(f);
+  StartNoResponseTimer();
+  return;
+}
+
+
+IAX2FullFrameProtocol *IAX2CallProcessor::BuildNewFrameForSending(IAX2FullFrameProtocol *inReplyTo)
+{
+  IAX2FullFrameProtocol * f;
+  if (inReplyTo == NULL)
+    f = new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdNew);
+  else
+    f = new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdNew, inReplyTo);
   f->AppendIe(new IAX2IeVersion());
   f->AppendIe(new IAX2IeFormat(con->GetPreferredCodec()));
   f->AppendIe(new IAX2IeCapability(con->GetSupportedCodecs()));
@@ -257,32 +289,26 @@ void IAX2CallProcessor::ConnectToRemoteNode(PString & newRemoteNode)
   if (!endpoint.GetLocalNumber().IsEmpty())
     f->AppendIe(new IAX2IeCallingNumber(endpoint.GetLocalNumber()));
 
-  if (!res[IAX2EndPoint::userIndex].IsEmpty()) {
-    f->AppendIe(new IAX2IeCallingName(res[IAX2EndPoint::userIndex]));
-    f->AppendIe(new IAX2IeUserName(res[IAX2EndPoint::userIndex]));
-  } else if (!GetUserName().IsEmpty()) {
-    f->AppendIe(new IAX2IeCallingName(GetUserName()));
-    f->AppendIe(new IAX2IeUserName(GetUserName()));
-  }
+  f->AppendIe(new IAX2IeCallingName(GetCallingName()));
+  f->AppendIe(new IAX2IeUserName(GetUserName()));
 
-  if (!res[IAX2EndPoint::extensionIndex].IsEmpty())
-    f->AppendIe(new IAX2IeCalledNumber(res[IAX2EndPoint::extensionIndex] ));
+  if (!callingExtension.IsEmpty())
+    f->AppendIe(new IAX2IeCalledNumber(callingExtension));
 
-  if (!res[IAX2EndPoint::extensionIndex].IsEmpty())
-    f->AppendIe(new IAX2IeDnid(res[IAX2EndPoint::extensionIndex]));
+  if (!callingDnid.IsEmpty())
+    f->AppendIe(new IAX2IeDnid(callingDnid));
 
-  if (!res[IAX2EndPoint::contextIndex].IsEmpty())
-    f->AppendIe(new IAX2IeCalledContext(res[IAX2EndPoint::contextIndex]));
+  if (!callingContext.IsEmpty())
+    f->AppendIe(new IAX2IeCalledContext(callingContext));
 
-#if OPAL_PTLIB_SSL_AES
+#ifdef P_SSL_AES
   f->AppendIe(new IAX2IeEncryption());
 #endif
 
-  TransmitFrameToRemoteEndpoint(f);
-  StartNoResponseTimer();
-  return;
+  return f;
 }
 
+
 void IAX2CallProcessor::ReportStatistics()
 {
 #if 0
@@ -402,6 +428,7 @@ void IAX2CallProcessor::ProcessLists()
   CheckForHangupMessages();
 }
 
+
 void IAX2CallProcessor::ProcessIncomingAudioFrame(IAX2Frame *newFrame)
 {
   PTRACE(5, "Processor\tProcessIncomingAudioframe " << newFrame->IdString());
@@ -418,7 +445,6 @@ void IAX2CallProcessor::ProcessIncomingVideoFrame(IAX2Frame *newFrame)
   delete newFrame;
 }
 
-
 void IAX2CallProcessor::SendSoundMessage(PBYTEArray *sound)
 {
   if (sound == NULL)
@@ -441,12 +467,12 @@ void IAX2CallProcessor::SendSoundMessage(PBYTEArray *sound)
   currentSoundTimeStamp = thisTimeStamp;
 
   if (sendFullFrame) {
-    audioFramesNotStarted = PFalse;
+    audioFramesNotStarted = false;
     IAX2FullFrameVoice *f = new IAX2FullFrameVoice(this, *sound, thisTimeStamp);
     PTRACE(5, "Send a full audio frame" << thisTimeStamp << " On " << f->IdString());
     TransmitFrameToRemoteEndpoint(f);
   } else {
-    IAX2MiniFrame *f = new IAX2MiniFrame(this, *sound, PTrue, thisTimeStamp & 0xffff);
+    IAX2MiniFrame *f = new IAX2MiniFrame(this, *sound, true, thisTimeStamp & 0xffff);
     TransmitFrameToRemoteEndpoint(f);
   }
   
@@ -469,12 +495,12 @@ void IAX2CallProcessor::SendText(const PString & text)
 
 void IAX2CallProcessor::SendHold()
 {
-  holdCall = PTrue;
+  holdCall = true;
 }
 
 void IAX2CallProcessor::SendHoldRelease()
 {
-  holdReleaseCall = PTrue;
+  holdReleaseCall = true;
 }
 
 void IAX2CallProcessor::SendTransfer(
@@ -483,7 +509,7 @@ void IAX2CallProcessor::SendTransfer(
 {
   {
     PWaitAndSignal m(transferMutex);
-    doTransfer = PTrue;
+    doTransfer = true;
     transferCalledNumber = calledNumber;
     transferCalledContext = calledContext;
   }
@@ -509,7 +535,7 @@ void IAX2CallProcessor::SendTransferMessage()
       f->AppendIe(new IAX2IeCalledContext(transferCalledContext));
     TransmitFrameToRemoteEndpoint(f);
     
-    doTransfer = PFalse;
+    doTransfer = false;
   }
 }
 
@@ -522,7 +548,7 @@ void IAX2CallProcessor::SendTextMessage(PString & message)
 
 void IAX2CallProcessor::SendQuelchMessage()
 {
-  holdCall = PFalse;
+  holdCall = false;
   
   IAX2FullFrameProtocol *f = new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdQuelch);
   //we'll request that music is played to them because everyone likes music when they're on hold ;)
@@ -532,12 +558,14 @@ void IAX2CallProcessor::SendQuelchMessage()
 
 void IAX2CallProcessor::SendUnQuelchMessage()
 {
-  holdReleaseCall = PFalse;
+  holdReleaseCall = false;
   
   IAX2FullFrameProtocol *f = new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdUnquelch); 
   TransmitFrameToRemoteEndpoint(f);
 }
 
+
+
 void IAX2CallProcessor::ProcessNetworkFrame(IAX2MiniFrame * src)
 {
   src->AlterTimeStamp(lastFullFrameTimeStamp);
@@ -592,7 +620,7 @@ void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameVoice * src)
 {
   if (firstMediaFrame) {
     PTRACE(5, "Processor\tReceived first voice media frame " << src->IdString());
-    firstMediaFrame = PFalse;
+    firstMediaFrame = false;
   }
   PTRACE(5, "ProcessNetworkFrame(IAX2FullFrameVoice * src)" << src->IdString());
   SendAckFrame(src);
@@ -605,7 +633,7 @@ void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameVideo * src)
 {
   if (firstMediaFrame) {
     PTRACE(5, "Processor\tReceived first video media frame ");
-    firstMediaFrame = PFalse;
+    firstMediaFrame = false;
   }
 
   PTRACE(5, "ProcessNetworkFrame(IAX2FullFrameVideo * src)");
@@ -615,16 +643,20 @@ void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameVideo * src)
   return;
 }
 
+
+
 void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameSessionControl * src)
-{ /* these frames are labelled as AST_FRAME_CONTROL in the asterisk souces.
-     We could get an Answer message from here., or a hangup., or...congestion control... */
+{
+ /* these frames are labelled as AST_FRAME_CONTROL in the asterisk
+     souces.  We could get an Answer message from here., or a hangup.,
+     or...congestion control... */
   
   PTRACE(4, "ProcessNetworkFrame(IAX2FullFrameSessionControl * src)");
   SendAckFrame(src);  
   
   switch(src->GetSubClass()) {
   case IAX2FullFrameSessionControl::hangup:          // Other end has hungup
-    SetCallTerminating(PTrue);
+    SetCallTerminating(true);
     //cout << "Other end has hungup, so exit" << endl;
     con->EndCallNow();
     break;
@@ -698,16 +730,17 @@ void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameSessionControl * src)
   return;
 }
 
+
 PBoolean IAX2CallProcessor::SetUpConnection()
 {
   PTRACE(3, "IAX2\tSet Up Connection to remote node " 
 	 << con->GetRemotePartyName() << " " 
-	 << con->GetRemotePartyAddress());
+	 << con->GetRemotePartyURL());
    
-  callList.AppendString(con->GetRemotePartyAddress());
+  callList.AppendString(con->GetRemotePartyURL());
   
   CleanPendingLists();
-  return PTrue;
+  return true;
 }
 
 void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameNull * src)
@@ -728,7 +761,7 @@ PBoolean IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameProtocol * src)
   
   //check if the common method can process it?
   if (IAX2Processor::ProcessNetworkFrame(src)) {
-    return PTrue;
+    return true;
   }
 
   switch(src->GetSubClass()) {
@@ -765,7 +798,8 @@ PBoolean IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameProtocol * src)
   case IAX2FullFrameProtocol::cmdDial:
     ProcessIaxCmdDial(src);
     break;
-  /*case IAX2FullFrameProtocol::cmdTxreq:
+#if DONTUSE
+  case IAX2FullFrameProtocol::cmdTxreq:
     ProcessIaxCmdTxreq(src);
     break;
   case IAX2FullFrameProtocol::cmdTxcnt:
@@ -782,7 +816,7 @@ PBoolean IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameProtocol * src)
     break;
   case IAX2FullFrameProtocol::cmdTxrej:
     ProcessIaxCmdTxrej(src);
-    break;*/
+    break;
   case IAX2FullFrameProtocol::cmdQuelch:
     ProcessIaxCmdQuelch(src);
     break;
@@ -791,7 +825,7 @@ PBoolean IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameProtocol * src)
     break;
   case IAX2FullFrameProtocol::cmdPage:
     ProcessIaxCmdPage(src);
-    break;/*
+    break;
   case IAX2FullFrameProtocol::cmdMwi:
     ProcessIaxCmdMwi(src);
     break;
@@ -809,15 +843,22 @@ PBoolean IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameProtocol * src)
     break;
   case IAX2FullFrameProtocol::cmdFwData:
     ProcessIaxCmdFwData(src);
-    break;*/
+    break;
+#endif
+  case IAX2FullFrameProtocol::cmdCallToken:
+    ProcessIaxCmdCallToken(src);
+    break;
   default:
     PTRACE(1, "Process Full Frame Protocol, Type not expected");
     SendUnsupportedFrame(src);   //This method will delete the frame src.
   };
   
-  return PFalse;
+  return false;
 }
 
+
+
+
 void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameText * src)
 {
   PTRACE(4, "ProcessNetworkFrame(IAX2FullFrameText * src)");
@@ -845,7 +886,7 @@ void IAX2CallProcessor::ProcessNetworkFrame(IAX2FullFrameCng * src)
   delete src;
   return;
 }
-
+  
 void IAX2CallProcessor::CheckForRemoteCapabilities(IAX2FullFrameProtocol *src)
 {
   unsigned int remoteCapability, format;
@@ -859,6 +900,8 @@ void IAX2CallProcessor::CheckForRemoteCapabilities(IAX2FullFrameProtocol *src)
   con->BuildRemoteCapabilityTable(remoteCapability, format);
 }
 
+
+  
 PBoolean IAX2CallProcessor::RemoteSelectedCodecOk()
 {
   selectedCodec = con->ChooseCodec();
@@ -870,11 +913,14 @@ PBoolean IAX2CallProcessor::RemoteSelectedCodecOk()
     reply->AppendIe(new IAX2IeCauseCode(IAX2IeCauseCode::BearerCapabilityNotAvail));
     TransmitFrameToRemoteEndpoint(reply);
     con->ClearCall(OpalConnection::EndedByCapabilityExchange);
-    return PFalse;
+    return false;
   }
   
-  return PTrue;
+  return true;
 }
+  
+
+ 
 
 void IAX2CallProcessor::ProcessIaxCmdNew(IAX2FullFrameProtocol *src)
 { /*That we are here indicates this connection is already in place */
@@ -882,6 +928,32 @@ void IAX2CallProcessor::ProcessIaxCmdNew(IAX2FullFrameProtocol *src)
   remote.SetRemoteAddress(src->GetRemoteInfo().RemoteAddress());
   remote.SetRemotePort(src->GetRemoteInfo().RemotePort());
 
+  PIPSocket::Address addr(remote.RemoteAddress());
+  IAX2IeCallToken token;
+  if (src->GetCallTokenIe(token)) {
+    PTRACE(3, "CallProc\tReceived a callToken");
+    if (token.GetLengthOfData() < 5) {
+      IAX2IeCallToken *tSend = new IAX2IeCallToken();
+      tSend->WriteKeySequence(addr);
+      IAX2FullFrameProtocol * reply = 
+	 new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdCallToken, src);
+      reply->AppendIe(tSend);
+      TransmitFrameToRemoteEndpoint(reply);
+      delete src;
+      return;
+    } else {
+      bool validCallToken = IAX2IeCallToken::ValidKeySequence(token, addr);
+      if (!validCallToken) {
+	PTRACE(3, "CallProc\tfailed CALLTOKEN test");
+	IAX2FullFrameProtocol * reply = 
+	  new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdReject, src);
+	TransmitFrameToRemoteEndpoint(reply);
+	delete src;
+	return;
+      }
+      PTRACE(3, "CallProc\tValid CALLTOKEN test");
+    }
+  }
   if (IsCallHappening()) {
     PTRACE(3, "Remote node has sent us a second new message. ignore");
     delete src;
@@ -889,15 +961,14 @@ void IAX2CallProcessor::ProcessIaxCmdNew(IAX2FullFrameProtocol *src)
   }
 
   IAX2FullFrameProtocol * reply;
-  
-  
+    
   if (!RemoteSelectedCodecOk()) {
-    PTRACE(3, "Remote node sected a bad codec, hangup call ");
+    PTRACE(3, "CallProc\tRemote node sected a bad codec, hangup call ");
     reply = new  IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdInval, src, IAX2FullFrame::callIrrelevant);
     TransmitFrameToRemoteEndpoint(reply);
     
     reply= new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdHangup, IAX2FullFrame::callIrrelevant);
-    PTRACE(3, "Send a hangup frame to the remote endpoint as there is no codec available");    
+    PTRACE(3, "CallProc\tSend a hangup frame to the remote endpoint as there is no codec available");    
     reply->AppendIe(new IAX2IeCause("No matching codec"));
     SetCallTerminating();
     //              f->AppendIe(new IeCauseCode(IeCauseCode::NormalClearing));
@@ -936,6 +1007,7 @@ void IAX2CallProcessor::ProcessIaxCmdNew(IAX2FullFrameProtocol *src)
   delete src;
 }
 
+
 void IAX2CallProcessor::ProcessIaxCmdAck(IAX2FullFrameProtocol *src)
 {
   PTRACE(4, "ProcessIaxCmdAck(IAX2FullFrameProtocol * /*src*/)");
@@ -968,7 +1040,7 @@ void IAX2CallProcessor::ProcessIaxCmdAck(IAX2FullFrameProtocol *src)
 
 void IAX2CallProcessor::RingingWasAcked()
 {
-  PTRACE(4, "Processor\t Remote node " << con->GetRemotePartyAddress() << " knows our phone is ringing");
+  PTRACE(4, "Processor\t Remote node " << con->GetRemotePartyURL() << " knows our phone is ringing");
 }
 
 void IAX2CallProcessor::AnswerWasAcked()
@@ -980,13 +1052,13 @@ void IAX2CallProcessor::AcceptIncomingCall()
 {
   PTRACE(4, "AcceptIncomingCall()");
 
-  answerCallNow = PTrue;
+  answerCallNow = true;
   CleanPendingLists();
 }
 
 void IAX2CallProcessor::SendAnswerMessageToRemoteNode()
 {
-  answerCallNow = PFalse;
+  answerCallNow = false;
   if (!IsCallAnswered()) {
     SetCallAnswered(true);
     PTRACE(4, "Processor\tSend Answer message");
@@ -999,18 +1071,18 @@ void IAX2CallProcessor::SendAnswerMessageToRemoteNode()
 PBoolean IAX2CallProcessor::SetAlerting(const PString & PTRACE_PARAM(calleeName), PBoolean /*withMedia*/)
 {
   PTRACE(3, "Processor\tSetAlerting from " << calleeName);
-  return PTrue;
+  return true;
 }
 
 /*The remote node has told us to hangup and go away */
 void IAX2CallProcessor::ProcessIaxCmdHangup(IAX2FullFrameProtocol *src)
 { 
-  SetCallTerminating(PTrue);
+  SetCallTerminating(true);
 
   PTRACE(3, "Processor\tProcessIaxCmdHangup(IAX2FullFrameProtocol *src)");
   SendAckFrame(src);
 
-  PTRACE(1, "The remote node (" << con->GetRemotePartyAddress()  << ") has closed the call");
+  PTRACE(1, "The remote node (" << con->GetRemotePartyURL()  << ") has closed the call");
 
   con->EndCallNow(OpalConnection::EndedByRemoteUser);
 
@@ -1019,6 +1091,7 @@ void IAX2CallProcessor::ProcessIaxCmdHangup(IAX2FullFrameProtocol *src)
 
 void IAX2CallProcessor::ProcessIaxCmdReject(IAX2FullFrameProtocol *src)
 {
+  suppressHangupFrame = true;
   PTRACE(3, "Processor\tProcessIaxCmdReject(IAX2FullFrameProtocol *src)");
   //cout << "Remote endpoint has rejected our call " << endl;
   //cout << "Cause \"" << ieData.cause << "\"" << endl;
@@ -1031,7 +1104,7 @@ void IAX2CallProcessor::ProcessIaxCmdReject(IAX2FullFrameProtocol *src)
 void IAX2CallProcessor::ProcessIaxCmdAccept(IAX2FullFrameProtocol *src)
 {
   if (IsCallAccepted()) {
-    PTRACE(3, "Second accept packet received. Ack, delete, & do nothing.");
+    PTRACE(3, "Processor\t2nd accept packet received. ack+del+ignore.");
     SendAckFrame(src);
     delete src;
     return;
@@ -1042,8 +1115,7 @@ void IAX2CallProcessor::ProcessIaxCmdAccept(IAX2FullFrameProtocol *src)
   /* Other end has acknowledged our request to make a call.
      Tell our connection that the phone is ringing at the other end. */
   PString calleeName = con->GetRemotePartyName();
-  con->SetAlerting(calleeName, PTrue);
-
+  con->SetAlerting(calleeName, true);
 
   SendAckFrame(src);
   SetCallAccepted();
@@ -1052,7 +1124,7 @@ void IAX2CallProcessor::ProcessIaxCmdAccept(IAX2FullFrameProtocol *src)
   
   if (!RemoteSelectedCodecOk()) {
     PTRACE(3, "Remote node sected a bad codec, hangup call ");
-    Release();
+    Hangup("bad codec");
     return;
   }
   PString codecName = IAX2FullFrameVoice::GetOpalNameOfCodec((unsigned short)selectedCodec);
@@ -1217,16 +1289,33 @@ void IAX2CallProcessor::ProcessIaxCmdFwData(IAX2FullFrameProtocol *src)
   delete src;
 }
 
+void IAX2CallProcessor::ProcessIaxCmdCallToken(IAX2FullFrameProtocol *src)
+{
+  PTRACE(4, "Processor\tProcessIaxCmdCallToken(IAX2FullFrameProtocol *src)");
+  IAX2IeCallToken *ie = new IAX2IeCallToken();
+  if (src->GetCallTokenIe(*ie)) {
+    IAX2FullFrameProtocol *f = BuildNewFrameForSending();
+    f->AppendIe(ie);
+    GetSequenceInfo().ZeroAllValues();
+    TransmitFrameToRemoteEndpoint(f);
+    StopNoResponseTimer();
+    StartNoResponseTimer();
+  } else
+    delete ie;
+  
+  delete src;
+}
+
 void IAX2CallProcessor::StartStatusCheckTimer(PINDEX msToWait)
 {
   PTRACE(4, "Processor\tStatusCheck time. Now set flag to send a ping+lagrq packets");
   PTRACE(4, "Processor\tStatusCheck timer set to " << msToWait << "  ms");
   statusCheckTimer = PTimeInterval(msToWait); 
-  statusCheckOtherEnd = PTrue;
+  statusCheckOtherEnd = true;
   CleanPendingLists();
 }
 
-void IAX2CallProcessor::OnStatusCheck(PTimer &, INT)
+void IAX2CallProcessor::OnStatusCheck(PTimer &, P_INT_PTR)
 {
   StartStatusCheckTimer();
 }
@@ -1240,7 +1329,7 @@ void IAX2CallProcessor::RemoteNodeIsRinging()
 
 void IAX2CallProcessor::DoStatusCheck()
 {
-  statusCheckOtherEnd = PFalse;
+  statusCheckOtherEnd = false;
   if (IsCallTerminating())
     return;
 
@@ -1281,29 +1370,27 @@ PBoolean IAX2CallProcessor::IncomingMessageOutOfOrder(IAX2FullFrame *f)
 	  PTRACE(4, "Skipped frame, received frame is " << f->GetSequenceInfo().AsString());
 	  SendVnakFrame(f);
 	  delete f;
-	  return PTrue;
+	  return true;
       } 
 	  break;
       case IAX2SequenceNumbers::RepeatedFrame: {
 	  SendAckFrame(f);
 	  delete f;
-	  return PTrue;
+	  return true;
       }
   }
   
   /*The frame is ok, and the order is "good". so leave it alone for later processing */
-  return PFalse;
+  return false;
 }
 
-
 #endif // OPAL_IAX2
 
 ////////////////////////////////////////////////////////////////////////////////
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 2 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file.
+ * With the comment below, the tab key does auto indent to 2 spaces.    
+ *
  * Local Variables:
  * mode:c
  * c-basic-offset:2
