@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 22710 $
- * $Author: dereksmithies $
- * $Date: 2009-05-24 22:17:57 -0500 (Sun, 24 May 2009) $
+ * $Revision: 28937 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 07:21:17 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -48,14 +48,6 @@
 
 #include <ptclib/cypher.h>
 
-#if OPAL_PTLIB_SSL_AES
-#include <openssl/aes.h>
-  #ifdef _MSC_VER
-    #pragma comment(lib, P_SSL_LIB1)
-    #pragma comment(lib, P_SSL_LIB2)
-  #endif
-#endif
-
 
 #define new PNEW
 
@@ -65,12 +57,12 @@ IAX2Frame::IAX2Frame(IAX2EndPoint &_endpoint)
 {
   ZeroAllValues();
 
-  PTRACE(6, "Construct IAX2Frame  " << IdString());
+  PTRACE(6, "Frame\tConstruct IAX2Frame  " << IdString());
 }
 
 IAX2Frame::~IAX2Frame()
 {
-  PTRACE(6, "Destructor for IAX2Frame  " << *this);
+  PTRACE(6, "Frame\tDestructor for IAX2Frame  " << *this);
 }
 
 PString IAX2Frame::IdString() const 
@@ -84,37 +76,28 @@ void IAX2Frame::ZeroAllValues()
 {
   data.SetSize(0);
   
-  isFullFrame       = PFalse;
-  isVideo           = PFalse;
-  isAudio           = PFalse;
+  isFullFrame       = false;
+  isVideo           = false;
+  isAudio           = false;
   
   currentReadIndex  = 0;
   currentWriteIndex = 0;
   timeStamp = 0;
   
-  canRetransmitFrame = PFalse;
+  canRetransmitFrame = false;
   presetTimeStamp = 0;
   
   frameType = undefType;
 }
 
-PBoolean IAX2Frame::IsVideo() const 
-{ 
-  return isVideo; 
-}
 
-PBoolean IAX2Frame::IsAudio() const
-{ 
-  return isAudio; 
-}
-
-
-PBoolean IAX2Frame::IsFullFrame()
+void IAX2Frame::SetTimeStamp(DWORD newValue)
 {
-  return isFullFrame;
+  timeStamp  = newValue;
+  presetTimeStamp = newValue;
+  PTRACE(5, "Frame\tPreset the timestamp to " << newValue);
 }
 
-
 PBoolean IAX2Frame::ReadNetworkPacket(PUDPSocket &sock)
 {
   data.SetSize(4096);  //Surely no packets > 4096 bytes in length
@@ -127,29 +110,29 @@ PBoolean IAX2Frame::ReadNetworkPacket(PUDPSocket &sock)
   remote.SetRemoteAddress(addr);
   remote.SetRemotePort(portNo);
   
-  if (res == PFalse) {
-    PTRACE(3, "Failed in reading from socket");
-    return PFalse;
+  if (res == false) {
+    PTRACE(3, "Frame\tFailed in reading from socket");
+    return false;
   }
   
   data.SetSize(sock.GetLastReadCount());
   
   if (data.GetSize() < 4) {
-    PTRACE(3, "Read a very very small packet from the network - < 4 bytes");
-    return PFalse;
+    PTRACE(3, "Frame\tRead a very very small packet from the network - < 4 bytes");
+    return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 PBoolean IAX2Frame::Read1Byte(BYTE & result)
 {
   if (currentReadIndex >= data.GetSize())
-    return PFalse;
+    return false;
   
   result = data[currentReadIndex];
   currentReadIndex++;
-  return PTrue;
+  return true;
 }
 
 PBoolean IAX2Frame::Read2Bytes(PINDEX & res)
@@ -158,10 +141,10 @@ PBoolean IAX2Frame::Read2Bytes(PINDEX & res)
   BYTE b = 0;
   if (Read1Byte(a) && Read1Byte(b)) {
     res = (a << 8) | b;
-    return PTrue;
+    return true;
   }
   
-  return PFalse;
+  return false;
 }
 
 PBoolean IAX2Frame::Read2Bytes(WORD & res)
@@ -169,10 +152,10 @@ PBoolean IAX2Frame::Read2Bytes(WORD & res)
   BYTE a = 0, b = 0;
   if (Read1Byte(a) && Read1Byte(b)) {
     res = (WORD)((a << 8) | b);
-    return PTrue;
+    return true;
   }
   
-  return PFalse;
+  return false;
 }
 
 PBoolean IAX2Frame::Read4Bytes(DWORD & res)
@@ -180,10 +163,10 @@ PBoolean IAX2Frame::Read4Bytes(DWORD & res)
   PINDEX a = 0, b = 0;
   if (Read2Bytes(a) && Read2Bytes(b)) {
     res = (a << 16) | b;
-    return PTrue;
+    return true;
   }
   
-  return PFalse;
+  return false;
 }
 
 void IAX2Frame::Write1Byte(PINDEX newVal)
@@ -224,33 +207,35 @@ PBoolean IAX2Frame::ProcessNetworkPacket()
   /*We are guaranteed to have a packet > 4 bytes in size */
   PINDEX a = 0;
   Read2Bytes(a);
-  remote.SetSourceCallNumber(a & 0x7fff);
+  PINDEX sourceCallNumber = a & 0x7fff;
+  remote.SetSourceCallNumber(sourceCallNumber);
 
-  if (a != 0)
-    BuildConnectionTokenId();
+  if (sourceCallNumber > 1)
+    /*Call token frames have a source call number of 1 */
+    BuildConnectionToken();
 
   if (a & 0x8000) {
-    isFullFrame = PTrue;
+    isFullFrame = true;
     Read2Bytes(a);
     remote.SetDestCallNumber(a & 0x7fff);
-    return PTrue;
+    return true;
   }
   if (a == 0) {    //We have a mini frame here, of video type.
-    isVideo = PTrue;
+    isVideo = true;
     PINDEX b = 0;
     Read2Bytes(b);
     remote.SetSourceCallNumber(b);
-    BuildConnectionTokenId();
-    return PTrue;
+    BuildConnectionToken();
+    return true;
   }
 
-  isAudio = PTrue;
-  return PTrue;
+  isAudio = true;
+  return true;
 }
 
-void IAX2Frame::BuildConnectionTokenId()
+void IAX2Frame::BuildConnectionToken()
 {
-  connectionToken = remote.BuildConnectionTokenId();
+  connectionToken = remote.BuildConnectionToken();
 }
 
 void IAX2Frame::PrintOn(ostream & strm) const
@@ -264,12 +249,13 @@ void IAX2Frame::BuildTimeStamp(const PTimeInterval & callStartTick)
     timeStamp = presetTimeStamp;
   else
     timeStamp = CalcTimeStamp(callStartTick);
+  PTRACE(5, "Frame\tBuild time stamp to " << PString(timeStamp) << " ms");
 }
 
 DWORD IAX2Frame::CalcTimeStamp(const PTimeInterval & callStartTick)
 {
   DWORD tVal = (DWORD)(PTimer::Tick() - callStartTick).GetMilliSeconds();
-  PTRACE(6, "Calculate timestamp as " << tVal);
+  PTRACE2(6, NULL, "Frame\tCalculate timestamp as " << tVal);
   return tVal;
 }
 
@@ -277,8 +263,9 @@ PBoolean IAX2Frame::TransmitPacket(PUDPSocket &sock)
 {
   if (CallMustBeActive()) {
     if (!endpoint.ConnectionForFrameIsAlive(this)) {
-      PTRACE(3, "Connection not found, call has been terminated. " << IdString());
-      return PFalse;   //This happens because the call has been terminated.
+      PTRACE(3, "Frame\tConnection not found, call has been terminated. " 
+	     << IdString());
+      return false;   //This happens because the call has been terminated.
     }
   }
   
@@ -286,10 +273,10 @@ PBoolean IAX2Frame::TransmitPacket(PUDPSocket &sock)
   //   	    ((FullFrameProtocol *)this)->SetRetransmissionRequired();
   //	    cout << endl;
   
-  PTRACE(6, "Now transmit " << endl << *this);
+  PTRACE(6, "Frame\tNow transmit " << endl << *this);
   PBoolean transmitResult = sock.WriteTo(data.GetPointer(), DataSize(), remote.RemoteAddress(), 
 				     (unsigned short)remote.RemotePort());
-  PTRACE(6, "transmission of packet gave a " << transmitResult);
+  PTRACE(6, "Frame\ttransmission of packet gave a " << transmitResult);
   return transmitResult;
 }
 
@@ -325,21 +312,21 @@ IAX2Frame *IAX2Frame::BuildAppropriateFrameType()
 PBoolean IAX2Frame::DecryptContents(IAX2Encryption &encryption)
 {
   if (!encryption.IsEncrypted())
-    return PTrue;
+    return true;
 
-#if OPAL_PTLIB_SSL_AES
+#ifdef P_SSL_AES
   PINDEX headerSize = GetEncryptionOffset();
-  PTRACE(4, "Decryption\tUnEncrypted headerSize for " << IdString() << " is " << headerSize);
+  PTRACE(4, "Frame\tUnEncrypted headerSize for " << IdString() << " is " << headerSize);
 
   if ((headerSize + 32) > data.GetSize())         //Make certain packet larger than minimum size.
-    return PFalse;
+    return false;
 
-  PTRACE(6, "DATA Raw is " << endl << ::hex << data << ::dec);
+  PTRACE(6, "Decryption\tDATA Raw is " << endl << ::hex << data << ::dec);
   PINDEX encDataSize = data.GetSize() - headerSize;
   PTRACE(4, "Decryption\tEncoded data size is " << encDataSize);
   if ((encDataSize % 16) != 0) {
     PTRACE(2, "Decryption\tData size is not a multiple of 16.. Error. ");    
-    return PFalse;
+    return false;
   }
 
   unsigned char lastblock[16];
@@ -347,7 +334,7 @@ PBoolean IAX2Frame::DecryptContents(IAX2Encryption &encryption)
   PBYTEArray working(encDataSize);
 
   for (PINDEX i = 0; i < encDataSize; i+= 16) {
-    AES_decrypt(data.GetPointer() + headerSize + i, working.GetPointer() + i, encryption.AesDecryptKey());
+    encryption.AesDecryptKey().Decrypt(data.GetPointer() + headerSize + i, working.GetPointer() + i);
     for (int x = 0; x < 16; x++)
       working[x + i] ^= lastblock[x];
     memcpy(lastblock, data.GetPointer() + headerSize + i, 16);
@@ -359,23 +346,23 @@ PBoolean IAX2Frame::DecryptContents(IAX2Encryption &encryption)
   PINDEX encryptedSize = encDataSize - padding;
   data.SetSize(encryptedSize + headerSize);
 
-  PTRACE(6, "DATA should have a size of " << data.GetSize());
-  PTRACE(6, "UNENCRYPTED DATA is " << endl << ::hex << working << ::dec);
+  PTRACE(6, "Decryption\tDATA should have a size of " << data.GetSize());
+  PTRACE(6, "Decryption\tUNENCRYPTED DATA is " << endl << ::hex << working << ::dec);
 
   memcpy(data.GetPointer() + headerSize, working.GetPointer() + padding, encryptedSize);
-  PTRACE(6, "Entire frame unencrypted is " << endl << ::hex << data << ::dec);
-  return PTrue;
+  PTRACE(6, "Decryption\tEntire frame unencrypted is " << endl << ::hex << data << ::dec);
+  return true;
 #else
-  return PFalse;
+  return false;
 #endif
 }
 
 PBoolean IAX2Frame::EncryptContents(IAX2Encryption &encryption)
 {
   if (!encryption.IsEncrypted())
-    return PTrue;
+    return true;
 
-#if OPAL_PTLIB_SSL_AES
+#ifdef P_SSL_AES
   PINDEX headerSize = GetEncryptionOffset();
   PINDEX eDataSize = data.GetSize() - headerSize;
   PINDEX padding = 16 + ((16 - (eDataSize % 16)) & 0x0f);
@@ -395,15 +382,15 @@ PBoolean IAX2Frame::EncryptContents(IAX2Encryption &encryption)
   for (PINDEX i = 0; i < (eDataSize + padding); i+= 16) {
     for (int x = 0; x < 16; x++)
       curblock[x] ^= working[x + i];
-    AES_encrypt(curblock, result.GetPointer() + i + headerSize, encryption.AesEncryptKey());
+    encryption.AesEncryptKey().Encrypt(curblock, result.GetPointer() + i + headerSize);
     memcpy(curblock, result.GetPointer() + i + headerSize, 16);
   }
 
   data = result;
-  return PTrue;
+  return true;
 #else
   PTRACE(1, "Frame\tEncryption is Flagged on, but AES routines in openssl are not available");
-  return PFalse;
+  return false;
 #endif
 }
 
@@ -470,7 +457,7 @@ PBoolean IAX2MiniFrame::ProcessNetworkPacket()
 
   PTRACE(5, "Mini frame, header processed. frame is " 
 	 << PString(isAudio ? " audio" : "video"));
-  return PTrue;
+  return true;
 }
 
 void IAX2MiniFrame::ZeroAllValues()
@@ -495,7 +482,7 @@ PBoolean IAX2MiniFrame::WriteHeader()
   Write2Bytes(remote.SourceCallNumber() & 0x7fff);
   Write2Bytes(timeStamp & 0xffff);
   
-  return PTrue;
+  return true;
 }
 
 void IAX2MiniFrame::PrintOn(ostream & strm) const
@@ -548,20 +535,19 @@ IAX2FullFrame::IAX2FullFrame(IAX2EndPoint &_newEndpoint)
 IAX2FullFrame::IAX2FullFrame(const IAX2Frame & srcFrame)
   : IAX2Frame(srcFrame)
 {
-  PTRACE(5, "START Constructor for a full frame");
+  PTRACE(6, "Frame\tConstructor for a full frame");
   ZeroAllValues();
-  PTRACE(5, "END Constructor for a full frame");
 }
 
 IAX2FullFrame::~IAX2FullFrame()
 {
-  PTRACE(6, "Destructor IAX2FullFrame:: " << IdString());
+  PTRACE(6, "Frame\tDestructor IAX2FullFrame:: " << IdString());
 }
 
 PBoolean IAX2FullFrame::operator*=(IAX2FullFrame & /*other*/)
 {
   PAssertAlways("Sorry, IAX2FullFrame comparison operator is Not implemented");
-  return PTrue;
+  return true;
 }
 
 void IAX2FullFrame::MarkAsResent()
@@ -575,25 +561,25 @@ void IAX2FullFrame::ZeroAllValues()
   subClass = 0;     
   timeStamp = 0;
   sequence.ZeroAllValues();
-  canRetransmitFrame = PTrue;
+  canRetransmitFrame = true;
   
   transmissionTimer.SetNotifier(PCREATE_NOTIFIER(OnTransmissionTimeout));
   
   retryDelta = PTimeInterval(minRetryTime);
   retries = maxRetries;
   
-  callMustBeActive = PTrue;
-  packetResent = PFalse;
+  callMustBeActive = true;
+  packetResent = false;
   ClearListFlags();
   
-  isFullFrame = PTrue;
-  isAckFrame = PFalse;
+  isFullFrame = true;
+  isAckFrame = false;
 }
 
 void IAX2FullFrame::ClearListFlags()
 {
-  deleteFrameNow = PFalse;
-  sendFrameNow   = PFalse;
+  deleteFrameNow = false;
+  sendFrameNow   = false;
 }
 
 PBoolean IAX2FullFrame::TransmitPacket(PUDPSocket &sock)
@@ -605,7 +591,7 @@ PBoolean IAX2FullFrame::TransmitPacket(PUDPSocket &sock)
   
   if (retries == P_MAX_INDEX) {
     PTRACE(4, "Retries count is now negative on. " << IdString());
-    return PFalse;    //Give up on this packet, it has exceeded the allowed number of retries.
+    return false;    //Give up on this packet, it has exceeded the allowed number of retries.
   }
   
   PTRACE(6, "Start timer running for " << IdString() << connectionToken);
@@ -621,7 +607,7 @@ PBoolean IAX2FullFrame::ProcessNetworkPacket()
   PTRACE(5, "ProcessNetworkPacket - read the frame header");
   if (data.GetSize() < 12) {
     PTRACE(2, "Incoming full frame is undersize - should have 12 bytes, but only read " << data.GetSize());
-    return PFalse;
+    return false;
   }
   
   Read4Bytes(timeStamp);
@@ -638,7 +624,7 @@ PBoolean IAX2FullFrame::ProcessNetworkPacket()
 
   if ((a >= numFrameTypes) || (a == undefType)) {
     PTRACE(3, "Incoming packet has invalid frame type of " << a);
-    return PFalse;
+    return false;
   }
   
   frameType = (IAX2FrameType)a;
@@ -649,80 +635,100 @@ PBoolean IAX2FullFrame::ProcessNetworkPacket()
   Read1Byte(a);
   
   UnCompressSubClass(a);
-  isAckFrame = (subClass == IAX2FullFrameProtocol::cmdAck) && (frameType == iax2ProtocolType);     
-  return PTrue;
+  isAckFrame = (subClass == IAX2FullFrameProtocol::cmdAck) && 
+    (frameType == iax2ProtocolType);     
+  return true;
 }
 
 PBoolean IAX2FullFrame::IsPingFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdPing) && (frameType == iax2ProtocolType);
+  return (subClass == IAX2FullFrameProtocol::cmdPing) && 
+    (frameType == iax2ProtocolType);
 }
 
 PBoolean IAX2FullFrame::IsNewFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdNew) && (frameType == iax2ProtocolType);
+  return (subClass == IAX2FullFrameProtocol::cmdNew) && 
+    (frameType == iax2ProtocolType);
 }
 
 PBoolean IAX2FullFrame::IsLagRqFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdLagRq) && (frameType == iax2ProtocolType);
+  return (subClass == IAX2FullFrameProtocol::cmdLagRq) && 
+    (frameType == iax2ProtocolType);
 }
 
 PBoolean IAX2FullFrame::IsLagRpFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdLagRp) && (frameType == iax2ProtocolType);     
+  return (subClass == IAX2FullFrameProtocol::cmdLagRp) && 
+    (frameType == iax2ProtocolType);     
 }
 
 PBoolean IAX2FullFrame::IsPongFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdPong) && (frameType == iax2ProtocolType);
+  return (subClass == IAX2FullFrameProtocol::cmdPong) && 
+    (frameType == iax2ProtocolType);
 }
 
 PBoolean IAX2FullFrame::IsAuthReqFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdAuthReq) && (frameType == iax2ProtocolType);
+  return (subClass == IAX2FullFrameProtocol::cmdAuthReq) && 
+    (frameType == iax2ProtocolType);
 }
 
 PBoolean IAX2FullFrame::IsVnakFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdVnak) && (frameType == iax2ProtocolType);
+  return (subClass == IAX2FullFrameProtocol::cmdVnak) && 
+    (frameType == iax2ProtocolType);
 }
 
 PBoolean IAX2FullFrame::IsRegReqFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdRegReq) && (frameType == iax2ProtocolType);
+  return (subClass == IAX2FullFrameProtocol::cmdRegReq) && 
+    (frameType == iax2ProtocolType);
 }
 
 PBoolean IAX2FullFrame::IsRegAuthFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdRegAuth) && (frameType == iax2ProtocolType);
+  return (subClass == IAX2FullFrameProtocol::cmdRegAuth) && 
+    (frameType == iax2ProtocolType);
 }
 
 PBoolean IAX2FullFrame::IsRegAckFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdRegAck) && (frameType == iax2ProtocolType);
+  return (subClass == IAX2FullFrameProtocol::cmdRegAck) && 
+    (frameType == iax2ProtocolType);
 }
 
 PBoolean IAX2FullFrame::IsRegRelFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdRegRel) && (frameType == iax2ProtocolType);
+  return (subClass == IAX2FullFrameProtocol::cmdRegRel) && 
+    (frameType == iax2ProtocolType);
 }
 
 PBoolean IAX2FullFrame::IsRegRejFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdRegRej) && (frameType == iax2ProtocolType);
+  return (subClass == IAX2FullFrameProtocol::cmdRegRej) && 
+    (frameType == iax2ProtocolType);
 }
 
 PBoolean IAX2FullFrame::IsHangupFrame()
 {
-  return (subClass == IAX2FullFrameProtocol::cmdHangup) && (frameType == iax2ProtocolType);
+  return (subClass == IAX2FullFrameProtocol::cmdHangup) && 
+    (frameType == iax2ProtocolType);
+}
+
+PBoolean IAX2FullFrame::IsCallTokenFrame()
+{
+  return (subClass == IAX2FullFrameProtocol::cmdCallToken) && 
+    (frameType == iax2ProtocolType);
 }
 
 PBoolean IAX2FullFrame::FrameIncrementsInSeqNo()
 {
   if (frameType != iax2ProtocolType) {
     PTRACE(5, "SeqNos\tFrameType is not iaxProtocol, so we do increment inseqno. FrameType is " << frameType);
-    return PTrue;
+    return true;
   }
 
   IAX2FullFrameProtocol::ProtocolSc cmdType = (IAX2FullFrameProtocol::ProtocolSc)subClass;
@@ -734,11 +740,11 @@ PBoolean IAX2FullFrame::FrameIncrementsInSeqNo()
       //  (cmdType == IAX2FullFrameProtocol::cmdAuthRep) ||
       (cmdType == IAX2FullFrameProtocol::cmdVnak))   {
     PTRACE(3, "SeqNos\tThis is a iaxProtocol cmd type that does not increment inseqno");
-    return PFalse;
+    return false;
   } else {
     PTRACE(5, "SeqNos\tThis is a iaxProtocol cmd type that increments inseqno");
   }
-  return PTrue;
+  return true;
 }
 
 void IAX2FullFrame::UnCompressSubClass(BYTE a)
@@ -757,7 +763,7 @@ int IAX2FullFrame::CompressSubClass()
   if (subClass < 0x80)
     return subClass;
   
-  for(PINDEX i = 0; i <  0x1f; i++) {
+  for(int i = 0; i <  0x1f; i++) {
     if (subClass & (1 << i)) 
       return i | 0x80;
   }
@@ -785,19 +791,19 @@ PBoolean IAX2FullFrame::WriteHeader()
   
   int a = CompressSubClass();
   if (a < 0)
-    Write1Byte(0xff);
+    Write1Byte((BYTE)0xff);
   else
     Write1Byte((BYTE)a);     
   PTRACE(6, "Comppressed sub class is " << a << " from " << subClass);
   
-  return PTrue;
+  return true;
 }
 
 void IAX2FullFrame::MarkVnakSendNow()
 {
   transmissionTimer.Stop();
-  sendFrameNow = PTrue;
-  deleteFrameNow = PFalse;    
+  sendFrameNow = true;
+  deleteFrameNow = false;    
   retryDelta = PTimeInterval(minRetryTime);
   retries = maxRetries;
 }
@@ -806,18 +812,18 @@ void IAX2FullFrame::MarkDeleteNow()
 {
   PTRACE(5, "MarkDeleteNow() method on " << IdString());
   transmissionTimer.Stop();
-  deleteFrameNow = PTrue;
+  deleteFrameNow = true;
   retries = P_MAX_INDEX;
 }
 
-void IAX2FullFrame::OnTransmissionTimeout(PTimer &, INT)
+void IAX2FullFrame::OnTransmissionTimeout(PTimer &, P_INT_PTR)
 {
   PTRACE(4, "Has had a TX timeout " << IdString() << " " << connectionToken);
   retryDelta = 4 * retryDelta.GetMilliSeconds();
   if (retryDelta > maxRetryTime)
     retryDelta = maxRetryTime;
   
-  packetResent = PTrue;
+  packetResent = true;
   if ((retries == P_MAX_INDEX) || (retries == 0)) {
     retries = P_MAX_INDEX;
     PTRACE(5, "Retries are " << PString(retries) 
@@ -825,7 +831,7 @@ void IAX2FullFrame::OnTransmissionTimeout(PTimer &, INT)
     MarkDeleteNow();
   } else {
     retries--;
-    sendFrameNow = PTrue;
+    sendFrameNow = true;
     PTRACE(5, "Tx timeout, so Mark as Send now " << IdString() << " " << connectionToken);
   }
 
@@ -1018,7 +1024,7 @@ unsigned short IAX2FullFrameVoice::OpalNameToIax2Value(const PString opalName)
   if (opalName.Find("iLBC-13k3") != P_MAX_INDEX) {
     return ilbc; 
     }
-  PTRACE(6, "Codec " << opalName << " is not supported in IAX2");
+  PTRACE2(6, NULL, "Codec " << opalName << " is not supported in IAX2");
   return 0;
 }
 
@@ -1088,9 +1094,9 @@ IAX2FullFrameSessionControl::IAX2FullFrameSessionControl(IAX2Processor *iax2Proc
   :  IAX2FullFrame(iax2Processor->GetEndPoint())          
 {
   SetSubClass((PINDEX)session);
-  isAckFrame = PFalse;
+  isAckFrame = false;
   InitialiseHeader(iax2Processor);
-  callMustBeActive = PTrue;
+  callMustBeActive = true;
 }
 
 IAX2FullFrameSessionControl::IAX2FullFrameSessionControl(IAX2Processor *iax2Processor,
@@ -1098,9 +1104,9 @@ IAX2FullFrameSessionControl::IAX2FullFrameSessionControl(IAX2Processor *iax2Proc
   :  IAX2FullFrame(iax2Processor->GetEndPoint())          
 {
   SetSubClass(subClassValue);
-  isAckFrame = PFalse;
+  isAckFrame = false;
   InitialiseHeader(iax2Processor);
-  callMustBeActive = PTrue;
+  callMustBeActive = true;
 }
 
 PString IAX2FullFrameSessionControl::GetSubClassName() const {
@@ -1176,13 +1182,10 @@ IAX2FullFrameProtocol::IAX2FullFrameProtocol(IAX2Processor *iax2Processor,  Prot
   if (iax2Processor == NULL) {
     IAX2Remote rem = inReplyTo->GetRemoteInfo();
     remote = rem;
-    ///	  remote.SetSourceCallNumber(rem.GetDestCallNumber());
-    ///       remote.SetDestCallNumber(rem.GetSourceCallNumber());	  
   } else {
     remote = iax2Processor->GetRemoteInfo();
     SetConnectionToken(iax2Processor->GetCallToken());
   }
-  //     processor->MassageSequenceForReply(sequence, inReplyTo->GetSequenceInfo());
   frameType = iax2ProtocolType;       
   callMustBeActive = (needCon == callActive);
   WriteHeader();
@@ -1205,35 +1208,47 @@ IAX2FullFrameProtocol::IAX2FullFrameProtocol(const IAX2FullFrame & srcFrame)
 
 IAX2FullFrameProtocol::~IAX2FullFrameProtocol()
 {
-  ieElements.AllowDeleteObjects(PTrue);
+  ieElements.AllowDeleteObjects(true);
   PTRACE(6, "Destroy this IAX2FullFrameProtocol(" << GetSubClassName() << ") " << IdString());
 }
 
 void IAX2FullFrameProtocol::SetRetransmissionRequired()
 {
   if (GetSubClass() == IAX2FullFrameProtocol::cmdAck)
-    canRetransmitFrame = PFalse;
+    canRetransmitFrame = false;
   
   if (GetSubClass() == IAX2FullFrameProtocol::cmdLagRp)
-    canRetransmitFrame = PFalse;
+    canRetransmitFrame = false;
   
   if (GetSubClass() == IAX2FullFrameProtocol::cmdPong)
-    canRetransmitFrame = PFalse;
+    canRetransmitFrame = false;
 }
 
 void IAX2FullFrameProtocol::WriteIeAsBinaryData()
 {
-  PTRACE(6, "Write the IE data (" << ieElements.GetSize() 
-	 << " elements) as binary data to frame");
+  PTRACE(6, "Frame\tWrite the IE data (" << ieElements.GetSize() 
+	 << " elements) as binary data to frame " << IdString());
   PINDEX headerSize = data.GetSize();
   data.SetSize(headerSize + ieElements.GetBinaryDataSize());
   
-  for(PINDEX i = 0; i < ieElements.GetSize(); i++) {
-    PTRACE(5, "Append to outgoing frame " << *ieElements.GetIeAt(i));
-    ieElements.GetIeAt(i)->WriteBinary(data.GetPointer(), headerSize);
+  for (IAX2IeList::iterator it = ieElements.begin(); it != ieElements.end(); ++it) {
+    PTRACE(6, "Frame\tAppend to outgoing frame " << *it);
+    it->WriteBinary(data.GetPointer(), headerSize);
   }    
 }
 
+PBoolean IAX2FullFrameProtocol::GetCallTokenIe(IAX2IeCallToken & callToken)
+{
+  for (IAX2IeList::iterator it = ieElements.begin(); it != ieElements.end(); ++it) {
+    if (it->GetKeyValue() == IAX2Ie::ie_callToken) {
+      IAX2IeCallToken *src = (IAX2IeCallToken *)&*it;
+      callToken.CopyData(src);
+      return true;
+    }
+  }
+  return false;
+}
+
 PBoolean IAX2FullFrameProtocol::ReadInformationElements()
 {
   IAX2Ie *elem = NULL;
@@ -1257,10 +1272,10 @@ PBoolean IAX2FullFrameProtocol::ReadInformationElements()
   }
   
   if (elem == NULL)
-    return PFalse;
+    return false;
   
   if (!elem->IsValid())
-    return PFalse;
+    return false;
   
   return GetUnReadBytes() == 0;
 }
@@ -1269,21 +1284,14 @@ void IAX2FullFrameProtocol::GetRemoteCapability(unsigned int & capability, unsig
 {
   capability = 0;
   preferred = 0;
-  IAX2Ie * p;
-  PINDEX i = 0;
-  for(;;) {
-    p = GetIeAt(i);
-    if (p == NULL)
-      break;
-    
-    i++;
-    if (p->IsValid()) {
-      if (PIsDescendant(p, IAX2IeCapability)) {
-	capability = ((IAX2IeCapability *)p)->ReadData();
+  for (IAX2IeList::iterator it = ieElements.begin(); it != ieElements.end(); ++it) {
+    if (it->IsValid()) {
+      if (PIsDescendant(&*it, IAX2IeCapability)) {
+	capability = ((IAX2IeCapability *)&*it)->ReadData();
 	PTRACE(5, "IAX2FullFrameProtocol\tCapability codecs are " << capability);
       }
-      if (PIsDescendant(p, IAX2IeFormat)) {
-	preferred = ((IAX2IeFormat *)p)->ReadData();
+      if (PIsDescendant(&*it, IAX2IeFormat)) {
+	preferred = ((IAX2IeFormat *)&*it)->ReadData();
 	PTRACE(4, "IAX2FullFrameProtocol\tPreferred codec is " << preferred);
       }
     } else {
@@ -1294,19 +1302,12 @@ void IAX2FullFrameProtocol::GetRemoteCapability(unsigned int & capability, unsig
 
 void IAX2FullFrameProtocol::CopyDataFromIeListTo(IAX2IeData &res)
 {
-  IAX2Ie * p;
-  PINDEX i = 0;
-  for(;;) {
-    p = GetIeAt(i);
-    if (p == NULL)
-      break;
-    
-    i++;
-    PTRACE(4, "From IAX2FullFrameProtocol, handle IAX2Ie of type " << *p);
-    if (p->IsValid()) 
-      p->StoreDataIn(res);
+  for (IAX2IeList::iterator it = ieElements.begin(); it != ieElements.end(); ++it) {
+    PTRACE(4, "From IAX2FullFrameProtocol, handle IAX2Ie of type " << *it);
+    if (it->IsValid()) 
+      it->StoreDataIn(res);
     else {
-      PTRACE(3, "Invalid data in IE. " << *p);
+      PTRACE(3, "Invalid data in IE. " << *it);
     }
   }
 }
@@ -1316,7 +1317,7 @@ PString IAX2FullFrameProtocol::GetSubClassName() const
   return GetSubClassName(GetSubClass());
 }
 
-PString IAX2FullFrameProtocol::GetSubClassName(PINDEX t)
+PString IAX2FullFrameProtocol::GetSubClassName(unsigned t)
 {
   switch (t) {
   case cmdNew:        return PString("new");
@@ -1411,8 +1412,7 @@ IAX2FullFrameText::IAX2FullFrameText(IAX2Processor *iaxProcessor, const PString&
 
   PINDEX headerSize = data.GetSize();
   data.SetSize(text.GetLength() + headerSize);
-  memcpy(data.GetPointer() + headerSize, 
-	 internalText.GetPointer(), internalText.GetLength());
+  memcpy(data.GetPointer() + headerSize, (const char *)internalText, internalText.GetLength());
 
   PTRACE(4, "Construct a full frame text" << IdString() << " for text " << text);
 }
@@ -1475,24 +1475,20 @@ PString IAX2FullFrameCng::GetSubClassName() const
 
 ////////////////////////////////////////////////////////////////////////////////
 
-IAX2FrameList::~IAX2FrameList()
-{   
-}
-
 void IAX2FrameList::ReportList(PString & answer)
 {
   PStringStream reply;
   {
     PWaitAndSignal m(mutex);
-    
-    for(PINDEX i = 0; i < PAbstractList::GetSize(); i++) {
-       IAX2Frame *frame = (IAX2Frame *)GetAt(i);
-       reply << "     #" << (i + 1) << " of " 
-	     << PAbstractList::GetSize() << "   "
-	     << frame->GetConnectionToken() << " " 
-	     << frame->GetTimeStamp();
-       if (frame->IsFullFrame()) {
-	 IAX2FullFrame *ff = (IAX2FullFrame *)frame;
+
+    PINDEX count = 0;
+    for (iterator it = begin(); it != end(); ++it) {
+       reply << "     #" << ++count << " of " 
+	     << GetSize() << "   "
+	     << it->GetConnectionToken() << " " 
+	     << it->GetTimeStamp();
+       if (it->IsFullFrame()) {
+	 IAX2FullFrame *ff = (IAX2FullFrame *)&*it;
 	 reply << " " << ff->GetSequenceInfo().AsString() << " " 
 	       << ff->GetFullFrameName() << endl;
        }
@@ -1505,9 +1501,10 @@ void IAX2FrameList::AddNewFrame(IAX2Frame *newFrame)
 {
   if (newFrame == NULL)
     return;
-  PTRACE(5, "AddNewFrame " << newFrame->IdString());
+  PTRACE(5, "Frame\tAdd " << newFrame->IdString() 
+	 << " " << newFrame->GetRemoteInfo());
   PWaitAndSignal m(mutex);
-  PAbstractList::Append(newFrame);
+  Append(newFrame);
 }
 
 void IAX2FrameList::GrabContents(IAX2FrameList &src)
@@ -1522,42 +1519,40 @@ void IAX2FrameList::GrabContents(IAX2FrameList &src)
 IAX2Frame *IAX2FrameList::GetLastFrame()
 {
   PWaitAndSignal m(mutex);
-  PINDEX elems = GetEntries();
-  if (elems ==  0) {
-    return NULL;
-  }
-  
-  return (IAX2Frame *)PAbstractList::RemoveAt(0);
+  return (IAX2Frame *)RemoveHead();
 }
 
 void IAX2FrameList::DeleteMatchingSendFrame(IAX2FullFrame *reply)
 {
-  PINDEX currentIndex;
+  iterator currentIter;
   IAX2FullFrame *sent;
 
   PWaitAndSignal m(mutex);
   //Look for a frame that has been sent, which is waiting for a reply/ack.
-  PTRACE(5, "ID# Delete matchingSendFrame start, test on " 
+  PTRACE(5, "Frame\tID# Delete matchingSendFrame start, test on " 
 	 << reply->IdString());
 
-  for (PINDEX i = 0; i < GetEntries(); i++) {
+  for (iterator it = begin(); it != end(); ++it) {
     sent = NULL;
-    IAX2Frame *frame = (IAX2Frame *)GetAt(i);
-    if (frame == NULL)
-      continue;
-    
-    PTRACE(5, "ID#DeleteMatching " << frame->IdString());
-    if (!frame->IsFullFrame())
+    PTRACE(5, "ID#DeleteMatching " << it->IdString());
+    if (!it->IsFullFrame())
       continue;
 
-    sent = (IAX2FullFrame *)frame;
-    currentIndex = i; /* If a match is found, we delete this one */
+    sent = (IAX2FullFrame *)&*it;
+    currentIter = it; /* If a match is found, we delete this one */
 
     if (sent->DeleteFrameNow()) {
       // Skip this frame, as it is marked, delete now
       continue;
     }
-    
+
+    if (sent->IsNewFrame() &&
+	reply->IsCallTokenFrame()) {
+      if(sent->GetRemoteInfo().SourceCallNumber() == 
+	 reply->GetRemoteInfo().DestCallNumber()) 
+	goto foundMatch;
+    }
+
     if (!(sent->GetRemoteInfo() *= reply->GetRemoteInfo())) {
       PTRACE(5, "mismatch in remote info");
       continue;
@@ -1565,10 +1560,9 @@ void IAX2FrameList::DeleteMatchingSendFrame(IAX2FullFrame *reply)
 
     if (sent->IsNewFrame() &&
 	reply->GetSequenceInfo().IsFirstReplyFrame()) {
-      PTRACE(5, "Have a match on a new frame we sent out");
+      PTRACE(5, "Frame\tHave a match on a new frame we sent out");
       goto foundMatch;
     }
-
         
     if (sent->IsRegReqFrame() && 
         (reply->IsRegAckFrame() || reply->IsRegAuthFrame() || reply->IsRegRejFrame())) {
@@ -1634,7 +1628,7 @@ void IAX2FrameList::DeleteMatchingSendFrame(IAX2FullFrame *reply)
  foundMatch:
 
   delete sent;
-  RemoveAt(currentIndex);
+  erase(currentIter);
   return;
 }  
 
@@ -1644,15 +1638,11 @@ void IAX2FrameList::SendVnakRequestedFrames(IAX2FullFrameProtocol &src)
   PWaitAndSignal m(mutex);
   PTRACE(4, "Look for a frame that has been sent, waiting to be acked etc, that matches the supplied Vnak frame");
   
-  for (PINDEX i = 0; i < GetEntries(); i++) {
-    IAX2Frame *frame = (IAX2Frame *)GetAt(i);
-    if (frame == NULL)
-      continue;
-    
-    if (!frame->IsFullFrame())
+  for (iterator it = begin(); it != end(); ++it) {
+    if (!it->IsFullFrame())
       continue;
 
-    IAX2FullFrame *sent = (IAX2FullFrame *)frame;
+    IAX2FullFrame *sent = (IAX2FullFrame *)&*it;
 
     if (sent->DeleteFrameNow()) {
       PTRACE(4, "Skip this frame, as it is marked, delete now" << sent->IdString());
@@ -1687,23 +1677,28 @@ void IAX2FrameList::GetResendFramesDeleteOldFrames(IAX2FrameList &framesToSend)
     return;
   }
   
-  for (PINDEX i = GetEntries(); i > 0; i--) {
-    IAX2FullFrame *active = (IAX2FullFrame *)PAbstractList::GetAt(i - 1);
-    if (active == NULL)
+  iterator it = rbegin();
+  while (it != rend()) {
+    IAX2FullFrame *active = dynamic_cast<IAX2FullFrame *>(&*it);
+    if (active == NULL) {
+      --it;
       continue;
+    }
     
     if (active->DeleteFrameNow()) { 
       PTRACE(5, "marked as delete now, so delete" << *active);
       delete active;
       active = NULL;
-      PAbstractList::RemoveAt(i - 1);
+      erase(it--);
       continue;
-      }
-    
+    }
+
     if (active->SendFrameNow()) {
-      PAbstractList::RemoveAt(i - 1);
+      erase(it--);
       framesToSend.AddNewFrame(active);
     }
+    else
+      --it;
   }
   PTRACE(4, "Have collected " << framesToSend.GetSize() << " frames to onsend");
   PTRACE(5, "ID# GetResendFramesDeleteOldFrames end ");
@@ -1714,9 +1709,10 @@ void IAX2FrameList::MarkAllAsResent()
 {
   PWaitAndSignal m(mutex);
 
-  for (PINDEX i = 0; i < GetEntries(); i++) {
-    IAX2FullFrame *active = (IAX2FullFrame *)PAbstractList::GetAt(i);
-    active->MarkAsResent();
+  for (iterator it = begin(); it != end(); ++it) {
+    IAX2FullFrame *active = dynamic_cast<IAX2FullFrame *>(&*it);
+    if (active != NULL)
+      active->MarkAsResent();
   }
 }
 
@@ -1725,10 +1721,9 @@ void IAX2FrameList::MarkAllAsResent()
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 4 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file. 
+ * With the comment below, the tab key does auto indent to 2 spaces.    
+ *
  * Local Variables:
  * mode:c
  * c-basic-offset:2
