@@ -25,9 +25,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 25336 $
+ * $Revision: 29059 $
  * $Author: rjongbloed $
- * $Date: 2011-03-16 03:51:39 -0500 (Wed, 16 Mar 2011) $
+ * $Date: 2013-01-30 21:12:49 -0600 (Wed, 30 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -57,7 +57,6 @@
 
 ////////////////////////////////////////////////////////////////////////////////
 
-
 IAX2Connection::IAX2Connection(OpalCall & call,               /* Owner call for connection        */
 			       IAX2EndPoint &ep, 
 			     const PString & token,         /* Token to identify the connection */
@@ -67,11 +66,10 @@ IAX2Connection::IAX2Connection(OpalCall & call,               /* Owner call for
   : OpalConnection(call, ep, token)
   , endpoint(ep)
   , iax2Processor(*new IAX2CallProcessor(ep))
-  , jitterBuffer(400, 2000)
+  , jitterBuffer(OpalJitterBuffer::Init(400, 2000))
 {  
   opalPayloadType = RTP_DataFrame::IllegalPayloadType;
 
-  remotePartyAddress = "iax2:" + inRemoteParty;
   if (inRemotePartyName.IsEmpty())
     remotePartyName = inRemoteParty;
   else
@@ -81,14 +79,13 @@ IAX2Connection::IAX2Connection(OpalCall & call,               /* Owner call for
   remotePartyNumber = res[IAX2EndPoint::extensionIndex];
 
   SetCallToken(token);
-  originating = PFalse;
 
   ep.CopyLocalMediaFormats(localMediaFormats);
   AdjustMediaFormats(true, NULL, localMediaFormats);
   PTRACE(5, "Local ordered codecs are " << localMediaFormats);
   
-  local_hold = PFalse;
-  remote_hold = PFalse;
+  local_hold = false;
+  remote_hold = false;
 
  
   PTRACE(6, "IAX2Connection class has been initialised, and is ready to run");
@@ -113,21 +110,11 @@ void IAX2Connection::StartOperation()
   SetPhase(SetUpPhase);
 }
 
-void IAX2Connection::Release( CallEndReason reason)		        
-{ 
-  PTRACE(4, "IAX2Con\tRelease( CallEndReason " << reason);
-
-  iax2Processor.Hangup(GetCallEndReasonText(reason)); ///Send hangup frame
-
-  iax2Processor.Release(reason); 
-
-  OpalConnection::Release(reason);
-}
-
 void IAX2Connection::OnReleased()
 {
   PTRACE(4, "IAX2Con\tOnReleased()" << *this);
 
+  iax2Processor.Hangup(GetCallEndReasonText(GetCallEndReason())); ///Send hangup frame
   iax2Processor.OnReleased();
   OpalConnection::OnReleased();  
 }
@@ -187,9 +174,9 @@ PBoolean IAX2Connection::SetConnected()
     PSafePtr<OpalConnection> otherParty = GetOtherPartyConnection();
     if (otherParty != NULL)
       ownerCall.OpenSourceMediaStreams(*otherParty, OpalMediaType::Audio(), 1);
-        
-    jitterBuffer.SetDelay(endpoint.GetManager().GetMinAudioJitterDelay() * 8, 
-			  endpoint.GetManager().GetMaxAudioJitterDelay() * 8);
+
+    jitterBuffer.SetDelay(OpalJitterBuffer::Init(endpoint.GetManager().GetMinAudioJitterDelay() * 8, 
+                                                 endpoint.GetManager().GetMaxAudioJitterDelay() * 8));
     PTRACE(5, "Iax2Con\t Start jitter buffer");
   }  
   return OpalConnection::SetConnected();
@@ -200,12 +187,12 @@ PBoolean IAX2Connection::SetAlerting(const PString & PTRACE_PARAM(calleeName), P
   {
     PSafeLockReadWrite safeLock(*this);
     if (!safeLock.IsLocked())
-      return PFalse;
+      return false;
     
     PTRACE(3, "IAX2Con\tSetAlerting  from " << calleeName << " " << *this); 
     
     if (GetPhase() == AlertingPhase)
-      return PFalse;
+      return false;
     
     SetPhase(AlertingPhase);
   }
@@ -213,7 +200,7 @@ PBoolean IAX2Connection::SetAlerting(const PString & PTRACE_PARAM(calleeName), P
   OpalConnection::OnAlerting();
 //Which puts the other Connection in this call to AlertingPhase
 
-  return PTrue;
+  return true;
 }
 
 void IAX2Connection::OnConnected()
@@ -228,8 +215,8 @@ void IAX2Connection::OnConnected()
       ownerCall.OpenSourceMediaStreams(*otherParty, OpalMediaType::Audio(), 1);
 
 
-    jitterBuffer.SetDelay(endpoint.GetManager().GetMinAudioJitterDelay() * 8, 
-			  endpoint.GetManager().GetMaxAudioJitterDelay() * 8);
+    jitterBuffer.SetDelay(OpalJitterBuffer::Init(endpoint.GetManager().GetMinAudioJitterDelay() * 8, 
+			                         endpoint.GetManager().GetMaxAudioJitterDelay() * 8));
     PTRACE(5, "Iax2Con\t Start jitter buffer");
   }
 
@@ -250,7 +237,7 @@ PBoolean IAX2Connection::SendUserInputString(const PString & value)
 
   if (mode == SendUserInputAsString) {
     iax2Processor.SendText(value); 
-    return PTrue;
+    return true;
   }
 
   return OpalConnection::SendUserInputString(value);
@@ -260,7 +247,7 @@ PBoolean IAX2Connection::SendUserInputString(const PString & value)
 PBoolean IAX2Connection::SendUserInputTone(char tone, unsigned /*duration*/ ) 
 { 
   iax2Processor.SendDtmf(tone); 
-  return PTrue;
+  return true;
 }
 
 void IAX2Connection::OnEstablished()
@@ -276,11 +263,6 @@ OpalMediaStream * IAX2Connection::CreateMediaStream(const OpalMediaFormat & medi
                                                    unsigned sessionID,
                                                    PBoolean isDataSource)
 {
-  if (ownerCall.IsMediaBypassPossible(*this, sessionID)) {
-    PTRACE(3, "connection\t  create a null media stream ");
-    return new OpalNullMediaStream(*this, mediaFormat, sessionID, isDataSource);
-  }
-
   PTRACE(4, "IAX2con\tCreate an OpalIAX2MediaStream");
   return new OpalIAX2MediaStream(*this, mediaFormat, sessionID, isDataSource);
 }
@@ -297,7 +279,8 @@ PBoolean IAX2Connection::SetUpConnection()
   iax2Processor.SetUserName(userName);
   iax2Processor.SetPassword(password);
   
-  originating = PTrue;
+  InternalSetAsOriginating();
+
   return iax2Processor.SetUpConnection(); 
 }
 
@@ -467,7 +450,7 @@ PBoolean IAX2Connection::ForwardCall(const PString & PTRACE_PARAM(forwardParty))
 {
   PTRACE(3, "Forward call to " + forwardParty);
   //we can not currently forward calls that have not been accepted.
-  return PFalse;
+  return false;
 }
 
 void IAX2Connection::ReceivedSoundPacketFromNetwork(IAX2Frame *soundFrame)
@@ -484,7 +467,7 @@ void IAX2Connection::ReceivedSoundPacketFromNetwork(IAX2Frame *soundFrame)
 
     RTP_DataFrame mediaFrame(soundFrame->GetMediaDataSize());
     mediaFrame.SetTimestamp(soundFrame->GetTimeStamp() * 8);
-    mediaFrame.SetMarker(PFalse);
+    mediaFrame.SetMarker(false);
     mediaFrame.SetPayloadType(opalPayloadType);
 
     mediaFrame.SetSize(mediaFrame.GetPayloadSize() + mediaFrame.GetHeaderSize());
@@ -504,11 +487,11 @@ PBoolean IAX2Connection::ReadSoundPacket(RTP_DataFrame & packet)
     PINDEX zeroBytes = packet.GetSize() - packet.GetHeaderSize();
     memset(packet.GetPayloadPtr() + packet.GetHeaderSize(), 0, zeroBytes);
     PTRACE(5, "Iax2Con\t faulty  read from  jitter buffer"); 
-    return PFalse;
+    return false;
   }
 
   packet.SetPayloadSize(packet.GetSize() - packet.GetHeaderSize());
-  return PTrue;
+  return true;
 }
 
 
