@@ -25,9 +25,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 23008 $
+ * $Revision: 29658 $
  * $Author: rjongbloed $
- * $Date: 2009-06-28 23:24:23 -0500 (Sun, 28 Jun 2009) $
+ * $Date: 2013-05-07 21:16:26 -0500 (Tue, 07 May 2013) $
  */
 
 #include <ptlib.h>
@@ -35,6 +35,10 @@
 
 #if OPAL_IAX2
 
+  #ifdef _MSC_VER
+    #pragma message("IAX2 support enabled")
+  #endif
+
 #ifdef P_USE_PRAGMA
 #pragma implementation "iax2ep.h"
 #endif
@@ -52,9 +56,8 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 IAX2EndPoint::IAX2EndPoint(OpalManager & mgr)
-  : OpalEndPoint(mgr, "iax2", CanTerminateCall)
+  : OpalEndPoint(mgr, "iax2", CanTerminateCall|SupportsE164)
 {
-  
   localUserName = mgr.GetDefaultUserName();
   localNumber   = "1234";
   
@@ -65,17 +68,18 @@ IAX2EndPoint::IAX2EndPoint(OpalManager & mgr)
   receiver = NULL;
   sock = NULL;
   callsEstablished.SetValue(0);
-  
+
+  IAX2IeCallToken::InitialiseKey();
   //We handle the deletion of regProcessor objects.
-  regProcessors.AllowDeleteObjects(PFalse);
+  regProcessors.AllowDeleteObjects(false);
 
   Initialise();
-  PTRACE(5, "IAX2\tCreated endpoint.");
+  PTRACE(5, "Iax2Ep\tCreated endpoint.");
 }
 
 IAX2EndPoint::~IAX2EndPoint()
 {
-  PTRACE(5, "Endpoint\tIaxEndPoint destructor. Terminate the  transmitter, receiver, and incoming frame handler.");
+  PTRACE(5, "Iax2Ep\tIaxEndPoint destructor. Terminate the  transmitter, receiver, and incoming frame handler.");
   
   //contents of this array are automatically shifted when removed
   //so we only need to loop through the first element until all
@@ -87,27 +91,27 @@ IAX2EndPoint::~IAX2EndPoint()
     delete regProcessor;
   }
 
-  PTRACE(6, "Iax2Endpoint\tDestructor - cleaned up the different registration processeors");
+  PTRACE(6, "Iax2Ep\tDestructor - cleaned up the different registration processeors");
 
   incomingFrameHandler.Terminate();
   incomingFrameHandler.WaitForTermination();
   packetsReadFromEthernet.AllowDeleteObjects();  
-  PTRACE(6, "Iax2Endpoint\tDestructor - cleaned up the incoming frame handler");
+  PTRACE(6, "Iax2Ep\tDestructor - cleaned up the incoming frame handler");
   
   if (receiver != NULL && transmitter != NULL) {
     transmitter->Terminate();
     receiver->Terminate();
     transmitter->WaitForTermination();
-    PTRACE(6, "Iax2Endpoint\tDestructor - cleaned up the iax2 transmitter");
+    PTRACE(6, "Iax2Ep\tDestructor - cleaned up the iax2 transmitter");
     receiver->WaitForTermination();
-    PTRACE(6, "Iax2Endpoint\tDestructor - cleaned up the iax2 receiver");
+    PTRACE(6, "Iax2Ep\tDestructor - cleaned up the iax2 receiver");
   }
 
   if (specialPacketHandler != NULL) {
     specialPacketHandler->Terminate();
     specialPacketHandler->WaitForTermination();
     delete specialPacketHandler;
-    PTRACE(6, "Iax2Endpoint\tDestructor - cleaned up the iax2 special packet handler");
+    PTRACE(6, "Iax2Ep\tDestructor - cleaned up the iax2 special packet handler");
   }
   specialPacketHandler = NULL;
   
@@ -119,18 +123,27 @@ IAX2EndPoint::~IAX2EndPoint()
   if (sock != NULL)
     delete sock; 
   
-  PTRACE(6, "Endpoint\tDESTRUCTOR of IAX2 endpoint has Finished.");  
+  PTRACE(6, "Iax2Ep\tDESTRUCTOR of IAX2 endpoint has Finished.");  
 }
 
+
+PString IAX2EndPoint::GetDefaultTransport() const
+{
+  return OpalTransportAddress::UdpPrefix();
+}
+
+
+WORD IAX2EndPoint::GetDefaultSignalPort() const
+{
+  return DefaultUdpPort;
+}
+
+
 void IAX2EndPoint::ReportTransmitterLists(PString & answer, bool getFullReport)
 {
   transmitter->ReportLists(answer, getFullReport); 
 }
 
-PBoolean IAX2EndPoint::NewIncomingConnection(OpalTransport * /*transport*/)
-{
-  return PTrue;
-}
 
 void IAX2EndPoint::NewIncomingConnection(IAX2Frame *f)
 {
@@ -198,7 +211,7 @@ void IAX2EndPoint::NewIncomingConnection(IAX2Frame *f)
 
 void IAX2EndPoint::OnEstablished(OpalConnection & con)
 {
-  PTRACE(3, "Endpoint\tOnEstablished for " << con);
+  PTRACE(3, "Iax2Ep\tOnEstablished for " << con);
 
   OpalEndPoint::OnEstablished(con);
 }
@@ -224,7 +237,7 @@ PBoolean IAX2EndPoint::ConnectionForFrameIsAlive(IAX2Frame *f)
 
   PBoolean res = connectionsActive.Contains(frameToken);
   if (res) {
-    return PTrue;
+    return true;
   }
 
   mutexTokenTable.StartRead();
@@ -233,37 +246,39 @@ PBoolean IAX2EndPoint::ConnectionForFrameIsAlive(IAX2Frame *f)
 
   if (tokenTranslated.IsEmpty()) {
     PTRACE(4, "No matching translation table entry token for \"" << frameToken << "\"");
-    return PFalse;
+    return false;
   }
 
   res = connectionsActive.Contains(tokenTranslated);
   if (res) {
     PTRACE(5, "Found \"" << tokenTranslated << "\" in the connectionsActive table");
-    return PTrue;
+    return true;
   }
 
   PTRACE(6, "ERR Could not find matching connection for \"" << tokenTranslated 
 	 << "\" or \"" << frameToken << "\"");
-  return PFalse;
+  return false;
 }
 
 void IAX2EndPoint::ReportStoredConnections()
 {
-  PStringArray cons = GetAllConnections();
-  PTRACE(5, " There are " << cons.GetSize() << " stored connections in connectionsActive");
-  PINDEX i;
-  for(i = 0; i < cons.GetSize(); i++) {
-    PTRACE(5, "    #" << (i + 1) << "                     \"" << cons[i] << "\"");
-  }
+#if PTRACING
+  if (PTrace::CanTrace(5)) {
+    PStringArray cons = GetAllConnections();
+    PTRACE(5, " There are " << cons.GetSize() << " stored connections in connectionsActive");
+    PINDEX i;
+    for(i = 0; i < cons.GetSize(); i++) {
+      PTRACE(5, "    #" << (i + 1) << "                     \"" << cons[i] << "\"");
+    }
 
-  mutexTokenTable.StartRead();
-  PTRACE(5, " There are " << tokenTable.GetSize() 
-	 << " stored connections in the token translation table.");
-  for (i = 0; i < tokenTable.GetSize(); i++) {
-    PTRACE(5, " token table at " << i << " is " 
-	   << tokenTable.GetKeyAt(i) << " " << tokenTable.GetDataAt(i));
+    mutexTokenTable.StartRead();
+    PTRACE(5, " There are " << tokenTable.GetSize() 
+	   << " stored connections in the token translation table.");
+    for (PStringToString::iterator it = tokenTable.begin(); it != tokenTable.end(); ++it)
+      PTRACE(5, " token table at " << i << " is " << it->first << " " << it->second);
+    mutexTokenTable.EndRead();
   }
-  mutexTokenTable.EndRead();
+#endif
 }
 
 PStringArray IAX2EndPoint::DissectRemoteParty(const PString & other)
@@ -286,9 +301,6 @@ PStringArray IAX2EndPoint::DissectRemoteParty(const PString & other)
   } else
     working = halfs[0];
 
-  if (working.IsEmpty())
-    goto finishedDissection;
-
   halfs = working.Tokenise("$");
   if (halfs.GetSize() == 2) {
     res[transportIndex] = halfs[0];
@@ -296,34 +308,30 @@ PStringArray IAX2EndPoint::DissectRemoteParty(const PString & other)
   } else
     working = halfs[0];
 
-  if (working.IsEmpty())
-    goto finishedDissection;
-
   halfs = working.Tokenise("/");
+  res[addressIndex] = halfs[0];
   if (halfs.GetSize() == 2) {
-    res[addressIndex] = halfs[0];
     working = halfs[1];
-  } else {
-    res[addressIndex] = halfs[0];
-    goto finishedDissection;
+    halfs = working.Tokenise("+");
+    res[extensionIndex] = halfs[0];
+    if (halfs.GetSize() == 2)
+	  res[contextIndex] = halfs[1];
   }
 
-
-  halfs = working.Tokenise("+");
+  halfs = res[addressIndex].Tokenise(":");
   if (halfs.GetSize() == 2) {
-    res[extensionIndex] = halfs[0];
-    res[contextIndex]   = halfs[1];
-  } else
-    res[extensionIndex] = halfs[0];
+	res[addressIndex] = halfs[0];
+	res[portIndex] = halfs[1];
+  }
 
- finishedDissection:
-
-  PTRACE(4, "Opal\t call protocol          " << res[protoIndex]);
-  PTRACE(4, "Opal\t destination user       " << res[userIndex]);
-  PTRACE(4, "Opal\t transport to use       " << res[transportIndex]);
-  PTRACE(4, "Opal\t destination address    " << res[addressIndex]);
-  PTRACE(4, "Opal\t destination extension  " << res[extensionIndex]);
-  PTRACE(4, "Opal\t destination context    " << res[contextIndex]);
+  PTRACE2(4, NULL, "IAX2\t"
+          "call protocol          " << res[protoIndex] << "\n "
+          "destination user       " << res[userIndex] << "\n "
+          "transport to use       " << res[transportIndex] << "\n "
+          "destination address    " << res[addressIndex] << "\n "
+          "destination port       " << res[portIndex] << "\n "
+          "destination extension  " << res[extensionIndex] << "\n "
+          "destination context    " << res[contextIndex]);
 
   return res;
 }
@@ -359,7 +367,7 @@ void IAX2EndPoint::OnReleased(OpalConnection & opalCon)
 {
   IAX2Connection &con((IAX2Connection &)opalCon);
 
-  PString token(con.GetRemoteInfo().BuildOurConnectionTokenId());
+  PString token(con.GetRemoteInfo().BuildOurConnectionToken());
   mutexTokenTable.StartWrite();
   tokenTable.RemoveAt(token);
   mutexTokenTable.EndWrite();
@@ -393,7 +401,7 @@ PSafePtr<OpalConnection> IAX2EndPoint::MakeConnection(OpalCall & call,
   }
 
   PStringStream callId;
-  callId << "iax2:" <<  ip.AsString() << "OutgoingCall" << PString(++callsEstablished);
+  callId << "iax2:" <<  ip.AsString() << "Out" << PString(++callsEstablished);
   IAX2Connection * connection = CreateConnection(call, callId, userData, remotePartyName);
   if (AddConnection(connection) == NULL)
     return NULL;
@@ -460,21 +468,21 @@ PBoolean IAX2EndPoint::Initialise()
   rand.SetSeed((DWORD)(PTime().GetTimeInSeconds() + 1));
   callnumbs = PRandom::Number() % 32000;
   
-  sock = new PUDPSocket(ListenPortNumber());
+  sock = new PUDPSocket(GetDefaultSignalPort());
   PTRACE(4, "IAX2EndPoint\tCreate Socket " << sock->GetPort());
   
-  if (!sock->Listen(INADDR_ANY, 0, ListenPortNumber())) {
-    PTRACE(3, "Receiver\tFailed to listen for incoming connections on " << ListenPortNumber());
-    PTRACE(3, "Receiver\tFailed because the socket:::" << sock->GetErrorText());
-    return PFalse;
+  if (!sock->Listen(INADDR_ANY, 0, sock->GetPort())) {
+    PTRACE(2, "Receiver\tFailed to listen for incoming connections on "
+           << sock->GetPort() << " - error: " << sock->GetErrorText());
+    return false;
   }
   
-  PTRACE(6, "Receiver\tYES.. Ready for incoming connections on " << ListenPortNumber());
+  PTRACE(5, "Receiver\tYES.. Ready for incoming connections on " << sock->GetPort());
   
   transmitter = new IAX2Transmit(*this, *sock);
   receiver    = new IAX2Receiver(*this, *sock);
   
-  return PTrue;
+  return true;
 }
 
 PINDEX IAX2EndPoint::GetOutSequenceNumberForStatusQuery()
@@ -488,30 +496,46 @@ PINDEX IAX2EndPoint::GetOutSequenceNumberForStatusQuery()
 }
 
 
-PBoolean IAX2EndPoint::AddNewTranslationEntry(IAX2Frame *frame)
-{   
+PBoolean IAX2EndPoint::ProcessInConnectionTestAll(IAX2Frame *frame)
+{
   if (!frame->IsFullFrame()) {
     // Do Not have a FullFrame, so dont add a translation entry.
-    return PFalse;
+    // Only process miniframes when the call is setup, when we have a 
+    // translation table in place and working...
+    return false;
   }
-  
-  PINDEX destCallNo = frame->GetRemoteInfo().DestCallNumber();  /*Call number at our end */
-  /* We do not know if the frame is encrypted, so examination of anything other than the 
+     
+  PINDEX destCallNo = frame->GetRemoteInfo().DestCallNumber();  
+  /*destCallNo is the call number at our end. We do not know if the
+     frame is encrypted, so examination of anything other than the
      source call number/dest call number is unwise */ 
 
-  PSafePtr<IAX2Connection> connection;
-  for (connection = PSafePtrCast<OpalConnection, IAX2Connection>(connectionsActive.GetAt(0)); 
-       connection != NULL; 
-       ++connection) {
-    if (connection->GetRemoteInfo().SourceCallNumber() == destCallNo) {
-      mutexTokenTable.StartWrite();
-      tokenTable.SetAt(frame->GetConnectionToken(), connection->GetCallToken());
-      mutexTokenTable.EndWrite();
-      return PTrue;
+  PString callToken;
+  {
+    PSafePtr<IAX2Connection> connection;
+    for (connection = PSafePtrCast<OpalConnection, IAX2Connection>
+	   (connectionsActive.GetAt(0)); 
+	 connection != NULL; 
+	 ++connection) {
+      if (connection->GetRemoteInfo().SourceCallNumber() == destCallNo) {
+	PString token(frame->GetConnectionToken());
+	callToken = connection->GetCallToken();
+	if (!token.IsEmpty()) /* token available, modify token table */ {
+	  mutexTokenTable.StartWrite();
+	  tokenTable.SetAt(token, callToken);
+	  mutexTokenTable.EndWrite();
+	}
+      }
     }
   }
 
-  return PFalse;
+  if (callToken.IsEmpty()) {
+    PTRACE(3, "Iax2Ep\tFail to find home for the frame " << *frame);
+    return false;
+  }
+
+  PTRACE(5, "Iax2Ep\tProcess " << *frame << " in connection" << callToken);
+  return ProcessFrameInConnection(frame, callToken);
 }
 
 
@@ -527,20 +551,31 @@ PBoolean IAX2EndPoint::ProcessInMatchingConnection(IAX2Frame *f)
   if (tokenTranslated.IsEmpty()) 
     tokenTranslated = f->GetConnectionToken();
 
-  IAX2Connection *connection;
-  connection = PSafePtrCast<OpalConnection, IAX2Connection>(connectionsActive.FindWithLock(tokenTranslated));
-  if (connection != NULL) {
-    connection->IncomingEthernetFrame(f);
-    return PTrue;
+  if (tokenTranslated.IsEmpty()) {
+    PTRACE(3, "Distribution\tERR Could not find matching connection "
+	   << "for incoming frame of " << f->GetRemoteInfo());
+    return false;
   }
-  
-  PTRACE(3, "ERR Could not find matching connection for \"" << tokenTranslated 
-	 << "\" or \"" << f->GetConnectionToken() << "\"");
-  return PFalse;
+
+  return ProcessFrameInConnection(f, tokenTranslated);
 }
 
-
-
+PBoolean IAX2EndPoint::ProcessFrameInConnection(IAX2Frame *f, 
+						const PString & token)
+{
+  IAX2Connection *connection;
+  connection = PSafePtrCast<OpalConnection, IAX2Connection>
+    (connectionsActive.FindWithLock(token));
+  if (connection != NULL) {
+    PTRACE(5, "Distribution\tHave a connection for " << f->GetRemoteInfo());
+    connection->IncomingEthernetFrame(f);
+    return true;
+  }
+  
+  PTRACE(3, "Distribution\tERR Could not find matching connection for \"" 
+	 << token << "\" or \"" << f->GetConnectionToken() << "\"");
+  return false;
+}
 
 //The receiving thread has finished reading a frame, and has droppped it here.
 //At this stage, we do not know the frame type. We just know the frame is
@@ -569,11 +604,8 @@ void IAX2EndPoint::ProcessReceivedEthernetFrames()
       continue;
     }
 
-    if (AddNewTranslationEntry(f)) {
-      if (ProcessInMatchingConnection(f)) {
+    if (ProcessInConnectionTestAll(f))
 	continue;
-      }
-    }
 
     /**These packets cannot be encrypted, as they are not going to a phone call */
     IAX2Frame *af = f->BuildAppropriateFrameType();
@@ -623,18 +655,17 @@ void IAX2EndPoint::ProcessReceivedEthernetFrames()
 
 PINDEX IAX2EndPoint::GetPreferredCodec(OpalMediaFormatList & list)
 {
-  PTRACE(3, "preferred codecs are " << list);
+  PTRACE(4, "Iax2Ep\tPreferred codecs are " << list);
 
   for (OpalMediaFormatList::iterator iterFormat = list.begin(); iterFormat != list.end(); ++iterFormat) {
     unsigned short val = IAX2FullFrameVoice::OpalNameToIax2Value(*iterFormat);
     if (val != 0) {
-      PTRACE(3, "EndPoint\tPreferred codec is  " << *iterFormat);
+      PTRACE(4, "Iax2Ep\tPreferred codec is  " << *iterFormat);
       return val;
     }
-
   }
   
-  PTRACE(3, "Preferred codec is empty");
+  PTRACE(4, "Preferred codec is empty");
   return 0;
 }
 
@@ -675,13 +706,13 @@ void IAX2EndPoint::GetCodecLengths(PINDEX codec, PINDEX &compressedBytes, PINDEX
 
 PINDEX IAX2EndPoint::GetSupportedCodecs(OpalMediaFormatList & list)
 {
-  PTRACE(3, "Supported codecs are " << list);
+  PTRACE(4, "Iax2Ep\tSupported codecs are " << list);
 
   PINDEX returnValue = 0;
   for (OpalMediaFormatList::iterator iterFormat = list.begin(); iterFormat != list.end(); ++iterFormat)
     returnValue += IAX2FullFrameVoice::OpalNameToIax2Value(*iterFormat);
 
-  PTRACE(5, "Bitmask of codecs we support is 0x" << ::hex << returnValue << ::dec);
+  PTRACE(5, "Iax2Ep\tBitmask of codecs we support is 0x" << ::hex << returnValue << ::dec);
   
   return  returnValue;
 }
@@ -778,11 +809,11 @@ PBoolean IAX2EndPoint::IsRegistered(const PString & host, const PString & userna
       
     if (regProcessor->GetHost() == host &&
       regProcessor->GetUserName() == username) {
-      return PTrue;
+      return true;
     }
   }
   
-  return PFalse;
+  return false;
 }
 
 PINDEX IAX2EndPoint::GetRegistrationsCount() {
@@ -795,7 +826,7 @@ PINDEX IAX2EndPoint::GetRegistrationsCount() {
 IAX2IncomingEthernetFrames::IAX2IncomingEthernetFrames() 
   : PThread(1000, NoAutoDeleteThread, NormalPriority, "IAX Incoming")
 {
-  keepGoing = PTrue;
+  keepGoing = true;
 }
 
 void IAX2IncomingEthernetFrames::Assign(IAX2EndPoint *ep)
@@ -807,7 +838,7 @@ void IAX2IncomingEthernetFrames::Assign(IAX2EndPoint *ep)
 void IAX2IncomingEthernetFrames::Terminate()
 {
   PTRACE(3, "Distribute\tEnd of thread - have received a terminate signal");
-  keepGoing = PFalse;
+  keepGoing = false;
   ProcessList();
 }
 
@@ -815,7 +846,8 @@ void IAX2IncomingEthernetFrames::Main()
 {
   SetThreadName("Distribute to Cons");
   while (keepGoing) {
-    activate.Wait();
+    if (!endpoint->EthernetFramesToBeProcessed())
+      activate.Wait();
     endpoint->ProcessReceivedEthernetFrames();
   }
 
@@ -823,18 +855,18 @@ void IAX2IncomingEthernetFrames::Main()
   return;
 }
 
-void IAX2IncomingEthernetFrames::ProcessList()
-{
-  activate.Signal(); 
-}
 
+#else
+
+  #ifdef _MSC_VER
+    #pragma message("IAX2 support DISABLED")
+  #endif
 
 #endif // OPAL_IAX2
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 2 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file. 
+ * With the comment below, the tab key does auto indent to 2 spaces.     
+ *
  * Local Variables:
  * mode:c
  * c-basic-offset:2
