@@ -24,13 +24,14 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21293 $
+ * $Revision: 28483 $
  * $Author: rjongbloed $
- * $Date: 2008-10-12 18:24:41 -0500 (Sun, 12 Oct 2008) $
+ * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
 #include <opal/buildopts.h>
+#include <ptclib/random.h>
 
 #if OPAL_IAX2
 
@@ -48,7 +49,7 @@
 
 IAX2Ie::IAX2Ie()
 {
-  validData = PFalse;
+  validData = false;
 }
 
 IAX2Ie * IAX2Ie::BuildInformationElement(BYTE _typeCode, BYTE length, BYTE *srcData)
@@ -105,8 +106,8 @@ IAX2Ie * IAX2Ie::BuildInformationElement(BYTE _typeCode, BYTE length, BYTE *srcD
   case ie_recDelay        : return new IAX2IeReceivedDelay(length, srcData);
   case ie_recDropped      : return new IAX2IeDroppedFrames(length, srcData);
   case ie_recOoo          : return new IAX2IeReceivedOoo(length, srcData);
-    
-  default: PTRACE(1, "Ie\t Invalid IE type code " << ::hex << ((int)_typeCode) << ::dec);
+  case ie_callToken       : return new IAX2IeCallToken(length, srcData);
+  default: PTRACE2(1, NULL, "Ie\t Invalid IE type code " << ::hex << ((int)_typeCode) << ::dec);
   };
   
   return new IAX2IeInvalidElement();
@@ -138,7 +139,7 @@ void IAX2Ie::WriteBinary(void *_data, PINDEX &writeIndex)
 
 IAX2IeNone::IAX2IeNone(BYTE /*length*/, BYTE * /*srcData*/)
 {
-  validData = PTrue;
+  validData = true;
 }
 
 
@@ -152,7 +153,7 @@ void IAX2IeNone::PrintOn(ostream & str) const
 IAX2IeDateAndTime::IAX2IeDateAndTime(BYTE length, BYTE *srcData)
 {
   if (length != sizeof(unsigned int)) {
-    validData = PFalse;
+    validData = false;
     return;
   }
   
@@ -165,7 +166,7 @@ IAX2IeDateAndTime::IAX2IeDateAndTime(BYTE length, BYTE *srcData)
   int year   = ((tmp >> 25) & 0x7f) + 2000;
   dataValue = PTime(second, minute, hour, day, month, year, PTime::Local);
   
-  validData = PTrue;
+  validData = true;
 }
 
 
@@ -195,11 +196,11 @@ IAX2IeByte::IAX2IeByte(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(BYTE)) {
-    validData = PFalse;
+    validData = false;
     return;
   }
   
-  validData = PTrue;
+  validData = true;
   dataValue = *((BYTE *)srcData);
 }
 
@@ -217,11 +218,11 @@ IAX2IeChar::IAX2IeChar(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(BYTE)) {
-    validData = PFalse;
+    validData = false;
     return;
   }
   
-  validData = PTrue;
+  validData = true;
   dataValue = *((char *)srcData);
 }
 
@@ -240,11 +241,11 @@ IAX2IeUShort::IAX2IeUShort(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(unsigned short)) {
-    validData = PFalse;
+    validData = false;
     return;
   }
   
-  validData = PTrue;
+  validData = true;
   dataValue = (unsigned short)((srcData[0] << 8) | srcData[1]);
 }
 
@@ -268,11 +269,11 @@ IAX2IeShort::IAX2IeShort(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(short)) {
-    validData = PFalse;
+    validData = false;
     return;
   }
   
-  validData = PTrue;
+  validData = true;
   dataValue = (short)((srcData[0] << 8) | srcData[1]);
 }
 
@@ -296,11 +297,11 @@ IAX2IeInt::IAX2IeInt(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(int)) {
-    validData = PFalse;
+    validData = false;
     return;
   }
   
-  validData = PTrue;
+  validData = true;
   dataValue = (srcData[0] << 24) | (srcData[1] << 16) | (srcData[2] << 8) | srcData[3];
 }
 
@@ -327,11 +328,11 @@ IAX2IeUInt::IAX2IeUInt(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(unsigned int)) {
-    validData = PFalse;
+    validData = false;
     return;
   }
   
-  validData = PTrue;
+  validData = true;
   dataValue = (srcData[0] << 24) | (srcData[1] << 16) | (srcData[2] << 8) | srcData[3];     
 }
 
@@ -356,7 +357,7 @@ void IAX2IeUInt::WriteBinary(BYTE *data)
 IAX2IeString::IAX2IeString(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
-  validData = PTrue;
+  validData = true;
   dataValue = PString((const char *)srcData, length);
 }
 
@@ -372,10 +373,10 @@ void IAX2IeString::PrintOn(ostream & str) const
 void IAX2IeString::WriteBinary(BYTE *data)
 {
   if(validData)
-    memcpy(data, dataValue.GetPointer(), GetLengthOfData());
+    memcpy(data, (const char *)dataValue, GetLengthOfData());
 }
 
-BYTE IAX2IeString::GetLengthOfData() 
+BYTE IAX2IeString::GetLengthOfData() const
 { 
   if (dataValue.GetSize() == 0)
     return 0;
@@ -386,13 +387,65 @@ BYTE IAX2IeString::GetLengthOfData()
 void IAX2IeString::SetData(const PString & newData) 
 { 
   dataValue = newData; 
-  validData = PTrue; 
+  validData = true; 
 }
 
 void IAX2IeString::SetData(const char * newData) 
 { 
   dataValue = PString(newData); 
-  validData = PTrue; 
+  validData = true; 
+}
+
+/////////////////////////////////////////////////////////////////////////////
+IAX2IeBinary::IAX2IeBinary(BYTE length, BYTE *srcData)
+  : IAX2Ie()
+{
+  validData = true;
+  if (length == 0)
+    dataValue = PBYTEArray(); /*A zero sized array */
+  else
+    dataValue = PBYTEArray(srcData, length);
+}
+
+void IAX2IeBinary::PrintOn(ostream & str) const
+{
+  if (validData)
+    str << setw(17) << Class() << " " << dataValue;
+  else
+    str << setw(17) << Class() << " does not hold valid data" ;
+}
+
+void IAX2IeBinary::WriteBinary(BYTE *data)
+{
+  if(validData) {
+    memcpy(data, dataValue.GetPointer(), GetLengthOfData());
+  }
+  else {
+    PTRACE(3, "Iax2IeBinary\tError - major error in IAX2IeBinary");
+    PAssertAlways("Attempt to use faulty data");
+  }
+}
+    
+void IAX2IeBinary::GetData(PBYTEArray & answer)
+{
+  if (!validData) {
+    answer.SetSize(0);
+    return;
+  }
+
+  answer = dataValue;
+}
+
+void IAX2IeBinary::CopyData(IAX2IeBinary *src)
+{
+  dataValue = src->dataValue;
+  validData = src->validData;
+}
+
+void IAX2IeBinary::SetData(const PBYTEArray & newData) 
+{ 
+  dataValue = newData; 
+  validData = true; 
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -401,11 +454,11 @@ IAX2IeSockaddrIn::IAX2IeSockaddrIn(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(sockaddr_in)) {
-    validData = PFalse;
+    validData = false;
     return;
   }
   
-  validData = PTrue;
+  validData = true;
   
   sockaddr_in a = * (sockaddr_in *)(srcData);
   portNumber = a.sin_port;
@@ -425,6 +478,7 @@ void IAX2IeSockaddrIn::PrintOn(ostream & str) const
 void IAX2IeSockaddrIn::WriteBinary(BYTE *data)
 {
   sockaddr_in a;
+  memset(&a, 0, sizeof(sockaddr_in));
   a.sin_addr = (in_addr)dataValue;
   a.sin_port = (unsigned short)portNumber;
   
@@ -436,7 +490,7 @@ void IAX2IeSockaddrIn::WriteBinary(BYTE *data)
 IAX2IeBlockOfData::IAX2IeBlockOfData(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
-  validData = PTrue;
+  validData = true;
   
   dataValue = PBYTEArray(srcData, length);
 }
@@ -454,58 +508,134 @@ void IAX2IeBlockOfData::WriteBinary(BYTE *data)
 }
 
 
-////////////////////////////////////////////////////////////////////////////////
-
-IAX2IeList::~IAX2IeList()
+///////////////////////////////////////////////////////////////////////////
+void IAX2IeCallToken::PrintOn(ostream & strm) const
 {
-    AllowDeleteObjects();
+  if (validData) 
+    strm << Class() << " " << dataValue.GetSize() 
+	 << " bytes ";
+  else
+    strm << "Invalid contents in " << Class();
 }
 
-IAX2Ie *IAX2IeList::RemoveIeAt(PINDEX i)
-{ 
-  if (i >= GetSize())
-    return NULL;
-  
-  return (IAX2Ie *) PAbstractList::RemoveAt(i);
-}
 
-IAX2Ie *IAX2IeList::RemoveLastIe()
+void IAX2IeCallToken::InitialiseKey()
 {
-  PINDEX elems = PAbstractList::GetSize();
-  if (elems > 0) {
-    return RemoveIeAt(elems - 1);	  
+  PINDEX i;
+  for (i = 0; i < blockSize; i++) {
+    iKeyPad[i] = iKeyValue;
+    oKeyPad[i] = oKeyValue;
   }
+  PStringStream msg;
+  msg << ::hex << PRandom::Number() << ::dec;
+  secretKey = msg;
+
+  BYTE block[20];
+  memcpy(block, (const char *)secretKey, secretKey.GetLength());
+  for (i = 0; i < secretKey.GetLength(); i++) {
+    iKeyPad[i] ^= block[i];
+    oKeyPad[i] ^= block[i];
+  }
+}
+    
+void IAX2IeCallToken::WriteKeySequence(PIPSocket::Address & remote)
+{
+  PString srcTime = PString(PTime().GetTimeInSeconds());
+  PString answer = srcTime + PString("?") + ReportKeySequence(srcTime, remote);
   
-  return NULL;
+  PBYTEArray data;
+  data.SetSize(answer.GetLength());
+  memcpy(data.GetPointer(), (const char *)answer, answer.GetLength());
+  SetData(data);
 }
 
-void IAX2IeList::DeleteAt(PINDEX idex)
+PBoolean IAX2IeCallToken::ValidKeySequence (IAX2IeCallToken & cf, 
+					    PIPSocket::Address & remote)
 {
-  if (idex >= PAbstractList::GetSize())
-    return;
+  time_t startTime = PTime().GetTimeInSeconds();
+  PBYTEArray srcData;
+  cf.GetData(srcData);
+  PString src((const char *)srcData.GetPointer(), srcData.GetSize());
+  PStringList bits = src.Tokenise("?");
+  if (bits.GetSize() != 2)
+    return false;
+  time_t srcTime = bits[0].AsInteger();
+
+  if ((srcTime != startTime)        && 
+      (srcTime != (startTime + 1))  && 
+      (srcTime != (startTime + 2))) 
+    return false;
+
+  PString keySequence = ReportKeySequence(bits[0], remote);
   
-  IAX2Ie *obj = RemoveIeAt(idex);
-  
-  delete obj;
+  return keySequence == bits[1];
 }
 
+#if P_SSL
+PString IAX2IeCallToken::ReportKeySequence(const PString & srcTime, PIPSocket::Address & remote)
+{
+  BYTE block[1000];
+
+  memcpy(block, iKeyPad, blockSize);
+  PString message = srcTime + remote.AsString();
+  memcpy(block + blockSize, (const char *)message, message.GetLength());
+  
+  PMessageDigest::Result bin_digest;
+  PMessageDigestSHA1::Encode((const void *)block,
+			     blockSize + message.GetLength(), bin_digest);
+  memcpy(block, oKeyPad, blockSize);
+  const BYTE *data = bin_digest.GetPointer();
+  memcpy(block + blockSize, data, bin_digest.GetSize());
+  PMessageDigestSHA1::Encode((const void *)block,
+			     blockSize + bin_digest.GetSize(),
+			     bin_digest);
+  
+  PString answer;
+  PINDEX max;
+  PINDEX i;
+  for (i = 0, max = bin_digest.GetSize(); i < max ; i++)
+    answer.sprintf("%02x", (unsigned)data[i]);
+
+  return answer;
+}
+#else 
+PString IAX2IeCallToken::ReportKeySequence(const PString &, PIPSocket::Address &)
+{
+  PAssertAlways("SSL should be enabled in the build when call token is used");
+  return PString::Empty();
+}
+#endif
+
+
+/************
+  These variables are static, as they are only accessed from the
+  IAX2IeCallToken class, and they never ever change during program
+  execution. they are initialised and setup at program startup, which
+  saves time on processing each incoming call packet*/
+
+/**The key used in generating the SHA1-HMAC hash. Key is required for
+   handling incoming calls. This key is set to a random value (hex
+   representation of a random number, which is in the range of
+   1..2^32 */
+PString IAX2IeCallToken::secretKey;
+
+/**Used for generating the SHA1-HMAC hash.  Initialised at program
+     startup, saves on time later. */
+BYTE IAX2IeCallToken::iKeyPad[IAX2IeCallToken::blockSize];
+
+/**Similar to the iKeyPad, an internal variable*/
+BYTE IAX2IeCallToken::oKeyPad[IAX2IeCallToken::blockSize];
+/////////////////////////////////////////////////////////////////////////////
+
 int IAX2IeList::GetBinaryDataSize() const
 {
   PINDEX totalSize = 0;
-  for (PINDEX i = 0; i < PAbstractList::GetSize(); i++)
-    totalSize += GetIeAt(i)->GetBinarySize();
+  for (IAX2IeList::const_iterator it = begin(); it != end(); ++it)
+    totalSize += it->GetBinarySize();
   
   return totalSize;
 }
 
-IAX2Ie *IAX2IeList::GetIeAt(int i) const
-{
-  if (i >= GetSize())
-    return NULL;
-  
-  return (IAX2Ie *)GetAt(i); 
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 
 void IAX2IeCalledNumber::PrintOn(ostream & str) const
@@ -1003,13 +1133,11 @@ void IAX2IeReceivedOoo::PrintOn(ostream & str) const
 #endif // OPAL_IAX2
 
 ////////////////////////////////////////////////////////////////////////////////
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 4 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file.
+ * With the comment below, the tab key does auto indent to 2 spaces.    
+ *
  * Local Variables:
  * mode:c
- * c-file-style:linux
  * c-basic-offset:2
  * End:
  */
