@@ -28,9 +28,9 @@
  * The processor parts were abstracted out by  by Stephen Cook (sitiveni@gmail.com)
  * and this work was sponsored by the Google summer of code 
  *
- * $Revision: 21293 $
- * $Author: rjongbloed $
- * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
+ * $Revision: 28937 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 07:21:17 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -81,15 +81,15 @@ PBoolean IAX2WaitingForAck::MatchingAckPacket(IAX2FullFrame *f)
   PTRACE(4, "MatchingAck\tCompare " << timeStamp << " and " << seqNo);
   if (f->GetTimeStamp() != timeStamp) {
     PTRACE(4, "MatchingAck\tTimstamps differ");
-    return PFalse;
+    return false;
   }
 
   if (f->GetSequenceInfo().OutSeqNo() != seqNo) {
     PTRACE(4, "MatchingAck\tOut seqnos differ");    
-    return PFalse;
+    return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 void IAX2WaitingForAck::PrintOn(ostream & strm) const
@@ -113,10 +113,10 @@ PString IAX2WaitingForAck::GetResponseAsString() const
 ////////////////////////////////////////////////////////////////////////////////
 
 IAX2Processor::IAX2Processor(IAX2EndPoint &ep)
-  : PThread(1000, NoAutoDeleteThread, NormalPriority, "IAX Processor"),
+  : PThread(1000, NoAutoDeleteThread, NormalPriority, "IAX2 Processor"),
     endpoint(ep)
 {
-  endThread = PFalse;
+  endThread = false;
   
   remote.SetDestCallNumber(0);
   remote.SetRemoteAddress(0);
@@ -125,7 +125,7 @@ IAX2Processor::IAX2Processor(IAX2EndPoint &ep)
   nextTask.ZeroValues();
   noResponseTimer.SetNotifier(PCREATE_NOTIFIER(OnNoResponseTimeoutStart));
   
-  specialPackets = PFalse;
+  specialPackets = false;
 
   currentSoundTimeStamp = 0;
 }
@@ -157,12 +157,12 @@ PString IAX2Processor::GetCallToken()
 
 void IAX2Processor::Main()
 {
-  PTRACE(3, "Start of iax2 processing thread");
+  PTRACE(3, "Processor\tStart of iax2 processing thread");
   PString name = GetThreadName();
   if (IsHandlingSpecialPackets())
     SetThreadName("Special Iax packets");
 
-  while(endThread == PFalse) {
+  while(endThread == false) {
     activate.Wait();
     ProcessLists();
   }
@@ -174,24 +174,24 @@ void IAX2Processor::Main()
 PBoolean IAX2Processor::IsStatusQueryEthernetFrame(IAX2Frame *frame)
 {
   if (!PIsDescendant(frame, IAX2FullFrame))
-    return PFalse;
+    return false;
    
   IAX2FullFrame *f = (IAX2FullFrame *)frame;
   if (f->GetFrameType() != IAX2FullFrame::iax2ProtocolType)
-    return PFalse;
+    return false;
    
   PINDEX subClass = f->GetSubClass();
    
   if (subClass == IAX2FullFrameProtocol::cmdLagRq) {
     PTRACE(4, "Special packet of  lagrq to process");
-    return PTrue;
+    return true;
   }
    
   if (subClass == IAX2FullFrameProtocol::cmdPing) {
     PTRACE(4, "Special packet of Ping to process");
-    return PTrue;
+    return true;
   }
-  return PFalse;
+  return false;
 }
 
 void IAX2Processor::IncomingEthernetFrame(IAX2Frame *frame)
@@ -219,7 +219,7 @@ void IAX2Processor::StartNoResponseTimer(PINDEX msToWait)
   noResponseTimer = PTimeInterval(msToWait); 
 }
 
-void IAX2Processor::OnNoResponseTimeoutStart(PTimer &, INT)
+void IAX2Processor::OnNoResponseTimeoutStart(PTimer &, P_INT_PTR)
 {
   //call sub class to alert that there was a timeout for a response from the server
   OnNoResponseTimeout();
@@ -232,11 +232,12 @@ void IAX2Processor::Activate()
 
 void IAX2Processor::Terminate()
 {
-  endThread = PTrue;
+  endThread = true;
   if (IsSuspended())
     Resume();
 
-  PTRACE(4, "Processor has been directed to end. " << (IsTerminated() ? "Has already ended" : "So end now."));
+  PTRACE(4, "Processor\tProcessor has been directed to end. " 
+	 << (IsTerminated() ? "Has already ended" : "So end now."));
   
   Activate();
 }
@@ -245,49 +246,62 @@ PBoolean IAX2Processor::ProcessOneIncomingEthernetFrame()
 {  
   IAX2Frame *frame = frameList.GetLastFrame();
   if (frame == NULL) {
-    return PFalse;
+    return false;
   }
-  
   //check the frame has not already been built
   if (!PIsDescendant(frame, IAX2MiniFrame) && !PIsDescendant(frame, IAX2FullFrame)) {
-    PTRACE(5, "Iax2Connection\tUnknown  incoming frame " << frame->IdString());
+    PTRACE(5, "Procesor\tUnknown  incoming frame " << frame->IdString()
+	   << " " << frame->GetRemoteInfo()
+	   << " " << frame->Class());
     IAX2Frame *af = frame->BuildAppropriateFrameType(encryption);
     delete frame;
     
     if (af == NULL)
-      return PTrue;  
+      return true;  
       
     frame = af;  
   }  
   
   if (PIsDescendant(frame, IAX2MiniFrame)) {
-    PTRACE(5, "IaxConnection\tIncoming mini frame" << frame->IdString());
+    PTRACE(5, "Processor\tIncoming mini frame" << frame->IdString());
     ProcessNetworkFrame((IAX2MiniFrame *)frame);
-    return PTrue;
+    return true;
   }
   
   IAX2FullFrame *f = (IAX2FullFrame *) frame;
-  PTRACE(5, "IaxConnection\tFullFrame incoming frame " << frame->IdString());
+  PTRACE(5, "Processor\tFullFrame incoming frame " << frame->IdString());
 
-  if (IncomingMessageOutOfOrder(f))
-    return PTrue;
+  /*This test must go here, before the duplicate frame check. The
+    other end can get the sequence numbers wrong at call startup. */
+  if ((remote.DestCallNumber() == 0) &&
+      (!f->IsCallTokenFrame())  ){
+    PTRACE(3, "Processor\tSet Destination call number to " 
+	   << frame->GetRemoteInfo().SourceCallNumber());
+    remote.SetDestCallNumber(frame->GetRemoteInfo().SourceCallNumber());
+  }
+
+
+  /*Due to the vagaries of the internet etc, the remote end can send
+    us a duplicate copy of a frame we have already processed. In this
+    case, we have to ignore the new frame. We never receive multiple miniframes. */
+  if (IncomingMessageOutOfOrder(f)) {
+    PTRACE(5, "Processor\tFullFrame incoming frame " 
+	   << frame->GetRemoteInfo() << " is out of order");
+    return true;
+  }
 
   // sequence numbers are Ok. Good.
   endpoint.transmitter->PurgeMatchingFullFrames(f);
   
   IncControlFramesRcvd();
-
-  if (remote.DestCallNumber() == 0) {
-    PTRACE(3, "Set Destination call number to " << frame->GetRemoteInfo().SourceCallNumber());
-    remote.SetDestCallNumber(frame->GetRemoteInfo().SourceCallNumber());
-  }
+  PTRACE(3, "Processor\t Our remote info is " << remote);
   
   ProcessFullFrame(*f);
  
   if (f != NULL)
     delete f;
 
-  return PTrue;   /*There could be more frames to process. */
+  return true;   /*There could be more frames to process. */
 }
 
 void IAX2Processor::TransmitFrameNow(IAX2Frame *src)
@@ -308,7 +322,7 @@ void IAX2Processor::TransmitFrameToRemoteEndpoint(IAX2FullFrameProtocol *src)
 
 void IAX2Processor::TransmitFrameToRemoteEndpoint(IAX2Frame *src)
 {
-  PTRACE(5, "Send frame " << src->GetClass() << " " << src->IdString() << " to remote endpoint");
+  PTRACE(5, "Processor\tSend frame " << src->GetClass() << " " << src->IdString() << " to remote endpoint");
   if (src->IsFullFrame()) {
     sequence.MassageSequenceForSending(*(IAX2FullFrame*)src);
     IncControlFramesSent();
@@ -327,26 +341,29 @@ void IAX2Processor::TransmitFrameToRemoteEndpoint(IAX2FullFrame *src,
 
 PBoolean IAX2Processor::Authenticate(IAX2FullFrameProtocol *reply, PString & password)
 {
-  PBoolean processed = PFalse;
+  PBoolean processed = false;
   IAX2IeAuthMethods ie(ieData.authMethods);
   
   if (ie.IsMd5Authentication()) {
     PTRACE(5, "Processor\tMD5 Authentiction yes, make reply up");
     IAX2IeMd5Result *res = new IAX2IeMd5Result(ieData.challenge, password);
     reply->AppendIe(res);
-    processed = PTrue;
+    processed = true;
     encryption.SetChallengeKey(ieData.challenge);
     encryption.SetEncryptionKey(password);
-  } else if (ie.IsPlainTextAuthentication()) {
-    /*TODO: in the future we might want a policy of only
-    allowing md5 passwords.  This would make
-    injecting plain auth IEs useless.*/
-    reply->AppendIe(new IAX2IePassword(password));
-    processed = PTrue;
-  } else if (ie.IsRsaAuthentication()) {
-    PTRACE(4, "DO NOT handle RSA authentication ");
-    reply->SetSubClass(IAX2FullFrameProtocol::cmdInval);
-    processed = PTrue;
+  } else  {
+    if (ie.IsPlainTextAuthentication()) {
+      /*TODO: in the future we might want a policy of only
+	allowing md5 passwords.  This would make
+	injecting plain auth IEs useless.*/
+      reply->AppendIe(new IAX2IePassword(password));
+      processed = true;
+    } else 
+      if (ie.IsRsaAuthentication()) {
+	PTRACE(4, "Processor\tDO NOT handle RSA authentication ");
+	reply->SetSubClass(IAX2FullFrameProtocol::cmdInval);
+	processed = true;
+      }
   }
   
   if (ieData.encryptionMethods == IAX2IeEncryption::encryptAes128) {
@@ -414,10 +431,10 @@ PBoolean IAX2Processor::ProcessNetworkFrame(IAX2FullFrameProtocol * src)
     ProcessIaxCmdPong(src);
     break;
   default:
-    return PFalse;
+    return false;
   }
   
-  return PTrue;
+  return true;
 }
 
 void IAX2Processor::ProcessIaxCmdPing(IAX2FullFrameProtocol *src)
