@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21421 $
- * $Author: dereksmithies $
- * $Date: 2008-10-30 00:36:33 +0000 (Thu, 30 Oct 2008) $
+ * $Revision: 28483 $
+ * $Author: rjongbloed $
+ * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -67,28 +67,28 @@ PBoolean IAX2Remote::operator==(IAX2Remote & other)
   if (remoteAddress != other.RemoteAddress()) {
     PTRACE(5, "Comparison of two remotes " << endl << other << endl << (*this) );
     PTRACE(5, "comparison of two remotes  Addresses are different");
-    return PFalse;
+    return false;
   }
   
   if (remotePort != other.RemotePort()) {
     PTRACE(5, "Comparison of two remotes " << endl << other << endl << (*this) );
     PTRACE(5, "comparison of two remotes  remote ports are different");
-    return PFalse;
+    return false;
   }
   
   if (destCallNumber != other.DestCallNumber()) {
     PTRACE(5, "Comparison of two remotes " << endl << other << endl << (*this) );
     PTRACE(5, "comparison of two remotes. Dest call numbers differ");	
-    return PFalse;
+    return false;
   }
   
   if (sourceCallNumber != other.SourceCallNumber()) {
     PTRACE(5, "Comparison of two remotes " << endl << other << endl << (*this) );
     PTRACE(5, "comparison of two remotes. Source call numbers differ");	
-    return PFalse;
+    return false;
   }
   
-  return PTrue;
+  return true;
 }
 
 
@@ -98,23 +98,23 @@ PBoolean IAX2Remote::operator*=(IAX2Remote & other)
   
   if (remoteAddress != other.RemoteAddress()) {
     PTRACE(3, "comparison of two remotes  Addresses are different");
-    return PFalse;
+    return false;
   }
   
   if (remotePort != other.RemotePort()) {
     PTRACE(5, "comparison of two remotes  remote ports are different");
-    return PFalse;
+    return false;
   }
   
   if (sourceCallNumber != other.DestCallNumber()) {
     PTRACE(5, "comparison of two remotes. Local source number differs to incoming dest call number");
     PTRACE(5, " local sourceCallNumber " << sourceCallNumber 
 	   << "        incoming Dest " << other.DestCallNumber());
-    return PFalse;
+    return false;
   }
   
   PTRACE(6, "comparison of two remotes  They are the same  ");
-  return PTrue;
+  return true;
 }
 
 
@@ -123,15 +123,20 @@ PBoolean IAX2Remote::operator!=(IAX2Remote & other)
   return !(*this == other);
 }
 
-void IAX2Remote::PrintOn(ostream & strm) const
-{
-  strm << "src call number" << sourceCallNumber 
-       << "        Dest call number" << destCallNumber 
-       << "        remote address" << remoteAddress 
-       << "        Remote port" << remotePort ;
+void IAX2Remote::SetDestCallNumber(PINDEX newVal) 
+{ 
+  destCallNumber = newVal; 
 }
 
-PString IAX2Remote::BuildConnectionTokenId()
+void IAX2Remote::PrintOn(ostream & strm) const
+{
+  strm << "src" << sourceCallNumber 
+       << " dest" << destCallNumber 
+       << " " << remoteAddress 
+       << ":" << remotePort ;
+}
+
+PString IAX2Remote::BuildConnectionToken()
 {
   return PString("iax2:") 
     + RemoteAddress().AsString() 
@@ -139,7 +144,7 @@ PString IAX2Remote::BuildConnectionTokenId()
     + PString(SourceCallNumber());  
 }
 
-PString IAX2Remote::BuildOurConnectionTokenId()
+PString IAX2Remote::BuildOurConnectionToken()
 {
   return PString("iax2:") 
     + RemoteAddress().AsString() 
@@ -218,7 +223,7 @@ PINDEX IAX2PacketIdList::GetFirstValue()
 
 void IAX2PacketIdList::RemoveOldContiguousValues()
 {
-  PBoolean contiguous = PTrue;
+  PBoolean contiguous = true;
   while((GetSize() > 1) && contiguous)  {
     PINDEX first = ((IAX2FrameIdValue *)GetAt(0))->GetPlainSequence();
     PINDEX second = ((IAX2FrameIdValue *)GetAt(1))->GetPlainSequence();
@@ -285,18 +290,18 @@ PBoolean  IAX2SequenceNumbers::operator != (IAX2SequenceNumbers &other)
 {
   PWaitAndSignal m(mutex);
   if (inSeqNo == other.InSeqNo())
-    return PFalse;
+    return false;
   
   if (inSeqNo == other.OutSeqNo())
-    return PFalse;
+    return false;
   
   if (outSeqNo == other.InSeqNo())
-    return PFalse;
+    return false;
   
   if (outSeqNo == other.OutSeqNo())
-    return PFalse;
+    return false;
   
-  return PTrue;
+  return true;
 }
 
 void IAX2SequenceNumbers::SetAckSequenceInfo(IAX2SequenceNumbers & other)
@@ -309,13 +314,13 @@ PBoolean IAX2SequenceNumbers::operator == (IAX2SequenceNumbers &other)
 {
   PWaitAndSignal m(mutex);
   if ((inSeqNo == other.InSeqNo()) && (outSeqNo == other.OutSeqNo()))
-    return PTrue;
+    return true;
   
   
   if ((inSeqNo == other.OutSeqNo()) && (outSeqNo == other.InSeqNo()))
-    return PTrue;
+    return true;
   
-  return PFalse;
+  return false;
 }
 
 
@@ -420,7 +425,7 @@ PBoolean IAX2SequenceNumbers::IsFirstReplyFrame()
 PBoolean IAX2SequenceNumbers::IsSequenceNosZero()
 {
   PWaitAndSignal m(mutex);
-  return ((inSeqNo && 0xff) == 0) && ((outSeqNo && 0xff) == 0); 
+  return ((inSeqNo & 0xff) == 0) && ((outSeqNo & 0xff) == 0); 
 }
 
 void IAX2SequenceNumbers::SetInSeqNo(PINDEX newVal) 
@@ -446,7 +451,7 @@ void IAX2SequenceNumbers::SetInOutSeqNo(PINDEX inVal, PINDEX outVal)
 ////////////////////////////////////////////////////////////////////////////////
 IAX2Encryption::IAX2Encryption() 
 { 
-  encryptionEnabled = PFalse; 
+  encryptionEnabled = false; 
 }
 
 
@@ -483,17 +488,6 @@ PBoolean IAX2Encryption::IsEncrypted() const
   return encryptionEnabled;
 }
 
-#if OPAL_PTLIB_SSL_AES
-AES_KEY *IAX2Encryption::AesEncryptKey()
-{
-  return &aesEncryptKey; 
-}
-
-AES_KEY *IAX2Encryption::AesDecryptKey()
-{
-  return &aesDecryptKey;
-}
-#endif
 
 void IAX2Encryption::CalculateAesKeys()
 {
@@ -504,25 +498,24 @@ void IAX2Encryption::CalculateAesKeys()
     return;
 
 
-#if OPAL_PTLIB_SSL_AES
+#ifdef P_SSL_AES
   IAX2IeMd5Result ie(*this);
   PBYTEArray context = ie.GetDataBlock();
 
   PTRACE(6, "Decryption\tContext has a size of " << context.GetSize());
 
-  AES_set_encrypt_key(context.GetPointer(), 128, &aesEncryptKey);
-  AES_set_decrypt_key(context.GetPointer(), 128, &aesDecryptKey);
+  m_aesEncryptKey.SetEncrypt(context.GetPointer(), 128);
+  m_aesDecryptKey.SetDecrypt(context.GetPointer(), 128);
 #endif
 }
   
 #endif // OPAL_IAX2
 
-////////////////////////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////////////////////
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 2 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file.
+ * With the comment below, the tab key does auto indent to 2 spaces.    
+ *
  * Local Variables:
  * mode:c
  * c-basic-offset:2
