@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21293 $
+ * $Revision: 28483 $
  * $Author: rjongbloed $
- * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
+ * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -79,12 +79,12 @@ PBoolean SafeStrings::GetNextString(PString & nextString)
   PWaitAndSignal m(accessMutex);
   
   if (data.GetSize() == 0)
-    return PFalse;
+    return false;
   
   nextString = data[0];
   data.RemoveAt(0);
   
-  return PTrue;
+  return true;
 }
 
 
@@ -162,13 +162,11 @@ PString SafeString::GetAndDelete()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 4 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file.
+ * With the comment below, the tab key does auto indent to 2 spaces.     
+ *
  * Local Variables:
  * mode:c
- * c-file-style:linux
  * c-basic-offset:2
  * End:
  */
