@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21293 $
+ * $Revision: 25515 $
  * $Author: rjongbloed $
- * $Date: 2008-10-12 18:24:41 -0500 (Sun, 12 Oct 2008) $
+ * $Date: 2011-04-06 01:51:53 -0500 (Wed, 06 Apr 2011) $
  */
 
 #include <ptlib.h>
@@ -62,21 +62,21 @@ PBYTEArray * IAX2SoundList::GetLastEntry()
   if (size == 0)
     return NULL;
   
-  return (PBYTEArray *) RemoveAt(size - 1);
+  return (PBYTEArray *)RemoveTail();
 }
 
 void IAX2SoundList::AddNewEntry(PBYTEArray *newElem)
 {
   PWaitAndSignal m(mutex);
   
-  InsertAt(0, newElem);
+  Prepend(newElem);
 }
 
 void IAX2SoundList::GetAllDeleteAll(IAX2SoundList &dest)
 {
   PWaitAndSignal m(mutex);
   while(PAbstractList::GetSize() > 0) {
-    dest.AddNewEntry((PBYTEArray *)RemoveAt(0));
+    dest.AddNewEntry((PBYTEArray *)RemoveHead());
   }
 }
 
@@ -86,10 +86,9 @@ void IAX2SoundList::GetAllDeleteAll(IAX2SoundList &dest)
 #endif // OPAL_IAX2
 
 ////////////////////////////////////////////////////////////////////////////////
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 4 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file. 
+ * With the comment below, the tab key does auto indent to 2 spaces.     
+ *
  * Local Variables:
  * mode:c
  * c-file-style:linux
