@@ -23,42 +23,121 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25019 $
- * $Author: willamowius $
- * $Date: 2011-01-06 03:44:41 -0600 (Thu, 06 Jan 2011) $
+ * $Revision: 29743 $
+ * $Author: rjongbloed $
+ * $Date: 2013-05-19 22:43:02 -0500 (Sun, 19 May 2013) $
  */
 
-#include <ptlib.h>
-#include <opal/buildopts.h>
-
 #ifdef __GNUC__
 #pragma implementation "msrp.h"
 #endif
 
-#include <ptlib/socket.h>
-#include <ptclib/random.h>
-#include <ptclib/pdns.h>
-#include <ptclib/mime.h>
-#include <ptclib/http.h>
+#include <ptlib.h>
 
-#include <opal/transports.h>
-#include <opal/mediatype.h>
-#include <opal/mediafmt.h>
-#include <opal/endpoint.h>
-
-#include <im/im.h>
 #include <im/msrp.h>
 
 #if OPAL_HAS_MSRP
 
+#include <im/im.h>
+
+#include <ptclib/pdns.h>
+#include <ptclib/http.h>
+
+#include <opal/connection.h>
+#include <opal/endpoint.h>
+#include <im/t140.h>
+
+#if OPAL_SIP
+#include <sip/sdp.h>
+#endif
+
+
 #define CRLF "\r\n"
 
-const char MSRP[] = "msrp";
+const PCaselessString & OpalMSRPMediaSession::TCP_MSRP() { static const PConstCaselessString s("TCP/MSRP"); return s; }
 
-OpalMSRPMediaType::OpalMSRPMediaType()
-  : OpalIMMediaType(MSRP, "message|tcp/msrp")
+static OpalMediaSessionFactory::Worker<OpalMSRPMediaSession> tcp_msrp_session(OpalMSRPMediaSession::TCP_MSRP());
+
+class OpalMSRPMediaType : public OpalMediaTypeDefinition 
 {
-}
+  public:
+    static const char * Name() { return OPAL_IM_MEDIA_TYPE_PREFIX"msrp"; }
+
+    OpalMSRPMediaType()
+      : OpalMediaTypeDefinition(Name(), OpalMSRPMediaSession::TCP_MSRP())
+    {
+    }
+
+#if OPAL_SIP
+    static const PCaselessString & GetSDPMediaType();
+    static const PCaselessString & GetSDPTransportType();
+
+    virtual bool MatchesSDP(
+      const PCaselessString & sdpMediaType,
+      const PCaselessString & sdpTransport,
+      const PStringArray & sdpLines,
+      PINDEX index
+    );
+
+    virtual SDPMediaDescription * CreateSDPMediaDescription(
+      const OpalTransportAddress & localAddress
+    ) const;
+#endif // OPAL_SIP
+};
+
+OPAL_INSTANTIATE_MEDIATYPE(OpalMSRPMediaType);
+
+
+/////////////////////////////////////////////////////////////////////////////
+
+#define DECLARE_MSRP_ENCODING(title, encoding) \
+class IM##title##OpalMSRPEncoding : public OpalMSRPEncoding \
+{ \
+}; \
+static PFactory<OpalMSRPEncoding>::Worker<IM##title##OpalMSRPEncoding> worker_##IM##title##OpalMSRPEncoding(encoding, true); \
+
+
+DECLARE_MSRP_ENCODING(Text, "text/plain");
+DECLARE_MSRP_ENCODING(CPIM, "message/cpim");
+DECLARE_MSRP_ENCODING(HTML, "message/html");
+
+const OpalMediaFormat & GetOpalMSRP() 
+{ 
+  static class IMMSRPMediaFormat : public OpalMediaFormat { 
+    public: 
+      IMMSRPMediaFormat() 
+        : OpalMediaFormat(OPAL_MSRP, 
+                          OpalMSRPMediaType::Name(),
+                          RTP_DataFrame::MaxPayloadType, 
+                          "+", 
+                          false,  
+                          1440, 
+                          512, 
+                          0, 
+                          1000)            // as defined in RFC 4103 - good as anything else   
+      { 
+        PFactory<OpalMSRPEncoding>::KeyList_T types = PFactory<OpalMSRPEncoding>::GetKeyList();
+        PFactory<OpalMSRPEncoding>::KeyList_T::iterator r;
+
+        PString acceptTypes;
+        for (r = types.begin(); r != types.end(); ++r) {
+          if (!acceptTypes.IsEmpty())
+            acceptTypes += " ";
+          acceptTypes += *r;
+        }
+
+        OpalMediaOptionString * option = new OpalMediaOptionString("Accept Types", false, acceptTypes);
+        option->SetMerge(OpalMediaOption::AlwaysMerge);
+        AddOption(option);
+
+        option = new OpalMediaOptionString("Path", false, "");
+        option->SetMerge(OpalMediaOption::MaxMerge);
+        AddOption(option);
+      } 
+  } const f; 
+  return f; 
+} 
+
 
 #if OPAL_SIP
 
@@ -83,26 +162,16 @@ class SDPMSRPMediaDescription : public SDPMediaDescription
 
     PCaselessString GetSDPTransportType() const
     {
-      return "tcp/msrp";
-    }
-
-    virtual SDPMediaDescription * CreateEmpty() const
-    {
-      return new SDPMSRPMediaDescription(OpalTransportAddress());
+      return OpalMSRPMediaType::GetSDPTransportType();
     }
 
     virtual PString GetSDPMediaType() const 
     {
-      return "message";
-    }
-
-    virtual PString GetSDPPortList() const
-    {
-      return " *";
+      return OpalMSRPMediaType::GetSDPMediaType();
     }
 
     virtual void CreateSDPMediaFormats(const PStringArray &);
-    virtual bool PrintOn(ostream & str, const PString & connectString) const;
+    virtual void OutputAttributes(ostream & strm) const;
     virtual void SetAttribute(const PString & attr, const PString & value);
     virtual void ProcessMediaOptions(SDPMediaFormat & sdpFormat, const OpalMediaFormat & mediaFormat);
     virtual void AddMediaFormat(const OpalMediaFormat & mediaFormat);
@@ -122,21 +191,39 @@ class SDPMSRPMediaDescription : public SDPMediaDescription
 
 ////////////////////////////////////////////////////////////////////////////////////////////
 
-SDPMediaDescription * OpalMSRPMediaType::CreateSDPMediaDescription(const OpalTransportAddress & localAddress)
+const PCaselessString & OpalMSRPMediaType::GetSDPMediaType()     { static PConstCaselessString const s("message"); return s; }
+const PCaselessString & OpalMSRPMediaType::GetSDPTransportType() { static PConstCaselessString const s("tcp/msrp"); return s; }
+
+bool OpalMSRPMediaType::MatchesSDP(const PCaselessString & sdpMediaType,
+                                   const PCaselessString & sdpTransport,
+                                   const PStringArray & /*sdpLines*/,
+                                   PINDEX /*index*/)
 {
-  return new SDPMSRPMediaDescription(localAddress);
+  return sdpMediaType == GetSDPMediaType() && sdpTransport == GetSDPTransportType();
 }
 
+
+SDPMediaDescription * OpalMSRPMediaType::CreateSDPMediaDescription(const OpalTransportAddress & localAddress) const
+{
+  return new SDPMSRPMediaDescription(localAddress, PURL());
+}
+
+SDPMediaDescription * OpalMSRPMediaSession::CreateSDPMediaDescription()
+{
+  return new SDPMSRPMediaDescription(GetLocalAddress(), GetLocalURL());
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////////////////
 
 SDPMSRPMediaDescription::SDPMSRPMediaDescription(const OpalTransportAddress & address)
-  : SDPMediaDescription(address, MSRP)
+  : SDPMediaDescription(address, OpalMSRPMediaType::Name())
 {
   SetDirection(SDPMediaDescription::SendRecv);
 }
 
 SDPMSRPMediaDescription::SDPMSRPMediaDescription(const OpalTransportAddress & address, const PString & _path)
-  : SDPMediaDescription(address, MSRP)
+  : SDPMediaDescription(address, OpalMSRPMediaType::Name())
   , path(_path)
 {
   SetDirection(SDPMediaDescription::SendRecv);
@@ -148,17 +235,14 @@ void SDPMSRPMediaDescription::CreateSDPMediaFormats(const PStringArray &)
 }
 
 
-bool SDPMSRPMediaDescription::PrintOn(ostream & str, const PString & /*connectString*/) const
+void SDPMSRPMediaDescription::OutputAttributes(ostream & strm) const
 {
   // call ancestor. Never output the connect string, as the listening TCP sockets 
   // for the MSRP manager will always give an address of 0.0.0.0
-  if (!SDPMediaDescription::PrintOn(str, ""))
-    return false;
+  SDPMediaDescription::OutputAttributes(strm);
 
-  str << "a=accept-types:" << types << "\r\n";
-  str << "a=path:" << path << "\r\n";
-
-  return true;
+  strm << "a=accept-types:" << types << "\r\n"
+          "a=path:" << path << "\r\n";
 }
 
 void SDPMSRPMediaDescription::SetAttribute(const PString & attr, const PString & value)
@@ -171,7 +255,7 @@ void SDPMSRPMediaDescription::SetAttribute(const PString & attr, const PString &
 
 void SDPMSRPMediaDescription::ProcessMediaOptions(SDPMediaFormat & /*sdpFormat*/, const OpalMediaFormat & mediaFormat)
 {
-  if (mediaFormat.GetMediaType() == MSRP) 
+  if (mediaFormat.GetMediaType() == OpalMSRPMediaType::Name()) 
     types = mediaFormat.GetOptionString("Accept Types").Trim();
 }
 
@@ -190,7 +274,7 @@ OpalMediaFormatList SDPMSRPMediaDescription::GetMediaFormats() const
 
 void SDPMSRPMediaDescription::AddMediaFormat(const OpalMediaFormat & mediaFormat)
 {
-  if (!mediaFormat.IsTransportable() || !mediaFormat.IsValidForProtocol("sip") || mediaFormat.GetMediaType() != MSRP) {
+  if (!mediaFormat.IsTransportable() || !mediaFormat.IsValidForProtocol("sip") || mediaFormat.GetMediaType() != OpalMSRPMediaType::Name()) {
     PTRACE(4, "MSRP\tSDP not including " << mediaFormat << " as it is not a valid MSRP format");
     return;
   }
@@ -203,6 +287,7 @@ void SDPMSRPMediaDescription::AddMediaFormat(const OpalMediaFormat & mediaFormat
 
 #endif // OPAL_SIP
 
+
 ////////////////////////////////////////////////////////////////////////////////////////////
 
 class MSRPInitialiser : public PProcessStartup
@@ -233,69 +318,72 @@ protected:
 PMutex MSRPInitialiser::mutex;
 OpalMSRPManager * MSRPInitialiser::manager = NULL;
 
-static PFactory<PProcessStartup>::Worker<MSRPInitialiser> opalpluginStartupFactory("MSRP", true);
+PFACTORY_CREATE_SINGLETON(PProcessStartupFactory, MSRPInitialiser);
+
 
 ////////////////////////////////////////////////////////////////////////////////////////////
 
-OpalMSRPMediaSession::OpalMSRPMediaSession(OpalConnection & _conn, unsigned _sessionId)
-  : OpalMediaSession(_conn, MSRP, _sessionId)
-  , m_manager(MSRPInitialiser::KickStart(_conn.GetEndPoint().GetManager()))
-  , m_isOriginating(_conn.IsOriginating())
-  , m_localMSRPSessionId(m_manager.CreateSessionID())
-  , m_localUrl(m_manager.SessionIDToURL(connection.GetTransport().GetLocalAddress(), m_localMSRPSessionId))
+OpalMSRPMediaSession::OpalMSRPMediaSession(const Init & init)
+  : OpalMediaSession(init)
+  , m_manager(MSRPInitialiser::KickStart(init.m_connection.GetEndPoint().GetManager()))
 {
-  // set the local URL
 }
 
-OpalMSRPMediaSession::OpalMSRPMediaSession(const OpalMSRPMediaSession & _obj)
-  : OpalMediaSession(_obj)
-  , m_manager(_obj.m_manager)
-  , m_isOriginating(_obj.m_isOriginating)
-  , m_localMSRPSessionId(_obj.m_localMSRPSessionId)
-  , m_localUrl(_obj.m_localUrl)
-  , m_remoteUrl(_obj.m_remoteUrl)
-  , m_connectionPtr(_obj.m_connectionPtr)
-  , m_remoteAddress(_obj.m_remoteAddress)
-{
-}
 
 OpalMSRPMediaSession::~OpalMSRPMediaSession()
 {
   CloseMSRP();
 }
 
-void OpalMSRPMediaSession::Close()
+
+bool OpalMSRPMediaSession::Open(const PString & localInterface, const OpalTransportAddress & remoteAddress, bool isMediaAddress)
 {
-  CloseMSRP();
+  if (!OpalMediaSession::Open(localInterface, remoteAddress, isMediaAddress))
+    return false;
+
+  m_isOriginating = m_connection.IsOriginating();
+  m_localMSRPSessionId = m_manager.CreateSessionID();
+  m_localUrl = m_manager.SessionIDToURL(localInterface, m_localMSRPSessionId);
+  return true;
 }
 
-OpalTransportAddress OpalMSRPMediaSession::GetLocalMediaAddress() const
+
+bool OpalMSRPMediaSession::Close()
+{
+  CloseMSRP();
+  return true;
+}
+
+
+OpalTransportAddress OpalMSRPMediaSession::GetLocalAddress(bool) const
 {
   return OpalTransportAddress(m_localUrl.GetHostName(), m_localUrl.GetPort());
 }
 
-#if OPAL_SIP
 
-SDPMediaDescription * OpalMSRPMediaSession::CreateSDPMediaDescription(const OpalTransportAddress & sdpContactAddress)
+OpalTransportAddress OpalMSRPMediaSession::GetRemoteAddress(bool) const
 {
-  return new SDPMSRPMediaDescription(sdpContactAddress, m_localUrl.AsString());
+  return m_remoteAddress;
+}
+
+
+bool OpalMSRPMediaSession::SetRemoteAddress(const OpalTransportAddress & transportAddress, bool)
+{
+  PTRACE(2, "MSRP\tSetting remote media address to " << transportAddress);
+  m_remoteAddress = transportAddress;
+  return true;
 }
 
-#endif
 
 OpalMediaStream * OpalMSRPMediaSession::CreateMediaStream(const OpalMediaFormat & mediaFormat, 
                                                                          unsigned sessionID, 
                                                                          PBoolean isSource)
 {
-  PTRACE(2, "MSRP\tCreated " << (isSource ? "source" : "sink") << " media stream in " << (connection.IsOriginating() ? "originator" : "receiver") << " with " << m_localUrl);
-  return new OpalMSRPMediaStream(connection, mediaFormat, sessionID, isSource, *this);
+  PTRACE(2, "MSRP\tCreated " << (isSource ? "source" : "sink") << " media stream in "
+         << (m_connection.IsOriginating() ? "originator" : "receiver") << " with " << m_localUrl);
+  return new OpalMSRPMediaStream(m_connection, mediaFormat, sessionID, isSource, *this);
 }
 
-void OpalMSRPMediaSession::SetRemoteMediaAddress(const OpalTransportAddress & transportAddress, const OpalMediaFormatList & )
-{
-  PTRACE(2, "MSRP\tSetting remote media address to " << transportAddress);
-  m_remoteAddress = transportAddress;
-}
 
 bool OpalMSRPMediaSession::WritePacket(RTP_DataFrame & frame)
 {
@@ -303,7 +391,7 @@ bool OpalMSRPMediaSession::WritePacket(RTP_DataFrame & frame)
     PTRACE(2, "MSRP\tCannot send MSRP message as no connection has been established");
   } 
   else {
-    RTP_IMFrame * imFrame = dynamic_cast<RTP_IMFrame *>(&frame);
+    OpalT140RTPFrame * imFrame = dynamic_cast<OpalT140RTPFrame *>(&frame);
     if (imFrame != NULL) {
       PString messageId;
       T140String content;
@@ -360,35 +448,14 @@ void OpalMSRPMediaSession::SetConnection(PSafePtr<OpalMSRPManager::Connection> &
 
 ////////////////////////////////////////////////////////
 
-OpalMediaSession * OpalMSRPMediaType::CreateMediaSession(OpalConnection & conn, unsigned sessionID) const
-{
-  // as this is called in the constructor of an OpalConnection descendant, 
-  // we can't use a virtual function on OpalConnection
-#if OPAL_SIP
-  if (conn.GetPrefixName() *= "sip") {
-    PTRACE(2, "MSRP\tCreating MSRP media session for SIP connection");
-    return new OpalMSRPMediaSession(conn, sessionID);
-  }
-#endif
-
-  PTRACE(2, "MSRP\tCannot create MSRP media session for unknown connection type " << conn.GetPrefixName());
-  
-  return NULL;
-}
-
-////////////////////////////////////////////////////////
-
-OpalMSRPMediaStream::OpalMSRPMediaStream(
-      OpalConnection & conn,
-      const OpalMediaFormat & mediaFormat, ///<  Media format for stream
-      unsigned sessionID,                  ///<  Session number for stream
-      bool isSource,                       ///<  Is a source stream
-      OpalMSRPMediaSession & msrpSession
-)
-  : OpalIMMediaStream(conn, mediaFormat, sessionID, isSource)
+OpalMSRPMediaStream::OpalMSRPMediaStream(OpalConnection & connection,
+                                         const OpalMediaFormat & mediaFormat,
+                                         unsigned sessionID,
+                                         bool isSource,
+                                         OpalMSRPMediaSession & msrpSession)
+  : OpalMediaStream(connection, mediaFormat, sessionID, isSource)
   , m_msrpSession(msrpSession)
   , m_remoteParty(mediaFormat.GetOptionString("Path"))
-  , m_rfc4103Context(mediaFormat)
 {
   PTRACE(3, "MSRP\tOpening MSRP connection from " << m_msrpSession.GetLocalURL() << " to " << m_remoteParty);
   if (isSource) 
@@ -396,22 +463,26 @@ OpalMSRPMediaStream::OpalMSRPMediaStream(
                                            PCREATE_NOTIFIER2(OnReceiveMSRP, OpalMSRPManager::IncomingMSRP &));
 }
 
+
 OpalMSRPMediaStream::~OpalMSRPMediaStream()
 {
   m_msrpSession.GetManager().RemoveNotifier(m_msrpSession.GetLocalURL(), m_remoteParty);
 }
 
+
 bool OpalMSRPMediaStream::Open()
 {
   return m_msrpSession.OpenMSRP(m_remoteParty) && OpalMediaStream::Open();
 }
 
+
 PBoolean OpalMSRPMediaStream::ReadPacket(RTP_DataFrame &)
 {
   PAssertAlways("Cannot ReadData from OpalMSRPMediaStream");
   return false;
 }
 
+
 PBoolean OpalMSRPMediaStream::WritePacket(RTP_DataFrame & frame)
 {
   if (!IsOpen())
@@ -420,10 +491,6 @@ PBoolean OpalMSRPMediaStream::WritePacket(RTP_DataFrame & frame)
   return m_msrpSession.WritePacket(frame);
 }
 
-PBoolean OpalMSRPMediaStream::Close()
-{
-  return OpalIMMediaStream::Close();
-}
 
 void OpalMSRPMediaStream::OnReceiveMSRP(OpalMSRPManager &, OpalMSRPManager::IncomingMSRP & incomingMSRP)
 {
@@ -434,12 +501,15 @@ void OpalMSRPMediaStream::OnReceiveMSRP(OpalMSRPManager &, OpalMSRPManager::Inco
   } 
   else if (incomingMSRP.m_command == MSRPProtocol::SEND) {
     PTRACE(3, "MSRP\tMediaStream " << *this << " received SEND");
+    OpalIM im;
+    im.m_from = connection.GetRemotePartyURL();
+    im.m_to = connection.GetLocalPartyURL();
+
     T140String t140(incomingMSRP.m_body);
-    RTP_DataFrameList frames = m_rfc4103Context.ConvertToFrames(incomingMSRP.m_mime.GetString(PHTTP::ContentTypeTag, "text/plain"), t140);
-    OpalMediaFormat fmt(m_rfc4103Context.m_mediaFormat);
-    for (PINDEX i = 0; i < frames.GetSize(); ++i) {
-      //connection.OnReceiveExternalIM(m_rfc4103Context.m_mediaFormat, (RTP_IMFrame &)frames[i]);
-    }
+    t140.AsString(im.m_bodies[PMIMEInfo::TextPlain()]);
+
+    PString error;
+//    connection.GetEndPoint().GetManager().GetIMManager().OnMessageReceived(im, &connection, error);
   }
   else {
     PTRACE(3, "MSRP\tMediaStream " << *this << " receiving unknown MSRP message");
@@ -522,7 +592,7 @@ void OpalMSRPManager::ListenerThread()
     {
       PWaitAndSignal m(m_connectionInfoMapAddMutex);
       connection.SetSafetyMode(PSafeReference);
-      m_connectionInfoMap.insert(ConnectionInfoMapType::value_type((const char *)remoteAddr.AsString(), connection));
+      m_connectionInfoMap[remoteAddr.AsString()] = connection;
       connection.SetSafetyMode(PSafeReadWrite);
     }
     connection->StartHandler();
@@ -534,8 +604,8 @@ void OpalMSRPManager::ListenerThread()
 PSafePtr<OpalMSRPManager::Connection> OpalMSRPManager::OpenConnection(const PURL & localURL, const PURL & remoteURL)
 {
   // get hostname of remote 
-  PIPSocket::Address ip = remoteURL.GetHostName();
-  WORD port             = remoteURL.GetPort();
+  PIPSocket::Address ip(remoteURL.GetHostName());
+  WORD port = remoteURL.GetPort();
   if (!ip.IsValid()) {
     if (remoteURL.GetPortSupplied()) {
       if (!PIPSocket::GetHostAddress(remoteURL.GetHostName(), ip)) {
@@ -544,7 +614,7 @@ PSafePtr<OpalMSRPManager::Connection> OpalMSRPManager::OpenConnection(const PURL
       }
     }
     else {
-#if P_DNS
+#if P_DNS_RESOLVER
       PIPSocketAddressAndPortVector addresses;
       if (PDNS::LookupSRV(remoteURL.GetHostName(), "_im._msrp", remoteURL.GetPort(), addresses) && !addresses.empty()) {
         ip   = addresses[0].GetAddress(); // Only use first entry
@@ -556,7 +626,7 @@ PSafePtr<OpalMSRPManager::Connection> OpalMSRPManager::OpenConnection(const PURL
       }
 #else
       return NULL;
-#endif
+#endif // P_DNS_RESOLVER
     }
   }
 
@@ -578,7 +648,7 @@ PSafePtr<OpalMSRPManager::Connection> OpalMSRPManager::OpenConnection(const PURL
     }
 
     connectionPtr = PSafePtr<Connection>(new Connection(*this, connectionKey));
-    m_connectionInfoMap.insert(ConnectionInfoMapType::value_type(connectionKey, connectionPtr));
+    m_connectionInfoMap[connectionKey] = connectionPtr;
   }
 
   connectionPtr.SetSafetyMode(PSafeReadWrite);
@@ -622,7 +692,7 @@ void OpalMSRPManager::SetNotifier(const PURL & localUrl,
   PString key(localUrl.AsString() + '\t' + remoteUrl.AsString());
   PTRACE(2, "MSRP\tRegistering callback for incoming MSRP messages with '" << key << "'");
   PWaitAndSignal m(m_callBacksMutex);
-  m_callBacks.insert(CallBackMap::value_type(key, CallBack(notifier)));
+  m_callBacks[key] = notifier;
 }
 
 
@@ -890,7 +960,7 @@ bool MSRPProtocol::ReadMessage(int & command,
   do {
     if (!ReadLine(line, false)) {
       PTRACE(2, "MSRP\tError while reading MSRP command");
-      return PFalse;
+      return false;
     }
   } while (line.IsEmpty());
 
