@@ -23,74 +23,31 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24092 $
+ * $Revision: 28208 $
  * $Author: rjongbloed $
- * $Date: 2010-03-04 00:22:29 -0600 (Thu, 04 Mar 2010) $
+ * $Date: 2012-08-16 20:54:46 -0500 (Thu, 16 Aug 2012) $
  */
 
-#include <ptlib.h>
-#include <opal/buildopts.h>
-
 #ifdef __GNUC__
 #pragma implementation "t140.h"
 #endif
 
+#include <ptlib.h>
+#include <opal/buildopts.h>
+
 #include <im/t140.h>
-#include <string.h>
 
-#if OPAL_SIP
-#include <sip/sdp.h>
-#endif
-
-
-const char T140[] = "t140";
-
-
-#if OPAL_SIP
-
-/////////////////////////////////////////////////////////
-//
-//  SDP media description for text media
-//
-
-class SDPT140MediaDescription : public SDPRTPAVPMediaDescription
-{
-  PCLASSINFO(SDPT140MediaDescription, SDPRTPAVPMediaDescription);
-  public:
-    SDPT140MediaDescription(const OpalTransportAddress & address)
-      : SDPRTPAVPMediaDescription(address, T140)
-    { }
-
-    virtual PString GetSDPMediaType() const
-    {  return "text";  }
-};
-
-#endif
-
-/////////////////////////////////////////////////////////////////////////////
-
-OpalT140MediaType::OpalT140MediaType()
-  : OpalRTPAVPMediaType(T140, "text")
-{
-}
-
-#if OPAL_SIP
-
-SDPMediaDescription * OpalT140MediaType::CreateSDPMediaDescription(const OpalTransportAddress & localAddress)
-{
-  return new SDPT140MediaDescription(localAddress);
-}
-
-#endif
 
 /////////////////////////////////////////////////////////////////////////////
 
 T140String::T140String()
   : length(0)
 { 
-  AppendUnicode16(ZERO_WIDTH_NO_BREAK); 
+  AppendUnicode16(ZERO_WIDTH_NO_BREAK);
+  SetAt(length, '\0');
 }
 
+
 T140String::T140String(const PBYTEArray & bytes)
   : length(0)
 { 
@@ -100,8 +57,10 @@ T140String::T140String(const PBYTEArray & bytes)
       ch != ZERO_WIDTH_NO_BREAK)
     AppendUnicode16(ZERO_WIDTH_NO_BREAK);
   AppendUTF((const BYTE *)bytes, bytes.GetSize());
+  SetAt(length, '\0');
 }
 
+
 T140String::T140String(const BYTE * data, PINDEX len)
   : length(0)
 { 
@@ -111,8 +70,10 @@ T140String::T140String(const BYTE * data, PINDEX len)
       ch != ZERO_WIDTH_NO_BREAK)
     AppendUnicode16(ZERO_WIDTH_NO_BREAK); 
   AppendUTF((const BYTE *)data, len);
+  SetAt(length, '\0');
 }
 
+
 T140String::T140String(const char * chars)
   : length(0)
 { 
@@ -122,8 +83,10 @@ T140String::T140String(const char * chars)
       ch != ZERO_WIDTH_NO_BREAK)
     AppendUnicode16(ZERO_WIDTH_NO_BREAK); 
   AppendUTF((const BYTE *)chars, strlen(chars));
+  SetAt(length, '\0');
 }
 
+
 T140String::T140String(const PString & str)
   : length(0)
 { 
@@ -133,17 +96,12 @@ T140String::T140String(const PString & str)
       ch != ZERO_WIDTH_NO_BREAK)
     AppendUnicode16(ZERO_WIDTH_NO_BREAK); 
   AppendUTF((const BYTE *)(const char *)str, str.GetLength());
+  SetAt(length, '\0');
 }
 
+
 PINDEX T140String::AppendUTF(const BYTE * utf, PINDEX utfLen)
 {
-  WORD ch;
-  if (utfLen >= 3 &&
-      GetUTF(utf, utfLen, ch) == 3 &&
-      ch == ZERO_WIDTH_NO_BREAK) {
-    utf += 3; 
-    utfLen -= 3;
-  }
   if (utfLen > 0) {
     memcpy(GetPointer(length+utfLen)+length, utf, utfLen);
     length += utfLen;
@@ -151,6 +109,7 @@ PINDEX T140String::AppendUTF(const BYTE * utf, PINDEX utfLen)
   return utfLen;
 }
 
+
 PINDEX T140String::GetUTFLen(WORD c)
 {
   if (c <= 0x7f)
@@ -161,6 +120,7 @@ PINDEX T140String::GetUTFLen(WORD c)
   return 3;
 }
 
+
 PINDEX T140String::AppendUnicode16(WORD c)
 {
   unsigned len = GetUTFLen(c);
@@ -188,17 +148,19 @@ PINDEX T140String::SetUTF(BYTE * ptr, WORD c)
   }
 
   //if (c <= 0xffff) {
-  ptr[0] = 0xe0 | (ch >> 4) | (cl >> 6); 
-  ptr[1] = 0x80 | (ch << 2) | (cl >> 6); 
+  ptr[0] = 0xe0 | (ch >> 4); 
+  ptr[1] = 0x80 | ((ch & 0xf) << 2) | (cl >> 6); 
   ptr[2] = 0x80 | (cl & 0x3f);
   return 3;
 }
 
+
 PINDEX T140String::GetUTF(PINDEX pos, WORD & ch)
 {
   return GetUTF(GetPointer()+pos, GetSize()-pos, ch);
 }
 
+
 PINDEX T140String::GetUTF(const BYTE * ptr, PINDEX len, WORD & ch)
 {
   if (len < 1)
@@ -215,7 +177,7 @@ PINDEX T140String::GetUTF(const BYTE * ptr, PINDEX len, WORD & ch)
 
   // 0x80 .. 0x7ff
   if (ptr[0] <= 0xdf) {
-    ch = (ptr[0] << 6) | (ptr[1] & 0x3f);
+    ch = ((ptr[0] & 0x1f) << 6) | (ptr[1] & 0x3f);
     return 2;
   }
 
@@ -223,11 +185,12 @@ PINDEX T140String::GetUTF(const BYTE * ptr, PINDEX len, WORD & ch)
     return 0;
 
   // 0x800 .. 0xffff
-  ch = (ptr[0] << 12) | ((ptr[1] & 0x3f) << 6) || (ptr[2] & 0x3f);
+  ch = ((ptr[0] & 0xf) << 12) | ((ptr[1] & 0x3f) << 6) | (ptr[2] & 0x3f);
 
   return 3;
 }
 
+
 bool T140String::AsString(PString & str)
 {
   PINDEX pos = 0;
@@ -249,3 +212,83 @@ bool T140String::AsString(PString & str)
 
   return true;
 }
+
+
+/////////////////////////////////////////////////////////////////////////////
+
+OpalT140RTPFrame::OpalT140RTPFrame(const RTP_DataFrame & frame)
+  : RTP_DataFrame(frame)
+{
+}
+
+
+OpalT140RTPFrame::OpalT140RTPFrame(const BYTE * data, PINDEX len, bool dynamic)
+  : RTP_DataFrame(data, len, dynamic)
+{
+}
+
+
+OpalT140RTPFrame::OpalT140RTPFrame()
+{
+  SetPayloadSize(0);
+}
+
+
+OpalT140RTPFrame::OpalT140RTPFrame(const PString & contentType)
+{
+  SetPayloadSize(0);
+  SetContentType(contentType);
+}
+
+
+OpalT140RTPFrame::OpalT140RTPFrame(const PString & contentType, const T140String & content)
+{
+  SetPayloadSize(0);
+  SetContentType(contentType);
+  SetContent(content);
+}
+
+
+void OpalT140RTPFrame::SetContentType(const PString & contentType)
+{
+  SetHeaderExtension(0, contentType.GetLength(), (const BYTE *)(const char *)contentType, RTP_DataFrame::RFC3550);
+}
+
+
+PString OpalT140RTPFrame::GetContentType() const
+{
+  unsigned id;
+  PINDEX length;
+  BYTE * ptr = GetHeaderExtension(id, length);
+  return ptr != NULL ? PString((const char *)ptr, length) : PString::Empty();
+}
+
+
+void OpalT140RTPFrame::SetContent(const T140String & text)
+{
+  SetPayloadSize(text.GetSize());
+  memcpy(GetPayloadPtr(), (const BYTE *)text, text.GetSize());
+}
+
+
+bool OpalT140RTPFrame::GetContent(T140String & text) const
+{
+  if (GetPayloadSize() == 0) 
+    return false;
+
+  text = T140String((const BYTE *)GetPayloadPtr(), GetPayloadSize());
+  return true;
+}
+
+
+bool OpalT140RTPFrame::GetContent(PString & str) const
+{
+  if (GetPayloadSize() == 0) 
+    return false;
+
+  T140String text((const BYTE *)GetPayloadPtr(), GetPayloadSize());
+  return text.AsString(str);
+}
+
+
+//////////////////////////////////////////////////////////////////////////////////////////
