@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27341 $
- * $Author: rjongbloed $
- * $Date: 2012-03-27 20:12:14 -0500 (Tue, 27 Mar 2012) $
+ * $Revision: 28937 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 07:21:17 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -33,6 +33,10 @@
 
 #if OPAL_CAPI
 
+  #ifdef _MSC_VER
+    #pragma message("CAPI ISDN support enabled")
+  #endif
+
 #ifdef _WIN32
 #include "capi_win32.h"
 #else
@@ -40,7 +44,7 @@
 #endif
 
 
-PURL_LEGACY_SCHEME(isdn, true, false, true, true, false, true, true, false, false, false, 0)
+PURL_LEGACY_SCHEME(isdn, true, false, true, true, false, false, true, false, false, false, 0)
 
 enum {
     MaxLineCount = 30,
@@ -218,17 +222,6 @@ struct OpalCapiMessage
     BYTE m_Params[200]; // Space for command specific parameters
   } param;
 
-  struct Bprotocol {
-    Bprotocol() { memset(this, 0, sizeof(*this)); }
-    WORD m_B1protocol;
-    WORD m_B2protocol;
-    WORD m_B3protocol;
-    BYTE m_B1config;      // Zero length place holder
-    BYTE m_B2config;      // Zero length place holder
-    BYTE m_B3config;      // Zero length place holder
-    BYTE m_GlobalConfig;  // Zero length place holder
-  };
-
   OpalCapiMessage(unsigned command, unsigned subcommand, size_t fixedParamSize)
   {
     header.m_Length = (WORD)(sizeof(Header)+fixedParamSize);
@@ -318,6 +311,7 @@ struct OpalCapiProfile {
   BYTE  m_Reserved[64];       // Make sure struct is big enough
 };
 
+#pragma pack()
 
 #if PTRACING
   #define MAKE_MSG_NAME(cmd,sub) { CAPICMD(CAPI_##cmd, CAPI_##sub), #cmd "_" #sub }
@@ -507,6 +501,24 @@ static BYTE const ReverseBits[256] = {
 };
 
 
+static struct {
+  WORD m_B1protocol;
+  WORD m_B2protocol;
+  WORD m_B3protocol;
+  BYTE m_B1config;      // Zero length place holder
+  BYTE m_B2config;      // Zero length place holder
+  BYTE m_B3config;      // Zero length place holder
+  BYTE m_GlobalConfig;  // Zero length place holder
+} const DefaultBprotocol = {
+  1, // Physical layer: 64 kbit/s bit-transparent operation with byte framing from the network
+  1, // Data link layer: Transparent
+  0, // Network layer: Transparent
+  0, // Zero length place holder
+  0, // Zero length place holder
+  0, // Zero length place holder
+  0, // Zero length place holder
+};
+
 ///////////////////////////////////////////////////////////////////////
 
 OpalCapiEndPoint::OpalCapiEndPoint(OpalManager & manager)
@@ -580,7 +592,8 @@ PSafePtr<OpalConnection> OpalCapiEndPoint::MakeConnection(OpalCall & call,
 
   stringOptions->ExtractFromURL(uri);
 
-  unsigned controller, bearer;
+  unsigned controller = uri.GetHostName().AsUnsigned();
+  unsigned bearer = uri.GetPort();
   if (GetFreeLine(controller, bearer)) {
     OpalCapiConnection * connection = CreateConnection(call, userData, options, stringOptions, controller, bearer);
     if (AddConnection(connection) != NULL) {
@@ -617,6 +630,9 @@ OpalCapiConnection * OpalCapiEndPoint::CreateConnection(OpalCall & call,
 
 PString OpalCapiEndPoint::GetDriverInfo() const
 {
+  if (m_capi == NULL)
+    return PString::Empty();
+
   OpalCapiProfile profile;
   DWORD capiversion_min;
   DWORD capiversion_maj;
@@ -648,7 +664,7 @@ PString OpalCapiEndPoint::GetDriverInfo() const
 unsigned OpalCapiEndPoint::OpenControllers()
 {
   {
-    PWaitAndSignal mutex(inUseFlag);
+    PWaitAndSignal mutex(m_controllerMutex);
 
     if (m_thread == NULL || m_applicationId == 0) {
       PTRACE(1, "CAPI\tNot registered with drivers, or not installed.");
@@ -680,7 +696,7 @@ unsigned OpalCapiEndPoint::OpenControllers()
         continue;
 
       PTRACE(3, "CAPI\tListening to controller " << controller);
-      m_controllers[controller].m_bearerInUse.resize(profile.m_NumBChannels);
+      m_controllers[controller].m_bearerInUse.resize(profile.m_NumBChannels+1);
 
       // Start listening for incoming calls
       OpalCapiMessage message(CAPI_LISTEN, CAPI_REQ, sizeof(OpalCapiMessage::Params::ListenReq));
@@ -705,22 +721,46 @@ unsigned OpalCapiEndPoint::OpenControllers()
 
 bool OpalCapiEndPoint::GetFreeLine(unsigned & controller, unsigned & bearer)
 {
-  PWaitAndSignal mutex(inUseFlag);
+  PWaitAndSignal mutex(m_controllerMutex);
+
+  if (controller != 0)
+    return controller < m_controllers.size() && m_controllers[controller].GetFreeLine(bearer);
 
   for (controller = 1; controller < m_controllers.size(); ++controller) {
-    for (bearer = 0; bearer < m_controllers[controller].m_bearerInUse.size(); ++bearer) {
-      if (m_controllers[controller].m_active && !m_controllers[controller].m_bearerInUse[bearer]) {
-        m_controllers[controller].m_bearerInUse[bearer] = true;
-        return true;
-      }
-    }
+    if (m_controllers[controller].GetFreeLine(bearer))
+      return true;
   }
 
+  controller = 0;
   return false;
 }
 
 
-void OpalCapiEndPoint::ProcessMessages(PThread &, INT)
+bool OpalCapiEndPoint::Controller::GetFreeLine(unsigned & bearer)
+{
+  if (!m_active)
+    return false;
+
+  if (bearer != 0) {
+    if (bearer >= m_bearerInUse.size() || m_bearerInUse[bearer])
+      return false;
+    m_bearerInUse[bearer] = true;
+    return true;
+  }
+
+  for (bearer = 1; bearer < m_bearerInUse.size(); ++bearer) {
+    if (!m_bearerInUse[bearer]) {
+      m_bearerInUse[bearer] = true;
+      return true;
+    }
+  }
+
+  bearer = 0;
+  return false;
+}
+
+
+void OpalCapiEndPoint::ProcessMessages(PThread &, P_INT_PTR)
 {
   PTRACE(4, "CAPI\tStarted message thread.");
 
@@ -819,7 +859,8 @@ void OpalCapiEndPoint::ProcessMessage(const OpalCapiMessage & message)
 
 void OpalCapiEndPoint::ProcessConnectInd(const OpalCapiMessage & message)
 {
-  unsigned controller, bearer;
+  unsigned controller = (message.param.connect_ind.m_PLCI&0xff);
+  unsigned bearer = 0;
   if (GetFreeLine(controller, bearer)) {
     // Get new instance of a call, abort if none created
     OpalCall * call = manager.InternalCreateCall();
@@ -870,6 +911,7 @@ OpalCapiConnection::OpalCapiConnection(OpalCall & call,
   , m_bearer(bearer)
   , m_PLCI(0)
   , m_NCCI(0)
+  , m_Bprotocol((const BYTE *)&DefaultBprotocol, sizeof(DefaultBprotocol))
 {
 }
 
@@ -880,9 +922,20 @@ bool OpalCapiConnection::IsNetworkConnection() const
 }
 
 
+void OpalCapiConnection::OnApplyStringOptions()
+{
+  OpalConnection::OnApplyStringOptions();
+
+  PStringStream bProto(m_stringOptions(OPAL_OPT_CAPI_B_PROTO));
+  if (!bProto.IsEmpty())
+    bProto >> m_Bprotocol;
+}
+
+
 PBoolean OpalCapiConnection::SetUpConnection()
 {
-  originating = true;
+  InternalSetAsOriginating();
+
   SetPhase(SetUpPhase);
 
   OnApplyStringOptions();
@@ -896,11 +949,7 @@ PBoolean OpalCapiConnection::SetUpConnection()
   message.AddEmpty(); // Called party subaddress
   message.AddEmpty(); // Calling party subaddress
 
-  OpalCapiMessage::Bprotocol proto;
-  proto.m_B1protocol = 1; // Physical layer: 64 kbit/s bit-transparent operation with byte framing from the network
-  proto.m_B2protocol = 1; // Data link layer: Transparent
-  proto.m_B3protocol = 0; // Network layer: Transparent
-  message.Add(&proto, sizeof(proto)); // B protocol
+  message.Add(m_Bprotocol, m_Bprotocol.GetSize());
 
   message.AddEmpty(); // Bearer Capabilities
   message.AddEmpty(); // Low Layer Compatibility
@@ -917,11 +966,7 @@ PBoolean OpalCapiConnection::SetAlerting(const PString & /*calleeName*/, PBoolea
   OpalCapiMessage message(CAPI_ALERT, CAPI_REQ, sizeof(OpalCapiMessage::Params::AlertReq));
   message.param.alert_req.m_PLCI = m_PLCI;
 
-  OpalCapiMessage::Bprotocol proto;
-  proto.m_B1protocol = 1; // Physical layer: 64 kbit/s bit-transparent operation with byte framing from the network
-  proto.m_B2protocol = 1; // Data link layer: Transparent
-  proto.m_B3protocol = 0; // Network layer: Transparent
-  message.Add(&proto, sizeof(proto)); // B protocol
+  message.Add(m_Bprotocol, m_Bprotocol.GetSize());
 
   message.AddEmpty(); // Keypad facility
   message.AddEmpty(); // User-user data
@@ -938,11 +983,7 @@ PBoolean OpalCapiConnection::SetConnected()
   OpalCapiMessage message(CAPI_CONNECT, CAPI_RESP, sizeof(OpalCapiMessage::Params::ConnectResp));
   message.param.connect_resp.m_PLCI = m_PLCI;
   message.param.connect_resp.m_Reject = 0;
-  OpalCapiMessage::Bprotocol proto;
-  proto.m_B1protocol = 1; // Physical layer: 64 kbit/s bit-transparent operation with byte framing from the network
-  proto.m_B2protocol = 1; // Data link layer: Transparent
-  proto.m_B3protocol = 0; // Network layer: Transparent
-  message.Add(&proto, sizeof(proto)); // B protocol
+  message.Add(m_Bprotocol, m_Bprotocol.GetSize());
   message.Add(localPartyName); // Connected party number
   message.AddEmpty(); // Connected party subaddress
   message.AddEmpty(); // Low Layer Compatibility
@@ -1030,7 +1071,6 @@ void OpalCapiConnection::ProcessMessage(const OpalCapiMessage & message)
       PINDEX pos = sizeof(OpalCapiMessage::Header) + sizeof(message.param.connect_ind);
       message.Get(pos, NULL, 1, m_calledPartyNumber); // Called party address
       message.Get(pos, NULL, 2, remotePartyNumber); // Calling party address
-      remotePartyAddress = remotePartyNumber;
 
       OnApplyStringOptions();
       if (OnIncomingConnection(0, NULL))
@@ -1061,7 +1101,7 @@ void OpalCapiConnection::ProcessMessage(const OpalCapiMessage & message)
       resp.param.connect_active_resp.m_PLCI = m_PLCI;
       PutMessage(resp);
 
-      if (originating) {
+      if (IsOriginating()) {
         OpalCapiMessage req(CAPI_CONNECT_B3, CAPI_REQ, sizeof(OpalCapiMessage::Params::ConnectB3Req));
         req.param.connect_b3_req.m_PLCI = m_PLCI;
         req.AddEmpty(); // Network Control Protocol Information
@@ -1300,6 +1340,12 @@ PBoolean OpalCapiMediaStream::IsSynchronous() const
 }
 
 
+#else
+
+  #ifdef _MSC_VER
+    #pragma message("CAPI ISDN support DISABLED")
+  #endif
+
 #endif // OPAL_CAPI
 
 
