@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21368 $
- * $Author: rjongbloed $
- * $Date: 2008-10-16 09:46:21 +0000 (Thu, 16 Oct 2008) $
+ * $Revision: 28932 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 06:34:20 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -53,7 +53,7 @@ ostream & operator<<(ostream & o, OpalLineInterfaceDevice::CallProgressTones t)
   static const char * const CallProgressTonesNames[OpalLineInterfaceDevice::NumTones+1] = {
     "NoTone", "DialTone", "RingTone", "BusyTone", "CongestionTone", "ClearTone", "MwiTone", "RoutingTone", "CNGTone", "CEDTone", "UserTone"
   };
-  if (t+1 < PARRAYSIZE(CallProgressTonesNames) && CallProgressTonesNames[t+1] != NULL)
+  if ((size_t)t+1 < PARRAYSIZE(CallProgressTonesNames) && CallProgressTonesNames[t+1] != NULL)
     return o << CallProgressTonesNames[t+1];
   else
     return o << "UnknownTone:" << (unsigned)t;
@@ -90,28 +90,28 @@ PBoolean OpalLineInterfaceDevice::IsOpen() const
 PBoolean OpalLineInterfaceDevice::Close()
 {
   if (os_handle < 0)
-    return PFalse;
+    return false;
 
   os_handle = -1;
-  return PTrue;
+  return true;
 }
 
 
 PBoolean OpalLineInterfaceDevice::IsLinePresent(unsigned, PBoolean)
 {
-  return PTrue;
+  return true;
 }
 
 
 PBoolean OpalLineInterfaceDevice::HookFlash(unsigned line, unsigned flashTime)
 {
   if (!IsLineOffHook(line))
-    return PFalse;
+    return false;
 
   if (!SetLineOnHook(line))
-    return PFalse;
+    return false;
 
-  PThread::Current()->Sleep(flashTime);
+  PThread::Sleep(flashTime);
 
   return SetLineOffHook(line);
 }
@@ -119,19 +119,19 @@ PBoolean OpalLineInterfaceDevice::HookFlash(unsigned line, unsigned flashTime)
 
 PBoolean OpalLineInterfaceDevice::HasHookFlash(unsigned)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::IsLineRinging(unsigned, DWORD *)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::RingLine(unsigned, PINDEX, const unsigned *, unsigned)
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -158,27 +158,27 @@ PBoolean OpalLineInterfaceDevice::IsLineDisconnected(unsigned line, PBoolean)
 
 PBoolean OpalLineInterfaceDevice::SetLineToLineDirect(unsigned, unsigned, PBoolean)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::IsLineToLineDirect(unsigned, unsigned)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::StopReading(unsigned)
 {
   m_readDeblockingOffset = P_MAX_INDEX;
-  return PTrue;
+  return true;
 }
 
 
 PBoolean OpalLineInterfaceDevice::StopWriting(unsigned)
 {
   m_writeDeblockingOffset = 0;
-  return PTrue;
+  return true;
 }
 
 
@@ -236,18 +236,19 @@ PBoolean OpalLineInterfaceDevice::ReadBlock(unsigned line, void * buffer, PINDEX
     else if (length < frameSize) {
       BYTE * deblockPtr = m_readDeblockingBuffer.GetPointer(frameSize);
       if (!ReadFrame(line, deblockPtr, readBytes))
-        return PFalse;
+        return false;
       m_readDeblockingOffset = 0;
     }
     else {
+      readBytes = frameSize;
       if (!ReadFrame(line, bufferPtr, readBytes))
-        return PFalse;
+        return false;
       bufferPtr += readBytes;
       length -= readBytes;
     }
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -278,7 +279,7 @@ PBoolean OpalLineInterfaceDevice::WriteBlock(unsigned line, const void * buffer,
     // straight on to the device.
     if (m_writeDeblockingOffset == 0 && length >= frameSize) {
       if (!WriteFrame(line, bufferPtr, frameSize, written))
-        return PFalse;
+        return false;
       bufferPtr += written;
       length -= written;
     }
@@ -290,7 +291,7 @@ PBoolean OpalLineInterfaceDevice::WriteBlock(unsigned line, const void * buffer,
         // Nope, just copy bytes into buffer and return
         memcpy(savedFramePtr + m_writeDeblockingOffset, bufferPtr, length);
         m_writeDeblockingOffset += length;
-        return PTrue;
+        return true;
       }
 
       /* Calculate bytes we want from the passed in buffer to fill a frame by
@@ -304,14 +305,14 @@ PBoolean OpalLineInterfaceDevice::WriteBlock(unsigned line, const void * buffer,
 
       // Write the saved frame out
       if (!WriteFrame(line, savedFramePtr, frameSize, written))
-        return PFalse;
+        return false;
 
       bufferPtr += left;
       length -= left;
     }
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -340,23 +341,23 @@ PBoolean OpalLineInterfaceDevice::IsAudioEnabled(unsigned line) const
 
 PBoolean OpalLineInterfaceDevice::SetRecordVolume(unsigned, unsigned)
 {
-  return PFalse;
+  return false;
 }
 
 PBoolean OpalLineInterfaceDevice::GetRecordVolume(unsigned, unsigned &)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::SetPlayVolume(unsigned, unsigned)
 {
-  return PFalse;
+  return false;
 }
 
 PBoolean OpalLineInterfaceDevice::GetPlayVolume(unsigned, unsigned &)
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -368,7 +369,7 @@ OpalLineInterfaceDevice::AECLevels OpalLineInterfaceDevice::GetAEC(unsigned) con
 
 PBoolean OpalLineInterfaceDevice::SetAEC(unsigned, AECLevels)
 {
-  return PFalse;
+  return false;
 }
 
 unsigned OpalLineInterfaceDevice::GetWinkDuration(unsigned)
@@ -379,43 +380,43 @@ unsigned OpalLineInterfaceDevice::GetWinkDuration(unsigned)
 
 PBoolean OpalLineInterfaceDevice::SetWinkDuration(unsigned, unsigned)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::GetVAD(unsigned) const
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::SetVAD(unsigned, PBoolean)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::GetCallerID(unsigned, PString & id, PBoolean)
 {
   id = PString();
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::SetCallerID(unsigned, const PString &)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::SendVisualMessageWaitingIndicator(unsigned, PBoolean)
 {
-  return PFalse;
+  return false;
 }
 
 PBoolean OpalLineInterfaceDevice::PlayDTMF(unsigned, const char *, DWORD, DWORD)
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -427,13 +428,13 @@ char OpalLineInterfaceDevice::ReadDTMF(unsigned)
 
 PBoolean OpalLineInterfaceDevice::GetRemoveDTMF(unsigned)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::SetRemoveDTMF(unsigned, PBoolean)
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -459,7 +460,7 @@ OpalLineInterfaceDevice::CallProgressTones OpalLineInterfaceDevice::WaitForToneD
       return tones;
     }
 
-    PThread::Current()->Sleep(sampleRate);
+    PThread::Sleep(sampleRate);
     retry++;
   } while (retry < timeout);
 
@@ -488,13 +489,8 @@ bool OpalLineInterfaceDevice::SetToneDescription(unsigned line,
     return true;
 
   PString freqDesc, cadenceDesc;
-  PINDEX colon = description.Find(':');
-  if (colon == P_MAX_INDEX)
+  if (!description.Split(':', freqDesc, cadenceDesc))
     freqDesc = description;
-  else {
-    freqDesc = description.Left(colon);
-    cadenceDesc = description.Mid(colon+1);
-  }
 
   ToneMixingModes mode = SimpleTone;
   unsigned low_freq, high_freq;
@@ -517,7 +513,7 @@ bool OpalLineInterfaceDevice::SetToneDescription(unsigned line,
       (mode != ModulatedTone ? (high_freq < 100 || high_freq > 3000 || low_freq > high_freq)
                              : (high_freq < 5 || high_freq > 100))) {
     PTRACE(1, "LID\tIllegal frequency specified: " << description);
-    return PFalse;
+    return false;
   }
 
   PStringArray times = cadenceDesc.Tokenise("-");
@@ -528,7 +524,7 @@ bool OpalLineInterfaceDevice::SetToneDescription(unsigned line,
     double time = times[i].AsReal();
     if (time <= 0.01 || time > 10) {
       PTRACE(1, "LID\tIllegal cadence time specified: " << description);
-      return PFalse;
+      return false;
     }
 
     if ((i&1) == 0)
@@ -550,51 +546,51 @@ bool OpalLineInterfaceDevice::SetToneParameters(unsigned /*line*/,
                                                 const unsigned * /*onTimes*/,
                                                 const unsigned * /*offTimes*/)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::PlayTone(unsigned, CallProgressTones)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::IsTonePlaying(unsigned)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::StopTone(unsigned)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::PlayAudio(unsigned /*line*/, const PString & /*filename*/)
 {
-  PTRACE(2, "LID\tBase Class PlayAudio method called, exiting with PFalse");
-  return PFalse;
+  PTRACE(2, "LID\tBase Class PlayAudio method called, exiting with false");
+  return false;
 }
 
 
 PBoolean OpalLineInterfaceDevice::StopAudio(unsigned /*line*/)
 {
-  PTRACE(2, "LID\tBase Class StopAudio method called, exiting with PFalse");
-  return PFalse;
+  PTRACE(2, "LID\tBase Class StopAudio method called, exiting with false");
+  return false;
 }
 	
 PBoolean OpalLineInterfaceDevice::RecordAudioStart(unsigned /*line*/, const PString & /*fn*/)
 {
   PTRACE(2, "LID\tRecordAudioStart must be implemented in concrete class");
-  return PFalse;
+  return false;
 }
 
 PBoolean OpalLineInterfaceDevice::RecordAudioStop(unsigned /*line*/)
 {
   PTRACE(2, "LID\tRecordAudioStop must be implemented in concrete class");
-  return PFalse;
+  return false;
 }
 
 OpalLineInterfaceDevice::CallProgressTones
@@ -627,7 +623,7 @@ OpalLineInterfaceDevice::CallProgressTones
   if (params.m_dialStartDelay > 0) {
     /* wait before dialing*/
     PTRACE(3, "LID\tDialOut wait " << params.m_dialStartDelay << "msec before dialing");
-    PThread::Current()->Sleep(params.m_dialStartDelay);
+    PThread::Sleep(params.m_dialStartDelay);
   }
 
   // Dial the string
@@ -651,7 +647,7 @@ OpalLineInterfaceDevice::CallProgressTones
         break;
 
       case ',' :
-        PThread::Current()->Sleep(params.m_commaDelay);
+        PThread::Sleep(params.m_commaDelay);
         break;
     }
   }
@@ -952,7 +948,7 @@ PBoolean OpalLineInterfaceDevice::SetCountryCodeName(const PString & countryName
   PTRACE(4, "LID\tSetting country code name to \"" << countryName << '"');
   PCaselessString spacelessAndCaseless = DeSpaced(countryName);
   if (spacelessAndCaseless.IsEmpty())
-    return PFalse;
+    return false;
 
   if (isdigit(spacelessAndCaseless[0]))
     return SetCountryCode((T35CountryCodes)spacelessAndCaseless.AsUnsigned());
@@ -976,7 +972,7 @@ PBoolean OpalLineInterfaceDevice::SetCountryCodeName(const PString & countryName
   }
 
   SetCountryCode(UnknownCountry);
-  return PFalse;
+  return false;
 }
 
 
@@ -1010,15 +1006,10 @@ OpalLineInterfaceDevice * OpalLineInterfaceDevice::CreateAndOpen(const PString &
                                                                  void * parameters)
 {
   PString deviceType, deviceName;
-
-  PINDEX colon = descriptor.Find(':');
-  if (colon != P_MAX_INDEX) {
-    deviceType = descriptor.Left(colon).Trim();
-    deviceName = descriptor.Mid(colon+1).Trim();
-  }
+  descriptor.Split(':', deviceType, deviceName);
 
   if (deviceType.IsEmpty() || deviceName.IsEmpty()) {
-    PTRACE(1, "LID\tInvalid device description \"" << descriptor << '"');
+    PTRACE2(1, NULL, "LID\tInvalid device description \"" << descriptor << '"');
     return NULL;
   }
 
