@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24212 $
- * $Author: rjongbloed $
- * $Date: 2010-04-07 22:03:34 -0500 (Wed, 07 Apr 2010) $
+ * $Revision: 28937 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 07:21:17 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -34,10 +34,14 @@
 #pragma implementation "lidep.cxx"
 #endif
 
-#include <opal/buildopts.h>
-
 #include <lids/lidep.h>
 
+#if OPAL_LID
+
+  #ifdef _MSC_VER
+    #pragma message("Line Interface Device support enabled")
+  #endif
+
 #include <opal/manager.h>
 #include <opal/call.h>
 #include <opal/patch.h>
@@ -52,7 +56,7 @@ static const char PrefixPOTS[] = "pots";
 /////////////////////////////////////////////////////////////////////////////
 
 OpalLineEndPoint::OpalLineEndPoint(OpalManager & mgr)
-  : OpalEndPoint(mgr, PrefixPOTS, CanTerminateCall),
+  : OpalEndPoint(mgr, PrefixPOTS, CanTerminateCall|SupportsE164),
     defaultLine("*")
 {
   PTRACE(4, "LID EP\tOpalLineEndPoint created");
@@ -223,7 +227,7 @@ void OpalLineEndPoint::RemoveAllLines()
 PBoolean OpalLineEndPoint::AddLinesFromDevice(OpalLineInterfaceDevice & device)
 {
   if (!device.IsOpen()){
-    PTRACE(1, "LID EP\tAddLinesFromDevice device " << device.GetDeviceName() << "is not opened");
+    PTRACE(1, "LID EP\tAddLinesFromDevice device " << device.GetDeviceName() << " is not opened");
     return false;
   }  
 
@@ -283,12 +287,7 @@ PBoolean OpalLineEndPoint::AddDeviceNames(const PStringArray & descriptors)
 const OpalLineInterfaceDevice * OpalLineEndPoint::GetDeviceByName(const PString & descriptor)
 {
   PString deviceType, deviceName;
-
-  PINDEX colon = descriptor.Find(':');
-  if (colon != P_MAX_INDEX) {
-    deviceType = descriptor.Left(colon).Trim();
-    deviceName = descriptor.Mid(colon+1).Trim();
-  }
+  descriptor.Split(':', deviceType, deviceName);
 
   if (deviceType.IsEmpty() || deviceName.IsEmpty()) {
     PTRACE(1, "LID EP\tInvalid device description \"" << descriptor << '"');
@@ -382,7 +381,31 @@ void OpalLineEndPoint::SetDefaultLine(const PString & lineName)
 }
 
 
-void OpalLineEndPoint::MonitorLines(PThread &, INT)
+bool OpalLineEndPoint::SetCountryCode(OpalLineInterfaceDevice::T35CountryCodes country)
+{
+  PWaitAndSignal mutex(linesMutex);
+  for (OpalLIDList::iterator iterDev = devices.begin(); iterDev != devices.end(); ++iterDev) {
+    if (!iterDev->SetCountryCode(country))
+      return false;
+  }
+
+  return true;
+}
+
+
+bool OpalLineEndPoint::SetCountryCodeName(const PString & countryName)
+{
+  PWaitAndSignal mutex(linesMutex);
+  for (OpalLIDList::iterator iterDev = devices.begin(); iterDev != devices.end(); ++iterDev) {
+    if (!iterDev->SetCountryCodeName(countryName))
+      return false;
+  }
+
+  return true;
+}
+
+
+void OpalLineEndPoint::MonitorLines(PThread &, P_INT_PTR)
 {
   PTRACE(4, "LID EP\tMonitor thread started for " << GetPrefixName());
 
@@ -466,11 +489,10 @@ OpalLineConnection::OpalLineConnection(OpalCall & call,
   localPartyName = ln.GetToken();
   remotePartyNumber = number.Right(number.FindSpan("0123456789*#,"));
   remotePartyName = number;
-  if (remotePartyName.IsEmpty())
-    remotePartyName = "Unknown";
-  else
-    remotePartyAddress = remotePartyName + '@';
-  remotePartyAddress += GetToken();
+  m_remotePartyURL = GetPrefixName() + ':';
+  if (!number.IsEmpty())
+    m_remotePartyURL += number + '@';
+  m_remotePartyURL += GetToken();
 
   silenceDetector = new OpalLineSilenceDetector(line, (endpoint.GetManager().GetSilenceDetectParams()));
 
@@ -504,15 +526,14 @@ void OpalLineConnection::OnReleased()
   else
     line.SetOnHook();
 
-  SetPhase(ReleasedPhase);
-
   OpalConnection::OnReleased();
 }
 
 
 PString OpalLineConnection::GetDestinationAddress()
 {
-  return line.IsTerminal() ? ReadUserInput() : GetLocalPartyName();
+  PTRACE(2, "OpalLineConnection::GetDestinationAddress called: " << line.IsTerminal() << " " << m_dialedNumber);
+  return line.IsTerminal() ? m_dialedNumber : GetLocalPartyName();
 }
 
 
@@ -525,7 +546,6 @@ PBoolean OpalLineConnection::SetAlerting(const PString & /*calleeName*/, PBoolea
 
   // switch phase 
   SetPhase(AlertingPhase);
-  alertingTime = PTime();
 
   if (line.IsTerminal() && GetMediaStream(OpalMediaType::Audio(), false) == NULL) {
     // Start ringing if we don't have an audio media stream
@@ -593,17 +613,22 @@ PBoolean OpalLineConnection::OnOpenMediaStream(OpalMediaStream & mediaStream)
   if (!OpalConnection::OnOpenMediaStream(mediaStream))
     return false;
 
-  if (mediaStream.IsSource()) {
-    OpalMediaPatch * patch = mediaStream.GetPatch();
-    if (patch != NULL)
-      patch->AddFilter(silenceDetector->GetReceiveHandler(), line.GetReadFormat());
-  }
+  if (mediaStream.IsSource())
+    mediaStream.AddFilter(silenceDetector->GetReceiveHandler(), line.GetReadFormat());
 
   line.StopTone(); // In case a RoutingTone or RingTone is going
   return true;
 }
 
 
+void OpalLineConnection::OnClosedMediaStream(const OpalMediaStream & mediaStream)
+{
+  mediaStream.RemoveFilter(silenceDetector->GetReceiveHandler(), line.GetReadFormat());
+
+  OpalConnection::OnClosedMediaStream(mediaStream);
+}
+
+
 PBoolean OpalLineConnection::SetAudioVolume(PBoolean source, unsigned percentage)
 {
   PSafePtr<OpalLineMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalLineMediaStream>(GetMediaStream(OpalMediaType::Audio(), source));
@@ -650,7 +675,7 @@ PBoolean OpalLineConnection::PromptUserInput(PBoolean play)
       PTRACE(3, "LID Con\tPlaying dial tone");
       return true;
     }
-    PTRACE(2, "LID Con\tCould not dial ring tone");
+    PTRACE(2, "LID Con\tCould not start dial tone");
     return false;
   }
 
@@ -724,14 +749,18 @@ void OpalLineConnection::Monitor()
 }
 
 
-void OpalLineConnection::HandleIncoming(PThread &, INT)
+void OpalLineConnection::HandleIncoming(PThread &, P_INT_PTR)
 {
   PTRACE(3, "LID Con\tHandling incoming call on " << *this);
 
   SetPhase(SetUpPhase);
 
-  if (line.IsTerminal())
+  if (line.IsTerminal()) {
+    PTRACE(3, "LID Con\tWaiting for number to be entered");
     wasOffHook = true;
+    m_dialedNumber = ReadUserInput();
+    PTRACE(3, "LID Con\tEntered number is '" << m_dialedNumber << "'");
+  }
   else {
     PTRACE(4, "LID Con\tCounting rings.");
     // Count incoming rings
@@ -744,7 +773,7 @@ void OpalLineConnection::HandleIncoming(PThread &, INT)
         return;
       }
       PThread::Sleep(100);
-      if (GetPhase() >= ReleasingPhase)
+      if (IsReleased())
         return;
     } while (count < minimumRingCount); // Wait till we have CLID
 
@@ -771,13 +800,12 @@ void OpalLineConnection::HandleIncoming(PThread &, INT)
 
     // switch phase 
     SetPhase(AlertingPhase);
-    alertingTime = PTime();
   }
 
   if (!OnIncomingConnection(0, NULL)) {
     PTRACE(3, "LID\tWaiting for RING to stop on " << *this);
     while (line.GetRingCount() > 0) {
-      if (GetPhase() >= ReleasingPhase)
+      if (IsReleased())
         return;
       PThread::Sleep(100);
     }
@@ -803,7 +831,7 @@ PBoolean OpalLineConnection::SetUpConnection()
   PTRACE(3, "LID Con\tSetUpConnection call on " << *this << " to \"" << remotePartyNumber << '"');
 
   SetPhase(SetUpPhase);
-  originating = true;
+  InternalSetAsOriginating();
 
   if (line.IsTerminal()) {
     PSafePtr<OpalConnection> partyA = ownerCall.GetConnection(0);
@@ -919,7 +947,7 @@ OpalLineMediaStream::~OpalLineMediaStream()
 
 PBoolean OpalLineMediaStream::Open()
 {
-  if (isOpen)
+  if (IsOpen())
     return true;
 
   if (IsSource()) {
@@ -939,7 +967,7 @@ PBoolean OpalLineMediaStream::Open()
 }
 
 
-PBoolean OpalLineMediaStream::Close()
+void OpalLineMediaStream::InternalClose()
 {
   if (directLineNumber != UINT_MAX)
     line.GetDevice().SetLineToLineDirect(line.GetLineNumber(), directLineNumber, false);
@@ -947,8 +975,6 @@ PBoolean OpalLineMediaStream::Close()
     line.StopReading();
   else
     line.StopWriting();
-
-  return OpalMediaStream::Close();
 }
 
 
@@ -998,6 +1024,7 @@ PBoolean OpalLineMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & leng
     }
   }
   else {
+    length = size;
     if (line.ReadFrame(buffer, length)) {
       // In the case of G.723.1 remember the last SID frame we sent and send
       // it again if the hardware sends us a DTX frame.
@@ -1108,7 +1135,9 @@ PBoolean OpalLineMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameTime)
     else
       useDeblocking = !line.SetWriteFrameSize(dataSize) || line.GetWriteFrameSize() != dataSize;
 
-    PTRACE(3, "LineMedia\tStream frame size: rd="
+    PTRACE(3, "LineMedia\tStream "
+           << (IsSource() ? "read" : "write")
+           << " frame size " << dataSize << " : rd="
            << line.GetReadFrameSize() << " wr="
            << line.GetWriteFrameSize() << ", "
            << (useDeblocking ? "needs" : "no") << " reblocking.");
@@ -1160,4 +1189,13 @@ unsigned OpalLineSilenceDetector::GetAverageSignalLevel(const BYTE *, PINDEX)
 }
 
 
+#else
+
+  #ifdef _MSC_VER
+    #pragma message("Line Interface Device support DISABLED")
+  #endif
+
+#endif // OPAL_LID
+
+
 /////////////////////////////////////////////////////////////////////////////
