@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27987 $
+ * $Revision: 29960 $
  * $Author: rjongbloed $
- * $Date: 2012-07-10 06:06:01 -0500 (Tue, 10 Jul 2012) $
+ * $Date: 2013-06-13 23:26:53 -0500 (Thu, 13 Jun 2013) $
  */
 
 #include <ptlib.h>
@@ -50,31 +50,40 @@
 
 /////////////////////////////////////////////////////////////////////////////
 
-#ifdef _MSC_VER
-#pragma warning(disable:4355)
-#endif
-
 OpalCall::OpalCall(OpalManager & mgr)
   : manager(mgr)
   , myToken(mgr.GetNextToken('C'))
-  , isEstablished(PFalse)
-  , isClearing(PFalse)
+  , m_networkOriginated(false)
+  , m_establishedTime(0)
+  , m_isEstablished(false)
+  , m_isClearing(false)
   , callEndReason(OpalConnection::NumCallEndReasons)
 #if OPAL_HAS_MIXER
   , m_recordManager(NULL)
 #endif
+#if OPAL_T38_CAPABILITY
+  , m_T38SwitchState(e_NotSwitchingT38)
+#endif
 {
+  PTRACE_CONTEXT_ID_NEW();
+
   manager.activeCalls.SetAt(myToken, this);
 
   connectionsActive.DisallowDeleteObjects();
 
+#if OPAL_SCRIPT
+  PScriptLanguage * script = manager.GetScript();
+  if (script != NULL) {
+    PString tableName = OPAL_SCRIPT_CALL_TABLE_NAME "." + GetToken();
+    script->CreateComposite(tableName);
+    script->SetFunction(tableName + ".Clear", PCREATE_NOTIFIER(ScriptClear));
+    script->Call("OnNewCall", "s", (const char *)GetToken());
+  }
+#endif
+
   PTRACE(3, "Call\tCreated " << *this);
 }
 
-#ifdef _MSC_VER
-#pragma warning(default:4355)
-#endif
-
 
 OpalCall::~OpalCall()
 {
@@ -82,6 +91,14 @@ OpalCall::~OpalCall()
   delete m_recordManager;
 #endif
 
+#if OPAL_SCRIPT
+  PScriptLanguage * script = manager.GetScript();
+  if (script != NULL) {
+    script->Call("OnDestroyCall", "s", (const char *)GetToken());
+    script->ReleaseVariable(OPAL_SCRIPT_CALL_TABLE_NAME "." + GetToken());
+  }
+#endif
+
   PTRACE(3, "Call\tDestroyed " << *this);
 }
 
@@ -110,17 +127,19 @@ void OpalCall::SetCallEndReason(OpalConnection::CallEndReason reason)
 
 void OpalCall::Clear(OpalConnection::CallEndReason reason, PSyncPoint * sync)
 {
+  PTRACE_CONTEXT_ID_PUSH_THREAD(this);
+
   PTRACE(3, "Call\tClearing " << (sync != NULL ? "(sync) " : "") << *this << " reason=" << reason);
 
   {
     PSafeLockReadWrite lock(*this);
-    if (!lock.IsLocked() || isClearing) {
+    if (!lock.IsLocked() || m_isClearing) {
       if (sync != NULL)
         sync->Signal();
       return;
     }
 
-    isClearing = true;
+    m_isClearing = true;
 
     SetCallEndReason(reason);
 
@@ -156,24 +175,28 @@ void OpalCall::Clear(OpalConnection::CallEndReason reason, PSyncPoint * sync)
 
 void OpalCall::InternalOnClear()
 {
-  if (connectionsActive.IsEmpty() && manager.activeCalls.Contains(GetToken())) {
-    OnCleared();
+  if (!connectionsActive.IsEmpty())
+    return;
+
+  if (m_isCleared.TestAndSet(true))
+    return;
+
+  OnCleared();
 
 #if OPAL_HAS_MIXER
-    StopRecording();
+  StopRecording();
 #endif
 
-    if (LockReadWrite()) {
-      while (!m_endCallSyncPoint.empty()) {
-        PTRACE(5, "Call\tSignalling end call.");
-        m_endCallSyncPoint.front()->Signal();
-        m_endCallSyncPoint.pop_front();
-      }
-      UnlockReadWrite();
+  if (LockReadWrite()) {
+    while (!m_endCallSyncPoint.empty()) {
+      PTRACE(5, "Call\tSignalling end call.");
+      m_endCallSyncPoint.front()->Signal();
+      m_endCallSyncPoint.pop_front();
     }
-
-    manager.activeCalls.RemoveAt(GetToken());
+    UnlockReadWrite();
   }
+
+  manager.activeCalls.RemoveAt(GetToken());
 }
 
 
@@ -185,6 +208,9 @@ void OpalCall::OnCleared()
 
 void OpalCall::OnNewConnection(OpalConnection & connection)
 {
+  if (connectionsActive.GetSize() == 1)
+    m_networkOriginated = connection.IsNetworkConnection();
+
   manager.OnNewConnection(connection);
   SetPartyNames();
 }
@@ -194,7 +220,7 @@ PBoolean OpalCall::OnSetUp(OpalConnection & connection)
 {
   PTRACE(3, "Call\tOnSetUp " << connection);
 
-  if (isClearing)
+  if (m_isClearing)
     return false;
 
   SetPartyNames();
@@ -221,7 +247,7 @@ PBoolean OpalCall::OnAlerting(OpalConnection & connection)
 {
   PTRACE(3, "Call\tOnAlerting " << connection);
 
-  if (isClearing)
+  if (m_isClearing)
     return false;
 
   PBoolean hasMedia = connection.GetMediaStream(OpalMediaType::Audio(), true) != NULL;
@@ -251,7 +277,7 @@ PBoolean OpalCall::OnConnected(OpalConnection & connection)
 {
   PTRACE(3, "Call\tOnConnected " << connection);
 
-  if (isClearing || !LockReadOnly())
+  if (m_isClearing || !LockReadOnly())
     return false;
 
   bool havePartyB = connectionsActive.GetSize() == 1 && !m_partyB.IsEmpty();
@@ -286,10 +312,10 @@ PBoolean OpalCall::OnEstablished(OpalConnection & connection)
   PTRACE(3, "Call\tOnEstablished " << connection);
 
   PSafeLockReadWrite lock(*this);
-  if (isClearing || !lock.IsLocked())
+  if (m_isClearing || !lock.IsLocked())
     return false;
 
-  if (isEstablished)
+  if (m_isEstablished)
     return true;
 
   if (connectionsActive.GetSize() < 2)
@@ -302,7 +328,8 @@ PBoolean OpalCall::OnEstablished(OpalConnection & connection)
       return false;
   }
 
-  isEstablished = true;
+  m_isEstablished = true;
+  m_establishedTime.SetCurrentTime();
   OnEstablishedCall();
 
   return true;
@@ -321,6 +348,8 @@ PSafePtr<OpalConnection> OpalCall::GetOtherPartyConnection(const OpalConnection
 
 bool OpalCall::Hold()
 {
+  PTRACE_CONTEXT_ID_PUSH_THREAD(this);
+
   PTRACE(3, "Call\tSetting to On Hold");
 
   bool ok = false;
@@ -337,6 +366,8 @@ bool OpalCall::Hold()
 
 bool OpalCall::Retrieve()
 {
+  PTRACE_CONTEXT_ID_PUSH_THREAD(this);
+
   PTRACE(3, "Call\tRetrieve from On Hold");
 
   bool ok = false;
@@ -365,6 +396,8 @@ bool OpalCall::IsOnHold() const
 
 bool OpalCall::Transfer(const PString & newAddress, OpalConnection * connection)
 {
+  PTRACE_CONTEXT_ID_PUSH_THREAD(this);
+
   PCaselessString prefix;
   PINDEX colon = newAddress.Find(':');
   if (colon != P_MAX_INDEX)
@@ -407,8 +440,8 @@ bool OpalCall::Transfer(const PString & newAddress, OpalConnection * connection)
   // Restart with new connection
   if (newConnection->SetUpConnection() && newConnection->OnSetUpConnection()) {
     connectionToKeep->AutoStartMediaStreams(true);
-    connection->Release(OpalConnection::EndedByCallForwarded);
-    newConnection->StartMediaStreams();
+    connection->Release(OpalConnection::EndedByCallForwarded, true);
+    StartMediaStreams(); // Just in case ...
     return true;
   }
 
@@ -423,10 +456,26 @@ OpalMediaFormatList OpalCall::GetMediaFormats(const OpalConnection & connection)
   OpalMediaFormatList commonFormats;
 
   bool first = true;
+  const OpalMediaTypeList allMediaTypes = OpalMediaType::GetList();
 
   PSafePtr<OpalConnection> otherConnection;
   while (EnumerateConnections(otherConnection, PSafeReadOnly, &connection)) {
-    OpalMediaFormatList possibleFormats = OpalTranscoder::GetPossibleFormats(otherConnection->GetMediaFormats());
+    const OpalMediaFormatList otherConnectionsFormats = otherConnection->GetMediaFormats();
+    OpalMediaFormatList possibleFormats;
+    for (OpalMediaTypeList::const_iterator iterMediaType = allMediaTypes.begin(); iterMediaType != allMediaTypes.end(); ++iterMediaType) {
+      OpalMediaFormatList formatsOfMediaType;
+      for (OpalMediaFormatList::const_iterator it = otherConnectionsFormats.begin(); it != otherConnectionsFormats.end(); ++it) {
+        if (it->GetMediaType() == *iterMediaType)
+          formatsOfMediaType += *it;
+      }
+      if (!formatsOfMediaType.IsEmpty()) {
+        if (connection.IsNetworkConnection() && otherConnection->IsNetworkConnection() &&
+              manager.GetMediaTransferMode(connection, *otherConnection, *iterMediaType) != OpalManager::MediaTransferTranscode)
+          possibleFormats += formatsOfMediaType;
+        else
+          possibleFormats += OpalTranscoder::GetPossibleFormats(formatsOfMediaType);
+      }
+    }
     if (first) {
       commonFormats = possibleFormats;
       first = false;
@@ -442,12 +491,6 @@ OpalMediaFormatList OpalCall::GetMediaFormats(const OpalConnection & connection)
     }
   }
 
-  // If nothing comes out of the above, e.g. when this is called very early
-  // in the call sequence and there is no second connection in call yet, then
-  // return all the possible media formats.
-  if (first)
-    commonFormats = OpalTranscoder::GetPossibleFormats(manager.GetCommonMediaFormats(false, true));
-
   connection.AdjustMediaFormats(true, NULL, commonFormats);
 
   PTRACE(4, "Call\tGetMediaFormats for " << connection << "\n    "
@@ -468,14 +511,16 @@ void OpalCall::AdjustMediaFormats(bool local, const OpalConnection & connection,
 PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection, 
                                      const OpalMediaType & mediaType,
                                                   unsigned sessionID, 
-                                   const OpalMediaFormat & preselectedFormat
+                                   const OpalMediaFormat & preselectedFormat,
 #if OPAL_VIDEO
-                            , OpalVideoFormat::ContentRole contentRole
+                              OpalVideoFormat::ContentRole contentRole,
 #endif
-                                   )
+                                                      bool transfer)
 {
+  PTRACE_CONTEXT_ID_PUSH_THREAD(this);
+
   PSafeLockReadOnly lock(*this);
-  if (isClearing || !lock.IsLocked())
+  if (m_isClearing || !lock.IsLocked())
     return false;
 
 #if PTRACING
@@ -496,11 +541,12 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
   // Check if already done
   OpalMediaStreamPtr sinkStream;
   OpalMediaStreamPtr sourceStream = connection.GetMediaStream(sessionID, true);
-  if (sourceStream != NULL) {
-    OpalMediaPatch * patch = sourceStream->GetPatch();
+  if (sourceStream != NULL && !transfer) {
+    OpalMediaPatchPtr patch = sourceStream->GetPatch();
     if (patch != NULL)
       sinkStream = patch->GetSink();
-    if (sourceStream->GetMediaFormat() == preselectedFormat ||
+    if (preselectedFormat.IsEmpty() ||
+        sourceStream->GetMediaFormat() == preselectedFormat ||
         (sinkStream != NULL && sinkStream->GetMediaFormat() == preselectedFormat)) {
       if (sourceStream->IsPaused()) {
         sourceStream->SetPaused(false);
@@ -575,23 +621,47 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
     }
 
 #if OPAL_VIDEO
+    if (contentRole == OpalVideoFormat::eNoRole && preselectedFormat.IsValid())
+      contentRole = preselectedFormat.GetOptionEnum(OpalVideoFormat::ContentRoleOption(), OpalVideoFormat::eNoRole);
     if (contentRole != OpalVideoFormat::eNoRole) {
-      // Remove all media formats no supporting the role
-      OpalMediaFormatList * lists[2] = { &sourceMediaFormats, &sinkMediaFormats };
+      PTRACE(4, "Call\tContent Role " << contentRole << traceText);
+
+      bool noDefaultMainRole = contentRole != OpalVideoFormat::eMainRole;
+
+      // Remove all media formats not supporting the role
+      OpalMediaFormatList * oldLists[2] = { &sourceMediaFormats, &sinkMediaFormats };
+      OpalMediaFormatList newLists[2];
       for (PINDEX i = 0; i < 2; i++) {
-        OpalMediaFormatList::iterator format = lists[i]->begin();
-        while (format != lists[i]->end()) {
-          if ( format->GetMediaType() != mediaType ||
-              (format->IsTransportable() &&
-              (format->GetOptionInteger(OpalVideoFormat::ContentRoleMaskOption())&OpalVideoFormat::ContentRoleBit(contentRole)) == 0))
-            *lists[i] -= *format++;
-          else
-            ++format;
+        for (OpalMediaFormatList::iterator format = oldLists[i]->begin(); format != oldLists[i]->end(); ++format) {
+          if (format->GetMediaType() != mediaType)
+            continue;
+
+          if (!format->IsTransportable()) {
+            newLists[i] += *format;
+            continue;
+          }
+
+          unsigned mask = format->GetOptionInteger(OpalVideoFormat::ContentRoleMaskOption());
+          if (mask != 0)
+            noDefaultMainRole = true;
+
+          if ((mask&OpalVideoFormat::ContentRoleBit(contentRole)) != 0)
+            newLists[i] += *format;
         }
-        if (lists[i]->IsEmpty()) {
+      }
+
+      if (noDefaultMainRole) {
+        if (newLists[0].IsEmpty() || newLists[1].IsEmpty()) {
           PTRACE(2, "Call\tUnsupported Content Role " << contentRole << traceText);
           return false;
         }
+
+        sourceMediaFormats = newLists[0];
+        sinkMediaFormats = newLists[1];
+      }
+      else {
+        PTRACE(4, "Call\tDefault main content role used " << traceText);
+        contentRole = OpalVideoFormat::eNoRole;
       }
     }
 #endif
@@ -629,7 +699,7 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
       if (!sourceStream.SetSafetyMode(PSafeReadOnly))
         return false;
 
-      OpalMediaPatch * patch = sourceStream->GetPatch();
+      OpalMediaPatchPtr patch = sourceStream->GetPatch();
       if (patch == NULL) {
         patch = manager.CreateMediaPatch(*sourceStream, sinkStream->RequiresPatchThread(sourceStream) &&
                                                         sourceStream->RequiresPatchThread(sinkStream));
@@ -648,7 +718,7 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
 
   if (!startedOne) {
     if (sourceStream != NULL)
-      connection.RemoveMediaStream(*sourceStream);
+      sourceStream->Close();
     return false;
   }
 
@@ -658,7 +728,7 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
     if (!sourceStream.SetSafetyMode(PSafeReadOnly))
       return false;
 
-    OpalMediaPatch * patch = sourceStream->GetPatch();
+    OpalMediaPatchPtr patch = sourceStream->GetPatch();
     if (patch == NULL)
       return false;
 
@@ -705,22 +775,6 @@ void OpalCall::CloseMediaStreams()
 }
 
 
-void OpalCall::OnRTPStatistics(const OpalConnection & /*connection*/, const RTP_Session & /*session*/)
-{
-}
-
-
-PBoolean OpalCall::IsMediaBypassPossible(const OpalConnection & connection,
-                                     unsigned sessionID) const
-{
-  PTRACE(3, "Call\tIsMediaBypassPossible " << connection << " session " << sessionID);
-
-  PSafePtr<OpalConnection> otherConnection;
-  return EnumerateConnections(otherConnection, PSafeReadOnly, &connection) &&
-         manager.IsMediaBypassPossible(connection, *otherConnection, sessionID);
-}
-
-
 void OpalCall::OnUserInputString(OpalConnection & connection, const PString & value)
 {
   PSafePtr<OpalConnection> otherConnection;
@@ -763,8 +817,10 @@ void OpalCall::OnReleased(OpalConnection & connection)
   if(connectionsActive.GetSize() == 1)
   {
     PSafePtr<OpalConnection> last = connectionsActive.GetAt(0, PSafeReference);
-    if (last != NULL)
+    if (last != NULL) {
+      PTRACE(4, "Call", "Releasing last connection in call");
       last->Release(connection.GetCallEndReason(), true);
+    }
   }
 
   InternalOnClear();
@@ -866,13 +922,6 @@ void OpalCall::OnRecordVideo(const PString & streamId, const RTP_DataFrame & fra
 #endif
 
 
-bool OpalCall::IsNetworkOriginated() const
-{
-  PSafePtr<OpalConnection> connection = PSafePtr<OpalConnection>(connectionsActive, PSafeReadOnly);
-  return connection == NULL || connection->IsNetworkConnection();
-}
-
-
 void OpalCall::SetPartyNames()
 {
   PSafeLockReadWrite lock(*this);
@@ -884,10 +933,14 @@ void OpalCall::SetPartyNames()
     return;
 
   bool networkA = connectionA->IsNetworkConnection();
-  if (networkA)
+  if (networkA) {
     m_partyA = connectionA->GetRemotePartyURL();
-  if (!networkA || m_partyA.IsEmpty())
+    m_nameA = connectionA->GetRemotePartyName();
+  }
+  if (!networkA || m_partyA.IsEmpty()) {
     m_partyA = connectionA->GetLocalPartyURL();
+    m_nameA = connectionA->GetLocalPartyName();
+  }
 
   PSafePtr<OpalConnection> connectionB = connectionsActive.GetAt(1, PSafeReadOnly);
   if (connectionB == NULL)
@@ -897,14 +950,18 @@ void OpalCall::SetPartyNames()
     if (!networkA)
       connectionA->CopyPartyNames(*connectionB);
     m_partyB = connectionB->GetRemotePartyURL();
+    m_nameB = connectionB->GetRemotePartyName();
   }
   else {
     if (networkA) {
       connectionB->CopyPartyNames(*connectionA);
-      m_partyB = connectionA->GetCalledPartyURL();
+      PString dest = connectionA->GetDestinationAddress();
+      if (!dest.IsEmpty() && dest != "*")
+        m_partyB = connectionA->GetCalledPartyURL();
     }
     if (m_partyB.IsEmpty())
       m_partyB = connectionB->GetLocalPartyURL();
+    m_nameB = connectionB->GetLocalPartyName();
   }
 }
 
@@ -932,4 +989,30 @@ bool OpalCall::EnumerateConnections(PSafePtr<OpalConnection> & connection,
 }
 
 
+#if OPAL_SCRIPT
+
+void OpalCall::ScriptClear(PScriptLanguage &, PScriptLanguage::Signature & sig)
+{
+  OpalConnection::CallEndReason reason;
+
+  switch (sig.m_arguments.size()) {
+    default :
+    case 2 :
+      reason.q931 = sig.m_arguments[1].AsInteger();
+      // Next case
+
+    case 1 :
+      reason.code = (OpalConnection::CallEndReasonCodes)sig.m_arguments[0].AsInteger();
+      // Next case
+
+    case 0 :
+      break;
+  }
+
+  Clear(reason);
+}
+
+#endif // OPAL_SCRIPT
+
+
 /////////////////////////////////////////////////////////////////////////////
