@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24442 $
+ * $Revision: 30003 $
  * $Author: rjongbloed $
- * $Date: 2010-06-02 03:17:31 -0500 (Wed, 02 Jun 2010) $
+ * $Date: 2013-06-19 22:38:44 -0500 (Wed, 19 Jun 2013) $
  */
 
 #include <ptlib.h>
@@ -40,13 +40,9 @@
 #include <opal/endpoint.h>
 #include <opal/manager.h>
 #include <opal/call.h>
-#include <rtp/rtp.h>
+#include <rtp/rtp_session.h>
 
-//
-// TODO find the correct value, usually the bandwidth for pure audio call is 1280 kb/sec 
-// Set the bandwidth to 10Mbits, as setting the bandwith to UINT_MAX causes problems with cisco gatekeepers 
-//
-#define BANDWITH_DEFAULT_INITIAL 100000
+static const OpalBandwidth DefaultInitialBandwidth = 4000000; // 4Mb/s
 
 #define new PNEW
 
@@ -56,20 +52,19 @@
 OpalEndPoint::OpalEndPoint(OpalManager & mgr,
                            const PCaselessString & prefix,
                            unsigned attributes)
-  : manager(mgr),
-    prefixName(prefix),
-    attributeBits(attributes),
-    productInfo(mgr.GetProductInfo()),
-    defaultLocalPartyName(manager.GetDefaultUserName()),
-    defaultDisplayName(manager.GetDefaultDisplayName())
+  : manager(mgr)
+  , prefixName(prefix)
+  , attributeBits(attributes)
+  , m_maxSizeUDP(4096)
+  , productInfo(mgr.GetProductInfo())
+  , defaultLocalPartyName(manager.GetDefaultUserName())
+  , defaultDisplayName(manager.GetDefaultDisplayName())
+  , m_initialRxBandwidth(DefaultInitialBandwidth)
+  , m_initialTxBandwidth(DefaultInitialBandwidth)
+  , defaultSendUserInputMode(OpalConnection::SendUserInputAsProtocolDefault)
 {
   manager.AttachEndPoint(this);
 
-  defaultSignalPort = 0;
-
-  initialBandwidth = BANDWITH_DEFAULT_INITIAL;
-  defaultSendUserInputMode = OpalConnection::SendUserInputAsProtocolDefault;
-
   if (defaultLocalPartyName.IsEmpty())
     defaultLocalPartyName = PProcess::Current().GetName() & "User";
 
@@ -98,42 +93,104 @@ void OpalEndPoint::PrintOn(ostream & strm) const
 }
 
 
+OpalBandwidth OpalEndPoint::GetInitialBandwidth(OpalBandwidth::Direction dir) const
+{
+  switch (dir) {
+    case OpalBandwidth::Rx :
+      return m_initialRxBandwidth;
+    case OpalBandwidth::Tx :
+      return m_initialTxBandwidth;
+    default :
+      return m_initialRxBandwidth+m_initialTxBandwidth;
+  }
+}
+
+
+void OpalEndPoint::SetInitialBandwidth(OpalBandwidth::Direction dir, OpalBandwidth bandwidth)
+{
+  switch (dir) {
+    case OpalBandwidth::Rx :
+      m_initialRxBandwidth = bandwidth;
+      break;
+
+    case OpalBandwidth::Tx :
+      m_initialTxBandwidth = bandwidth;
+      break;
+
+    default :
+      OpalBandwidth rx = (PUInt64)(unsigned)bandwidth*m_initialRxBandwidth/(m_initialRxBandwidth+m_initialTxBandwidth);
+      OpalBandwidth tx = (PUInt64)(unsigned)bandwidth*m_initialTxBandwidth/(m_initialRxBandwidth+m_initialTxBandwidth);
+      m_initialRxBandwidth = rx;
+      m_initialTxBandwidth = tx;
+  }
+}
+
+
 PBoolean OpalEndPoint::GarbageCollection()
 {
-  for (PSafePtr<OpalConnection> connection(connectionsActive, PSafeReference); connection != NULL; ++connection)
+  for (PSafePtr<OpalConnection> connection(connectionsActive, PSafeReference); connection != NULL; ++connection) {
+    PTRACE_CONTEXT_ID_PUSH_THREAD(connection);
     connection->GarbageCollection();
+  }
 
   return connectionsActive.DeleteObjectsToBeRemoved();
 }
 
 
-PBoolean OpalEndPoint::StartListeners(const PStringArray & listenerAddresses)
+bool OpalEndPoint::StartListeners(const PStringArray & listenerAddresses, bool add)
 {
-  PStringArray interfaces = listenerAddresses;
-  if (interfaces.IsEmpty()) {
+  OpalTransportAddressArray interfaces;
+  if (listenerAddresses.IsEmpty()) {
     interfaces = GetDefaultListeners();
     if (interfaces.IsEmpty())
-      return PFalse;
+      return false;
   }
-
-  PBoolean startedOne = PFalse;
-
-  for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
-    if (interfaces[i].Find('$') != P_MAX_INDEX) {
-      if (StartListener(interfaces[i]))
-        startedOne = PTrue;
-    }
-    else {
-      PStringArray transports = GetDefaultTransport().Tokenise(',');
-      for (PINDEX j = 0; j < transports.GetSize(); j++) {
-        OpalTransportAddress iface(interfaces[i], GetDefaultSignalPort(), transports[j]);
-        if (StartListener(iface))
-          startedOne = PTrue;
+  else {
+    for (PINDEX i = 0; i < listenerAddresses.GetSize(); i++) {
+      if (listenerAddresses[i].Find('$') != P_MAX_INDEX)
+        interfaces.AppendAddress(listenerAddresses[i]);
+      else {
+        PStringArray transports = GetDefaultTransport().Tokenise(',');
+        for (PINDEX j = 0; j < transports.GetSize(); j++) {
+          PString transport = transports[j];
+          WORD port = GetDefaultSignalPort();
+          PINDEX colon = transport.Find(':');
+          if (colon != P_MAX_INDEX) {
+            port = (WORD)transport.Mid(colon+1).AsUnsigned();
+            transport.Delete(colon, P_MAX_INDEX);
+          }
+          interfaces.AppendAddress(OpalTransportAddress(interfaces[i], port, transport));
+        }
       }
     }
   }
 
-  return startedOne;
+  bool atLeastOne = false;
+
+  // Stop listeners not in list
+  if (!add) {
+    for (OpalListenerList::iterator it = listeners.begin(); it != listeners.end(); ) {
+      bool removeListener = true;
+      for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
+        if (it->GetLocalAddress().IsEquivalent(interfaces[i])) {
+          interfaces.RemoveAt(i);
+          removeListener = false;
+          atLeastOne = true;
+          ++it;
+          break;
+        }
+      }
+      if (removeListener)
+        listeners.erase(it++);
+    }
+  }
+
+  for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
+    if (StartListener(interfaces[i]))
+      atLeastOne = true;
+  }
+
+  return atLeastOne;
 }
 
 
@@ -146,62 +203,88 @@ PBoolean OpalEndPoint::StartListener(const OpalTransportAddress & listenerAddres
   if (iface.IsEmpty()) {
     PStringArray interfaces = GetDefaultListeners();
     if (interfaces.IsEmpty())
-      return PFalse;
-    iface = OpalTransportAddress(interfaces[0], defaultSignalPort);
+      return false;
+    iface = OpalTransportAddress(interfaces[0], GetDefaultSignalPort());
+  }
+
+  // Check for already listening
+  for (OpalListenerList::iterator it = listeners.begin(); it != listeners.end(); ++it) {
+    if (it->GetLocalAddress().IsEquivalent(iface)) {
+      PTRACE(4, "OpalEP\tAlready listening on " << iface);
+      return true;
+    }
   }
 
   listener = iface.CreateListener(*this, OpalTransportAddress::FullTSAP);
   if (listener == NULL) {
     PTRACE(1, "OpalEP\tCould not create listener: " << iface);
-    return PFalse;
+    return false;
   }
 
   if (StartListener(listener))
-    return PTrue;
+    return true;
 
   PTRACE(1, "OpalEP\tCould not start listener: " << iface);
-  return PFalse;
+  return false;
 }
 
 
 PBoolean OpalEndPoint::StartListener(OpalListener * listener)
 {
   if (listener == NULL)
-    return PFalse;
+    return false;
+
+  OpalListenerUDP * udpListener = dynamic_cast<OpalListenerUDP *>(listener);
+  if (udpListener != NULL)
+    udpListener->SetBufferSize(m_maxSizeUDP);
 
   // as the listener is not open, this will have the effect of immediately
   // stopping the listener thread. This is good - it means that the 
   // listener Close function will appear to have stopped the thread
-  if (!listener->Open(PCREATE_NOTIFIER(ListenerCallback))) {
+  if (!listener->Open(PCREATE_NOTIFIER(NewIncomingConnection))) {
     delete listener;
-    return PFalse;
+    return false;
   }
 
   listeners.Append(listener);
-  return PTrue;
+  return true;
 }
 
+
 PString OpalEndPoint::GetDefaultTransport() const
 {
-  return "tcp$";
+  return PString::Empty();
 }
 
+
+WORD OpalEndPoint::GetDefaultSignalPort() const
+{
+  return 0;
+}
+
+
 PStringArray OpalEndPoint::GetDefaultListeners() const
 {
   PStringArray listenerAddresses;
   PStringArray transports = GetDefaultTransport().Tokenise(',');
   for (PINDEX i = 0; i < transports.GetSize(); i++) {
-    PString t = transports[i];
-    WORD port = defaultSignalPort;
-    PINDEX pos = t.Find(':');
+    PCaselessString transProto = transports[i];
+    WORD port = GetDefaultSignalPort();
+    PINDEX pos = transProto.Find(':');
     if (pos != P_MAX_INDEX) {
-      port = (WORD)t.Mid(pos+1).AsUnsigned();
-      t = t.Left(pos);
+      port = (WORD)transProto.Mid(pos+1).AsUnsigned();
+      transProto.Delete(pos, P_MAX_INDEX);
+    }
+    if (port != 0) {
+      if (transProto == OpalTransportAddress::UdpPrefix())
+        listenerAddresses += transProto + psprintf("*:%u", port);
+      else {
+        listenerAddresses += OpalTransportAddress(PIPSocket::Address::GetAny(4), port, transProto);
+#if OPAL_PTLIB_IPV6
+        listenerAddresses += OpalTransportAddress(PIPSocket::Address::GetAny(6), port, transProto);
+#endif
+      }
     }
-    PString listenerAddress = t + '*';
-    if (defaultSignalPort != 0)
-      listenerAddress.sprintf(":%u", port);
-    listenerAddresses += listenerAddress;
   }
   return listenerAddresses;
 }
@@ -230,7 +313,7 @@ PBoolean OpalEndPoint::RemoveListener(OpalListener * listener)
     return listeners.Remove(listener);
 
   listeners.RemoveAll();
-  return PTrue;
+  return true;
 }
 
 
@@ -255,8 +338,13 @@ static void AddTransportAddresses(OpalTransportAddressArray & interfaceAddresses
                                   const PIPSocket::Address & natInterfaceIP,
                                   const PIPSocket::Address & natExternalIP,
                                   const OpalTransportAddress & preferredAddress,
-                                  const OpalTransportAddress & localAddress)
+                                  const OpalTransportAddress & localAddress,
+                                  bool includeProto)
 {
+  PCaselessString proto = localAddress.GetProtoPrefix();
+  if (includeProto && proto != preferredAddress.GetProto(true))
+    return;
+
   if (!preferredAddress.IsEmpty() && !preferredAddress.IsEquivalent(localAddress, true))
     return;
 
@@ -265,7 +353,6 @@ static void AddTransportAddresses(OpalTransportAddressArray & interfaceAddresses
   if (!localAddress.GetIpAndPort(localIP, port))
     return;
 
-  PCaselessString proto = localAddress.GetProto();
   PIPSocket::InterfaceTable interfaces;
   if (!localIP.IsAny() || !PIPSocket::GetInterfaceTable(interfaces)) {
     AddTransportAddress(interfaceAddresses, natInterfaceIP, natExternalIP, localIP, port, proto);
@@ -274,7 +361,7 @@ static void AddTransportAddresses(OpalTransportAddressArray & interfaceAddresses
 
 
   PIPSocket::Address preferredIP;
-  if (preferredAddress.GetIpAddress(preferredIP)) {
+  if (preferredAddress.GetIpAddress(preferredIP) && !preferredIP.IsAny()) {
     for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
       PIPSocket::Address ip = interfaces[i].GetAddress();
       if (ip == preferredIP)
@@ -284,20 +371,20 @@ static void AddTransportAddresses(OpalTransportAddressArray & interfaceAddresses
 
   for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
     PIPSocket::Address ip = interfaces[i].GetAddress();
-    if (!excludeLocalHost || !ip.IsLoopback())
+    if (ip.GetVersion() == localIP.GetVersion() && !(excludeLocalHost && ip.IsLoopback()))
       AddTransportAddress(interfaceAddresses, natInterfaceIP, natExternalIP, ip, port, proto);
   }
 }
 
 
 OpalTransportAddressArray OpalEndPoint::GetInterfaceAddresses(PBoolean excludeLocalHost,
-                                                              const OpalTransport * associatedTransport)
+                                                              const OpalTransport * associatedTransport) const
 {
   OpalTransportAddressArray interfaceAddresses;
 
-  OpalTransportAddress associatedLocalAddress, associatedRemoteAddress;
-  PIPSocket::Address natInterfaceIP;
-  PIPSocket::Address natExternalIP;
+  OpalTransportAddress associatedLocalAddress, associatedRemoteAddress, associatedWildcard;
+  PIPSocket::Address natInterfaceIP = PIPSocket::GetInvalidAddress();
+  PIPSocket::Address natExternalIP = PIPSocket::GetInvalidAddress();
   if (associatedTransport != NULL) {
     associatedLocalAddress = associatedTransport->GetLocalAddress();
     associatedRemoteAddress = associatedTransport->GetRemoteAddress();
@@ -305,18 +392,19 @@ OpalTransportAddressArray OpalEndPoint::GetInterfaceAddresses(PBoolean excludeLo
     PIPSocket::Address remoteIP;
     associatedRemoteAddress.GetIpAddress(remoteIP);
 
+    associatedWildcard = OpalTransportAddress(PIPSocket::Address::GetAny(remoteIP.GetVersion()), 0,
+                                              associatedLocalAddress.GetProtoPrefix());
+
+#if P_NAT
     PNatMethod * natMethod = manager.GetNatMethod(remoteIP);
     if (natMethod != NULL) {
       natMethod->GetInterfaceAddress(natInterfaceIP);
       natMethod->GetExternalAddress(natExternalIP);
     }
-    else if (manager.GetTranslationAddress().IsValid()) {
-      natInterfaceIP = PIPSocket::GetDefaultIpAny();
-      natExternalIP = manager.GetTranslationAddress();
-    }
+#endif
   }
 
-  OpalListenerList::iterator listener;
+  OpalListenerList::const_iterator listener;
 
   if (!associatedLocalAddress.IsEmpty()) {
     for (listener = listeners.begin(); listener != listeners.end(); ++listener)
@@ -325,7 +413,24 @@ OpalTransportAddressArray OpalEndPoint::GetInterfaceAddresses(PBoolean excludeLo
                             natInterfaceIP,
                             natExternalIP,
                             associatedLocalAddress,
-                            listener->GetLocalAddress(associatedRemoteAddress));
+                            listener->GetLocalAddress(associatedRemoteAddress),
+                            true);
+    for (listener = listeners.begin(); listener != listeners.end(); ++listener)
+      AddTransportAddresses(interfaceAddresses,
+                            excludeLocalHost,
+                            natInterfaceIP,
+                            natExternalIP,
+                            associatedWildcard,
+                            listener->GetLocalAddress(associatedRemoteAddress),
+                            true);
+    for (listener = listeners.begin(); listener != listeners.end(); ++listener)
+      AddTransportAddresses(interfaceAddresses,
+                            excludeLocalHost,
+                            natInterfaceIP,
+                            natExternalIP,
+                            associatedLocalAddress,
+                            listener->GetLocalAddress(associatedRemoteAddress),
+                            false);
   }
 
   for (listener = listeners.begin(); listener != listeners.end(); ++listener)
@@ -334,7 +439,8 @@ OpalTransportAddressArray OpalEndPoint::GetInterfaceAddresses(PBoolean excludeLo
                           natInterfaceIP,
                           natExternalIP,
                           OpalTransportAddress(),
-                          listener->GetLocalAddress());
+                          listener->GetLocalAddress(),
+                          false);
 
   PTRACE(4, "OpalMan\tListener interfaces: associated transport="
          << (associatedTransport != NULL ? (const char *)associatedLocalAddress : "None")
@@ -343,21 +449,38 @@ OpalTransportAddressArray OpalEndPoint::GetInterfaceAddresses(PBoolean excludeLo
 }
 
 
-void OpalEndPoint::ListenerCallback(PThread &, INT param)
+void OpalEndPoint::NewIncomingConnection(OpalListener & /*listener*/, const OpalTransportPtr & /*transport*/)
 {
-  // Error in accept, usually means listener thread stopped, so just exit
-  if (param == 0)
-    return;
-
-  OpalTransport * transport = (OpalTransport *)param;
-  if (NewIncomingConnection(transport))
-    delete transport;
 }
 
 
-PBoolean OpalEndPoint::NewIncomingConnection(OpalTransport * /*transport*/)
+PSafePtr<OpalConnection> OpalEndPoint::GetConnectionWithLock(const PString & token, PSafetyMode mode) const
 {
-  return PTrue;
+  if (token.IsEmpty())
+    return NULL;
+
+  PSafePtr<OpalConnection> connection = connectionsActive.FindWithLock(token, mode);
+  if (connection != NULL)
+    return connection;
+
+  PSafePtr<OpalCall> call = manager.FindCallWithLock(token, PSafeReadOnly);
+  if (call != NULL) {
+    for (PINDEX i = 0; (connection = call->GetConnection(i)) != NULL; ++i) {
+      if (&connection->GetEndPoint() == this)
+        return connection.SetSafetyMode(mode) ? connection : NULL;
+    }
+  }
+
+  if (token.NumCompare(GetPrefixName()+':') != EqualTo)
+    return NULL;
+
+  PString name = token.Mid(GetPrefixName().GetLength()+1);
+  for (connection = PSafePtr<OpalConnection>(connectionsActive, PSafeReference); connection != NULL; ++connection) {
+    if (connection->GetLocalPartyName() == name)
+      return connection.SetSafetyMode(mode) ? connection : NULL;
+  }
+
+  return NULL;
 }
 
 
@@ -374,7 +497,6 @@ PStringList OpalEndPoint::GetAllConnections()
 
 PBoolean OpalEndPoint::HasConnection(const PString & token)
 {
-  PWaitAndSignal wait(inUseFlag);
   return connectionsActive.Contains(token);
 }
 
@@ -403,7 +525,7 @@ void OpalEndPoint::DestroyConnection(OpalConnection * connection)
 PBoolean OpalEndPoint::OnSetUpConnection(OpalConnection & PTRACE_PARAM(connection))
 {
   PTRACE(3, "OpalEP\tOnSetUpConnection " << connection);
-  return PTrue;
+  return true;
 }
 
 
@@ -470,6 +592,12 @@ PBoolean OpalEndPoint::OnForwarded(OpalConnection & connection,
 }
 
 
+bool OpalEndPoint::OnTransferNotify(OpalConnection & connection, const PStringToString & info)
+{
+  return manager.OnTransferNotify(connection, info);
+}
+
+
 void OpalEndPoint::ConnectionDict::DeleteObject(PObject * object) const
 {
   OpalConnection * connection = PDownCast(OpalConnection, object);
@@ -487,13 +615,19 @@ PBoolean OpalEndPoint::ClearCall(const PString & token,
 
 
 PBoolean OpalEndPoint::ClearCallSynchronous(const PString & token,
-                                        OpalConnection::CallEndReason reason,
-                                        PSyncPoint * sync)
+                                            OpalConnection::CallEndReason reason,
+                                            PSyncPoint * sync)
 {
   PSyncPoint syncPoint;
   if (sync == NULL)
     sync = &syncPoint;
-  return manager.ClearCall(token, reason, sync);
+
+  if (!ClearCall(token, reason, sync))
+    return false;
+
+  PTRACE(5, "OpalCon\tSynchronous wait for " << token);
+  sync->Wait();
+  return true;
 }
 
 
@@ -523,6 +657,57 @@ void OpalEndPoint::OnClosedMediaStream(const OpalMediaStream & stream)
   manager.OnClosedMediaStream(stream);
 }
 
+
+void OpalEndPoint::SetMediaCryptoSuites(const PStringArray & security)
+{
+  m_mediaCryptoSuites = security;
+
+  PStringArray valid = GetAllMediaCryptoSuites();
+  PAssert(!valid.IsEmpty(), PInvalidParameter);
+
+  PINDEX i = 0;
+  while (i < m_mediaCryptoSuites.GetSize()) {
+    if (valid.GetValuesIndex(m_mediaCryptoSuites[i]) != P_MAX_INDEX)
+      ++i;
+    else
+      m_mediaCryptoSuites.RemoveAt(i);
+  }
+
+  if (m_mediaCryptoSuites.IsEmpty())
+    m_mediaCryptoSuites.AppendString(valid[0]);
+}
+
+
+PStringArray OpalEndPoint::GetAllMediaCryptoSuites() const
+{
+  PStringArray cryptoSuites;
+
+  cryptoSuites.AppendString(OpalMediaCryptoSuite::ClearText());
+
+  OpalMediaCryptoSuiteFactory::KeyList_T all = OpalMediaCryptoSuiteFactory::GetKeyList();
+  for  (OpalMediaCryptoSuiteFactory::KeyList_T::iterator it = all.begin(); it != all.end(); ++it) {
+    if (*it != OpalMediaCryptoSuite::ClearText() && OpalMediaCryptoSuiteFactory::CreateInstance(*it)->Supports(GetPrefixName()))
+      cryptoSuites.AppendString(*it);
+  }
+
+  return cryptoSuites;
+}
+
+
+#if P_NAT
+PNatStrategy & OpalEndPoint::GetNatMethods() const
+{
+  return manager.GetNatMethods();
+}
+
+
+PNatMethod * OpalEndPoint::GetNatMethod(const PIPSocket::Address & remoteAddress) const
+{
+  return manager.GetNatMethod(remoteAddress);
+}
+#endif
+
+
 #if OPAL_VIDEO
 PBoolean OpalEndPoint::CreateVideoInputDevice(const OpalConnection & connection,
                                           const OpalMediaFormat & mediaFormat,
@@ -580,6 +765,46 @@ void OpalEndPoint::OnMWIReceived(const PString & party,
   manager.OnMWIReceived(party, type, extraInfo);
 }
 
+
+bool OpalEndPoint::GetConferenceStates(OpalConferenceStates &, const PString &) const
+{
+  return false;
+}
+
+
+void OpalEndPoint::OnConferenceStatusChanged(OpalEndPoint &, const PString &, OpalConferenceState::ChangeType)
+{
+}
+
+
+PStringList OpalEndPoint::GetNetworkURIs(const PString & name) const
+{
+  PStringList list;
+
+  const PStringList prefixes = manager.GetPrefixNames(this);
+
+  OpalTransportAddressArray addresses = GetInterfaceAddresses();
+  for (PINDEX i = 0; i < addresses.GetSize(); ++i) {
+    PIPSocket::Address ip;
+    WORD port = GetDefaultSignalPort();
+    if (addresses[i].GetIpAndPort(ip, port)) {
+      for (PStringList::const_iterator it = prefixes.begin(); it != prefixes.end(); ++it) {
+        PURL uri;
+        if (uri.SetScheme(*it)) {
+          uri.SetUserName(name);
+          uri.SetHostName(ip.AsString());
+          if (uri.GetPort() != port)
+            uri.SetPort(port);
+          list += uri.AsString();
+        }
+      }
+    }
+  }
+
+  return list;
+}
+
+
 bool OpalEndPoint::FindListenerForProtocol(const char * protoPrefix, OpalTransportAddress & addr)
 {
   OpalTransportAddress compatibleTo("*", 0, protoPrefix);
@@ -592,53 +817,48 @@ bool OpalEndPoint::FindListenerForProtocol(const char * protoPrefix, OpalTranspo
 }
 
 #if OPAL_PTLIB_SSL
-PString OpalEndPoint::GetSSLCertificate() const
+bool OpalEndPoint::ApplySSLCredentials(PSSLContext & context, bool create) const
 {
-  return "server.pem";
+  return manager.ApplySSLCredentials(*this, context, create);
 }
 #endif
 
 
+#if OPAL_HAS_IM
+
 bool OpalEndPoint::Message(const PString & to, const PString & body)
 {
-  PURL from;
-  PString conversationId;
-  return Message(to, "text/plain", body, from, conversationId);
+  return manager.Message(to, body);
 }
 
 
 PBoolean OpalEndPoint::Message(
-  const PURL & /*to*/, 
-  const PString & /*type*/,
-  const PString & /*body*/,
-  PURL & /*from*/, 
-  PString & /*conversationId*/
-)
-{
-  return false;
-}
-
-void OpalEndPoint::OnMessageReceived(
-  const PURL & from, 
-  const PString & fromName,
   const PURL & to, 
   const PString & type,
   const PString & body,
-  const PString & conversationId
+  PURL & from, 
+  PString & conversationId
 )
 {
-  manager.OnMessageReceived(from, fromName, to, type, body, conversationId);
+  return manager.Message(to, type, body, from, conversationId);
 }
 
-#if OPAL_HAS_IM
 
-bool OpalEndPoint::TransmitExternalIM(OpalConnection & conn, const OpalMediaFormat & format, RTP_IMFrame & frame)
+PBoolean OpalEndPoint::Message(OpalIM & message)
 {
-  return manager.TransmitExternalIM(conn, format, frame);
+  return manager.Message(message);
 }
 
-#endif
 
+void OpalEndPoint::OnMessageReceived(const OpalIM & message)
+{
+  manager.OnMessageReceived(message);
+}
+
+#endif // OPAL_HAS_IM
+
+
+/////////////////////////////////////////////////////////////////////////////
 
 bool OpalIsE164(const PString & number, bool strict)
 {
