@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24504 $
+ * $Revision: 29631 $
  * $Author: rjongbloed $
- * $Date: 2010-06-17 17:49:01 -0500 (Thu, 17 Jun 2010) $
+ * $Date: 2013-05-02 09:28:54 -0500 (Thu, 02 May 2013) $
  */
 
 #include <ptlib.h>
@@ -39,7 +39,7 @@
 
 #include <opal/mediafmt.h>
 #include <opal/mediacmd.h>
-#include <codec/opalplugin.h>
+#include <codec/opalplugin.hpp>
 #include <codec/opalwavfile.h>
 #include <ptlib/videoio.h>
 #include <ptclib/cypher.h>
@@ -88,61 +88,104 @@ public:
   }
 };
 
-
-const OpalAudioFormat & GetOpalL16_STEREO_48KHZ()
+const OpalAudioFormat & GetOpalPCM16S()
 {
-  static OpalStereoAudioFormat stereo48k("Linear-16-Stereo-48kHz",    // name of the meda format
-                                         RTP_DataFrame::DynamicBase,  // RTP payload code
-                                         "L16",                       // encoding name
-                                         16,                          // frame size in bytes
-                                         48,                          // frame time (1 ms in clock units)
-                                         20,                          // recommended rx frames/packet
-                                         20,                          // recommended tx frames/packet
-                                         50,                          // max tx frame size
-                                         48000);                      // clock rate
+  static OpalStereoAudioFormat stereo8k(OPAL_PCM16S,        		// name of the media format
+					RTP_DataFrame::MaxPayloadType,	// RTP payload code
+					"",				// encoding name
+					64,				// frame size in bytes
+					16,				// frame time (1 ms in clock units)
+					240,				// recommended rx frames/packet
+					0,				// recommended tx frames/packet
+					256,				// max tx frame size
+					8000);				// clock rate
+  return stereo8k;
+};
+
+const OpalAudioFormat & GetOpalPCM16S_16KHZ()
+{
+  static OpalStereoAudioFormat stereo16k(OPAL_PCM16S_16KHZ,		// name of the media format
+					RTP_DataFrame::MaxPayloadType,	// RTP payload code
+					"",				// encoding name
+					64,				// frame size in bytes
+					16,				// frame time (1 ms in clock units)
+					240,				// recommended rx frames/packet
+					0,				// recommended tx frames/packet
+					256,				// max tx frame size
+					16000);				// clock rate
+  return stereo16k;
+};
+
+const OpalAudioFormat & GetOpalL16_STEREO_16KHZ()
+{
+  static OpalStereoAudioFormat stereo16k(OPAL_L16_STEREO_16KHZ,         // name of the media format
+					RTP_DataFrame::L16_Stereo,      // RTP payload code
+					"L16S",                         // encoding name
+					64,                             // frame size in bytes
+					16,                             // frame time (1 ms in clock units)
+					240,                            // recommended rx frames/packet
+					30,                             // recommended tx frames/packet
+					256,                            // max tx frame size
+					16000);                         // clock rate
+  return stereo16k;
+};
+
+const OpalAudioFormat & GetOpalPCM16S_32KHZ()
+{
+  static OpalStereoAudioFormat stereo32k(OPAL_PCM16S_32KHZ,		// name of the media format
+					RTP_DataFrame::MaxPayloadType,	// RTP payload code
+					"",				// encoding name
+					128,				// frame size in bytes
+					32,				// frame time (1 ms in clock units)
+					240,				// recommended rx frames/packet
+					0,				// recommended tx frames/packet
+					256,				// max tx frame size
+					32000);				// clock rate
+  return stereo32k;
+};
+
+const OpalAudioFormat & GetOpalL16_STEREO_32KHZ()
+{
+  static OpalStereoAudioFormat stereo32k(OPAL_L16_STEREO_32KHZ,         // name of the media format
+					RTP_DataFrame::L16_Stereo,      // RTP payload code
+					"L16S",                         // encoding name
+					128,                            // frame size in bytes
+					32,                             // frame time (1 ms in clock units)
+					240,                            // recommended rx frames/packet
+					30,                             // recommended tx frames/packet
+					256,                            // max tx frame size
+					32000);                         // clock rate
+  return stereo32k;
+};
+
+const OpalAudioFormat & GetOpalPCM16S_48KHZ()
+{
+  static OpalStereoAudioFormat stereo48k(OPAL_PCM16S_48KHZ,		// name of the media format
+					RTP_DataFrame::MaxPayloadType,	// RTP payload code
+					"",				// encoding name
+					192,				// frame size in bytes
+					48,				// frame time (1 ms in clock units)
+					240,				// recommended rx frames/packet
+					0,				// recommended tx frames/packet
+					256,				// max tx frame size
+					48000);				// clock rate
   return stereo48k;
 };
 
-
-const OpalMediaFormat & GetOpalRFC2833()
+const OpalAudioFormat & GetOpalL16_STEREO_48KHZ()
 {
-  static struct OpalRFC2833MediaFormat : public OpalMediaFormat {
-    OpalRFC2833MediaFormat()
-      : OpalMediaFormat(OPAL_RFC2833,
-                        "userinput",
-                        (RTP_DataFrame::PayloadTypes)101,  // Set to this for Cisco compatibility
-                        "telephone-event",
-                        true,   // Needs jitter
-                        32*(1000/50), // bits/sec  (32 bits every 50ms)
-                        4,      // bytes/frame
-                        10*8,   // 10 millisecond
-                        OpalMediaFormat::AudioClockRate)
-    {
-      AddOption(new OpalMediaOptionString("FMTP", false, "0-16,32,36"));
-    }
-  } const RFC2833;
-  return RFC2833;
-}
+  static OpalStereoAudioFormat stereo48k(OPAL_L16_STEREO_48KHZ,         // name of the media format
+					RTP_DataFrame::L16_Stereo,      // RTP payload code
+					"L16S",                         // encoding name
+					192,                            // frame size in bytes
+					48,                             // frame time (1 ms in clock units)
+					240,                            // recommended rx frames/packet
+					30,                             // recommended tx frames/packet
+					256,                            // max tx frame size
+					48000);                         // clock rate
+  return stereo48k;
+};
 
-const OpalMediaFormat & GetOpalCiscoNSE()
-{
-  static struct OpalCiscoNSEMediaFormat : public OpalMediaFormat {
-    OpalCiscoNSEMediaFormat()
-      : OpalMediaFormat(OPAL_CISCONSE,
-                        "userinput",
-                        (RTP_DataFrame::PayloadTypes)100,  // Set to this for Cisco compatibility
-                        "NSE",
-                        true,   // Needs jitter
-                        32*(1000/50), // bits/sec  (32 bits every 50ms)
-                        4,      // bytes/frame
-                        10*8,   // 10 millisecond
-                        OpalMediaFormat::AudioClockRate)
-    {
-      AddOption(new OpalMediaOptionString("FMTP", false, "192,193"));
-    }
-  } const CiscoNSE;
-  return CiscoNSE;
-}
 
 static OpalMediaFormatList & GetMediaFormatsList()
 {
@@ -185,6 +228,49 @@ static void Clamp(OpalMediaFormatInternal & fmt1, const OpalMediaFormatInternal
 }
 
 
+/////////////////////////////////////////////////////////////////////////////
+
+std::ostream & operator<<(std::ostream & strm, OpalBandwidth::Direction dir)
+{
+  switch (dir) {
+    case OpalBandwidth::Rx :
+      return strm << "rx";
+    case OpalBandwidth::Tx :
+      return strm << "tx";
+    default :
+      return strm << "rx/tx";
+  }
+}
+
+
+std::ostream & operator<<(std::ostream & strm, const OpalBandwidth & bw)
+{
+  return strm << PString(PString::ScaleSI, bw.m_bps) << "b/s";
+}
+
+
+std::istream & operator>>(std::istream & strm, OpalBandwidth & bw)
+{
+  strm >> bw.m_bps;
+  if (strm.good()) {
+    switch (strm.peek()) {
+      case 'k' :
+        bw.m_bps *= 1000;
+        strm.ignore(1);
+        break;
+      case 'M' :
+        bw.m_bps *= 1000000;
+        strm.ignore(1);
+        break;
+      case 'G' :
+        bw.m_bps *= 1000000000;
+        strm.ignore(1);
+        break;
+    }
+  }
+  return strm;
+}
+
 /////////////////////////////////////////////////////////////////////////////
 
 OpalMediaOption::OpalMediaOption(const PString & name)
@@ -249,7 +335,8 @@ bool OpalMediaOption::Merge(const OpalMediaOption & option)
   }
 
   if (assign) {
-    PTRACE(4, "MediaFormat\tChanged media option \"" << m_name << "\" from " << *this << " to " << option);
+    PTRACE(4, "MediaFormat\tChanged media option \"" << m_name << "\" "
+              "from \"" << *this << "\" to \"" << option << '"');
     Assign(option);
   }
 
@@ -296,6 +383,37 @@ bool OpalMediaOption::FromString(const PString & value)
 }
 
 
+///////////////////////////////////////
+
+#if OPAL_H323
+OpalMediaOption::H245GenericInfo::H245GenericInfo()
+  : ordinal(0)
+  , mode(None)
+  , integerType(UnsignedInt)
+  , excludeTCS(false)
+  , excludeOLC(false)
+  , excludeReqMode(false)
+  , position(-1)
+{
+}
+
+
+OpalMediaOption::H245GenericInfo::H245GenericInfo(unsigned mask, const char * dflt)
+  : ordinal(mask&PluginCodec_H245_OrdinalMask)
+  , mode((mask&PluginCodec_H245_Collapsing) != 0 ? Collapsing : ((mask&PluginCodec_H245_NonCollapsing) != 0 ? NonCollapsing : None))
+  , integerType((mask&PluginCodec_H245_Unsigned32) != 0 ? Unsigned32 : ((mask&PluginCodec_H245_BooleanArray) != 0 ? BooleanArray : UnsignedInt))
+  , excludeTCS((mask&PluginCodec_H245_TCS) == 0)
+  , excludeOLC((mask&PluginCodec_H245_OLC) == 0)
+  , excludeReqMode((mask&PluginCodec_H245_ReqMode) == 0)
+  , position((mask&PluginCodec_H245_PositionMask)>>PluginCodec_H245_PositionShift)
+  , defaultValue(dflt)
+{
+  if (position == 0)
+    position = ordinal;
+}
+#endif
+
+
 ///////////////////////////////////////
 
 OpalMediaOptionEnum::OpalMediaOptionEnum(const char * name, bool readOnly)
@@ -328,15 +446,27 @@ PObject * OpalMediaOptionEnum::Clone() const
 
 void OpalMediaOptionEnum::PrintOn(ostream & strm) const
 {
-  if (m_value < m_enumerations.GetSize())
+  if (m_merge == IntersectionMerge) {
+    char ** arr = m_enumerations.ToCharArray();
+    PPrintBitwiseEnum(strm, m_value, arr);
+    free(arr);
+  }
+  else if (m_value < m_enumerations.GetSize())
     strm << m_enumerations[m_value];
   else
-    strm << psprintf("<%u>", m_value); // Don't output direct to stream so width() works correctly
+    strm << psprintf("<%u>", m_value); // Don't output direct to stream or width() is on '<' only
 }
 
 
 void OpalMediaOptionEnum::ReadFrom(istream & strm)
 {
+  if (m_merge == IntersectionMerge) {
+    char ** arr = m_enumerations.ToCharArray();
+    m_value = PReadBitwiseEnum(strm, arr);
+    free(arr);
+    return;
+  }
+
   m_value = m_enumerations.GetSize();
 
   PINDEX longestMatch = 0;
@@ -396,10 +526,15 @@ void OpalMediaOptionEnum::Assign(const OpalMediaOption & option)
 
 void OpalMediaOptionEnum::SetValue(PINDEX value)
 {
-  if (value < m_enumerations.GetSize())
+  PINDEX maxEnum = m_merge != IntersectionMerge ? m_enumerations.GetSize() : (1LL << m_enumerations.GetSize());
+  if (
+#ifndef __GNUC__
+      value >= 0 &&
+#endif
+      value < maxEnum)
     m_value = value;
   else {
-    m_value = m_enumerations.GetSize();
+    m_value = maxEnum;
     PTRACE(1, "MediaFormat\tIllegal value (" << value << ") for OpalMediaOptionEnum");
   }
 }
@@ -570,9 +705,9 @@ void OpalMediaOptionOctets::PrintOn(ostream & strm) const
     ios::fmtflags flags = strm.flags();
     char fill = strm.fill();
 
-    int fillLength = width - m_value.GetSize()*2;
+    streamsize fillLength = width - m_value.GetSize()*2;
     if (fillLength > 0 && (flags&ios_base::adjustfield) == ios::right) {
-      for (int i = 0; i < fillLength; i++)
+      for (streamsize i = 0; i < fillLength; i++)
         strm << fill;
     }
 
@@ -582,7 +717,7 @@ void OpalMediaOptionOctets::PrintOn(ostream & strm) const
 
     if (fillLength > 0 && (flags&ios_base::adjustfield) == ios::left) {
       strm << setw(1);
-      for (int i = 0; i < fillLength; i++)
+      for (std::streamsize i = 0; i < fillLength; i++)
         strm << fill;
     }
 
@@ -667,20 +802,21 @@ void OpalMediaOptionOctets::SetValue(const BYTE * data, PINDEX length)
 
 /////////////////////////////////////////////////////////////////////////////
 
-const PString & OpalMediaFormat::NeedsJitterOption()   { static PString s = PLUGINCODEC_OPTION_NEEDS_JITTER;    return s; }
-const PString & OpalMediaFormat::MaxFrameSizeOption()  { static PString s = PLUGINCODEC_OPTION_MAX_FRAME_SIZE;  return s; }
-const PString & OpalMediaFormat::FrameTimeOption()     { static PString s = PLUGINCODEC_OPTION_FRAME_TIME;      return s; }
-const PString & OpalMediaFormat::ClockRateOption()     { static PString s = PLUGINCODEC_OPTION_CLOCK_RATE;      return s; }
-const PString & OpalMediaFormat::MaxBitRateOption()    { static PString s = PLUGINCODEC_OPTION_MAX_BIT_RATE;    return s; }
-const PString & OpalMediaFormat::TargetBitRateOption() { static PString s = PLUGINCODEC_OPTION_TARGET_BIT_RATE; return s; }
+const PString & OpalMediaFormat::NeedsJitterOption()   { static const PConstString s(PLUGINCODEC_OPTION_NEEDS_JITTER);    return s; }
+const PString & OpalMediaFormat::MaxFrameSizeOption()  { static const PConstString s(PLUGINCODEC_OPTION_MAX_FRAME_SIZE);  return s; }
+const PString & OpalMediaFormat::FrameTimeOption()     { static const PConstString s(PLUGINCODEC_OPTION_FRAME_TIME);      return s; }
+const PString & OpalMediaFormat::ClockRateOption()     { static const PConstString s(PLUGINCODEC_OPTION_CLOCK_RATE);      return s; }
+const PString & OpalMediaFormat::MaxBitRateOption()    { static const PConstString s(PLUGINCODEC_OPTION_MAX_BIT_RATE);    return s; }
+const PString & OpalMediaFormat::TargetBitRateOption() { static const PConstString s(PLUGINCODEC_OPTION_TARGET_BIT_RATE); return s; }
 
 #if OPAL_H323
-const PString & OpalMediaFormat::MediaPacketizationOption()  { static PString s = PLUGINCODEC_MEDIA_PACKETIZATION;  return s; }
-const PString & OpalMediaFormat::MediaPacketizationsOption() { static PString s = PLUGINCODEC_MEDIA_PACKETIZATIONS; return s; }
+const PString & OpalMediaFormat::MediaPacketizationOption()  { static const PConstString s(PLUGINCODEC_MEDIA_PACKETIZATION);  return s; }
+const PString & OpalMediaFormat::MediaPacketizationsOption() { static const PConstString s(PLUGINCODEC_MEDIA_PACKETIZATIONS); return s; }
 #endif
 
-const PString & OpalMediaFormat::ProtocolOption()        { static PString s = "Protocol";           return s; }
-const PString & OpalMediaFormat::MaxTxPacketSizeOption() { static PString s = "Max Tx Packet Size"; return s; }
+const PString & OpalMediaFormat::ProtocolOption()        { static const PConstString s(PLUGINCODEC_OPTION_PROTOCOL); return s; }
+const PString & OpalMediaFormat::MaxTxPacketSizeOption() { static const PConstString s(PLUGINCODEC_OPTION_MAX_TX_PACKET_SIZE); return s; }
+
 
 OpalMediaFormat::OpalMediaFormat(OpalMediaFormatInternal * info)
   : m_info(NULL)
@@ -720,7 +856,7 @@ OpalMediaFormat::OpalMediaFormat(const char * fullName,
                                  RTP_DataFrame::PayloadTypes pt,
                                  const char * en,
                                  PBoolean     nj,
-                                 unsigned bw,
+                                 OpalBandwidth bw,
                                  PINDEX   fs,
                                  unsigned ft,
                                  unsigned cr,
@@ -730,6 +866,28 @@ OpalMediaFormat::OpalMediaFormat(const char * fullName,
 }
 
 
+OpalMediaFormat::OpalMediaFormat(const OpalMediaFormat & c)
+  : PContainer() // can't use PContainer copy c-tor as this c-tor must be synchronized
+  , m_info(NULL)
+{
+  PWaitAndSignal m(c.m_mutex); // here is no need to use mutex of the shared object
+  PContainer::AssignContents(c);
+  m_info = c.m_info;
+}
+
+
+OpalMediaFormat::~OpalMediaFormat()
+{
+  if (m_info != NULL)
+    m_info->media_format_mutex.Wait(); // don't use PWaitAndSignal as m_info can be removed
+
+  Destruct();
+
+  if (m_info != NULL)
+    m_info->media_format_mutex.Signal();
+}
+
+
 void OpalMediaFormat::Construct(OpalMediaFormatInternal * info)
 {
   if (info == NULL)
@@ -809,12 +967,21 @@ PBoolean OpalMediaFormat::MakeUnique()
 
 void OpalMediaFormat::AssignContents(const PContainer & c)
 {
-  m_mutex.Wait();
+  PWaitAndSignal m1(m_mutex);
 
-  PContainer::AssignContents(c);
-  m_info = ((const OpalMediaFormat &)c).m_info;
+  const OpalMediaFormat & other = (const OpalMediaFormat &)c;
+  PWaitAndSignal m2(other.m_mutex);
 
-  m_mutex.Signal();
+  if (m_info != NULL) {
+    m_info->media_format_mutex.Wait(); // don't use PWaitAndSignal as m_info can be removed
+    PContainer::AssignContents(c);
+    if (m_info != NULL)
+      m_info->media_format_mutex.Signal();
+  }
+  else // current object can't be removed
+    PContainer::AssignContents(c);
+
+  m_info = other.m_info;
 }
 
 
@@ -823,8 +990,8 @@ void OpalMediaFormat::DestroyContents()
   m_mutex.Wait();
 
   if (m_info != NULL) {
-    m_info->media_format_mutex.Wait();
     delete m_info;
+    m_info = NULL;
   }
 
   m_mutex.Signal();
@@ -921,30 +1088,30 @@ bool OpalMediaFormat::ValidateMerge(const OpalMediaFormat & mediaFormat) const
 }
 
 
-OpalMediaFormatList OpalMediaFormat::GetMatchingRegisteredMediaFormats(RTP_DataFrame::PayloadTypes rtpPayloadType,
-                                                                       const unsigned clockRate,
-                                                                       const char * rtpEncodingName,
-                                                                       const char * protocol)
+#if OPAL_H323
+PStringArray OpalMediaFormat::GetMediaPacketizations() const
 {
-  OpalMediaFormatList matches;
-
-  PWaitAndSignal mutex(GetMediaFormatsListMutex());
-
-  const OpalMediaFormatList & registeredFormats = GetMediaFormatsList();
-  OpalMediaFormatList::const_iterator format = registeredFormats.FindFormat(rtpPayloadType, clockRate, rtpEncodingName, protocol);
-  while (format != registeredFormats.end()) {
-    matches += *format;
-
-    if (++format == registeredFormats.end())
-      break;
-
-    format = registeredFormats.FindFormat(rtpPayloadType, clockRate, rtpEncodingName, protocol, format);
-  }
-
-  return matches;
+  return GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
+                         GetOptionString(OpalMediaFormat::MediaPacketizationOption())).Tokenise(",");
 }
 
 
+void OpalMediaFormat::SetMediaPacketizations(const PStringSet & packetizations)
+{
+  if (packetizations.IsEmpty()) {
+    SetOptionString(MediaPacketizationsOption(), PString::Empty());
+    SetOptionString(MediaPacketizationOption(),  PString::Empty());
+  }
+  else {
+    PStringStream strm;
+    strm << setfill(',') << packetizations;
+    SetOptionString(MediaPacketizationsOption(), strm);
+    SetOptionString(MediaPacketizationOption(),  packetizations.GetKeyAt(0));
+  }
+}
+#endif
+
+
 OpalMediaFormatList OpalMediaFormat::GetAllRegisteredMediaFormats()
 {
   OpalMediaFormatList copy;
@@ -983,6 +1150,22 @@ bool OpalMediaFormat::SetRegisteredMediaFormat(const OpalMediaFormat & mediaForm
 }
 
 
+bool OpalMediaFormat::RemoveRegisteredMediaFormat(const OpalMediaFormat & mediaFormat)
+{
+  PWaitAndSignal mutex(GetMediaFormatsListMutex());
+  OpalMediaFormatList & registeredFormats = GetMediaFormatsList();
+
+  for (OpalMediaFormatList::iterator format = registeredFormats.begin(); format != registeredFormats.end(); ++format) {
+    if (*format == mediaFormat) {
+      registeredFormats.erase(format);
+      return true;
+    }
+  }
+
+  return false;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 OpalMediaFormatInternal::OpalMediaFormatInternal(const char * fullName,
@@ -990,16 +1173,17 @@ OpalMediaFormatInternal::OpalMediaFormatInternal(const char * fullName,
                                                  RTP_DataFrame::PayloadTypes pt,
                                                  const char * en,
                                                  PBoolean     nj,
-                                                 unsigned bw,
+                                                 OpalBandwidth bw,
                                                  PINDEX   fs,
                                                  unsigned ft,
                                                  unsigned cr,
-                                                 time_t ts)
+                                                 time_t   ts)
   : formatName(fullName), mediaType(_mediaType), forceIsTransportable(false)
 {
-  codecVersionTime = ts;
-  rtpPayloadType = pt;
-  rtpEncodingName = en;
+  codecVersionTime = ts != 0 ? ts : PTime().GetTimeInSeconds();
+  rtpPayloadType   = pt;
+  rtpEncodingName  = en;
+  m_channels       = 1;    // this is the default - it's up to descendant classes to change it
 
   if (nj)
     AddOption(new OpalMediaOptionBoolean(OpalMediaFormat::NeedsJitterOption(), true, OpalMediaOption::OrMerge, true));
@@ -1024,7 +1208,7 @@ OpalMediaFormatInternal::OpalMediaFormatInternal(const char * fullName,
         rtpEncodingName.GetLength() > 0 &&
         rtpEncodingName[0] == '+') {
       forceIsTransportable = true;
-      rtpEncodingName = rtpEncodingName.Mid(1);
+      rtpEncodingName.Delete(0, 1);
     }
     return;
   }
@@ -1386,7 +1570,39 @@ bool OpalMediaFormatInternal::IsValidForProtocol(const PString & protocol) const
 
   // the protocol is only valid for SIP if the RTP name is not NULL
   if (protocol *= "sip")
-    return (rtpEncodingName != NULL) || (forceIsTransportable);
+    return !rtpEncodingName.IsEmpty() || forceIsTransportable;
+
+  return true;
+}
+
+
+bool OpalMediaFormatInternal::AdjustByOptionMaps(PTRACE_PARAM(const char * operation,)
+                  bool (*adjuster)(PluginCodec_OptionMap & original, PluginCodec_OptionMap & changed))
+{
+  PWaitAndSignal m(media_format_mutex);
+
+#if PTRACING
+  if (PTrace::CanTrace(5))
+    PTRACE(5, "OpalPlugin\t" << operation << ":\n" << setw(-1) << *this);
+  else
+    PTRACE(4, "OpalPlugin\t" << operation << ": " << *this);
+#endif
+
+  PluginCodec_OptionMap original;
+  for (PINDEX i = 0; i < options.GetSize(); i++)
+    original[options[i].GetName()] = options[i].AsString().GetPointer();
+
+  PluginCodec_OptionMap changed;
+  if (!adjuster(original, changed))
+    return false;
+
+  for (PluginCodec_OptionMap::const_iterator it = changed.begin(); it != changed.end(); ++it) {
+    PString oldValue;
+    if (GetOptionValue(it->first, oldValue) && oldValue != it->second.c_str()) {
+      PTRACE(3, "MediaFormat\tChanged option \"" << it->first << "\" from \"" << oldValue << "\" to \"" << it->second << '"');
+      SetOptionValue(it->first, it->second);
+    }
+  }
 
   return true;
 }
@@ -1473,10 +1689,10 @@ void OpalMediaFormatInternal::PrintOn(ostream & strm) const
 
 ///////////////////////////////////////////////////////////////////////////////
 
-const PString & OpalAudioFormat::RxFramesPerPacketOption() { static PString s = PLUGINCODEC_OPTION_RX_FRAMES_PER_PACKET; return s; }
-const PString & OpalAudioFormat::TxFramesPerPacketOption() { static PString s = PLUGINCODEC_OPTION_TX_FRAMES_PER_PACKET; return s; }
-const PString & OpalAudioFormat::MaxFramesPerPacketOption(){ static PString s = "Max Frames Per Packet"; return s; }
-const PString & OpalAudioFormat::ChannelsOption()          { static PString s = "Channels"; return s; }
+const PString & OpalAudioFormat::RxFramesPerPacketOption() { static const PConstString s(PLUGINCODEC_OPTION_RX_FRAMES_PER_PACKET); return s; }
+const PString & OpalAudioFormat::TxFramesPerPacketOption() { static const PConstString s(PLUGINCODEC_OPTION_TX_FRAMES_PER_PACKET); return s; }
+const PString & OpalAudioFormat::MaxFramesPerPacketOption(){ static const PConstString s("Max Frames Per Packet"); return s; }
+const PString & OpalAudioFormat::ChannelsOption()          { static const PConstString s("Channels"); return s; }
 
 OpalAudioFormat::OpalAudioFormat(const char * fullName,
                                  RTP_DataFrame::PayloadTypes rtpPayloadType,
@@ -1524,12 +1740,12 @@ OpalAudioFormatInternal::OpalAudioFormatInternal(const char * fullName,
                             timeStamp)
 {
   if (rxFrames > 0)
-    AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::RxFramesPerPacketOption(), false, OpalMediaOption::NoMerge,  rxFrames, 1, maxFrames));
+    AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::RxFramesPerPacketOption(), false, OpalMediaOption::MinMerge,  rxFrames, 1, maxFrames));
   if (txFrames > 0)
     AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::TxFramesPerPacketOption(), false, OpalMediaOption::AlwaysMerge, txFrames, 1, maxFrames));
 
   AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::MaxFramesPerPacketOption(), true,  OpalMediaOption::NoMerge,  maxFrames));
-  AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::ChannelsOption(),           false, OpalMediaOption::NoMerge,  1, 1, 5));
+  AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::ChannelsOption(),           false, OpalMediaOption::NoMerge,  m_channels, 1, 5));
 }
 
 
@@ -1557,18 +1773,20 @@ bool OpalAudioFormatInternal::Merge(const OpalMediaFormatInternal & mediaFormat)
 
 #if OPAL_VIDEO
 
-const PString & OpalVideoFormat::FrameWidthOption()               { static PString s = PLUGINCODEC_OPTION_FRAME_WIDTH;               return s; }
-const PString & OpalVideoFormat::FrameHeightOption()              { static PString s = PLUGINCODEC_OPTION_FRAME_HEIGHT;              return s; }
-const PString & OpalVideoFormat::MinRxFrameWidthOption()          { static PString s = PLUGINCODEC_OPTION_MIN_RX_FRAME_WIDTH;        return s; }
-const PString & OpalVideoFormat::MinRxFrameHeightOption()         { static PString s = PLUGINCODEC_OPTION_MIN_RX_FRAME_HEIGHT;       return s; }
-const PString & OpalVideoFormat::MaxRxFrameWidthOption()          { static PString s = PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH;        return s; }
-const PString & OpalVideoFormat::MaxRxFrameHeightOption()         { static PString s = PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT;       return s; }
-const PString & OpalVideoFormat::TemporalSpatialTradeOffOption()  { static PString s = PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF;return s; }
-const PString & OpalVideoFormat::TxKeyFramePeriodOption()         { static PString s = PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD;       return s; }
-const PString & OpalVideoFormat::RateControlEnableOption()        { static PString s = "Rate Control Enable";                        return s; }
-const PString & OpalVideoFormat::RateControllerOption()           { static PString s = "Rate Controller";                            return s; }
-const PString & OpalVideoFormat::ContentRoleOption()              { static PString s = "Content Role";                               return s; }
-const PString & OpalVideoFormat::ContentRoleMaskOption()          { static PString s = "Content Role Mask";                          return s; }
+const PString & OpalVideoFormat::FrameWidthOption()               { static const PConstString s(PLUGINCODEC_OPTION_FRAME_WIDTH);               return s; }
+const PString & OpalVideoFormat::FrameHeightOption()              { static const PConstString s(PLUGINCODEC_OPTION_FRAME_HEIGHT);              return s; }
+const PString & OpalVideoFormat::MinRxFrameWidthOption()          { static const PConstString s(PLUGINCODEC_OPTION_MIN_RX_FRAME_WIDTH);        return s; }
+const PString & OpalVideoFormat::MinRxFrameHeightOption()         { static const PConstString s(PLUGINCODEC_OPTION_MIN_RX_FRAME_HEIGHT);       return s; }
+const PString & OpalVideoFormat::MaxRxFrameWidthOption()          { static const PConstString s(PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH);        return s; }
+const PString & OpalVideoFormat::MaxRxFrameHeightOption()         { static const PConstString s(PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT);       return s; }
+const PString & OpalVideoFormat::TemporalSpatialTradeOffOption()  { static const PConstString s(PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF);return s; }
+const PString & OpalVideoFormat::TxKeyFramePeriodOption()         { static const PConstString s(PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD);       return s; }
+const PString & OpalVideoFormat::RateControlPeriodOption()        { static const PConstString s(PLUGINCODEC_OPTION_RATE_CONTROL_PERIOD);       return s; }
+const PString & OpalVideoFormat::RateControllerOption()           { static const PConstString s("Rate Controller");                            return s; }
+const PString & OpalVideoFormat::ContentRoleOption()              { static const PConstString s("Content Role");                               return s; }
+const PString & OpalVideoFormat::ContentRoleMaskOption()          { static const PConstString s("Content Role Mask");                          return s; }
+const PString & OpalVideoFormat::RTCPFeedbackOption()             { static const PConstString s("RTCP Feedback"); return s; }
+
 
 OpalVideoFormat::OpalVideoFormat(const char * fullName,
                                  RTP_DataFrame::PayloadTypes rtpPayloadType,
@@ -1602,7 +1820,7 @@ OpalVideoFormatInternal::OpalVideoFormatInternal(const char * fullName,
                             "video",
                             rtpPayloadType,
                             encodingName,
-                            PFalse,
+                            false,
                             maxBitRate,
                             0,
                             OpalMediaFormat::VideoClockRate/maxFrameRate,
@@ -1617,12 +1835,13 @@ OpalVideoFormatInternal::OpalVideoFormatInternal(const char * fullName,
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::MaxRxFrameHeightOption(),         false, OpalMediaOption::MinMerge,    maxFrameHeight,              16,  32767));
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TargetBitRateOption(),            false, OpalMediaOption::AlwaysMerge, maxBitRate,                  1000      ));
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TxKeyFramePeriodOption(),         false, OpalMediaOption::AlwaysMerge, 125,                         0,    1000));
-  AddOption(new OpalMediaOptionBoolean (OpalVideoFormat::RateControlEnableOption(),        false, OpalMediaOption::NoMerge,     false                                  ));
-  AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::MaxTxPacketSizeOption(),          false, OpalMediaOption::NoMerge,     8192,                        100,  8192));
+  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlPeriodOption(),        false, OpalMediaOption::AlwaysMerge, 1000,                        100, 60000));
+  AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::MaxTxPacketSizeOption(),          true,  OpalMediaOption::AlwaysMerge, PluginCodec_RTP_MaxPayloadSize, 100    ));
   AddOption(new OpalMediaOptionString  (OpalVideoFormat::RateControllerOption(),           false                                                                       ));
+  AddOption(new OpalMediaOptionEnum    (OpalVideoFormat::RTCPFeedbackOption(),             false,  OpalVideoFormat::RTCPFeedback::Names(), P_MAX_INDEX,
+                                     OpalMediaOption::IntersectionMerge, OpalVideoFormat::e_PLI|OpalVideoFormat::e_FIR|OpalVideoFormat::e_TMMBR|OpalVideoFormat::e_TSTR));
 
-
-  static const char * const RoleEnumerations[OpalVideoFormat::eNumRoles] = {
+  static const char * const RoleEnumerations[OpalVideoFormat::EndContentRole] = {
     "No Role",
     "Presentation",
     "Main",
@@ -1635,7 +1854,8 @@ OpalVideoFormatInternal::OpalVideoFormatInternal(const char * fullName,
 
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::ContentRoleMaskOption(),
                                         false, OpalMediaOption::IntersectionMerge,
-                                        0, 0, OpalVideoFormat::ContentRoleMask));
+                                        OpalVideoFormat::ContentRoleMask,
+                                        0, OpalVideoFormat::ContentRoleMask));
 
   // For video the max bit rate and frame rate is adjustable by user
   FindOption(OpalVideoFormat::MaxBitRateOption())->SetReadOnly(false);
@@ -1690,6 +1910,23 @@ OpalMediaFormatList::OpalMediaFormatList(const OpalMediaFormat & format)
 }
 
 
+OpalMediaFormatList & OpalMediaFormatList::operator+=(const PString & wildcard)
+{
+  MakeUnique();
+
+  PWaitAndSignal mutex(GetMediaFormatsListMutex());
+  OpalMediaFormatList & registeredFormats = GetMediaFormatsList();
+
+  OpalMediaFormatList::const_iterator fmt;
+  while ((fmt = registeredFormats.FindFormat(wildcard, fmt)) != registeredFormats.end()) {
+    if (!HasFormat(*fmt))
+      OpalMediaFormatBaseList::Append(fmt->Clone());
+  }
+
+  return *this;
+}
+
+
 OpalMediaFormatList & OpalMediaFormatList::operator+=(const OpalMediaFormat & format)
 {
   MakeUnique();
@@ -1735,19 +1972,65 @@ void OpalMediaFormatList::Remove(const PStringArray & maskList)
 
   PTRACE(4,"MediaFormat\tRemoving codecs " << setfill(',') << maskList);
 
-  for (PINDEX i = 0; i < maskList.GetSize(); i++) {
-    OpalMediaFormatList::const_iterator fmt;
-    while ((fmt = FindFormat(maskList[i])) != end())
-      erase(fmt);
+  PINDEX i;
+  PStringList notMasks;
+  const_iterator fmt;
+
+  for (i = 0; i < maskList.GetSize(); i++) {
+    PString mask = maskList[i];
+    if (mask[0] == '!')
+      notMasks.AppendString(mask);
+    else {
+      while ((fmt = FindFormat(mask)) != end())
+        erase(fmt);
+    }
+  }
+
+  switch (notMasks.GetSize()) {
+    case 0 :
+      return;
+
+    case 1 :
+      while ((fmt = FindFormat(notMasks[0])) != end())
+        erase(fmt);
+      return;
+  }
+
+  OpalMediaFormatList formatsToKeep;
+  for (i = 0; i < notMasks.GetSize(); i++) {
+    PString keeper = notMasks[i].Mid(1);
+    fmt = const_iterator();
+    while ((fmt = FindFormat(keeper, fmt)) != end())
+      formatsToKeep += *fmt;
+  }
+
+  *this = formatsToKeep;
+}
+
+
+void OpalMediaFormatList::RemoveNonTransportable()
+{
+  iterator it = begin();
+  while (it != end()) {
+    if (it->IsTransportable())
+      ++it;
+    else
+      erase(it++);
   }
 }
 
 
-OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(RTP_DataFrame::PayloadTypes pt, unsigned clockRate, const char * name, const char * protocol, const_iterator iter) const
+OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(RTP_DataFrame::PayloadTypes pt, unsigned clockRate, const char * name, const char * protocol, const_iterator format) const
 {
+  if (format == const_iterator())
+    format = begin();
+  else
+    ++format;
+
   // First look for a matching encoding name
   if (name != NULL && *name != '\0') {
-    OpalMediaFormatList::const_iterator format ((iter == const_iterator()) ? begin() : iter);
+    OpalMediaFormatList::const_iterator savedIterationStart = format;
+
     for (; format != end(); ++format) {
       // If encoding name matches exactly, then use it regardless of payload code.
       const char * otherName = format->GetEncodingName();
@@ -1756,20 +2039,19 @@ OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(RTP_DataFram
           (protocol  == NULL || format->IsValidForProtocol(protocol))) // if protocol is specified, must be valid for the protocol
         return format;
     }
+
+    format = savedIterationStart;
   }
 
   // Can't match by encoding name, try by known payload type.
   // Note we do two separate loops as it is possible (though discouraged) for
   // someone to override a standard payload type with another encoding name, so
   // have to search all formats by name before trying by number.
-  if (pt < RTP_DataFrame::DynamicBase) {
-    OpalMediaFormatList::const_iterator format ((iter == const_iterator()) ? begin() : iter);
-    for (; format != end(); ++format) {
-      if (format->GetPayloadType() == pt &&
-          (clockRate == 0    || clockRate == format->GetClockRate()) && // if have clock rate, clock rate must match
-          (protocol  == NULL || format->IsValidForProtocol(protocol))) // if protocol is specified, must be valid for the protocol
-        return format;
-    }
+  for (; format != end(); ++format) {
+    if (format->GetPayloadType() == pt &&
+        (clockRate == 0    || clockRate == format->GetClockRate()) && // if have clock rate, clock rate must match
+        (protocol  == NULL || format->IsValidForProtocol(protocol))) // if protocol is specified, must be valid for the protocol
+      return format;
   }
 
   return end();
@@ -1817,6 +2099,8 @@ OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(const PStrin
 
   if (iter == const_iterator())
     iter = begin();
+  else
+    ++iter;
 
   bool negative = search[0] == '!';
 
@@ -1848,41 +2132,52 @@ OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(const PStrin
 void OpalMediaFormatList::Reorder(const PStringArray & order)
 {
   DisallowDeleteObjects();
-  PINDEX nextPos = 0;
+
+  OpalMediaFormatBaseList::iterator orderedIter = begin();
+
   for (PINDEX i = 0; i < order.GetSize(); i++) {
     if (order[i][0] == '@') {
       OpalMediaType mediaType = order[i].Mid(1);
-      PINDEX findPos = 0;
-      while (findPos < GetSize()) {
-        if ((*this)[findPos].GetMediaType() == mediaType) {
-          if (findPos > nextPos)
-            OpalMediaFormatBaseList::InsertAt(nextPos, RemoveAt(findPos));
-          nextPos++;
+
+      OpalMediaFormatBaseList::iterator findIter = orderedIter;
+      while (findIter != end()) {
+        if (findIter->GetMediaType() != mediaType)
+          ++findIter;
+        else if (findIter == orderedIter) {
+          ++orderedIter;
+          ++findIter;
+        }
+        else {
+          insert(orderedIter, &*findIter);
+          erase(findIter++);
         }
-        findPos++;
       }
     }
     else {
       PStringArray wildcards = order[i].Tokenise('*', true);
 
-      PINDEX findPos = 0;
-      while (findPos < GetSize()) {
-        if (WildcardMatch((*this)[findPos].m_info->formatName, wildcards)) {
-          if (findPos > nextPos)
-            OpalMediaFormatBaseList::InsertAt(nextPos, RemoveAt(findPos));
-          nextPos++;
+      OpalMediaFormatBaseList::iterator findIter = orderedIter;
+      while (findIter != end()) {
+        if (!WildcardMatch(findIter->GetName(), wildcards))
+          ++findIter;
+        else if (findIter == orderedIter) {
+          ++orderedIter;
+          ++findIter;
+        }
+        else {
+          insert(orderedIter, &*findIter);
+          erase(findIter++);
         }
-        findPos++;
       }
     }
   }
+
   AllowDeleteObjects();
 }
 
 bool OpalMediaFormatList::HasType(const OpalMediaType & type, bool mustBeTransportable) const
 {
-  OpalMediaFormatList::const_iterator format;
-  for (format = begin(); format != end(); ++format) {
+  for (OpalMediaFormatList::const_iterator format = begin(); format != end(); ++format) {
     if (format->GetMediaType() == type && (!mustBeTransportable || format->IsTransportable()))
       return true;
   }
@@ -1891,4 +2186,18 @@ bool OpalMediaFormatList::HasType(const OpalMediaType & type, bool mustBeTranspo
 }
 
 
+OpalMediaTypeList OpalMediaFormatList::GetMediaTypes() const
+{
+  OpalMediaTypeList mediaTypes;
+
+  for (OpalMediaFormatList::const_iterator format = begin(); format != end(); ++format) {
+    OpalMediaType mediaType = format->GetMediaType();
+    if (std::find(mediaTypes.begin(), mediaTypes.end(), mediaType) == mediaTypes.end())
+      mediaTypes.push_back(mediaType);
+  }
+
+  return mediaTypes;
+}
+
+
 // End of File ///////////////////////////////////////////////////////////////
