@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23686 $
+ * $Revision: 28481 $
  * $Author: rjongbloed $
- * $Date: 2009-10-19 23:19:11 -0500 (Mon, 19 Oct 2009) $
+ * $Date: 2012-10-05 03:13:17 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -37,7 +37,7 @@
 #include <opal/buildopts.h>
 
 #include <opal/mediatype.h>
-#include <opal/rtpconn.h>
+#include <rtp/rtpconn.h>
 #include <opal/call.h>
 #include <opal/endpoint.h>
 #include <opal/manager.h>
@@ -46,14 +46,16 @@
 #include <h323/channels.h>
 #endif
 
+#include <algorithm>
 
-OPAL_INSTANTIATE_MEDIATYPE(audio, OpalAudioMediaType);
+
+OPAL_INSTANTIATE_MEDIATYPE(OpalAudioMediaType);
 
 #if OPAL_VIDEO
-OPAL_INSTANTIATE_MEDIATYPE(video, OpalVideoMediaType);
+OPAL_INSTANTIATE_MEDIATYPE(OpalVideoMediaType);
 #endif
 
-OPAL_INSTANTIATE_SIMPLE_MEDIATYPE_NO_SDP(userinput); 
+OPAL_INSTANTIATE_SIMPLE_MEDIATYPE(UserInputMediaType, "userinput");
 
 
 typedef std::map<unsigned, OpalMediaTypeDefinition *> SessionIDToMediaTypeMap_T;
@@ -73,23 +75,27 @@ static PMutex & GetMapMutex()
 
 ///////////////////////////////////////////////////////////////////////////////
 
-const OpalMediaType & OpalMediaType::Audio()     { static const OpalMediaType type = "audio";     return type; }
-const OpalMediaType & OpalMediaType::Video()     { static const OpalMediaType type = "video";     return type; }
-const OpalMediaType & OpalMediaType::Fax()       { static const OpalMediaType type = "fax";       return type; };
-const OpalMediaType & OpalMediaType::UserInput() { static const OpalMediaType type = "userinput"; return type; };
+const OpalMediaType & OpalMediaType::Audio()     { static const OpalMediaType type = OpalAudioMediaType::Name(); return type; }
+#if OPAL_VIDEO
+const OpalMediaType & OpalMediaType::Video()     { static const OpalMediaType type = OpalVideoMediaType::Name(); return type; }
+#endif
+#if OPAL_T38_CAPABILITY
+const OpalMediaType & OpalMediaType::Fax()       { static const OpalMediaType type = OpalFaxMediaType::Name();   return type; };
+#endif
+const OpalMediaType & OpalMediaType::UserInput() { static const OpalMediaType type = UserInputMediaType::Name(); return type; };
 
 
 ///////////////////////////////////////////////////////////////////////////////
 
 OpalMediaTypeDefinition * OpalMediaType::GetDefinition() const
 {
-  return OpalMediaTypeFactory::CreateInstance(*this);
+  return OpalMediaTypesFactory::CreateInstance(*this);
 }
 
 
 OpalMediaTypeDefinition * OpalMediaType::GetDefinition(const OpalMediaType & key)
 {
-  return OpalMediaTypeFactory::CreateInstance(key);
+  return OpalMediaTypesFactory::CreateInstance(key);
 }
 
 
@@ -102,21 +108,34 @@ OpalMediaTypeDefinition * OpalMediaType::GetDefinition(unsigned sessionId)
 }
 
 
+OpalMediaTypeList OpalMediaType::GetList()
+{
+  OpalMediaTypesFactory::KeyList_T types = OpalMediaTypesFactory::GetKeyList();
+  OpalMediaTypesFactory::KeyList_T::iterator it;
+
+  it = std::find(types.begin(), types.end(), Audio());
+  if (it != types.end())
+    std::swap(*types.begin(), *it);
+
+#if OPAL_VIDEO
+  it = std::find(types.begin(), types.end(), Video());
+  if (it != types.end())
+    std::swap(*(types.begin()+1), *it);
+#endif
+
+  return types;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 OpalMediaTypeDefinition::OpalMediaTypeDefinition(const char * mediaType,
-#if OPAL_SIP
-                                                 const char * sdpType,
-#else
-                                                 const char *,
-#endif
+                                                 const char * mediaSession,
                                                  unsigned requiredSessionId,
                                                  OpalMediaType::AutoStartMode autoStart)
   : m_mediaType(mediaType)
+  , m_mediaSessionType(mediaSession)
   , m_autoStart(autoStart)
-#if OPAL_SIP
-  , m_sdpType(sdpType != NULL ? sdpType : "")
-#endif
 {
   PWaitAndSignal mutex(GetMapMutex());
 
@@ -137,51 +156,28 @@ OpalMediaTypeDefinition::OpalMediaTypeDefinition(const char * mediaType,
 }
 
 
-OpalMediaSession * OpalMediaTypeDefinition::CreateMediaSession(OpalConnection & /*conn*/, unsigned /* sessionID*/) const
-{ 
-  return NULL; 
-}
-
-
-RTP_UDP * OpalMediaTypeDefinition::CreateRTPSession(OpalRTPConnection & connection,
-                                                    unsigned sessionID,
-                                                    bool remoteIsNAT)
+OpalMediaTypeDefinition::~OpalMediaTypeDefinition()
 {
-  RTP_Session::Params params;
-  params.id = sessionID;
-  params.encoding = GetRTPEncoding();
-  params.isAudio = m_mediaType == OpalMediaType::Audio();
-  params.remoteIsNAT = remoteIsNAT;
-
-  return connection.GetEndPoint().GetManager().CreateRTPSession(params);
+  GetSessionIDToMediaTypeMap().erase(m_defaultSessionId);
 }
 
 
 ///////////////////////////////////////////////////////////////////////////////
 
 OpalRTPAVPMediaType::OpalRTPAVPMediaType(const char * mediaType,
-                                         const char * sdpType,
                                          unsigned requiredSessionId,
                                          OpalMediaType::AutoStartMode autoStart)
-  : OpalMediaTypeDefinition(mediaType, sdpType, requiredSessionId, autoStart)
+  : OpalMediaTypeDefinition(mediaType, OpalRTPSession::RTP_AVP(), requiredSessionId, autoStart)
 {
 }
 
-PString OpalRTPAVPMediaType::GetRTPEncoding() const
-{
-  return "rtp/avp";
-}
-
-OpalMediaSession * OpalRTPAVPMediaType::CreateMediaSession(OpalConnection & conn, unsigned sessionID) const
-{
-  return new OpalRTPMediaSession(conn, m_mediaType, sessionID, NULL);
-}
-
 
 ///////////////////////////////////////////////////////////////////////////////
 
+const char * OpalAudioMediaType::Name() { return "audio"; }
+
 OpalAudioMediaType::OpalAudioMediaType()
-  : OpalRTPAVPMediaType("audio", "audio", 1, OpalMediaType::ReceiveTransmit)
+  : OpalRTPAVPMediaType(Name(), 1, OpalMediaType::ReceiveTransmit)
 {
 }
 
@@ -190,8 +186,10 @@ OpalAudioMediaType::OpalAudioMediaType()
 
 #if OPAL_VIDEO
 
+const char * OpalVideoMediaType::Name() { return "video"; }
+
 OpalVideoMediaType::OpalVideoMediaType()
-  : OpalRTPAVPMediaType("video", "video", 2)
+  : OpalRTPAVPMediaType(Name(), 2)
 { }
 
 #endif // OPAL_VIDEO
