@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26088 $
+ * $Revision: 29992 $
  * $Author: rjongbloed $
- * $Date: 2011-06-28 01:51:16 -0500 (Tue, 28 Jun 2011) $
+ * $Date: 2013-06-19 00:11:09 -0500 (Wed, 19 Jun 2013) $
  */
 
 #include <ptlib.h>
@@ -40,6 +40,7 @@
 
 #include <opal/patch.h>
 #include <opal/mediastrm.h>
+#include <opal/endpoint.h>
 #include <opal/transcoders.h>
 
 #if OPAL_VIDEO
@@ -53,37 +54,32 @@
 
 OpalMediaPatch::OpalMediaPatch(OpalMediaStream & src)
   : source(src)
-#if OPAL_VIDEO
-  , m_videoDecoder(src.GetMediaFormat().IsTransportable() && src.GetMediaFormat().GetMediaType() == OpalMediaType::Video())
-#endif
-  , m_bypassActive(false)
   , m_bypassToPatch(NULL)
   , m_bypassFromPatch(NULL)
   , patchThread(NULL)
 {
+  PTRACE_CONTEXT_ID_FROM(src);
+
   PTRACE(5, "Patch\tCreated media patch " << this << ", session " << src.GetSessionID());
   src.SetPatch(this);
+  source.SafeReference();
 }
 
 
 OpalMediaPatch::~OpalMediaPatch()
 {
-  PWaitAndSignal m(patchThreadMutex);
-  inUse.StartWrite();
-  if (patchThread != NULL) {
-    PAssert(patchThread->WaitForTermination(10000), "Media patch thread not terminated.");
-    delete patchThread;
-    patchThread = NULL;
-  }
+  StopThread();
+  source.SafeDereference();
   PTRACE(5, "Patch\tDestroyed media patch " << this);
 }
 
 
 void OpalMediaPatch::PrintOn(ostream & strm) const
 {
-  strm << "Patch[" << this << "] " << source;
+  strm << GetClass() << '[' << this << "] " << source;
 
-  inUse.StartRead();
+  if (!LockReadOnly())
+    return;
 
   if (sinks.GetSize() > 0) {
     strm << " -> ";
@@ -99,28 +95,67 @@ void OpalMediaPatch::PrintOn(ostream & strm) const
     }
   }
 
-  inUse.EndRead();
+  UnlockReadOnly();
 }
 
 void OpalMediaPatch::Start()
 {
   PWaitAndSignal m(patchThreadMutex);
 	
-  if(patchThread != NULL) 
+  if(patchThread != NULL) {
+    PTRACE(5, "Media\tAlready started thread " << patchThread->GetThreadName());
     return;
-	
-  patchThread = new Thread(*this);
-  patchThread->Resume();
+  }
+
+  if (!source.IsOpen()) {
+    PTRACE(4, "Media\tDelaying thread starting till source stream open");
+    return;
+  }
+
+  if (sinks.IsEmpty()) {
+    PTRACE(4, "Media\tDelaying thread starting till have sink stream");
+    return;
+  }
+
+  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
+    if (!s->stream->IsOpen()) {
+      PTRACE(4, "Media\tDelaying thread starting till sink stream open");
+      return;
+    }
+  }
+
+  patchThread = new PThreadObj<OpalMediaPatch>(*this, &OpalMediaPatch::Main, false, "Media Patch", PThread::HighPriority);
+  PTRACE_CONTEXT_ID_TO(patchThread);
   PThread::Yield();
   PTRACE(4, "Media\tStarting thread " << patchThread->GetThreadName());
 }
 
 
+void OpalMediaPatch::StopThread()
+{
+  patchThreadMutex.Wait();
+  PThread * thread = patchThread;
+  patchThread = NULL;
+  patchThreadMutex.Signal();
+
+  if (thread == NULL)
+    return;
+
+  if (!thread->IsSuspended()) {
+    PTRACE(4, "Patch\tWaiting for media patch thread to stop " << *this);
+    PAssert(thread->WaitForTermination(10000), "Media patch thread not terminated.");
+  }
+
+  delete thread;
+}
+
+
 void OpalMediaPatch::Close()
 {
   PTRACE(3, "Patch\tClosing media patch " << *this);
 
-  inUse.StartWrite();
+  if (!LockReadWrite())
+    return;
 
   if (m_bypassFromPatch != NULL)
     m_bypassFromPatch->SetBypassPatch(NULL);
@@ -128,38 +163,34 @@ void OpalMediaPatch::Close()
     SetBypassPatch(NULL);
 
   filters.RemoveAll();
-  if (source.GetPatch() == this)
+  if (source.GetPatch() == this) {
+    UnlockReadWrite();
     source.Close();
+    if (!LockReadWrite())
+      return;
+  }
 
   while (sinks.GetSize() > 0) {
     OpalMediaStreamPtr stream = sinks.front().stream;
-    inUse.EndWrite();
-    if (!stream->Close()) {
+    UnlockReadWrite();
+    if (stream == NULL || !stream->Close()) {
       // The only way we can get here is if the sink is in the proccess of being closed
-      // but is blocked on the inUse mutex waiting to remove the sink from this patch.
+      // but is blocked on the mutex waiting to remove the sink from this patch.
       // Se we unlock it, and wait for it to do it in the other thread.
       PThread::Sleep(10);
     }
-    inUse.StartWrite();
-  }
-
-  PTRACE(4, "Patch\tWaiting for media patch thread to stop " << *this);
-  {
-    PWaitAndSignal m(patchThreadMutex);
-    if (patchThread != NULL && !patchThread->IsSuspended()) {
-      inUse.EndWrite();
-      PAssert(patchThread->WaitForTermination(10000), "Media patch thread not terminated.");
+    if (!LockReadWrite())
       return;
-    }
   }
+  UnlockReadWrite();
 
-  inUse.EndWrite();
+  StopThread();
 }
 
 
 PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   if (PAssertNULL(sinkStream) == NULL)
     return false;
@@ -173,10 +204,37 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
 
   Sink * sink = new Sink(*this, sinkStream);
   sinks.Append(sink);
+  if (!sink->CreateTranscoders())
+    return false;
+
+  EnableJitterBuffer();
+  return true;
+}
+
+
+bool OpalMediaPatch::ResetTranscoders()
+{
+  PSafeLockReadWrite mutex(*this);
+
+  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
+    if (!s->CreateTranscoders())
+      return false;
+  }
+
+  return true;
+}
+
+
+bool OpalMediaPatch::Sink::CreateTranscoders()
+{
+  delete primaryCodec;
+  primaryCodec =  NULL;
+  delete secondaryCodec;
+  secondaryCodec = NULL;
 
   // Find the media formats than can be used to get from source to sink
-  OpalMediaFormat sourceFormat = source.GetMediaFormat();
-  OpalMediaFormat destinationFormat = sinkStream->GetMediaFormat();
+  OpalMediaFormat sourceFormat = patch.source.GetMediaFormat();
+  OpalMediaFormat destinationFormat = stream->GetMediaFormat();
 
   PTRACE(5, "Patch\tAddSink\n"
             "Source format:\n" << setw(-1) << sourceFormat << "\n"
@@ -187,99 +245,110 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
                                   sourceFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1));
     PINDEX packetSize = sourceFormat.GetFrameSize()*framesPerPacket;
     PINDEX packetTime = sourceFormat.GetFrameTime()*framesPerPacket;
-    source.SetDataSize(packetSize, packetTime);
-    sinkStream->SetDataSize(packetSize, packetTime);
-    PTRACE(3, "Patch\tAdded direct media stream sink " << *sinkStream);
-    EnableJitterBuffer();
+    patch.source.SetDataSize(packetSize, packetTime);
+    stream->SetDataSize(packetSize, packetTime);
+    stream->InternalUpdateMediaFormat(stream->GetMediaFormat());
+    patch.source.InternalUpdateMediaFormat(patch.source.GetMediaFormat());
+    PTRACE(3, "Patch\tAdded direct media stream sink " << *stream);
     return true;
   }
 
-  PString id = sinkStream->GetID();
-  sink->primaryCodec = OpalTranscoder::Create(sourceFormat, destinationFormat, (const BYTE *)id, id.GetLength());
-  if (sink->primaryCodec != NULL) {
+  PString id = stream->GetID();
+  primaryCodec = OpalTranscoder::Create(sourceFormat, destinationFormat, (const BYTE *)id, id.GetLength());
+  if (primaryCodec != NULL) {
+    PTRACE_CONTEXT_ID_TO(primaryCodec);
     PTRACE(4, "Patch\tCreated primary codec " << sourceFormat << "->" << destinationFormat << " with ID " << id);
 
-    if (!sinkStream->SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(false), sourceFormat.GetFrameTime())) {
-      PTRACE(1, "Patch\tSink stream " << *sinkStream << " cannot support data size "
-              << sink->primaryCodec->GetOptimalDataFrameSize(PFalse));
+    if (!stream->SetDataSize(primaryCodec->GetOptimalDataFrameSize(false), primaryCodec->GetInputFormat().GetFrameTime())) {
+      PTRACE(1, "Patch\tSink stream " << *stream << " cannot support data size "
+              << primaryCodec->GetOptimalDataFrameSize(false));
       return false;
     }
-    sink->primaryCodec->SetMaxOutputSize(sinkStream->GetDataSize());
-    sink->primaryCodec->SetSessionID(source.GetSessionID());
+    primaryCodec->SetMaxOutputSize(stream->GetDataSize());
+    primaryCodec->SetSessionID(patch.source.GetSessionID());
+    primaryCodec->SetCommandNotifier(PCREATE_NOTIFIER_EXT(&patch, OpalMediaPatch, InternalOnMediaCommand1));
 
-    PTRACE(3, "Patch\tAdded media stream sink " << *sinkStream
-           << " using transcoder " << *sink->primaryCodec << ", data size=" << sinkStream->GetDataSize());
-  }
-  else {
-    PTRACE(4, "Patch\tCreating two stage transcoders for " << sourceFormat << "->" << destinationFormat << " with ID " << id);
-    OpalMediaFormat intermediateFormat;
-    if (!OpalTranscoder::FindIntermediateFormat(sourceFormat, destinationFormat,
-                                                intermediateFormat)) {
-      PTRACE(1, "Patch\tCould find compatible media format for " << *sinkStream);
-      return false;
-    }
+    patch.source.SetDataSize(primaryCodec->GetOptimalDataFrameSize(true), primaryCodec->GetOutputFormat().GetFrameTime());
+    patch.source.InternalUpdateMediaFormat(primaryCodec->GetInputFormat());
+    stream->InternalUpdateMediaFormat(primaryCodec->GetOutputFormat());
 
-    if (intermediateFormat.GetMediaType() == OpalMediaType::Audio()) {
-      // try prepare intermediateFormat for correct frames to frames transcoding
-      // so we need make sure that tx frames time of destinationFormat be equal 
-      // to tx frames time of intermediateFormat (all this does not produce during
-      // Merge phase in FindIntermediateFormat)
-      int destinationPacketTime = destinationFormat.GetFrameTime()*destinationFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1);
-      if ((destinationPacketTime % intermediateFormat.GetFrameTime()) != 0) {
-        PTRACE(1, "Patch\tCould produce without buffered media format converting (which not implemented yet) for " << *sinkStream);
-        return false;
-      }
-      intermediateFormat.AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::TxFramesPerPacketOption(),
-                                                               true,
-                                                               OpalMediaOption::NoMerge,
-                                                               destinationPacketTime/intermediateFormat.GetFrameTime()),
-                                   true);
-    }
-
-    sink->primaryCodec = OpalTranscoder::Create(sourceFormat, intermediateFormat, (const BYTE *)id, id.GetLength());
-    sink->secondaryCodec = OpalTranscoder::Create(intermediateFormat, destinationFormat, (const BYTE *)id, id.GetLength());
-    if (sink->primaryCodec == NULL || sink->secondaryCodec == NULL)
-      return false;
-
-    PTRACE(3, "Patch\tCreated two stage codec " << sourceFormat << "/" << intermediateFormat << "/" << destinationFormat << " with ID " << id);
-
-    sink->primaryCodec->SetMaxOutputSize(sink->secondaryCodec->GetOptimalDataFrameSize(true));
-    sink->primaryCodec->SetSessionID(source.GetSessionID());
-
-    if (!sinkStream->SetDataSize(sink->secondaryCodec->GetOptimalDataFrameSize(false), sourceFormat.GetFrameTime())) {
-      PTRACE(1, "Patch\tSink stream " << *sinkStream << " cannot support data size "
-              << sink->secondaryCodec->GetOptimalDataFrameSize(PFalse));
-      return false;
-    }
-    sink->secondaryCodec->SetMaxOutputSize(sinkStream->GetDataSize());
-    sink->secondaryCodec->SetSessionID(source.GetSessionID());
-
-    PTRACE(3, "Patch\tAdded media stream sink " << *sinkStream
-           << " using transcoders " << *sink->primaryCodec
-           << " and " << *sink->secondaryCodec << ", data size=" << sinkStream->GetDataSize());
+    PTRACE(3, "Patch\tAdded media stream sink " << *stream
+           << " using transcoder " << *primaryCodec << ", data size=" << stream->GetDataSize());
+    return true;
   }
 
-  source.SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(true), destinationFormat.GetFrameTime());
-  EnableJitterBuffer();
+  PTRACE(4, "Patch\tCreating two stage transcoders for " << sourceFormat << "->" << destinationFormat << " with ID " << id);
+  OpalMediaFormat intermediateFormat;
+  if (!OpalTranscoder::FindIntermediateFormat(sourceFormat, destinationFormat, intermediateFormat)) {
+    PTRACE(1, "Patch\tCould find compatible media format for " << *stream);
+    return false;
+  }
+
+  if (intermediateFormat.GetMediaType() == OpalMediaType::Audio()) {
+    // try prepare intermediateFormat for correct frames to frames transcoding
+    // so we need make sure that tx frames time of destinationFormat be equal 
+    // to tx frames time of intermediateFormat (all this does not produce during
+    // Merge phase in FindIntermediateFormat)
+    int destinationPacketTime = destinationFormat.GetFrameTime()*destinationFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1);
+    if ((destinationPacketTime % intermediateFormat.GetFrameTime()) != 0) {
+      PTRACE(1, "Patch\tCould produce without buffered media format converting (which not implemented yet) for " << *stream);
+      return false;
+    }
+    intermediateFormat.AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::TxFramesPerPacketOption(),
+                                                              true,
+                                                              OpalMediaOption::NoMerge,
+                                                              destinationPacketTime/intermediateFormat.GetFrameTime()),
+                                  true);
+  }
+
+  primaryCodec = OpalTranscoder::Create(sourceFormat, intermediateFormat, (const BYTE *)id, id.GetLength());
+  secondaryCodec = OpalTranscoder::Create(intermediateFormat, destinationFormat, (const BYTE *)id, id.GetLength());
+  if (primaryCodec == NULL || secondaryCodec == NULL)
+    return false;
+
+  PTRACE_CONTEXT_ID_TO(primaryCodec);
+  PTRACE_CONTEXT_ID_TO(secondaryCodec);
+  PTRACE(3, "Patch\tCreated two stage codec " << sourceFormat << "/" << intermediateFormat << "/" << destinationFormat << " with ID " << id);
+
+  primaryCodec->SetMaxOutputSize(secondaryCodec->GetOptimalDataFrameSize(true));
+  primaryCodec->SetSessionID(patch.source.GetSessionID());
+  primaryCodec->SetCommandNotifier(PCREATE_NOTIFIER_EXT(&patch, OpalMediaPatch, InternalOnMediaCommand1));
+  primaryCodec->UpdateMediaFormats(OpalMediaFormat(), secondaryCodec->GetInputFormat());
+
+  if (!stream->SetDataSize(secondaryCodec->GetOptimalDataFrameSize(false), secondaryCodec->GetInputFormat().GetFrameTime())) {
+    PTRACE(1, "Patch\tSink stream " << *stream << " cannot support data size "
+            << secondaryCodec->GetOptimalDataFrameSize(false));
+    return false;
+  }
+  secondaryCodec->SetMaxOutputSize(stream->GetDataSize());
+  secondaryCodec->SetSessionID(patch.source.GetSessionID());
+  secondaryCodec->SetCommandNotifier(PCREATE_NOTIFIER_EXT(&patch, OpalMediaPatch, InternalOnMediaCommand1));
+  secondaryCodec->UpdateMediaFormats(primaryCodec->GetInputFormat(), OpalMediaFormat());
+
+  patch.source.SetDataSize(primaryCodec->GetOptimalDataFrameSize(true), primaryCodec->GetOutputFormat().GetFrameTime());
+  patch.source.InternalUpdateMediaFormat(primaryCodec->GetInputFormat());
+  stream->InternalUpdateMediaFormat(secondaryCodec->GetOutputFormat());
+
+  PTRACE(3, "Patch\tAdded media stream sink " << *stream
+          << " using transcoders " << *primaryCodec
+          << " and " << *secondaryCodec << ", data size=" << stream->GetDataSize());
   return true;
 }
 
 
-void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
+void OpalMediaPatch::RemoveSink(const OpalMediaStream & stream)
 {
-  if (PAssertNULL(stream) == NULL)
-    return;
-
-  PTRACE(3, "Patch\tRemoving media stream sink " << *stream);
+  PTRACE(3, "Patch\tRemoving sink " << stream << " from " << *this);
 
   bool closeSource = false;
 
-  inUse.StartWrite();
+  if (!LockReadWrite())
+    return;
 
   for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
-    if (s->stream == stream) {
+    if (s->stream == &stream) {
       sinks.erase(s);
-      PTRACE(5, "Patch\tRemoved media stream sink " << *stream);
+      PTRACE(5, "Patch\tRemoved sink " << stream << " from " << *this);
       break;
     }
   }
@@ -290,7 +359,7 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
       m_bypassFromPatch->SetBypassPatch(NULL);
   }
 
-  inUse.EndWrite();
+  UnlockReadWrite();
 
   if (closeSource  && source.GetPatch() == this)
     source.Close();
@@ -299,7 +368,7 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
 
 OpalMediaStreamPtr OpalMediaPatch::GetSink(PINDEX i) const
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
   return i < sinks.GetSize() ? sinks[i].stream : OpalMediaStreamPtr();
 }
 
@@ -320,7 +389,8 @@ OpalMediaFormat OpalMediaPatch::GetSinkFormat(PINDEX i) const
 
 OpalTranscoder * OpalMediaPatch::GetAndLockSinkTranscoder(PINDEX i) const
 {
-  inUse.StartRead();
+  if (!LockReadOnly())
+    return NULL;
 
   if (i >= sinks.GetSize()) {
     UnLockSinkTranscoder();
@@ -342,14 +412,15 @@ OpalTranscoder * OpalMediaPatch::GetAndLockSinkTranscoder(PINDEX i) const
 
 void OpalMediaPatch::UnLockSinkTranscoder() const
 {
-  inUse.EndRead();
+  UnlockReadOnly();
 }
 
 
 #if OPAL_STATISTICS
 void OpalMediaPatch::GetStatistics(OpalMediaStatistics & statistics, bool fromSink) const
 {
-  inUse.StartRead();
+  if (!LockReadOnly())
+    return;
 
   if (fromSink)
     source.GetStatistics(statistics, true);
@@ -357,7 +428,7 @@ void OpalMediaPatch::GetStatistics(OpalMediaStatistics & statistics, bool fromSi
   if (!sinks.IsEmpty())
     sinks.front().GetStatistics(statistics, !fromSink);
 
-  inUse.EndRead();
+  UnlockReadOnly();
 }
 
 
@@ -387,6 +458,8 @@ OpalMediaPatch::Sink::Sink(OpalMediaPatch & p, const OpalMediaStreamPtr & s)
   , rateController(NULL)
 #endif
 {
+  PTRACE_CONTEXT_ID_FROM(p);
+
 #if OPAL_VIDEO
   SetRateControlParameters(stream->GetMediaFormat());
 #endif
@@ -407,7 +480,7 @@ OpalMediaPatch::Sink::~Sink()
 
 void OpalMediaPatch::AddFilter(const PNotifier & filter, const OpalMediaFormat & stage)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   if (source.GetMediaFormat().GetMediaType() != stage.GetMediaType())
     return;
@@ -425,7 +498,7 @@ void OpalMediaPatch::AddFilter(const PNotifier & filter, const OpalMediaFormat &
 
 PBoolean OpalMediaPatch::RemoveFilter(const PNotifier & filter, const OpalMediaFormat & stage)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   for (PList<Filter>::iterator f = filters.begin(); f != filters.end(); ++f) {
     if (f->notifier == filter && f->stage == stage) {
@@ -442,25 +515,30 @@ PBoolean OpalMediaPatch::RemoveFilter(const PNotifier & filter, const OpalMediaF
 void OpalMediaPatch::FilterFrame(RTP_DataFrame & frame,
                                  const OpalMediaFormat & mediaFormat)
 {
-  inUse.StartRead();
+  if (!LockReadOnly())
+    return;
 
   for (PList<Filter>::iterator f = filters.begin(); f != filters.end(); ++f) {
     if (f->stage.IsEmpty() || f->stage == mediaFormat)
-      f->notifier(frame, (INT)this);
+      f->notifier(frame, (P_INT_PTR)this);
   }
 
-  inUse.EndRead();
+  UnlockReadOnly();
 }
 
 
 bool OpalMediaPatch::UpdateMediaFormat(const OpalMediaFormat & mediaFormat)
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
 
-  bool atLeastOne = source.UpdateMediaFormat(mediaFormat, true);
+  bool atLeastOne = source.InternalUpdateMediaFormat(mediaFormat);
 
-  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s)
-    atLeastOne = s->UpdateMediaFormat(mediaFormat) || atLeastOne;
+  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
+    if (s->UpdateMediaFormat(mediaFormat)) {
+      source.InternalUpdateMediaFormat(s->stream->GetMediaFormat());
+      atLeastOne = true;
+    }
+  }
 
   PTRACE_IF(2, !atLeastOne, "Patch\tCould not update media format for any stream/transcoder in " << *this);
   return atLeastOne;
@@ -469,16 +547,20 @@ bool OpalMediaPatch::UpdateMediaFormat(const OpalMediaFormat & mediaFormat)
 
 PBoolean OpalMediaPatch::ExecuteCommand(const OpalMediaCommand & command, PBoolean fromSink)
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
 
-  if (fromSink)
-    return source.ExecuteCommand(command);
+  if (fromSink) {
+    OpalMediaPatch * patch = m_bypassToPatch;
+    if (patch == NULL)
+      patch = this;
+    PTRACE(5, "Patch\tExecute command \"" << command << "\" "
+           << (m_bypassToPatch != NULL ? "bypassed" : "on") << ' ' << *this);
+    return patch->source.ExecuteCommand(command);
+  }
 
   bool atLeastOne = false;
 
-  PList<Sink> & sinksToUse = m_bypassActive && m_bypassToPatch != NULL ? m_bypassToPatch->sinks : sinks;
-
-  for (PList<Sink>::iterator s = sinksToUse.begin(); s != sinksToUse.end(); ++s) {
+  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
     if (s->ExecuteCommand(command))
       atLeastOne = true;
   }
@@ -487,22 +569,23 @@ PBoolean OpalMediaPatch::ExecuteCommand(const OpalMediaCommand & command, PBoole
 }
 
 
-void OpalMediaPatch::SetCommandNotifier(const PNotifier & notifier, PBoolean fromSink)
+void OpalMediaPatch::InternalOnMediaCommand1(OpalMediaCommand & command, P_INT_PTR)
 {
-  PReadWaitAndSignal mutex(inUse);
+  source.GetConnection().GetEndPoint().GetManager().QueueDecoupledEvent(new PSafeWorkArg1<OpalMediaPatch, OpalMediaCommand *>(
+              this, command.CloneAs<OpalMediaCommand>(), &OpalMediaPatch::InternalOnMediaCommand2));
+}
 
-  if (fromSink)
-    source.SetCommandNotifier(notifier);
-  else {
-    for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s)
-      s->SetCommandNotifier(notifier);
-  }
+
+void OpalMediaPatch::InternalOnMediaCommand2(OpalMediaCommand * command)
+{
+  source.ExecuteCommand(*command);
+  delete command;
 }
 
 
 bool OpalMediaPatch::SetPaused(bool pause)
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
 
   bool atLeastOne = source.SetPaused(pause, true);
 
@@ -529,11 +612,12 @@ bool OpalMediaPatch::OnStartMediaPatch()
 }
 
 
-bool OpalMediaPatch::EnableJitterBuffer()
+bool OpalMediaPatch::EnableJitterBuffer(bool enab)
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
 
-  bool enab = m_bypassToPatch == NULL;
+  if (m_bypassToPatch != NULL)
+    enab = false;
 
   PList<Sink>::iterator s;
   for (s = sinks.begin(); s != sinks.end(); ++s) {
@@ -544,7 +628,7 @@ bool OpalMediaPatch::EnableJitterBuffer()
   }
 
   for (s = sinks.begin(); s != sinks.end(); ++s) {
-    if (s->stream->IsSynchronous() && source.EnableJitterBuffer(enab))
+    if (source.EnableJitterBuffer(enab && s->stream->IsSynchronous()))
       return false;
   }
 
@@ -589,11 +673,7 @@ void OpalMediaPatch::Main()
       break;
     }
  
-    inUse.StartRead();
-    bool written = DispatchFrame(sourceFrame);
-    inUse.EndRead();
-
-    if (!written) {
+    if (!DispatchFrame(sourceFrame)) {
       PTRACE(4, "Patch\tThread ended because all sink writes failed");
       break;
     }
@@ -636,9 +716,9 @@ void OpalMediaPatch::Main()
 }
 
 
-bool OpalMediaPatch::SetBypassPatch(OpalMediaPatch * patch)
+bool OpalMediaPatch::SetBypassPatch(const OpalMediaPatchPtr & patch)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   if (!PAssert(m_bypassFromPatch == NULL, PLogicError))
     return false; // Can't be both!
@@ -652,8 +732,11 @@ bool OpalMediaPatch::SetBypassPatch(OpalMediaPatch * patch)
     if (!PAssert(m_bypassToPatch->m_bypassFromPatch == this, PLogicError))
       return false;
 
-    m_bypassToPatch->m_bypassFromPatch = NULL;
+    m_bypassToPatch->m_bypassFromPatch.SetNULL();
     m_bypassToPatch->m_bypassEnded.Signal();
+
+    if (patch == NULL)
+      m_bypassToPatch->EnableJitterBuffer();
   }
 
   if (patch != NULL) {
@@ -666,36 +749,38 @@ bool OpalMediaPatch::SetBypassPatch(OpalMediaPatch * patch)
   m_bypassToPatch = patch;
 
 #if OPAL_VIDEO
-  if (m_videoDecoder)
+  OpalMediaFormat format = source.GetMediaFormat();
+  if (format.IsTransportable() && format.GetMediaType() == OpalMediaType::Video())
     source.ExecuteCommand(OpalVideoUpdatePicture());
-  else
-    EnableJitterBuffer();
-
-  m_bypassActive = m_bypassToPatch != NULL && !m_videoDecoder;
-#else
-  EnableJitterBuffer();
-  m_bypassActive = m_bypassToPatch != NULL;
 #endif
 
+  if (patch == NULL)
+    EnableJitterBuffer();
+  else {
+    EnableJitterBuffer(false);
+    patch->EnableJitterBuffer(false);
+  }
+
   return true;
 }
 
 
 PBoolean OpalMediaPatch::PushFrame(RTP_DataFrame & frame)
 {
-  PReadWaitAndSignal mutex(inUse);
   return DispatchFrame(frame);
 }
 
 
 bool OpalMediaPatch::DispatchFrame(RTP_DataFrame & frame)
 {
+  if (!LockReadOnly())
+    return false;
+
   if (m_bypassFromPatch != NULL) {
-    PTRACE(3, "Patch\tMedia patch bypass started by " << m_bypassFromPatch << " on " << *this);
-    inUse.EndRead();
+    PTRACE(3, "Patch\tMedia patch bypass started by " << *m_bypassFromPatch << " on " << *this);
+    UnlockReadOnly();
     m_bypassEnded.Wait();
     PTRACE(4, "Patch\tMedia patch bypass ended on " << *this);
-    inUse.StartRead();
     return true;
   }
 
@@ -703,17 +788,19 @@ bool OpalMediaPatch::DispatchFrame(RTP_DataFrame & frame)
 
   bool written = false;
 
-  // This is not an if/else as WriteFrame can change the state of m_bypassActive
-  if (!m_bypassActive) {
+  if (m_bypassToPatch == NULL) {
     for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
       if (s->WriteFrame(frame))
         written = true;
     }
+    UnlockReadOnly();
   }
+  else {
+    OpalMediaPatchPtr bypassToPatch = m_bypassToPatch;
+    UnlockReadOnly();
 
-  if (m_bypassActive) {
-    written = false;
-    for (PList<Sink>::iterator s = m_bypassToPatch->sinks.begin(); s != m_bypassToPatch->sinks.end(); ++s) {
+    PSafeLockReadOnly guard(*bypassToPatch);
+    for (PList<Sink>::iterator s = bypassToPatch->sinks.begin(); s != bypassToPatch->sinks.end(); ++s) {
       if (s->stream->WritePacket(frame))
         written = true;
     }
@@ -728,16 +815,16 @@ bool OpalMediaPatch::Sink::UpdateMediaFormat(const OpalMediaFormat & mediaFormat
   bool ok;
 
   if (primaryCodec == NULL)
-    ok = stream->UpdateMediaFormat(mediaFormat, true);
+    ok = stream->InternalUpdateMediaFormat(mediaFormat);
   else if (secondaryCodec != NULL && secondaryCodec->GetOutputFormat() == mediaFormat)
     ok = secondaryCodec->UpdateMediaFormats(OpalMediaFormat(), mediaFormat) &&
-         stream->UpdateMediaFormat(secondaryCodec->GetOutputFormat(), true);
+         stream->InternalUpdateMediaFormat(secondaryCodec->GetOutputFormat());
   else if (primaryCodec->GetOutputFormat() == mediaFormat)
     ok = primaryCodec->UpdateMediaFormats(OpalMediaFormat(), mediaFormat) &&
-         stream->UpdateMediaFormat(primaryCodec->GetOutputFormat(), true);
+         stream->InternalUpdateMediaFormat(primaryCodec->GetOutputFormat());
   else
     ok = primaryCodec->UpdateMediaFormats(mediaFormat, OpalMediaFormat()) &&
-         stream->UpdateMediaFormat(primaryCodec->GetInputFormat(), true);
+         stream->InternalUpdateMediaFormat(primaryCodec->GetInputFormat());
 
 #if OPAL_VIDEO
     SetRateControlParameters(stream->GetMediaFormat());
@@ -750,7 +837,7 @@ bool OpalMediaPatch::Sink::UpdateMediaFormat(const OpalMediaFormat & mediaFormat
 
 bool OpalMediaPatch::Sink::ExecuteCommand(const OpalMediaCommand & command)
 {
-  PBoolean atLeastOne = PFalse;
+  PBoolean atLeastOne = false;
 
   if (secondaryCodec != NULL)
     atLeastOne = secondaryCodec->ExecuteCommand(command) || atLeastOne;
@@ -762,16 +849,6 @@ bool OpalMediaPatch::Sink::ExecuteCommand(const OpalMediaCommand & command)
 }
 
 
-void OpalMediaPatch::Sink::SetCommandNotifier(const PNotifier & notifier)
-{
-  if (secondaryCodec != NULL)
-    secondaryCodec->SetCommandNotifier(notifier);
-
-  if (primaryCodec != NULL)
-    primaryCodec->SetCommandNotifier(notifier);
-}
-
-
 bool OpalMediaPatch::Sink::CannotTranscodeFrame(OpalTranscoder & codec, RTP_DataFrame & frame)
 {
   RTP_DataFrame::PayloadTypes pt = frame.GetPayloadType();
@@ -819,8 +896,6 @@ void OpalMediaPatch::Sink::SetRateControlParameters(const OpalMediaFormat & medi
   if ((mediaFormat.GetMediaType() == OpalMediaType::Video()) && mediaFormat != OpalYUV420P) {
     rateController = NULL;
     PString rc = mediaFormat.GetOptionString(OpalVideoFormat::RateControllerOption());
-    if (rc.IsEmpty() && mediaFormat.GetOptionBoolean(OpalVideoFormat::RateControlEnableOption()))
-      rc = "Standard";
     if (!rc.IsEmpty()) {
       rateController = PFactory<OpalVideoRateController>::CreateInstance(rc);
       if (rateController != NULL) {   
@@ -895,12 +970,6 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
   }
 
 #if OPAL_VIDEO
-  if (!patch.m_bypassActive && patch.m_videoDecoder && patch.m_bypassToPatch != NULL &&
-                dynamic_cast<OpalVideoTranscoder *>(primaryCodec)->WasLastFrameIFrame()) {
-    PTRACE(3, "Patch\tActivating video patch bypass to " << patch.m_bypassToPatch << " on " << patch);
-    patch.m_bypassActive = true;
-  }
-
   if (secondaryCodec == NULL && rateController != NULL) {
     PTRACE(4, "Patch\tPushing " << intermediateFrames.GetSize() << " packet into RC");
     rateController->Push(intermediateFrames, ((OpalVideoTranscoder *)primaryCodec)->WasLastFrameIFrame());
@@ -911,6 +980,8 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
         patch.FilterFrame(*interFrame, primaryCodec->GetOutputFormat());
         if (!stream->WritePacket(*interFrame))
           return (writeSuccessful = false);
+        if (primaryCodec == NULL)
+          return true;
         primaryCodec->CopyTimestamp(sourceFrame, *interFrame, false);
         continue;
       }
@@ -925,6 +996,8 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
     if (secondaryCodec == NULL) {
       if (!stream->WritePacket(*interFrame))
         return (writeSuccessful = false);
+      if (primaryCodec == NULL)
+        return true;
       primaryCodec->CopyTimestamp(sourceFrame, *interFrame, false);
       continue;
     }
@@ -944,6 +1017,8 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
       patch.FilterFrame(*finalFrame, secondaryCodec->GetOutputFormat());
       if (!stream->WritePacket(*finalFrame))
         return (writeSuccessful = false);
+      if (secondaryCodec == NULL)
+        return true;
       secondaryCodec->CopyTimestamp(sourceFrame, *finalFrame, false);
     }
   }
@@ -957,27 +1032,22 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
 }
 
 
-OpalMediaPatch::Thread::Thread(OpalMediaPatch & p)
-  : PThread(65536,  //16*4kpage size
-            NoAutoDeleteThread,
-            HighPriority,
-            "Media Patch"),
-    patch(p)
-{
-}
-
-
 /////////////////////////////////////////////////////////////////////////////
 
 
 OpalPassiveMediaPatch::OpalPassiveMediaPatch(OpalMediaStream & source)
   : OpalMediaPatch(source)
+  , m_started(false)
 {
 }
 
 
 void OpalPassiveMediaPatch::Start()
 {
+  if (m_started)
+    return;
+
+  m_started = true;
   OnStartMediaPatch();
 }
 
