@@ -23,138 +23,94 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25492 $
+ * $Revision: 29768 $
  * $Author: rjongbloed $
- * $Date: 2011-04-03 19:10:06 -0500 (Sun, 03 Apr 2011) $
+ * $Date: 2013-05-23 02:27:37 -0500 (Thu, 23 May 2013) $
  */
 
 
 #include <ptlib.h>
 #include <opal/buildopts.h>
 
+#if OPAL_HAS_PRESENCE
+
 #include <opal/pres_ent.h>
 
 #include <opal/manager.h>
+#include <opal/endpoint.h>
 #include <ptclib/url.h>
 #include <sip/sipep.h>
 
 ///////////////////////////////////////////////////////////////////////
 
+PURL_LEGACY_SCHEME(pres, true, false, true, true, false, true, true, false, false, false, 0)
+
 const PCaselessString & OpalPresentity::AuthNameKey()     { static const PConstCaselessString s("Auth ID");       return s; }
 const PCaselessString & OpalPresentity::AuthPasswordKey() { static const PConstCaselessString s("Auth Password"); return s; }
 const PCaselessString & OpalPresentity::TimeToLiveKey()   { static const PConstCaselessString s("Time to Live");  return s; }
 
 
+OpalPresenceInfo::OpalPresenceInfo(State state)
+  : m_state(state)
+{
+}
+
+
 PString OpalPresenceInfo::AsString() const
 {
   return AsString(m_state);
 }
 
-ostream & operator<<(ostream & strm, OpalPresenceInfo::State state)
-{
-  return strm << OpalPresenceInfo::AsString(state);
-}
-
-//
-//  defined in RFC 3856
-//
-static const char * const BasicNames[] = {
-  "Internal Error",
-  "Forbidden",
-  "No Presence",
-  "Unchanged",
-  "Available",
-  "Unavailable"
-};
-
-//
-// defined in RFC 4480 as "activities"
-//
-static const char * const ExtendedNames[] = {
-  "UnknownExtended",
-  "Appointment",
-  "Away",
-  "Breakfast",
-  "Busy",
-  "Dinner",
-  "Holiday",
-  "InTransit",
-  "LookingForWork",
-  "Lunch",
-  "Meal",
-  "Meeting",
-  "OnThePhone",
-  "Other",
-  "Performance",
-  "PermanentAbsence",
-  "Playing",
-  "Presentation",
-  "Shopping",
-  "Sleeping",
-  "Spectator",
-  "Steering",
-  "Travel",
-  "TV",
-  "Vacation",
-  "Working",
-  "Worship"
-};
-
 PString OpalPresenceInfo::AsString(State state)
 {
-  if (state >= OpalPresenceInfo::InternalError) {
-    PINDEX index = state - OpalPresenceInfo::InternalError;
-    if (index < PARRAYSIZE(BasicNames)) 
-      return BasicNames[index];
-  }
-
-  if (state >= OpalPresenceInfo::ExtendedBase) {
-    PINDEX index = state - OpalPresenceInfo::ExtendedBase;
-    if (index < PARRAYSIZE(ExtendedNames))
-      return ExtendedNames[index];
-  }
-
   PStringStream strm;
-  strm << "Presence<" << (unsigned)state << '>';
+  strm << state;
   return strm;
 }
 
 
 OpalPresenceInfo::State OpalPresenceInfo::FromString(const PString & stateString)
 {
-  if (stateString.IsEmpty() || (stateString *= "Unchanged"))
-    return Unchanged;
-
-  if (stateString *= "Available")
-    return Available;
-
-  if (stateString *= "Unavailable")
-    return Unavailable;
-
-  if ((stateString *= "Invisible") ||
+  if ((stateString *= "None") ||
       (stateString *= "Offline") ||
-      (stateString *= "NoPresence"))
+      (stateString *= "Invisible"))
     return NoPresence;
 
-  for (size_t k = 0; k < sizeof(ExtendedNames)/sizeof(ExtendedNames[0]); ++k) {
-    if (stateString *= ExtendedNames[k]) 
-      return (State)(ExtendedBase + k);
-  }
+  State state = InternalError;
+  PStringStream strm(stateString);
+  strm >> state;
+  return strm.fail() ? InternalError : state;
+}
 
-  return InternalError;
+
+PObject::Comparison OpalPresenceInfo::Compare(const PObject & obj) const
+{
+  const OpalPresenceInfo & other = dynamic_cast<const OpalPresenceInfo &>(obj);
+  if (m_entity < other.m_entity)
+    return LessThan;
+  if (m_entity > other.m_entity)
+    return GreaterThan;
+
+  if (m_target < other.m_target)
+    return LessThan;
+  if (m_target > other.m_target)
+    return GreaterThan;
+
+  if (m_when < other.m_when)
+    return LessThan;
+  if (m_when > other.m_when)
+    return GreaterThan;
+
+  return EqualTo;
 }
 
 
 ///////////////////////////////////////////////////////////////////////
 
-static PAtomicInteger::IntegerType g_idNumber = 1;
-
-
 OpalPresentity::OpalPresentity()
   : m_manager(NULL)
-  , m_idNumber(g_idNumber++)
   , m_temporarilyUnavailable(false)
-  , m_localState(OpalPresenceInfo::NoPresence)
+  , m_localInfo(OpalPresenceInfo::NoPresence)
 {
 }
 
@@ -163,9 +119,8 @@ OpalPresentity::OpalPresentity(const OpalPresentity & other)
   : PSafeObject(other)
   , m_manager(other.m_manager)
   , m_attributes(other.m_attributes)
-  , m_idNumber(g_idNumber++)
   , m_temporarilyUnavailable(false)
-  , m_localState(OpalPresenceInfo::NoPresence)
+  , m_localInfo(OpalPresenceInfo::NoPresence)
 {
 }
 
@@ -194,6 +149,26 @@ OpalPresentity * OpalPresentity::Create(OpalManager & manager, const PURL & url,
 }
 
 
+bool OpalPresentity::Open()
+{
+  if (m_open.TestAndSet(true))
+    return false; // Already open
+
+  PTRACE(3, "OpalPres\t'" << m_aor << "' opening.");
+  return true;
+}
+
+
+bool OpalPresentity::Close()
+{
+  if (!m_open.TestAndSet(false))
+    return false; // Aleady closed
+
+  PTRACE(3, "OpalPres\t'" << m_aor << "' closing.");
+  return true;
+}
+
+
 bool OpalPresentity::SubscribeToPresence(const PURL & presentity, bool subscribe, const PString & note)
 {
   if (!IsOpen())
@@ -233,35 +208,52 @@ bool OpalPresentity::SetPresenceAuthorisation(const PURL & presentity, Authorisa
 }
 
 
-bool OpalPresentity::SetLocalPresence(OpalPresenceInfo::State state, const PString & note)
+bool OpalPresentity::SetLocalPresence(const OpalPresenceInfo & info)
 {
   if (!IsOpen())
     return false;
 
-  m_localState     = state;
-  m_localStateNote = note;
+  m_localInfo = info;
 
   OpalSetLocalPresenceCommand * cmd = CreateCommand<OpalSetLocalPresenceCommand>();
   if (cmd == NULL)
     return false;
 
-  cmd->m_state = state;
-  cmd->m_note = note;
+  *static_cast<OpalPresenceInfo *>(cmd) = info;
   SendCommand(cmd);
   return true;
 }
 
+
+bool OpalPresentity::SetLocalPresence(OpalPresenceInfo::State state, const PString & note)
+{
+  m_localInfo.m_state = state;
+  m_localInfo.m_note = note;
+  return SetLocalPresence(m_localInfo);
+}
+
+
+bool OpalPresentity::GetLocalPresence(OpalPresenceInfo & info)
+{
+  info = m_localInfo;
+  return true;
+}
+
+
 bool OpalPresentity::GetLocalPresence(OpalPresenceInfo::State & state, PString & note)
 {
   if (!IsOpen())
     return false;
 
-  state = m_localState;
-  note  = m_localStateNote;
+  state = m_localInfo.m_state;
+  note  = m_localInfo.m_note;
 
   return true;
 }
 
+
+#if OPAL_HAS_IM
+
 bool OpalPresentity::SendMessageTo(const OpalIM & message)
 {
   if (!IsOpen())
@@ -292,6 +284,8 @@ void OpalPresentity::SetReceivedMessageNotifier(const ReceivedMessageNotifier &
   m_onReceivedMessageNotifier = notifier;
 }
 
+#endif // OPAL_HAS_IM
+
 
 void OpalPresentity::OnAuthorisationRequest(const AuthorisationRequest & request)
 {
@@ -316,7 +310,7 @@ void OpalPresentity::OnPresenceChange(const OpalPresenceInfo & info)
   PWaitAndSignal mutex(m_notificationMutex);
 
   if (!m_onPresenceChangeNotifier.IsNULL())
-    m_onPresenceChangeNotifier(*this, info);
+    m_onPresenceChangeNotifier(*this, std::auto_ptr<OpalPresenceInfo>(info.CloneAs<OpalPresenceInfo>()));
 }
 
 
@@ -498,13 +492,7 @@ OpalPresentityCommand * OpalPresentity::InternalCreateCommand(const char * cmdNa
 }
 
 
-PString OpalPresentity::GetID() const 
-{ 
-  PStringStream strm;
-  strm << "id" << (unsigned)m_idNumber;
-  return strm;
-}
-
+#if OPAL_HAS_IM
 
 void OpalPresentity::Internal_SendMessageToCommand(const OpalSendMessageToCommand & cmd)
 {
@@ -526,6 +514,8 @@ void OpalPresentity::Internal_SendMessageToCommand(const OpalSendMessageToComman
 
 OPAL_DEFINE_COMMAND(OpalSendMessageToCommand, OpalPresentity, Internal_SendMessageToCommand);
 
+#endif // OPAL_HAS_IM
+
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -638,4 +628,6 @@ void OpalPresentityWithCommandThread::ThreadMain()
   PTRACE(4, "OpalPres\tCommand thread ended");
 }
 
+#endif // OPAL_HAS_PRESENCE
+
 /////////////////////////////////////////////////////////////////////////////
