@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24673 $
+ * $Revision: 28483 $
  * $Author: rjongbloed $
- * $Date: 2010-08-22 20:02:47 -0500 (Sun, 22 Aug 2010) $
+ * $Date: 2012-10-05 03:52:19 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -81,6 +81,8 @@ OpalTranscoder::OpalTranscoder(const OpalMediaFormat & inputMediaFormat,
   , inputIsRTP(false)
   , acceptEmptyPayload(false)
   , acceptOtherPayloads(false)
+  , m_inClockRate(inputMediaFormat.GetClockRate())
+  , m_outClockRate(outputMediaFormat.GetClockRate())
 {
 }
 
@@ -88,13 +90,41 @@ OpalTranscoder::OpalTranscoder(const OpalMediaFormat & inputMediaFormat,
 bool OpalTranscoder::UpdateMediaFormats(const OpalMediaFormat & input, const OpalMediaFormat & output)
 {
   PWaitAndSignal mutex(updateMutex);
-  return inputMediaFormat.Update(input) && outputMediaFormat.Update(output);
+
+  bool ok = inputMediaFormat.Update(input);
+  ok = outputMediaFormat.Update(output) && ok; // Avoid McCarthy boolean
+
+  m_inClockRate = inputMediaFormat.GetClockRate();
+  m_outClockRate = outputMediaFormat.GetClockRate();
+
+  return ok;
 }
 
 
-PBoolean OpalTranscoder::ExecuteCommand(const OpalMediaCommand & /*command*/)
+PBoolean OpalTranscoder::ExecuteCommand(const OpalMediaCommand & command)
 {
-  return PFalse;
+  if (outputMediaFormat.IsTransportable()) {
+    const OpalMediaFlowControl * flow = dynamic_cast<const OpalMediaFlowControl *>(&command);
+    if (flow != NULL) {
+      unsigned bitsPerSecond = flow->GetMaxBitRate();
+      outputMediaFormat.SetOptionInteger(OpalMediaFormat::MaxBitRateOption(), bitsPerSecond);
+      if ((unsigned)outputMediaFormat.GetOptionInteger(OpalMediaFormat::TargetBitRateOption()) > bitsPerSecond)
+        outputMediaFormat.SetOptionInteger(OpalMediaFormat::TargetBitRateOption(), bitsPerSecond);
+      UpdateMediaFormats(OpalMediaFormat(), outputMediaFormat);
+      return true;
+    }
+  }
+
+  return false;
+}
+
+
+void OpalTranscoder::NotifyCommand(const OpalMediaCommand & command) const
+{
+  if (commandNotifier != PNotifier())
+    commandNotifier(const_cast<OpalMediaCommand &>(command), m_sessionID);
+  else
+    PTRACE(4, "Opal\tNo command notifier available for transcoder " << this);
 }
 
 
@@ -117,26 +147,35 @@ void OpalTranscoder::GetStatistics(OpalMediaStatistics & /*statistics*/) const
 #endif
 
 
+void OpalTranscoder::CopyTimestamp(RTP_DataFrame & dst, const RTP_DataFrame & src, bool inToOut) const
+{
+  unsigned timestamp = src.GetTimestamp();
+  if (m_inClockRate != m_outClockRate) {
+    if (inToOut)
+      timestamp = (unsigned)((PUInt64)timestamp*m_outClockRate/m_inClockRate);
+    else
+      timestamp = (unsigned)((PUInt64)timestamp*m_inClockRate/m_outClockRate);
+  }
+  dst.SetTimestamp(timestamp);
+}
+
+
 PBoolean OpalTranscoder::ConvertFrames(const RTP_DataFrame & input, RTP_DataFrameList & output)
 {
   PWaitAndSignal mutex(updateMutex);
 
   // make sure there is at least one output frame available
   if (output.IsEmpty())
-    output.Append(new RTP_DataFrame(0, maxOutputSize));
+    output.Append(new RTP_DataFrame((PINDEX)0, maxOutputSize));
   else {
     while (output.GetSize() > 1)
-      output.RemoveAt(1);
+      output.RemoveTail();
   }
 
+  output.front().CopyHeader(input);
+
   // set the output timestamp and marker bit
-  unsigned timestamp = input.GetTimestamp();
-  unsigned inClockRate = inputMediaFormat.GetClockRate();
-  unsigned outClockRate = outputMediaFormat.GetClockRate();
-  if (inClockRate != outClockRate)
-    timestamp = (unsigned)((PUInt64)timestamp*outClockRate/inClockRate);
-  output.front().SetTimestamp(timestamp);
-  output.front().SetMarker(input.GetMarker());
+  CopyTimestamp(output.front(), input, true);
 
   // set the output payload type directly from the output media format
   // and the input payload directly from the input media format
@@ -148,7 +187,7 @@ PBoolean OpalTranscoder::ConvertFrames(const RTP_DataFrame & input, RTP_DataFram
   if (formatPayloadType != RTP_DataFrame::MaxPayloadType && packetPayloadType != formatPayloadType && input.GetPayloadSize() > 0) {
     PTRACE(2, "Opal\tExpected payload type " << formatPayloadType << ", but received " << packetPayloadType << ". Ignoring packet");
     output.RemoveAll();
-    return PTrue;
+    return true;
   }
 
   return Convert(input, output.front());
@@ -161,12 +200,27 @@ OpalTranscoder * OpalTranscoder::Create(const OpalMediaFormat & srcFormat,
                                                        unsigned instanceLen)
 {
   OpalTranscoder * transcoder = OpalTranscoderFactory::CreateInstance(OpalTranscoderKey(srcFormat, destFormat));
-  if (transcoder != NULL) {
-    transcoder->SetInstanceID(instance, instanceLen); // Make sure this is done first
-    transcoder->UpdateMediaFormats(srcFormat, destFormat);
+  if (transcoder == NULL) {
+    PTRACE2(2, NULL, "Opal\tCould not create transcoder instance from " << srcFormat << " to " << destFormat);
+    return NULL;
   }
 
-  return transcoder;
+  transcoder->SetInstanceID(instance, instanceLen); // Make sure this is done first
+
+  /* Set the actual media formats used to create the transcoder.
+     This information got lost as OpalTranscoderKey is just two PStrings
+     and not OpalMediaFormats. Need to copy all of the actual options
+     over the top of the global ones just created. */
+  transcoder->inputMediaFormat = srcFormat;
+  transcoder->outputMediaFormat = destFormat;
+
+  // This is called to make sure options are sent to plug ins.
+  if (transcoder->UpdateMediaFormats(srcFormat, destFormat))
+    return transcoder;
+
+  PTRACE2(2, transcoder, "Opal\tError creating transcoder instance from " << srcFormat << " to " << destFormat);
+  delete transcoder;
+  return NULL;
 }
 
 
@@ -242,8 +296,8 @@ static bool MergeFormats(const OpalMediaFormatList & masterFormats,
   else {
     srcFormat = *masterFormat;
     PTRACE(5, "Opal\tInitial source format from master:\n" << setw(-1) << srcFormat
-                        << "Merging with capability\n" << setw(-1) << srcCapability);
-    if (!srcFormat.Merge(srcCapability))
+                            << "Merging with capability\n" << setw(-1) << srcCapability);
+    if (!srcFormat.Update(srcCapability)) // This includes the PayloadType, a normal Merge does not.
       return false;
   }
 
@@ -255,13 +309,9 @@ static bool MergeFormats(const OpalMediaFormatList & masterFormats,
   else {
     dstFormat = *masterFormat;
     PTRACE(5, "Opal\tInitial destination format from master:\n" << setw(-1) << dstFormat
-                             << "Merging with capability\n" << setw(-1) << dstCapability);
+                                 << "Merging with capability\n" << setw(-1) << dstCapability);
 
-    OpalMediaFormat tmpCapability = dstCapability;
-    if (!tmpCapability.Merge(dstFormat))
-      return false;
-
-    if (!dstFormat.Update(tmpCapability))
+    if (!dstFormat.Update(dstCapability)) // This includes the PayloadType, a normal Merge does not.
       return false;
   }
 
@@ -319,7 +369,7 @@ bool OpalTranscoder::SelectFormats(const OpalMediaType & mediaType,
     }
   }
 
-  return PFalse;
+  return false;
 }
 
 
@@ -346,7 +396,7 @@ bool OpalTranscoder::FindIntermediateFormat(const OpalMediaFormat & srcFormat,
     }
   }
 
-  return PFalse;
+  return false;
 }
 
 
@@ -385,9 +435,8 @@ OpalMediaFormatList OpalTranscoder::GetPossibleFormats(const OpalMediaFormatList
   // Run through the formats connection can do directly and calculate all of
   // the possible formats, including ones via a transcoder
   for (OpalMediaFormatList::const_iterator f = formats.begin(); f != formats.end(); ++f) {
-    OpalMediaFormat format = *f;
-    possibleFormats += format;
-    OpalMediaFormatList srcFormats = GetSourceFormats(format);
+    possibleFormats += *f;
+    OpalMediaFormatList srcFormats = GetSourceFormats(*f);
     for (OpalMediaFormatList::iterator s = srcFormats.begin(); s != srcFormats.end(); ++s) {
       OpalMediaFormatList dstFormats = GetDestinationFormats(*s);
       if (dstFormats.GetSize() > 0) {
@@ -435,13 +484,15 @@ void OpalFramedTranscoder::CalculateSizes()
 {
   unsigned framesPerPacket = outputMediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(),
                               inputMediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1));
+  unsigned channels = outputMediaFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption(),
+                       inputMediaFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption(), 1));
   unsigned inFrameSize = inputMediaFormat.GetFrameSize();
   unsigned outFrameSize = outputMediaFormat.GetFrameSize();
   unsigned inFrameTime = inputMediaFormat.GetFrameTime();
   unsigned outFrameTime = outputMediaFormat.GetFrameTime();
   unsigned leastCommonMultiple = inFrameTime*outFrameTime/GreatestCommonDivisor(inFrameTime, outFrameTime);
-  inputBytesPerFrame = leastCommonMultiple/inFrameTime*inFrameSize*framesPerPacket;
-  outputBytesPerFrame = leastCommonMultiple/outFrameTime*outFrameSize*framesPerPacket;
+  inputBytesPerFrame = leastCommonMultiple/inFrameTime*inFrameSize*framesPerPacket*channels;
+  outputBytesPerFrame = leastCommonMultiple/outFrameTime*outFrameSize*framesPerPacket*channels;
 
   PINDEX inMaxTimePerFrame  = inputMediaFormat.GetOptionInteger(OpalAudioFormat::MaxFramesPerPacketOption()) * 
                               inputMediaFormat.GetOptionInteger(OpalAudioFormat::FrameTimeOption());
@@ -492,7 +543,7 @@ PBoolean OpalFramedTranscoder::Convert(const RTP_DataFrame & input, RTP_DataFram
     }
 
     if (!ConvertFrame(inputPtr, inLen, outputPtr, outLen))
-      return PFalse;
+      return false;
 
     if (!outputIsRTP)
       output.SetPayloadSize(outLen);
@@ -503,7 +554,7 @@ PBoolean OpalFramedTranscoder::Convert(const RTP_DataFrame & input, RTP_DataFram
     else 
       output.SetPayloadSize(outLen - output.GetHeaderSize());
 
-    return PTrue;
+    return true;
   }
 
   const BYTE * inputPtr = input.GetPayloadPtr();
@@ -516,22 +567,21 @@ PBoolean OpalFramedTranscoder::Convert(const RTP_DataFrame & input, RTP_DataFram
 
   // set maximum output payload size
   if (!output.SetPayloadSize(maxOutputDataSize))
-    return PFalse;
+    return false;
 
   BYTE * outputPtr = output.GetPayloadPtr();
-
   PINDEX outLen = 0;
 
-  while (inputLength > 0) {
+  while (inputLength > 0 && outLen < maxOutputDataSize) {
 
-    PINDEX consumed = inputLength; // PMIN(inputBytesPerFrame, inputLength);
-    PINDEX created  = output.GetPayloadSize() - outLen;
+    PINDEX consumed = inputLength;
+    PINDEX created  = maxOutputDataSize - outLen;
 
     if (!ConvertFrame(inputPtr, consumed, outputPtr, created))
-      return PFalse;
+      return false;
 
-    // If did not consume any data, codec must think it is rubbish, skip rest of packet.
-    if (consumed == 0)
+    // If did not consume or produce any data, codec has gone wrong, abort!
+    if (consumed == 0 && created == 0)
       break;
 
     outputPtr   += created;
@@ -543,7 +593,7 @@ PBoolean OpalFramedTranscoder::Convert(const RTP_DataFrame & input, RTP_DataFram
   // set actual output payload size
   output.SetPayloadSize(outLen);
 
-  return PTrue;
+  return true;
 }
 
 PBoolean OpalFramedTranscoder::ConvertFrame(const BYTE * inputPtr, PINDEX & /*consumed*/, BYTE * outputPtr, PINDEX & /*created*/)
@@ -553,13 +603,13 @@ PBoolean OpalFramedTranscoder::ConvertFrame(const BYTE * inputPtr, PINDEX & /*co
 
 PBoolean OpalFramedTranscoder::ConvertFrame(const BYTE * /*inputPtr*/, BYTE * /*outputPtr*/)
 {
-  return PFalse;
+  return false;
 }
 
 PBoolean OpalFramedTranscoder::ConvertSilentFrame(BYTE *dst)
 {
   memset(dst, 0, outputBytesPerFrame);
-  return PTrue;
+  return true;
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -578,8 +628,10 @@ OpalStreamedTranscoder::OpalStreamedTranscoder(const OpalMediaFormat & inputMedi
 PINDEX OpalStreamedTranscoder::GetOptimalDataFrameSize(PBoolean input) const
 {
   // For streamed codecs a "frame" is one milliseconds worth of data
-  PINDEX size = outputMediaFormat.GetOptionInteger(input ? OpalAudioFormat::TxFramesPerPacketOption()
-                                                         : OpalAudioFormat::RxFramesPerPacketOption(), 1);
+  PString framesPerPacketOption = input ? OpalAudioFormat::TxFramesPerPacketOption()
+                                        : OpalAudioFormat::RxFramesPerPacketOption();
+  PINDEX size = outputMediaFormat.GetOptionInteger(framesPerPacketOption,
+                 inputMediaFormat.GetOptionInteger(framesPerPacketOption, 1));
 
   size *= outputMediaFormat.GetClockRate()/1000;            // Convert to milliseconds
   size *= input ? inputBitsPerSample : outputBitsPerSample; // Total bits
@@ -650,7 +702,7 @@ PBoolean OpalStreamedTranscoder::Convert(const RTP_DataFrame & input,
 
         default :
           PAssertAlways("Unsupported bit size");
-          return PFalse;
+          return false;
       }
       break;
 
@@ -695,7 +747,7 @@ PBoolean OpalStreamedTranscoder::Convert(const RTP_DataFrame & input,
 
         default :
           PAssertAlways("Unsupported bit size");
-          return PFalse;
+          return false;
       }
       break;
 
@@ -722,7 +774,7 @@ PBoolean OpalStreamedTranscoder::Convert(const RTP_DataFrame & input,
 
         default :
           PAssertAlways("Unsupported bit size");
-          return PFalse;
+          return false;
       }
       break;
 
@@ -758,16 +810,16 @@ PBoolean OpalStreamedTranscoder::Convert(const RTP_DataFrame & input,
 
         default :
           PAssertAlways("Unsupported bit size");
-          return PFalse;
+          return false;
       }
       break;
 
     default :
       PAssertAlways("Unsupported bit size");
-      return PFalse;
+      return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
