@@ -8,17 +8,155 @@
 
 #if OPAL_ZRTP
 
-#define BUILD_ZRTP_MUTEXES 1
+  #define BUILD_ZRTP_MUTEXES 1
 
-#ifdef _WIN32
-#define ZRTP_PLATFORM ZP_WIN32
+  #ifdef _WIN32
+  #define ZRTP_PLATFORM ZP_WIN32
+  #endif
+
+  #ifdef P_LINUX
+  #define ZRTP_PLATFORM ZP_LINUX
+  #endif
+
+  #include "rtp/zrtpudp.h"
+
+  extern "C" {
+  #include <zrtp.h>
+  };
+
+class OpalLibZRTPConnectionInfo : public OpalZRTPConnectionInfo {
+  public:
+    OpalLibZRTPConnectionInfo();
+
+    bool Open();
+    virtual OpalMediaSession * CreateMediaSession(OpalConnection & conn, unsigned sessionId);
+
+    static PMutex globalMutex;
+    static bool globalInited;
+    static zrtp_global_ctx_t zrtpGlobal;
+
+    zrtp_conn_ctx_t zrtpConnContext;
+};
+
+class OpalLibZRTPStreamInfo : public OpalZRTPStreamInfo {
+  public:
+    virtual bool Open();
+};
+
+
+PMutex OpalLibZRTPConnectionInfo::globalMutex;
+bool OpalLibZRTPConnectionInfo::globalInited = false;
+zrtp_global_ctx_t OpalLibZRTPConnectionInfo::zrtpGlobal;
+
+//////////////////////////////////////////////////////////////////////
+
+class OpalZrtp_UDP : public SecureRTP_UDP
+{
+  PCLASSINFO(OpalZrtp_UDP, SecureRTP_UDP);
+  public:
+    OpalZrtp_UDP(
+      const PString & encoding,       ///<  identifies initial RTP encoding (RTP/AVP, UDPTL etc)
+      bool audio,                     ///<  is audio RTP data
+      unsigned id,                    ///<  Session ID for RTP channel
+      PBoolean remoteIsNAT            ///<  TRUE is remote is behind NAT
+    );
+
+    virtual ~OpalZrtp_UDP();
+
+    virtual PBoolean WriteZrtpData(RTP_DataFrame & frame);
+
+    virtual SendReceiveStatus OnSendData(RTP_DataFrame & frame);
+    virtual SendReceiveStatus OnReceiveData(RTP_DataFrame & frame);
+    virtual SendReceiveStatus OnSendControl(RTP_ControlFrame & frame, PINDEX & len);
+    virtual SendReceiveStatus OnReceiveControl(RTP_ControlFrame & frame);
+    virtual DWORD GetOutgoingSSRC();
+
+  protected:
+    zrtp_conn_ctx_t	zrtpSession;
+};
+
+
+class OpalZrtpSecurityMode : public OpalSecurityMode
+{
+  PCLASSINFO(OpalZrtpSecurityMode, OpalSecurityMode);
+};
+
+class LibZrtpSecurityMode_Base : public OpalZrtpSecurityMode
+{
+    PCLASSINFO(LibZrtpSecurityMode_Base, OpalZrtpSecurityMode);
+  public:
+    LibZrtpSecurityMode_Base();
+    ~LibZrtpSecurityMode_Base();
+
+    OpalMediaSession * CreateMediaSession(
+      OpalRTPConnection & connection,     ///< Connection creating session (may be needed by secure connections)
+      const OpalRTPSession::Params & options ///< Parameters to construct with session.
+    );
+
+    PBoolean Open();
+
+    zrtp_profile_t * GetZrtpProfile();
+
+    zrtp_conn_ctx_t	* zrtpSession;
+
+  protected:
+    // last element of each array mush be 0
+    void Init(int *sas, int *pk, int *auth, int *cipher, int *hash);
+    zrtp_profile_t *profile;
+};
+
+//////////////////////////////////////////////////////////////////////
+
+OpalLibZRTPConnectionInfo * OpalLibZRTPConnectionInfo_Create()
+{
+  OpalLibZRTPConnectionInfo * zrtp = new OpalLibZRTPConnectionInfo();
+  if (zrtp == NULL || !zrtp->Open()) {
+    delete zrtp;
+    return NULL;
+  }
+
+  return zrtp;
+}
+
+OpalLibZRTPConnectionInfo::OpalLibZRTPConnectionInfo()
+{
+}
+
+bool OpalLibZRTPConnectionInfo::Open()
+{
+  // do the global init
+  {
+    PWaitAndSignal m(globalMutex);
+    if (!globalInited) {
+      if (zrtp_init(&zrtpGlobal, 
+                    "Opal", 
+#ifdef _DEBUG
+                    ZRTP_LICENSE_MODE_UNLIMITED
+#else
+                    ZRTP_LICENSE_MODE_PASSIVE
 #endif
+      ) != zrtp_status_ok)
+        return false;
 
-#ifdef P_LINUX
-#define ZRTP_PLATFORM ZP_LINUX
-#endif
+      globalInited = true;
+    }
 
-#include "rtp/zrtpudp.h"
+    // do the per-session init
+    if (zrtp_init_session_ctx(&zrtpConnContext, &zrtpGlobal, NULL, 0) != zrtp_status_ok) 
+      return false;
+  }
+
+  return true;
+}
+
+OpalMediaSession * OpalLibZRTPConnectionInfo::CreateMediaSession(OpalConnection & conn, unsigned sessionId)
+{
+  return new OpalZrtp_UDP(
+  //OpalZRTPStreamInfo * zrtpStreamInfo = zrtpConnInfo->CreateStream();
+  //if (zrtpStreamInfo != NULL) {
+
+  return NULL;
+}
 
 
 //////////////////////////////////////////////////////////////////////
@@ -27,6 +165,7 @@ OpalZrtp_UDP::OpalZrtp_UDP(const Params & params)
   : SecureRTP_UDP(params)
   , zrtpStream(NULL)
 {
+  zrtp_conn_ctx_t	* zrtpSession;
 }
  
 
@@ -39,27 +178,26 @@ OpalZrtp_UDP::~OpalZrtp_UDP() {
 PBoolean OpalZrtp_UDP::WriteZrtpData(RTP_DataFrame & frame) {
 	if (shutdownWrite) {
 		shutdownWrite = FALSE;
-		return PFalse;
+		return false;
 	}
 
 	// Trying to send a PDU before we are set up!
 	if (!remoteAddress.IsValid() || remoteDataPort == 0) {
-		return PFalse; //libzrtp has to wait
+		return false; //libzrtp has to wait
 	}
 
 	while (!dataSocket->WriteTo(frame.GetPointer(), 
 								frame.GetHeaderSize()+frame.GetPayloadSize(),
 								remoteAddress, remoteDataPort))
     {
-		switch (dataSocket->GetErrorNumber()) {
-			case ECONNRESET :
-			case ECONNREFUSED :
+		switch (dataSocket->GetErrorCode(LastWriteError)) {
+			case PChannel::Unavailable :
     			break;
 			default:
-				return PFalse;
+				return false;
 		}
 	}
-	return PTrue;
+	return true;
 }
  
 
@@ -75,7 +213,7 @@ RTP_UDP::SendReceiveStatus OpalZrtp_UDP::OnSendData(RTP_DataFrame & frame) {
  	zrtp_status_t err = ::zrtp_process_rtp(zrtpStream, (char *)frame.GetPointer(), &len);
    
 	if (err != zrtp_status_ok) {
-		return RTP_Session::e_IgnorePacket;
+		return OpalRTPSession::e_IgnorePacket;
 	}
  
 	frame.SetPayloadSize(len - frame.GetHeaderSize());
@@ -87,7 +225,7 @@ RTP_UDP::SendReceiveStatus OpalZrtp_UDP::OnReceiveData(RTP_DataFrame & frame) {
 	zrtp_status_t err = ::zrtp_process_srtp(zrtpStream, (char *)frame.GetPointer(), &len);
  
 	if (err != zrtp_status_ok) {
-		return RTP_Session::e_IgnorePacket;
+		return OpalRTPSession::e_IgnorePacket;
 	}
 	
 	frame.SetPayloadSize(len - frame.GetHeaderSize());
@@ -106,7 +244,7 @@ RTP_UDP::SendReceiveStatus OpalZrtp_UDP::OnSendControl(RTP_ControlFrame & frame,
 
 	zrtp_status_t err = ::zrtp_process_srtcp(zrtpStream, (char *)frame.GetPointer(), &len);
 	if (err != zrtp_status_ok) {
-		return RTP_Session::e_IgnorePacket;
+		return OpalRTPSession::e_IgnorePacket;
 	}
 	
     transmittedLen = len;
@@ -118,7 +256,7 @@ RTP_UDP::SendReceiveStatus OpalZrtp_UDP::OnReceiveControl(RTP_ControlFrame & fra
 	unsigned len = frame.GetSize();
 	zrtp_status_t err = ::zrtp_process_rtcp(zrtpStream, (char *)frame.GetPointer(), &len);
 	if (err != zrtp_status_ok) {
-		return RTP_Session::e_IgnorePacket;
+		return OpalRTPSession::e_IgnorePacket;
 	}
 	
 	frame.SetSize(len);
@@ -147,7 +285,7 @@ class OpalZrtpSecurityMode_##name : public LibZrtpSecurityMode_Base \
 		if (profile) delete profile; \
 	} \
 }; \
-static PFactory<OpalSecurityMode>::Worker<OpalZrtpSecurityMode_##name> factoryZrtpSecurityMode_##name("ZRTP|" #name);
+PFACTORY_CREATE(PFactory<OpalSecurityMode>, OpalZrtpSecurityMode_##name, "ZRTP|" #name, false);
 
 DECLARE_LIBZRTP_CRYPTO_ALG( DEFAULT, NULL, NULL, NULL, NULL, NULL);
 #if 0
@@ -214,8 +352,8 @@ void LibZrtpSecurityMode_Base::Init(int *sas, int *pk, int *auth, int *cipher, i
 	printf("init zrtp security mode\n");
 }
  
-RTP_UDP * LibZrtpSecurityMode_Base::CreateRTPSession(OpalRTPConnection & connection,
-                                                     const RTP_Session::Params & options)
+OpalMediaSession * LibZrtpSecurityMode_Base::CreateMediaSession(OpalRTPConnection & connection,
+                                                     const OpalRTPSession::Params & options)
 {
   OpalZrtp_UDP * session = new OpalZrtp_UDP(options);
   session->SetSecurityMode(this);
@@ -234,8 +372,7 @@ RTP_UDP * LibZrtpSecurityMode_Base::CreateRTPSession(OpalRTPConnection & connect
  
 PBoolean LibZrtpSecurityMode_Base::Open() 
 {
-	return PTrue;
+	return true;
 }
 
-
 #endif
