@@ -49,7 +49,7 @@
 #include <ptlib.h>
 #include <opal/buildopts.h>
 
-#if P_EXPAT && OPAL_SIP
+#if OPAL_SIP_PRESENCE
 
 #include <sip/sippres.h>
 #include <ptclib/pdns.h>
@@ -60,11 +60,10 @@
 PFACTORY_CREATE(PFactory<OpalPresentity>, SIP_Presentity, "sip", false);
 static bool Synonym_for_pres_URL = PFactory<OpalPresentity>::RegisterAs("pres", "sip");
 
-PURL_LEGACY_SCHEME(pres, true, false, true, true, false, true, true, false, false, false, 0)
-
 const PCaselessString & SIP_Presentity::PIDFEntityKey()    { static const PConstCaselessString s("PIDF-Entity");    return s; }
 const PCaselessString & SIP_Presentity::SubProtocolKey()   { static const PConstCaselessString s("Sub-Protocol");   return s; }
 const PCaselessString & SIP_Presentity::PresenceAgentKey() { static const PConstCaselessString s("Presence Agent"); return s; }
+const PCaselessString & SIP_Presentity::TransportKey()     { static const PConstCaselessString s("Transport");      return s; }
 const PCaselessString & SIP_Presentity::XcapRootKey()      { static const PConstCaselessString s("XCAP Root");      return s; }
 const PCaselessString & SIP_Presentity::XcapAuthIdKey()    { static const PConstCaselessString s("XCAP Auth ID");   return s; }
 const PCaselessString & SIP_Presentity::XcapPasswordKey()  { static const PConstCaselessString s("XCAP Password");  return s; }
@@ -72,6 +71,25 @@ const PCaselessString & SIP_Presentity::XcapAuthAuidKey()  { static const PConst
 const PCaselessString & SIP_Presentity::XcapAuthFileKey()  { static const PConstCaselessString s("XCAP AuthFile");  return s; }
 const PCaselessString & SIP_Presentity::XcapBuddyListKey() { static const PConstCaselessString s("XCAP BuddyList"); return s; }
 
+static struct SIPAttributeInfo {
+  const char * m_name;
+  const char * m_type;
+} const AttributeInfo[] = {
+  /*  0 */ { SIP_Presentity::PIDFEntityKey(),    "URL" },
+  /*  1 */ { SIP_Presentity::SubProtocolKey(),   "Enum\nPeerToPeer,Agent,XCAP,OMA\nAgent" },
+  /*  2 */ { SIP_Presentity::PresenceAgentKey(), "String" },
+  /*  3 */ { SIP_Presentity::TransportKey(),     "Enum\nUDP,TCP,TLS\nTCP" },
+  /*  4 */ { OpalPresentity::AuthNameKey(),      "String" },
+  /*  5 */ { OpalPresentity::AuthPasswordKey(),  "Password" },
+  /*  6 */ { SIP_Presentity::XcapRootKey(),      "URL" },
+  /*  7 */ { SIP_Presentity::XcapAuthIdKey(),    "String" },
+  /*  8 */ { SIP_Presentity::XcapPasswordKey(),  "Password" },
+  /*  9 */ { SIP_Presentity::XcapAuthAuidKey(),  "String" },
+  /* 10 */ { SIP_Presentity::XcapAuthFileKey(),  "String" },
+  /* 11 */ { SIP_Presentity::XcapBuddyListKey(), "String" },
+  /* 12 */ { OpalPresentity::TimeToLiveKey(),    "Integer\n1,999999999\n300" }
+};
+
 OPAL_DEFINE_COMMAND(OpalSetLocalPresenceCommand,     SIP_Presentity, Internal_SendLocalPresence);
 OPAL_DEFINE_COMMAND(OpalSubscribeToPresenceCommand,  SIP_Presentity, Internal_SubscribeToPresence);
 OPAL_DEFINE_COMMAND(SIPWatcherInfoCommand,           SIP_Presentity, Internal_SubscribeToWatcherInfo);
@@ -87,13 +105,14 @@ SIP_Presentity::SIP_Presentity()
   , m_subProtocol(e_OMA)
   , m_watcherInfoVersion(-1)
 {
-  m_attributes.Set(SubProtocolKey, "OMA");
+  m_attributes.Set(SubProtocolKey, "Agent");
 }
 
 
 SIP_Presentity::SIP_Presentity(const SIP_Presentity & other)
   : OpalPresentityWithCommandThread(other)
-  , m_endpoint(other.m_endpoint)
+  , PValidatedNotifierTarget(other)
+  , m_endpoint(NULL)
   , m_watcherInfoVersion(-1)
 {
 }
@@ -108,47 +127,28 @@ SIP_Presentity::~SIP_Presentity()
 PStringArray SIP_Presentity::GetAttributeNames() const
 {
   PStringArray names;
-  names.AppendString(PIDFEntityKey());
-  names.AppendString(SubProtocolKey());
-  names.AppendString(PresenceAgentKey());
-  names.AppendString(AuthNameKey());
-  names.AppendString(AuthPasswordKey());
-  names.AppendString(XcapRootKey());
-  names.AppendString(XcapAuthIdKey());
-  names.AppendString(XcapPasswordKey());
-  names.AppendString(XcapAuthAuidKey());
-  names.AppendString(XcapAuthFileKey());
-  names.AppendString(XcapBuddyListKey());
-  names.AppendString(TimeToLiveKey());
+  for (PINDEX i = 0; i < PARRAYSIZE(AttributeInfo); ++i)
+    names += AttributeInfo[i].m_name;
   return names;
 }
 
 
 PStringArray SIP_Presentity::GetAttributeTypes() const
 {
-  PStringArray names;
-  names.AppendString("URL");
-  names.AppendString("Enum\nPeerToPeer,Agent,XCAP,OMA\nAgent");
-  names.AppendString("String");
-  names.AppendString("String");
-  names.AppendString("Password");
-  names.AppendString("URL");
-  names.AppendString("String");
-  names.AppendString("Password");
-  names.AppendString("String");
-  names.AppendString("String");
-  names.AppendString("String");
-  names.AppendString("Integer\n1,999999999\n300");
-  return names;
+  PStringArray types;
+  for (PINDEX i = 0; i < PARRAYSIZE(AttributeInfo); ++i)
+    types += AttributeInfo[i].m_type;
+  return types;
 }
 
 
 bool SIP_Presentity::Open()
 {
-  Close();
+  if (!OpalPresentityWithCommandThread::Open())
+    return false;
 
   // find the endpoint
-  m_endpoint = dynamic_cast<SIPEndPoint *>(m_manager->FindEndPoint("sip"));
+  m_endpoint = m_manager->FindEndPointAs<SIPEndPoint>("sip");
   if (m_endpoint == NULL) {
     PTRACE(1, "SIPPres\tCannot open SIP_Presentity without sip endpoint");
     return false;
@@ -169,40 +169,41 @@ bool SIP_Presentity::Open()
   }
 
   if (m_subProtocol == e_PeerToPeer) {
-    m_presenceAgent.SetPort(0); // Make invalid
+    m_presenceAgentURL = PString::Empty();
     PTRACE(3, "SIPPres\tUsing peer to peer mode for " << m_aor);
   }
   else {
     // find presence server for Presentity as per RFC 3861
     // if not found, look for default presence server setting
     // if none, use hostname portion of domain name
-    PString agent = m_attributes.Get(PresenceAgentKey);
-    if (agent.IsEmpty()) {
-      PString hostname = m_aor.GetHostName();
-      WORD port = m_aor.GetPort();
-      if (port == 0)
-        port = 5060;
+    PCaselessString presenceAgent = m_attributes.Get(PresenceAgentKey);
+    if (presenceAgent.IsEmpty()) {
+      presenceAgent = m_aor.GetHostPort();
 
-#if P_DNS
-      PIPSocketAddressAndPortVector addrs;
-      bool found = PDNS::LookupSRV(hostname, "_pres._sip", port, addrs) && addrs.size() > 0;
-      PTRACE(2, "SIPPres\tSRV lookup for '_pres._sip." << hostname << "' " << (found ? "succeeded" : "failed"));
-      if (found)
-        m_presenceAgent = addrs[0];
-      else
-#endif
-      if (!m_presenceAgent.Parse(hostname, port)) {
-        PTRACE(3, "SIPPres\tCould not determine Presence Server from aor \"" << m_aor << '"');
-        return false;
+#if OPAL_PTLIB_DNS_RESOLVER
+      if (m_aor.GetScheme() == "pres") {
+        PStringList hosts;
+        bool found = PDNS::LookupSRV(m_aor.GetHostName(), "_pres._sip", hosts) && !hosts.IsEmpty();
+        PTRACE(2, "SIPPres\tSRV lookup for '_pres._sip." << m_aor.GetHostName()
+               << "' " << (found ? "succeeded" : "failed"));
+        if (found)
+          presenceAgent = hosts.front();
       }
+#endif // OPAL_PTLIB_DNS_RESOLVER
     }
-    else {
-      if (!m_presenceAgent.Parse(agent)) {
-        PTRACE(3, "SIPPres\tCould not parse Presence Server \"" << agent << '"');
-        return false;
-      }
+
+    m_presenceAgentURL.Parse(presenceAgent);
+    m_presenceAgentURL.SetParamVar("transport", m_attributes.Get(TransportKey, "tcp").ToLower());
+
+    m_defaultRootURL.Parse(m_attributes.Get(XcapRootKey), "http");
+    if (m_defaultRootURL.IsEmpty()) {
+      presenceAgent.Splice("http://", 0, presenceAgent.NumCompare("sip:") == EqualTo ? 4 : 0);
+      presenceAgent += '/';
+      m_defaultRootURL.Parse(presenceAgent);
+      m_defaultRootURL.SetPort(0);
     }
-    PTRACE(3, "SIPPres\tUsing " << m_presenceAgent << " as presence server for " << m_aor);
+
+    PTRACE(3, "SIPPres\tPresentity " << m_aor << " using agent " << m_presenceAgentURL << " and XCAP root " << m_defaultRootURL);
   }
 
   m_watcherSubscriptionAOR.MakeEmpty();
@@ -217,15 +218,9 @@ bool SIP_Presentity::Open()
 }
 
 
-bool SIP_Presentity::IsOpen() const
-{
-  return m_endpoint != NULL;
-}
-
-
 bool SIP_Presentity::Close()
 {
-  if (!IsOpen())
+  if (!OpalPresentityWithCommandThread::Close())
     return false;
 
   StopThread();
@@ -238,22 +233,40 @@ bool SIP_Presentity::Close()
 
   m_notificationMutex.Wait();
 
-  if (!m_watcherSubscriptionAOR.IsEmpty()) {
+  PString watcherSubscriptionAOR = m_watcherSubscriptionAOR;
+  m_watcherSubscriptionAOR.MakeEmpty();
+
+  StringMap presenceIdByAor = m_presenceIdByAor;
+  m_watcherAorById.clear();
+  m_presenceIdByAor.clear();
+  m_presenceAorById.clear();
+  m_authorisationIdByAor.clear();
+
+  m_notificationMutex.Signal();
+
+  if (!watcherSubscriptionAOR.IsEmpty()) {
     PTRACE(3, "SIPPres\t'" << m_aor << "' sending final unsubscribe for own presence watcher");
-    m_endpoint->Unsubscribe(SIPSubscribe::Presence | SIPSubscribe::Watcher, m_watcherSubscriptionAOR, true);
+    m_endpoint->Unsubscribe(SIPSubscribe::Presence | SIPSubscribe::Watcher, watcherSubscriptionAOR, true);
   }
 
-  for (StringMap::iterator subs = m_presenceIdByAor.begin(); subs != m_presenceIdByAor.end(); ++subs) {
+  for (StringMap::iterator subs = presenceIdByAor.begin(); subs != presenceIdByAor.end(); ++subs) {
     PTRACE(3, "SIPPres\t'" << m_aor << "' sending final unsubscribe to " << subs->first);
     m_endpoint->Unsubscribe(SIPSubscribe::Presence, subs->second, true);
   }
 
-  m_notificationMutex.Signal();
-
   if (!m_publishedTupleId.IsEmpty() && m_subProtocol != e_PeerToPeer)
     m_endpoint->Publish(m_aor.AsString(), PString::Empty(), 0);
 
+  PTRACE(4, "SIPPres\t'" << m_aor << "' awaiting unsubscriptions to complete.");
+  while (m_endpoint->IsSubscribed(SIPSubscribe::Presence | SIPSubscribe::Watcher, watcherSubscriptionAOR, true))
+    PThread::Sleep(100);
+  for (StringMap::iterator subs = presenceIdByAor.begin(); subs != presenceIdByAor.end(); ++subs) {
+    while (m_endpoint->IsSubscribed(SIPSubscribe::Presence, subs->second, true))
+      PThread::Sleep(100);
+  }
+
   m_endpoint = NULL;
+  PTRACE(3, "SIPPres\t'" << m_aor << "' closed.");
   return true;
 }
 
@@ -273,16 +286,15 @@ void SIP_Presentity::Internal_SubscribeToPresence(const OpalSubscribeToPresenceC
 
     param.m_localAddress    = m_aor.AsString();
     param.m_addressOfRecord = cmd.m_presentity;
-    if (m_subProtocol >= e_XCAP)
-      param.m_remoteAddress = m_presenceAgent.AsString()+";transport=tcp";
+    param.m_remoteAddress   = m_presenceAgentURL;
     param.m_authID          = m_attributes.Get(OpalPresentity::AuthNameKey, m_aor.GetUserName());
     param.m_password        = m_attributes.Get(OpalPresentity::AuthPasswordKey);
     param.m_expire          = GetExpiryTime();
     param.m_contentType     = "application/pidf+xml";
     param.m_eventList       = true;
 
-    param.m_onSubcribeStatus = PCREATE_NOTIFIER2(OnPresenceSubscriptionStatus, const SIPSubscribe::SubscriptionStatus &);
-    param.m_onNotify         = PCREATE_NOTIFIER2(OnPresenceNotify, SIPSubscribe::NotifyCallbackInfo &);
+    param.m_onSubcribeStatus = PCREATE_NOTIFIER(OnPresenceSubscriptionStatus);
+    param.m_onNotify         = PCREATE_NOTIFIER(OnPresenceNotify);
 
     PString id;
     if (m_endpoint->Subscribe(param, id, false)) {
@@ -305,42 +317,46 @@ void SIP_Presentity::Internal_SubscribeToPresence(const OpalSubscribeToPresenceC
 
 void SIP_Presentity::OnPresenceSubscriptionStatus(SIPSubscribeHandler &, const SIPSubscribe::SubscriptionStatus & status)
 {
-  if (status.m_reason == SIP_PDU::Information_Trying)
+  if (status.m_reason/100 == 1)
     return;
 
   m_notificationMutex.Wait();
-  if (!status.m_wasSubscribing || status.m_reason >= 400) {
-    PString id = status.m_handler->GetCallID();
-    StringMap::iterator aor = m_presenceAorById.find(id);
-    if (aor != m_presenceAorById.end()) {
-      PTRACE(status.m_reason >= 400 ? 2 : 3, "SIPPres\t'" << m_aor << "' "
-             << (status.m_wasSubscribing ? "error " : "un")
-             << "subscribing to presence of '" << aor->second << '\'');
-      m_endpoint->Unsubscribe(SIPSubscribe::Presence, status.m_addressofRecord, true);
+
+  PString id = status.m_handler->GetCallID();
+  StringMap::iterator aor = m_presenceAorById.find(id);
+  if (aor == m_presenceAorById.end()) {
+    PTRACE(2, "SIPPres\t'" << m_aor << "' " << "recieved subscription status for unknown ID " << id);
+  }
+  else {
+    SIPPresenceInfo info(status.m_reason, status.m_wasSubscribing);
+    SetPIDFEntity(info.m_entity);
+    info.m_target = aor->second;
+
+    OnPresenceChange(info);
+
+    if (!status.m_wasSubscribing) {
       m_presenceIdByAor.erase(aor->second);
       m_presenceAorById.erase(aor);
     }
   }
+
   m_notificationMutex.Signal();
 }
 
 
-void SIP_Presentity::OnPresenceNotify(SIPSubscribeHandler &, SIPSubscribe::NotifyCallbackInfo & status)
+void SIP_Presentity::OnPresenceNotify(SIPSubscribeHandler &, SIPSubscribe::NotifyCallbackInfo & notifyInfo)
 {
   list<SIPPresenceInfo> infoList;
-  PString error;
-  if (!SIPPresenceInfo::ParseXML(status.m_notify.GetEntityBody(), infoList, error)) {
-    status.m_response.SetEntityBody(error);
+  if (!SIPPresenceInfo::ParseNotify(notifyInfo, infoList))
     return;
-  }
 
   // send 200 OK now, and flag caller NOT to send the response
-  status.SendResponse(SIP_PDU::Successful_OK);
+  notifyInfo.SendResponse();
 
   m_notificationMutex.Wait();
   for (list<SIPPresenceInfo>::iterator it = infoList.begin(); it != infoList.end(); ++it) {
     SetPIDFEntity(it->m_target);
-    PTRACE(3, "SIPPres\t'" << it->m_entity << "' request for presence of '" << m_aor << "' is " << it->m_state);
+    PTRACE(3, "SIPPres\t'" << m_aor << "' request for presence of '" << it->m_entity << "' is " << it->m_state);
     OnPresenceChange(*it);
   }
   m_notificationMutex.Signal();
@@ -380,12 +396,12 @@ void SIP_Presentity::Internal_SubscribeToWatcherInfo(const SIPWatcherInfoCommand
   param.m_contentType      = "application/watcherinfo+xml";
   param.m_localAddress     = aorStr;
   param.m_addressOfRecord  = aorStr;
-  //param.m_remoteAddress    = m_presenceAgent.AsString() + ";transport=tcp";
+  param.m_remoteAddress    = m_presenceAgentURL;
   param.m_authID           = m_attributes.Get(OpalPresentity::AuthNameKey, m_aor.GetUserName());
   param.m_password         = m_attributes.Get(OpalPresentity::AuthPasswordKey);
   param.m_expire           = GetExpiryTime();
-  param.m_onSubcribeStatus = PCREATE_NOTIFIER2(OnWatcherInfoSubscriptionStatus, const SIPSubscribe::SubscriptionStatus &);
-  param.m_onNotify         = PCREATE_NOTIFIER2(OnWatcherInfoNotify, SIPSubscribe::NotifyCallbackInfo &);
+  param.m_onSubcribeStatus = PCREATE_NOTIFIER(OnWatcherInfoSubscriptionStatus);
+  param.m_onNotify         = PCREATE_NOTIFIER(OnWatcherInfoNotify);
 
   m_endpoint->Subscribe(param, m_watcherSubscriptionAOR);
 }
@@ -415,40 +431,26 @@ void SIP_Presentity::SetPIDFEntity(PURL & entity)
 
 void SIP_Presentity::OnWatcherInfoSubscriptionStatus(SIPSubscribeHandler &, const SIPSubscribe::SubscriptionStatus & status)
 {
-   if (status.m_reason == SIP_PDU::Information_Trying)
+  if (status.m_reason/100 == 1)
     return;
 
-  OpalPresenceInfo info(status.m_wasSubscribing ? OpalPresenceInfo::Unchanged : OpalPresenceInfo::NoPresence);
+  SIPPresenceInfo info(status.m_reason, status.m_wasSubscribing);
   SetPIDFEntity(info.m_entity);
   info.m_target = info.m_entity;
 
   m_notificationMutex.Wait();
 
-  if (status.m_reason/100 == 4)
-    info.m_state = OpalPresenceInfo::Forbidden;
-  else if (status.m_reason/100 != 2) 
-    info.m_state = OpalPresenceInfo::InternalError;
-
   OnPresenceChange(info);
 
-  if (!status.m_wasSubscribing) {
-    m_endpoint->Unsubscribe(SIPSubscribe::Presence | SIPSubscribe::Watcher, status.m_addressofRecord, true);
+  if (!status.m_wasSubscribing)
     m_watcherSubscriptionAOR.MakeEmpty();
-  }
 
   m_notificationMutex.Signal();
 }
 
 
-void SIP_Presentity::OnWatcherInfoNotify(SIPSubscribeHandler &, SIPSubscribe::NotifyCallbackInfo & status)
+void SIP_Presentity::OnWatcherInfoNotify(SIPSubscribeHandler &, SIPSubscribe::NotifyCallbackInfo & notifyInfo)
 {
-  // Check for empty body, if so then is OK, just a ping ...
-  if (status.m_notify.GetEntityBody().IsEmpty()) {
-    PTRACE(4, "SIPPres\tEmpty body on presence watcher NOTIFY, ignoring");
-    status.m_response.SetStatusCode(SIP_PDU::Successful_OK);
-    return;
-  }
-
   static PXML::ValidationInfo const WatcherValidation[] = {
     { PXML::RequiredNonEmptyAttribute,  "id"  },
     { PXML::RequiredAttributeWithValue, "status",  { "pending\nactive\nwaiting\nterminated" } },
@@ -475,17 +477,11 @@ void SIP_Presentity::OnWatcherInfoNotify(SIPSubscribeHandler &, SIPSubscribe::No
   };
 
   PXML xml;
-  PString error;
-  if (!xml.LoadAndValidate(status.m_notify.GetEntityBody(), WatcherInfoValidation, error, PXML::WithNS)) {
-    status.m_response.SetEntityBody(error);
-    PTRACE(3, "SIPPres\tError parsing XML in presence watcher NOTIFY: " << error);
+  if (!notifyInfo.LoadAndValidate(xml, WatcherInfoValidation))
     return;
-  }
 
   // send 200 OK now, and flag caller NOT to send the response
-  status.SendResponse(SIP_PDU::Successful_OK);
-
-  PTRACE(3, "SIPPres\t'" << m_aor << "' received NOTIFY for own presence.watcherinfo");
+  notifyInfo.SendResponse();
 
   PXMLElement * rootElement = xml.GetRootElement();
 
@@ -494,19 +490,31 @@ void SIP_Presentity::OnWatcherInfoNotify(SIPSubscribeHandler &, SIPSubscribe::No
   PWaitAndSignal mutex(m_notificationMutex);
 
   // check version number
-  bool sendRefresh = false;
-  if (m_watcherInfoVersion < 0)
-    m_watcherInfoVersion = version;
-  else {
-    sendRefresh = version != m_watcherInfoVersion+1;
-    version = m_watcherInfoVersion;
+  if (m_watcherInfoVersion >= 0 && version <= m_watcherInfoVersion) {
+    PTRACE(3, "SIPPres\t'" << m_aor << "' received repeated NOTIFY for own presence.watcherinfo, already processed");
+    return;
   }
 
   // if this is a full list of watcher info, we can empty out our pending lists
   if (rootElement->GetAttribute("state") *= "full") {
-    PTRACE(3, "SIPPres\t'" << m_aor << "' received full watcher list");
+    PTRACE(3, "SIPPres\t'" << m_aor << "' received full watcher list for own presence.watcherinfo");
     m_watcherAorById.clear();
   }
+  else if (m_watcherInfoVersion < 0) {
+    PTRACE(2, "SIPPres\t'" << m_aor << "' received partial watcher list for own presence.watcherinfo, but awaiting full list");
+    return;
+  }
+  else if (version != m_watcherInfoVersion+1) {
+    PTRACE(2, "SIPPres\t'" << m_aor << "' received partial watcher list for own presence.watcherinfo, but have missing sequence number, resubscribing");
+    m_watcherInfoVersion = -1;
+    SendCommand(CreateCommand<SIPWatcherInfoCommand>());
+    return;
+  }
+  else {
+    PTRACE(3, "SIPPres\t'" << m_aor << "' received partial watcher list for own presence.watcherinfo");
+  }
+
+  m_watcherInfoVersion = version;
 
   // go through list of watcher information
   PINDEX watcherListIndex = 0;
@@ -517,12 +525,6 @@ void SIP_Presentity::OnWatcherInfoNotify(SIPSubscribeHandler &, SIPSubscribe::No
     while ((watcher = watcherList->GetElement("watcher", watcherIndex++)) != NULL)
       OnReceivedWatcherStatus(watcher);
   }
-
-  // send refresh, if needed
-  if (sendRefresh) {
-    PTRACE(3, "SIPPres\t'" << m_aor << "' received NOTIFY for own presence.watcherinfo without out of sequence version");
-    // TODO
-  }
 }
 
 
@@ -558,24 +560,22 @@ void SIP_Presentity::Internal_SendLocalPresence(const OpalSetLocalPresenceComman
 {
   PTRACE(3, "SIPPres\t'" << m_aor << "' sending own presence " << cmd.m_state << "/" << cmd.m_note);
 
-  SIPPresenceInfo sipPresence;
-  sipPresence.m_personId = GetID();
+  SIPPresenceInfo sipPresence(cmd);
+
+  static PAtomicInteger::IntegerType g_idNumber;
+  sipPresence.m_personId = PString(++g_idNumber);
   SetPIDFEntity(sipPresence.m_entity);
   sipPresence.m_contact =  m_aor;  // As required by OMA-TS-Presence_SIMPLE-V2_0-20090917-C
-  if (m_subProtocol != e_PeerToPeer)
-    sipPresence.m_presenceAgent = m_presenceAgent.AsString();
-  sipPresence.m_state = cmd.m_state;
-  sipPresence.m_note = cmd.m_note;
+  sipPresence.m_presenceAgent = m_presenceAgentURL.AsString();
 
   if (m_publishedTupleId.IsEmpty())
-    m_publishedTupleId = sipPresence.m_tupleId;
-  else
-    sipPresence.m_tupleId = m_publishedTupleId;
+    m_publishedTupleId = PGloballyUniqueID().AsString();
+  sipPresence.m_service = m_publishedTupleId;
 
   if (m_subProtocol != e_PeerToPeer)
     m_endpoint->PublishPresence(sipPresence, GetExpiryTime());
   else
-    m_endpoint->Notify(SIPURL(m_aor.AsString()), SIPSubscribe::Presence, sipPresence.AsXML());
+    m_endpoint->Notify(m_aor, SIPSubscribe::Presence, sipPresence.AsXML());
 }
 
 
@@ -648,7 +648,7 @@ void SIP_Presentity::Internal_AuthorisationRequest(const OpalAuthorisationReques
   }
 
   XCAPClient xcap;
-  InitRootXcap(xcap);
+  xcap.SetRoot(m_defaultRootURL);
   xcap.SetApplicationUniqueID(m_attributes.Get(XcapAuthAuidKey,
                 m_subProtocol == e_OMA ? "org.openmobilealliance.pres-rules" : "pres-rules")); // As per RFC5025/9.1
   xcap.SetContentType("application/auth-policy+xml");   // As per RFC5025/9.4
@@ -792,18 +792,9 @@ void SIP_Presentity::Internal_AuthorisationRequest(const OpalAuthorisationReques
 }
 
 
-void SIP_Presentity::InitRootXcap(XCAPClient & xcap)
-{
-  PString root = m_attributes.Get(XcapRootKey);
-  if (root.IsEmpty())
-    root = "http:" + m_presenceAgent.AsString() + '/';
-  xcap.SetRoot(root);
-}
-
-
 void SIP_Presentity::InitBuddyXcap(XCAPClient & xcap, const PString & entryName, const PString & listName)
 {
-  InitRootXcap(xcap);
+  xcap.SetRoot(m_defaultRootURL);
   xcap.SetApplicationUniqueID("resource-lists");            // As per RFC5025/9.1
   xcap.SetContentType("application/resource-lists+xml");    // As per RFC5025/9.4
   xcap.SetUserIdentifier(m_aor.AsString());                 // As per RFC5025/9.7
@@ -824,15 +815,13 @@ void SIP_Presentity::InitBuddyXcap(XCAPClient & xcap, const PString & entryName,
 }
 
 
-static PXMLElement * BuddyInfoToXML(const OpalPresentity::BuddyInfo & buddy, PXMLElement * parent)
+static void BuddyInfoToXML(const OpalPresentity::BuddyInfo & buddy, PXMLElement * parent)
 {
-  PXMLElement * element = new PXMLElement(parent, "entry");
+  PXMLElement * element = parent->GetName() == "entry" ? parent : parent->AddElement("entry");
   element->SetAttribute("uri", buddy.m_presentity);
 
   if (!buddy.m_displayName.IsEmpty())
     element->AddElement("display-name", buddy.m_displayName);
-
-  return element;
 }
 
 
@@ -847,6 +836,7 @@ static bool XMLToBuddyInfo(const PXMLElement * element, OpalPresentity::BuddyInf
   if ((itemElement= element->GetElement("urn:ietf:params:xml:ns:pidf:cipid:display-name")) != NULL)
     buddy.m_displayName = itemElement->GetData();
 
+#if P_VCARD
   if ((itemElement= element->GetElement("urn:ietf:params:xml:ns:pidf:cipid:card")) != NULL) {
     PURL url;
     if (url.Parse(itemElement->GetData())) {
@@ -855,6 +845,7 @@ static bool XMLToBuddyInfo(const PXMLElement * element, OpalPresentity::BuddyInf
         buddy.m_vCard.Parse(str);
     }
   }
+#endif // P_VCARD
 
   if ((itemElement= element->GetElement("urn:ietf:params:xml:ns:pidf:cipid:icon")) != NULL)
     buddy.m_icon = itemElement->GetData();
@@ -939,7 +930,7 @@ OpalPresentity::BuddyStatus SIP_Presentity::SetBuddyListEx(const BuddyList & bud
   root->SetAttribute("name", m_attributes.Get(XcapBuddyListKey, buddyListKey));
 
   for (BuddyList::const_iterator it = buddies.begin(); it != buddies.end(); ++it)
-    root->AddChild(BuddyInfoToXML(*it, root));
+    BuddyInfoToXML(*it, root);
 
   XCAPClient xcap;
   InitBuddyXcap(xcap);
@@ -956,7 +947,7 @@ OpalPresentity::BuddyStatus SIP_Presentity::SetBuddyListEx(const BuddyList & bud
     PXMLElement * listElement = root->AddElement("list", "name", m_attributes.Get(XcapBuddyListKey, buddyListKey));
 
     for (BuddyList::const_iterator it = buddies.begin(); it != buddies.end(); ++it)
-      listElement->AddChild(BuddyInfoToXML(*it, listElement));
+      BuddyInfoToXML(*it, listElement);
 
     xcap.ClearNode();
     if (xcap.PutXml(xml))
@@ -1021,7 +1012,7 @@ OpalPresentity::BuddyStatus SIP_Presentity::SetBuddyEx(const BuddyInfo & buddy)
   InitBuddyXcap(xcap, buddy.m_presentity);
 
   PXML xml(PXML::FragmentOnly);
-  xml.SetRootElement(BuddyInfoToXML(buddy, NULL));
+  BuddyInfoToXML(buddy, xml.SetRootElement("entry"));
 
   if (xcap.PutXml(xml))
     return BuddyStatus_OK;
@@ -1067,7 +1058,7 @@ OpalPresentity::BuddyStatus SIP_Presentity::SubscribeBuddyListEx(PINDEX & numSuc
 
   PXML xml;
   XCAPClient xcap;
-  InitRootXcap(xcap);
+  xcap.SetRoot(m_defaultRootURL);
   xcap.SetApplicationUniqueID("rls-services");
   xcap.SetContentType("application/rls-services+xml");
   xcap.SetUserIdentifier(m_aor.AsString());
@@ -1238,4 +1229,4 @@ void XCAPClient::NodeSelector::AddToURL(PURL & uri) const
 
 
 
-#endif // P_EXPAT && OPAL_SIP
+#endif // OPAL_SIP_PRESENCE
