@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24474 $
+ * $Revision: 28678 $
  * $Author: rjongbloed $
- * $Date: 2010-06-07 20:19:34 -0500 (Mon, 07 Jun 2010) $
+ * $Date: 2012-12-17 01:43:06 -0600 (Mon, 17 Dec 2012) $
  */
 
 #include <ptlib.h>
@@ -36,8 +36,7 @@
 
 #include <opal/buildopts.h>
 
-#if OPAL_FAX
-#if OPAL_H323
+#if OPAL_H323 && OPAL_T38_CAPABILITY
 
 #include <t38/h323t38.h>
 
@@ -108,7 +107,7 @@ H323Channel * H323_T38Capability::CreateChannel(H323Connection & connection,
                                                 unsigned int sessionID,
                              const H245_H2250LogicalChannelParameters * params) const
 {
-  return connection.CreateRealTimeLogicalChannel(*this, direction, sessionID, params, NULL);
+  return connection.CreateRealTimeLogicalChannel(*this, direction, sessionID, params);
 }
 
 
@@ -162,7 +161,7 @@ PBoolean H323_T38Capability::OnSendingPDU(H245_DataProtocolCapability & proto,
     profile.m_t38FaxTcpOptions.m_t38TCPBidirectionalMode = mode == e_SingleTCP;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -171,7 +170,7 @@ PBoolean H323_T38Capability::OnReceivedPDU(const H245_DataApplicationCapability
   PTRACE(3, "H323T38\tOnRecievedPDU for capability");
 
   if (cap.m_application.GetTag() != H245_DataApplicationCapability_application::e_t38fax)
-    return PFalse;
+    return false;
 
   const H245_DataApplicationCapability_application_t38fax & fax = cap.m_application;
   const H245_DataProtocolCapability & proto = fax.m_t38FaxProtocol;
@@ -207,7 +206,7 @@ PBoolean H323_T38Capability::OnReceivedPDU(const H245_DataApplicationCapability
       mode = e_DualTCP;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -243,12 +242,11 @@ H323Channel * H323_T38NonStandardCapability::CreateChannel(H323Connection & conn
                                                 unsigned int sessionID,
                              const H245_H2250LogicalChannelParameters * params) const
 {
-  return connection.CreateRealTimeLogicalChannel(*this, direction, sessionID, params, NULL);
+  return connection.CreateRealTimeLogicalChannel(*this, direction, sessionID, params);
 }
 
 
-#endif // OPAL_H323
-#endif // OPAL_FAX
+#endif // OPAL_H323 && OPAL_T38_CAPABILITY
 
 
 /////////////////////////////////////////////////////////////////////////////
